@@ -1,7 +1,6 @@
-#!/bin/bash
-#	$Id$
-#
-gmt pstext -R0/5/0/3 -Jx1i -P -K -F+f+j << EOF > GMT_-J.ps
+#!/usr/bin/env bash
+gmt begin GMT_-J
+	gmt text -R0/5/0/3 -Jx1i -F+f+j << EOF
 2.5	2.8	16p,Helvetica-Bold	BC	GMT PROJECTIONS
 2	2.25	12p,Helvetica-Bold	BC	GEOGRAPHIC PROJECTIONS
 0	1.75	11p,Helvetica		BL	CYLINDRICAL
@@ -10,7 +9,7 @@ gmt pstext -R0/5/0/3 -Jx1i -P -K -F+f+j << EOF > GMT_-J.ps
 3	1.75	11p,Helvetica		BL	THEMATIC
 4	1.75	11p,Helvetica		BL	OTHER
 EOF
-gmt pstext -R -J -O -K -F+f8p+jBL << EOF >> GMT_-J.ps
+	gmt text -F+f8p+jBL << EOF 
 # Cylindrical
 0	1.35	Basic [E]
 0	1.2	Cassini
@@ -48,11 +47,11 @@ gmt pstext -R -J -O -K -F+f8p+jBL << EOF >> GMT_-J.ps
 4	0.9	Time
 4	0.75	Polar
 #
-0.05	2.75	C = Conformal
-0.05	2.6	E = Equal Area
+0.075	2.75	C = Conformal
+0.075	2.6	E = Equal Area
 EOF
 
-gmt psxy -R -J -O -Wthinner << EOF >> GMT_-J.ps
+	gmt plot -Wthinner << EOF 
 >
 2.3	2.75
 2	2.4
@@ -87,9 +86,10 @@ gmt psxy -R -J -O -Wthinner << EOF >> GMT_-J.ps
 4.2	1.7
 4.2	1.5
 >
-0	2.55
-0.85	2.55
-0.85	2.87
-0	2.87
-0	2.55
+0.025	2.55
+0.875	2.55
+0.875	2.87
+0.025	2.87
+0.025	2.55
 EOF
+gmt end show
