@@ -1,5 +1,4 @@
-#!/bin/bash
-#	$Id$
+#!/usr/bin/env bash
 #
 # Script to draw the impulse responses and transfer functions
 # for GMT cookbook Appendix_J.
@@ -585,11 +584,12 @@ EOF
 # These were pre-computed because of the need to do a numerical Hankel transform.
 # Also, I found that j0(x) and j1(x) are not reliable on some machines....
 #
-gmt gmtset FONT_ANNOT_PRIMARY 10p,Times-Roman FONT_TITLE 14p,Times-Roman FONT_LABEL 12p,Times-Roman
-cut -f1,2 tt.r_tr_fns | gmt psxy -R0/5/-0.3/1 -JX4i/2i -P -Bxa1f0.2+l"Frequency (cycles per filter width)" -Bya0.2f0.1g1+l"Gain" -BWeSn -K -Wthick > GMT_App_J_3.ps
-cut -f1,3 tt.r_tr_fns | gmt psxy -R -J -O -K -Wthick,- >> GMT_App_J_3.ps
-cut -f1,4 tt.r_tr_fns | gmt psxy -R -J -O -K -Wthick,. >> GMT_App_J_3.ps
-gmt pstext -R -J -O -F+f9p,Times-Roman+j << END >> GMT_App_J_3.ps
+gmt begin GMT_App_J_3
+gmt set FONT_ANNOT_PRIMARY 10p,Times-Roman FONT_TITLE 14p,Times-Roman FONT_LABEL 12p,Times-Roman
+cut -f1,2 tt.r_tr_fns | gmt plot -R0/5/-0.3/1 -JX4i/2i -Bxa1f0.2+l"Frequency (cycles per filter width)" -Bya0.2f0.1g1+l"Gain" -BWeSn -Wthick
+cut -f1,3 tt.r_tr_fns | gmt plot -Wthick,- 
+cut -f1,4 tt.r_tr_fns | gmt plot -Wthick,. 
+gmt text -F+f9p,Times-Roman+j << END
 2.2	0.6	LM	Solid Line:
 2.2	0.5	LM	Dotted Line:
 2.2	0.4	LM	Dashed Line:
@@ -597,3 +597,4 @@ gmt pstext -R -J -O -F+f9p,Times-Roman+j << END >> GMT_App_J_3.ps
 3.8	0.5	RM	Gaussian
 3.8	0.4	RM	Cosine
 END
+gmt end show
