@@ -1,22 +1,21 @@
-#!/bin/bash
+#!/usr/bin/env bash
 #		GMT Appendix P, example 2
-#		$Id$
 #
 # Purpose:	Illustrates the use of isolation mode
-# GMT progs:	gmt gmtset, gmt grdimage, gmt grdmath, gmt makecpt, gmt pscoast
+# GMT progs:	gmt set, gmt grdimage, gmt grdmath, gmt makecpt, gmt pscoast
 # GMT funcs:	gmt_init_tmpdir, gmt_remove_tmpdir
 #
 ps=GMT_App_P_2.ps
 
 # Make GMT shell functions accessible the the script
-. gmt_shell_functions.sh
+. ${GMT_SOURCE_DIR}/src/gmt_shell_functions.sh
 
 # Create a temporary directory. $GMT_TMPDIR will be set to its pathname.
 gmt_init_tmpdir
 
 # These settings will be local to this script only since it writes to
 # $GMT_TMPDIR/gmt.conf
-gmt gmtset FONT_ANNOT_PRIMARY 14p
+gmt set FONT_ANNOT_PRIMARY 14p
 
 # Make grid file and color map in temporary directory
 gmt grdmath -Rd -I1 Y = $GMT_TMPDIR/lat.nc
