@@ -1,6 +1,5 @@
-#!/bin/bash
+#!/usr/bin/env bash
 #
-#	$Id$
 #
 #	This plots the given encoding vector to stdout
 #
@@ -24,9 +23,11 @@ cat << EOF > tt.awk
 	printf "\n"
 }
 EOF
+# Need access to PS source files for this plot
+GMT_SHAREDIR=`gmt --show-sharedir`
 egrep -v '\[|\]' "${GMT_SHAREDIR}"/share/postscriptlight/$1.ps | $AWK -f tt.awk > tt.chart
 cat << EOF > tt.awk
-# This awk script creates a file for gmt psxy to plot a rectangle for undefined entries
+# This awk script creates a file for gmt plot to plot a rectangle for undefined entries
 {
 	for (i = 1; i <= 8; i++)
 	{
@@ -53,10 +54,11 @@ BEGIN {
 }
 EOF
 
-gmt gmtset PS_CHAR_ENCODING $1
-gmt psxy -R0/9/-1/32 -Jx0.345i/-0.21i -Bg1 -B+t"Octal codes for $1" -P -K -Ggray -X3i -Sri tt.empty
-$AWK -f tt.awk tt.chart | gmt pstext -R -J -O -K -F+f10p,Times-Roman
-gmt psxy -R -J -O -Wthick << EOF
+gmt begin GMT_encoding
+gmt set PS_CHAR_ENCODING $1
+gmt plot -R0/9/-1/32 -Jx0.345i/-0.21i -Bg1 -B+t"Octal codes for $1" -Ggray -X3i -Sri tt.empty
+$AWK -f tt.awk tt.chart | gmt text -F+f10p,Times-Roman
+gmt plot -Wthick << EOF
 >
 0	0
 9	0
@@ -64,3 +66,4 @@ gmt psxy -R -J -O -Wthick << EOF
 1	0
 1	32
 EOF
+gmt end show
