@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *    $Id$
  *
- *	Copyright (c) 1991-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 
 /**
@@ -33,8 +32,15 @@
 #define BLOCKMODE_CTRL BLOCK_CTRL
 #endif
 
+#define BLK_N_FIELDS	8
+
 /*! All control options for this program (except common args) */
 struct BLOCK_CTRL {
+	struct A {	/* -A<fields> */
+		bool active;
+		unsigned int n_selected;
+		bool selected[BLK_N_FIELDS];
+	} A;
 	struct C {	/* -C */
 		bool active;
 	} C;
@@ -52,13 +58,14 @@ struct BLOCK_CTRL {
 	} E;
 	struct G {	/* -G<outfile> */
 		bool active;
-		char *file;
+		unsigned int n;			/* Number of output grids specified via -G */
+		char *file[BLK_N_FIELDS];	/* Only first is used for commandline but API may need many */
 	} G;
 	struct N {	/* -N<empty> */
 		bool active;
 		double no_data;
 	} N;
-#if !defined(BLOCKMEAN)		/* Only blockmedian & blockmode has a -Q option */
+#if !defined(BLOCKMEAN)		/* Only blockmedian & blockmode have a -Q option */
 	struct Q {	/* -Q */
 		bool active;
 	} Q;
@@ -157,10 +164,10 @@ enum GMT_enum_blks {BLK_Z	= 2,
 		BLK_W		= 3};
 
 struct BLK_DATA {
-	double a[4];		/*!< a[0] = x, a[1] = y, a[2] = z, a[3] = w  */
-	uint64_t ij;	/*!< Grid index for data value */
+	double a[4];		/* < a[0] = x, a[1] = y, a[2] = z, a[3] = w  */
+	uint64_t ij;		/* < Grid index for data value */
 #if !defined(BLOCKMEAN)		/* Only blockmedian & blockmode has a -Q option */
-	uint64_t src_id;	/*!< Source id [Data record] on input */
+	uint64_t src_id;	/* < Source id [Data record] on input */
 #endif
 };
 #endif
@@ -185,8 +192,10 @@ GMT_LOCAL void *New_Ctrl (struct GMT_CTRL *GMT) {
 
 /*! Deallocate control structure */
 GMT_LOCAL void Free_Ctrl (struct GMT_CTRL *GMT, struct  BLOCK_CTRL *C) {
+	unsigned int k;
 	if (!C) return;
-	gmt_M_str_free (C->G.file);	
+	for (k = 0; k < C->G.n; k++)
+		gmt_M_str_free (C->G.file[k]);	
 	gmt_M_free (GMT, C);	
 }
 
