@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *    $Id$
  *
- *	Copyright (c) 1991-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,14 +12,14 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  * API functions to support the blockmode application.
  *
  * Author:	Walter H.F. Smith
  * Date:	1-JAN-2010
- * Version:	5 API
+ * Version:	6 API
  *
  * Brief synopsis: reads records of x, y, data, [weight] and writes out mode
  * value per cell, where cellular region is bounded by West East South North
@@ -31,10 +30,11 @@
 
 #include "gmt_dev.h"
 
-#define THIS_MODULE_NAME	"blockmode"
+#define THIS_MODULE_CLASSIC_NAME	"blockmode"
+#define THIS_MODULE_MODERN_NAME	"blockmode"
 #define THIS_MODULE_LIB		"core"
 #define THIS_MODULE_PURPOSE	"Block average (x,y,z) data tables by mode estimation"
-#define THIS_MODULE_KEYS	"<D{,>D}"
+#define THIS_MODULE_KEYS	"<D{,>D},GG),A->"
 #define THIS_MODULE_NEEDS	"R"
 #define THIS_MODULE_OPTIONS "-:>RVabdefghior" GMT_OPT("FH")
 
@@ -51,18 +51,32 @@ struct BIN_MODE_INFO {	/* Used for histogram binning */
 	unsigned int n_bins;/* Number of bins required */
 };
 
+/* Note: For external calls to block* we do not allow explicit -G options; these should be added by examining -A which
+ * is required for external calls to make grids, even if just z is requested.  This differs from the command line where
+ * -Az is the default and -G is required to set file name format.  */
+
 GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
-	gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
+	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: blockmode [<table>] %s\n", GMT_I_OPT);
-	GMT_Message (API, GMT_TIME_NONE, "\t%s [-C] [-D<width>[+c][+l|h]] [-E] [-Er|s[-]] [-Q]\n\t[%s] [-W[i][o][+s]] [%s] [%s] [%s] [%s]\n\t[%s] [%s]\n\t[%s] [%s] [%s] [%s]\n\n",
-		GMT_Rgeo_OPT, GMT_V_OPT, GMT_a_OPT, GMT_b_OPT, GMT_d_OPT, GMT_e_OPT, GMT_f_OPT, GMT_h_OPT, GMT_i_OPT, GMT_o_OPT, GMT_r_OPT, GMT_colon_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "usage: %s [<table>] %s %s\n", name, GMT_I_OPT, GMT_Rgeo_OPT);
+	if (API->external)
+		GMT_Message (API, GMT_TIME_NONE, "\t[-A<fields>] [-C] [-D<width>[+c][+l|h]] [-E] [-Er|s[+l|h]] [-Q] [-W[i][o][+s]]\n", GMT_V_OPT);
+	else
+		GMT_Message (API, GMT_TIME_NONE, "\t[-A<fields>] [-C] [-D<width>[+c][+l|h]] [-E] [-Er|s[+l|h]] [-G<grdfile>] [-Q] [-W[i][o][+s]]\n", GMT_V_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "\t[%s] [%s] [%s] [%s] [%s]\n\t[%s] [%s]\n\t[%s] [%s] [%s] [%s] [%s]\n\n",
+		GMT_a_OPT, GMT_b_OPT, GMT_d_OPT, GMT_e_OPT, GMT_f_OPT, GMT_h_OPT, GMT_i_OPT, GMT_o_OPT, GMT_r_OPT, GMT_colon_OPT, GMT_PAR_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
 	GMT_Option (API, "I,R");
 	GMT_Message (API, GMT_TIME_NONE, "\n\tOPTIONS:\n");
 	GMT_Option (API, "<");
+	if (API->external)
+		GMT_Message (API, GMT_TIME_NONE, "\t-A List of comma-separated fields to be written as grids. Choose from\n");
+	else
+		GMT_Message (API, GMT_TIME_NONE, "\t-A List of comma-separated fields to be written as grids (requires -G). Choose from\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   z, s, l, h, and w. s|l|h requires -E; w requires -W[o].\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   Cannot be used with -Er|s [Default is z only].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-C Output center of block and mode z-value [Default is mode location (but see -Q)].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-D Compute modes via binning using <width>; append +c to center bins. If there are multiple\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   modes we return the average mode [+a]; append +l or +h to pick the low or high mode instead.\n");
@@ -71,9 +85,13 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t   [Default computes the mode as the Least Median of Squares (LMS) estimate].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-E Extend output with LMS scale (s), low (l), and high (h) value per block, i.e.,\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   output (x,y,z,s,l,h[,w]) [Default outputs (x,y,z[,w])]; see -W regarding w.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Use -Er to report record number of the median value per block,\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   Use -Er to report record number of the modal value per block,\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   or -Es to report an unsigned integer source id (sid) taken from the x,y,z[,w],sid input.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   For ties, report record number (or sid) of largest value; append - for smallest.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   For ties, report record number (or sid) of highest value (+h) or append +l for lowest [highest].\n");
+	if (!API->external) {
+		GMT_Message (API, GMT_TIME_NONE, "\t-G Specify output grid file name; no table results will be written to stdout.\n");
+		GMT_Message (API, GMT_TIME_NONE, "\t   If more than one field is set via -A then <grdfile> must contain  %%s to format field code.\n");
+	}
 	GMT_Message (API, GMT_TIME_NONE, "\t-Q Quicker; get mode z and mean x,y [Default gets mode x, mode y, mode z].\n");
 	GMT_Option (API, "V");
 	GMT_Message (API, GMT_TIME_NONE, "\t-W Set Weight options.\n");
@@ -97,9 +115,8 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct BLOCKMODE_CTRL *Ctrl, struct G
 
 	unsigned int n_errors = 0, pos = 0;
 	bool sigma;
-	char arg[GMT_LEN16] = {""};
+	char arg[GMT_LEN16] = {""}, p[GMT_BUFSIZ] = {""}, *c = NULL;
 	struct GMT_OPTION *opt = NULL;
-	char p[GMT_BUFSIZ] = {""}, *c = NULL;
 
 	for (opt = options; opt; opt = opt->next) {
 		switch (opt->option) {
@@ -110,21 +127,45 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct BLOCKMODE_CTRL *Ctrl, struct G
 
 			/* Processes program-specific parameters */
 
+				case 'A':	/* Requires -G and selects which fields should be written as grids */
+				Ctrl->A.active = true;
+				pos = 0;
+				while ((gmt_strtok (opt->arg, ",", &pos, p)) && Ctrl->A.n_selected < BLK_N_FIELDS) {
+					switch (p[0]) {	/* z,s,l,h,w */
+						case 'z':	Ctrl->A.selected[0] = true;	break;
+						case 's':	Ctrl->A.selected[1] = true;	break;
+						case 'l':	Ctrl->A.selected[2] = true;	break;
+						case 'h':	Ctrl->A.selected[3] = true;	break;
+						case 'w':	Ctrl->A.selected[4] = true;	break;
+						default:
+							GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Unrecognized field argument %s in -A!\n", p);
+							n_errors++;
+							break;
+					}
+					Ctrl->A.n_selected++;
+				}
+				if (Ctrl->A.n_selected == 0) {	/* Let -Az be the default */
+					GMT_Report (GMT->parent, GMT_MSG_DEBUG, "-A interpreted to mean -Az.\n");
+					Ctrl->A.selected[0] = true;
+					Ctrl->A.n_selected = 1;
+				}
+				break;
 			case 'C':	/* Report center of block instead */
 				Ctrl->C.active = true;
 				break;
 			case 'D':	/* Histogram mode estimate */
 				Ctrl->D.active = true;
 				Ctrl->D.width = atof (opt->arg);
+				pos = 0;
 				if ((c = strchr (opt->arg, '+')) != NULL) {	/* Found modifiers */
 					while ((gmt_strtok (c, "+", &pos, p))) {
 						switch (p[0]) {
 							case 'c': Ctrl->D.center = true; break;	/* Center the histogram */
-							case 'a': Ctrl->D.mode = BLOCKMODE_AVE; break;	/* Pick average mode */
-							case 'l': Ctrl->D.mode = BLOCKMODE_LOW; break;	/* Pick low mode */
+							case 'a': Ctrl->D.mode = BLOCKMODE_AVE;  break;	/* Pick average mode */
+							case 'l': Ctrl->D.mode = BLOCKMODE_LOW;  break;	/* Pick low mode */
 							case 'h': Ctrl->D.mode = BLOCKMODE_HIGH; break;	/* Pick high mode */
 							default:	/* Bad modifier */
-								GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Error: Unrecognized modifier +%c.\n", p[0]);
+								GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Unrecognized modifier +%c.\n", p[0]);
 								n_errors++;
 								break;
 						}
@@ -133,13 +174,40 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct BLOCKMODE_CTRL *Ctrl, struct G
 				break;
 			case 'E':
 				Ctrl->E.active = true;		/* Extended report with standard deviation, min, and max in cols 4-6 */
-				if (opt->arg[0] == 'r' || opt->arg[0] == 's') {
-					Ctrl->E.mode = (opt->arg[1] == '-') ? BLK_DO_INDEX_LO : BLK_DO_INDEX_HI;	/* Report row number or sid of median */
-					if (opt->arg[0] == 's') /* report sid */
+				if (opt->arg[0] == 'r' || opt->arg[0] == 's') {	/* Report row number or sid of median */
+					switch (opt->arg[1]) {	/* Look for modifiers */
+						case '+':	/* New syntax with +h or +l to parse */
+							if (opt->arg[2] == 'l')
+								Ctrl->E.mode = BLK_DO_INDEX_LO;
+							else if (opt->arg[2] == 'h' || opt->arg[2] == '\0')	/* E.g., let Er+ be thought of as -Er+h */
+								Ctrl->E.mode = BLK_DO_INDEX_HI;
+							else {	/* Neither +l, +h, or just + is bad */
+								GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Unrecognized argument -E%s!\n", opt->arg);
+								n_errors++;
+							}
+							break;
+						case '-':	/* Old syntax -Er- or -Es- for reporting index/source of lower value */
+							Ctrl->E.mode = BLK_DO_INDEX_LO;
+							break;
+						default:	/* Default reports index/source of higher value */
+							Ctrl->E.mode = BLK_DO_INDEX_HI;
+							break;
+					}
+					if (opt->arg[0] == 's') /* report sid, add in flag */
 						Ctrl->E.mode |= BLK_DO_SRC_ID;
 				}
-				else if (opt->arg[0] == '\0')
-					Ctrl->E.mode = BLK_DO_EXTEND3;		/* Report LMSscale, low, high in cols 4-6 */
+				else if (opt->arg[0] == '\0')	/* Plain -E : Report LMSscale, low, high in cols 4-6 */
+					Ctrl->E.mode = BLK_DO_EXTEND3;
+				else	/* WTF? */
+					n_errors++;
+				break;
+			case 'G':	/* Write output grid(s) */
+				if (!GMT->parent->external && Ctrl->G.n) {	/* Command line interface */
+					GMT_Report (GMT->parent, GMT_MSG_NORMAL, "-G can only be set once!\n");
+					n_errors++;
+				}
+				else if ((Ctrl->G.active = gmt_check_filearg (GMT, 'G', opt->arg, GMT_OUT, GMT_IS_GRID)) != 0)
+					Ctrl->G.file[Ctrl->G.n++] = strdup (opt->arg);
 				else
 					n_errors++;
 				break;
@@ -151,6 +219,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct BLOCKMODE_CTRL *Ctrl, struct G
 				break;
 			case 'W':	/* Use in|out weights */
 				Ctrl->W.active = true;
+				if (gmt_validate_modifiers (GMT, opt->arg, 'W', "s")) n_errors++;
 				sigma = (gmt_get_modifier (opt->arg, 's', arg)) ? true : false;
 				switch (arg[0]) {
 					case '\0':
@@ -177,9 +246,46 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct BLOCKMODE_CTRL *Ctrl, struct G
 		}
 	}
 
+	if (Ctrl->G.active) {	/* Make sure -A sets valid fields, some require -E */
+		if (!Ctrl->E.active && (Ctrl->A.selected[1] || Ctrl->A.selected[2] || Ctrl->A.selected[3])) {
+			/* -E is required if -A specifies l or h */
+			Ctrl->E.active = true;			/* Extended report with standard deviation, min, and max in cols 4-6 */
+			Ctrl->E.mode = BLK_DO_EXTEND3;	/* Report LMSscale, low, high in cols 4-6 */
+		}
+		if (GMT->parent->external && !Ctrl->A.active) {		/* From externals let -G equals -Az */
+			Ctrl->A.active = true;
+			Ctrl->A.selected[0] = true;
+			Ctrl->A.n_selected = 1;
+		}
+	}
+
+	n_errors += gmt_M_check_condition (GMT, Ctrl->A.active && !Ctrl->G.active, "Syntax error: -A requires -G\n");
+	n_errors += gmt_M_check_condition (GMT, GMT->parent->external && Ctrl->G.active && !Ctrl->A.active,
+	                                   "Syntax error: -G requires -A\n");
+	n_errors += gmt_M_check_condition (GMT, Ctrl->G.active && (Ctrl->E.mode == BLK_DO_INDEX_LO || Ctrl->E.mode == BLK_DO_INDEX_HI), "Syntax error: -Es|r are incompatible with -G\n");
+	if (Ctrl->G.active) {	/* Make sure -A sets valid fields, some require -E */
+		if (Ctrl->A.active && Ctrl->A.n_selected > 1 && !GMT->parent->external && !strstr (Ctrl->G.file[0], "%s")) {
+			GMT_Report (GMT->parent, GMT_MSG_NORMAL, "-G file format must contain a %%s for field type substitution.\n");
+			n_errors++; 
+		}
+		else if (!Ctrl->A.active)	/* Set default z output grid */
+			Ctrl->A.selected[0] = true, Ctrl->A.n_selected = 1;
+		else {	/* Make sure -A choices are valid and that -E is set if extended fields are selected */
+			if (!Ctrl->E.active && (Ctrl->A.selected[1] || Ctrl->A.selected[2] || Ctrl->A.selected[3])) {
+				GMT_Report (GMT->parent, GMT_MSG_VERBOSE, "-E is required if -A specifies s, l, or h.  -E was added.\n");
+				Ctrl->E.active = true; 
+			}
+			if (Ctrl->A.selected[4] && !Ctrl->W.weighted[GMT_OUT]) {
+				GMT_Report (GMT->parent, GMT_MSG_NORMAL, "-W or -Wo is required if -A specifies w.\n");
+				n_errors++; 
+			}
+		}
+	}
 	n_errors += gmt_M_check_condition (GMT, !GMT->common.R.active[RSET], "Syntax error: Must specify -R option\n");
-	n_errors += gmt_M_check_condition (GMT, GMT->common.R.inc[GMT_X] <= 0.0 || GMT->common.R.inc[GMT_Y] <= 0.0, "Syntax error -I option: Must specify positive increment(s)\n");
-	n_errors += gmt_M_check_condition (GMT, Ctrl->D.active && Ctrl->E.active, "Syntax error -D option: Cannot be combined with -E\n");
+	n_errors += gmt_M_check_condition (GMT, GMT->common.R.inc[GMT_X] <= 0.0 || GMT->common.R.inc[GMT_Y] <= 0.0,
+	                                   "Syntax error -I option: Must specify positive increment(s)\n");
+	n_errors += gmt_M_check_condition (GMT, Ctrl->D.active && Ctrl->E.active,
+	                                   "Syntax error -D option: Cannot be combined with -E\n");
 	n_errors += gmt_check_binary_io (GMT, (Ctrl->W.weighted[GMT_IN]) ? 4 : 3);
 
 	return (n_errors ? GMT_PARSE_ERROR : GMT_NOERROR);
@@ -267,7 +373,7 @@ GMT_LOCAL double bin_mode (struct GMT_CTRL *GMT, struct BLK_DATA *d, uint64_t n,
 			case BLOCKMODE_AVE:		/* Get the average of the modes */
 				value += ((bin + B->min) + B->o_offset) * B->width;
 				break;
-			case BLOCKMODE_HIGH:	/* Update highest mode so far, when loop exits we have the hightest mode */
+			case BLOCKMODE_HIGH:	/* Update highest mode so far, when loop exits we have the highest mode */
 			 	value = ((bin + B->min) + B->o_offset) * B->width;
 				break;
 		}
@@ -277,88 +383,58 @@ GMT_LOCAL double bin_mode (struct GMT_CTRL *GMT, struct BLK_DATA *d, uint64_t n,
 	return (value);
 }
 
-#if 0
-GMT_LOCAL double weighted_mode (struct BLK_DATA *d, double wsum, unsigned int emode, uint64_t n, unsigned int k, uint64_t *index) {
-	/* Estimate mode by finding a maximum in the estimated
-	   pdf of weighted data.  Estimate the pdf as the finite
-	   difference of the cumulative frequency distribution
-	   over points from i to j.  This has the form top/bottom,
-	   where top is the sum of the weights from i to j, and
-	   bottom is (data[j] - data[i]).  Strategy is to start
-	   with i=0, j=n-1, and then move i or j toward middle
-	   while j-i > n/2 and bottom > 0.  At end while, midpoint
-	   of range from i to j is the mode estimate.  Choose
-	   to move either i or j depending on which one will
-	   cause greatest increase in pdf estimate.  If a tie,
-	   move both.
-
-	   Strictly, the pdf estimated this way would need to be
-	   scaled by (1/wsum), but this is constant so we don't
-	   use it here, as we are seeking a relative minimum.
-
-	   I assumed n > 2 when I wrote this. [WHFS] */
-
-	double top, topj, topi, bottomj, bottomi, pj, pi;
-	int64_t i = 0, j = n - 1, nh = n / 2;
-	int way;
-
-	top = wsum;
-
-	while ((j-i) > nh) {
-		topi = top - d[i].a[BLK_W];
-		topj = top - d[j].a[BLK_W];
-		bottomi = d[j].a[k] - d[i+1].a[k];
-		bottomj = d[j-1].a[k] - d[i].a[k];
-
-		if (bottomj == 0.0) {
-			if (index) *index = d[j-1].src_id;
-			return (d[j-1].a[k]);
-		}
-		if (bottomi == 0.0) {
-			if (index) *index = d[i+1].src_id;
-			return (d[i+1].a[k]);
-		}
-		pi = topi / bottomi;
-		pj = topj / bottomj;
-		if (pi > pj) {
-			i++;
-			top = topi;
-		}
-		else if (pi < pj) {
-			j--;
-			top = topj;
-		}
-		else {
-			top -= (d[i].a[BLK_W] + d[j].a[BLK_W]);
-			i++;
-			j--;
-		}
-	}
-	if (emode && index) {	/* Also return best src_id for this mode */
-		way = (d[j].a[k] >= d[i].a[k]) ? +1 : -1;
-		if (emode & BLK_DO_INDEX_HI) *index = (way == +1) ? d[j].src_id : d[i].src_id;
-		else *index = (way == +1) ? d[i].src_id : d[j].src_id;
-	}
-	return (0.5 * (d[j].a[k] + d[i].a[k]));
+GMT_LOCAL uint64_t get_source (struct BLK_DATA *d, uint64_t i, uint64_t j, unsigned int emode) {
+	/* Return the source at the center or the lo/hi of the two in the middle */
+	uint64_t s = j - i + 1, mid = i + s/2, src;
+	if (s%2)	/* A single central point */
+		src = d[mid].src_id;
+	else	/* Must go low or high */
+		src = (emode & BLK_DO_INDEX_HI) ? d[mid-1].src_id : d[mid].src_id;
+	return src;
 }
-#endif
 
 GMT_LOCAL double weighted_mode (struct BLK_DATA *d, double wsum, unsigned int emode, uint64_t n, unsigned int k, uint64_t *index) {
 	/* Looks for the "shortest 50%". This means that when the cumulative weight
 	   (y) is plotted against the value (x) then the line between (xi,yi) and
 	   (xj,yj) should be the steepest for any combination where (yj-yi) is 50%
-	   of the total sum of weights */
+	   of the total sum of weights.
+	   Here, n > 2. */
 
-	double top, bottom, p, p_max, mode;
+	double top, bottom, p, p_max, mode, last_del = 0.0;
 	uint64_t i, j, src = 0, n_modes = 0;
+	bool equidistant = true;
 
 	/* Do some initializations */
 	wsum = 0.5 * wsum; /* Sets the 50% range */
-	/* First check if any single point has 50% or more of the total weights; if so we are done */
-	for (i = 0; i < n; i++) if (d[i].a[BLK_W] >= wsum) {
-		if (index) *index = d[i].src_id;
-		return d[i].a[k];
+	/* First check if any single point has 50% or more of the total weights; if so we are done.
+	 * While at it, check if points are equally spaced, inc which case we pick mode as the center */
+	for (i = 0; i < n; i++) {
+		if (d[i].a[BLK_W] >= wsum) {	/* Found a mighty and weighty single point */
+			if (index) *index = d[i].src_id;
+			return d[i].a[k];
+		}
+		else if (i && equidistant) {	/* May compute delta from 2nd point onwards */
+			double del = d[i].a[k] - d[i-1].a[k];
+			if (i > 1 && !doubleAlmostEqualZero (del, last_del))	/* Check for identical spacing */
+				equidistant = false;	/* Well, not any more */
+			else
+				last_del = del;
+		}
 	}
+	/* Deal with special cases */
+	if (equidistant) {	/* Equidistant data set, pick middle */
+		i = n / 2;
+		if (n % 2) {	/* Single middle point */
+			if (index) *index = d[i].src_id;
+			mode = d[i].a[k];
+		}
+		else {	/* Shared middle points */
+			if (index) *index = (emode & BLK_DO_INDEX_HI) ? d[i-1].src_id : d[i].src_id;
+			mode = 0.5 * (d[i-1].a[k] + d[i].a[k]);
+		}
+		return mode;
+	}
+	/* Must find maximum slope in the cdf */
 	top = p_max = 0.0;
 	mode = 0.5 * (d[0].a[k] + d[n-1].a[k]);
 	if (index) src = (emode & BLK_DO_INDEX_HI) ? d[n-1].src_id : d[0].src_id;
@@ -382,11 +458,19 @@ GMT_LOCAL double weighted_mode (struct BLK_DATA *d, double wsum, unsigned int em
 			p_max = p;
 			mode = 0.5 * (d[i].a[k] + d[j].a[k]);
 			n_modes = 1;
-			if (index) src = (emode & BLK_DO_INDEX_HI) ? d[j].src_id : d[i].src_id;
+			if (index) src = get_source (d, i, j, emode);	/* Must find corresponding middle or low/high index */
 		}
 		else if (doubleAlmostEqual (p, p_max)) {	/* Same peak as previous best mode, get average of these modes */
-			mode += 0.5 * (d[i].a[k] + d[j].a[k]);
-			n_modes++;
+			if (index) {	/* Cannot have multiple modes when we are requesting teh source, go with high or low per emode setting */
+				if (emode & BLK_DO_INDEX_HI) {	/* OK, we are interested in the higher mode only */
+					src = get_source (d, i, j, emode);	/* Must find corresponding high index */
+					mode = 0.5 * (d[i].a[k] + d[j].a[k]);
+				}
+			}
+			else {	/* Can average modes if need be */
+				mode += 0.5 * (d[i].a[k] + d[j].a[k]);
+				n_modes++;
+			}
 		}
 	}
 	if (n_modes > 1) mode /= n_modes;
@@ -396,7 +480,7 @@ GMT_LOCAL double weighted_mode (struct BLK_DATA *d, double wsum, unsigned int em
 
 /* Must free allocated memory before returning */
 #define bailout(code) {gmt_M_free_options (mode); return (code);}
-#define Return(code) {GMT_Destroy_Data (API, &Grid); Free_Ctrl (GMT, Ctrl); gmt_end_module (GMT, GMT_cpy); bailout (code);}
+#define Return(code) {GMT_Destroy_Data (API, &Grid); gmt_M_free (GMT, Out); gmt_M_free (GMT, data); Free_Ctrl (GMT, Ctrl); gmt_end_module (GMT, GMT_cpy); bailout (code);}
 
 int GMT_blockmode (void *V_API, int mode, void *args) {
 	bool mode_xy, do_extra = false, is_integer, duplicate_col;
@@ -404,20 +488,22 @@ int GMT_blockmode (void *V_API, int mode, void *args) {
 	int way, error = 0;
 
 	unsigned int row, col, emode = 0, n_input, n_output;
+	unsigned int k, NF = 0, fcol[BLK_N_FIELDS] = {2,3,4,5,6,7,0,0}, field[BLK_N_FIELDS];
 
 	uint64_t node, first_in_cell, first_in_new_cell, n_lost, n_read;
 	uint64_t n_cells_filled, n_in_cell, nz, n_pitched, src_id = 0;
-	uint64_t w_col, i_col = 0, sid_col;
+	uint64_t w_col, i_col = 0, sid_col, *src_id_ptr = NULL;
 
 	size_t n_alloc = 0, nz_alloc = 0;
 
 	double out[7], wesn[4], i_n_in_cell, d_intval, weight, half_dx, *in = NULL, *z_tmp = NULL;
 	double z_min = DBL_MAX, z_max = -DBL_MAX;
 
-	char format[GMT_LEN256] = {""}, *old_format = NULL;
+	char format[GMT_LEN512] = {""}, *old_format = NULL, *fcode[BLK_N_FIELDS] = {"z", "s", "l", "h", "w", "", "", ""}, *code[BLK_N_FIELDS];
 
 	struct GMT_OPTION *options = NULL;
-	struct GMT_GRID *Grid = NULL;
+	struct GMT_GRID *Grid = NULL, *G = NULL, *GridOut[BLK_N_FIELDS];
+	struct GMT_RECORD *In = NULL, *Out = NULL;
 	struct BIN_MODE_INFO *B = NULL;
 	struct BLK_DATA *data = NULL;
 	struct BLOCKMODE_CTRL *Ctrl = NULL;
@@ -429,42 +515,43 @@ int GMT_blockmode (void *V_API, int mode, void *args) {
 	if (API == NULL) return (GMT_NOT_A_SESSION);
 	if (mode == GMT_MODULE_PURPOSE) return (usage (API, GMT_MODULE_PURPOSE));	/* Return the purpose of program */
 	options = GMT_Create_Options (API, mode, args);	if (API->error) return (API->error);
-	if (!options || options->option == GMT_OPT_USAGE) bailout (usage (API, GMT_USAGE));	/* Return the usage message */
-	if (options->option == GMT_OPT_SYNOPSIS) bailout (usage (API, GMT_SYNOPSIS));	/* Return the synopsis */
+	if ((error = gmt_report_usage (API, options, 0, usage)) != GMT_NOERROR) bailout (error);	/* Give usage if requested */
 
 	/* Parse the command-line arguments */
 
-	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
 	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
 	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
 	if ((error = parse (GMT, Ctrl, options)) != 0) Return (error);
 
 	/*---------------------------- This is the blockmode main code ----------------------------*/
 
-	GMT_Report (API, GMT_MSG_VERBOSE, "Processing input table data\n");
+	gmt_M_memset (GridOut, BLK_N_FIELDS, struct GMT_GRID *);	/* Initialize all pointers to NULL */
+
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Processing input table data\n");
 
 	if (Ctrl->C.active && Ctrl->Q.active) {
-		GMT_Report (API, GMT_MSG_NORMAL, "Warning: -C overrides -Q\n");
+		GMT_Report (API, GMT_MSG_VERBOSE, "-C overrides -Q\n");
 		Ctrl->Q.active = false;
 	}
 
 	if ((Grid = GMT_Create_Data (API, GMT_IS_GRID, GMT_IS_SURFACE, GMT_CONTAINER_ONLY, NULL, NULL, NULL, \
-		GMT_GRID_DEFAULT_REG, 0, NULL)) == NULL) Return (API->error);	/* Note: 0 for pad since no BC work needed */
+		GMT_GRID_DEFAULT_REG, GMT_NOTSET, NULL)) == NULL) Return (API->error);
 
 	duplicate_col = (gmt_M_360_range (Grid->header->wesn[XLO], Grid->header->wesn[XHI]) && Grid->header->registration == GMT_GRID_NODE_REG);	/* E.g., lon = 0 column should match lon = 360 column */
 	half_dx = 0.5 * Grid->header->inc[GMT_X];
 	mode_xy = !Ctrl->C.active;
 
-	if (gmt_M_is_verbose (GMT, GMT_MSG_VERBOSE)) {
-		snprintf (format, GMT_LEN256, "W: %s E: %s S: %s N: %s n_columns: %%d n_rows: %%d\n", GMT->current.setting.format_float_out, GMT->current.setting.format_float_out, GMT->current.setting.format_float_out, GMT->current.setting.format_float_out);
-		GMT_Report (API, GMT_MSG_VERBOSE, format, Grid->header->wesn[XLO], Grid->header->wesn[XHI], Grid->header->wesn[YLO], Grid->header->wesn[YHI], Grid->header->n_columns, Grid->header->n_rows);
+	if (gmt_M_is_verbose (GMT, GMT_MSG_LONG_VERBOSE)) {
+		snprintf (format, GMT_LEN512, "W: %s E: %s S: %s N: %s n_columns: %%d n_rows: %%d\n", GMT->current.setting.format_float_out, GMT->current.setting.format_float_out, GMT->current.setting.format_float_out, GMT->current.setting.format_float_out);
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, format, Grid->header->wesn[XLO], Grid->header->wesn[XHI], Grid->header->wesn[YLO], Grid->header->wesn[YHI], Grid->header->n_columns, Grid->header->n_rows);
 	}
 
 	gmt_set_xy_domain (GMT, wesn, Grid->header);	/* May include some padding if gridline-registered */
 
 	/* Specify input and output expected columns */
 	n_input = 3 + Ctrl->W.weighted[GMT_IN] + ((Ctrl->E.mode & BLK_DO_SRC_ID) ? 1 : 0);	/* 3 columns on output, plus 1 extra if -W and another if -Es  */
-	if ((error = gmt_set_cols (GMT, GMT_IN, n_input)) != GMT_NOERROR) {
+	if ((error = GMT_Set_Columns (API, GMT_IN, n_input, GMT_COL_FIX)) != GMT_NOERROR) {
 		Return (error);
 	}
 	n_output = (Ctrl->W.weighted[GMT_OUT]) ? 4 : 3;
@@ -476,7 +563,7 @@ int GMT_blockmode (void *V_API, int mode, void *args) {
 		n_output++;
 		emode = Ctrl->E.mode & (BLK_DO_INDEX_LO + BLK_DO_INDEX_HI);
 	}
-	if ((error = gmt_set_cols (GMT, GMT_OUT, n_output)) != GMT_NOERROR) {
+	if (!Ctrl->G.active && (error = GMT_Set_Columns (API, GMT_OUT, n_output, GMT_COL_FIX)) != GMT_NOERROR) {
 		Return (error);
 	}
 
@@ -484,7 +571,7 @@ int GMT_blockmode (void *V_API, int mode, void *args) {
 	if (GMT_Init_IO (API, GMT_IS_DATASET, GMT_IS_POINT, GMT_IN,  GMT_ADD_DEFAULT, 0, options) != GMT_NOERROR) {	/* Registers default input sources, unless already set */
 		Return (API->error);
 	}
-	if (GMT_Init_IO (API, GMT_IS_DATASET, GMT_IS_POINT, GMT_OUT, GMT_ADD_DEFAULT, 0, options) != GMT_NOERROR) {	/* Registers default output destination, unless already set */
+	if (!Ctrl->G.active && GMT_Init_IO (API, GMT_IS_DATASET, GMT_IS_POINT, GMT_OUT, GMT_ADD_DEFAULT, 0, options) != GMT_NOERROR) {	/* Registers default output destination, unless already set */
 		Return (API->error);
 	}
 
@@ -502,7 +589,7 @@ int GMT_blockmode (void *V_API, int mode, void *args) {
 	is_integer = true;	/* Until proven otherwise */
 
 	do {	/* Keep returning records until we reach EOF */
-		if ((in = GMT_Get_Record (API, GMT_READ_DATA, NULL)) == NULL) {	/* Read next record, get NULL if special case */
+		if ((In = GMT_Get_Record (API, GMT_READ_DATA, NULL)) == NULL) {	/* Read next record, get NULL if special case */
 			if (gmt_M_rec_is_error (GMT)) {		/* Bail if there are any read errors */
 				Return (GMT_RUNTIME_ERROR);
 			}
@@ -510,6 +597,7 @@ int GMT_blockmode (void *V_API, int mode, void *args) {
 				break;
 			continue;							/* Go back and read the next record */
 		}
+		in = In->data;	/* Only need to process numerical part here */
 
 		if (gmt_M_is_dnan (in[GMT_Z])) 		/* Skip if z = NaN */
 			continue;
@@ -578,18 +666,37 @@ int GMT_blockmode (void *V_API, int mode, void *args) {
 		n_alloc = n_pitched;
 		data = gmt_M_malloc (GMT, data, 0, &n_alloc, struct BLK_DATA);
 	}
+	w_col = gmt_get_cols (GMT, GMT_OUT) - 1;	/* Weights always reported in last output column */
+	fcol[4] = (unsigned int)w_col;				/* Since we don't know what it is until parsed */
 
 	/* Ready to go. */
 
-	if (GMT_Begin_IO (API, GMT_IS_DATASET, GMT_OUT, GMT_HEADER_ON) != GMT_NOERROR) {	/* Enables data output and sets access mode */
-		gmt_M_free (GMT, data);
-		Return (API->error);
+	if (Ctrl->G.active) {	/* Create the grid(s) */
+		char *remarks[BLK_N_FIELDS] = {"Median value per bin", "L1 scale per bin", "Lowest value per bin", "Highest value per bin", "Weight per bin"};
+		for (k = 0; k < BLK_N_FIELDS; k++) {
+			if (!Ctrl->A.selected[k]) continue;
+			field[NF] = fcol[k];	/* Just keep record of which fields we are actually using */
+			code[NF]  = fcode[k];
+			if ((GridOut[NF] = GMT_Create_Data (API, GMT_IS_GRID, GMT_IS_SURFACE, GMT_CONTAINER_AND_DATA, NULL, NULL, NULL, \
+				GMT_GRID_DEFAULT_REG, GMT_NOTSET, NULL)) == NULL) Return (API->error);
+			if (GMT_Set_Comment (API, GMT_IS_GRID, GMT_COMMENT_IS_TITLE, "Grid produced by blockmode", GridOut[NF]) != GMT_NOERROR) Return (API->error);
+			if (GMT_Set_Comment (API, GMT_IS_GRID, GMT_COMMENT_IS_OPTION | GMT_COMMENT_IS_COMMAND, options, GridOut[NF]) != GMT_NOERROR) Return (API->error);
+			if (GMT_Set_Comment (API, GMT_IS_GRID, GMT_COMMENT_IS_REMARK, remarks[k], GridOut[NF])) Return (API->error);
+			if (G == NULL) G = GridOut[NF];	/* First grid header used to get node later */
+			for (node = 0; node < G->header->size; node++) 
+				GridOut[NF]->data[node] = GMT->session.f_NaN;
+			NF++;	/* Number of actual field grids */
+		}
 	}
-	if (GMT_Set_Geometry (API, GMT_OUT, GMT_IS_POINT) != GMT_NOERROR) {	/* Sets output geometry */
-		Return (API->error);
+	else {	/* Get ready for rec-by-rec output */
+		if (GMT_Begin_IO (API, GMT_IS_DATASET, GMT_OUT, GMT_HEADER_ON) != GMT_NOERROR) {	/* Enables data output and sets access mode */
+			Return (API->error);
+		}
+		if (GMT_Set_Geometry (API, GMT_OUT, GMT_IS_POINT) != GMT_NOERROR) {	/* Sets output geometry */
+			Return (API->error);
+		}
 	}
 
-	w_col = gmt_get_cols (GMT, GMT_OUT) - 1;	/* Weights always reported in last output column */
 	if (emode) {					/* Index column last, with weight col just before */
 		i_col = w_col--;
 		old_format = GMT->current.io.o_format[i_col];		/* Need to restore this at end */
@@ -605,6 +712,10 @@ int GMT_blockmode (void *V_API, int mode, void *args) {
 		Ctrl->Q.active = true;	/* Cannot do modal positions */
 	}
 
+	Out = gmt_new_record (GMT, out, NULL);	/* Since we only need to worry about numerics in this module */
+
+	if (emode) src_id_ptr = &src_id;
+	
 	/* Find n_in_cell and write appropriate output  */
 
 	first_in_cell = n_cells_filled = nz = 0;
@@ -644,7 +755,7 @@ int GMT_blockmode (void *V_API, int mode, void *args) {
 			if (Ctrl->D.active)
 				out[GMT_Z] = bin_mode (GMT, &data[first_in_cell], n_in_cell, GMT_Z, B);
 			else
-				out[GMT_Z] = weighted_mode (&data[first_in_cell], weight, emode, n_in_cell, GMT_Z, &src_id);
+				out[GMT_Z] = weighted_mode (&data[first_in_cell], weight, emode, n_in_cell, GMT_Z, src_id_ptr);
 			if (Ctrl->Q.active) {
 				i_n_in_cell = 1.0 / n_in_cell;
 				out[GMT_X] *= i_n_in_cell;
@@ -728,19 +839,43 @@ int GMT_blockmode (void *V_API, int mode, void *args) {
 		if (Ctrl->W.weighted[GMT_OUT]) out[w_col] = (Ctrl->W.sigma[GMT_OUT]) ? 1.0 / weight : weight;
 		if (emode) out[i_col] = (double)src_id;
 
-		GMT_Put_Record (API, GMT_WRITE_DATA, out);	/* Write this to output */
+		if (Ctrl->G.active) {
+			row = gmt_M_grd_y_to_row (GMT, out[GMT_Y], Grid->header);
+			col = gmt_M_grd_x_to_col (GMT, out[GMT_X], Grid->header);
+			node = gmt_M_ijp (Grid->header, row, col);	/* Bin node */
+			for (k = 0; k < NF; k++)
+				GridOut[k]->data[node] = (gmt_grdfloat)out[field[k]];
+		}
+		else
+		GMT_Put_Record (API, GMT_WRITE_DATA, Out);	/* Write this to output */
 
 		n_cells_filled++;
 		first_in_cell = first_in_new_cell;
 	}
-	if (GMT_End_IO (API, GMT_OUT, 0) != GMT_NOERROR) {	/* Disables further data output */
-		Return (API->error);
+
+	if (Ctrl->G.active) {	/* Writes the grid(s) */
+		unsigned int kk;
+		char file[PATH_MAX] = {""};
+		for (k = kk = 0; k < NF; k++) {
+			if (strstr (Ctrl->G.file[kk], "%s"))
+				sprintf (file, Ctrl->G.file[kk], code[k]);
+			else
+				strncpy (file, Ctrl->G.file[kk], PATH_MAX-1);
+			if (GMT_Write_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_CONTAINER_AND_DATA, NULL, file, GridOut[k]) != GMT_NOERROR) {
+				Return (API->error);
+			}
+			if (Ctrl->G.n > 1) kk++;	/* Only true for APIs */
+		}
+	}
+	else if (GMT_End_IO (API, GMT_OUT, 0) != GMT_NOERROR) {	/* Disables further data output */
+		error = API->error;
+	}
+	else {
+		n_lost = n_read - n_pitched;	/* Number of points that did not get used */
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "N read: %" PRIu64 " N used: %" PRIu64 " outside_area: %" PRIu64 " N cells filled: %" PRIu64 "\n", n_read, n_pitched, n_lost, n_cells_filled);
+		error = GMT_NOERROR;
 	}
 
-	n_lost = n_read - n_pitched;	/* Number of points that did not get used */
-	GMT_Report (API, GMT_MSG_VERBOSE, "N read: %" PRIu64 " N used: %" PRIu64 " outside_area: %" PRIu64 " N cells filled: %" PRIu64 "\n", n_read, n_pitched, n_lost, n_cells_filled);
-
-	gmt_M_free (GMT, data);
 	if (do_extra) gmt_M_free (GMT, z_tmp);
 
 	if (emode) {
@@ -752,5 +887,5 @@ int GMT_blockmode (void *V_API, int mode, void *args) {
 		gmt_M_free (GMT, B);
 	}
 
-	Return (GMT_NOERROR);
+	Return (error);
 }
