@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id: common_string.c 19521 2017-12-27 22:38:24Z pwessel $
  *
- *	Copyright (c) 1991-2018 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  * common_string.c contains code shared between GMT and PSL
@@ -30,6 +29,7 @@
  *  gmt_strstrip            Strip leading and trailing whitespace from string
  *  gmt_strlshift           Left shift a string by n characters
  *  gmt_strrepc             Replaces all occurrences of a char in the string
+ *  gmt_strrep              Replaces all occurrences of a string s2 in the string s1 by s3
  *  gmt_strlcmp             Compares strings (ignoring case) until first reaches null character
  *  gmt_strtok              Reiterant replacement of strtok
  *  gmt_strtok_m            A Matlab style strtok  
@@ -40,7 +40,6 @@
  *  strsepz                 Like strsep but ignores empty fields
  *  stresep                 Like strsep but takes an additional argument esc in order
  *                          to ignore escaped chars (from NetBSD)
- *  gmt_match_string_in_file    Return true if a string is found in file
  *  basename                Extract the base portion of a pathname
  */
 
@@ -82,7 +81,7 @@ void gmt_chop (char *string) {
 		*p = '\0';
 }
 
-char *gmt_get_ext (char *string) {
+char *gmt_get_ext (const char *string) {
 	/* Returns a pointer to the filename extension (e.g., .ps)  or NULL if not found. */
 	char *p;
 	assert (string != NULL); /* NULL pointer */
@@ -199,7 +198,7 @@ unsigned int gmt_strtok (const char *string, const char *sep, unsigned int *pos,
 void gmt_strtok_m (char *in, char **token, char **remain, char *sep) {
 	/* A Matlab style strtok. Note that 'token' and 'remain' must be virgin pointers,
 	   otherwise the memory they point to will be leaked because they are allocated here
-	   with strdup. For that reason the caller is responsable to free them after being consumed.
+	   with strdup. For that reason the caller is responsible to free them after being consumed.
 	 */
 	unsigned int pos = 0;
 	char *p, *s;
@@ -211,7 +210,7 @@ void gmt_strtok_m (char *in, char **token, char **remain, char *sep) {
 
 	token[0] = NULL;		remain[0] = NULL;
 
-	p = malloc(strlen(in)+1);
+	p = calloc(strlen(in)+1, sizeof (char));
 	if (gmt_strtok (in, s, &pos, p)) {
 		token[0] = strdup(p);
 		if (gmt_strtok (in, s, &pos, p))
@@ -232,19 +231,17 @@ unsigned int gmt_get_modifier (const char *string, char modifier, char *token) {
 	if (!string || string[0] == 0) return 0;	/* No hope */
 	len = strlen (string);
 	for (k = 0; start == 0 && k < (len-1); k++) {
-		if (string[k] == '\"') quoted = !quoted;	/* Initially false, becomes true at start of quote, then false when exit quote */
+		if (string[k] == '\"' || string[k] == '\'') quoted = !quoted;	/* Initially false, becomes true at start of quote, then false when exit quote */
 		if (quoted) continue;		/* Not look inside quoted strings */
 		if (string[k] == '+' && string[k+1] == modifier)	/* Found the start */
 			start = k+2;
 	}
 	if (start == 0) return 0;	/* Not found */
-	k = start;
-	while (k < len) {
-		if (string[k] == '\"') quoted = !quoted;	/* Initially false, becomes true at start of quote, then false when exit quote */
+	for (k = start; k < len; k++) {
+		if (string[k] == '\"' || string[k] == '\'') quoted = !quoted;	/* Initially false, becomes true at start of quote, then false when exit quote */
 		if (quoted) continue;	/* Not look inside quoted strings */
 		if (string[k] == '+')	/* Found the end */
 			break;
-		k++;
 	}
 	len = k - start;
 	if (token) {	/* Only pass back when token is not NULL */
@@ -529,13 +526,27 @@ char * strsep (char **stringp, const char *delim) {
 }
 #endif /* ifndef HAVE_STRSEP */
 
-/* Like strsep but ignores empty fields */
-char *strsepz (char **stringp, const char *delim) {
+/* Like strsep but ignores empty fields.
+ * PW: Made two near-identical version of strsepz:
+ *   strsepz:  As it was, but with an unused 3rd argument.
+ *   strsepzp: 3rd argument returns position in original stringp.
+ */
+
+char *strsepz (char **stringp, const char *delim, size_t *pos) {
 	char *c;
+	(void)(pos);
 	while ( (c = strsep(stringp, delim)) != NULL && *c == '\0' );
 	return c;
 }
 
+char *strsepzp (char **stringp, const char *delim, size_t *pos) {
+	char *c;
+	while ( (c = strsep(stringp, delim)) != NULL && *c == '\0' )
+		(*pos)++;
+	if (c) (*pos) += strlen (c) + 1;
+	return c;
+}
+
 /* $NetBSD: stresep.c,v 1.2 2007/12/06 22:07:07 seb Exp $
  *
  * Copyright (c) 1990, 1993
@@ -610,28 +621,6 @@ char *stresep(char **stringp, const char *delim, int esc) {
 	}
 }
 
-/* Return true if a string is found in file */
-int gmt_match_string_in_file (const char *filename, const char *string) {
-	FILE *fp;
-	char line[BUF_SIZE] = {""};
-
-	fp = fopen (filename, "r");
-	if (fp == NULL) return false;
-
-	/* search for string in each line */
-	while (fgets (line, BUF_SIZE-1, fp)) {
-		if (strstr (line, string)) {
-			/* line matches */
-			fclose (fp);
-			return true;
-		}
-	}
-
-	/* string not found in file */
-	fclose (fp);
-	return false;
-}
-
 /* $OpenBSD: basename.c,v 1.14 2005/08/08 08:05:33 espie Exp $
  *
  * Copyright (c) 1997, 2004 Todd C. Miller <Todd.Miller@courtesan.com>
@@ -728,3 +717,116 @@ char *basename(char *path) {
 	return (bname);
 }
 #endif
+
+/*
+ * strrep.c - C substring replacement.
+ *
+ * Written in 2011 by Drew Hess <dhess-src@bothan.net>.
+ * https://gist.github.com/dhess/975639
+ *
+ * To the extent possible under law, the author(s) have dedicated all
+ * copyright and related and neighboring rights to this software to
+ * the public domain worldwide. This software is distributed without
+ * any warranty.
+ *
+ * For the full statement of the dedication, see the Creative Commons
+ * CC0 Public Domain Dedication at
+ * <http://creativecommons.org/publicdomain/zero/1.0/>.
+ */
+
+ /*
+  * This file includes a main() function so that the file can be
+  * compiled into an executable, in order to run some simple test cases
+  * on the included strrep() function. If you want to use strrep in
+  * your own project, make sure you cut or comment out the main()
+  * function below.
+  *
+  * This function returns string s1 if string s2 is an empty string, or
+  * if s2 is not found in s1. If s2 is found in s1, the function
+  * returns a new null-terminated string whose contents are identical
+  * to s1, except that all occurrences of s2 in the original string s1
+  * are, in the new string, replaced by the string s3. The caller owns
+  * the new string.
+  *
+  * Strings s1, s2, and s3 must all be null-terminated strings. If any
+  * of s1, s2, or s3 are NULL, the function returns NULL, indicating an
+  * error condition. If any other error occurs, the function returns NULL.
+  *
+  * This code is written pedantically, primarily so that asserts can be
+  * used liberally. The code could certainly be optimized and/or made
+  * less verbose, and I encourage you to do that if you use strstr in
+  * your production code, once you're comfortable that it functions as
+  * intended. Each assert makes plain an invariant condition that is
+  * assumed to be true by the statement(s) that immediately follow the
+  * assert.  Some of the asserts are trivially true, as written, but
+  * they're included, nonetheless, in case you, in the process of
+  * optimizing or adapting the code for your own purposes, make a
+  * change that breaks an assumption made downstream by the original code.
+  */
+ 
+ char *gmt_strrep(const char *s1, const char *s2, const char *s3) {
+	 size_t s1_len, s2_len, s3_len, count, s1_without_s2_len, newstr_len, i, substr_len, remains;
+	 const char *p, *start_substr, *end_substr;
+	 char *newstr, *dst;
+	 if (!s1 || !s2 || !s3)
+		 return 0;
+	 s1_len = strlen(s1);
+	 if (!s1_len)
+		 return (char *)s1;
+	 s2_len = strlen(s2);
+	 if (!s2_len)
+		 return (char *)s1;
+ 
+	 /*
+	  * Two-pass approach: figure out how much space to allocate for
+	  * the new string, pre-allocate it, then perform replacement(s).
+	  */
+	 count = 0;
+	 p = s1;
+	 assert(s2_len); /* otherwise, strstr(s1,s2) will return s1. */
+	 do {
+		 p = strstr(p, s2);
+		 if (p) {
+			 p += s2_len;
+			 count++;
+		 }
+	 } while (p);
+ 
+	 if (!count)
+		 return (char *)s1;
+ 
+	 /*
+	  * The following size arithmetic is extremely cautious, to guard against size_t overflows.
+	  */
+	 assert(s1_len >= count * s2_len);
+	 assert(count);
+	 s1_without_s2_len = s1_len - count * s2_len;
+	 s3_len = strlen(s3);
+	 newstr_len = s1_without_s2_len + count * s3_len;
+	 if (s3_len && ((newstr_len <= s1_without_s2_len) || (newstr_len + 1 == 0))) /* Overflow. */
+		 return 0;
+	 
+	 newstr = (char *)calloc(newstr_len + 1, sizeof(char)); /* w/ terminator */
+	 if (!newstr)		/* ENOMEM, but no good way to signal it. */
+		 return 0;
+	 
+	 dst = newstr;
+	 start_substr = s1;
+	 for (i = 0; i != count; ++i) {
+		 end_substr = strstr(start_substr, s2);
+		 assert(end_substr);
+		 substr_len = end_substr - start_substr;
+		 memcpy(dst, start_substr, substr_len);
+		 dst += substr_len;
+		 memcpy(dst, s3, s3_len);
+		 dst += s3_len;
+		 start_substr = end_substr + s2_len;
+	 }
+ 
+	 /* copy remainder of s1, including trailing '\0' */
+	 remains = s1_len - (start_substr - s1) + 1;
+	 assert(dst + remains == newstr + newstr_len + 1);
+	 memcpy(dst, start_substr, remains);
+	 assert(strlen(newstr) == newstr_len);
+	 return newstr;
+}
