@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id: common_string.h 19521 2017-12-27 22:38:24Z pwessel $
  *
- *	Copyright (c) 1991-2018 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  * common_string.h contains prototypes of functions shared between GMT and PSL
@@ -29,8 +28,8 @@
  */
 
 #pragma once
-#ifndef _COMMON_STRING_H
-#define _COMMON_STRING_H
+#ifndef COMMON_STRING_H
+#define COMMON_STRING_H
 
 #ifdef __cplusplus      /* Basic C++ support */
 extern "C" {
@@ -46,9 +45,6 @@ extern "C" {
 
 #include <limits.h> /* defines PATH_MAX */
 #include <stdlib.h> /* defines _MAX_PATH on WIN32 */
-#if ! defined PATH_MAX && defined _MAX_PATH
-#	define PATH_MAX _MAX_PATH
-#endif
 #ifndef PATH_MAX
 #	define PATH_MAX 1024
 #endif
@@ -58,10 +54,11 @@ EXTERN_MSC void gmt_strtok_m (char *in, char **token, char **remain, char *sep);
 EXTERN_MSC unsigned int gmt_get_modifier (const char *string, char modifier, char *token);
 EXTERN_MSC void gmt_chop (char *string);
 EXTERN_MSC char *gmt_chop_ext (char *string);
-EXTERN_MSC char *gmt_get_ext (char *string);
+EXTERN_MSC char *gmt_get_ext (const char *string);
 EXTERN_MSC void gmt_strstrip(char *string, bool strip_leading);
 EXTERN_MSC void gmt_strlshift (char *string, size_t n);
 EXTERN_MSC void gmt_strrepc (char *string, int c, int r);
+EXTERN_MSC char *gmt_strrep(const char *s1, const char *s2, const char *s3);
 EXTERN_MSC size_t gmt_strlcmp (char *str1, char *str2);
 
 #ifdef WIN32
@@ -76,16 +73,19 @@ EXTERN_MSC char *strtok_r (char *s, const char *delim, char **save_ptr);
 
 #ifndef DECLARED_STRSEP
 EXTERN_MSC char *strsep (char **stringp, const char *delim);
+#elif !defined(HAVE_STRSEP)
+EXTERN_MSC char *strsep (char **stringp, const char *delim);
 #endif
-EXTERN_MSC char *strsepz (char **stringp, const char *delim);
+EXTERN_MSC char *strsepz  (char **stringp, const char *delim, size_t *pos);
+EXTERN_MSC char *strsepzp (char **stringp, const char *delim, size_t *pos);
 EXTERN_MSC char *stresep (char **stringp, const char *delim, int esc);
 
-EXTERN_MSC int gmt_match_string_in_file (const char *filename, const char *string);
-
+#ifndef HAVE_BASENAME
 EXTERN_MSC char *basename(char *path);
+#endif
 
 #ifdef __cplusplus
 }
 #endif
 
-#endif /* !_COMMON_STRING_H */
+#endif /* !COMMON_STRING_H */
