@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *	Copyright (c) 1991-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,14 +12,14 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  * API functions to support the filter1d application.
  *
  * Author:	Walter H. F. Smith
  * Date:	1-JAN-2010
- * Version:	5 API
+ * Version:	6 API
  *
  * Brief synopsis:  filter1d_func will read N columns of data from file
  * or stdin and return filtered output at user-selected positions.
@@ -40,12 +39,13 @@
 
 #include "gmt_dev.h"
 
-#define THIS_MODULE_NAME	"filter1d"
+#define THIS_MODULE_CLASSIC_NAME	"filter1d"
+#define THIS_MODULE_MODERN_NAME	"filter1d"
 #define THIS_MODULE_LIB		"core"
 #define THIS_MODULE_PURPOSE	"Time domain filtering of 1-D data tables"
 #define THIS_MODULE_KEYS	"<D{,>D},FD(1"
 #define THIS_MODULE_NEEDS	""
-#define THIS_MODULE_OPTIONS "-:>Vabdefghio" GMT_OPT("HMm")
+#define THIS_MODULE_OPTIONS "-:>Vabdefghijo" GMT_OPT("HMm")
 
 /* Control structure for filter1d */
 
@@ -69,8 +69,12 @@ struct FILTER1D_CTRL {
 		bool active;
 		double value;
 	} L;
-	struct N {	/* -N<t_col> */
+	struct N {	/* -N<t_col> or -Nc|<unit>[+a] */
 		bool active;
+		bool add_col;
+		char unit;
+		unsigned int mode;
+		unsigned int spatial;
 		int col;
 	} N;
 	struct Q {	/* -Q<factor> */
@@ -81,9 +85,9 @@ struct FILTER1D_CTRL {
 		bool active;
 		double value;
 	} S;
-	struct T {	/* -T<tmin/tmax/tinc>[+n] */
+	struct T {	/* -T<tmin/tmax/tinc>[+a|n] */
 		bool active;
-		double min, max, inc;
+		struct GMT_ARRAY T;
 	} T;
 };
 
@@ -152,6 +156,7 @@ struct FILTER1D_INFO {	/* Control structure for all aspects of the filter setup
 	double extreme;			/* Extreme value [for the l|L|u|U filter] */
 
 	struct GMT_DATASET *Fin;	/* Pointer to table with custom weight coefficients (optional) */
+	struct GMT_ARRAY T;
 };
 
 EXTERN_MSC unsigned int gmt_parse_d_option (struct GMT_CTRL *GMT, char *arg);
@@ -169,16 +174,17 @@ GMT_LOCAL void *New_Ctrl (struct GMT_CTRL *GMT) {	/* Allocate and initialize a n
 GMT_LOCAL void Free_Ctrl (struct GMT_CTRL *GMT,struct FILTER1D_CTRL *C) {	/* Deallocate control structure */
 	if (!C) return;
 	gmt_M_str_free (C->F.file);
+	gmt_free_array (GMT, &(C->T.T));
 	gmt_M_free (GMT, C);
 }
 
 GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
-	gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
+	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: filter1d [<table>] -F<type><width>[<modifiers>] [-D<increment>] [-E] [-I<ignore_val>]\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t[-L<lack_width>] [-N<t_col>] [-Q<q_factor>] [-S<symmetry>] [-T<t_min>/<t_max>/<t_inc>[+n]]\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t[%s] [%s] [%s] [%s] [%s]\n\t[%s] [%s]\n\t[%s] [%s] [%s]\n\n",
-		GMT_V_OPT, GMT_b_OPT, GMT_d_OPT, GMT_e_OPT, GMT_f_OPT, GMT_g_OPT, GMT_h_OPT, GMT_i_OPT, GMT_o_OPT, GMT_colon_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "usage: %s [<table>] -F<type><width>[<modifiers>] [-D<increment>] [-E] [-I<ignore_val>]\n", name);
+	GMT_Message (API, GMT_TIME_NONE, "\t[-L<lack_width>] [-N<t_col>] [-Q<q_factor>] [-S<symmetry>] [-T[<min>/<max>/]<inc>[<unit>][+e|n|a]]\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t[%s] [%s] [%s] [%s] [%s]\n\t[%s] [%s]\n\t[%s] [%s] [%s] [%s] [%s]\n\n",
+		GMT_V_OPT, GMT_b_OPT, GMT_d_OPT, GMT_e_OPT, GMT_f_OPT, GMT_g_OPT, GMT_h_OPT, GMT_i_OPT, GMT_j_OPT, GMT_o_OPT, GMT_colon_OPT, GMT_PAR_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
@@ -201,7 +207,7 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t     u: Upper : Return maximum of all points.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t     U: Upper- : Return maximum of all negative points.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   Upper case type B, C, G, M, P, F will use robust filter versions,\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   i.e., replace outliers (2.5 L1 scale off median) with median during filtering.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   i.e., replace outliers (2.5 L1 scale (MAD) of median) with median during filtering.\n");
 
 	GMT_Message (API, GMT_TIME_NONE, "\n\tOPTIONS:\n");
 
@@ -221,13 +227,29 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t-S Check symmetry of data about window center.  Enter a factor\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   between 0 and 1.  If ( (abs(n_left - n_right)) / (n_left + n_right) ) > factor,\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   then no output will be given at this point [Default does not check Symmetry].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-T Make evenly spaced output time steps from <t_min> to <t_max> by <t_inc> [Default uses input times].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Append +n to <t_inc> to indicate number of t-values to produce instead.\n");
-	GMT_Option (API, "V,bi,bo,d,e,f,g,h,i,o,:,.");
+	GMT_Message (API, GMT_TIME_NONE, "\t-T Make evenly spaced output time steps from <min> to <max> by <inc> [Default uses input times].\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   Append +n to indicate <inc> is the number of t-values to produce instead.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   If only <inc> is given, optionally append +e to keep increment exact [Default will adjust to fit range].\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   For absolute time filtering, append a valid time unit (%s) to the increment.\n", GMT_TIME_UNITS_DISPLAY);
+	GMT_Message (API, GMT_TIME_NONE, "\t   For spatial filtering with distance computed from the first two columns, specify increment as\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   <inc>[<unit>] and append a geospatial distance unit (%s) or c (for Cartesian distances).\n", GMT_LEN_UNITS_DISPLAY);
+	GMT_Message (API, GMT_TIME_NONE, "\t   Optionally, append +a to add such internal distances as a final output column [no distances added].\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   Alternatively, give a file with output times in the first column or a comma-separated list.\n");
+	GMT_Option (API, "V,bi,bo,d,e,f,g,h,i,j,o,:,.");
 	
 	return (GMT_MODULE_USAGE);
 }
 
+GMT_LOCAL char set_unit_and_mode (const char *arg, unsigned int *mode) {
+	unsigned int k = 0;
+	*mode = GMT_GREATCIRCLE;	/* Default is great circle distances */
+	switch (arg[0]) {
+		case '-': *mode = GMT_FLATEARTH;	k = 1; break;
+		case '+': *mode = GMT_GEODESIC;		k = 1; break;
+	}
+	return (arg[k]);
+}
+
 GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct FILTER1D_CTRL *Ctrl, struct GMT_OPTION *options) {
 	/* This parses the options provided to filter1d and sets parameters in CTRL.
 	 * Any GMT common options will override values set previously by other commands.
@@ -237,7 +259,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct FILTER1D_CTRL *Ctrl, struct GM
 
 	unsigned int n_errors = 0;
 	int sval = 0;
-	char *c = NULL, p, txt_a[GMT_LEN64] = {""}, txt_b[GMT_LEN64] = {""};
+	char *c = NULL, p, txt[GMT_LEN64] = {""}, *t_arg = NULL;
 	struct GMT_OPTION *opt = NULL;
 	struct GMTAPI_CTRL *API = GMT->parent;
 
@@ -295,8 +317,8 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct FILTER1D_CTRL *Ctrl, struct GM
 				}
 				break;
 			case 'I':	/* DEPRECATED: Activate the ignore option.  This is now -di<value> and happens during reading */
-				sprintf (txt_a, "i%s", opt->arg);
-				n_errors += gmt_parse_d_option (GMT, txt_a);
+				sprintf (txt, "i%s", opt->arg);
+				n_errors += gmt_parse_d_option (GMT, txt);
 				break;
 			case 'L':	/* Check for lack of data */
 				Ctrl->L.active = true;
@@ -307,7 +329,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct FILTER1D_CTRL *Ctrl, struct GM
 				if (gmt_M_compat_check (GMT, 4)) {	/* GMT4 LEVEL */
 					if (strchr (opt->arg, '/')) { /* Gave obsolete format */
 						int sval0;
-						GMT_Report (API, GMT_MSG_COMPAT, "Warning: -N<ncol>/<tcol> option is deprecated; use -N<tcol> instead.\n");
+						GMT_Report (API, GMT_MSG_COMPAT, "-N<ncol>/<tcol> option is deprecated; use -N<tcol> instead.\n");
 						if (sscanf (opt->arg, "%d/%d", &sval0, &sval) != 2) {
 							GMT_Report (API, GMT_MSG_NORMAL, "Syntax error -N option: Syntax is -N<tcol>\n");
 							++n_errors;
@@ -318,8 +340,30 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct FILTER1D_CTRL *Ctrl, struct GM
 				}
 				else
 					sval = atoi (opt->arg);
-				n_errors += gmt_M_check_condition (GMT, sval < 0, "Syntax error -N option: Time column cannot be negative.\n");
-				Ctrl->N.col = sval;
+				if (gmt_M_compat_check (GMT, 5) && (strstr (opt->arg, "+a") || (opt->arg[0] == 'g' || opt->arg[0] == 'c'))) {	/* Deprecated syntax */
+					GMT_Report (API, GMT_MSG_COMPAT, "-Nc|g[<unit>][+a] option is deprecated; use -T[<min>/<max>/]<int>[<unit>][+a|n] instead.\n");
+					if ((c = strstr (opt->arg, "+a"))) {	/* Want to output any spatial distances computed */
+						c[0] = '\0';	/* Chop off the modifier */
+						Ctrl->N.add_col = true;
+					}
+					if ((opt->arg[0] == 'g' || opt->arg[0] == 'c')) {	/* Spatial filtering */
+						Ctrl->N.spatial = 1;
+						if (opt->arg[0] == 'g')	{	/* Geospatial filtering */
+							if (opt->arg[1])
+								Ctrl->N.unit = set_unit_and_mode (&opt->arg[1], &Ctrl->N.mode);
+							else	/* Default to meter and great circle distances */
+								Ctrl->N.unit = set_unit_and_mode ("e", &Ctrl->N.mode);
+							Ctrl->N.spatial = 2;
+						}
+						else
+							Ctrl->N.unit = 'X';	/* For Cartesian distances */
+						if (c) c[0] = '+';	/* Restore */
+					}
+				}
+				else {
+					n_errors += gmt_M_check_condition (GMT, sval < 0, "Syntax error -N option: Time column cannot be negative.\n");
+					Ctrl->N.col = sval;
+				}
 				break;
 			case 'Q':	/* Assess quality of output */
 				Ctrl->Q.value = atof (opt->arg);
@@ -331,17 +375,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct FILTER1D_CTRL *Ctrl, struct GM
 				break;
 			case 'T':	/* Set output knots */
 				Ctrl->T.active = true;
-				if (sscanf (opt->arg, "%[^/]/%[^/]/%lf", txt_a, txt_b, &Ctrl->T.inc) != 3) {
-					GMT_Report (API, GMT_MSG_NORMAL, "Syntax error -T option: Syntax is -T<tmin>/<tmax>/<tinc>[+]\n");
-					++n_errors;
-				}
-				else {
-					gmt_scanf_arg (GMT, txt_a, GMT_IS_UNKNOWN, &Ctrl->T.min);
-					gmt_scanf_arg (GMT, txt_b, GMT_IS_UNKNOWN, &Ctrl->T.max);
-					if (strstr (opt->arg, "+n") || opt->arg[strlen(opt->arg)-1] == '+') {	/* Gave number of points instead; calculate inc */
-						Ctrl->T.inc = (Ctrl->T.max - Ctrl->T.min) / (Ctrl->T.inc - 1.0);
-					}
-				}
+				t_arg = opt->arg;
 				break;
 
 			default:	/* Report bad options */
@@ -350,11 +384,20 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct FILTER1D_CTRL *Ctrl, struct GM
 		}
 	}
 
+	if (Ctrl->T.active)	/* Do this one here since we need Ctrl->N.col to be set first, if selected */
+		n_errors += gmt_parse_array (GMT, 'T', t_arg, &(Ctrl->T.T), GMT_ARRAY_TIME | GMT_ARRAY_DIST | GMT_ARRAY_ROUND, Ctrl->N.col);
+	if (Ctrl->N.spatial) {	/* Obsolete -N settings propagated to -T */
+		Ctrl->T.T.spatial = Ctrl->N.spatial;
+		Ctrl->T.T.unit = Ctrl->N.unit;
+		Ctrl->T.T.distmode = Ctrl->N.mode;
+	}
+	if (Ctrl->N.add_col) Ctrl->T.T.add = true;	/* Obsolete -N+a settings propagated to -T */
+	
 	/* Check arguments */
 
 	n_errors += gmt_M_check_condition (GMT, !Ctrl->F.active, "Syntax error: -F is required\n");
 	n_errors += gmt_M_check_condition (GMT, Ctrl->D.active && Ctrl->D.inc <= 0.0, "Syntax error -D: must give positive increment\n");
-	n_errors += gmt_M_check_condition (GMT, Ctrl->T.active && (Ctrl->T.max - Ctrl->T.min) < Ctrl->F.width, "Syntax error -T option: Output interval < filterwidth\n");
+	n_errors += gmt_M_check_condition (GMT, Ctrl->T.active && Ctrl->T.T.set == 3 && (Ctrl->T.T.max - Ctrl->T.T.min) < Ctrl->F.width, "Syntax error -T option: Output interval < filterwidth\n");
 	n_errors += gmt_M_check_condition (GMT, Ctrl->L.active && (Ctrl->L.value < 0.0 || Ctrl->L.value > Ctrl->F.width) , "Syntax error -L option: Unreasonable lack-of-data interval\n");
 	n_errors += gmt_M_check_condition (GMT, Ctrl->S.active && (Ctrl->S.value < 0.0 || Ctrl->S.value > 1.0) , "Syntax error -S option: Enter a factor between 0 and 1\n");
 	n_errors += gmt_M_check_condition (GMT, Ctrl->Q.active && (Ctrl->Q.value < 0.0 || Ctrl->Q.value > 1.0), "Syntax error -Q option: Enter a factor between 0 and 1\n");
@@ -435,11 +478,11 @@ GMT_LOCAL int set_up_filter (struct GMT_CTRL *GMT, struct FILTER1D_INFO *F) {
 	if (F->out_at_time) {
 		/* populate F->t_start and F->t_stop */
 		double	t_shift;
-		if (F->t_start_t < t_0) /* user defined t_start_t outside bounds */
+		if (F->T.set == 1 || F->t_start_t < t_0) /* Not set or user defined t_start_t outside bounds */
 			F->t_start = t_0;
 		else
 			F->t_start = F->t_start_t;
-		if (F->t_stop_t > t_1) /* user defined t_stop_t outside bounds */
+		if (F->T.set == 1 || F->t_stop_t > t_1) /* Not set or user defined t_stop_t outside bounds */
 			F->t_stop = t_1;
 		else
 			F->t_stop = F->t_stop_t;
@@ -447,7 +490,7 @@ GMT_LOCAL int set_up_filter (struct GMT_CTRL *GMT, struct FILTER1D_INFO *F) {
 		if (!F->use_ends) {
 			/* remove filter half width from bounds */
 			F->t_start += F->half_width;
-			F->t_stop -= F->half_width;
+			F->t_stop  -= F->half_width;
 		}
 
 		/* align F->t_start and F->t_stop to F->t_int */
@@ -473,7 +516,7 @@ GMT_LOCAL int set_up_filter (struct GMT_CTRL *GMT, struct FILTER1D_INFO *F) {
 		}
 	}
 
-	GMT_Report (GMT->parent, GMT_MSG_VERBOSE, "F width: %g Resolution: %g Start: %g Stop: %g\n", F->filter_width, F->dt, F->t_start, F->t_stop);
+	GMT_Report (GMT->parent, GMT_MSG_LONG_VERBOSE, "F width: %g Resolution: %g Start: %g Stop: %g\n", F->filter_width, F->dt, F->t_start, F->t_stop);
 
 	return (0);
 }
@@ -533,49 +576,49 @@ GMT_LOCAL void get_robust_estimates (struct GMT_CTRL *GMT, struct FILTER1D_INFO
 }
 
 GMT_LOCAL int do_the_filter (struct GMTAPI_CTRL *C, struct FILTER1D_INFO *F) {
-	uint64_t i_row, left, right, n_l, n_r;
-	uint64_t i_t_output = 0, n_in_filter, n_for_call, n_good_ones;
+	uint64_t i_row, left, right, n_l, n_r, k = 0, n_for_call, n_good_ones, last_k;
 	uint64_t iq, i_col, diff;
-	int64_t i_f_wt;
+	int64_t i_f_wt, n_in_filter;
 	bool *good_one = NULL;	/* Pointer to array of logicals [one per column]  */
 	double t_time, delta_time, wt, val, med, scl, small, symmetry;
 	double *wt_sum = NULL;		/* Pointer for array of weight sums [each column]  */
 	double *data_sum = NULL;	/* Pointer for array of data * weight sums [columns]  */
 	double *outval = NULL;
+	struct GMT_RECORD *Out = NULL;
 	struct GMT_CTRL *GMT = C->GMT;
 
 	outval = gmt_M_memory (GMT, NULL, F->n_cols, double);
 	good_one = gmt_M_memory (GMT, NULL, F->n_cols, bool);
 	wt_sum = gmt_M_memory (GMT, NULL, F->n_cols, double);
 	data_sum = gmt_M_memory (GMT, NULL, F->n_cols, double);
+	Out = gmt_new_record (GMT, NULL, NULL);
 
-	if(!F->out_at_time) {	/* Position i_t_output at first output time  */
-		for(i_t_output = 0; F->data[F->t_col][i_t_output] < F->t_start; ++i_t_output);
-		small = (F->data[F->t_col][1] - F->data[F->t_col][0]);
-	}
-	else
-		small = F->t_int;
-
-	small *= GMT_CONV8_LIMIT;
-	t_time = F->t_start;
 	left = right = 0;		/* Left/right end of filter window */
-
 	iq = lrint (F->q_factor);
 
-	while (t_time <= (F->t_stop + small)) {
-		while ((t_time - F->data[F->t_col][left] - small) > F->half_width) ++left;
-		while (right < F->n_rows && (F->data[F->t_col][right] - t_time - small) <= F->half_width) ++right;
-		n_in_filter = right - left;
-		if ( (!(n_in_filter)) || (F->check_lack && ( (F->filter_width / n_in_filter) > F->lack_width) ) ) {
-			if (F->out_at_time)
-				t_time += F->t_int;
-			else {
-				++i_t_output;
-				t_time = (i_t_output < F->n_rows) ? F->data[F->t_col][i_t_output] : F->t_stop + 1.0;
-			}
+	if (F->out_at_time) {	/* Set up equidistant output times given the min/max/inc given */
+		if (gmt_create_array (GMT, 'T', &(F->T), &(F->t_start), &(F->t_stop)))
+			return GMT_RUNTIME_ERROR;
+		k = 0;	/* Start at first output point */
+		last_k = F->T.n - 1;
+	}
+	else {	/* Duplicate an output time array from the input times but exclude ends (unless -E) */
+		F->T.array = gmt_M_memory (GMT, NULL, F->n_rows, double);
+		gmt_M_memcpy (F->T.array, F->data[F->t_col], F->n_rows, double);
+		for (k = 0; F->data[F->t_col][k] < F->t_start; ++k);	/* Bypass points outside */
+		for (last_k = F->n_rows-1; last_k && F->data[F->t_col][last_k] > F->t_stop; --last_k);	/* Bypass points outside */
+	}
+	small = (F->T.array[1] - F->T.array[0]) * GMT_CONV8_LIMIT;
+	
+	while (k <= last_k) {
+		while ((F->T.array[k] - F->data[F->t_col][left] - small) > F->half_width) ++left;
+		while (right < F->n_rows && (F->data[F->t_col][right] - F->T.array[k] - small) <= F->half_width) ++right;
+		n_in_filter = (int64_t)(right - left);
+		if ( n_in_filter <= 0 || (F->check_lack && ( (F->filter_width / n_in_filter) > F->lack_width) ) ) {
+			k++;	/* Nothing to filter, go to next output time */
 			continue;
 		}
-
+		t_time = F->T.array[k];	/* Current output time */
 		for (i_col = 0; i_col < F->n_cols; ++i_col) {
 			F->n_this_col[i_col] = 0;
 			wt_sum[i_col] = data_sum[i_col] = 0.0;
@@ -589,8 +632,8 @@ GMT_LOCAL int do_the_filter (struct GMTAPI_CTRL *C, struct FILTER1D_INFO *F) {
 		}
 
 		if (F->robust || F->filter_type > FILTER1D_CONVOLVE) {
-			if (n_in_filter > F->n_work_alloc) {
-				F->n_work_alloc = n_in_filter;
+			if (n_in_filter > (int64_t)F->n_work_alloc) {
+				F->n_work_alloc = (size_t)n_in_filter;
 				allocate_more_work_space (GMT, F);
 			}
 			for (i_row = left; i_row < right; ++i_row) {
@@ -634,19 +677,20 @@ GMT_LOCAL int do_the_filter (struct GMTAPI_CTRL *C, struct FILTER1D_INFO *F) {
 		if (F->filter_type > FILTER1D_CONVOLVE) {
 
 			/* Need to count how many good ones; use data_sum area  */
-
+			
+			Out->data = data_sum;
 			n_good_ones = 0;
 			for (i_col = 0; i_col < F->n_cols; ++i_col) {
 				if (i_col == F->t_col)
 					data_sum[i_col] = t_time;
 				else if (good_one[i_col]) {
-					data_sum[i_col] = (F->highpass) ? F->data[i_col][i_t_output] - F->this_loc[i_col] : F->this_loc[i_col];
+					data_sum[i_col] = (F->highpass) ? F->data[i_col][k] - F->this_loc[i_col] : F->this_loc[i_col];
 					++n_good_ones;
 				}
 				else
 					data_sum[i_col] = GMT->session.d_NaN;
 			}
-			if (n_good_ones) GMT_Put_Record (C, GMT_WRITE_DATA, data_sum);
+			if (n_good_ones) GMT_Put_Record (C, GMT_WRITE_DATA, Out);
 		}
 		else {
 			if (F->robust) for (i_col = 0; i_col < F->n_cols; ++i_col) F->n_this_col[i_col] = 0;
@@ -700,34 +744,29 @@ GMT_LOCAL int do_the_filter (struct GMTAPI_CTRL *C, struct FILTER1D_INFO *F) {
 				++n_good_ones;
 			}
 			if (n_good_ones) {
+				Out->data = outval;
 				for (i_col = 0; i_col < F->n_cols; ++i_col) {
 					if (i_col == F->t_col)
 						outval[i_col] = t_time;
 					else if (good_one[i_col]) {
 						outval[i_col] = (F->f_operator) ? data_sum[i_col] : data_sum[i_col] / wt_sum[i_col];
-						if (F->highpass) outval[i_col] = F->data[i_col][i_t_output] - outval[i_col];
+						if (F->highpass) outval[i_col] = F->data[i_col][k] - outval[i_col];
 					}
 					else
 						outval[i_col] = GMT->session.d_NaN;
 				}
-				GMT_Put_Record (C, GMT_WRITE_DATA, outval);
+				GMT_Put_Record (C, GMT_WRITE_DATA, Out);
 			}
 		}
-
-		/* Go to next output time */
-
-		if (F->out_at_time)
-			t_time += F->t_int;
-		else {
-			++i_t_output;
-			t_time = (i_t_output < F->n_rows) ? F->data[F->t_col][i_t_output] : F->t_stop + 1.0;
-		}
+		k++;	/* Go to next output point */
 	}
 
 	gmt_M_free (GMT, outval);
 	gmt_M_free (GMT, good_one);
 	gmt_M_free (GMT, wt_sum);
 	gmt_M_free (GMT, data_sum);
+	gmt_M_free (GMT, Out);
+	gmt_M_free (GMT, F->T.array);
 
 	return (0);
 }
@@ -792,9 +831,10 @@ GMT_LOCAL void load_parameters_filter1d (struct FILTER1D_INFO *F, struct FILTER1
 	F->check_q = Ctrl->Q.active;
 	F->check_asym = Ctrl->S.active;
 	F->sym_coeff = Ctrl->S.value;
-	F->t_start_t = F->t_start = Ctrl->T.min;
-	F->t_stop_t = F->t_stop = Ctrl->T.max;
-	F->t_int =Ctrl->T.inc;
+	F->t_start_t = F->t_start = Ctrl->T.T.min;
+	F->t_stop_t = F->t_stop = Ctrl->T.T.max;
+	F->t_int = Ctrl->T.T.inc;
+	F->T = Ctrl->T.T;
 	F->out_at_time = Ctrl->T.active;
 	F->highpass = Ctrl->F.highpass;
 }
@@ -807,13 +847,14 @@ GMT_LOCAL void load_parameters_filter1d (struct FILTER1D_INFO *F, struct FILTER1
 int GMT_filter1d (void *V_API, int mode, void *args) {
 	uint64_t col, tbl, row, seg;
 	int error;
-	unsigned int save_col;
+	unsigned int save_col, n_out_cols;
 
 	double last_time, new_time, in;
 	
 	struct GMT_OPTION *options = NULL;
 	struct FILTER1D_INFO F;
 	struct GMT_DATASET *D = NULL;
+	struct GMT_DATASEGMENT *S = NULL;
 	struct FILTER1D_CTRL *Ctrl = NULL;
 
 	struct GMT_CTRL *GMT = NULL, *GMT_cpy = NULL;
@@ -825,12 +866,11 @@ int GMT_filter1d (void *V_API, int mode, void *args) {
 	if (mode == GMT_MODULE_PURPOSE) return (usage (API, GMT_MODULE_PURPOSE));	/* Return the purpose of program */
 	options = GMT_Create_Options (API, mode, args);	if (API->error) return (API->error);	/* Set or get option list */
 
-	if (!options || options->option == GMT_OPT_USAGE) bailout (usage (API, GMT_USAGE));	/* Return the usage message */
-	if (options->option == GMT_OPT_SYNOPSIS) bailout (usage (API, GMT_SYNOPSIS));	/* Return the synopsis */
+	if ((error = gmt_report_usage (API, options, 0, usage)) != GMT_NOERROR) bailout (error);	/* Give usage if requested */
 
 	/* Parse the command-line arguments */
 
-	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
 	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error, "Error parsing filter1d options\n");
 	Ctrl = New_Ctrl (GMT);		/* Allocate and initialize a new control structure */
 	if ((error = parse (GMT, Ctrl, options)) != 0) Return (error, "Error parsing filter1d options\n");
@@ -841,10 +881,10 @@ int GMT_filter1d (void *V_API, int mode, void *args) {
 	F.n_work_alloc = GMT_CHUNK;
 	F.equidist = true;
 
-	GMT_Report (API, GMT_MSG_VERBOSE, "Processing input table data\n");
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Processing input table data\n");
 
 	/* Register all data sources */
-	if (GMT_Init_IO (API, GMT_IS_DATASET, GMT_IS_NONE, GMT_IN, GMT_ADD_DEFAULT, 0, options) != GMT_NOERROR) {
+	if (GMT_Init_IO (API, GMT_IS_DATASET, GMT_IS_LINE, GMT_IN, GMT_ADD_DEFAULT, 0, options) != GMT_NOERROR) {
 		Return (API->error, "Error initializing input\n");
 	}
 	/* Read the input data into memory */
@@ -855,6 +895,20 @@ int GMT_filter1d (void *V_API, int mode, void *args) {
 		GMT_Report (API, GMT_MSG_NORMAL, "Input data have %d column(s) but at least 2 are needed\n", (int)D->n_columns);
 		Return (GMT_DIM_TOO_SMALL, "Not enough input columns\n");
 	}
+	n_out_cols = (unsigned int)D->n_columns;
+	if (Ctrl->T.T.spatial) {	/* Must add extra column to store distances and then compute them */
+		Ctrl->N.col = (int)D->n_columns;
+		gmt_adjust_dataset (GMT, D, D->n_columns + 1);
+		gmt_init_distaz (GMT, Ctrl->T.T.unit, Ctrl->T.T.distmode, GMT_MAP_DIST);
+		for (tbl = 0; tbl < D->n_tables; ++tbl) {	/* For each input table */
+			for (seg = 0; seg < D->table[tbl]->n_segments; ++seg) {	/* For each segment */
+				S = D->table[tbl]->segment[seg];
+				gmt_M_free (GMT, S->data[Ctrl->N.col]);
+				S->data[Ctrl->N.col] = gmt_dist_array (GMT, S->data[GMT_X], S->data[GMT_Y], S->n_rows, true);
+			}
+		}
+		if (Ctrl->T.T.add) n_out_cols++;
+	}
 	
 	load_parameters_filter1d (&F, Ctrl, D->n_columns);	/* Pass parameters from Control structure to Filter structure */
 
@@ -904,16 +958,16 @@ int GMT_filter1d (void *V_API, int mode, void *args) {
 			break;
 		case 'f':
 			F.filter_type = FILTER1D_CUSTOM;
-			if ((error = gmt_set_cols (GMT, GMT_IN, 1)) != 0) Return (error, "Error in gmt_set_cols");
+			if ((error = GMT_Set_Columns (API, GMT_IN, 1, GMT_COL_FIX_NO_TEXT)) != 0) Return (error, "Error in GMT_Set_Columns");
 			save_col = GMT->current.io.col_type[GMT_IN][GMT_X];	/* Save col type in case it is a time column */
-			GMT->current.io.col_type[GMT_IN][GMT_X] = GMT_IS_FLOAT;	/* Always read the weights as floats */
+			gmt_set_column (GMT, GMT_IN, GMT_X, GMT_IS_FLOAT);	/* Always read the weights as floats */
 			gmt_disable_bhi_opts (GMT);	/* Do not want any -b -h -i to affect the reading from -F files */
 			if ((F.Fin = GMT_Read_Data (API, GMT_IS_DATASET, GMT_IS_FILE, GMT_IS_NONE, GMT_READ_NORMAL, NULL, Ctrl->F.file, NULL)) == NULL) {
 				Return (API->error, "Error Reading input\n");
 			}
 			gmt_reenable_bhi_opts (GMT);	/* Recover settings provided by user (if -b -h -i were used at all) */
-			GMT->current.io.col_type[GMT_IN][GMT_X] = save_col;	/* Reset this col type to whatever it actually is */
-			GMT_Report (API, GMT_MSG_VERBOSE, "Read %" PRIu64 " filter weights from file %s.\n", F.Fin->n_records, Ctrl->F.file);
+			gmt_set_column (GMT, GMT_IN, GMT_X, save_col);	/* Reset this col type to whatever it actually is */
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Read %" PRIu64 " filter weights from file %s.\n", F.Fin->n_records, Ctrl->F.file);
 			break;
 	}
 	if (F.filter_type > FILTER1D_CONVOLVE) F.robust = false;
@@ -930,16 +984,20 @@ int GMT_filter1d (void *V_API, int mode, void *args) {
 	if (GMT_Set_Geometry (API, GMT_OUT, GMT_IS_LINE) != GMT_NOERROR) {	/* Sets output geometry */
 		Return (API->error, "Error in GMT_Set_Geometry\n");
 	}
+	if (GMT_Set_Columns (API, GMT_OUT, n_out_cols, GMT_COL_FIX_NO_TEXT) != GMT_NOERROR) {
+		Return (API->error, "Error in GMT_Set_Columns\n");
+	}
 
 	allocate_space (GMT, &F);	/* Gets column-specific flags and uint64_t space */
 	
-	GMT_Report (API, GMT_MSG_VERBOSE, "Filter the data columns\n");
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Filter the data columns\n");
 	
 	for (tbl = 0; tbl < D->n_tables; ++tbl) {	/* For each input table */
 		for (seg = 0; seg < D->table[tbl]->n_segments; ++seg) {	/* For each segment */
 			/* Duplicate data and set up arrays and parameters needed to filter this segment */
-			if (D->table[tbl]->segment[seg]->n_rows > F.n_row_alloc) {
-				F.n_row_alloc = MAX (GMT_CHUNK, D->table[tbl]->segment[seg]->n_rows);
+			S = D->table[tbl]->segment[seg];
+			if (S->n_rows > F.n_row_alloc) {
+				F.n_row_alloc = MAX (GMT_CHUNK, S->n_rows);
 				allocate_data_space (GMT, &F);
 			}
 
@@ -950,15 +1008,17 @@ int GMT_filter1d (void *V_API, int mode, void *args) {
 				}
 			}
 			last_time = -DBL_MAX;
+			if (Ctrl->T.T.spatial == 2)	/* Ensure longitudes are in the same quadrants */
+				gmt_eliminate_lon_jumps (GMT, S->data[GMT_X], S->n_rows);
 
-			for (row = F.n_rows = 0; row < D->table[tbl]->segment[seg]->n_rows; ++row, ++F.n_rows) {
-				in = D->table[tbl]->segment[seg]->data[F.t_col][row];
+			for (row = F.n_rows = 0; row < S->n_rows; ++row, ++F.n_rows) {
+				in = S->data[F.t_col][row];
 				if (gmt_M_is_dnan (in)) continue;	/* Skip records with time == NaN */
 				new_time = in;
 				if (new_time < last_time) Return (GMT_DATA_READ_ERROR, "Error! Time decreases at line # %" PRIu64 "\n\tUse UNIX utility sort and then try again.\n", row);
 				last_time = new_time;
 				for (col = 0; col < F.n_cols; ++col) {
-					in = D->table[tbl]->segment[seg]->data[col][row];
+					in = S->data[col][row];
 					F.data[col][F.n_rows] = in;
 					if (F.robust || (F.filter_type == FILTER1D_MEDIAN) ) {
 						if (in > F.max_loc[col]) F.max_loc[col] = in;
@@ -966,14 +1026,14 @@ int GMT_filter1d (void *V_API, int mode, void *args) {
 					}
 				}
 			}
-			GMT_Report (API, GMT_MSG_VERBOSE, "Read %" PRIu64 " records from table %" PRIu64 ", segment %" PRIu64 "\n", F.n_rows, tbl, seg);
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Read %" PRIu64 " records from table %" PRIu64 ", segment %" PRIu64 "\n", F.n_rows, tbl, seg);
 			
 			/* FILTER: Initialize scale parameters and last_loc based on min and max of data  */
 
 			if (F.robust || (F.filter_type == FILTER1D_MEDIAN) ) {
 				for (col = 0; col < F.n_cols; ++col) {
 					F.min_scl[col] = 0.0;
-					F.max_scl[col] = 0.5 * (F.max_loc[col] - F.min_loc[col]);
+					F.max_scl[col]  = 0.5 * (F.max_loc[col] - F.min_loc[col]);
 					F.last_scl[col] = 0.5 * F.max_scl[col];
 					F.last_loc[col] = 0.5 * (F.max_loc[col] + F.min_loc[col]);
 				}
@@ -981,7 +1041,7 @@ int GMT_filter1d (void *V_API, int mode, void *args) {
 
 			if (set_up_filter (GMT, &F)) Return (GMT_RUNTIME_ERROR, "Fatal error during coefficient setup.\n");
 
-			if (GMT->current.io.multi_segments[GMT_OUT]) GMT_Put_Record (API, GMT_WRITE_SEGMENT_HEADER, D->table[tbl]->segment[seg]->header);
+			if (GMT->current.io.multi_segments[GMT_OUT]) GMT_Put_Record (API, GMT_WRITE_SEGMENT_HEADER, S->header);
 			
 			if (do_the_filter (API, &F)) Return (GMT_RUNTIME_ERROR, "Fatal error in filtering routine.\n");
 		}
@@ -991,7 +1051,7 @@ int GMT_filter1d (void *V_API, int mode, void *args) {
 		Return (API->error, "Error in End_IO\n");
 	}
 
-	if (F.n_multiples > 0) GMT_Report (API, GMT_MSG_VERBOSE, "Warning: %d multiple modes found\n", F.n_multiples);
+	if (F.n_multiples > 0) GMT_Report (API, GMT_MSG_VERBOSE, "%d multiple modes found\n", F.n_multiples);
 
 	free_space_filter1d (GMT, &F);
 
