@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt.c 19521 2017-12-27 22:38:24Z pwessel $
  *
- *	Copyright (c) 1991-2018 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,16 +12,16 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
- * Launcher for any GMT5 module via the corresponding function.
+ * Command-line launcher for any GMT module via the corresponding function.
  * Modules are loaded dynamically from the GMT core library, the
  * optional supplemental library, and any number of custom libraries
  * listed via GMT_CUSTOM_LIBS in gmt.conf.  Note: GMT_Call_Module checks
  * both <module> and gmt<module> in case the user left that part off.
  *
- * Version:	5
+ * Version:	6
  * Created:	17-June-2013
  *
  */
@@ -30,7 +29,7 @@
 #include "gmt_dev.h"
 
 #if !(defined(WIN32) || defined(NO_SIGHANDLER))
-#if	__APPLE__
+#ifdef	__APPLE__
 	/* Apple Xcode expects _Nullable to be defined but it is not if gcc */
 #ifndef _Nullable
 #	define _Nullable
@@ -56,7 +55,7 @@ int main (int argc, char *argv[]) {
 	bool gmt_main = false;			/* Set to true if no module was specified */
 	unsigned int modulename_arg_n = 0;	/* Argument index in argv[] that contains module name */
 	unsigned int mode = GMT_SESSION_NORMAL;	/* Default API mode */
-	unsigned int v_mode = GMT_MSG_COMPAT;		/* Default verbosity */
+	unsigned int v_mode = GMT_MSG_COMPAT;	/* Default verbosity */
 	struct GMTAPI_CTRL *api_ctrl = NULL;	/* GMT API control structure */
 	char *progname = NULL;			/* Last component from the pathname */
 	char *module = NULL;			/* Module name */
@@ -81,10 +80,7 @@ int main (int argc, char *argv[]) {
 	 * Note: Because first 16 bits of mode may be used for other things we must left-shift by 16 */
 	for (k = 1; k < argc; k++) if (!strncmp (argv[k], "-V", 2U)) v_mode = gmt_get_V (argv[k][2]);
 	if (v_mode) mode = (v_mode << 16);	/* Left-shift the mode by 16 */
-	/* Initialize new GMT session */
-	if ((api_ctrl = GMT_Create_Session (argv[0], GMT_PAD_DEFAULT, mode, NULL)) == NULL)
-		return GMT_RUNTIME_ERROR;
-	api_ctrl->internal = true;	/* This is a proper GMT commandline session (external programs will default to false) */
+
 	progname = strdup (basename (argv[0])); /* Last component from the pathname */
 	/* Remove any filename extensions added for example by the MSYS shell when executing gmt via symlinks */
 	gmt_chop_ext (progname);
@@ -92,21 +88,12 @@ int main (int argc, char *argv[]) {
 	/* Test if argv[0] contains a valid module name: */
 	module = progname;	/* Try this module name unless it equals PROGRAM_NAME in which case we just enter the test if argc > 1 */
 	gmt_main = !strcmp (module, PROGRAM_NAME);	/* true if running the main program, false otherwise */
-	if (gmt_main && argc == 3 && !strcmp (argv[1], "clear")) {	/* Clear something. */
-		if (!strcmp (argv[2], "cache") || !strcmp (argv[2], "all")) {	/* Clear the cache, then recreate empty directory */
-			if (gmt_remove_dir (api_ctrl, api_ctrl->GMT->session.CACHEDIR, true))
-				return GMT_RUNTIME_ERROR;
-		}
-		if (!strcmp (argv[2], "history") || !strcmp (argv[2], "all")) {	/* Clear the history */
-			if (gmt_remove_file (api_ctrl->GMT, "gmt.history"))
-				return GMT_RUNTIME_ERROR;
-		}
-		if (!strcmp (argv[2], "conf") || !strcmp (argv[2], "all")) {	/* Clear the configuration */
-			if (gmt_remove_file (api_ctrl->GMT, "gmt.conf"))
-				return GMT_RUNTIME_ERROR;
-		}
-		return GMT_NOERROR;
-	}
+
+	/* Initialize new GMT session */
+	if ((api_ctrl = GMT_Create_Session (argv[0], GMT_PAD_DEFAULT, mode, NULL)) == NULL)
+		return GMT_RUNTIME_ERROR;
+
+	api_ctrl->internal = true;	/* This is a proper GMT commandline session (external programs will default to false) */
 	if (gmt_main && argc > 1 && (!strcmp (argv[1], "gmtread") || !strcmp (argv[1], "read") || !strcmp (argv[1], "gmtwrite") || !strcmp (argv[1], "write"))) {
 		/* Cannot call [gmt]read or [gmt]write module from the command-line - only external APIs can do that. */
 		module = argv[1];	/* Name of module that does not exist, but will give reasonable message */
@@ -116,7 +103,7 @@ int main (int argc, char *argv[]) {
 		/* argv[0] does not contain a valid module name, and
 		 * argv[1] either holds the name of the module or an option: */
 		modulename_arg_n = 1;
-		module = argv[1];	/* Try this module name (Note: GMT_Call_Module will also check "gmt"<module> if <module> fails) */
+		module = argv[1];	/* Try this module name (Note: GMT_Call_Module will also check "gmt" <module> if just <module> fails) */
 		status = GMT_Call_Module (api_ctrl, module, GMT_MODULE_EXIST, NULL);
 	}
 
@@ -145,9 +132,9 @@ int main (int argc, char *argv[]) {
 			/* Print module list */
 			if (!strcmp (argv[arg_n], "--help")) {
 				fprintf (stderr, "\n\tGMT - The Generic Mapping Tools, Version %s [%u cores]\n", GMT_VERSION, api_ctrl->n_cores);
-				fprintf (stderr, "(c) 1991-%d Paul Wessel, Walter H. F. Smith, Remko Scharroo, Joaquim Luis, and Florian Wobbe\n\n", GMT_VERSION_YEAR);
+				fprintf (stderr, "\t(c) 1991-%d The GMT Team (https://www.generic-mapping-tools.org/team.html).\n\n", GMT_VERSION_YEAR);
 				fprintf (stderr, "Supported in part by the US National Science Foundation (http://www.nsf.gov/)\n");
-				fprintf (stderr, "and volunteers from around the world (see http://gmt.soest.hawaii.edu/).\n\n");
+				fprintf (stderr, "and volunteers from around the world.\n\n");
 
 				GMT_Call_Module (api_ctrl, NULL, GMT_MODULE_PURPOSE, NULL);
 				status = GMT_NOERROR;
@@ -161,7 +148,13 @@ int main (int argc, char *argv[]) {
 
 			/* Print version and exit */
 			else if (!strncmp (argv[arg_n], "--version", 5U)) {
-				fprintf (stdout, "%s\n", GMT_PACKAGE_VERSION_WITH_SVN_REVISION);
+				fprintf (stdout, "%s\n", GMT_PACKAGE_VERSION_WITH_GIT_REVISION);
+				status = GMT_NOERROR;
+			}
+
+			/* Show citation of the current release */
+			else if (!strncmp (argv[arg_n], "--show-citation", 15U)) {
+				fprintf(stdout, "%s\n", GMT_VERSION_CITATION);
 				status = GMT_NOERROR;
 			}
 
@@ -172,7 +165,7 @@ int main (int argc, char *argv[]) {
 			}
 
 			/* Show share directory */
-			else if (!strncmp (argv[arg_n], "--show-datadir", 11U)) {
+			else if (!strncmp (argv[arg_n], "--show-datadir", 14U)) {
 				if (api_ctrl->GMT->session.DATADIR == NULL)
 					fprintf(stdout, "Not set\n");
 				else
@@ -180,6 +173,18 @@ int main (int argc, char *argv[]) {
 				status = GMT_NOERROR;
 			}
 
+			/* Show URL of the remote GMT data server */
+			else if (!strncmp (argv[arg_n], "--show-dataserver", 17U)) {
+				fprintf(stdout, "%s\n", api_ctrl->GMT->session.DATASERVER);
+				status = GMT_NOERROR;
+			}
+
+			/* Show DOI of the current release */
+			else if (!strncmp (argv[arg_n], "--show-doi", 10U)) {
+				fprintf(stdout, "%s\n", GMT_VERSION_DOI);
+				status = GMT_NOERROR;
+			}
+
 			/* Show the directory that contains the 'gmt' executable */
 			else if (!strncmp (argv[arg_n], "--show-bindir", 10U)) {
 				fprintf (stdout, "%s\n", api_ctrl->GMT->init.runtime_bindir);
@@ -204,53 +209,102 @@ int main (int argc, char *argv[]) {
 				status = GMT_NOERROR;
 			}
 
+			/* print new shell template */
+			else if (!strncmp (argv[arg_n], "--new-script", 12U)) {
+				unsigned int type = 0;	/* Default is bash */
+				time_t right_now = time (NULL);
+				char *s = NULL, *txt = NULL, *shell[3] = {"bash", "csh", "batch"}, stamp[GMT_LEN32] = {""};
+				char *comment[3] = {"#", "#", "REM"};
+				char *name = gmt_putusername (NULL);
+				
+				strftime (stamp, GMT_LEN32, "%FT%T", localtime (&right_now));
+				if ((s = strchr (argv[arg_n], '=')) && s[1]) {	/* Gave a specific script language name */
+					if ((strstr (&s[1], shell[0]) || strstr (&s[1], shell[1]) || strstr (&s[1], shell[2]) || strstr (&s[1], "dos")))
+						txt = &s[1];
+					else
+						fprintf (stderr, "gmt: ERROR: --new-script language %s not recognized; default to bash\n\n", &s[1]);
+				}
+				else if ((txt = getenv ("shell")) == NULL) /* Likely not in a csh-type environment, try the Bourne shell environment variable SHELL */
+					txt = getenv ("SHELL");	/* Here txt is either a shell path or NULL */
+				if (txt && (!strcmp (txt, "batch") || !strcmp (txt, "dos"))) {	/* User asked for batch */
+					type = 2;		/* Select batch */
+					txt = shell[type];	/* Since user may have typed dos instead of batch */
+					printf ("@echo off\n");	/* Turn of the default echo-ing of commands */
+				}
+#ifdef WIN32
+				else if (txt == NULL) {	/* Assume batch if no shell setting exist under Windows */
+					type = 2;		/* Select batch */
+					printf ("@echo off\n");	/* Turn of the default echo-ing of commands */
+				}
+#endif
+				if (type == 0 && txt && (strstr (txt, "csh") || strstr (txt, "tcsh")))	/* Got csh or tcsh */
+					type = 1;	/* Select csh */
+				if (type < 2) {	/* Start the shell via env and pass -e to exit script upon error */
+					printf ("#!/usr/bin/env -S %s -e\n", shell[type]);
+					printf ("%s GMT modern mode %s template\n", comment[type], shell[type]);
+				}
+				printf ("%s Date:    %s\n%s User:    %s\n%s Purpose: Purpose of this script\n", comment[type], stamp, comment[type], name, comment[type]);
+				switch (type) {
+					case 0: printf ("export GMT_SESSION_NAME=$$	# Set a unique session name\n"); break;
+					case 1: printf ("setenv GMT_SESSION_NAME $$	# Set a unique session name\n"); break;
+					case 2: printf ("REM Set a unique session name:\n");	/* Can't use $$ so output the PPID of this process */
+						printf ("set GMT_SESSION_NAME=%s\n", api_ctrl->session_name);
+						break;
+				}
+				printf ("gmt begin figurename\n\t%s Place modern session commands here\ngmt end show\n", comment[type]);
+				gmt_M_str_free (name);
+				status = GMT_NOERROR;
+			}
+
 		} /* for (arg_n = 1; arg_n < argc; ++arg_n) */
 	} /* status == GMT_NOERROR */
 
 	if (status == GMT_NOT_A_VALID_MODULE) {
-		/* If we get here, we were called without a recognized modulename or option
+		/* If we get here, we were called without a recognized modulename or option.
 		 *
 		 * gmt.c is itself not a module and hence can use fprintf (stderr, ...). Any API needing a
 		 * gmt-like application will write one separately [see mex API and documentation] */
 
-		fprintf (stderr, "\n\tGMT - The Generic Mapping Tools, Version %s [%u cores]\n", GMT_VERSION, api_ctrl->n_cores);
-		fprintf (stderr, "(c) 1991-%d Paul Wessel, Walter H. F. Smith, Remko Scharroo, Joaquim Luis, and Florian Wobbe\n\n", GMT_VERSION_YEAR);
-		fprintf (stderr, "Supported in part by the US National Science Foundation (http://www.nsf.gov/)\n");
-		fprintf (stderr, "and volunteers from around the world (see http://gmt.soest.hawaii.edu/).\n\n");
-
-		fprintf (stderr, "This program comes with NO WARRANTY, to the extent permitted by law.\n");
-		fprintf (stderr, "You may redistribute copies of this program under the terms of the\n");
-		fprintf (stderr, "GNU Lesser General Public License (http://www.gnu.org/licenses/lgpl.html).\n");
-		fprintf (stderr, "For more information about legal matters, see the file named LICENSE.TXT.\n\n");
-		fprintf (stderr, "usage: %s [options]\n", PROGRAM_NAME);
-		fprintf (stderr, "       %s <module name> [<module-options>]\n\n", PROGRAM_NAME);
-		fprintf (stderr, "Session management:\n");
-		fprintf (stderr, "  gmt clear history | conf | cache | all\n");
-		fprintf (stderr, "                    Deletes gmt.history, gmt.conf, the user cache dir, or all of them\n\n");
-		fprintf (stderr, "options:\n");
-		fprintf (stderr, "  --help            List descriptions of available GMT modules.\n");
-		fprintf (stderr, "  --show-bindir     Show directory with GMT executables.\n");
-		fprintf (stderr, "  --show-cores      Print number of available cores.\n");
-		fprintf (stderr, "  --show-datadir    Show directory/ies with user data.\n");
-		fprintf (stderr, "  --show-modules    List all module names.\n");
-		fprintf (stderr, "  --show-library    Show path of the shared GMT library.\n");
-		fprintf (stderr, "  --show-plugindir  Show directory for plug-ins.\n");
-		fprintf (stderr, "  --show-sharedir   Show directory for shared GMT resources.\n");
-		fprintf (stderr, "  --version         Print GMT version number.\n\n");
-		fprintf (stderr, "if <module-options> is \'=\' we call exit (0) if module exist and non-zero otherwise.\n\n");
 		if (modulename_arg_n == 1 && module[0] != '-') {
-			fprintf (stderr, "ERROR: No module named %s was found.  This could mean one of three cases:\n", module);
+			fprintf (stderr, "\nERROR: No module named %s was found.  This could mean one of four cases:\n", module);
 			fprintf (stderr, "  1. There actually is no such module; please check your spelling.\n");
+			fprintf (stderr, "  2. You used a modern mode module name while running in GMT classic mode.\n");
 			if (strlen (GMT_SUPPL_LIB_NAME))
-				fprintf (stderr, "  2. Module exists in the GMT supplemental library, but the library could not be found.\n");
+				fprintf (stderr, "  3. Module exists in the GMT supplemental library, but the library could not be found.\n");
 			else
-				fprintf (stderr, "  2. Module exists in the GMT supplemental library, but the library was not installed.\n");
+				fprintf (stderr, "  3. Module exists in the GMT supplemental library, but the library was not installed.\n");
 			if (api_ctrl->n_shared_libs > 2)
-				fprintf (stderr, "  3. Module exists in a GMT custom library, but the library could not be found.\n");
+				fprintf (stderr, "  4. Module exists in a GMT custom library, but the library could not be found.\n");
 			else
-				fprintf (stderr, "  3. Module exists in a GMT custom library, but none was specified via GMT_CUSTOM_LIBS.\n");
+				fprintf (stderr, "  4. Module exists in a GMT custom library, but none was specified via GMT_CUSTOM_LIBS.\n");
 			fprintf (stderr, "Shared libraries must be in standard system paths or set via environmental parameter %s.\n\n", LIB_PATH);
 		}
+		else {
+			fprintf (stderr, "\n\tGMT - The Generic Mapping Tools, Version %s [%u cores]\n", GMT_VERSION, api_ctrl->n_cores);
+			fprintf (stderr, "\t(c) 1991-%d The GMT Team (https://www.generic-mapping-tools.org/team.html).\n\n", GMT_VERSION_YEAR);
+			fprintf (stderr, "Supported in part by the US National Science Foundation (http://www.nsf.gov/)\n");
+			fprintf (stderr, "and volunteers from around the world.\n\n");
+
+			fprintf (stderr, "GMT is distributed under the GNU LGP License (http://www.gnu.org/licenses/lgpl.html).\n\n");
+			fprintf (stderr, "usage: %s [options]\n", PROGRAM_NAME);
+			fprintf (stderr, "       %s <module name> [<module-options>]\n\n", PROGRAM_NAME);
+			fprintf (stderr, "options:\n");
+			fprintf (stderr, "  --help            List descriptions of available GMT modules.\n");
+			fprintf (stderr, "  --new-script[=L]  Write GMT modern mode script template to stdout.\n");
+			fprintf (stderr, "                    Optionally specify bash|csh|batch [Default is current shell]\n");
+			fprintf (stderr, "  --show-bindir     Show directory with GMT executables.\n");
+			fprintf (stderr, "  --show-citation   Show the most recent citation for GMT.\n");
+			fprintf (stderr, "  --show-cores      Show number of available cores.\n");
+			fprintf (stderr, "  --show-datadir    Show directory/ies with user data.\n");
+			fprintf (stderr, "  --show-dataserver Show URL of the remote GMT data server.\n");
+			fprintf (stderr, "  --show-doi        Show the DOI for the current release.\n");
+			fprintf (stderr, "  --show-modules    Show all module names.\n");
+			fprintf (stderr, "  --show-library    Show path of the shared GMT library.\n");
+			fprintf (stderr, "  --show-plugindir  Show directory for plug-ins.\n");
+			fprintf (stderr, "  --show-sharedir   Show directory for shared GMT resources.\n");
+			fprintf (stderr, "  --version         Print GMT version number.\n\n");
+			fprintf (stderr, "if <module-options> is \'=\' we call exit (0) if module exist and non-zero otherwise.\n\n");
+		}
 		status = GMT_RUNTIME_ERROR;
 	} /* status == GMT_NOT_A_VALID_OPTION */
 
