@@ -1,6 +1,5 @@
-/*      $Id$
- *
- *	Copyright (c) 1991-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+/*
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -12,7 +11,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  * Rewritten from original code by Robert Helie.  That code was hard-wired
@@ -40,8 +39,8 @@
 int gmt_is_agc_grid (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header)
 int gmt_agc_read_grd_info (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header)
 int gmt_agc_write_grd_info (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header)
-int gmt_agc_read_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, float *grid, double wesn[], unsigned int *pad, unsigned int complex_mode)
-int gmt_agc_write_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, float *grid, double wesn[], unsigned int *pad, unsigned int complex_mode)
+int gmt_agc_read_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, gmt_grdfloat *grid, double wesn[], unsigned int *pad, unsigned int complex_mode)
+int gmt_agc_write_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, gmt_grdfloat *grid, double wesn[], unsigned int *pad, unsigned int complex_mode)
 
 */
 
@@ -58,47 +57,47 @@ int gmt_agc_write_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, flo
 
 /* Local runctions used by the public functions: */
 
-GMT_LOCAL int agc_read_record (FILE *fpi, float z[ZBLOCKWIDTH][ZBLOCKHEIGHT]) {
+GMT_LOCAL int agc_read_record (FILE *fpi, gmt_grdfloat z[ZBLOCKWIDTH][ZBLOCKHEIGHT]) {
 	/* Reads one block of data, including pre- and post-headers */
 	size_t nitems;
-	float garbage[PREHEADSIZE];
+	gmt_grdfloat garbage[PREHEADSIZE];
 
-	if (gmt_M_fread (garbage, sizeof(float), PREHEADSIZE, fpi) < PREHEADSIZE)
+	if (gmt_M_fread (garbage, sizeof(gmt_grdfloat), PREHEADSIZE, fpi) < PREHEADSIZE)
 		return (GMT_GRDIO_READ_FAILED);
-	nitems = gmt_M_fread (z, sizeof(float), ZBLOCKWIDTH * ZBLOCKHEIGHT, fpi);
+	nitems = gmt_M_fread (z, sizeof(gmt_grdfloat), ZBLOCKWIDTH * ZBLOCKHEIGHT, fpi);
 
 	if (nitems != ZBLOCKWIDTH * ZBLOCKHEIGHT && !feof(fpi))
 		return (GMT_GRDIO_READ_FAILED);	/* Bad stuff */
-	if (gmt_M_fread (garbage, sizeof(float), POSTHEADSIZE, fpi) < POSTHEADSIZE)
+	if (gmt_M_fread (garbage, sizeof(gmt_grdfloat), POSTHEADSIZE, fpi) < POSTHEADSIZE)
 		return (GMT_GRDIO_READ_FAILED);
 	return (GMT_NOERROR);
 }
 
-GMT_LOCAL int agc_write_record (FILE *file, float rec[ZBLOCKWIDTH][ZBLOCKHEIGHT], float *prerec, float *postrec) {
+GMT_LOCAL int agc_write_record (FILE *file, gmt_grdfloat rec[ZBLOCKWIDTH][ZBLOCKHEIGHT], gmt_grdfloat *prerec, gmt_grdfloat *postrec) {
 	/* Writes one block of data, including pre- and post-headers */
-	if (gmt_M_fwrite (prerec, sizeof(float), PREHEADSIZE, file) < PREHEADSIZE)
+	if (gmt_M_fwrite (prerec, sizeof(gmt_grdfloat), PREHEADSIZE, file) < PREHEADSIZE)
 		return (GMT_GRDIO_WRITE_FAILED);
-	if (gmt_M_fwrite (rec, sizeof(float), ZBLOCKWIDTH * ZBLOCKHEIGHT, file) < ZBLOCKWIDTH * ZBLOCKHEIGHT)
+	if (gmt_M_fwrite (rec, sizeof(gmt_grdfloat), ZBLOCKWIDTH * ZBLOCKHEIGHT, file) < ZBLOCKWIDTH * ZBLOCKHEIGHT)
 		return (GMT_GRDIO_WRITE_FAILED);
-	if (gmt_M_fwrite (postrec, sizeof(float), POSTHEADSIZE, file) < POSTHEADSIZE)
+	if (gmt_M_fwrite (postrec, sizeof(gmt_grdfloat), POSTHEADSIZE, file) < POSTHEADSIZE)
 		return (GMT_GRDIO_WRITE_FAILED);
 	return (GMT_NOERROR);
 }
 
-GMT_LOCAL void agc_pack_header (float *prez, float *postz, struct GMT_GRID_HEADER *header) {
+GMT_LOCAL void agc_pack_header (gmt_grdfloat *prez, gmt_grdfloat *postz, struct GMT_GRID_HEADER *header) {
 	/* Places grd header info in the AGC header array */
-	gmt_M_memset (prez,  PREHEADSIZE,  float);
-	gmt_M_memset (postz, POSTHEADSIZE, float);
-	prez[0] = (float)header->wesn[YLO];
-	prez[1] = (float)header->wesn[YHI];
-	prez[2] = (float)header->wesn[XLO];
-	prez[3] = (float)header->wesn[XHI];
-	prez[4] = (float)header->inc[GMT_Y];
-	prez[5] = (float)header->inc[GMT_X];
-	prez[PREHEADSIZE-1] = (float)RECORDLENGTH;
+	gmt_M_memset (prez,  PREHEADSIZE,  gmt_grdfloat);
+	gmt_M_memset (postz, POSTHEADSIZE, gmt_grdfloat);
+	prez[0] = (gmt_grdfloat)header->wesn[YLO];
+	prez[1] = (gmt_grdfloat)header->wesn[YHI];
+	prez[2] = (gmt_grdfloat)header->wesn[XLO];
+	prez[3] = (gmt_grdfloat)header->wesn[XHI];
+	prez[4] = (gmt_grdfloat)header->inc[GMT_Y];
+	prez[5] = (gmt_grdfloat)header->inc[GMT_X];
+	prez[PREHEADSIZE-1] = (gmt_grdfloat)RECORDLENGTH;
 }
 
-GMT_LOCAL void agc_save_header (char *remark, float *agchead) {
+GMT_LOCAL void agc_save_header (char *remark, gmt_grdfloat *agchead) {
 	/* Place AGC header data in remark string */
 	char floatvalue[PARAMSIZE+1];	/* Allow space for final \0 */
 	unsigned int i;
@@ -122,16 +121,16 @@ int gmt_is_agc_grid (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header) {
 	FILE *fp = NULL;
 	int n_columns, n_rows;
 	off_t predicted_size;
-	float recdata[RECORDLENGTH], x_min, x_max, y_min, y_max, x_inc, y_inc;
+	gmt_grdfloat recdata[RECORDLENGTH], x_min, x_max, y_min, y_max, x_inc, y_inc;
 	struct stat buf;
-
-	if (!strcmp (header->name, "="))
+	struct GMT_GRID_HEADER_HIDDEN *HH = gmt_get_H_hidden (header);
+	if (!strcmp (HH->name, "="))
 		return (GMT_GRDIO_PIPE_CODECHECK);		/* Cannot check on pipes */
-	if (stat (header->name, &buf))
+	if (stat (HH->name, &buf))
 		return (GMT_GRDIO_STAT_FAILED);			/* Inquiry about file failed somehow */
-	if ((fp = gmt_fopen (GMT, header->name, "rb")) == NULL)
+	if ((fp = gmt_fopen (GMT, HH->name, "rb")) == NULL)
 		return (GMT_GRDIO_OPEN_FAILED);			/* Opening the file failed somehow */
-	if (gmt_M_fread (recdata, sizeof(float), RECORDLENGTH, fp) < RECORDLENGTH) {
+	if (gmt_M_fread (recdata, sizeof(gmt_grdfloat), RECORDLENGTH, fp) < RECORDLENGTH) {
 		gmt_fclose (GMT, fp);
 		return (GMT_GRDIO_READ_FAILED);
 	}
@@ -163,11 +162,12 @@ int gmt_is_agc_grid (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header) {
 	}
 	gmt_fclose (GMT, fp);
 	/* OK so far; see if file size matches the predicted size given the header info */
-	predicted_size = lrint (ceil ((double)n_rows /ZBLOCKHEIGHT) * ceil ((double)n_columns / ZBLOCKWIDTH)) * (ZBLOCKHEIGHT * ZBLOCKWIDTH + PREHEADSIZE + POSTHEADSIZE) * sizeof (float);
+	predicted_size = lrint (ceil ((double)n_rows /ZBLOCKHEIGHT) * ceil ((double)n_columns / ZBLOCKWIDTH)) * (ZBLOCKHEIGHT * ZBLOCKWIDTH + PREHEADSIZE + POSTHEADSIZE) * sizeof (gmt_grdfloat);
 	if (predicted_size == buf.st_size) {
 		/* Yes, appears to be an AGC grid */
 		header->type = GMT_GRID_IS_AF;
 		header->nan_value = 0.0f; /* NaN value for AGC format */
+		HH->orig_datatype = (sizeof (gmt_grdfloat) == sizeof (float)) ? GMT_FLOAT : GMT_DOUBLE;
 		return GMT_NOERROR;
 	}
 	return GMT_GRDIO_BAD_VAL;
@@ -177,23 +177,25 @@ int gmt_agc_read_grd_info (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header)
 	/* Read header info. NOTE: All AGC files are assumed to be gridline-registered */
 	unsigned int i;
 	FILE *fp = NULL;
-	float recdata[RECORDLENGTH], agchead[BUFFHEADSIZE];
+	gmt_grdfloat recdata[RECORDLENGTH], agchead[BUFFHEADSIZE];
+	struct GMT_GRID_HEADER_HIDDEN *HH = gmt_get_H_hidden (header);
 
-	if (!strcmp (header->name, "=")) {
+	if (!strcmp (HH->name, "=")) {
 #ifdef SET_IO_MODE
 		gmt_setmode (GMT, GMT_IN);
 #endif
 		fp = GMT->session.std[GMT_IN];
 	}
-	else if ((fp = gmt_fopen (GMT, header->name, "rb")) == NULL)
+	else if ((fp = gmt_fopen (GMT, HH->name, "rb")) == NULL)
 		return (GMT_GRDIO_OPEN_FAILED);
 
-	if (gmt_M_fread (recdata, sizeof(float), RECORDLENGTH, fp) < RECORDLENGTH) {
+	if (gmt_M_fread (recdata, sizeof(gmt_grdfloat), RECORDLENGTH, fp) < RECORDLENGTH) {
 		gmt_fclose (GMT, fp);
 		return (GMT_GRDIO_READ_FAILED);
 	}
 	
 	header->registration = GMT_GRID_NODE_REG;	/* Hardwired since no info about this in the header */
+	HH->orig_datatype = (sizeof (gmt_grdfloat) == sizeof (float)) ? GMT_FLOAT : GMT_DOUBLE;
 	header->wesn[XLO]  = recdata[2];
 	header->wesn[XHI]  = recdata[3];
 	header->wesn[YLO]  = recdata[0];
@@ -217,20 +219,21 @@ int gmt_agc_read_grd_info (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header)
 int gmt_agc_write_grd_info (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header) {
 	/* Write grd header info to file */
 	FILE *fp = NULL;
-	float prez[PREHEADSIZE], postz[POSTHEADSIZE];
+	gmt_grdfloat prez[PREHEADSIZE], postz[POSTHEADSIZE];
+	struct GMT_GRID_HEADER_HIDDEN *HH = gmt_get_H_hidden (header);
 
-	if (!strcmp (header->name, "=")) {
+	if (!strcmp (HH->name, "=")) {
 #ifdef SET_IO_MODE
 		gmt_setmode (GMT, GMT_OUT);
 #endif
 		fp = GMT->session.std[GMT_OUT];
 	}
-	else if ((fp = gmt_fopen (GMT, header->name, "rb+")) == NULL && (fp = gmt_fopen (GMT, header->name, "wb")) == NULL)
+	else if ((fp = gmt_fopen (GMT, HH->name, "rb+")) == NULL && (fp = gmt_fopen (GMT, HH->name, "wb")) == NULL)
 		return (GMT_GRDIO_CREATE_FAILED);
 	
 	agc_pack_header (prez, postz, header);	/* Stuff header info into the AGC arrays */
 
-	if (gmt_M_fwrite (prez, sizeof(float), PREHEADSIZE, fp) < PREHEADSIZE) {
+	if (gmt_M_fwrite (prez, sizeof(gmt_grdfloat), PREHEADSIZE, fp) < PREHEADSIZE) {
 		gmt_fclose (GMT, fp);
 		return (GMT_GRDIO_WRITE_FAILED);
 	}
@@ -240,7 +243,7 @@ int gmt_agc_write_grd_info (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header
 	return (GMT_NOERROR);
 }
 
-int gmt_agc_read_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, float *grid, double wesn[], unsigned int *pad, unsigned int complex_mode) {
+int gmt_agc_read_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, gmt_grdfloat *grid, double wesn[], unsigned int *pad, unsigned int complex_mode) {
 	/* header:     	grid structure header
 	 * grid:	array with final grid
 	 * wesn:	Sub-region to extract  [Use entire file if 0,0,0,0]
@@ -260,19 +263,20 @@ int gmt_agc_read_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, floa
 	unsigned int block, n_blocks, n_blocks_x, n_blocks_y;	/* Misc. counters */
 	unsigned int datablockcol, datablockrow, rowstart, rowend, colstart, row;
 	uint64_t ij, imag_offset, width_out;
-	float z[ZBLOCKWIDTH][ZBLOCKHEIGHT];
+	gmt_grdfloat z[ZBLOCKWIDTH][ZBLOCKHEIGHT];
 	FILE *fp = NULL;			/* File pointer to data or pipe */
+	struct GMT_GRID_HEADER_HIDDEN *HH = gmt_get_H_hidden (header);
 	
-	if (!strcmp (header->name, "=")) {	/* Read from pipe */
+	if (!strcmp (HH->name, "=")) {	/* Read from pipe */
 #ifdef SET_IO_MODE
 		gmt_setmode (GMT, GMT_IN);
 #endif
 		fp = GMT->session.std[GMT_IN];
 	}
-	else if ((fp = gmt_fopen (GMT, header->name, "rb")) == NULL)
+	else if ((fp = gmt_fopen (GMT, HH->name, "rb")) == NULL)
 		return (GMT_GRDIO_OPEN_FAILED);
 
-	gmt_M_err_pass (GMT, gmt_grd_prep_io (GMT, header, wesn, &width_in, &height_in, &first_col, &last_col, &first_row, &last_row, &k), header->name);
+	gmt_M_err_pass (GMT, gmt_grd_prep_io (GMT, header, wesn, &width_in, &height_in, &first_col, &last_col, &first_row, &last_row, &k), HH->name);
 	(void)gmtlib_init_complex (header, complex_mode, &imag_offset);	/* Set offset for imaginary complex component */
 
 	width_out = width_in;		/* Width of output array */
@@ -283,10 +287,10 @@ int gmt_agc_read_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, floa
 
 	/* Rows are read south to north */
 	
-	gmt_M_memset (z, ZBLOCKWIDTH * ZBLOCKHEIGHT, float); /* Initialize buffer to zero */
+	gmt_M_memset (z, ZBLOCKWIDTH * ZBLOCKHEIGHT, gmt_grdfloat); /* Initialize buffer to zero */
 
 	header->z_min = +DBL_MAX;	header->z_max = -DBL_MAX;
-	header->has_NaNs = GMT_GRID_NO_NANS;	/* We are about to check for NaNs and if none are found we retain 1, else 2 */
+	HH->has_NaNs = GMT_GRID_NO_NANS;	/* We are about to check for NaNs and if none are found we retain 1, else 2 */
 	
 	n_blocks_y = urint (ceil ((double)header->n_rows / (double)ZBLOCKHEIGHT));
 	n_blocks_x = urint (ceil ((double)header->n_columns / (double)ZBLOCKWIDTH));
@@ -310,7 +314,7 @@ int gmt_agc_read_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, floa
 				ij = imag_offset + ((uint64_t)(((j_gmt - first_row) + pad[YHI])) * width_out + col - first_col) + pad[XLO];
 				grid[ij] = (z[j][i] == 0.0) ? GMT->session.f_NaN : z[j][i];	/* AGC uses exact zero as NaN flag */
 				if (gmt_M_is_fnan (grid[ij])) {
-					header->has_NaNs = GMT_GRID_HAS_NANS;
+					HH->has_NaNs = GMT_GRID_HAS_NANS;
 					continue;
 				}
 				header->z_min = MIN (header->z_min, (double)grid[ij]);
@@ -324,6 +328,8 @@ int gmt_agc_read_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, floa
 		}
 	}
 	gmt_M_free (GMT, k);
+	if (header->z_min == DBL_MAX && header->z_max == -DBL_MAX) /* No valid data values in the grid */
+		header->z_min = header->z_max = NAN;
 
 	header->n_columns = width_in;	header->n_rows = height_in;
 	gmt_M_memcpy (header->wesn, wesn, 4, double);
@@ -333,7 +339,7 @@ int gmt_agc_read_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, floa
 	return (GMT_NOERROR);
 }
 
-int gmt_agc_write_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, float *grid, double wesn[], unsigned int *pad, unsigned int complex_mode) {
+int gmt_agc_write_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, gmt_grdfloat *grid, double wesn[], unsigned int *pad, unsigned int complex_mode) {
 	/* header:	grid structure header
 	 * grid:	array with final grid
 	 * wesn:	Sub-region to write out  [Use entire file if 0,0,0,0]
@@ -353,20 +359,21 @@ int gmt_agc_write_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, flo
 	unsigned int block, n_blocks, n_blocks_x, n_blocks_y;	/* Misc. counters */
 	unsigned int row, rowstart, rowend, colstart, datablockcol, datablockrow;
 	uint64_t kk, ij, imag_offset, width_in;
-	float prez[PREHEADSIZE], postz[POSTHEADSIZE];
-	float outz[ZBLOCKWIDTH][ZBLOCKHEIGHT];
+	gmt_grdfloat prez[PREHEADSIZE], postz[POSTHEADSIZE];
+	gmt_grdfloat outz[ZBLOCKWIDTH][ZBLOCKHEIGHT];
 	FILE *fp = NULL;			/* File pointer to data or pipe */
+	struct GMT_GRID_HEADER_HIDDEN *HH = gmt_get_H_hidden (header);
 
-	if (!strcmp (header->name, "=")) {	/* Write to pipe */
+	if (!strcmp (HH->name, "=")) {	/* Write to pipe */
 #ifdef SET_IO_MODE
 		gmt_setmode (GMT, GMT_OUT);
 #endif
 		fp = GMT->session.std[GMT_OUT];
 	}
-	else if ((fp = gmt_fopen (GMT, header->name, "wb")) == NULL)
+	else if ((fp = gmt_fopen (GMT, HH->name, "wb")) == NULL)
 		return (GMT_GRDIO_CREATE_FAILED);
 	
-	gmt_M_err_pass (GMT, gmt_grd_prep_io (GMT, header, wesn, &width_out, &height_out, &first_col, &last_col, &first_row, &last_row, &k), header->name);
+	gmt_M_err_pass (GMT, gmt_grd_prep_io (GMT, header, wesn, &width_out, &height_out, &first_col, &last_col, &first_row, &last_row, &k), HH->name);
 	(void)gmtlib_init_complex (header, complex_mode, &imag_offset);	/* Set offset for imaginary complex component */
 
 	width_in = width_out;		/* Physical width of input array */
@@ -390,17 +397,19 @@ int gmt_agc_write_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, flo
 			}
 		}
 	}
+	if (header->z_min == DBL_MAX && header->z_max == -DBL_MAX) /* No valid data values in the grid */
+		header->z_min = header->z_max = NAN;
 
 	/* Since AGC files are always gridline-registered we must change -R when a pixel grid is to be written */
 	if (header->registration == GMT_GRID_PIXEL_REG) {
 		gmt_change_grdreg (GMT, header, GMT_GRID_NODE_REG);
-		GMT_Report (GMT->parent, GMT_MSG_VERBOSE, "Warning: AGC grids are always gridline-registered.  Your pixel-registered grid will be converted.\n");
-		GMT_Report (GMT->parent, GMT_MSG_VERBOSE, "Warning: AGC grid region in file %s reset to %g/%g/%g/%g\n", header->name, header->wesn[XLO], header->wesn[XHI], header->wesn[YLO], header->wesn[YHI]);
+		GMT_Report (GMT->parent, GMT_MSG_VERBOSE, "AGC grids are always gridline-registered.  Your pixel-registered grid will be converted.\n");
+		GMT_Report (GMT->parent, GMT_MSG_VERBOSE, "AGC grid region in file %s reset to %g/%g/%g/%g\n", HH->name, header->wesn[XLO], header->wesn[XHI], header->wesn[YLO], header->wesn[YHI]);
 	}
 
 	agc_pack_header (prez, postz, header);	/* Stuff header info into the AGC arrays */
 
-	gmt_M_memset (outz, ZBLOCKWIDTH * ZBLOCKHEIGHT, float); /* or agc_write_record (fp, outz, prez, postz); points to uninitialised buffer */
+	gmt_M_memset (outz, ZBLOCKWIDTH * ZBLOCKHEIGHT, gmt_grdfloat); /* or agc_write_record (fp, outz, prez, postz); points to uninitialised buffer */
 
 	n_blocks_y = urint (ceil ((double)header->n_rows / (double)ZBLOCKHEIGHT));
 	n_blocks_x = urint (ceil ((double)header->n_columns / (double)ZBLOCKWIDTH));
