@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt_calclock.c 18312 2017-06-02 17:21:39Z pwessel $
  *
- *	Copyright (c) 1991-2017 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /* Routines for Generic Mapping Tools conversions
  *       between [calendar, clock] and time.
@@ -35,7 +34,7 @@
  * gmt_dt2rdc           : 
  * gmt_format_calendar  : 
  * gmtlib_g_ymd_is_bad     : 
- * gmtlib_gcal_from_dt     : 
+ * gmt_gcal_from_dt     : 
  * gmt_gcal_from_rd     : 
  * gmtlib_get_time_label   : 
  * gmtlib_gmonth_length    : 
@@ -418,7 +417,7 @@ bool gmtlib_iso_ywd_is_bad (int y, int w, int d) {
 	return (false);
 }
 
-void gmtlib_gcal_from_dt (struct GMT_CTRL *GMT, double t, struct GMT_GCAL *cal) {
+void gmt_gcal_from_dt (struct GMT_CTRL *GMT, double t, struct GMT_GCAL *cal) {
 
 	/* Given time in internal units, load cal and clock info in cal.
 	   Note: uses 0 through 23 for hours (no am/pm inside here).
@@ -451,7 +450,7 @@ int gmtlib_verify_time_step (struct GMT_CTRL *GMT, int step, char unit) {
 		case 'c':
 		case 'C':
 			if (gmt_M_compat_check (GMT, 4)) {
-				GMT_Report (GMT->parent, GMT_MSG_COMPAT, "Warning: Unit c for seconds is deprecated; use s.\n");
+				GMT_Report (GMT->parent, GMT_MSG_COMPAT, "Unit c for seconds is deprecated; use s.\n");
 				if (step > 60) {
 					GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Syntax error: time steps in seconds must be <= 60\n");
 					retval = -1;
@@ -602,7 +601,7 @@ void gmtlib_moment_interval (struct GMT_CTRL *GMT, struct GMT_MOMENT_INTERVAL *p
 		case 'c':
 		case 'C':
 			if (gmt_M_compat_check (GMT, 4)) {
-				GMT_Report (GMT->parent, GMT_MSG_COMPAT, "Warning: Unit c for seconds is deprecated; use s.\n");
+				GMT_Report (GMT->parent, GMT_MSG_COMPAT, "Unit c for seconds is deprecated; use s.\n");
 				k = p->step;
 				calclock_small_moment_interval (GMT, p, k, init);
 			}
@@ -870,7 +869,7 @@ void gmt_format_calendar (struct GMT_CTRL *GMT, char *date, char *clock, struct
 
 	step = 0.5 / W->f_sec_to_int / GMT->current.setting.time_system.scale;	/* Precision desired in time units */
 
-	gmtlib_gcal_from_dt (GMT, dt + step, &calendar);			/* Convert dt to a complete calendar structure */
+	gmt_gcal_from_dt (GMT, dt + step, &calendar);			/* Convert dt to a complete calendar structure */
 
 	if (date) date[0] = 0;
 	if (date && !D->skip) {	/* Not NULL, want to format this string */
@@ -944,7 +943,7 @@ void gmtlib_get_time_label (struct GMT_CTRL *GMT, char *string, struct GMT_PLOT_
 	/* Assemble the annotation label given the formatting options presented */
 	struct GMT_GCAL calendar;
 
-	gmtlib_gcal_from_dt (GMT, t, &calendar);			/* Convert t to a complete calendar structure */
+	gmt_gcal_from_dt (GMT, t, &calendar);			/* Convert t to a complete calendar structure */
 
 	switch (T->unit) {
 		case 'Y':	/* 4-digit integer year */
@@ -997,11 +996,11 @@ void gmtlib_get_time_label (struct GMT_CTRL *GMT, char *string, struct GMT_PLOT_
 			break;
 		case 'C':
 			if (gmt_M_compat_check (GMT, 4)) {
-				GMT_Report (GMT->parent, GMT_MSG_COMPAT, "Warning: Unit C for seconds is deprecated; use S.\n");
+				GMT_Report (GMT->parent, GMT_MSG_COMPAT, "Unit C for seconds is deprecated; use S.\n");
 				gmt_format_calendar (GMT, NULL, string, &P->date, &P->clock, T->upper_case, T->flavor, t);
 			}
 			else {
-				GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Error: wrong unit passed to gmtlib_get_time_label\n");
+				GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Wrong unit passed to gmtlib_get_time_label\n");
 				sprintf (string, "NaN");
 			}
 			break;
@@ -1010,11 +1009,11 @@ void gmtlib_get_time_label (struct GMT_CTRL *GMT, char *string, struct GMT_PLOT_
 			break;
 		case 'c':
 			if (gmt_M_compat_check (GMT, 4)) {
-				GMT_Report (GMT->parent, GMT_MSG_COMPAT, "Warning: Unit c for seconds is deprecated; use s.\n");
+				GMT_Report (GMT->parent, GMT_MSG_COMPAT, "Unit c for seconds is deprecated; use s.\n");
 				(P->date.compact) ? sprintf (string, "%d", irint(calendar.sec)) : sprintf (string, "%02d", irint(calendar.sec));
 			}
 			else {
-				GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Error: wrong unit passed to gmtlib_get_time_label\n");
+				GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Wrong unit passed to gmtlib_get_time_label\n");
 				sprintf (string, "NaN");
 			}
 			break;
@@ -1022,7 +1021,7 @@ void gmtlib_get_time_label (struct GMT_CTRL *GMT, char *string, struct GMT_PLOT_
 			(P->date.compact) ? sprintf (string, "%d", irint(calendar.sec)) : sprintf (string, "%02d", irint(calendar.sec));
 			break;
 		default:
-			GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Error: wrong unit passed to gmtlib_get_time_label\n");
+			GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Wrong unit passed to gmtlib_get_time_label\n");
 			sprintf (string, "NaN");
 			break;
 	}
