@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *	Copyright (c) 1991-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  *
@@ -48,7 +47,7 @@ int gmt_cdf_grd_info (struct GMT_CTRL *GMT, int ncid, struct GMT_GRID_HEADER *he
 	char text[GMT_GRID_COMMAND_LEN320+GMT_GRID_REMARK_LEN160];
 	size_t limit = 2147483647U;	/* 2^31 - 1 is the max length of a 1-D array in netCDF */
 	nc_type z_type;
-
+	struct GMT_GRID_HEADER_HIDDEN *HH = gmt_get_H_hidden (header);
 	/* Dimension ids, variable ids, etc. */
 	int side_dim, xysize_dim, x_range_id, y_range_id, z_range_id, inc_id, nm_id, z_id, dims[1];
 
@@ -58,8 +57,8 @@ int gmt_cdf_grd_info (struct GMT_CTRL *GMT, int ncid, struct GMT_GRID_HEADER *he
 
 	if (job == 'w') {
 		if (header->nm > limit) {	/* Print error message and let things crash */
-			GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Error: Your grid contains more than 2^31 - 1 nodes (%" PRIu64 ") and cannot be stored with the deprecated GMT netCDF format.\n", header->nm);
-			GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Error: Please choose another grid format such as the default netCDF 4 COARDS-compliant grid format.\n");
+			GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Your grid contains more than 2^31 - 1 nodes (%" PRIu64 ") and cannot be stored with the deprecated GMT netCDF format.\n", header->nm);
+			GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Please choose another grid format such as the default netCDF 4 COARDS-compliant grid format.\n");
 			return (GMT_DIM_TOO_LARGE);
 		}
 		gmt_M_err_trap (nc_def_dim (ncid, "side", 2U, &side_dim));
@@ -73,10 +72,10 @@ int gmt_cdf_grd_info (struct GMT_CTRL *GMT, int ncid, struct GMT_GRID_HEADER *he
 		gmt_M_err_trap (nc_def_var (ncid, "dimension", NC_LONG, 1, dims, &nm_id));
 
 		switch (header->type) {
-			case GMT_GRID_IS_CB: z_type = NC_BYTE; break;
-			case GMT_GRID_IS_CS: z_type = NC_SHORT; break;
-			case GMT_GRID_IS_CI: z_type = NC_INT; break;
-			case GMT_GRID_IS_CF: z_type = NC_FLOAT; break;
+			case GMT_GRID_IS_CB: z_type = NC_BYTE;   break;
+			case GMT_GRID_IS_CS: z_type = NC_SHORT;  break;
+			case GMT_GRID_IS_CI: z_type = NC_INT;    break;
+			case GMT_GRID_IS_CF: z_type = NC_FLOAT;  break;
 			case GMT_GRID_IS_CD: z_type = NC_DOUBLE; break;
 			default:			z_type = NC_NAT;
 		}
@@ -93,15 +92,15 @@ int gmt_cdf_grd_info (struct GMT_CTRL *GMT, int ncid, struct GMT_GRID_HEADER *he
 		gmt_M_err_trap (nc_inq_varid (ncid, "z", &z_id));
 		gmt_M_err_trap (nc_inq_vartype (ncid, z_id, &z_type));
 		switch (z_type) {
-			case NC_BYTE:   header->type = GMT_GRID_IS_CB; break;
-			case NC_SHORT:  header->type = GMT_GRID_IS_CS; break;
-			case NC_INT:    header->type = GMT_GRID_IS_CI; break;
-			case NC_FLOAT:  header->type = GMT_GRID_IS_CF; break;
-			case NC_DOUBLE: header->type = GMT_GRID_IS_CD; break;
+			case NC_BYTE:   header->type = GMT_GRID_IS_CB; HH->orig_datatype = GMT_CHAR;   break;
+			case NC_SHORT:  header->type = GMT_GRID_IS_CS; HH->orig_datatype = GMT_SHORT;  break;
+			case NC_INT:    header->type = GMT_GRID_IS_CI; HH->orig_datatype = GMT_INT;    break;
+			case NC_FLOAT:  header->type = GMT_GRID_IS_CF; HH->orig_datatype = GMT_FLOAT;  break;
+			case NC_DOUBLE: header->type = GMT_GRID_IS_CD; HH->orig_datatype = GMT_DOUBLE; break;
 			default:        header->type = k_grd_unknown_fmt; break;
 		}
 	}
-	header->z_id = z_id;
+	HH->z_id = z_id;
 
 	/* Get or assign attributes */
 
@@ -118,7 +117,11 @@ int gmt_cdf_grd_info (struct GMT_CTRL *GMT, int ncid, struct GMT_GRID_HEADER *he
 		gmt_M_err_trap (nc_get_att_double (ncid, z_id, "add_offset", &header->z_add_offset));
 		gmt_M_err_trap (nc_get_att_int (ncid, z_id, "node_offset", &reg));
 		header->registration = reg;
+#ifdef DOUBLE_PRECISION_GRID
+		nc_get_att_double (ncid, z_id, "_FillValue", &header->nan_value);
+#else
 		nc_get_att_float (ncid, z_id, "_FillValue", &header->nan_value);
+#endif
 		gmt_M_err_trap (nc_get_att_text (ncid, NC_GLOBAL, "title", header->title));
 		gmt_M_err_trap (nc_get_att_text (ncid, NC_GLOBAL, "source", text));
 		strncpy (header->command, text, GMT_GRID_COMMAND_LEN320-1);
@@ -139,7 +142,7 @@ int gmt_cdf_grd_info (struct GMT_CTRL *GMT, int ncid, struct GMT_GRID_HEADER *he
 		gmt_M_err_trap (nc_get_var_double (ncid, z_range_id, dummy));
 		header->z_min = dummy[0];
 		header->z_max = dummy[1];
-		header->row_order = k_nc_start_north; /* N->S */
+		//HH->row_order = k_nc_start_north; /* N->S but that is just when read.  We don't want to set this for writing later */
 	}
 	else {
 		int reg;
@@ -152,7 +155,11 @@ int gmt_cdf_grd_info (struct GMT_CTRL *GMT, int ncid, struct GMT_GRID_HEADER *he
 		gmt_M_err_trap (nc_put_att_double (ncid, z_id, "add_offset", NC_DOUBLE, 1U, &header->z_add_offset));
 		if (z_type != NC_FLOAT && z_type != NC_DOUBLE)
 			header->nan_value = rintf (header->nan_value); /* round to integer */
+#ifdef DOUBLE_PRECISION_GRID
+		gmt_M_err_trap (nc_put_att_double (ncid, z_id, "_FillValue", z_type, 1U, &header->nan_value));
+#else
 		gmt_M_err_trap (nc_put_att_float (ncid, z_id, "_FillValue", z_type, 1U, &header->nan_value));
+#endif
 		reg = header->registration;
 		gmt_M_err_trap (nc_put_att_int (ncid, z_id, "node_offset", NC_LONG, 1U, &reg));
 		gmt_M_err_trap (nc_put_att_text (ncid, NC_GLOBAL, "title", GMT_GRID_TITLE_LEN80, header->title));
@@ -180,20 +187,20 @@ int gmt_cdf_grd_info (struct GMT_CTRL *GMT, int ncid, struct GMT_GRID_HEADER *he
 }
 
 int gmt_cdf_read_grd_info (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header) {
-	int ncid;
-	int err;
-	if (!strcmp (header->name,"=")) return (GMT_GRDIO_NC_NO_PIPE);
-	gmt_M_err_trap (nc_open (header->name, NC_NOWRITE, &ncid));
+	int ncid, err;
+	struct GMT_GRID_HEADER_HIDDEN *HH = gmt_get_H_hidden (header);
+	if (!strcmp (HH->name,"=")) return (GMT_GRDIO_NC_NO_PIPE);
+	gmt_M_err_trap (nc_open (HH->name, NC_NOWRITE, &ncid));
 	gmt_M_err_trap (gmt_cdf_grd_info (GMT, ncid, header, 'r'));
 	gmt_M_err_trap (nc_close (ncid));
 	return (GMT_NOERROR);
 }
 
 int gmt_cdf_update_grd_info (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header) {
-	int ncid, old_fill_mode;
-	int err;
-	if (!strcmp (header->name,"=")) return (GMT_GRDIO_NC_NO_PIPE);
-	gmt_M_err_trap (nc_open (header->name, NC_WRITE, &ncid));
+	int ncid, old_fill_mode, err;
+	struct GMT_GRID_HEADER_HIDDEN *HH = gmt_get_H_hidden (header);
+	if (!strcmp (HH->name,"=")) return (GMT_GRDIO_NC_NO_PIPE);
+	gmt_M_err_trap (nc_open (HH->name, NC_WRITE, &ncid));
 	gmt_M_err_trap (nc_set_fill (ncid, NC_NOFILL, &old_fill_mode)); 
 	gmt_M_err_trap (gmt_cdf_grd_info (GMT, ncid, header, 'u'));
 	gmt_M_err_trap (nc_close (ncid));
@@ -201,17 +208,17 @@ int gmt_cdf_update_grd_info (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *heade
 }
 
 int gmt_cdf_write_grd_info (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header) {
-	int ncid, old_fill_mode;
-	int err;
-	if (!strcmp (header->name,"=")) return (GMT_GRDIO_NC_NO_PIPE);
-	gmt_M_err_trap (nc_create (header->name, NC_CLOBBER, &ncid));
+	int ncid, old_fill_mode, err;
+	struct GMT_GRID_HEADER_HIDDEN *HH = gmt_get_H_hidden (header);
+	if (!strcmp (HH->name,"=")) return (GMT_GRDIO_NC_NO_PIPE);
+	gmt_M_err_trap (nc_create (HH->name, NC_CLOBBER, &ncid));
 	gmt_M_err_trap (nc_set_fill (ncid, NC_NOFILL, &old_fill_mode)); 
 	gmt_M_err_trap (gmt_cdf_grd_info (GMT, ncid, header, 'w'));
 	gmt_M_err_trap (nc_close (ncid));
 	return (GMT_NOERROR);
 }
 
-int gmt_cdf_read_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, float *grid, double wesn[], unsigned int *pad, unsigned int complex_mode) {
+int gmt_cdf_read_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, gmt_grdfloat *grid, double wesn[], unsigned int *pad, unsigned int complex_mode) {
 	/* header:	grid structure header
 	 * grid:	array with final grid
 	 * wesn:	Sub-region to extract  [Use entire file if 0,0,0,0]
@@ -225,17 +232,16 @@ int gmt_cdf_read_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, floa
 	 * not the physical size (i.e., the padding is not counted in n_columns and n_rows)
 	 */
 
-	int  ncid;
-	size_t start[1], edge[1];
 	bool check;
-	int j, err;
-	int first_col, last_col, first_row, last_row;
+	int  ncid, j, err, first_col, last_col, first_row, last_row;
 	unsigned int i, width_in, height_in;
 	unsigned int width_out, *actual_col = NULL;
 	uint64_t ij, kk, imag_offset;
-	float *tmp = NULL;
+	size_t start[1], edge[1];
+	gmt_grdfloat *tmp = NULL;
+	struct GMT_GRID_HEADER_HIDDEN *HH = gmt_get_H_hidden (header);
 
-	gmt_M_err_pass (GMT, gmt_grd_prep_io (GMT, header, wesn, &width_in, &height_in, &first_col, &last_col, &first_row, &last_row, &actual_col), header->name);
+	gmt_M_err_pass (GMT, gmt_grd_prep_io (GMT, header, wesn, &width_in, &height_in, &first_col, &last_col, &first_row, &last_row, &actual_col), HH->name);
 	(void)gmtlib_init_complex (header, complex_mode, &imag_offset);	/* Set offset for imaginary complex component */
 
 	width_out = width_in;		/* Width of output array */
@@ -244,11 +250,11 @@ int gmt_cdf_read_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, floa
 
 	/* Open the NetCDF file */
 
-	if (!strcmp (header->name,"=")) {
+	if (!strcmp (HH->name,"=")) {
 		gmt_M_free (GMT, actual_col);
 		return (GMT_GRDIO_NC_NO_PIPE);
 	}
-	if ((err = nc_open (header->name, NC_NOWRITE, &ncid)) != 0) {
+	if ((err = nc_open (HH->name, NC_NOWRITE, &ncid)) != 0) {
 		gmt_M_free (GMT, actual_col);
 		return err;
 	}
@@ -257,17 +263,17 @@ int gmt_cdf_read_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, floa
 	/* Load data row by row. The data in the file is stored in the same
 	 * "upside down" fashion as within GMT. The first row is the top row */
 
-	tmp = gmt_M_memory (GMT, NULL, header->n_columns, float);
+	tmp = gmt_M_memory (GMT, NULL, header->n_columns, gmt_grdfloat);
 
 	edge[0] = header->n_columns;
 	ij = imag_offset + pad[YHI] * width_out + pad[XLO];
 	header->z_min =  DBL_MAX;
 	header->z_max = -DBL_MAX;
-	header->has_NaNs = GMT_GRID_NO_NANS;	/* We are about to check for NaNs and if none are found we retain 1, else 2 */
+	HH->has_NaNs = GMT_GRID_NO_NANS;	/* We are about to check for NaNs and if none are found we retain 1, else 2 */
 
 	for (j = first_row; j <= last_row; j++, ij += width_out) {
 		start[0] = j * header->n_columns;
-		if ((err = nc_get_vara_float (ncid, header->z_id, start, edge, tmp))) {	/* Get one row */
+		if ((err = gmt_nc_get_vara_grdfloat (ncid, HH->z_id, start, edge, tmp))) {	/* Get one row */
 			gmt_M_free (GMT, actual_col);
 			gmt_M_free (GMT, tmp);
 			nc_close (ncid);
@@ -279,7 +285,7 @@ int gmt_cdf_read_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, floa
 			if (check && grid[kk] == header->nan_value)
 				grid[kk] = GMT->session.f_NaN;
 			if (gmt_M_is_fnan (grid[kk])) {
-				header->has_NaNs = GMT_GRID_HAS_NANS;
+				HH->has_NaNs = GMT_GRID_HAS_NANS;
 				continue;
 			}
 			header->z_min = MIN (header->z_min, (double)grid[kk]);
@@ -298,7 +304,7 @@ int gmt_cdf_read_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, floa
 	return (GMT_NOERROR);
 }
 
-int gmt_cdf_write_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, float *grid, double wesn[], unsigned int *pad, unsigned int complex_mode) {
+int gmt_cdf_write_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, gmt_grdfloat *grid, double wesn[], unsigned int *pad, unsigned int complex_mode) {
 	/* header:	grid structure header
 	 * grid:	array with final grid
 	 * wesn:	Sub-region to write out  [Use entire file if 0,0,0,0]
@@ -309,19 +315,18 @@ int gmt_cdf_write_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, flo
 	 */
 
 	size_t start[1], edge[1];
-	int ncid, old_fill_mode;
+	int ncid, old_fill_mode, err, first_col, last_col, first_row, last_row;
 	long *tmp_i = NULL;
-	int err;
 	unsigned int i, *actual_col = NULL;
 	unsigned int j, width_out, height_out, width_in;
-	int first_col, last_col, first_row, last_row;
 	uint64_t ij, nr_oor = 0, imag_offset;
-	float *tmp_f = NULL;
+	gmt_grdfloat *tmp_f = NULL;
 	double limit[2] = {-FLT_MAX, FLT_MAX};
-	float value;
+	gmt_grdfloat value;
 	nc_type z_type;
+	struct GMT_GRID_HEADER_HIDDEN *HH = gmt_get_H_hidden (header);
 
-	if (!strcmp (header->name,"=")) return (GMT_GRDIO_NC_NO_PIPE);	/* Cannot do piping on netCDF files */
+	if (!strcmp (HH->name,"=")) return (GMT_GRDIO_NC_NO_PIPE);	/* Cannot do piping on netCDF files */
 
 	/* Determine the value to be assigned to missing data, if not already done so */
 
@@ -346,7 +351,7 @@ int gmt_cdf_write_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, flo
 			z_type = NC_NAT;
 	}
 
-	gmt_M_err_pass (GMT, gmt_grd_prep_io (GMT, header, wesn, &width_out, &height_out, &first_col, &last_col, &first_row, &last_row, &actual_col), header->name);
+	gmt_M_err_pass (GMT, gmt_grd_prep_io (GMT, header, wesn, &width_out, &height_out, &first_col, &last_col, &first_row, &last_row, &actual_col), HH->name);
 	(void)gmtlib_init_complex (header, complex_mode, &imag_offset);	/* Set offset for imaginary complex component */
 
 	width_in = width_out;		/* Physical width of input array */
@@ -359,7 +364,7 @@ int gmt_cdf_write_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, flo
 
 	/* Write grid header */
 
-	if ((err = nc_create (header->name, NC_CLOBBER, &ncid))) {
+	if ((err = nc_create (HH->name, NC_CLOBBER, &ncid))) {
 		gmt_M_free (GMT, actual_col);
 		return (err);
 	}
@@ -382,14 +387,14 @@ int gmt_cdf_write_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, flo
 	/* Store z-variable */
 
 	if (z_type == NC_FLOAT || z_type == NC_DOUBLE) {
-		tmp_f = gmt_M_memory (GMT, NULL, width_in, float);
+		tmp_f = gmt_M_memory (GMT, NULL, width_in, gmt_grdfloat);
 		for (j = 0; j < height_out; j++, ij += width_in) {
 			start[0] = j * width_out;
 			for (i = 0; i < width_out; i++) {
 				value = grid[ij+actual_col[i]+imag_offset];
 				if (!isfinite (value)) {
 					if (isinf(value))
-						nr_oor++; /* out of float range */
+						nr_oor++; /* out of gmt_grdfloat range */
 					tmp_f[i] = header->nan_value;
 				}
 				else {
@@ -398,7 +403,7 @@ int gmt_cdf_write_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, flo
 					header->z_max = MAX (header->z_max, (double)tmp_f[i]);
 				}
 			}
-			if ((err = nc_put_vara_float (ncid, header->z_id, start, edge, tmp_f))) {
+			if ((err = gmt_nc_put_vara_grdfloat (ncid, HH->z_id, start, edge, tmp_f))) {
 				gmt_M_free (GMT, actual_col);
 				gmt_M_free (GMT, tmp_f);
 				return (err);
@@ -414,7 +419,7 @@ int gmt_cdf_write_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, flo
 				value = grid[ij+actual_col[i]+imag_offset];
 				if (!isfinite (value)) {
 					if (isinf(value))
-						nr_oor++; /* out of float range */
+						nr_oor++; /* out of gmt_grdfloat range */
 					tmp_i[i] = lrintf (header->nan_value);
 				}
 				else if (value <= limit[0] || value >= limit[1]) {
@@ -427,7 +432,7 @@ int gmt_cdf_write_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, flo
 					header->z_max = MAX (header->z_max, (double)tmp_i[i]);
 				}
 			}
-			if ((err = nc_put_vara_long (ncid, header->z_id, start, edge, tmp_i))) {
+			if ((err = nc_put_vara_long (ncid, HH->z_id, start, edge, tmp_i))) {
 				gmt_M_free (GMT, actual_col);
 				gmt_M_free (GMT, tmp_i);
 				return (err);
@@ -436,7 +441,7 @@ int gmt_cdf_write_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, flo
 		gmt_M_free (GMT, tmp_i);
 	}
 
-	if (nr_oor > 0) GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Warning: %" PRIu64 " out-of-range grid values converted to _FillValue [%s]\n", nr_oor, header->name);
+	if (nr_oor > 0) GMT_Report (GMT->parent, GMT_MSG_LONG_VERBOSE, "%" PRIu64 " out-of-range grid values converted to _FillValue [%s]\n", nr_oor, HH->name);
 
 	gmt_M_free (GMT, actual_col);
 
@@ -444,10 +449,10 @@ int gmt_cdf_write_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, flo
 		limit[0] = header->z_min; limit[1] = header->z_max;
 	}
 	else {
-		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Warning: No valid values in grid [%s]\n", header->name);
+		GMT_Report (GMT->parent, GMT_MSG_LONG_VERBOSE, "No valid values in grid [%s]\n", HH->name);
 		limit[0] = 0.0; limit[1] = 0.0;
 	}
-	gmt_M_err_trap (nc_put_var_double (ncid, header->z_id - 3, limit));
+	gmt_M_err_trap (nc_put_var_double (ncid, HH->z_id - 3, limit));
 
 	/* Close grid */
 
