@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *	Copyright (c) 1991-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 
 /*
@@ -21,7 +20,7 @@
  *
  * Author: 	Paul Wessel
  * Date:	01-JAN-2011
- * Version:	5 API
+ * Version:	6 API
  */
 
 /*!
@@ -29,8 +28,8 @@
  * \brief Holds current selections for the family of common GMT options
  */
 
-#ifndef _GMT_COMMON_H
-#define _GMT_COMMON_H
+#ifndef GMT_COMMON_H
+#define GMT_COMMON_H
 
 /*! Constants related to detecting data gaps which should be treated as segment boundaries */
 enum GMT_enum_gaps {GMT_NEGGAP_IN_COL = 0,	/* Check if previous minus current column value exceeds <gap> */
@@ -48,13 +47,29 @@ enum GMT_enum_gaps {GMT_NEGGAP_IN_COL = 0,	/* Check if previous minus current co
 #define MAX_ASPATIAL 64		/* No more than 64 aspatial options in -a */
 
 #define GMT_SHORTHAND_OPTIONS	"BJRXYcp"	/* All of the shorthand options */
-#define GMT_CRITICAL_OPT_ORDER	"V-JfrRb"	/* If given options among these must be parsed first and in this order */
+#define GMT_CRITICAL_OPT_ORDER	"V-JfrRbi"	/* If given options among these must be parsed first and in this order */
 
 #define RSET	0	/* Index into R.active[] for -R */
 #define ISET	1	/* Index into R.active[] for -I (or similar option) */
 #define GSET	2	/* Index into R.active[] for -r */
 #define FSET	3	/* Index into R.active[] for "got -R -I -r from a grid file" */
 
+struct GMT_LEGEND_ITEM {	/* Information about one item in a legend */
+	char label[GMT_LEN128];		/* The symbol label */
+	char header[GMT_LEN128];	/* Header for the whole legend H */
+	char subheader[GMT_LEN128];	/* Subheader, i.e., line label L*/
+	char font[GMT_LEN32];		/* Fontsize to use for current H or L */
+	char gap[GMT_LEN32];		/* Move this much down before placing symbol entry */
+	char pen[2][GMT_LEN32];		/* Pens to use with +d and +v */
+	int draw;			/* 0 no draw, 1 draw horizontal +d, 2 draw vertical +v */
+	int just;			/* Legend placement [TR] */
+	char code;			/* Label justification code (L|C|R) [L] */
+	double size;			/* Fixed symbol size when otherwise cannot set it */
+	double scale;			/* Scale all given sizes, including +s<length> of a line */
+	double width;			/* Override auto-width with a fixed legend width */
+	unsigned int ncols;		/* How many columns to use for symbols */
+};
+
 /*! Structure with all information given via the common GMT command-line options -R -J .. */
 struct GMT_COMMON {
 	struct synopsis {	/* \0 (zero) or ^ */
@@ -69,9 +84,11 @@ struct GMT_COMMON {
 	struct J {	/* -J<params> */
 		bool active, zactive;
 		unsigned int id;
+		char string[GMT_LEN128];
+		char zstring[GMT_LEN128];	/* For -Jz|Z */
+		char proj4string[GMT_LEN256];
+		char WKTstring[GMT_LEN1024];
 		double par[6];
-		char string[GMT_LEN256];
-		char zstring[GMT_LEN256];	/* For -Jz|Z */
 	} J;
 	struct K {	/* -K */
 		bool active;
@@ -86,6 +103,8 @@ struct GMT_COMMON {
 		bool active[4];	/* RSET = 0: -R, ISET = 1: inc, GSET = 2: -r, FSET = 3: read grid */
 		bool oblique;	/* true when -R...r was given (oblique map, probably), else false (map borders are meridians/parallels) */
 		uint32_t registration;	/* Registration mode of a grid given via -r or -Rgrid */
+		int row_order;	/* Order of rows in NetCDF output: 0 (not set) or k_nc_start_north or k_nc_start_south */
+		unsigned int mode;	/* For modern mode only: 0 = get exact region from data, 1 = rounded region from data */
 		double wesn[6];		/* Boundaries of west, east, south, north, low-z and hi-z */
 		double wesn_orig[4];	/* Original Boundaries of west, east, south, north (oblique projection may reset wesn above) */
 		double inc[2];	/* For grid increments set via -Idx/dy or implicitly via -Ggrid */
@@ -103,22 +122,22 @@ struct GMT_COMMON {
 	struct X {	/* -X */
 		bool active;
 		double off;
-		char mode;	/* r, a, or c */
+		char mode;	/* a, c, f, or r */
 	} X;
 	struct Y {	/* -Y */
 		bool active;
 		double off;
-		char mode;	/* r, a, or c */
+		char mode;	/* a, c, f, or r */
 	} Y;
 	struct a {	/* -a<col>=<name>[:<type>][,col>=<name>[:<type>], etc][+g<geometry>] */
 		bool active;
-		unsigned int geometry;
+		enum GMT_enum_ogr geometry;
 		unsigned int n_aspatial;
 		bool clip;		/* true if we wish to clip lines/polygons at Dateline [false] */
 		bool output;		/* true when we wish to build OGR output */
 		int col[MAX_ASPATIAL];	/* Col id, include negative items such as GMT_IS_T (-5) */
 		int ogr[MAX_ASPATIAL];	/* Column order, or -1 if not set */
-		unsigned int type[MAX_ASPATIAL];
+		enum GMT_enum_type type[MAX_ASPATIAL];
 		char *name[MAX_ASPATIAL];
 		char string[GMT_LEN256];
 	} a;
@@ -134,10 +153,6 @@ struct GMT_COMMON {
 		char varnames[GMT_BUFSIZ];	/* List of variable names to be input/output in netCDF mode [GMT4 COMPATIBILITY ONLY] */
 		char string[GMT_LEN256];
 	} b;
-	struct c {	/* -c */
-		bool active;
-		unsigned int copies;
-	} c;
 	struct d {	/* -d[i][o]<nan_proxy> */
 		bool active[2];
 		bool is_zero[2];
@@ -175,27 +190,36 @@ struct GMT_COMMON {
 		char *multi_segment;    /* To hold a multi-segment string */
 		char string[GMT_LEN256];
 	} h;
-	struct i {	/* -i<col>|<colrange>,... */
-		bool active, orig;
-		uint64_t n_cols;
+	struct i {	/* -i[<col>|<colrange>,...][t[<word>]] */
+		bool active, select, orig, word;
+		uint64_t n_cols, w_col;
 		uint64_t n_actual_cols;
 		char string[GMT_LEN64];
 	} i;
+	struct j {	/* -je|f|g [g] */
+		bool active;
+		enum GMT_enum_mdist mode;	/* Defaults to GMT_GREATCIRCLE */
+		char string[GMT_LEN8];
+	} j;
+	struct l {	/* -l[<label>][+s<size>][+t<title>][+n<ncols>][+d<gap>/<pen>] */
+		bool active;
+		struct GMT_LEGEND_ITEM item;
+	} l;
 	struct n {	/* -n[b|c|l|n][+a][+b<BC>][+c][+t<threshold>] */
 		bool active;
-		bool antialias;	/* Defaults to true, if supported */
-		bool truncate;	/* Defaults to false */
+		bool antialias;		/* Defaults to true, if supported */
+		bool truncate;		/* Defaults to false */
 		unsigned int interpolant;	/* Defaults to BCR_BICUBIC */
-		bool bc_set;	/* true if +b was parsed */
+		bool bc_set;		/* true if +b was parsed */
 		bool periodic[2];	/* For periodic non-geographic grids */
 		char BC[4];		/* For BC settings via +bg|n[x|y]|p[x|y] */
 		double threshold;	/* Defaults to 0.5 */
 		double range[2], half_range[2];	/* For periodic non-geographic grids */
 		char string[GMT_LEN64];	/* Copy of argument */
 	} n;
-	struct o {	/* -o<col>|<colrange>,... */
-		bool active;
-		uint64_t n_cols;
+	struct o {	/* -o[<col>|<colrange>,...][t[<word>]] */
+		bool active, select, orig, word;
+		uint64_t n_cols, w_col;
 	} o;
 	struct p {	/* -p<az>[/<el>[/<z0>]]+wlon0/lat0[/z0]][+vx0[cip]/y0[cip]] */
 		bool active;
@@ -206,8 +230,9 @@ struct GMT_COMMON {
 		bool active;
 		char string[GMT_LEN64];
 	} s;
-	struct t {	/* -t<transparency> */
+	struct t {	/* -t[<transparency>] */
 		bool active;
+		bool variable;
 		double value;
 	} t;
 	struct x {	/* -x[[-]<n>] */
@@ -221,4 +246,4 @@ struct GMT_COMMON {
 	} colon;
 };
 
-#endif /* _GMT_COMMON_H */
+#endif /* GMT_COMMON_H */
