@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *	Copyright (c) 1991-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 
 /*
@@ -26,18 +25,20 @@
 
 #include "gmt_dev.h"
 
+/* Extinct names */
 EXTERN_MSC int GMT_gmtdp (void *V_API, int mode, void *args);
 EXTERN_MSC int GMT_grdreformat (void *V_API, int mode, void *args);
 EXTERN_MSC int GMT_minmax (void *V_API, int mode, void *args);
 EXTERN_MSC int GMT_gmtstitch (void *V_API, int mode, void *args);
 EXTERN_MSC int GMT_gmt2rgb (void *V_API, int mode, void *args);
 EXTERN_MSC int GMT_ps2raster (void *V_API, int mode, void *args);
+EXTERN_MSC int GMT_originator (void *V_API, int mode, void *args);
 
 int GMT_gmtdp (void *V_API, int mode, void *args) {
 	/* This was the GMT4 name */
 	struct GMTAPI_CTRL *API = gmt_get_api_ptr (V_API);	/* Cast from void to GMTAPI_CTRL pointer */
 	if (gmt_M_compat_check (API->GMT, 4)) {
-		GMT_Report (API, GMT_MSG_COMPAT, "Warning: module gmtdp is deprecated; use gmtsimplify.\n");
+		GMT_Report (API, GMT_MSG_COMPAT, "Module gmtdp is deprecated; use gmtsimplify.\n");
 		return (GMT_Call_Module (API, "gmtsimplify", mode, args));
 	}
 	GMT_Report (API, GMT_MSG_NORMAL, "Shared GMT module not found: gmtdp\n");
@@ -48,7 +49,7 @@ int GMT_grdreformat (void *V_API, int mode, void *args) {
 	/* This was the GMT5.1 name */
 	struct GMTAPI_CTRL *API = gmt_get_api_ptr (V_API);	/* Cast from void to GMTAPI_CTRL pointer */
 	if (gmt_M_compat_check (API->GMT, 5)) {
-		GMT_Report (API, GMT_MSG_COMPAT, "Warning: module grdreformat is deprecated; use grdconvert.\n");
+		GMT_Report (API, GMT_MSG_COMPAT, "Module grdreformat is deprecated; use grdconvert.\n");
 		return (GMT_Call_Module (API, "grdconvert", mode, args));
 	}
 	GMT_Report (API, GMT_MSG_NORMAL, "Shared GMT module not found: grdreformat\n");
@@ -59,7 +60,7 @@ int GMT_minmax (void *V_API, int mode, void *args) {
 	/* This was the GMT4 name */
 	struct GMTAPI_CTRL *API = gmt_get_api_ptr (V_API);	/* Cast from void to GMTAPI_CTRL pointer */
 	if (gmt_M_compat_check (API->GMT, 4)) {
-		GMT_Report (API, GMT_MSG_COMPAT, "Warning: module minmax is deprecated; use gmtinfo.\n");
+		GMT_Report (API, GMT_MSG_COMPAT, "Module minmax is deprecated; use gmtinfo.\n");
 		return (GMT_Call_Module (API, "gmtinfo", mode, args));
 	}
 	GMT_Report (API, GMT_MSG_NORMAL, "Shared GMT module not found: minmax\n");
@@ -70,31 +71,32 @@ int GMT_gmtstitch (void *V_API, int mode, void *args) {
 	/* This was the GMT4 name */
 	struct GMTAPI_CTRL *API = gmt_get_api_ptr (V_API);	/* Cast from void to GMTAPI_CTRL pointer */
 	if (gmt_M_compat_check (API->GMT, 4)) {
-		GMT_Report (API, GMT_MSG_COMPAT, "Warning: module gmtstitch is deprecated; use gmtconnect.\n");
+		GMT_Report (API, GMT_MSG_COMPAT, "Module gmtstitch is deprecated; use gmtconnect.\n");
 		return (GMT_Call_Module (API, "gmtconnect", mode, args));
 	}
 	GMT_Report (API, GMT_MSG_NORMAL, "Shared GMT module not found: gmtstitch\n");
 	return (GMT_NOT_A_VALID_MODULE);
 }
 
-int GMT_gmt2rgb (void *V_API, int mode, void *args) {
-	/* This was the GMT4 name */
-	struct GMTAPI_CTRL *API = gmt_get_api_ptr (V_API);	/* Cast from void to GMTAPI_CTRL pointer */
-	if (gmt_M_compat_check (API->GMT, 4)) {
-		GMT_Report (API, GMT_MSG_COMPAT, "Warning: module gmt2rgb is deprecated; use grd2rgb.\n");
-		return (GMT_Call_Module (API, "grd2rgb", mode, args));
-	}
-	GMT_Report (API, GMT_MSG_NORMAL, "Shared GMT module not found: gmt2rgb\n");
-	return (GMT_NOT_A_VALID_MODULE);
-}
-
 int GMT_ps2raster (void *V_API, int mode, void *args) {
 	/* This was the GMT5.1 name */
 	struct GMTAPI_CTRL *API = gmt_get_api_ptr (V_API);	/* Cast from void to GMTAPI_CTRL pointer */
 	if (gmt_M_compat_check (API->GMT, 5)) {
-		GMT_Report (API, GMT_MSG_COMPAT, "Warning: module ps2raster is deprecated; use psconvert.\n");
+		GMT_Report (API, GMT_MSG_COMPAT, "Module ps2raster is deprecated; use psconvert.\n");
 		return (GMT_Call_Module (API, "psconvert", mode, args));
 	}
 	GMT_Report (API, GMT_MSG_NORMAL, "Shared GMT module not found: ps2raster\n");
 	return (GMT_NOT_A_VALID_MODULE);
 }
+
+int GMT_originator (void *V_API, int mode, void *args) {
+	/* This was the GMT5 name */
+	struct GMTAPI_CTRL *API = gmt_get_api_ptr (V_API);	/* Cast from void to GMTAPI_CTRL pointer */
+	if (gmt_M_compat_check (API->GMT, 4)) {
+		GMT_Report (API, GMT_MSG_COMPAT, "Module originator is deprecated; use originater.\n");
+		return (GMT_Call_Module (API, "originater", mode, args));
+	}
+	GMT_Report (API, GMT_MSG_NORMAL, "Shared GMT module not found: originator\n");
+	return (GMT_NOT_A_VALID_MODULE);
+}
+
