@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *	Copyright (c) 1991-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,14 +12,14 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  * gmt_constants.h contains definitions of constants used through GMT.
  *
  * Author:	Paul Wessel
  * Date:	01-OCT-2009
- * Version:	5 API
+ * Version:	6 API
  */
 
 /*!
@@ -28,8 +27,8 @@
  * \brief Definitions of constants used through GMT.
  */
 
-#ifndef _GMT_CONSTANTS_H
-#define _GMT_CONSTANTS_H
+#ifndef GMT_CONSTANTS_H
+#define GMT_CONSTANTS_H
 
 /*=====================================================================================
  *	GMT API CONSTANTS DEFINITIONS
@@ -65,6 +64,9 @@
 #ifndef M_EULER
 #define M_EULER		0.577215664901532860606512	/* Euler's constant (gamma) */
 #endif
+#ifndef M_PHI
+#define M_PHI		1.618033988749894848204587	/* Golden ratio (phi) */
+#endif
 #define MAD_NORMALIZE	1.4826	/*  1/N^{-1}(0.75), where z = \N^{-1}(p) is the inverse cumulative normal distribution */
 
 #define GMT_CONV15_LIMIT 1.0e-15	/* Very tight convergence limit or "close to zero" limit */
@@ -73,27 +75,56 @@
 #define GMT_CONV6_LIMIT	 1.0e-6		/* 1 ppm */
 #define GMT_CONV4_LIMIT	 1.0e-4		/* Less tight convergence limit or "close to zero" limit */
 
-#define GMT_PAD_DEFAULT	2U		/* Default is 2 rows and 2 cols for grid padding */
-
 #define GMT_ASCII_GS	29	/* ASCII code for group separator (temporarily replacing tabs) */
+#define GMT_ASCII_RS	30	/* ASCII code for record separator (temporarily replacing spaces in filenames) */
 #define GMT_ASCII_US	31	/* ASCII code for unit separator (temporarily replacing spaces in quoted text) */
 
+#define GMT_RENAME_FILE	0
+#define GMT_COPY_FILE	1
+
+#define GMT_CONTROLLER	0	/* func_level of controlling process (gmt.c or external API) */
+#define GMT_TOP_MODULE	1	/* func_level of top-level module being called */
+
+#define GMT_PAPER_DIM		32767	/* Upper limit on PostScript paper size under modern mode, in points (~11.6 meters) */
+#define GMT_PAPER_MARGIN	5	/* Default paper margin under modern mode, in inches (~1 meter) */
+
+/*! whether to ignore/read/write history file gmt.history */
+enum GMT_enum_history {
+	/*! 0 */	GMT_HISTORY_OFF = 0,
+	/*! 1 */	GMT_HISTORY_READ,
+	/*! 2 */	GMT_HISTORY_WRITE
+};
+
+/* Specific feature geometries as obtained from OGR */
+/* Note: As far as registering or reading data, GMT only needs to know if data type is POINT, LINE, or POLY */
+
+enum GMT_enum_ogr {
+	GMT_IS_LINESTRING	= 2U,
+	GMT_IS_POLYGON		= 4U,
+	GMT_IS_MULTI		= 64U,	/* Flag for multi-versions of geometry */
+	GMT_IS_MULTIPOINT	= 65U,	/* 1 plus 64 for multi */
+	GMT_IS_MULTILINESTRING	= 66U,	/* 2 plus 64 for multi */
+	GMT_IS_MULTIPOLYGON	= 68U};	/* 4 plus 64 for multi */
+
 /*! Various allocation-length parameters */
 enum GMT_enum_length {
-	GMT_DIM_SIZE	= 4U,	/* Length of dim array used in Read|Creat Data */
+	GMT_DIM_SIZE	= 4U,          /* Length of dim array used in Read|Create Data */
 	GMT_TINY_CHUNK  = 8U,
 	GMT_SMALL_CHUNK = 64U,
 	GMT_CHUNK       = 2048U,
 	GMT_BIG_CHUNK   = 65536U,
-	GMT_LEN16	= 16U,		/* All strings used to format date/clock output must be this length */
-	GMT_LEN32  = 32U,          /* Small length of texts */
-	GMT_LEN64  = 64U,          /* Intermediate length of texts */
-	GMT_LEN128 = 128U,         /* Double of 64 */
-	GMT_LEN256 = 256U,         /* Max size of some text items */
-	GMT_LEN512 = 512U,         /* Max size of other text items */
+	GMT_LEN8	= 8U,
+	GMT_LEN16	= 16U,          /* All strings used to format date/clock output must be this length */
+	GMT_LEN32       = 32U,          /* Small length of texts */
+	GMT_LEN64       = 64U,          /* Intermediate length of texts */
+	GMT_LEN128      = 128U,         /* Double of 64 */
+	GMT_LEN256      = 256U,         /* Max size of some text items */
+	GMT_LEN512      = 512U,         /* Max size of other text items */
+	GMT_LEN1024     = 1024U,        /* For file names (antecipating web fnames) */
 	GMT_MAX_COLUMNS = 4096U,        /* Limit on number of columns in data tables (not grids) */
 	GMT_BUFSIZ      = 4096U,        /* Size of char record for i/o */
-	GMT_MIN_MEMINC  = 1U,        /* E.g., 16 kb of 8-byte doubles */
+	GMT_MSGSIZ      = 16384U,       /* Size of char record for messages and report */
+	GMT_MIN_MEMINC  = 1U,           /* E.g., 16 kb of 8-byte doubles */
 	GMT_MAX_MEMINC  = 67108864U};   /* E.g., 512 Mb of 8-byte doubles */
 
 /*! The four plot length units [m just used internally] */
@@ -103,6 +134,16 @@ enum GMT_enum_unit {
 	GMT_M,
 	GMT_PT};
 
+/*! Flags for 1-D array arg parsing */
+enum GMT_enum_array {
+	GMT_ARRAY_RANGE = 1,
+	GMT_ARRAY_TIME  = 2,
+	GMT_ARRAY_DIST  = 4,
+	GMT_ARRAY_NOINC = 8,
+	GMT_ARRAY_SCALAR = 16,
+	GMT_ARRAY_NOMINMAX = 32,
+	GMT_ARRAY_ROUND = 64};
+
 /*! Handling of swap/no swap in i/o */
 enum GMT_swap_direction {
 	k_swap_none = 0,
@@ -112,9 +153,12 @@ enum GMT_swap_direction {
 #define GMT_DIM_UNITS	"cip"		/* Plot dimensions in cm, inch, or point */
 #define GMT_LEN_UNITS2	"efkMnu"	/* Distances in meter, foot, survey foot, km, Mile, nautical mile */
 #define GMT_LEN_UNITS	"dmsefkMnu"	/* Distances in arc-{degree,minute,second} or meter, foot, km, Mile, nautical mile, survey foot */
+#define GMT_TIME_UNITS	"yowdhms"	/* Time increments in year, month, week, day, hour, min, sec */
+#define GMT_TIME_VAR_UNITS	"yo"	/* Variable time increments in year or month*/
 #define GMT_DIM_UNITS_DISPLAY	"c|i|p"			/* Same, used to display as options */
 #define GMT_LEN_UNITS_DISPLAY	"d|m|s|e|f|k|M|n|u"	/* Same, used to display as options */
 #define GMT_LEN_UNITS2_DISPLAY	"e|f|k|M|n|u"		/* Same, used to display as options */
+#define GMT_TIME_UNITS_DISPLAY	"y|o|w|d|h|m|s"		/* Same, used to display as options */
 #define GMT_DEG2SEC_F	3600.0
 #define GMT_DEG2SEC_I	3600
 #define GMT_SEC2DEG	(1.0 / GMT_DEG2SEC_F)
@@ -148,8 +192,10 @@ enum GMT_swap_direction {
 #define GMT_DEC_WIDTH	0.54	/* Width of a decimal number compared to point size */
 #define GMT_PER_WIDTH	0.30	/* Width of a decimal point compared to point size */
 
-#define GMT_PEN_LEN	128
-#define GMT_PENWIDTH	0.25	/* Default pen width in points */
+#define GMT_PEN_LEN		128
+#define GMT_PENWIDTH		0.25	/* Default pen width in points */
+#define GMT_TEXT_CLEARANCE	15	/* Clearance around text in textboxes, in percent */
+#define GMT_TEXT_OFFSET		20	/* Offset of text from refpoint, in percent */
 
 #define GMT_N_MAX_MODEL	20	/* No more than 20 basis functions in a trend model */
 
@@ -166,11 +212,37 @@ enum GMT_swap_direction {
 #define GMT_CPT_C_REVERSE	1	/* Reverse CPT colors */
 #define GMT_CPT_Z_REVERSE	2	/* Reverse CPT z-values */
 
+/* Default CPTs are initialized in gmt_init.c; see end of gmtinit_new_GMT_ctrl */ 
+#define GMT_DEFAULT_CPT		0	/* Default index into GMT->init.cpt[] array */
+#define GMT_N_CPT		3		/* Number of default CPT types (see GMT->init.cpt in gmt_init.c) */
+#define GMT_DEFAULT_CPT_NAME	"turbo"
+#define GMT_DEM_CPT_NAME		"geo"
+#define GMT_SRTM_CPT_NAME		"srtm"
+
+#define GMT_IS_ROMAN_LCASE	1	/* For converting arabic numerals to Roman */
+#define GMT_IS_ROMAN_UCASE	2
+
+/* Settings for the MAP_FRAME_TYPE = graph */
+#define GMT_GRAPH_EXTENSION		7.5	/* In percent */
+#define GMT_GRAPH_EXTENSION_UNIT	'%'	/* In percent */
+
+/* Modes for subplot status */
+#define GMT_SUBPLOT_ACTIVE	1
+#define GMT_PANEL_NOTSET	2
+
 /*! Codes for grdtrack */
 enum GMT_enum_tracklayout {
-	GMT_LEFT_RIGHT = 0,
-	GMT_EW_SN,
-	GMT_ALTERNATE};
+	GMT_LEFT_RIGHT = 1,
+	GMT_EW_SN = 2,
+	GMT_LEFT_ONLY = 4,
+	GMT_RIGHT_ONLY = 8,
+	GMT_ALTERNATE = 16};
+
+/*! Codes for first segment header */
+enum GMT_enum_firstseg {
+	GMT_FIRST_SEGHEADER_MAYBE = 0,
+	GMT_FIRST_SEGHEADER_ALWAYS,
+	GMT_FIRST_SEGHEADER_NEVER};
 
 /*! Return codes from gmt_inonout */
 enum GMT_enum_inside {
@@ -183,7 +255,7 @@ enum GMT_enum_refpoint {
 	GMT_REFPOINT_NOTSET = -1,	/* -D */
 	GMT_REFPOINT_MAP,		/* -Dg */
 	GMT_REFPOINT_JUST,		/* -Dj */
-	GMT_REFPOINT_JUST_FLIP,	/* -DJ */
+	GMT_REFPOINT_JUST_FLIP,		/* -DJ */
 	GMT_REFPOINT_NORM,		/* -Dn */
 	GMT_REFPOINT_PLOT};		/* -Dx */
 
@@ -197,18 +269,45 @@ enum GMT_enum_rose {
 
 /*! Various types of trend model */
 enum GMT_enum_model {
-	GMT_POLYNOMIAL = 0, GMT_CHEBYSHEV, GMT_COSINE, GMT_SINE, GMT_FOURIER
-	};
+	GMT_POLYNOMIAL = 0, GMT_CHEBYSHEV, GMT_COSINE, GMT_SINE, GMT_FOURIER};
 
 /*! Various array indices of fonts and pens */
 enum GMT_enum_index {
-	GMT_PRIMARY = 0, GMT_SECONDARY = 1
-	};
+	GMT_PRIMARY = 0, GMT_SECONDARY = 1};
 
 /*! Various mode for auto-download */
 enum GMT_enum_download {
-	GMT_NO_DOWNLOAD = 0, GMT_YES_DOWNLOAD = 1
-	};
+	GMT_NO_DOWNLOAD = 0, GMT_YES_DOWNLOAD = 1};
+
+/*! Various mode for auto-legend pens */
+enum GMT_enum_autolegend {
+	GMT_LEGEND_PEN_D  = 0, GMT_LEGEND_PEN_V  = 1,
+	GMT_LEGEND_DRAW_D = 1, GMT_LEGEND_DRAW_V = 2};
+
+//#define GMT_LEGEND_DX1_MUL 1.0	/* Default offset from margin to center of symbol if given as '-' times max symbol size */
+//#define GMT_LEGEND_DX2_MUL 2.0	/* Default offset from margin to start of label if given as '-' times max symbol size */
+#define GMT_LEGEND_DX1_MUL 0.5	/* Default offset from margin to center of symbol if given as '-' times max symbol size */
+#define GMT_LEGEND_DX2_MUL 1.5	/* Default offset from margin to start of label if given as '-' times max symbol size */
+
+/*! Various mode for axes */
+enum GMT_enum_oblique {
+	GMT_OBL_ANNOT_LON_X_LAT_Y    =  0,
+	GMT_OBL_ANNOT_ANYWHERE       =  1,
+	GMT_OBL_ANNOT_LON_HORIZONTAL =  2,
+	GMT_OBL_ANNOT_LAT_HORIZONTAL =  4,
+	GMT_OBL_ANNOT_EXTEND_TICKS   =  8,
+	GMT_OBL_ANNOT_NORMAL_TICKS   = 16,
+	GMT_OBL_ANNOT_LAT_PARALLEL   = 32,
+	GMT_OBL_ANNOT_FLAG_LIMIT     = 64};
+
+/*! Various mode for axes */
+enum GMT_enum_axes {
+	GMT_AXIS_NONE  = 0,
+	GMT_AXIS_DRAW  = 1,
+	GMT_AXIS_TICK  = 2,
+	GMT_AXIS_BARB  = 3,
+	GMT_AXIS_ANNOT = 4,
+	GMT_AXIS_ALL   = 7};
 
 /*! Various options for FFT calculations [Default is 0] */
 enum FFT_implementations {
@@ -298,11 +397,16 @@ enum GMT_enum_anchors {	/* Various anchor strings */
 	GMT_ANCHOR_IMAGE,	/* Anchor for image */
 	GMT_ANCHOR_LEGEND,	/* Anchor for legend */
 	GMT_ANCHOR_COLORBAR,	/* Anchor for colorbar */
-	GMT_ANCHOR_INSERT,	/* Anchor for insert */
+	GMT_ANCHOR_INSET,	/* Anchor for inset */
 	GMT_ANCHOR_MAPSCALE,	/* Anchor for map scale */
 	GMT_ANCHOR_MAPROSE,	/* Anchor for map rose */
+	GMT_ANCHOR_VSCALE,	/* Anchor for vertical scale */
 	GMT_ANCHOR_NTYPES};	/* Number of such types */
 
+enum GMT_enum_scales {	/* Various scale issues */
+	GMT_SCALE_MAP = 1,
+	GMT_SCALE_FREE = 2};
+		
 enum GMT_enum_radius {	/* Various "average" radii for an ellipsoid with axes a,a,b */
 	GMT_RADIUS_MEAN = 0,	/* Mean radius IUGG R_1 = (2*a+b)/3 = a (1 - f/3) */
 	GMT_RADIUS_AUTHALIC,	/* Authalic radius 4*pi*r^2 = surface area of ellipsoid, R_2 = sqrt (0.5a^2 + 0.5b^2 (tanh^-1 e)/e) */
@@ -328,14 +432,14 @@ enum GMT_enum_latswap {GMT_LATSWAP_NONE = -1,	/* Deactivate latswapping */
 enum GMT_enum_geodesic {	/* Various geodesic algorithms */
 	GMT_GEODESIC_VINCENTY = 0,	/* Best possible, currently Vincenty */
 	GMT_GEODESIC_ANDOYER,		/* Faster approximation, currently Andoyer */
-	GMT_GEODESIC_RUDOE};		/* For legacy calculations */
+	GMT_GEODESIC_RUDOE};		/* For legacy GMT4 calculations */
 
 #define METERS_IN_A_FOOT		0.3048			/* 2.54 * 12 / 100 */
 #define METERS_IN_A_SURVEY_FOOT		(1200.0/3937.0)		/* ~0.3048006096 m */
 #define METERS_IN_A_KM			1000.0
-#define METERS_IN_A_MILE		1609.433	/* meters in statute mile */
-#define METERS_IN_A_NAUTICAL_MILE	1852.0
-#define GMT_MAP_DIST_UNIT		'e'		/* Default distance is the meter */
+#define METERS_IN_A_MILE		1609.433		/* meters in statute mile */
+#define METERS_IN_A_NAUTICAL_MILE	1852.0			/* meters in a nautical mile */
+#define GMT_MAP_DIST_UNIT		'e'			/* Default distance is the meter */
 
 enum GMT_enum_coord {GMT_GEOGRAPHIC = 0,	/* Means coordinates are lon,lat : compute spherical distances */
 	GMT_CARTESIAN,	/* Means coordinates are Cartesian x,y : compute Cartesian distances */
@@ -346,6 +450,10 @@ enum GMT_enum_dist {GMT_MAP_DIST = 0,	/* Distance in the map */
 	GMT_CONT_DIST,		/* Distance along a contour or line in dist units */
 	GMT_LABEL_DIST};	/* Distance along a contour or line in dist label units */
 
+enum GMT_enum_inonout {GMT_IOO_UNKNOWN = 0,	/* Decide based on range and type */
+	GMT_IOO_CARTESIAN,			/* Use Cartesian inside function */
+	GMT_IOO_SPHERICAL};			/* Use spherical inside function */
+
 enum GMT_enum_path {GMT_RESAMPLE_PATH = 0,	/* Default: Resample geographic paths based in a max gap allowed (path_step) */
 	GMT_LEAVE_PATH};	/* Options like -A can turn of this resampling, where available */
 
@@ -357,9 +465,11 @@ enum GMT_enum_cdist {GMT_CARTESIAN_DIST	 = 0,	/* Cartesian 2-D x,y data, r = hyp
 	GMT_CARTESIAN_DIST_PERIODIC,		/* Cartesian 2-D x,y data,but with periodic BCs */
 	GMT_CARTESIAN_DIST2,		/* Cartesian 2-D x,y data, return r^2 to avoid hypot */
 	GMT_CARTESIAN_DIST_PROJ,	/* Project lon,lat to Cartesian 2-D x,y data, then get distance */
-	GMT_CARTESIAN_DIST_PROJ2,	/* Same as --"-- but return r^2 to avoid hypot */
+	GMT_CARTESIAN_DIST_PROJ2,	/* Same as --"-- but return r^2 to avoid hypot calls */
 	GMT_CARTESIAN_DIST_PROJ_INV};	/* Project Cartesian 2-D x,y data to lon,lat, then get distance */
-enum GMT_enum_mdist {GMT_FLATEARTH = 1,	/* Compute Flat Earth distances */
+enum GMT_enum_mdist {
+	GMT_NO_MODE = 0,
+	GMT_FLATEARTH,		/* Compute Flat Earth distances */
 	GMT_GREATCIRCLE,	/* Compute great circle distances */
 	GMT_GEODESIC,		/* Compute geodesic distances */
 	GMT_LOXODROME};		/* Compute loxodrome distances (otherwise same as great circle machinery) */
@@ -384,9 +494,14 @@ enum GMT_enum_curl {GMT_REGULAR_FILE = 0,	/* Regular file the may or may not exi
 	GMT_URL_FILE   = 3,	/* Data given by an URL destined for the cache */
 	GMT_URL_QUERY  = 4,	/* Data given by an URL CGI command destined for the cache */
 	GMT_CACHE_DIR  = 0,	/* Use the cache directory */
-	GMT_DATA_DIR   = 1};	/* Use the data directory */
+	GMT_DATA_DIR   = 1,	/* Use the data directory */
+	GMT_LOCAL_DIR  = 2};	/* Use the local (current) directory */
 
-#define GMT_DATA_URL "ftp://ftp.soest.hawaii.edu/gmt/data"	/* URL to GMT data distribution site */
 #define GMT_DATA_PREFIX "earth_relief_"				/* Special prefix for global relief data sets */
+#define GMT_SRTM_PREFIX "srtm_relief_"				/* Special prefix for srtm relief data sets when ocean is not requested */
+#define GMT_SRTM_EXTENSION_REMOTE  "jp2"			/* Tile extension of JPEG2000 files to be downloaded */
+#define GMT_SRTM_EXTENSION_REMOTE_LEN  4U			/* Length of JPEG2000 file extension (+1) */
+#define GMT_SRTM_EXTENSION_LOCAL "nc"				/* Tile extension of nc short int files to be saved */
+#define GMT_SRTM_EXTENSION_LOCAL_LEN 2U				/* Length of  nc short int file extension */
 
-#endif  /* _GMT_CONSTANTS_H */
+#endif  /* GMT_CONSTANTS_H */
