@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *	Copyright (c) 1991-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,14 +12,14 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  * gmt_defaults.h contains definition of the structure with default settings.
  *
  * Author:	Paul Wessel
  * Date:	01-OCT-2009
- * Version:	5 API
+ * Version:	6 API
  */
 
 /*!
@@ -28,8 +27,8 @@
  * \brief Definition of the structure with default settings.
  */
 
-#ifndef _GMT_DEFAULTS_H
-#define _GMT_DEFAULTS_H
+#ifndef GMT_DEFAULTS_H
+#define GMT_DEFAULTS_H
 
 /*--------------------------------------------------------------------
  *			GMT DEFAULTS STRUCTURE DEFINITION
@@ -54,7 +53,7 @@ struct DATUM {
 
 struct GMT_ENCODING {
 	char name[GMT_LEN64];
-	int code[5]; /* Codes for symbols we print. */
+	int code[GMT_LEN8]; /* Codes for 7 symbols we plot but whose code depends on character set (ring, degree, colon, squote, dquote, minus, hyphen). */
 };
 
 /*! Holds all variables directly controlled by GMT Default parameters */
@@ -69,10 +68,12 @@ struct GMT_DEFAULTS {
 	double color_hsv_max_v;			/* For largest or most positive intensity [1.0] */
 	/* DIR group */
 	/* FONT group */
-	struct GMT_FONT font_annot[2];		/* Font for primary and secondary annotations [14p|16p,Helvetica,black] */
-	struct GMT_FONT font_label;		/* Font for labels [24p,Helvetica,black] */
+	struct GMT_FONT font_annot[2];		/* Font for primary and secondary annotations [12p|14p,Helvetica,black] */
+	struct GMT_FONT font_heading;		/* Font for headings above subplots [32p,Helvetica,black] */
+	struct GMT_FONT font_label;		/* Font for labels [16p,Helvetica,black] */
 	struct GMT_FONT font_logo;		/* Font for GMT logo [8p,Helvetica,black] */
-	struct GMT_FONT font_title;		/* Font for headers [36p,Helvetica,black] */
+	struct GMT_FONT font_tag;		/* Font for plot figure tags, e.g., a) [20p,Helvetica,black] */
+	struct GMT_FONT font_title;		/* Font for plot titles [24p,Helvetica,black] */
 	/* FORMAT group */
 	char format_clock_in[GMT_LEN64];	/* How to decode an incoming clock string [hh:mm:ss] */
 	char format_clock_out[GMT_LEN64];	/* Controls how clocks are written on output [hh:mm:ss] */
@@ -88,6 +89,7 @@ struct GMT_DEFAULTS {
 	char format_time[2][GMT_LEN64];	/* Controls annotation format for Months/Weeks/Weekdays for primary and secondary axes */
 	char format_time_stamp[GMT_LEN256];	/* Specify the format for writing time stamps (see strftime) */
 	/* GMT group */
+	size_t url_size_limit;
 	unsigned int compatibility; /* Choose between 4 (GMT4) and up to latest version (5 for now) */
 	unsigned int auto_download;   /* 0 (GMT_NO_DOWNLOAD) or 1 (GMT_YES_DOWNLOAD): For auto-downlaod of known files */
 	unsigned int interpolant; /* Choose between 0 (Linear), 1 (Akima), or 2 (Cubic spline) */
@@ -96,16 +98,20 @@ struct GMT_DEFAULTS {
 	unsigned int fft;         /* Any of FFT_implementations: k_fft_auto, k_fft_accelerate, k_fft_fftw3, k_fft_kiss, k_fft_brenner */
 	unsigned int fftw_plan;   /* Only accessed if HAVE_FFTW3F is defined: Any of FFTW_planner_flags: FFTW_ESTIMATE, FFTW_MEASURE, FFTW_PATIENT, FFTW_EXHAUSTIVE */
 	unsigned int run_mode;     /* Either classic [0] or modern [1] */
-	bool fftwf_threads;   /* Only accessed if HAVE_FFTW3F_THREADS is defined: Any of FFTW_planner_flags: FFTW_ESTIMATE, FFTW_MEASURE, FFTW_PATIENT, FFTW_EXHAUSTIVE */
-	unsigned int history;     /* mode to pass information via history file gmt.history (k_history_off, k_history_read, k_history_write) */
-	unsigned int export_type;     /* What data type to export to external APIs [GMT_DOUBLE] */
+	bool use_modern_name;     /* true if we should use the modern name in usage message */
 	double extrapolate_val[2];/* Choose between [0] = 0, 1D extrapolated vals are NaN, = 1 -> extrapolate, = 2 -> set to const stored in [1] */
+	bool fftwf_threads;   /* Only accessed if HAVE_FFTW3F_THREADS is defined: Any of FFTW_planner_flags: FFTW_ESTIMATE, FFTW_MEASURE, FFTW_PATIENT, FFTW_EXHAUSTIVE */
+	unsigned int history;     /* mode to pass information via history file gmt.history (GMT_HISTORY_OFF, GMT_HISTORY_READ, GMT_HISTORY_WRITE) */
+	unsigned int history_orig;     /* Copy of history */
+	unsigned int export_type;     /* What data type to export to external APIs [GMT_DOUBLE] */
+	unsigned graphics_format;	/* The default graphics format in modern mode [GMT_SESSION_FORMAT] */
 	/* IO group */
 	uint64_t n_bin_header_cols;		/* Minimum number of columns in a binary file for which the all cols == NaN means segment header [2] */
 	unsigned int io_n_header_items;		/* Number of header records expected when -h is used [1]; else 0 */
 	unsigned int io_nan_mode;		/* -s: 1 means skip NaN (x,y) records on output, 2 = inverse (only output nan-records; -sr), 0 reports all records */
 	size_t io_nc4_chunksize[2];         /* NetCDF chunk size (lat,lon) on output [0] */
 	unsigned int io_nc4_deflation_level;/* NetCDF deflation level on output [0] */
+	unsigned int io_first_header;       /* 2 means never write single-segment header, 1 means we always write segment header, 0 means write if needed [0] */
 	bool io_gridfile_shorthand;         /* Use shorthand suffix notation for embedded grid file formats [false] */
 	bool io_header[2];                  /* Input & Output data has header records [false, false] */
 	bool io_header_orig;                /* Copy of Input io_header */
@@ -113,15 +119,17 @@ struct GMT_DEFAULTS {
 	bool io_lonlat_toggle[2];           /* true means read/write I/O as lat/lon instead of lon/lat [false,false] */
 	bool io_blankline[2];               /* true means blank lines should be treated as segment breaks [false,false] */
 	bool io_nanline[2];                 /* true means lines with all NaNs should be treated as segment breaks [false,false] */
-	char io_col_separator[8];           /* Separator between output ASCII data columns [tab] */
+	char io_col_separator[GMT_LEN8];    /* Separator between output ASCII data columns [tab] */
 	char io_gridfile_format[GMT_LEN64]; /* Default grid file format */
 	char io_seg_marker[2];              /* Character used to recognize and write segment headers [>,>] */
+	char io_head_marker[2];             /* Character used to recognize and write header records [#,#] */
 	/* MAP group */
 	double map_annot_offset[2];		/* Distance between primary or secondary annotation and tickmarks [5p/5p] */
 	double map_annot_min_angle;		/* If angle between map boundary and annotation is less, no annotation is drawn [20] */
 	double map_annot_min_spacing;	/* If an annotation is closer that this to an older annotation, the annotation is skipped [0.0] */
 	double map_frame_width;			/* Thickness of fancy map frame [5p] */
 	double map_grid_cross_size[2];	/* Size of primary & secondary gridcrosses.  0 means draw continuous gridlines */
+	double map_heading_offset;		/* Distance between top of panel title and base of subplot heading [18p] */
 	double map_label_offset;		/* Distance between lowermost annotation and top of label [8p] */
 	double map_line_step;			/* Maximum straight linesegment length for arcuate lines [0.75p] */
 	double map_logo_pos[2];			/* Where to plot timestamp relative to origin [BL/-54p/-54p] */
@@ -131,9 +139,11 @@ struct GMT_DEFAULTS {
 	double map_tick_length[4];		/* Length of primary and secondary major and minor tickmarks [5p/2.5p/15p/3.75p] */
 	double map_title_offset;		/* Distance between lowermost annotation (or label) and base of plot title [14p] */
 	double map_vector_shape;		/* 0.0 = straight vectorhead, 1.0 = arrowshape, with continuous range in between */
-	unsigned int map_annot_oblique;	/* Controls annotations and tick angles etc. [0] */
+	double map_graph_extension;		/* If mapframetype is graph, how must longer to make axis length. [7.5%] */
+	unsigned int map_annot_oblique;	/* Controls annotations and tick angles etc. [GMT_OBL_ANNOT_ANYWHERE] */
 	unsigned int map_logo_justify;		/* Justification of the GMT timestamp box [1 (BL)] */
 	unsigned int map_frame_type;		/* Fancy (0), plain (1), or graph (2) [0] */
+	unsigned int map_graph_extension_unit;	/* If mapframetype is graph, the unit is GMT_CM, GMT_INCH, GMT_PT [%] */
 	bool map_logo;			/* Plot time and map projection on map [false] */
 	struct GMT_PEN map_default_pen;		/* Default pen for most pens [0.25p] */
 	struct GMT_PEN map_frame_pen;		/* Pen attributes for map boundary [1.25p] */
@@ -141,7 +151,7 @@ struct GMT_DEFAULTS {
 	struct GMT_PEN map_tick_pen[2];		/* Pen attributes for primary and secondary tickmarks [thinner,black] */
 	char map_frame_axes[6];			/* Which axes to draw and annotate ["WESNZ"]  */
 	char map_annot_ortho[6];		/* Which axes have orthogonal annotations in linear projections ["we"] */
-	enum GMT_enum_symbol { gmt_none = -1, gmt_ring, gmt_degree, gmt_colon, gmt_squote, gmt_dquote, gmt_lastsym } map_degree_symbol;
+	enum GMT_enum_symbol { gmt_none = -1, gmt_ring, gmt_degree, gmt_colon, gmt_squote, gmt_dquote, gmt_minus, gmt_hyphen, gmt_lastsym } map_degree_symbol;
 	/* PROJ group */
 	double proj_scale_factor;		/* Central mapscale factor, typically 0.9996-1 (or -1 for default action) */
 	unsigned int proj_ellipsoid;		/* Which ellipsoid to use [0 = GRS 80] */
@@ -157,12 +167,14 @@ struct GMT_DEFAULTS {
 	double ps_page_rgb[4];			/* Default paper color [white] */
 	double ps_magnify[2];			/* Width and height of paper to plot on in points [Letter or A4] */
 	double ps_transparency;			/* Later transparency [0] */
+	double ps_penwidth;			/* Current pen width */
 	unsigned int ps_color_mode;		/* Postscript encoding of color [PSL_RGB | PSL_CMYK | PSL_HSV | PSL_GRAY] */
 	unsigned int ps_copies;		/* How man copies of each plot [>=1] [GMT4 COMPATIBILITY ONLY] */
 	int ps_media;			/* Default paper media [25(Letter)]; negative if custom size */
 	bool ps_orientation;			/* Orientation of page [false = Landscape, true = Portrait] */
 	bool ps_comments;			/* true if we write comments in the PS file */
-	char ps_transpmode[16];			/* Transparency mode for PDF only */
+	char ps_transpmode[GMT_LEN16];		/* Transparency mode for PDF only */
+	char ps_convert[GMT_LEN256];		/* Arguments for implicit psconvert calls under modern mode [""] */
 	struct GMT_ENCODING ps_encoding;
 	/* TIME group */
 	enum GMT_enum_tictoc timer_mode;	/* Choose GMT_NO_TIMER, GMT_ABS_TIMER, GMT_ELAPSED_TIMER [GMT_NO_TIMER] */
@@ -177,4 +189,4 @@ struct GMT_DEFAULTS {
 	char given_unit[GMT_N_KEYS];		/* Unit given or implied for each setting */
 };
 
-#endif  /* _GMT_DEFAULTS_H */
+#endif  /* GMT_DEFAULTS_H */
