@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *	Copyright (c) 1991-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 
 /*
@@ -21,7 +20,7 @@
  *
  * Author:	Paul Wessel
  * Date:	26-JUN-2012
- * Version:	5 API
+ * Version:	6 API
  */
 
 const char *gmt_api_error_string[] = {
@@ -49,13 +48,19 @@ const char *gmt_api_error_string[] = {
 	/* GMT_IMAGE_BC_ERROR */		"GMT_IMAGE_BC_ERROR",
 	/* GMT_IMAGE_READ_ERROR */		"GMT_IMAGE_READ_ERROR",
 	/* GMT_IMAGE_WRITE_ERROR */		"GMT_IMAGE_WRITE_ERROR",
+	/* GMT_IMAGE_NO_DATA */			"GMT_IMAGE_NO_DATA",
 	/* GMT_MEMORY_ERROR */			"GMT_MEMORY_ERROR",
 	/* GMT_FREE_EXTERNAL_NOT_ALLOWED */	"GMT_FREE_EXTERNAL_NOT_ALLOWED",
 	/* GMT_FREE_WRONG_LEVEL */		"GMT_FREE_WRONG_LEVEL",
+	/* GMT_LOGGING_ALREADY_ACTIVE */		"GMT_LOGGING_ALREADY_ACTIVE",
 	/* GMT_NOT_A_SESSION */			"GMT_NOT_A_SESSION",
+	/* GMT_NOT_MODERN_MODE */		"GMT_NOT_MODERN_MODE",
 	/* GMT_NOT_A_VALID_ARG */		"GMT_NOT_A_VALID_ARG",
 	/* GMT_NOT_A_VALID_DIRECTION */		"GMT_NOT_A_VALID_DIRECTION",
+	/* GMT_NOT_A_VALID_DIRECTORY */		"GMT_NOT_A_VALID_DIRECTORY",
 	/* GMT_NOT_A_VALID_FAMILY */		"GMT_NOT_A_VALID_FAMILY",
+	/* GMT_NOT_A_VALID_LOGMODE */		"GMT_NOT_A_VALID_LOGMODE",
+	/* GMT_WRONG_FAMILY */			"GMT_WRONG_FAMILY",
 	/* GMT_NOT_A_VALID_ID */		"GMT_NOT_A_VALID_ID",
 	/* GMT_NOT_A_VALID_IO_ACCESS */		"GMT_NOT_A_VALID_IO_ACCESS",
 	/* GMT_NOT_A_VALID_IO_MODE */		"GMT_NOT_A_VALID_IO_MODE",
@@ -64,6 +69,7 @@ const char *gmt_api_error_string[] = {
 	/* GMT_NOT_A_VALID_MODE */		"GMT_NOT_A_VALID_MODE",
 	/* GMT_NOT_A_VALID_MODIFIER */		"GMT_NOT_A_VALID_MODIFIER",
 	/* GMT_NOT_A_VALID_MODULE */		"GMT_NOT_A_VALID_MODULE",
+	/* GMT_NOT_A_VALID_OPTION */		"GMT_NOT_A_VALID_OPTION",
 	/* GMT_NOT_A_VALID_PARAMETER */		"GMT_NOT_A_VALID_PARAMETER",
 	/* GMT_NOT_A_VALID_TYPE */		"GMT_NOT_A_VALID_TYPE",
 	/* GMT_NOT_INPUT_OBJECT */		"GMT_NOT_INPUT_OBJECT",
@@ -101,5 +107,6 @@ const char *gmt_api_error_string[] = {
 	/* GMT_STREAM_NOT_ALLOWED */		"GMT_STREAM_NOT_ALLOWED",
 	/* GMT_SUBSET_NOT_ALLOWED */		"GMT_SUBSET_NOT_ALLOWED",
 	/* GMT_VALUE_NOT_SET */			"GMT_VALUE_NOT_SET",
+	/* GMT_NO_STRINGS_ALLOWED */		"GMT_NO_STRINGS_ALLOWED",
 	/* GMT_WRITTEN_ONCE */			"GMT_WRITTEN_ONCE"
 };
