@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *	Copyright (c) 1991-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  * Here are various ways to perform 1-D and 2-D Fourier transforms.
@@ -190,7 +189,7 @@ void gmtlib_suggest_fft_dim (struct GMT_CTRL *GMT, unsigned int n_columns, unsig
 	given_space += n_columns * n_rows;
 	given_space *= 8;
 	if (do_print)
-		GMT_Report (GMT->parent, GMT_MSG_NORMAL, " Data dimension\t%d %d\ttime factor %.8g\trms error %.8e\tbytes %" PRIuS "\n", n_columns, n_rows, given_time, given_err, given_space);
+		GMT_Report (GMT->parent, GMT_MSG_LONG_VERBOSE, " Data dimension\t%d %d\ttime factor %.8g\trms error %.8e\tbytes %" PRIuS "\n", n_columns, n_rows, given_time, given_err, given_space);
 
 	best_err = s_err = t_err = given_err;
 	best_time = s_time = e_time = given_time;
@@ -247,11 +246,11 @@ void gmtlib_suggest_fft_dim (struct GMT_CTRL *GMT, unsigned int n_columns, unsig
 	}
 
 	if (do_print) {
-		GMT_Report (GMT->parent, GMT_MSG_NORMAL, " Highest speed\t%d %d\ttime factor %.8g\trms error %.8e\tbytes %" PRIuS "\n",
+		GMT_Report (GMT->parent, GMT_MSG_LONG_VERBOSE, " Highest speed\t%d %d\ttime factor %.8g\trms error %.8e\tbytes %" PRIuS "\n",
 			nx_best_t, ny_best_t, best_time, t_err, t_space);
-		GMT_Report (GMT->parent, GMT_MSG_NORMAL, " Most accurate\t%d %d\ttime factor %.8g\trms error %.8e\tbytes %" PRIuS "\n",
+		GMT_Report (GMT->parent, GMT_MSG_LONG_VERBOSE, " Most accurate\t%d %d\ttime factor %.8g\trms error %.8e\tbytes %" PRIuS "\n",
 			nx_best_e, ny_best_e, e_time, best_err, e_space);
-		GMT_Report (GMT->parent, GMT_MSG_NORMAL, " Least storage\t%d %d\ttime factor %.8g\trms error %.8e\tbytes %" PRIuS "\n",
+		GMT_Report (GMT->parent, GMT_MSG_LONG_VERBOSE, " Least storage\t%d %d\ttime factor %.8g\trms error %.8e\tbytes %" PRIuS "\n",
 			nx_best_s, ny_best_s, s_time, s_err, best_space);
 	}
 	/* Fastest solution */
@@ -353,16 +352,12 @@ double GMT_FFT_Wavenumber (void *V_API, uint64_t k, unsigned int mode, void *v_K
 	else return (fft_kx (k, K));
 }
 
-#ifdef FORTRAN_API
-double GMT_FFT_Wavenumber_ (uint64_t *k, unsigned int *mode, void *v_K) {
-	/* Fortran version: We pass the global GMT_FORTRAN structure */
-	return (GMT_FFT_Wavenumber (GMT_FORTRAN, *k, *mode, v_K));
-}
-#endif
-
 #ifdef HAVE_FFTW3F
 
 #include <fftw3.h>
+#ifdef _WIN32
+#include <winsock.h>
+#endif
 
 #define FFTWF_WISDOM_FILENAME "fftwf_wisdom"
 
@@ -482,8 +477,8 @@ GMT_LOCAL fftwf_plan gmt_fftwf_plan_dft(struct GMT_CTRL *GMT, unsigned n_rows, u
 		if (plan == NULL) {
 			/* No Wisdom available
 			 * Need extra memory to prevent overwriting data while planning */
-			fftwf_complex *in_place_tmp = fftwf_malloc (2 * (n_rows == 0 ? 1 : n_rows) * n_columns * sizeof(float));
-			GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Generating new FFTW Wisdom, be patient...\n");
+			fftwf_complex *in_place_tmp = fftwf_malloc (2 * (n_rows == 0 ? 1 : n_rows) * n_columns * sizeof(gmt_grdfloat));
+			GMT_Report (GMT->parent, GMT_MSG_LONG_VERBOSE, "Generating new FFTW Wisdom, be patient...\n");
 			if (n_rows == 0) /* 1d DFT */
 				plan = fftwf_plan_dft_1d(n_columns, in_place_tmp, in_place_tmp, sign, GMT->current.setting.fftw_plan);
 			else /* 2d DFT */
@@ -508,14 +503,14 @@ GMT_LOCAL fftwf_plan gmt_fftwf_plan_dft(struct GMT_CTRL *GMT, unsigned n_rows, u
 	}
 
 	if (plan == NULL) { /* There was a problem creating a plan */
-		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Error: Could not create FFTW plan.\n");
+		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Could not create FFTW plan.\n");
 		GMT_exit (GMT, GMT_ARG_IS_NULL); return NULL;
 	}
 
 	return plan;
 }
 
-GMT_LOCAL int fft_1d_fftwf (struct GMT_CTRL *GMT, float *data, unsigned int n, int direction, unsigned int mode) {
+GMT_LOCAL int fft_1d_fftwf (struct GMT_CTRL *GMT, gmt_grdfloat *data, unsigned int n, int direction, unsigned int mode) {
 	fftwf_plan plan = NULL;
 	gmt_M_unused(mode);
 
@@ -527,7 +522,7 @@ GMT_LOCAL int fft_1d_fftwf (struct GMT_CTRL *GMT, float *data, unsigned int n, i
 	return GMT_NOERROR;
 }
 
-GMT_LOCAL int fft_2d_fftwf (struct GMT_CTRL *GMT, float *data, unsigned int n_columns, unsigned int n_rows, int direction, unsigned int mode) {
+GMT_LOCAL int fft_2d_fftwf (struct GMT_CTRL *GMT, gmt_grdfloat *data, unsigned int n_columns, unsigned int n_rows, int direction, unsigned int mode) {
 	fftwf_plan plan = NULL;
 	gmt_M_unused(mode);
 
@@ -551,7 +546,7 @@ GMT_LOCAL void fft_1d_vDSP_reset (struct GMT_FFT_HIDDEN *Z) {
 	}
 }
 
-GMT_LOCAL int fft_1d_vDSP (struct GMT_CTRL *GMT, float *data, unsigned int n, int direction, unsigned int mode) {
+GMT_LOCAL int fft_1d_vDSP (struct GMT_CTRL *GMT, gmt_grdfloat *data, unsigned int n, int direction, unsigned int mode) {
 	FFTDirection fft_direction = direction == GMT_FFT_FWD ?
 			kFFTDirection_Forward : kFFTDirection_Inverse;
 	DSPComplex *dsp_complex = (DSPComplex *)data;
@@ -571,8 +566,8 @@ GMT_LOCAL int fft_1d_vDSP (struct GMT_CTRL *GMT, float *data, unsigned int n, in
 		 * single-precision FFT functions: */
 		fft_1d_vDSP_reset (&GMT->current.fft);
 		GMT->current.fft.setup_1d = vDSP_create_fftsetup (log2n, kFFTRadix2);
-		GMT->current.fft.dsp_split_complex_1d.realp = malloc (n * sizeof(float));
-		GMT->current.fft.dsp_split_complex_1d.imagp = malloc (n * sizeof(float));
+		GMT->current.fft.dsp_split_complex_1d.realp = calloc (n, sizeof(gmt_grdfloat));
+		GMT->current.fft.dsp_split_complex_1d.imagp = calloc (n, sizeof(gmt_grdfloat));
 		if (GMT->current.fft.dsp_split_complex_1d.realp == NULL || GMT->current.fft.dsp_split_complex_1d.imagp == NULL) {
 			GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Unable to allocate dsp_split_complex array of length %u\n", n);
 			return -1; /* out of memory */
@@ -596,7 +591,7 @@ GMT_LOCAL void fft_2d_vDSP_reset (struct GMT_FFT_HIDDEN *Z) {
 	}
 }
 
-GMT_LOCAL int fft_2d_vDSP (struct GMT_CTRL *GMT, float *data, unsigned int n_columns, unsigned int n_rows, int direction, unsigned int mode) {
+GMT_LOCAL int fft_2d_vDSP (struct GMT_CTRL *GMT, gmt_grdfloat *data, unsigned int n_columns, unsigned int n_rows, int direction, unsigned int mode) {
 	FFTDirection fft_direction = direction == GMT_FFT_FWD ?
 			kFFTDirection_Forward : kFFTDirection_Inverse;
 	DSPComplex *dsp_complex = (DSPComplex *)data;
@@ -619,8 +614,8 @@ GMT_LOCAL int fft_2d_vDSP (struct GMT_CTRL *GMT, float *data, unsigned int n_col
 	 	* single-precision FFT functions: */
 		fft_2d_vDSP_reset (&GMT->current.fft);
 		GMT->current.fft.setup_2d = vDSP_create_fftsetup (MAX (log2nx, log2ny), kFFTRadix2);
-		GMT->current.fft.dsp_split_complex_2d.realp = malloc (n_xy * sizeof(float));
-		GMT->current.fft.dsp_split_complex_2d.imagp = malloc (n_xy * sizeof(float));
+		GMT->current.fft.dsp_split_complex_2d.realp = calloc (n_xy, sizeof(gmt_grdfloat));
+		GMT->current.fft.dsp_split_complex_2d.imagp = calloc (n_xy, sizeof(gmt_grdfloat));
 		if (GMT->current.fft.dsp_split_complex_2d.realp == NULL || GMT->current.fft.dsp_split_complex_2d.imagp == NULL) {
 			GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Unable to allocate dsp_split_complex array of length %u\n", n_xy);
 			return -1; /* out of memory */
@@ -646,7 +641,7 @@ GMT_LOCAL int fft_2d_vDSP (struct GMT_CTRL *GMT, float *data, unsigned int n_col
 
 #include "kiss_fft/kiss_fftnd.h"
 
-GMT_LOCAL int fft_1d_kiss (struct GMT_CTRL *GMT, float *data, unsigned int n, int direction, unsigned int mode) {
+GMT_LOCAL int fft_1d_kiss (struct GMT_CTRL *GMT, gmt_grdfloat *data, unsigned int n, int direction, unsigned int mode) {
 	kiss_fft_cpx *fin, *fout;
 	kiss_fft_cfg config;
 	gmt_M_unused(GMT); gmt_M_unused(mode);
@@ -660,7 +655,7 @@ GMT_LOCAL int fft_1d_kiss (struct GMT_CTRL *GMT, float *data, unsigned int n, in
 	return GMT_NOERROR;
 }
 
-GMT_LOCAL int fft_2d_kiss (struct GMT_CTRL *GMT, float *data, unsigned int n_columns, unsigned int n_rows, int direction, unsigned int mode) {
+GMT_LOCAL int fft_2d_kiss (struct GMT_CTRL *GMT, gmt_grdfloat *data, unsigned int n_columns, unsigned int n_rows, int direction, unsigned int mode) {
 	const int dim[2] = {n_rows, n_columns}; /* dimensions of fft */
 	const int dimcount = 2;      /* number of dimensions */
 	kiss_fft_cpx *fin, *fout;
@@ -678,7 +673,7 @@ GMT_LOCAL int fft_2d_kiss (struct GMT_CTRL *GMT, float *data, unsigned int n_col
 }
 
 
-GMT_LOCAL int fft_brenner_fourt_f (float *data, int *nn, int *ndim, int *ksign, int *iform, float *work) {
+GMT_LOCAL int fft_brenner_fourt_f (gmt_grdfloat *data, int *nn, int *ndim, int *ksign, int *iform, gmt_grdfloat *work) {
 
     /* System generated locals */
     int i__1, i__2, i__3, i__4, i__5, i__6, i__7, i__8, i__9, i__10, i__11, i__12;
@@ -695,9 +690,9 @@ GMT_LOCAL int fft_brenner_fourt_f (float *data, int *nn, int *ndim, int *ksign,
     static int nhalf, krang, kconj, kdif, idim, ntot, kstep, k2, k3, k4, nprev, iquot;
     static int i2, i1, i3, j3, k1, j2, j1, if_, np0, np1, np2, ifp1, ifp2, non2;
 
-    static float theta, oldsi, tempi, oldsr, sinth, difi, difr, sumi, sumr, tempr, twopi;
-    static float wstpi, wstpr, twowr, wi, wr, u1i, w2i, w3i, u2i, u3i, u4i, t2i, u1r;
-    static float u2r, u3r, w2r, w3r, u4r, t2r, t3r, t3i, t4r, t4i;
+    static gmt_grdfloat theta, oldsi, tempi, oldsr, sinth, difi, difr, sumi, sumr, tempr, twopi;
+    static gmt_grdfloat wstpi, wstpr, twowr, wi, wr, u1i, w2i, w3i, u2i, u3i, u4i, t2i, u1r;
+    static gmt_grdfloat u2r, u3r, w2r, w3r, u4r, t2r, t3r, t3i, t4r, t4i;
     static double wrd, wid;
 
 /*---------------------------------------------------------------------------
@@ -724,8 +719,8 @@ GMT_LOCAL int fft_brenner_fourt_f (float *data, int *nn, int *ndim, int *ksign,
     --data;
 
     /* Function Body */
-    wr = wi = wstpr = wstpi = (float)0.0;
-    twopi = (float)6.283185307;
+    wr = wi = wstpr = wstpi = (gmt_grdfloat)0.0;
+    twopi = (gmt_grdfloat)6.283185307;
     if (*ndim - 1 >= 0) {
 	goto L1;
     } else {
@@ -964,7 +959,7 @@ L370:
 	    goto L380;
 	}
 L380:
-	theta = -twopi * (float) non2 / (float) (mmax <<  2);
+	theta = -twopi * (gmt_grdfloat) non2 / (gmt_grdfloat) (mmax <<  2);
 	if (*ksign >= 0) {
 	    goto L390;
 	} else {
@@ -974,10 +969,10 @@ L390:
 	theta = -theta;
 L400:
 	sincos ((double)theta, &wid, &wrd);
-	wr = (float)wrd;
-	wi = (float)wid;
-	wstpr = (float)-2.0 * wi * wi;
-	wstpi = (float)2.0 * wr * wi;
+	wr = (gmt_grdfloat)wrd;
+	wi = (gmt_grdfloat)wid;
+	wstpr = (gmt_grdfloat)-2.0 * wi * wi;
+	wstpi = (gmt_grdfloat)2.0 * wr * wi;
 L405:
 	i__4 = lmax;
 	i__5 = non2t;
@@ -990,7 +985,7 @@ L405:
 	    }
 L410:
 	    w2r = wr * wr - wi * wi;
-	    w2i = (float)(wr * 2.0 * wi);
+	    w2i = (gmt_grdfloat)(wr * 2.0 * wi);
 	    w3r = w2r * wr - w2i * wi;
 	    w3i = w2r * wi + w2i * wr;
 L420:
@@ -1150,7 +1145,7 @@ L615:
 	i__5 = ifp1;
 	i__4 = ifp2;
 	for (j2 = j2min; i__4 < 0 ? j2 >= i__5 : j2 <= i__5; j2 +=  i__4) {
-	    theta = -twopi * (float) (j2 - 1) / (float)  np2;
+	    theta = -twopi * (gmt_grdfloat) (j2 - 1) / (gmt_grdfloat)  np2;
 	    if (*ksign >= 0) {
 		goto L620;
 	    } else {
@@ -1159,10 +1154,10 @@ L615:
 L620:
 	    theta = -theta;
 L625:
-	    sinth = (float)sin((double)(0.5 * theta));
-	    wstpr = sinth * (float)(-2. * sinth);
-	    wstpi = (float)sin((double)theta);
-	    wr = wstpr + (float)1.0;
+	    sinth = (gmt_grdfloat)sin((double)(0.5 * theta));
+	    wstpr = sinth * (gmt_grdfloat)(-2. * sinth);
+	    wstpi = (gmt_grdfloat)sin((double)theta);
+	    wr = wstpr + (gmt_grdfloat)1.0;
 	    wi = wstpi;
 	    j1min = j2 + ifp1;
 	    i__3 = j1rng;
@@ -1191,7 +1186,7 @@ L625:
 	    }
 	}
 L640:
-	theta = -twopi / (float) ifact[if_ - 1];
+	theta = -twopi / (gmt_grdfloat) ifact[if_ - 1];
 	if (*ksign >= 0) {
 	    goto L645;
 	} else {
@@ -1200,9 +1195,9 @@ L640:
 L645:
 	theta = -theta;
 L650:
-	sinth = (float)sin((double)(0.5 * theta));
-	wstpr = sinth * (float)(-2. * sinth);
-	wstpi = (float)sin((double)theta);
+	sinth = (gmt_grdfloat)sin((double)(0.5 * theta));
+	wstpr = sinth * (gmt_grdfloat)(-2. * sinth);
+	wstpi = (gmt_grdfloat)sin((double)theta);
 	kstep = (n << 1) / ifact[if_ - 1];
 	krang = kstep * (ifact[if_ - 1] / 2) + 1;
 	i__2 = i1rng;
@@ -1229,8 +1224,8 @@ L650:
 				goto L665;
 			    }
 L655:
-			    sumr = (float)0.0;
-			    sumi = (float)0.0;
+			    sumr = (gmt_grdfloat)0.0;
+			    sumi = (gmt_grdfloat)0.0;
 			    i__11 = j2max;
 			    i__12 = ifp2;
 			    for (j2 = j3; i__12 < 0 ? j2 >= i__11 : j2 <= i__11; j2 += i__12) {
@@ -1245,8 +1240,8 @@ L665:
 			    j2 = j2max;
 			    sumr = data[j2];
 			    sumi = data[j2 + 1];
-			    oldsr = (float)0.0;
-			    oldsi = (float)0.0;
+			    oldsr = (gmt_grdfloat)0.0;
+			    oldsi = (gmt_grdfloat)0.0;
 			    j2 -= ifp2;
 L670:
 			    tempr = sumr;
@@ -1280,7 +1275,7 @@ L680:
 			goto L686;
 		    }
 L685:
-		    wr = wstpr + (float)1.0;
+		    wr = wstpr + (gmt_grdfloat)1.0;
 		    wi = wstpi;
 		    goto L690;
 L686:
@@ -1363,7 +1358,7 @@ L700:
 L701:
 	nhalf = n;
 	n += n;
-	theta = -twopi / (float) n;
+	theta = -twopi / (gmt_grdfloat) n;
 	if (*ksign >= 0) {
 	    goto L702;
 	} else {
@@ -1372,10 +1367,10 @@ L701:
 L702:
 	theta = -theta;
 L703:
-	sinth = (float)sin((double)(0.5 * theta));
-	wstpr = sinth * (float)(-2. * sinth);
-	wstpi = (float)sin((double)theta);
-	wr = wstpr + (float)1.0;
+	sinth = (gmt_grdfloat)sin((double)(0.5 * theta));
+	wstpr = sinth * (gmt_grdfloat)(-2. * sinth);
+	wstpi = (gmt_grdfloat)sin((double)theta);
+	wr = wstpr + (gmt_grdfloat)1.0;
 	wi = wstpi;
 	imin = 3;
 	jmin = (nhalf << 1) - 1;
@@ -1385,10 +1380,10 @@ L710:
 	i__4 = ntot;
 	i__3 = np2;
 	for (i = imin; i__3 < 0 ? i >= i__4 : i <= i__4; i += i__3) {
-	    sumr = (float)0.5 * (data[i] + data[j]);
-	    sumi = (float)0.5 * (data[i + 1] + data[j + 1]);
-	    difr = (float)0.5 * (data[i] - data[j]);
-	    difi = (float)0.5 * (data[i + 1] - data[j + 1]);
+	    sumr = (gmt_grdfloat)0.5 * (data[i] + data[j]);
+	    sumi = (gmt_grdfloat)0.5 * (data[i + 1] + data[j + 1]);
+	    difr = (gmt_grdfloat)0.5 * (data[i] - data[j]);
+	    difi = (gmt_grdfloat)0.5 * (data[i + 1] - data[j + 1]);
 	    tempr = wr * sumi + wi * difr;
 	    tempi = wi * sumi - wr * difr;
 	    data[i] = sumr + tempr;
@@ -1444,7 +1439,7 @@ L755:
 	}
 L760:
 	data[j] = data[imin] - data[imin + 1];
-	data[j + 1] = (float)0.0;
+	data[j + 1] = (gmt_grdfloat)0.0;
 	if (i - j >= 0) {
 	    goto L780;
 	} else {
@@ -1463,12 +1458,12 @@ L770:
 	}
 L775:
 	data[j] = data[imin] + data[imin + 1];
-	data[j + 1] = (float)0.0;
+	data[j + 1] = (gmt_grdfloat)0.0;
 	imax = imin;
 	goto L745;
 L780:
 	data[1] += data[2];
-	data[2] = (float)0.0;
+	data[2] = (gmt_grdfloat)0.0;
 	goto L900;
 L800:
 	if (i1rng - np1 >= 0) {
@@ -1565,8 +1560,8 @@ GMT_LOCAL size_t fft_brenner_worksize (struct GMT_CTRL *GMT, unsigned int n_colu
 }
 
 /* C-callable wrapper for fft_brenner_fourt_f */
-GMT_LOCAL int fft_1d_brenner (struct GMT_CTRL *GMT, float *data, unsigned int n, int direction, unsigned int mode) {
-        /* void GMT_fourt (struct GMT_CTRL *GMT, float *data, int *nn, int ndim, int ksign, int iform, float *work) */
+GMT_LOCAL int fft_1d_brenner (struct GMT_CTRL *GMT, gmt_grdfloat *data, unsigned int n, int direction, unsigned int mode) {
+        /* void GMT_fourt (struct GMT_CTRL *GMT, gmt_grdfloat *data, int *nn, int ndim, int ksign, int iform, gmt_grdfloat *work) */
         /* Data array */
         /* Dimension array */
         /* Number of dimensions */
@@ -1576,16 +1571,16 @@ GMT_LOCAL int fft_1d_brenner (struct GMT_CTRL *GMT, float *data, unsigned int n,
 	
         int ksign, ndim = 1, n_signed = n, kmode = mode;
         size_t work_size = 0;
-        float *work = NULL;
+        gmt_grdfloat *work = NULL;
 	
         ksign = (direction == GMT_FFT_INV) ? +1 : -1;
-        if ((work_size = fft_brenner_worksize (GMT, n, 1))) work = gmt_M_memory (GMT, NULL, work_size, float);
+        if ((work_size = fft_brenner_worksize (GMT, n, 1))) work = gmt_M_memory (GMT, NULL, work_size, gmt_grdfloat);
         (void) fft_brenner_fourt_f (data, &n_signed, &ndim, &ksign, &kmode, work);
         gmt_M_free (GMT, work);	
         return (GMT_OK);
 }
 	
-GMT_LOCAL int fft_2d_brenner (struct GMT_CTRL *GMT, float *data, unsigned int n_columns, unsigned int n_rows, int direction, unsigned int mode) {
+GMT_LOCAL int fft_2d_brenner (struct GMT_CTRL *GMT, gmt_grdfloat *data, unsigned int n_columns, unsigned int n_rows, int direction, unsigned int mode) {
         /* Data array */
         /* Dimension array */
         /* Number of dimensions */
@@ -1595,11 +1590,11 @@ GMT_LOCAL int fft_2d_brenner (struct GMT_CTRL *GMT, float *data, unsigned int n_
 
         int ksign, ndim = 2, nn[2] = {n_columns, n_rows}, kmode = mode;
         size_t work_size = 0;
-        float *work = NULL;
+        gmt_grdfloat *work = NULL;
 
         ksign = (direction == GMT_FFT_INV) ? +1 : -1;
-        if ((work_size = fft_brenner_worksize (GMT, n_columns, n_rows))) work = gmt_M_memory (GMT, NULL, work_size, float);
-        GMT_Report (GMT->parent, GMT_MSG_LONG_VERBOSE, "Brenner_fourt_ work size = %" PRIuS "\n", work_size);
+        if ((work_size = fft_brenner_worksize (GMT, n_columns, n_rows))) work = gmt_M_memory (GMT, NULL, work_size, gmt_grdfloat);
+        GMT_Report (GMT->parent, GMT_MSG_DEBUG, "Brenner_fourt_ work size = %" PRIuS "\n", work_size);
         (void) fft_brenner_fourt_f (data, nn, &ndim, &ksign, &kmode, work);
         gmt_M_free (GMT, work);
         return (GMT_OK);
@@ -1611,7 +1606,7 @@ GMT_LOCAL int fft_1d_selection (struct GMT_CTRL *GMT, uint64_t n) {
 		/* Specific selection requested */
 		if (GMT->session.fft1d[GMT->current.setting.fft])
 			return GMT->current.setting.fft; /* User defined FFT */
-		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Desired FFT Algorithm (%s) not configured - choosing suitable alternative.\n", GMT_fft_algo[GMT->current.setting.fft]);
+		GMT_Report (GMT->parent, GMT_MSG_VERBOSE, "Desired FFT Algorithm (%s) not configured - choosing suitable alternative.\n", GMT_fft_algo[GMT->current.setting.fft]);
 	}
 	/* Here we want automatic selection from available candidates */
 	if (GMT->session.fft1d[k_fft_accelerate] && radix2 (n))
@@ -1627,7 +1622,7 @@ GMT_LOCAL int fft_2d_selection (struct GMT_CTRL *GMT, unsigned int n_columns, un
 		/* Specific selection requested */
 		if (GMT->session.fft2d[GMT->current.setting.fft])
 			return GMT->current.setting.fft; /* User defined FFT */
-		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Desired FFT Algorithm (%s) not configured - choosing suitable alternative.\n", GMT_fft_algo[GMT->current.setting.fft]);
+		GMT_Report (GMT->parent, GMT_MSG_VERBOSE, "Desired FFT Algorithm (%s) not configured - choosing suitable alternative.\n", GMT_fft_algo[GMT->current.setting.fft]);
 	}
 	/* Here we want automatic selection from available candidates */
 	if (GMT->session.fft2d[k_fft_accelerate] && radix2 (n_columns) && radix2 (n_rows))
@@ -1637,7 +1632,7 @@ GMT_LOCAL int fft_2d_selection (struct GMT_CTRL *GMT, unsigned int n_columns, un
 	return k_fft_kiss; /* Default/fallback general-purpose FFT */
 }
 
-int GMT_FFT_1D (void *V_API, float *data, uint64_t n, int direction, unsigned int mode) {
+int GMT_FFT_1D (void *V_API, gmt_grdfloat *data, uint64_t n, int direction, unsigned int mode) {
 	/* data is an array of length n (or 2*n for complex) data points
 	 * n is the number of data points
 	 * direction is either GMT_FFT_FWD (forward) or GMT_FFT_INV (inverse)
@@ -1657,14 +1652,7 @@ int GMT_FFT_1D (void *V_API, float *data, uint64_t n, int direction, unsigned in
 	return status;
 }
 
-#ifdef FORTRAN_API
-int GMT_FFT_1D_ (float *data, uint64_t *n, int *direction, unsigned int *mode) {
-	/* Fortran version: We pass the global GMT_FORTRAN structure */
-	return (GMT_FFT_1D (GMT_FORTRAN, data, *n, *direction, *mode));
-}
-#endif
-
-int GMT_FFT_2D (void *V_API, float *data, unsigned int n_columns, unsigned int n_rows, int direction, unsigned int mode) {
+int GMT_FFT_2D (void *V_API, gmt_grdfloat *data, unsigned int n_columns, unsigned int n_rows, int direction, unsigned int mode) {
 	/* data is an array of length n_columns*n_rows (or 2*n_columns*n_rows for complex) data points
 	 * n_columns, n_rows is the number of data nodes
 	 * direction is either GMT_FFT_FWD (forward) or GMT_FFT_INV (inverse)
@@ -1685,13 +1673,6 @@ int GMT_FFT_2D (void *V_API, float *data, unsigned int n_columns, unsigned int n
 	return status;
 }
 
-#ifdef FORTRAN_API
-int GMT_FFT_2D_ (float *data, unsigned int *n_columns, unsigned int *n_rows, int *direction, unsigned int *mode) {
-	/* Fortran version: We pass the global GMT_FORTRAN structure */
-	return (GMT_FFT_2D (GMT_FORTRAN, data, *n_columns, *n_rows, *direction, *mode));
-}
-#endif
-
 void gmt_fft_initialization (struct GMT_CTRL *GMT) {
 	/* Called by gmt_begin and sets up pointers to the available FFT calls */
 #if defined HAVE_FFTW3F_THREADS
@@ -1701,7 +1682,7 @@ void gmt_fft_initialization (struct GMT_CTRL *GMT) {
 		/* one-time initialization required to use FFTW3 threads */
 		if ( fftwf_init_threads() ) {
 			fftwf_plan_with_nthreads(n_cpu);
-			GMT_Report (GMT->parent, GMT_MSG_LONG_VERBOSE, "Initialize FFTW with %d threads.\n", n_cpu);
+			GMT_Report (GMT->parent, GMT_MSG_DEBUG, "Initialize FFTW with %d threads.\n", n_cpu);
 		}
 	}
 
