@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *	Copyright (c) 1991-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *      This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 
 GMT_LOCAL char **breakMe(struct GMT_CTRL *GMT, char *in) {
@@ -80,7 +79,7 @@ GMT_LOCAL int grid_gdal_librarified (struct GMT_CTRL *GMT, char *gdal_filename,
 	args = breakMe(GMT, opts);
 	psOptions = GDALInfoOptionsNew(args, NULL); 
 	info = GDALInfo(hDataset, psOptions);
-	GMT_Report (GMT->parent, GMT_MSG_NORMAL, "GDAL Info\n\n%s\n", info);
+	GMT_Report (GMT->parent, GMT_MSG_LONG_VERBOSE, "GDAL Info\n\n%s\n", info);
 
 	GDALInfoOptionsFree(psOptions);
 	GDALClose(hDataset);
