@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *	Copyright (c) 1991-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *      This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /* Program:	gmt_gdalread.c
  * Purpose:	routine to read files supported by gdal
@@ -35,8 +34,7 @@
 GMT_LOCAL GDALDatasetH gdal_open (struct GMT_CTRL *GMT, char *gdal_filename) {
 	char *file = NULL, path[PATH_MAX] = {""};
 	if (gmtlib_check_url_name (gdal_filename))	/* A vis*** URL, pass to GDAL as is */
-		//file = gdal_filename;
-		strcpy (path, gdal_filename);
+		strncpy (path, gdal_filename, PATH_MAX-1);
 	else if (strchr(gdal_filename, ':'))		/* Assume it is a SUBDATASET */
 		strncpy (path, gdal_filename, PATH_MAX-1);
 	else if ((file = gmt_getdatapath (GMT, gdal_filename, path, R_OK)) == NULL) {	/* Local file not found */
@@ -46,24 +44,21 @@ GMT_LOCAL GDALDatasetH gdal_open (struct GMT_CTRL *GMT, char *gdal_filename) {
 	return (GDALOpen (path, GA_ReadOnly));
 }
 
-GMT_LOCAL int gdal_decode_columns (struct GMT_CTRL *GMT, char *txt, int *whichBands, unsigned int n_col) {
+GMT_LOCAL int gdal_decode_columns (struct GMT_CTRL *GMT, char *txt, int *whichBands) {
 	unsigned int n = 0, i, start, stop, pos = 0;
-	char p[GMT_BUFSIZ];
+	char p[GMT_LEN256];
 	gmt_M_unused(GMT);
 
 	while ((gmt_strtok (txt, ",", &pos, p))) {
-		if (strchr (p, '-'))
+		if (strchr (p, '-')) {
 			sscanf (p, "%d-%d", &start, &stop);
-		else {
-			sscanf (p, "%d", &start);
-			stop = start;
-		}
-		stop = MIN (stop, n_col);
-		for (i = start; i <= stop; i++) {
-			whichBands[n] = i + 1;			/* Band numbering in GMT is 0 based */
-			n++;
+			for (i = start; i <= stop; i++)
+				whichBands[n++] = i;
 		}
+		else
+			sscanf (p, "%d", &whichBands[n++]);
 	}
+	for (i = 0; i < n; i++) whichBands[i] += 1;	/* Band numbering in GMT is 0 based */
 	return ((int)n);
 }
 
@@ -86,7 +81,7 @@ GMT_LOCAL int record_geotransform (char *gdal_filename, GDALDatasetH hDataset, d
 		return (0);
 
 	/* Try again, but try "a.tif.wld" instead. */
-	sprintf (generic_buffer, "%s.xxx", gdal_filename);
+	snprintf (generic_buffer, 5000, "%s.xxx", gdal_filename);
 	status = GDALReadWorldFile (generic_buffer, "wld", adfGeoTransform);
 	if (status == 1)
 		return (0);
@@ -98,19 +93,16 @@ GMT_LOCAL int record_geotransform (char *gdal_filename, GDALDatasetH hDataset, d
 /*                        ReportCorner()                                */
 /************************************************************************/
 
-GMT_LOCAL int ReportCorner (struct GMT_CTRL *GMT, GDALDatasetH hDataset, double x, double y, double *xy_c, double *xy_geo) {
+GMT_LOCAL int ReportCorner (struct GMT_CTRL *GMT, GDALDatasetH hDataset, OGRCoordinateTransformationH hTransform,
+                            double x, double y, double *xy_c, double *xy_geo) {
 	double	dfGeoX, dfGeoY;
-	const char  *pszProjection = NULL;
 	double	adfGeoTransform[6];
-	OGRCoordinateTransformationH hTransform = NULL;
-	OGRSpatialReferenceH hProj, hLatLong = NULL;
 
 	xy_geo[0] = xy_geo[1] = GMT->session.d_NaN;		/* Default return values */
-/* -------------------------------------------------------------------- */
-/*      Transform the point into georeferenced coordinates.             */
-/* -------------------------------------------------------------------- */
+	/* -------------------------------------------------------------------- */
+	/*      Transform the point into georeferenced coordinates.             */
+	/* -------------------------------------------------------------------- */
 	if (GDALGetGeoTransform(hDataset, adfGeoTransform) == CE_None) {
-		pszProjection = GDALGetProjectionRef(hDataset);
 		if (!strcmp(GDALGetDriverShortName(GDALGetDatasetDriver(hDataset)),"netCDF") && GDAL_VERSION_NUM <= 1450) {
 			adfGeoTransform[3] *= -1;
 			adfGeoTransform[5] *= -1;
@@ -123,42 +115,20 @@ GMT_LOCAL int ReportCorner (struct GMT_CTRL *GMT, GDALDatasetH hDataset, double
 		return false;
 	}
 
-/* -------------------------------------------------------------------- */
-/*      Report the georeferenced coordinates.                           */
-/* -------------------------------------------------------------------- */
+	/* -------------------------------------------------------------------- */
+	/*      Report the georeferenced coordinates.                           */
+	/* -------------------------------------------------------------------- */
 	xy_c[0] = dfGeoX;	xy_c[1] = dfGeoY;
 
-/* -------------------------------------------------------------------- */
-/*      Setup transformation to lat/long.                               */
-/* -------------------------------------------------------------------- */
-	if (pszProjection != NULL && pszProjection[0] != '\0') {
-
-		hProj = OSRNewSpatialReference(pszProjection);
-		if (hProj != NULL) hLatLong = OSRCloneGeogCS(hProj);
-
-		if (hLatLong != NULL) {
-			CPLPushErrorHandler(CPLQuietErrorHandler);
-			hTransform = OCTNewCoordinateTransformation(hProj, hLatLong);
-			CPLPopErrorHandler();
-			OSRDestroySpatialReference(hLatLong);
-		}
-
-		if (hProj != NULL) OSRDestroySpatialReference(hProj);
-	}
-/*
- * --------------------------------------------------------------------
- *      Transform to latlong and report.
- * --------------------------------------------------------------------
- */
+	/* --------------------------------------------------------------------
+	 *      Transform to latlong and report.
+	 * -------------------------------------------------------------------- */
 	if (hTransform != NULL && OCTTransform(hTransform,1,&dfGeoX,&dfGeoY,NULL)) {
 		xy_geo[0] = dfGeoX;
 		xy_geo[1] = dfGeoY;
 	}
 
-	if (hTransform != NULL)
-		OCTDestroyCoordinateTransformation(hTransform);
-
-	return true;
+	return 0;
 }
 
 /* -------------------------------------------------------------------- */
@@ -278,8 +248,9 @@ int get_attrib_from_string(struct GMT_GDALREAD_OUT_CTRL *Ctrl, GDALRasterBandH h
 	return (GMT_NOERROR);
 }
 
-GMT_LOCAL int populate_metadata (struct GMT_CTRL *GMT, struct GMT_GDALREAD_OUT_CTRL *Ctrl, char *gdal_filename, int got_R, int nXSize,
-                                 int nYSize, double dfULX, double dfULY, double dfLRX, double dfLRY, double z_min, double z_max) {
+GMT_LOCAL int populate_metadata (struct GMT_CTRL *GMT, struct GMT_GDALREAD_OUT_CTRL *Ctrl, char *gdal_filename, int got_R,
+                                 int nXSize, int nYSize, double dfULX, double dfULY, double dfLRX, double dfLRY,
+								 double z_min, double z_max, int first_layer) {
 /* =============================================================================================== */
 /*
  * This routine queries the GDAL raster file for some metadata
@@ -336,7 +307,9 @@ GMT_LOCAL int populate_metadata (struct GMT_CTRL *GMT, struct GMT_GDALREAD_OUT_C
 	GDALRasterBandH hBand;
 	GDALColorTableH	hTable;
 	GDALColorEntry	sEntry;
+	OGRCoordinateTransformationH hTransform = NULL;
 
+	const char  *pszProjection = NULL;
 	int     i, j;
 	int     status, bSuccess;	/* success or failure */
 	int     nBand, raster_count;
@@ -412,7 +385,7 @@ GMT_LOCAL int populate_metadata (struct GMT_CTRL *GMT, struct GMT_GDALREAD_OUT_C
 	}
 	else if (adfGeoTransform[1] != 1 && adfGeoTransform[5] != 1) {	/* Patch a possible GDAL bug. Raised after issue #1030 */
 		adfGeoTransform[1] = adfGeoTransform[5] = 1;
-		GMT_Report(GMT->parent, GMT_MSG_VERBOSE, "Warning: GDAL seamed to have returned garbage in adfGeoTransform. Arbitrarily setting inc = 1.\n");
+		GMT_Report(GMT->parent, GMT_MSG_VERBOSE, "GDAL seamed to have returned garbage in adfGeoTransform. Arbitrarily setting inc = 1.\n");
 	}
 
 	/* ------------------------------------------------------------------------- */
@@ -432,7 +405,7 @@ GMT_LOCAL int populate_metadata (struct GMT_CTRL *GMT, struct GMT_GDALREAD_OUT_C
 		anSrcWin[0] = anSrcWin[1] = anSrcWin[2] = anSrcWin[3] = 0;
 		if (adfGeoTransform[2] != 0.0 || adfGeoTransform[4] != 0.0) {
 			GMT_Report (GMT->parent, GMT_MSG_NORMAL, "The -projwin option was used, but the geotransform is rotated."
-							" This configuration is not supported.\n");
+			                                         " This configuration is not supported.\n");
 			GDALClose(hDataset);
 			GDALDestroyDriverManager();
 			GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Quitting with error\n");
@@ -520,7 +493,7 @@ GMT_LOCAL int populate_metadata (struct GMT_CTRL *GMT, struct GMT_GDALREAD_OUT_C
 		   Note, however, that first band colormap is captured anyway (if exists, off course) */
 
 		if (bSuccess == 0 && strstr(Ctrl->DriverShortName, "HDF5") != NULL) {	/* several methods for HDF5 driver are not implemented */
-			GMT_Report(GMT->parent, GMT_MSG_VERBOSE, "An HDF5 file. Trying to get scale_offset from string metadata.\n");
+			GMT_Report(GMT->parent, GMT_MSG_LONG_VERBOSE, "An HDF5 file. Trying to get scale_offset from string metadata.\n");
 			dfNoDataValue = GMT->session.d_NaN;
 			get_attrib_from_string(Ctrl, hBand, nBand, &dfNoDataValue);			/* Go get them from the metadata in strings. */
 			if (!isnan(dfNoDataValue)) got_noDataValue = true;
@@ -530,7 +503,7 @@ GMT_LOCAL int populate_metadata (struct GMT_CTRL *GMT, struct GMT_GDALREAD_OUT_C
 	/* ------------------------------------------------------------------------- */
 	/* Get the Color Interpretation Name */
 	/* ------------------------------------------------------------------------- */
-	hBand = GDALGetRasterBand(hDataset, 1);
+	hBand = GDALGetRasterBand(hDataset, first_layer);
 	if (raster_count > 0)
 		Ctrl->color_interp = GDALGetColorInterpretationName(GDALGetRasterColorInterpretation(hBand));
 	else
@@ -572,34 +545,63 @@ GMT_LOCAL int populate_metadata (struct GMT_CTRL *GMT, struct GMT_GDALREAD_OUT_C
 	/* ------------------------------------------------------------------------- */
 	/* Record corners. */
 	/* ------------------------------------------------------------------------- */
+
+	/* -------------------------------------------------------------------- */
+	/*      Setup projected to lat/long transform if appropriate.           */
+	/* -------------------------------------------------------------------- */
+	if (!got_R) {
+		if (GDALGetGeoTransform(hDataset, adfGeoTransform) == CE_None)
+			pszProjection = GDALGetProjectionRef(hDataset);
+
+		if (pszProjection != NULL && strlen(pszProjection) > 0) {
+			OGRSpatialReferenceH hProj, hLatLong = NULL;
+			hProj = OSRNewSpatialReference( pszProjection );
+			if (hProj != NULL)
+				hLatLong = OSRCloneGeogCS(hProj);
+
+			if (hLatLong != NULL) {
+				CPLPushErrorHandler( CPLQuietErrorHandler );
+				hTransform = OCTNewCoordinateTransformation( hProj, hLatLong );
+				CPLPopErrorHandler();
+				OSRDestroySpatialReference(hLatLong);
+			}
+
+			if (hProj != NULL)
+				OSRDestroySpatialReference(hProj);
+		}
+	}
+
 	if (!got_R)					/* Lower Left */
-		ReportCorner (GMT, hDataset, 0.0, GDALGetRasterYSize(hDataset), xy_c, xy_geo[0]);
+		ReportCorner (GMT, hDataset, hTransform, 0.0, GDALGetRasterYSize(hDataset), xy_c, xy_geo[0]);
 	else
 		xy_c[0] = dfULX, xy_c[1] = dfLRY;
 	Ctrl->Corners.LL[0] = Ctrl->hdr[0] = xy_c[0];	/* xmin, ymin */
 	Ctrl->Corners.LL[1] = Ctrl->hdr[2] = xy_c[1];
 
 	if (!got_R)					/* Upper Left */
-		ReportCorner (GMT, hDataset, 0.0, 0.0, xy_c, xy_geo[1]);
+		ReportCorner (GMT, hDataset, hTransform, 0.0, 0.0, xy_c, xy_geo[1]);
 	else
 		xy_c[0] = dfULX, xy_c[1] = dfULY;
 	Ctrl->Corners.UL[0] = xy_c[0];
 	Ctrl->Corners.UL[1] = xy_c[1];
 
 	if (!got_R)					/* Upper Right */
-		ReportCorner (GMT, hDataset, GDALGetRasterXSize(hDataset), 0.0, xy_c, xy_geo[2]);
+		ReportCorner (GMT, hDataset, hTransform, GDALGetRasterXSize(hDataset), 0.0, xy_c, xy_geo[2]);
 	else
 		xy_c[0] = dfLRX, xy_c[1] = dfULY;
 	Ctrl->Corners.UR[0] = Ctrl->hdr[1] = xy_c[0];	/* xmax, ymax */
 	Ctrl->Corners.UR[1] = Ctrl->hdr[3] = xy_c[1];
 
 	if (!got_R)					/* Lower Right */
-		ReportCorner (GMT, hDataset, GDALGetRasterXSize(hDataset), GDALGetRasterYSize(hDataset), xy_c, xy_geo[3]);
+		ReportCorner (GMT, hDataset, hTransform, GDALGetRasterXSize(hDataset), GDALGetRasterYSize(hDataset), xy_c, xy_geo[3]);
 	else
 		xy_c[0] = dfLRX, xy_c[1] = dfLRY;
 	Ctrl->Corners.LR[0] = xy_c[0];
 	Ctrl->Corners.LR[1] = xy_c[1];
 
+	if (hTransform != NULL)
+		OCTDestroyCoordinateTransformation(hTransform);
+
 	/* Must check that geog grids have not y_max > 90.0 We'll be eps tollerant ... but how do I know that it's geog?*/
 
 	/* --------------------------------------------------------------------------------------
@@ -674,14 +676,19 @@ GMT_LOCAL int populate_metadata (struct GMT_CTRL *GMT, struct GMT_GDALREAD_OUT_C
 
 int gmt_is_gdal_grid (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header) {
 	GDALDatasetH hDataset;
+	GDALDriverH	hDriver;
+	struct GMT_GRID_HEADER_HIDDEN *HH = gmt_get_H_hidden (header);
 
 	GDALAllRegister();
-	hDataset = gdal_open (GMT, header->name);
+	if (strstr(HH->name, ".jp2") || strstr(HH->name, ".JP2"))
+		if ((hDriver = GDALGetDriverByName("JP2OpenJPEG")) != NULL && (hDriver = GDALGetDriverByName("JP2ECW")) != NULL)
+			GDALDeregisterDriver(hDriver);		/* Deregister the JP2ECW driver. That is, prefer the OpenJPEG one */
+	hDataset = gdal_open (GMT, HH->name);
 
 	if (hDataset == NULL)
 		return (GMT_GRDIO_BAD_VAL);
-	GMT_Report (GMT->parent, GMT_MSG_VERBOSE, "File %s reads with GDAL driver %s\n",
-	            header->name, GDALGetDriverShortName(GDALGetDatasetDriver(hDataset)));
+	GMT_Report (GMT->parent, GMT_MSG_LONG_VERBOSE, "File %s reads with GDAL driver %s\n",
+	            HH->name, GDALGetDriverShortName(GDALGetDatasetDriver(hDataset)));
 	GDALClose (hDataset);
 	GDALDestroyDriverManager();
 	header->type = GMT_GRID_IS_GD;
@@ -695,22 +702,22 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 	int	complex_mode = 0;	/* 0 real only. 1|2 if complex array is to hold real (1) and imaginary (2) parts */
 	int	nPixelSize, nBands, i, nReqBands = 0;
 	int	anSrcWin[4], xOrigin = 0, yOrigin = 0;
-	int	jump = 0, nXSize = 0, nYSize = 0, nX, nY, nBufXSize, nBufYSize;
-	int	n, m;
+	int	jump = 0, nXSize = 0, nYSize = 0, nX, nY;
+	int nBufXSize, nBufYSize, buffy, startRow = 0, endRow;
+	int nRowsPerBlock, nBlocks, nYOff, row_i, row_e;
+	int k, pad = 0, pad_w = 0, pad_e = 0, pad_s = 0, pad_n = 0;    /* Different pads for when sub-regioning near the edges */
 	int	incStep = 1;	/* 1 for real only arrays and 2 for complex arrays (index step increment) */
-	int error = 0, gdal_code = 0;
+	int error = 0, gdal_code = 0, first_layer;
 	bool   do_BIP;		/* For images if BIP == true data is stored Pixel interleaved, otherwise Band interleaved */
 	bool   metadata_only;
 	bool   pixel_reg = false;	/* GDAL decides everything is pixel reg, we make our decisions based on data type */
 	bool   fliplr, got_R = false, got_r = false;
 	bool   topdown = false, rowmajor = true;               /* arrays from GDAL have this order */
 	bool   just_copy = false, copy_flipud = false;
-	int	   *whichBands = NULL, *rowVec = NULL, *colVec = NULL;
-	int     off, pad = 0, i_x_nXYSize, startColPos = 0, startRow = 0, nXSize_withPad = 0, nYSize_withPad;
-	int     pad_w = 0, pad_e = 0, pad_s = 0, pad_n = 0;    /* Different pads for when sub-regioning near the edges */
-	unsigned int nn, mm;
-	uint64_t ij;
-	size_t  n_alloc;
+	int	   *whichBands = NULL;
+	int64_t *rowVec = NULL, *colVec = NULL;
+	int64_t  off, i_x_nXYSize, startColPos = 0, nXSize_withPad = 0, nYSize_withPad;
+	int64_t  n_alloc, n, m, nn, mm, ij;
 	unsigned char *tmp = NULL;
 	double  adfMinMax[2];
 	double  dfULX = 0.0, dfULY = 0.0, dfLRX = 0.0, dfLRY = 0.0;
@@ -724,20 +731,8 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 	gmt_M_memset (anSrcWin, 4, int);
 
 	if (prhs->B.active) {		/* We have a selected bands request */
-		int nc_ind, n_commas = 0, n_dash = 0;
-		for (nc_ind = 0; prhs->B.bands[nc_ind]; nc_ind++)
-			if (prhs->B.bands[nc_ind] == ',') n_commas++;
-		for (n = 0; prhs->B.bands[n]; n++)
-			if (prhs->B.bands[n] == '-') n_dash = (int)n;
-		nn = MAX(n_commas+1, n_dash);
-		if (nn) {
-			nn = MAX(nn, (unsigned int)atoi(&prhs->B.bands[nc_ind-1])+1);	/* +1 because band numbering in GMT is zero based */
-			if (n_dash)	nn = MAX(nn, (unsigned int)atoi(&prhs->B.bands[nn+1])+1);
-		}
-		else		/* Hmm, this else case is never reached */
-			nn = atoi(prhs->B.bands);
-		whichBands = gmt_M_memory (GMT, NULL, nn, int);
-		nReqBands = gdal_decode_columns (GMT, prhs->B.bands, whichBands, nn);
+		whichBands = gmt_M_memory (GMT, NULL, 128, int);		/* 128 is huge enough */
+		nReqBands = gdal_decode_columns (GMT, prhs->B.bands, whichBands);
 	}
 	else if (prhs->f_ptr.active) {
 		/* Here we are going to read to a grid so if no band info was provided, default to read only the
@@ -748,6 +743,14 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 		whichBands[0] = 1;
 		Ctrl->Float.active = true;		/* Signals that output will be in the float array, no matter input type */
 	}
+	first_layer = (nReqBands) ? whichBands[0] : 1;		/* The one used to get data type and header info */
+
+	if (nReqBands && GMT->current.setting.verbose >= GMT_MSG_LONG_VERBOSE) {
+		GMT_Report (GMT->parent, GMT_MSG_LONG_VERBOSE, "Read band(s):");
+		for (k = 0; k < nReqBands; k++)
+			GMT_Message (GMT->parent, GMT_TIME_NONE, "\t%d", whichBands[k]);
+		GMT_Message (GMT->parent, GMT_TIME_NONE, "\n");
+	}
 
 	do_BIP = prhs->I.active;
 	if (nReqBands == 1) do_BIP = false;	/* This must overrule any -I option settings */
@@ -779,41 +782,39 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 	if (prhs->R.active) {
 		double wesn[4];
 		got_R = true;
-		wesn[XLO] = GMT->common.R.wesn[XLO];		wesn[XHI] = GMT->common.R.wesn[XHI];
-		wesn[YLO] = GMT->common.R.wesn[YLO];		wesn[YHI] = GMT->common.R.wesn[YHI];
-		GMT->common.R.active[RSET] = false;	/* Reset because -R was already parsed when reading header info */
-		error += gmt_parse_common_options (GMT, "R", 'R', prhs->R.region);
+		error += (GMT_Get_Values (GMT->parent, prhs->R.region, wesn, 4) < 4);
+		
 		if (!error) {
 			double dx = 0, dy = 0;
 			if (!prhs->registration.val) {	/* Subregion coords are grid-reg. Need to convert to pix-reg */
 				dx = prhs->registration.x_inc / 2;
 				dy = prhs->registration.y_inc / 2;
 			}
-			dfULX = GMT->common.R.wesn[XLO] - dx;
-			dfLRX = GMT->common.R.wesn[XHI] + dx;
-			dfLRY = GMT->common.R.wesn[YLO] - dy;
-			dfULY = GMT->common.R.wesn[YHI] + dy;
+			dfULX = wesn[XLO] - dx;
+			dfLRX = wesn[XHI] + dx;
+			dfLRY = wesn[YLO] - dy;
+			dfULY = wesn[YHI] + dy;
 			if (pad) {
-				pad_w = (int)((wesn[XLO] - GMT->common.R.wesn[XLO]) / prhs->registration.x_inc + 0.5);
-				pad_e = (int)((GMT->common.R.wesn[XHI] - wesn[XHI]) / prhs->registration.x_inc + 0.5);
-				pad_s = (int)((wesn[YLO] - GMT->common.R.wesn[YLO]) / prhs->registration.y_inc + 0.5);
-				pad_n = (int)((GMT->common.R.wesn[YHI] - wesn[YHI]) / prhs->registration.y_inc + 0.5);
+				pad_w = (int)((GMT->common.R.wesn[XLO] - wesn[XLO]) / prhs->registration.x_inc + 0.5);
+				pad_e = (int)((wesn[XHI] - GMT->common.R.wesn[XHI]) / prhs->registration.x_inc + 0.5);
+				pad_s = (int)((GMT->common.R.wesn[YLO] - wesn[YLO]) / prhs->registration.y_inc + 0.5);
+				pad_n = (int)((wesn[YHI] - GMT->common.R.wesn[YHI]) / prhs->registration.y_inc + 0.5);
 			}
 		}
 	}
 
 	if (prhs->r.active) { 		/* Region is given in pixels */
+		double wesn[4];
 		got_r = true;
-		GMT->common.R.active[RSET] = false;
-		error += gmt_parse_common_options (GMT, "R", 'R', prhs->r.region);
+		error += (GMT_Get_Values (GMT->parent, prhs->R.region, wesn, 4) < 4);
 		if (!error) {
-			dfULX = GMT->common.R.wesn[XLO];	dfLRX = GMT->common.R.wesn[XHI];
-			dfLRY = GMT->common.R.wesn[YLO];	dfULY = GMT->common.R.wesn[YHI];
+			dfULX = wesn[XLO];	dfLRX = wesn[XHI];
+			dfLRY = wesn[YLO];	dfULY = wesn[YHI];
 		}
 	}
 
 	if (error) {
-		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Error: gmt_gdalread failed to extract a Sub-region\n");
+		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "gmt_gdalread failed to extract a Sub-region\n");
 		return (-1);
 	}
 
@@ -831,7 +832,6 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 
 	if (prhs->W.active) {
 		OGRSpatialReferenceH  hSRS;
-
 		hSRS = OSRNewSpatialReference(NULL);
 
 		if (OSRImportFromProj4(hSRS, Ctrl->ProjRefPROJ4) == CE_None) {
@@ -842,16 +842,26 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 		}
 		else {
 			Ctrl->ProjRefWKT = NULL;
-			GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Warning: gmt_gdalread failed to convert the proj4 string\n%s\n to WKT\n",
+			GMT_Report (GMT->parent, GMT_MSG_VERBOSE, "gmt_gdalread failed to convert the proj4 string\n%s\n to WKT\n",
 					Ctrl->ProjRefPROJ4);
 		}
 
 		OSRDestroySpatialReference(hSRS);
+		gmt_M_free (GMT, whichBands);
 		return (GMT_NOERROR);
 	}
 
+	if (getenv("GDAL_HTTP_UNSAFESSL"))	/* The fact that it exist is not enough. It might be from a GMT process only. */
+		CPLSetConfigOption ("GDAL_HTTP_UNSAFESSL", getenv("GDAL_HTTP_UNSAFESSL"));
+	if (getenv("CURL_CA_BUNDLE"))		/* And the same for this one. */
+		CPLSetConfigOption ("CURL_CA_BUNDLE", getenv("CURL_CA_BUNDLE"));
+
+	if (strstr(gdal_filename, ".jp2") || strstr(gdal_filename, ".JP2"))
+		if ((hDriver = GDALGetDriverByName("JP2OpenJPEG")) != NULL && (hDriver = GDALGetDriverByName("JP2ECW")) != NULL)
+			GDALDeregisterDriver(hDriver);		/* Deregister the JP2ECW driver. That is, prefer the OpenJPEG one */
+
 	if (metadata_only) {
-		if (populate_metadata (GMT, Ctrl, gdal_filename, got_R, nXSize, nYSize, dfULX, dfULY, dfLRX, dfLRY, z_min, z_max))
+		if (populate_metadata (GMT, Ctrl, gdal_filename, got_R, nXSize, nYSize, dfULX, dfULY, dfLRX, dfLRY, z_min, z_max, first_layer))
 			return(-1);
 
 		/* Return registration based on data type of first band. Byte is pixel reg otherwise set grid registration */
@@ -866,6 +876,7 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 
 	if (hDataset == NULL) {
 		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "GDALOpen failed %s\n", CPLGetLastErrorMsg());
+		gmt_M_free (GMT, whichBands);
 		return (-1);
 	}
 
@@ -895,6 +906,7 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 			            "The -projwin option was used, but the geotransform is rotated. This configuration is not supported.\n");
 			GDALClose(hDataset);
 			GDALDestroyDriverManager();
+			gmt_M_free (GMT, whichBands);
 			return (-1);
 		}
 
@@ -921,6 +933,7 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 			GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Computed -srcwin falls outside raster size of %dx%d.\n",
 			            GDALGetRasterXSize(hDataset), GDALGetRasterYSize(hDataset));
 			GDALDestroyDriverManager();
+			gmt_M_free (GMT, whichBands);
 			return (-1);
 		}
 		xOrigin = anSrcWin[0];
@@ -934,8 +947,8 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 		nYSize = GDALGetRasterYSize(hDataset);
 	}
 
-	/* The following assumes that all bands have the same PixelSize. Otherwise ... */
-	hBand = GDALGetRasterBand(hDataset,1);
+	/* The following assumes that all bands have the same PixelSize, data type. Otherwise ... */
+	hBand = GDALGetRasterBand(hDataset, first_layer);
 	nPixelSize = GDALGetDataTypeSize(GDALGetRasterDataType(hBand)) / 8;	/* /8 because return value is in BITS */
 
 	if (jump) {
@@ -950,7 +963,7 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 
 	if (nReqBands) nBands = MIN(nBands,nReqBands);	/* If a band selection was made */
 
-	n_alloc = nBands * (nBufXSize + pad_w + pad_e) * (nBufYSize + pad_s + pad_n);
+	n_alloc = ((size_t)nBands) * ((size_t)nBufXSize + pad_w + pad_e) * ((size_t)nBufYSize + pad_s + pad_n);
 	switch (GDALGetRasterDataType(hBand)) {
 		case GDT_Byte:
 			if (prhs->c_ptr.active)	/* We have a pointer with already allocated memory ready to use */
@@ -974,8 +987,10 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 				Ctrl->Int16.data = gmt_M_memory (GMT, NULL, n_alloc, int16_t);
 			break;
 		case GDT_UInt16:
-			if (prhs->f_ptr.active)		/* Use the previously allocated float pointer */
+			if (prhs->f_ptr.active) {	/* Use the previously allocated float pointer */
 				Ctrl->Float.data = prhs->f_ptr.grd;
+				Ctrl->Float.active = true;		/* In case it was not set yet */
+			}
 			else
 				Ctrl->UInt16.data = gmt_M_memory (GMT, NULL, n_alloc, uint16_t);
 			break;
@@ -1007,11 +1022,21 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 			break;
 	}
 
-	tmp = calloc(nBufYSize * nBufXSize, nPixelSize);
-	if (tmp == NULL) {
-		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "gdalread: failure to allocate enough memory\n");
-		GDALDestroyDriverManager();
-		return(-1);
+	/* 16 Mb worth of rows */
+	nRowsPerBlock = MIN(nYSize, (int)(1024 * 1024 * 16 / (nXSize * nPixelSize)));
+	nBlocks = (int)ceil((float)nYSize / nRowsPerBlock);
+
+	if (!(just_copy || copy_flipud)) {
+		if ((tmp = calloc((size_t)nRowsPerBlock * (size_t)nBufXSize, nPixelSize)) == NULL) {
+			GMT_Report (GMT->parent, GMT_MSG_NORMAL, "gdalread: failure to allocate enough memory\n");
+			GDALDestroyDriverManager();
+			gmt_M_free (GMT, whichBands);
+			return(-1);
+		}
+	}
+	else {
+		nRowsPerBlock = nYSize;
+		nBlocks = 1;
 	}
 
 	/* ------ compute two vectors indices that will be used inside loops below --------- */
@@ -1021,9 +1046,9 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 	else
 		nX = nXSize,	nY = nYSize;
 
-	rowVec = gmt_M_memory(GMT, NULL, nY, int);
+	rowVec = gmt_M_memory(GMT, NULL, nRowsPerBlock*nBlocks, size_t);
 	for (m = 0; m < nY; m++) rowVec[m] = m * nX;
-	colVec = gmt_M_memory(GMT, NULL, nX+pad_w+pad_e, int);	/* For now this will be used only to select BIP ordering */
+	colVec = gmt_M_memory(GMT, NULL, nX+pad_w+pad_e, size_t);	/* For now this will be used only to select BIP ordering */
 	/* --------------------------------------------------------------------------------- */
 
 	for (i = 0; i < nBands; i++) {
@@ -1049,8 +1074,27 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 				pixel_reg = true;
 		}
 
-		if ((gdal_code = GDALRasterIO(hBand, GF_Read, xOrigin, yOrigin, nXSize, nYSize, tmp,
-		                 nBufXSize, nBufYSize, GDALGetRasterDataType(hBand), 0, 0)) != CE_None) {
+		i_x_nXYSize = i * ((size_t)nBufXSize + pad_w + pad_e) * ((size_t)nBufYSize + pad_s + pad_n);
+
+		for (k = 0; k < nBlocks; k++) {
+			nYOff = yOrigin + k * nRowsPerBlock;	/* Move data Y origin to the beginning of next block to be read */
+			row_i = k * nRowsPerBlock;
+			row_e = (k + 1) * nRowsPerBlock;
+			buffy = nRowsPerBlock;
+			for (m = 0; m < nRowsPerBlock; m++) rowVec[m+k*nRowsPerBlock] = m * nX;
+			if (k == nBlocks-1) {					/* Last block only by chance is not smaller than the others */
+				buffy = nBufYSize - k * nRowsPerBlock;
+				row_e = nYSize;
+				nYOff = yOrigin + k * nRowsPerBlock;
+			}
+			startRow = row_i;
+			endRow   = row_e;
+
+			if (just_copy || copy_flipud)					/* In this case nBlocks was set to 1 above */
+				tmp = &Ctrl->UInt8.data[i_x_nXYSize];		/* These cases don't need any temporary array */
+
+		if ((gdal_code = GDALRasterIO(hBand, GF_Read, xOrigin, nYOff, nXSize, buffy, tmp,
+		                 nBufXSize, buffy, GDALGetRasterDataType(hBand), 0, 0)) != CE_None) {
 			GMT_Report (GMT->parent, GMT_MSG_NORMAL, "GDALRasterIO failed to open band %d [err = %d]\n", i, gdal_code);
 			continue;
 		}
@@ -1060,25 +1104,24 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 
 		/* In the "Preview" mode those guys below are different and what we need is the BufSize */
 		if (jump) {
-			nXSize = nBufXSize;
-			nYSize = nBufYSize;
-			i_x_nXYSize = i*(nXSize + pad_w + pad_e)*(nYSize + pad_s + pad_n);		/* We don't need to recompute this every time */
+			nXSize = nBufXSize;		nYSize = nBufYSize;
 		}
-		else
-			i_x_nXYSize = i * (nBufXSize + pad_w + pad_e) * (nBufYSize + pad_s + pad_n);
 
 		startColPos = pad_w + i_x_nXYSize + (complex_mode > 1);	/* Take into account nBands, Padding and Complex */
 		nXSize_withPad = nXSize + pad_w + pad_e;
 		nYSize_withPad = nYSize + pad_n + pad_s;
 
-		if (prhs->mini_hdr.active) {
+		if (prhs->mini_hdr.active) {				/* Read into a padded zone that can be larger then grid dims (grdpaste) */
 			if (prhs->mini_hdr.side[0] == 'l' || prhs->mini_hdr.side[0] == 'r') {
 				nXSize_withPad = prhs->mini_hdr.mx;
 				if (prhs->mini_hdr.side[0] == 'r')
 					startColPos += prhs->mini_hdr.offset;
 			}
 			else if (prhs->mini_hdr.side[0] == 'b') {
-				startRow = prhs->mini_hdr.offset;
+				startRow = prhs->mini_hdr.offset + k * nRowsPerBlock;
+				endRow = startRow + nRowsPerBlock;
+				if (k == nBlocks-1)
+					endRow = prhs->mini_hdr.offset + nYSize;
 			}
 		}
 
@@ -1100,7 +1143,7 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 				if (do_BIP || !GMT->current.gdal_read_in.O.mem_layout[0]) {
 					/* Currently all calls to send image to GMT (BIP case) must come through here */
 					if (rowmajor) {
-						for (m = 0; m < nYSize; m++) {
+						for (m = row_i; m < row_e; m++) {
 							off = nRGBA * pad_w + (pad_w+m) * (nRGBA * (nXSize_withPad)); /* Remember, nRGBA is variable */
 							for (n = 0; n < nXSize; n++)
 								Ctrl->UInt8.data[colVec[n] + off] = tmp[rowVec[m]+n];
@@ -1109,29 +1152,29 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 					else {
 						for (n = 0; n < nXSize; n++) {
 							off = nRGBA * pad_n + (pad_n+n) * (nRGBA * nYSize_withPad) + i;
-							for (m = 0; m < nYSize; m++)
+							for (m = row_i; m < row_e; m++)
 								Ctrl->UInt8.data[m * nRGBA + off] = tmp[rowVec[m] + n];
 						}
 					}
 				}
 				else {
-					if (just_copy) {	/* Here we send out the array as is, but the usage of a tmp array was an waste. Needs fix */
-						memcpy (&Ctrl->UInt8.data[i_x_nXYSize], tmp, nBufYSize * nBufXSize);
+					if (just_copy) {	/* Here we send out the array as is, but the usage of a tmp array was a waste. Needs fix */
+						memcpy (&Ctrl->UInt8.data[i_x_nXYSize], tmp, (size_t)nBufYSize * (size_t)nBufXSize);
 					}
 					else if (copy_flipud) {
 						/* We could copy and flip but the idea here is also to not use a tmp array too */
-						memcpy (&Ctrl->UInt8.data[i_x_nXYSize], tmp, nBufYSize * nBufXSize);
-						gmtlib_grd_flip_vertical (&Ctrl->UInt8.data[i_x_nXYSize], (unsigned)nX, (unsigned)nY, 0, 1);
+						memcpy (&Ctrl->UInt8.data[i_x_nXYSize], tmp, (size_t)nBufYSize * (size_t)nBufXSize);
+						gmt_grd_flip_vertical (&Ctrl->UInt8.data[i_x_nXYSize], (unsigned)nX, (unsigned)nY, 0, 1);
 					}
 					else if (fliplr) {				/* No BIP option yet, and maybe never */
-						for (m = 0; m < nYSize; m++) {
+						for (m = row_i; m < row_e; m++) {
 							nn = (pad_w+m)*(nXSize_withPad) + startColPos;
 							for (n = nXSize-1; n >= 0; n--)
 								Ctrl->UInt8.data[nn++] = tmp[rowVec[m]+n];
 						}
 					}
 					else if (topdown) {			/* No BIP option yet, and maybe never */
-						for (m = 0; m < nYSize; m++) {
+						for (m = row_i; m < row_e; m++) {
 							for (n = 0; n < nXSize; n++) {
 								ij = colVec[n] + m;
 								Ctrl->UInt8.data[ij] = tmp[rowVec[m]+n];
@@ -1142,7 +1185,7 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 				break;
 			case GDT_Int16:
 				if (!prhs->f_ptr.active) Ctrl->Int16.active = true;
-				for (m = startRow, mm = 0; m < nYSize + startRow ; m++, mm++) {
+				for (m = startRow, mm = 0; m < endRow; m++, mm++) {
 					nn = (pad_w+m)*(nXSize_withPad) + startColPos;
 					for (n = fliplr ? nXSize-1 : 0; fliplr ? n >= 0 : n < nXSize; fliplr ? n-- : n++)
 						if (prhs->f_ptr.active) {
@@ -1156,7 +1199,7 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 				break;
 			case GDT_UInt16:
 				if (!prhs->f_ptr.active) Ctrl->UInt16.active = true;
-				for (m = startRow, mm = 0; m < nYSize + startRow ; m++, mm++) {
+				for (m = startRow, mm = 0; m < endRow; m++, mm++) {
 					nn = (pad_w+m)*(nXSize_withPad) + startColPos;
 					for (n = fliplr ? nXSize-1 : 0; fliplr ? n >= 0 : n < nXSize; fliplr ? n-- : n++)
 						if (prhs->f_ptr.active) {
@@ -1170,7 +1213,7 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 				break;
 			case GDT_Int32:
 				if (!prhs->f_ptr.active) Ctrl->Int32.active = true;
-				for (m = startRow, mm = 0; m < nYSize + startRow ; m++, mm++) {
+				for (m = startRow, mm = 0; m < endRow; m++, mm++) {
 					nn = (pad_w+m)*(nXSize_withPad) + startColPos;
 					for (n = fliplr ? nXSize-1 : 0; fliplr ? n >= 0 : n < nXSize; fliplr ? n-- : n++)
 						if (prhs->f_ptr.active) {
@@ -1184,7 +1227,7 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 				break;
 			case GDT_UInt32:
 				if (!prhs->f_ptr.active) Ctrl->UInt32.active = true;
-				for (m = startRow, mm = 0; m < nYSize + startRow ; m++, mm++) {
+				for (m = startRow, mm = 0; m < endRow; m++, mm++) {
 					nn = (pad_w+m)*(nXSize_withPad) + startColPos;
 					for (n = fliplr ? nXSize-1 : 0; fliplr ? n >= 0 : n < nXSize; fliplr ? n-- : n++)
 						if (prhs->f_ptr.active) {
@@ -1198,7 +1241,7 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 				break;
 			case GDT_Float32:
 				Ctrl->Float.active = true;
-				for (m = startRow, mm = 0; m < nYSize + startRow ; m++, mm++) {
+				for (m = startRow, mm = 0; m < endRow; m++, mm++) {
 					nn = (pad_w+m)*(nXSize_withPad) + startColPos;
 					for (n = 0; n < nXSize; n++) {
 						memcpy (&Ctrl->Float.data[nn], &tmp[(rowVec[mm]+n) * sizeof(float)], sizeof(float));
@@ -1208,7 +1251,7 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 				break;
 			case GDT_Float64:	/* For now we don't care about doubles */
 				Ctrl->Float.active = true;
-				for (m = startRow, mm = 0; m < nYSize + startRow ; m++, mm++) {
+				for (m = startRow, mm = 0; m < endRow; m++, mm++) {
 					nn = (pad_w+m)*(nXSize_withPad) + startColPos;
 					for (n = 0; n < nXSize; n++) {
 						double tmpF64;
@@ -1221,6 +1264,7 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 			default:
 				CPLAssert(false);
 		}
+}
 	}
 
 #if 0	/* This code is problematic and commented out for now. PW, 5/15/2016 */
@@ -1235,13 +1279,14 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 	}
 #endif
 	gmt_M_free (GMT, rowVec);
-	gmt_M_str_free (tmp);
+	if (!(just_copy || copy_flipud))
+		gmt_M_str_free (tmp);
 	gmt_M_free (GMT, whichBands);
 	gmt_M_free (GMT, colVec);
 
 	GDALClose(hDataset);
 
-	populate_metadata (GMT, Ctrl, gdal_filename, got_R, nXSize, nYSize, dfULX, dfULY, dfLRX, dfLRY, z_min, z_max);
+	populate_metadata (GMT, Ctrl, gdal_filename, got_R, nXSize, nYSize, dfULX, dfULY, dfLRX, dfLRY, z_min, z_max, first_layer);
 
 	GDALDestroyDriverManager();
 
