@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt_gdalread.h 19521 2017-12-27 22:38:24Z pwessel $
  *
- *	Copyright (c) 1991-2018 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 
 /*!
@@ -21,8 +20,8 @@
  * \brief Define structures to interface with gdalread|write
  */
 
-#ifndef _GMT_GDALREAD_H
-#define _GMT_GDALREAD_H
+#ifndef GMT_GDALREAD_H
+#define GMT_GDALREAD_H
 
 #include <gdal.h>
 #include <ogr_srs_api.h>
@@ -32,6 +31,7 @@
 /*! Structure to control which options are transmitted to gmt_gdalwrite */
 struct GMT_GDALWRITE_CTRL {
 	char   *driver;            /* The GDAL diver name */
+	char   *co_options;        /* The GDAL -co options */
 	char   *type;              /* Data type */
 	char   *command;           /* command line */
 	char   *title;
@@ -39,6 +39,7 @@ struct GMT_GDALWRITE_CTRL {
 	unsigned char   *alpha;    /* In case this is used to transmit an image that has a transparency layer */
 	void   *data;              /* To store the grid/image array */
 	char    layout[4];         /* A 3 letter code specifying the image memory layout plus a A|a if alpha data in array */
+	int     orig_type;         /* Original grid data type. Typed as in to match the orig_datatype in GMT_GRID_HEADER */
 	int     geog;
 	int     n_columns, n_rows; /* Number of columns & rows of the region to be saved */
 	int     nXSizeFull;        /* Total number of columns of the data array including padding */
@@ -50,31 +51,32 @@ struct GMT_GDALWRITE_CTRL {
 	double	x_inc, y_inc;      /* Grid/Image increments */
 	double	nan_value;         /* unlike the nan_value in struct GMT_GRID_HEADER this one is of type double */
 	struct  GW_C {             /* Color map */
-		int    active;
+		bool   active;
 		int    n_colors;
 		float *cpt;
 	} C;
 	struct  GW_P {             /* Proj4/WKT string */
-		int    active;
-		char  *ProjRefPROJ4;
-		char  *ProjRefWKT;
+		bool  active;
+		char *ProjRefPROJ4;             /* To store a referencing system string in PROJ.4 format */
+		char *ProjRefWKT;               /* To store a referencing system string in WKT format */
+		int   ProjRefEPSG;              /* To store a referencing system EPSG code */
 	} P;
 };
 
 /*! Structure to control which options are transmitted to gmt_gdalread */
 struct GMT_GDALREAD_IN_CTRL {
 	struct GD_B {	/* Band selection */
-		int active;
+		bool active;
 		char *bands;
 	} B;
 	struct GD_I {	/* Interleaving by pixel (only for char data) */
-		int active;
+		bool active;
 	} I;
 	struct GD_L {	/* Left-Right flip */
-		int active;
+		bool active;
 	} L;
 	struct GD_M {	/* Metadata only */
-		int active;
+		bool active;
 	} M;
 	struct GD_O {	/* Three chars code to specify the array layout in memory */
 		/* first char T(op)|B(ot), second R(ow)|C(ol), third P(ix)|L(ine)|S(equencial) */
@@ -84,34 +86,34 @@ struct GMT_GDALREAD_IN_CTRL {
 		float nan_value;
 	} N;
 	struct GD_P {	/* Preview mode */
-		int active;
+		bool active;
 		char *jump;
 	} P;
 	struct GD_p {	/* Pad array in output */
-		int active;
+		bool active;
 		int pad;
 	} p;
 	struct GD_W {	/* Convert proj4 string into WKT */
-		int active;
+		bool active;
 	} W;
 	struct GD_R {	/* Sub-region in referenced coords */
-		int active;
+		bool active;
 		char *region;
 	} R;
 	struct GD_Z {	/* Tell to store data in a complex array */
-		int active;
+		bool active;
 		int complex_mode; /* 1|2 if complex array is to hold real (1) and imaginary (2) parts (0 = read as real only) */
 	} Z;
 	struct GD_cp {	/* Send in a pointer with allocated chars */
-		int active;
+		bool active;
 		unsigned char *grd;
 	} c_ptr;
 	struct GD_fp {	/* Send in a pointer with allocated floats */
-		int active;
-		float *grd;
+		bool active;
+		gmt_grdfloat *grd;
 	} f_ptr;
 	struct GD_r {	/* Sub-region in row/column coords */
-		int active;
+		bool active;
 		char *region;
 	} r;
 	struct GD_reg {	/* Registration type. Used only when sending a sub-region request. Than we need to know this */
@@ -119,11 +121,13 @@ struct GMT_GDALREAD_IN_CTRL {
 		int val;	/* 0 [default] means grid registration, 1 -> pixel registration */
 	} registration;
 	struct GD_hdr {	/* Some fields of the header structure */
-		int active;
+		bool active;
 		unsigned int mx, my;
 		char side[1];		/* If array is going to pasted (grdpaste), tell in what side 'lrtb' */
 		int offset;
 	} mini_hdr;
+	OGRCoordinateTransformationH hCT_fwd;		// TEMP TEMP TEMP. Only to quick try access GDAL coordinates transforms
+	OGRCoordinateTransformationH hCT_inv;		// TEMP TEMP TEMP. Only to quick try access GDAL coordinates transforms
 };
 
 /*! Structure to hold metadata info in a per bands basis read */
@@ -140,31 +144,35 @@ struct GDAL_BAND_FNAMES {
 struct GMT_GDALREAD_OUT_CTRL {
 	/* active is true if the option has been activated */
 	struct UInt8 {			/* Declare byte pointer */
-		int active;
+		bool active;
 		unsigned char *data;
 	} UInt8;
 	struct UInt16 {			/* Declare short int pointers */
-		int active;
+		bool active;
 		unsigned short int *data;
 	} UInt16;
 	struct Int16 {			/* Declare unsigned short int pointers */
-		int active;
+		bool active;
 		short int *data;
 	} Int16;
 	struct UInt32 {			/* Declare unsigned int pointers */
-		int active;
+		bool active;
 		int *data;
 	} UInt32;
 	struct Int32 {			/* Declare int pointers */
-		int active;
+		bool active;
 		int *data;
 	} Int32;
 	struct Float {			/* Declare float pointers */
-		int active;
+		bool active;
+#ifdef DOUBLE_PRECISION_GRID
+		double *data;
+#else
 		float *data;
+#endif
 	} Float;
 	struct Double {			/* Declare double pointers */
-		int active;
+		bool active;
 		double *data;
 	} Double;
 
@@ -192,4 +200,21 @@ struct GMT_GDALREAD_OUT_CTRL {
 	struct GDAL_BAND_FNAMES *band_field_names;
 };
 
-#endif  /* _GMT_GDALREAD_H */
+struct OGR_FEATURES {
+	int     n_rows, n_cols, n_layers;	/* n_rows, n_column, n_layers of the struct array */
+	int     n_filled;   /* Number of actually filled elements in the matrix */
+	int     is3D;       /* True when geometries have a z component */
+	unsigned int np;    /* Number of data points in this feature */
+	int     att_number; /* Feature's number of attributes */
+	char   *name, *wkt, *proj4; 
+	char   *type;	    /* Geometry type. E.g. Point, Polygon or LineString */
+	char  **att_names;	/* Names of the attributes of a Feature */
+	char  **att_values;	/* Values of the attributes of a Feature as strings */
+	int    *att_types;
+	int    *islands;
+	double  BoundingBox[6];
+	double *BBgeom;     /* Not currently assigned (would be the BoundingBox of each individual geometry) */
+	double *x, *y, *z;
+};
+
+#endif  /* GMT_GDALREAD_H */
