@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *	Copyright (c) 1991-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *      This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu *
+ *	Contact info: www.generic-mapping-tools.org *
  *--------------------------------------------------------------------*/
 
 /* Program:	GMT_gdawrite.c
@@ -56,8 +55,9 @@ int gmt_export_image (struct GMT_CTRL *GMT, char *fname, struct GMT_IMAGE *I) {
 	uint32_t row, col, band;
 	uint64_t k, ijk, b;
 	bool     free_data = false;
-	char    *ext = NULL, *c = NULL;
+	char    *ext = NULL, *c = NULL, *pch;
 	unsigned char *data = NULL;
+	struct GMT_GRID_HEADER_HIDDEN *HH = NULL;
 	struct GMT_GDALWRITE_CTRL *to_GDALW = NULL;
 
 	/* NOTE: in grdimage we have done this so may have to deal with this here.
@@ -103,16 +103,22 @@ int gmt_export_image (struct GMT_CTRL *GMT, char *fname, struct GMT_IMAGE *I) {
 	for (k = 0; to_GDALW->driver == NULL && k < N_GDAL_EXTENSIONS; k++) {
 		if (k == 0 && (!strcasecmp(ext, "tif") || !strcasecmp(ext, "tiff")))	/* Tiffs happen to have a different extension<->driver naming */
 			to_GDALW->driver = strdup(gdal_drv[k]);
+		else if (!strcasecmp(ext, "jpg"))
+			to_GDALW->driver = strdup("JPEG");	/* Jpegs too */
 		else if (!strcasecmp (ext, gdal_drv[k]))
 			to_GDALW->driver = strdup(gdal_drv[k]);
 	}
-	if (to_GDALW->driver == NULL) {	/* None of those; need to give a driver */
-		if ((c = strchr(fname, ':'))) {	/* Found an ':<driver>' part */
+	if (to_GDALW->driver == NULL) {				/* None of those; need to give a driver */
+		if ((c = strchr(fname, ':'))) {			/* Found an ':<driver>' part */
 			to_GDALW->driver = strdup(&c[1]);
-			c[0] = '\0';			/* Remove the driver code from the name */
+			c[0] = '\0';						/* Remove the driver code from the name */
 			if ((c = strstr(fname, "=gd")))		/* Check if we have not a stray =gd in name*/
 				c[0] = '\0';					/* Yes, remove it too */
 		}
+		else if ((c = strchr (fname, '='))) {    /* Gave file=<format> so pass that along */
+			to_GDALW->driver = strdup(&c[1]);
+			c[0] = '\0';
+		}
 		else {
 			GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Unupported image format. Supported formats are:\nBMP,GIF,JPG,PNG & TIF\n");
 			GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Alternatively, append :<driver> for a valid GDAL driver\n");
@@ -120,7 +126,7 @@ int gmt_export_image (struct GMT_CTRL *GMT, char *fname, struct GMT_IMAGE *I) {
 		}
 	}
 
-	if (!strncmp(I->header->mem_layout, "TCB", 3)) {
+	if (!strncmp (I->header->mem_layout, "TCB", 3)) {
 		to_GDALW->type = strdup("uint8");
 		data = gmt_M_memory (GMT, NULL, I->header->nm * I->header->n_bands, char);
 
@@ -141,7 +147,7 @@ int gmt_export_image (struct GMT_CTRL *GMT, char *fname, struct GMT_IMAGE *I) {
 		}
 		free_data = true;
 	}
-	else if (!strncmp(I->header->mem_layout, "TRP", 3)) {
+	else if (!strncmp (I->header->mem_layout, "TRP", 3)) {
 		to_GDALW->type = strdup("byte");
 		data = I->data;
 		if (to_GDALW->P.ProjRefPROJ4) {
@@ -153,12 +159,16 @@ int gmt_export_image (struct GMT_CTRL *GMT, char *fname, struct GMT_IMAGE *I) {
 		to_GDALW->alpha = I->alpha;
 	}
 	else {
-		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Error: %s memory layout is not supported, for now only: T(op)C(ol)B(and) or TRP\n",
+		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "%s memory layout is not supported, for now only: T(op)C(ol)B(and) or TRP\n",
 		            I->header->mem_layout);
 		gmt_M_free (GMT, to_GDALW);
 		return GMT_NOTSET;
 	}
 
+	HH = gmt_get_H_hidden (I->header);
+	if (HH->pocket && (pch = strstr(HH->pocket, "+c")) != NULL) 		/* If we have a list of +c<options> */
+		to_GDALW->co_options = strdup(pch);				/* This memory is freed in gmt_gdalwrite */
+
 	strncpy(to_GDALW->layout, I->header->mem_layout, 4);
 	to_GDALW->data = data;
 	gmt_gdalwrite (GMT, fname, to_GDALW);
@@ -171,6 +181,69 @@ int gmt_export_image (struct GMT_CTRL *GMT, char *fname, struct GMT_IMAGE *I) {
 	return GMT_NOERROR;
 }
 
+GMT_LOCAL int write_jp2 (struct GMT_CTRL *GMT, struct GMT_GDALWRITE_CTRL *prhs, GDALRasterBandH hBand, void *data, int n_rows, int n_cols) {
+	int error = 0, i, j;
+	float *t = (float *)data;
+	uint64_t k, n, nm = (uint64_t)n_rows * n_cols;
+	/* In gmt_gdal_write_grd we made the pointer to point to the beginning of the non-padded zone, so to make it
+	   coherent we retriet pad[0]. However, nothing of this is taking into account a -R subregion so all of this
+	   (and not only this case) will probably fail for that case.
+	*/
+	t -= prhs->pad[0];
+	if (prhs->orig_type == GMT_UCHAR) {
+		char *dataT = gmt_M_memory(GMT, NULL, nm, char);
+		for (i = 0, k = 0; i < n_rows; i++) {
+			n = (uint64_t)i*prhs->nXSizeFull + prhs->pad[0];
+			for (j = 0; j < n_cols; j++)
+				dataT[k++] = (char)t[n + j];
+		}
+		error = GDALRasterIO(hBand, GF_Write, 0, 0, n_cols, n_rows, dataT, n_cols, n_rows, GDT_Byte, 0, 0);
+		gmt_M_free (GMT, dataT);
+	}
+	else if (prhs->orig_type == GMT_USHORT) {
+		short int *dataT = gmt_M_memory(GMT, NULL, nm, unsigned short int);
+		for (i = 0, k = 0; i < n_rows; i++) {
+			n = (uint64_t)i*prhs->nXSizeFull + prhs->pad[0];
+			for (j = 0; j < n_cols; j++)
+				dataT[k++] = (unsigned short int)t[n + j];
+		}
+		error = GDALRasterIO(hBand, GF_Write, 0, 0, n_cols, n_rows, dataT, n_cols, n_rows, GDT_UInt16, 0, 0);
+		gmt_M_free (GMT, dataT);
+	}
+	else if (prhs->orig_type == GMT_SHORT) {
+		short int *dataT = gmt_M_memory(GMT, NULL, nm, short int);
+		for (i = 0, k = 0; i < n_rows; i++) {
+			n = (uint64_t)i*prhs->nXSizeFull + prhs->pad[0];
+			for (j = 0; j < n_cols; j++)
+				dataT[k++] = (short int)t[n + j];
+		}
+		error = GDALRasterIO(hBand, GF_Write, 0, 0, n_cols, n_rows, dataT, n_cols, n_rows, GDT_Int16, 0, 0);
+		gmt_M_free (GMT, dataT);
+	}
+	else if (prhs->orig_type == GMT_UINT) {
+		unsigned int *dataT = gmt_M_memory(GMT, NULL, nm, unsigned int);
+		for (i = 0, k = 0; i < n_rows; i++) {
+			n = (uint64_t)i*prhs->nXSizeFull + prhs->pad[0];
+			for (j = 0; j < n_cols; j++)
+				dataT[k++] = (unsigned int)t[n + j];
+		}
+		error = GDALRasterIO(hBand, GF_Write, 0, 0, n_cols, n_rows, dataT, n_cols, n_rows, GDT_UInt32, 0, 0);
+		gmt_M_free (GMT, dataT);
+	}
+	else if (prhs->orig_type == GMT_INT) {
+		int *dataT = gmt_M_memory(GMT, NULL, nm, int);
+		for (i = 0, k = 0; i < n_rows; i++) {
+			n = (uint64_t)i*prhs->nXSizeFull + prhs->pad[0];
+			for (j = 0; j < n_cols; j++)
+				dataT[k++] = (int)t[n + j];
+		}
+		error = GDALRasterIO(hBand, GF_Write, 0, 0, n_cols, n_rows, dataT, n_cols, n_rows, GDT_Int32, 0, 0);
+		gmt_M_free (GMT, dataT);
+	}
+
+	return error;
+}
+
 int gmt_gdalwrite (struct GMT_CTRL *GMT, char *fname, struct GMT_GDALWRITE_CTRL *prhs) {
 	int	bStrict = false;
 	char **papszOptions = NULL, *projWKT = NULL;
@@ -186,7 +259,7 @@ int gmt_gdalwrite (struct GMT_CTRL *GMT, char *fname, struct GMT_GDALWRITE_CTRL
 	GDALProgressFunc pfnProgress = GDALTermProgress;
 
 	int  n_cols, n_rows, i;
-	int  typeCLASS, nColors, n_byteOffset, n_bands, registration;
+	int  typeCLASS, typeCLASS_f, nColors, n_byteOffset, n_bands, registration;
 	int  is_geog = 0, gdal_err = 0;
 	uint64_t nn, ijk = 0;
 	void *data;
@@ -243,6 +316,22 @@ int gmt_gdalwrite (struct GMT_CTRL *GMT, char *fname, struct GMT_GDALWRITE_CTRL
 		return -1;
 	}
 
+	/* Jpeg2000 driver doesn't accept float arrays so we'll have to copy it into a int16 */
+	if (!strcasecmp(pszFormat,"JP2OpenJPEG")) {
+		if (prhs->orig_type == GMT_UCHAR)       typeCLASS_f = GDT_Byte;
+		else if (prhs->orig_type == GMT_USHORT) typeCLASS_f = GDT_UInt16;
+		else if (prhs->orig_type == GMT_SHORT)  typeCLASS_f = GDT_Int16;
+		else if (prhs->orig_type == GMT_UINT)   typeCLASS_f = GDT_Int32;
+		else if (prhs->orig_type == GMT_INT)    typeCLASS_f = GDT_UInt32;
+		else {
+			GMT_Report (GMT->parent, GMT_MSG_NORMAL, "gmt_gdalwrite: The Jpeg2000 driver does not support floats.\n");
+			gmt_M_free(GMT, outByte);
+			return -1;
+		}
+	}
+	else
+		typeCLASS_f = typeCLASS;
+
 	if (prhs->C.active) {
 		nColors = prhs->C.n_colors;
 		ptr = prhs->C.cpt;
@@ -277,7 +366,7 @@ int gmt_gdalwrite (struct GMT_CTRL *GMT, char *fname, struct GMT_GDALWRITE_CTRL
 			projWKT = pszPrettyWkt;
 		}
 		else {
-			GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Warning: gmt_gdalwrite failed to convert the proj4 string\n%s\n to WKT\n",
+			GMT_Report (GMT->parent, GMT_MSG_NORMAL, "gmt_gdalwrite failed to convert the proj4 string\n%s\n to WKT\n",
 					prhs->P.ProjRefPROJ4);
 		}
 
@@ -293,14 +382,15 @@ int gmt_gdalwrite (struct GMT_CTRL *GMT, char *fname, struct GMT_GDALWRITE_CTRL
 
 	if (hDriverOut == NULL) {
 		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "gmt_gdalwrite: Output driver %s not recognized\n", pszFormat);
-		/* The following is s bit idiot. The loop should only be executed is verbose so requires */
-		GMT_Report (GMT->parent, GMT_MSG_VERBOSE, "The following format drivers are configured and support output:\n");
-		for (i = 0; i < GDALGetDriverCount(); i++) {
-			hDriver = GDALGetDriver(i);
-			if (GDALGetMetadataItem(hDriver, GDAL_DCAP_CREATE, NULL) != NULL ||
-			    GDALGetMetadataItem(hDriver, GDAL_DCAP_CREATECOPY, NULL) != NULL)
-				GMT_Report (GMT->parent, GMT_MSG_VERBOSE, "  %s: %s\n",
-				            GDALGetDriverShortName(hDriver), GDALGetDriverLongName(hDriver));
+		if (gmt_M_is_verbose (GMT, GMT_MSG_VERBOSE)) {
+			GMT_Report (GMT->parent, GMT_MSG_VERBOSE, "The following format drivers are configured and support output:\n");
+			for (i = 0; i < GDALGetDriverCount(); i++) {
+				hDriver = GDALGetDriver(i);
+				if (GDALGetMetadataItem(hDriver, GDAL_DCAP_CREATE, NULL) != NULL ||
+				    GDALGetMetadataItem(hDriver, GDAL_DCAP_CREATECOPY, NULL) != NULL)
+					GMT_Report (GMT->parent, GMT_MSG_VERBOSE, "  %s: %s\n",
+					            GDALGetDriverShortName(hDriver), GDALGetDriverLongName(hDriver));
+			}
 		}
 		GDALDestroyDriverManager();
 		gmt_M_free(GMT, outByte);
@@ -310,7 +400,8 @@ int gmt_gdalwrite (struct GMT_CTRL *GMT, char *fname, struct GMT_GDALWRITE_CTRL
 	if (prhs->alpha)		/* If transparency, number of requested bands must increment by one */
 		hDstDS = GDALCreate(hDriver, "mem", n_cols, n_rows, n_bands+1, typeCLASS, NULL);
 	else
-		hDstDS = GDALCreate(hDriver, "mem", n_cols, n_rows, n_bands, typeCLASS, NULL);
+		hDstDS = GDALCreate(hDriver, "mem", n_cols, n_rows, n_bands, typeCLASS_f, NULL);
+		
 
 	if (hDstDS == NULL) {
 		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "GDALOpen failed - %d\n%s\n", CPLGetLastErrorNo(), CPLGetLastErrorMsg());
@@ -320,8 +411,17 @@ int gmt_gdalwrite (struct GMT_CTRL *GMT, char *fname, struct GMT_GDALWRITE_CTRL
 	}
 	GDALSetGeoTransform(hDstDS, adfGeoTransform);
 
-	/* Use compression with GeoTiff driver */
-	if (!strcasecmp(pszFormat,"GTiff")) {
+	if (prhs->co_options) {
+		unsigned int  pos = 0;
+		char token[64];
+		while (gmt_strtok (prhs->co_options, "+", &pos, token)) {
+			if (token[0] == 'c')
+				papszOptions = CSLAddString(papszOptions, &token[1]);	/* Jump the 'c' */
+		}
+	}
+
+	/* Use default compression with GeoTiff driver, unless co_options were passed. Than the above should have taken care of it. */
+	if (!strcasecmp(pszFormat,"GTiff") && !prhs->co_options) {
 		papszOptions = CSLAddString(papszOptions, "COMPRESS=DEFLATE");
 		/* tiles are less efficient in small grids (padding) and are not
 		 * supported everywhere, when n_cols < tile_width || n_rows < tile_height */
@@ -334,6 +434,8 @@ int gmt_gdalwrite (struct GMT_CTRL *GMT, char *fname, struct GMT_GDALWRITE_CTRL
 		else
 			GDALSetMetadataItem(hDstDS, "AREA_OR_POINT", "Area", NULL);
 	}
+	if (prhs->co_options)
+		free (prhs->co_options);		/* Was allocated with an strdup() in gmt_gdal_write_grd() */
 
 	/* This was the only trick I found to set a "projection". */
 	if (is_geog || projWKT || !strcasecmp(pszFormat,"GTiff")) {
@@ -404,9 +506,15 @@ int gmt_gdalwrite (struct GMT_CTRL *GMT, char *fname, struct GMT_GDALWRITE_CTRL
 				break;
 			case GDT_Float32:
 				GDALSetRasterNoDataValue(hBand, prhs->nan_value);
-				if ((gdal_err = GDALRasterIO(hBand, GF_Write, 0, 0, n_cols, n_rows, data, n_cols, n_rows, typeCLASS, 0,
-				                 prhs->nXSizeFull * n_byteOffset)) != CE_None)
-					GMT_Report (GMT->parent, GMT_MSG_NORMAL, "GDALRasterIO failed to write band %d [err = %d]\n", i, gdal_err);
+				if (!strcasecmp(pszFormat,"JP2OpenJPEG")) {			/* JP2 driver doesn't accept floats, so we must make a copy */
+					if ((gdal_err = write_jp2 (GMT, prhs, hBand, data, n_rows, n_cols)) != CE_None)
+						GMT_Report (GMT->parent, GMT_MSG_NORMAL, "GDALRasterIO failed to write band %d [err = %d]\n", i, gdal_err);
+				}
+				else {
+					if ((gdal_err = GDALRasterIO(hBand, GF_Write, 0, 0, n_cols, n_rows, data, n_cols, n_rows, typeCLASS, 0,
+					                             prhs->nXSizeFull * n_byteOffset)) != CE_None)
+						GMT_Report (GMT->parent, GMT_MSG_NORMAL, "GDALRasterIO failed to write band %d [err = %d]\n", i, gdal_err);
+				}
 				break;
 		}
 
