@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *	Copyright (c) 1991-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 
 /*!
@@ -23,14 +22,14 @@
 
 /* Authors:	J. Luis
    Date:	1-OCT-2012
-   Version:	5 API
+   Version:	6 API
 
 */
 
 /* Include glib header and define mutex calls that are no-op when not linking against glib
    These are used only GLIB based multi-threading */
 
-#ifndef _GMT_GLIB_H
+#ifndef GMT_GLIB_H
 
 #ifdef HAVE_GLIB_GTHREAD
 #include <glib.h>
