@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *	Copyright (c) 1991-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  *  Management of internal temporary memory.
@@ -59,18 +58,27 @@
  *	     simpler and the number of columns is typically very small (2-3).
  */
 
-GMT_LOCAL void memory_init_tmp_arrays (struct GMT_CTRL *GMT, size_t n_cols) {
+GMT_LOCAL void memory_init_tmp_arrays (struct GMT_CTRL *GMT, int direction, size_t n_cols) {
 	/* Initialization of GMT coordinate temp arrays - this is called at most once per GMT session  */
 
-	size_t col;
-
-	if (n_cols == 0) n_cols = GMT_INITIAL_MEM_COL_ALLOC;	/* Allocate at least this many */
-	GMT->hidden.mem_coord  = gmt_M_memory (GMT, GMT->hidden.mem_coord, n_cols, double *);	/* These are all NULL */
-	GMT->hidden.mem_cols = n_cols;	/* How many columns we have initialized */
-	for (col = 0; col < n_cols; col++)	/* For each column, reallocate space for n_rows */
-		GMT->hidden.mem_coord[col] = gmt_M_memory (GMT, NULL, GMT_INITIAL_MEM_ROW_ALLOC, double);
-	GMT->hidden.mem_rows = GMT_INITIAL_MEM_ROW_ALLOC;
-	GMT_Report (GMT->parent, GMT_MSG_DEBUG, "GMT memory: Initialize %" PRIuS " temporary column arrays, each of length : %" PRIuS "\n", GMT->hidden.mem_cols, GMT->hidden.mem_rows);
+	if (!GMT->hidden.mem_set) {
+		if (n_cols == 0 && (direction == GMT_NOTSET || (GMT->current.io.record_type[direction] & GMT_READ_DATA))) n_cols = GMT_INITIAL_MEM_COL_ALLOC;	/* Allocate at least this many */
+	}
+	if (n_cols) {	/* Records have numerical content */
+		size_t col;
+		GMT->hidden.mem_coord  = gmt_M_memory (GMT, GMT->hidden.mem_coord, n_cols, double *);	/* These are all NULL */
+		GMT->hidden.mem_cols = n_cols;	/* How many columns we have initialized */
+		for (col = 0; col < n_cols; col++)	/* For each column, reallocate space for n_rows */
+			GMT->hidden.mem_coord[col] = gmt_M_memory (GMT, NULL, GMT_INITIAL_MEM_ROW_ALLOC, double);
+		GMT_Report (GMT->parent, GMT_MSG_DEBUG, "GMT memory: Initialize %" PRIuS " temporary column double arrays, each of length : %" PRIuS "\n", GMT->hidden.mem_cols, GMT->hidden.mem_rows);
+		GMT->hidden.mem_rows = GMT_INITIAL_MEM_ROW_ALLOC;
+	}
+	if (direction != GMT_NOTSET && GMT->current.io.record_type[direction] & GMT_READ_TEXT) {	/* For text or mixed records */
+		GMT->hidden.mem_txt = gmt_M_memory (GMT, NULL, GMT_INITIAL_MEM_ROW_ALLOC, char *);
+		GMT->hidden.mem_rows = GMT_INITIAL_MEM_ROW_ALLOC;
+		GMT_Report (GMT->parent, GMT_MSG_DEBUG, "GMT memory: Initialize a temporary column char * array of length : %" PRIuS "\n", GMT->hidden.mem_rows);
+	}
+	GMT->hidden.mem_set = true;
 }
 
 GMT_LOCAL int memory_die_if_memfail (struct GMT_CTRL *GMT, size_t nelem, size_t size, const char *where) {
@@ -79,7 +87,7 @@ GMT_LOCAL int memory_die_if_memfail (struct GMT_CTRL *GMT, size_t nelem, size_t
 	unsigned int k = 0;
 	static char *m_unit[4] = {"bytes", "kb", "Mb", "Gb"};
 	while (mem >= 1024.0 && k < 3) mem /= 1024.0, k++;
-	gmtlib_report_func (GMT, GMT_MSG_NORMAL, where, "Error: Could not reallocate memory [%.2f %s, %" PRIuS " items of %" PRIuS " bytes]\n", mem, m_unit[k], nelem, size);
+	gmtlib_report_func (GMT, GMT_MSG_NORMAL, where, "Could not reallocate memory [%.2f %s, %" PRIuS " items of %" PRIuS " bytes]\n", mem, m_unit[k], nelem, size);
 #ifdef DEBUG
 	gmtlib_report_func (GMT, GMT_MSG_NORMAL, where, "gmt_M_memory [realloc] called\n");
 #endif
@@ -135,8 +143,8 @@ int gmt_memtrack_init (struct GMT_CTRL *GMT) {
 	else
 	{
 		int pid = getpid();
-		char logfile[32];
-		snprintf (logfile, 32, "gmt_memtrack_%d.log", pid);
+		char logfile[GMT_LEN32];
+		snprintf (logfile, GMT_LEN32, "gmt_memtrack_%d.log", pid);
 		if ((M->fp = fopen (logfile, "w")) == NULL) {
 			GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Could not create log file gmt_memtrack_%d.log\n", pid);
 			GMT_exit (GMT, GMT_ERROR_ON_FOPEN); return GMT_ERROR_ON_FOPEN;
@@ -383,8 +391,8 @@ void gmt_memtrack_report (struct GMT_CTRL *GMT) {
 		excess = M->n_allocated - M->n_freed;
 	else if (M->n_freed > M->n_allocated)
 		n_multi_frees = M->n_freed - M->n_allocated;
-	/* Only insist on report if a leak or multi free, otherwise requires -V: */
-	level = (excess || n_multi_frees) ? GMT_MSG_NORMAL : GMT_MSG_VERBOSE;
+	/* Only insist on report if a leak or multi free, otherwise requires -Vd: */
+	level = (excess || n_multi_frees) ? GMT_MSG_NORMAL : GMT_MSG_DEBUG;
 	size = gmt_memtrack_mem (M->maximum, &u);
 	GMT_Report (GMT->parent, level, "Max total memory allocated was %.3f %s [%" PRIuS " bytes]\n",
 							size, unit[u], M->maximum);
@@ -439,15 +447,18 @@ void gmtlib_free_tmp_arrays (struct GMT_CTRL *GMT) {
 		gmt_M_free (GMT, GMT->hidden.mem_coord[col]);
 	}
 	gmt_M_free (GMT, GMT->hidden.mem_coord);
+	if (GMT->hidden.mem_txt)
+		gmt_M_free (GMT, GMT->hidden.mem_txt);
 	GMT->hidden.mem_rows = GMT->hidden.mem_cols = 0;
+	GMT->hidden.mem_set = false;	/* Back to where we started */
 }
 
-void gmt_prep_tmp_arrays (struct GMT_CTRL *GMT, size_t row, size_t n_cols) {
+void gmt_prep_tmp_arrays (struct GMT_CTRL *GMT, int direction, size_t row, size_t n_cols) {
 	size_t col;
 
 	/* Check if this is the very first time, if so we initialize the arrays */
-	if (GMT->hidden.mem_cols == 0)
-		memory_init_tmp_arrays (GMT, n_cols);	/* First time we get here */
+	if (!GMT->hidden.mem_set)
+		memory_init_tmp_arrays (GMT, direction, n_cols);	/* First time we get here */
 
 	/* Check if we are exceeding our column count so far, if so we must allocate more columns */
 	else if (n_cols > GMT->hidden.mem_cols) {	/* Must allocate more columns, this is expected to happen rarely */
@@ -466,6 +477,8 @@ void gmt_prep_tmp_arrays (struct GMT_CTRL *GMT, size_t row, size_t n_cols) {
 	while (row >= GMT->hidden.mem_rows) GMT->hidden.mem_rows = (size_t)lrint (1.5 * GMT->hidden.mem_rows);	/* Increase by 50% */
 	for (col = 0; col < GMT->hidden.mem_cols; col++)	/* Add more memory via realloc */
 		GMT->hidden.mem_coord[col] = gmt_M_memory (GMT, GMT->hidden.mem_coord[col], GMT->hidden.mem_rows, double);
+	if (direction != GMT_NOTSET && GMT->current.io.record_type[direction] & GMT_READ_TEXT)
+		GMT->hidden.mem_txt = gmt_M_memory (GMT, GMT->hidden.mem_txt, GMT->hidden.mem_rows, char *);
 
 	GMT_Report (GMT->parent, GMT_MSG_DEBUG, "GMT memory: Increase %" PRIuS " temporary column arrays to new length : %" PRIuS "\n", GMT->hidden.mem_cols, GMT->hidden.mem_rows);
 	/* Note: Any additions to these arrays are not guaranteed to be set to zero */
@@ -483,7 +496,7 @@ void *gmt_memory_func (struct GMT_CTRL *GMT, void *prev_addr, size_t nelem, size
 	void *tmp = NULL;
 
 	if (nelem == SIZE_MAX) {	/* Probably 32-bit overflow */
-		gmtlib_report_func (GMT, GMT_MSG_NORMAL, where, "Error: Requesting SIZE_MAX number of items (%" PRIuS ") - exceeding 32-bit counting?\n", nelem);
+		gmtlib_report_func (GMT, GMT_MSG_NORMAL, where, "Requesting SIZE_MAX number of items (%" PRIuS ") - exceeding 32-bit counting?\n", nelem);
 #ifdef DEBUG
 		gmtlib_report_func (GMT, GMT_MSG_NORMAL, where, "gmt_M_memory called\n");
 #endif
@@ -558,7 +571,7 @@ void gmt_free_func (struct GMT_CTRL *GMT, void *addr, bool align, const char *wh
 	}
 
 #ifdef MEMDEBUG
-	if (GMT->hidden.mem_keeper->active) {
+	if (GMT->hidden.mem_keeper && GMT->hidden.mem_keeper->active) {
 		bool is_safe_to_free = gmt_memtrack_sub (GMT, where, addr);
 		if (is_safe_to_free == false)
 			return; /* Address addr was not allocated by gmt_memory_func before */
