@@ -1,5 +1,4 @@
-/*	$Id: gmt_mgg_header2.c 17328 2016-11-08 20:38:56Z pwessel $
- *
+/*
  *	Code donated by David Divens, NOAA/NGDC
  *	Distributed under the GNU Public License (see LICENSE.TXT for details)
  *--------------------------------------------------------------------*/
@@ -149,6 +148,7 @@ GMT_LOCAL int grd98_GMTtoMGG2 (struct GMT_GRID_HEADER *gmt, MGG_GRID_HEADER_2 *m
 
 GMT_LOCAL void grd98_MGG2toGMT (MGG_GRID_HEADER_2 *mgg, struct GMT_GRID_HEADER *gmt) {
 	int one_or_zero;
+	struct GMT_GRID_HEADER_HIDDEN *HH = gmt_get_H_hidden (gmt);
 
 	/* Do not memset the gmt header since it has the file name set */
 
@@ -169,6 +169,13 @@ GMT_LOCAL void grd98_MGG2toGMT (MGG_GRID_HEADER_2 *mgg, struct GMT_GRID_HEADER *
 	gmt->z_max = (double)mgg->maxValue / (double)mgg->precision;
 	gmt->z_scale_factor = 1.0;
 	gmt->z_add_offset = 0.0;
+	switch (mgg->numType) {
+		case  1:	HH->orig_datatype = GMT_CHAR;  break;
+		case  2:	HH->orig_datatype = GMT_SHORT; break;
+		case  4:	HH->orig_datatype = GMT_INT;   break;
+		case -4:	HH->orig_datatype = GMT_FLOAT; break;
+		default:	HH->orig_datatype = GMT_INT;   break;
+	}
 }
 
 /*----------------------------------------------------------|
@@ -181,10 +188,11 @@ int gmt_is_mgg2_grid (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header) {
 	FILE *fp = NULL;
 	MGG_GRID_HEADER_2 mggHeader;
 	int ok;
+	struct GMT_GRID_HEADER_HIDDEN *HH = gmt_get_H_hidden (header);
 
-	if (!strcmp (header->name, "="))
+	if (!strcmp (HH->name, "="))
 		return (GMT_GRDIO_PIPE_CODECHECK);	/* Cannot check on pipes */
-	if ((fp = gmt_fopen (GMT, header->name, GMT->current.io.r_mode)) == NULL)
+	if ((fp = gmt_fopen (GMT, HH->name, GMT->current.io.r_mode)) == NULL)
 		return (GMT_GRDIO_OPEN_FAILED);
 
 	gmt_M_memset (&mggHeader, 1, MGG_GRID_HEADER_2);
@@ -210,10 +218,11 @@ int gmt_mgg2_read_grd_info (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header
 	FILE *fp = NULL;
 	MGG_GRID_HEADER_2 mggHeader;
 	int ok;
+	struct GMT_GRID_HEADER_HIDDEN *HH = gmt_get_H_hidden (header);
 
-	if (!strcmp (header->name, "="))
+	if (!strcmp (HH->name, "="))
 		fp = GMT->session.std[GMT_IN];
-	else if ((fp = gmt_fopen (GMT, header->name, GMT->current.io.r_mode)) == NULL)
+	else if ((fp = gmt_fopen (GMT, HH->name, GMT->current.io.r_mode)) == NULL)
 		return (GMT_GRDIO_OPEN_FAILED);
 
 	gmt_M_memset (&mggHeader, 1, MGG_GRID_HEADER_2);
@@ -227,14 +236,14 @@ int gmt_mgg2_read_grd_info (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header
 
 	/* Check the magic number and size of header */
 	if (ok == -1) {
-		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Error: Unrecognized header, expected 0x%04X saw 0x%04X\n",
+		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Unrecognized header, expected 0x%04X saw 0x%04X\n",
 		            GRD98_MAGIC_NUM + GRD98_VERSION, mggHeader.version);
 		gmt_fclose (GMT, fp);
 		return (GMT_GRDIO_GRD98_BADMAGIC);
 	}
 
 	if (mggHeader.length != sizeof (MGG_GRID_HEADER_2)) {
-		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Error: Invalid grid header size, expected %d, found %d\n",
+		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Invalid grid header size, expected %d, found %d\n",
 		            (int)sizeof (MGG_GRID_HEADER_2), mggHeader.length);
 		gmt_fclose (GMT, fp);
 		return (GMT_GRDIO_GRD98_BADLENGTH);
@@ -251,10 +260,11 @@ int gmt_mgg2_write_grd_info (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *heade
 	FILE *fp = NULL;
 	MGG_GRID_HEADER_2 mggHeader;
 	int err;
+	struct GMT_GRID_HEADER_HIDDEN *HH = gmt_get_H_hidden (header);
 
-	if (!strcmp (header->name, "="))
+	if (!strcmp (HH->name, "="))
 		fp = GMT->session.std[GMT_OUT];
-	else if ((fp = gmt_fopen (GMT, header->name, GMT->current.io.w_mode)) == NULL)
+	else if ((fp = gmt_fopen (GMT, HH->name, GMT->current.io.w_mode)) == NULL)
 		return (GMT_GRDIO_CREATE_FAILED);
 
 	if ((err = grd98_GMTtoMGG2 (header, &mggHeader)) != 0) {
@@ -272,13 +282,13 @@ int gmt_mgg2_write_grd_info (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *heade
 	return (GMT_NOERROR);
 }
 
-int gmt_mgg2_read_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, float *grid, double wesn[], unsigned int *pad, unsigned int complex_mode) {
+int gmt_mgg2_read_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, gmt_grdfloat *grid, double wesn[], unsigned int *pad, unsigned int complex_mode) {
 	MGG_GRID_HEADER_2 mggHeader;
 	FILE *fp = NULL;
 	int *tLong = NULL;
 	short *tShort = NULL;
 	char *tChar = NULL;
-	float *tFloat = NULL;
+	gmt_grdfloat *tFloat = NULL;
 	bool piping = false, is_float = false;
 	int j, first_col, last_col, first_row, last_row, swap_all = 0;
 	unsigned int width_in, height_in;
@@ -287,13 +297,14 @@ int gmt_mgg2_read_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, flo
 	off_t long_offset;	/* For fseek only */
 	size_t n_expected;	/* Items in one row */
 	size_t size;		/* Item size */
+	struct GMT_GRID_HEADER_HIDDEN *HH = gmt_get_H_hidden (header);
 
 	gmt_M_memset (&mggHeader, 1, MGG_GRID_HEADER_2);
-	if (!strcmp (header->name, "=")) {
+	if (!strcmp (HH->name, "=")) {
 		fp = GMT->session.std[GMT_IN];
 		piping = true;
 	}
-	else if ((fp = gmt_fopen (GMT, header->name, GMT->current.io.r_mode)) != NULL) {
+	else if ((fp = gmt_fopen (GMT, HH->name, GMT->current.io.r_mode)) != NULL) {
 		if (gmt_M_fread (&mggHeader, sizeof (MGG_GRID_HEADER_2), 1U, fp) != 1) {
 			gmt_fclose (GMT, fp);
 			return (GMT_GRDIO_READ_FAILED);
@@ -308,9 +319,9 @@ int gmt_mgg2_read_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, flo
 	else
 		return (GMT_GRDIO_OPEN_FAILED);
 
-	is_float = (mggHeader.numType < 0 && abs (mggHeader.numType) == (int)sizeof (float));	/* Float file */
+	is_float = (mggHeader.numType < 0 && abs (mggHeader.numType) == (int)sizeof (gmt_grdfloat));	/* Float file */
 
-	gmt_M_err_pass (GMT, gmt_grd_prep_io (GMT, header, wesn, &width_in, &height_in, &first_col, &last_col, &first_row, &last_row, &actual_col), header->name);
+	gmt_M_err_pass (GMT, gmt_grd_prep_io (GMT, header, wesn, &width_in, &height_in, &first_col, &last_col, &first_row, &last_row, &actual_col), HH->name);
 	(void)gmtlib_init_complex (header, complex_mode, &imag_offset);	/* Set offset for imaginary complex component */
 
 	width_out = width_in;		/* Width of output array */
@@ -319,7 +330,7 @@ int gmt_mgg2_read_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, flo
 
 	n_expected = header->n_columns;
 	tLong  = gmt_M_memory (GMT, NULL, n_expected, int);
-	tShort = (short *)tLong;	tChar = (char *)tLong;	tFloat = (float *)tLong;
+	tShort = (short *)tLong;	tChar = (char *)tLong;	tFloat = (gmt_grdfloat *)tLong;
 	size = abs (mggHeader.numType);
 
 	if (piping)	{ /* Skip data by reading it */
@@ -339,7 +350,7 @@ int gmt_mgg2_read_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, flo
 	}
 
 	header->z_min = DBL_MAX;	header->z_max = -DBL_MAX;
-	header->has_NaNs = GMT_GRID_NO_NANS;	/* We are about to check for NaNs and if none are found we retain 1, else 2 */
+	HH->has_NaNs = GMT_GRID_NO_NANS;	/* We are about to check for NaNs and if none are found we retain 1, else 2 */
 	for (j = first_row, j2 = 0; j <= last_row; j++, j2++) {
 		if (gmt_M_fread (tLong, size, n_expected, fp) != n_expected) {
 			gmt_fclose (GMT, fp);
@@ -352,7 +363,7 @@ int gmt_mgg2_read_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, flo
 			if (mggHeader.numType == sizeof (int)) {
 				if (swap_all) grd98_swap_long (&tLong[actual_col[i]]);
 				if (tLong[actual_col[i]] == mggHeader.nanValue) grid[kk] = GMT->session.f_NaN;
-				else grid[kk] = (float) tLong[actual_col[i]] / (float) mggHeader.precision;
+				else grid[kk] = (gmt_grdfloat) tLong[actual_col[i]] / (gmt_grdfloat) mggHeader.precision;
 			}
 			else if (is_float) {
 				if (swap_all) grd98_swap_long (&tLong[actual_col[i]]);
@@ -363,12 +374,12 @@ int gmt_mgg2_read_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, flo
 			else if (mggHeader.numType == sizeof (short)) {
 				if (swap_all) grd98_swap_word(&tShort[actual_col[i]]);
 				if (tShort[actual_col[i]] == mggHeader.nanValue) grid[kk] = GMT->session.f_NaN;
-				else grid[kk] = (float) tShort[actual_col[i]] / (float) mggHeader.precision;
+				else grid[kk] = (gmt_grdfloat) tShort[actual_col[i]] / (gmt_grdfloat) mggHeader.precision;
 			}
 
 			else if (mggHeader.numType == sizeof (char)) {
 				if (tChar[actual_col[i]] == mggHeader.nanValue) grid[kk] = GMT->session.f_NaN;
-				else grid[kk] = (float) tChar[actual_col[i]] / (float) mggHeader.precision;
+				else grid[kk] = (gmt_grdfloat) tChar[actual_col[i]] / (gmt_grdfloat) mggHeader.precision;
 			}
 			else {
 				gmt_fclose (GMT, fp);
@@ -377,7 +388,7 @@ int gmt_mgg2_read_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, flo
 				return (GMT_GRDIO_UNKNOWN_TYPE);
 			}
 			if (gmt_M_is_fnan (grid[kk])) {
-				header->has_NaNs = GMT_GRID_HAS_NANS;
+				HH->has_NaNs = GMT_GRID_HAS_NANS;
 				continue;
 			}
 			/* Update z_min, z_max */
@@ -385,6 +396,8 @@ int gmt_mgg2_read_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, flo
 			header->z_max = MAX (header->z_max, (double)grid[kk]);
 		}
 	}
+	if (header->z_min == DBL_MAX && header->z_max == -DBL_MAX) /* No valid data values in the grid */
+		header->z_min = header->z_max = NAN;
 	if (piping)	{ /* Skip data by reading it */
 		int n_rows = header->n_rows;
 		for (j = last_row + 1; j < n_rows; j++) {
@@ -409,7 +422,7 @@ int gmt_mgg2_read_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, flo
 	return (GMT_NOERROR);
 }
 
-int gmt_mgg2_write_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, float *grid, double wesn[], unsigned int *pad, unsigned int complex_mode) {
+int gmt_mgg2_write_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, gmt_grdfloat *grid, double wesn[], unsigned int *pad, unsigned int complex_mode) {
 	MGG_GRID_HEADER_2 mggHeader;
 	bool is_float = false, check;
 	int i, j, err;
@@ -421,17 +434,18 @@ int gmt_mgg2_write_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, fl
 	int *tLong = NULL;
 	short *tShort = NULL;
 	char *tChar = NULL;
-	float *tFloat = NULL;
+	gmt_grdfloat *tFloat = NULL;
 	FILE *fp = NULL;
+	struct GMT_GRID_HEADER_HIDDEN *HH = gmt_get_H_hidden (header);
 
-	if (!strcmp (header->name, "="))
+	if (!strcmp (HH->name, "="))
 		fp = GMT->session.std[GMT_OUT];
-	else if ((fp = gmt_fopen (GMT, header->name, GMT->current.io.w_mode)) == NULL)
+	else if ((fp = gmt_fopen (GMT, HH->name, GMT->current.io.w_mode)) == NULL)
 		return (GMT_GRDIO_CREATE_FAILED);
 
 	check = !isnan (header->nan_value);
 
-	gmt_M_err_pass (GMT, gmt_grd_prep_io (GMT, header, wesn, &width_out, &height_out, &first_col, &last_col, &first_row, &last_row, &actual_col), header->name);
+	gmt_M_err_pass (GMT, gmt_grd_prep_io (GMT, header, wesn, &width_out, &height_out, &first_col, &last_col, &first_row, &last_row, &actual_col), HH->name);
 	(void)gmtlib_init_complex (header, complex_mode, &imag_offset);	/* Set offset for imaginary complex component */
 
 	width_in = width_out;		/* Physical width of input array */
@@ -456,6 +470,8 @@ int gmt_mgg2_write_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, fl
 			}
 		}
 	}
+	if (header->z_min == DBL_MAX && header->z_max == -DBL_MAX) /* No valid data values in the grid */
+		header->z_min = header->z_max = NAN;
 
 	/* store header information and array */
 	if ((err = grd98_GMTtoMGG2(header, &mggHeader)) != 0) {
@@ -468,10 +484,10 @@ int gmt_mgg2_write_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, fl
 		gmt_fclose (GMT, fp);
 		return (GMT_GRDIO_WRITE_FAILED);
 	}
-	is_float = (mggHeader.numType < 0 && abs (mggHeader.numType) == (int)sizeof (float));	/* Float file */
+	is_float = (mggHeader.numType < 0 && abs (mggHeader.numType) == (int)sizeof (gmt_grdfloat));	/* Float file */
 
 	tLong = gmt_M_memory (GMT, NULL, width_in, int);
-	tShort = (short *) tLong;	tChar = (char *)tLong;	tFloat = (float *) tLong;
+	tShort = (short *) tLong;	tChar = (char *)tLong;	tFloat = (gmt_grdfloat *) tLong;
 
 	i2 = first_col + pad[XLO];
 	size = abs (mggHeader.numType);
@@ -481,7 +497,7 @@ int gmt_mgg2_write_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, fl
 			kk = ij+actual_col[iu];
 			if (gmt_M_is_fnan (grid[kk])) {
 				if (mggHeader.numType == sizeof (int))       tLong[iu]  = mggHeader.nanValue;
-				else if (is_float) tFloat[iu]  = (float)mggHeader.nanValue;
+				else if (is_float) tFloat[iu]  = (gmt_grdfloat)mggHeader.nanValue;
 				else if (mggHeader.numType == sizeof (short)) tShort[iu] = (short)mggHeader.nanValue;
 				else if (mggHeader.numType == sizeof (char))  tChar[iu] = (char)mggHeader.nanValue;
 				else {
@@ -491,7 +507,7 @@ int gmt_mgg2_write_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, fl
 					return (GMT_GRDIO_UNKNOWN_TYPE);
 				}
 			} else {
-				if (grid[kk] > -0.1 && grid[kk] < 0) grid[kk] = (float)(-0.1);
+				if (grid[kk] > -0.1 && grid[kk] < 0) grid[kk] = (gmt_grdfloat)(-0.1);
 
 				if (mggHeader.numType == sizeof (int))
 					tLong[iu] = (int)rint ((double)grid[kk] * mggHeader.precision);
