@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *	Copyright (c) 1991-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,13 +12,13 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  * Machine-dependent macros for generation and testing of NaNs.
  *
  * These routines use the IEEE definition of Silent NaNs to set NaNs and
- * use the avialable isnan* routines to test NaNs whenever available.
+ * use the available isnan* routines to test NaNs whenever available.
  *
  * Notes:
  *    If your system has no IEEE support, add -DNO_IEEE to CFLAGS
@@ -27,7 +26,7 @@
  *
  * Author:	Remko Scharroo
  * Date:	1-JAN-2010
- * Ver:		5 API
+ * Ver:		6 API
  */
 
 /*!
@@ -35,8 +34,8 @@
  * \brief Machine-dependent macros for generation and testing of NaNs
  */
 
-#ifndef _GMT_NAN_H
-#define _GMT_NAN_H
+#ifndef GMT_NAN_H
+#define GMT_NAN_H
 
 #include "gmt_notposix.h"
 
@@ -56,4 +55,4 @@
 #	define gmt_M_is_dinf isinf
 #endif
 
-#endif /* _GMT_NAN_H */
+#endif /* GMT_NAN_H */
