@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *	Copyright (c) 1991-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  * Public function prototypes for GMT API option parsing.
@@ -134,37 +133,45 @@ GMT_LOCAL int parse_B_arg_inspector (struct GMT_CTRL *GMT, char *in) {
 				else if (k < last && in[k+1] == 'o') {mod = 'o'; ignore5 = false; gmt5++;}	/* oblique pole settings */
 				else if (k < last && in[k+1] == 'p') {mod = 'p'; ignore5 = true;  gmt5++;}	/* prefix settings */
 				else if (k < last && in[k+1] == 'l') {mod = 'l'; ignore5 = true;  gmt5++;}	/* Label */
+				else if (k < last && in[k+1] == 'L') {mod = 'L'; ignore5 = true;  gmt5++;}	/* Forced horizontal Label */
+				else if (k < last && in[k+1] == 's') {mod = 's'; ignore5 = true;  gmt5++;}	/* Secondary label */
+				else if (k < last && in[k+1] == 'S') {mod = 'S'; ignore5 = true;  gmt5++;}	/* Forced horizontal Secondary lLabel */
 				else if (k < last && in[k+1] == 't') {mod = 't'; ignore5 = true;  gmt5++;}	/* title */
+				else if (k < last && in[k+1] == 'n') {mod = 'n'; ignore5 = true;  gmt5++;}	/* Turn off frames and annotations */
 				else if (k && (in[k-1] == 'Z' || in[k-1] == 'z')) {ignore5 = false; gmt4++;}	/* Z-axis with 3-D box */
 				break;
 			case 'c':	/* If following a number this is unit c for seconds in GMT4 */
 				if (!custom && k && (in[k-1] == '.' || isdigit (in[k-1]))) gmt4++;	/* Old-style second unit */
-			case 'W': case 'E': case 'S': case 'N': case 'Z': case 'w': case 'e': case 'n': case 'z':	/* Not checking s as confusion with seconds */
+				break;
+			case 'W': case 'E': case 'S': case 'N': case 'Z': case 'w': case 'e': case 'z':	/* Not checking s as confusion with seconds and n because of +n */
 				if (k > 1) wesn_at_end++;	/* GMT5 has -B<WESNwesn> up front while GMT4 usually has them at the end */
 				break;
+			case 'n':	/* Tell this apart from +n */
+				if (!(k && in[k-1] == '+') && k > 1) wesn_at_end++;	/* GMT5 has -B<WESNwesn> up front while GMT4 usually has them at the end */
+				break;
 		}
 	}
 	if (!gmt5 && wesn_at_end) gmt4++;		/* Presumably got WESNwesn stuff towards the end of the option */
 	if (n_colons && (n_colons % 2) == 0) gmt4++;	/* Presumably :labels: in GMT4 style as any +mod would have kicked in above */
 	if (!custom && n_slashes) gmt4++;		/* Presumably / to separate axis in GMT4 style */
 	if (colon_text) gmt4++;				/* Gave title, suffix, prefix in GMT4 style */
-	GMT_Report (GMT->parent, GMT_MSG_DEBUG, "parse_B_arg_inspector: GMT4 = %d vs. GMT5 = %d\n", gmt4, gmt5);
+	GMT_Report (GMT->parent, GMT_MSG_DEBUG, "parse_B_arg_inspector: GMT4 = %d vs. GMT = %d\n", gmt4, gmt5);
 	if (gmt5 && !gmt4) {
-		GMT_Report (GMT->parent, GMT_MSG_DEBUG, "parse_B_arg_inspector: Detected GMT 5 style elements in -B option\n");
+		GMT_Report (GMT->parent, GMT_MSG_DEBUG, "parse_B_arg_inspector: Detected GMT >=5 style elements in -B option\n");
 		return (5);
 	}
 	else if (gmt4 && !gmt5) {
-		GMT_Report (GMT->parent, GMT_MSG_VERBOSE, "parse_B_arg_inspector: Detected GMT 4 style elements in -B option\n");
+		GMT_Report (GMT->parent, GMT_MSG_DEBUG, "parse_B_arg_inspector: Detected GMT 4 style elements in -B option\n");
 		return (4);
 	}
 	else if (gmt4 && gmt5) {	/* Mixed case is never allowed */
-		GMT_Report (GMT->parent, GMT_MSG_VERBOSE, "parse_B_arg_inspector: Error: Detected both GMT 4 and GMT 5 style elements in -B option. Unable to parse.\n");
-		if (n_slashes) GMT_Report (GMT->parent, GMT_MSG_VERBOSE, "parse_B_arg_inspector: Slashes no longer separate axis specifications, use -B[xyz] and repeat\n");
-		if (colon_text || n_colons) GMT_Report (GMT->parent, GMT_MSG_VERBOSE, "parse_B_arg_inspector: Colons no longer used for titles, labels, prefix, and suffix; see +t, +l, +p, +s\n");
+		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "parse_B_arg_inspector: Detected both GMT 4 and >= style elements in -B option. Unable to parse.\n");
+		if (n_slashes) GMT_Report (GMT->parent, GMT_MSG_NORMAL, "parse_B_arg_inspector: Slashes no longer separate axis specifications, use -B[xyz] and repeat\n");
+		if (colon_text || n_colons) GMT_Report (GMT->parent, GMT_MSG_NORMAL, "parse_B_arg_inspector: Colons no longer used for titles, labels, prefix, and suffix; see +t, +l, +p, +s\n");
 		return (-1);
 	}
 	else {
-		GMT_Report (GMT->parent, GMT_MSG_DEBUG, "parse_B_arg_inspector: Assume GMT 5 style format in -B option\n");
+		GMT_Report (GMT->parent, GMT_MSG_DEBUG, "parse_B_arg_inspector: Assume new GMT style format in -B option\n");
 		return (9);
 	}
 }
@@ -230,7 +237,7 @@ GMT_LOCAL unsigned int parse_check_extended_R (struct GMT_CTRL *GMT, struct GMT_
 	}
 	if (GMT->common.R.active[ISET])
 		return 0;
-	GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Error: -R[L|C|R][T|M|B]<x0>/<y0>/<n_columns>/<ny> requires grid spacings via -I\n");
+	GMT_Report (GMT->parent, GMT_MSG_NORMAL, "-R[L|C|R][T|M|B]<x0>/<y0>/<n_columns>/<ny> requires grid spacings via -I\n");
 	return 1;
 }
 
@@ -243,7 +250,7 @@ GMT_LOCAL int parse_complete_options (struct GMT_CTRL *GMT, struct GMT_OPTION *o
 	 * If found, see if we have a matching command line history and then
 	 * update that entry in the option list.
 	 * Finally, keep the option arguments in the history list.
-	 * However, when func_level > 1, do update the entry, but do not
+	 * However, when func_level > GMT_TOP_MODULE, do update the entry, but do not
 	 * remember it in history. Note, there are two special cases here:
 	 * -J is special since we also need to deal with the sub-species
 	 *    like -JM, -JX, etc.  So these all have separate entries.
@@ -253,13 +260,13 @@ GMT_LOCAL int parse_complete_options (struct GMT_CTRL *GMT, struct GMT_OPTION *o
 	 *    Also, a single -B in the options may expand to several
 	 *    separate -B<args> so the linked options list may grow.
 	 */
-	int id = 0, k, n_B = 0, B_id;
+	int id = 0, k, n_B = 0, B_id, update_id = 0;
 	unsigned int pos = 0, B_replace = 1;
-	bool update, remember;
+	bool update, remember, check_B;
 	struct GMT_OPTION *opt = NULL, *opt2 = NULL, *B_next = NULL;
 	char str[3] = {""}, B_string[GMT_BUFSIZ] = {""}, p[GMT_BUFSIZ] = {""}, B_delim[2] = {30, 0};	/* Use ASCII 30 RS Record Separator between -B strings */
 
-	remember = (GMT->hidden.func_level == 1);   /* Only update the history for top level function */
+	remember = (GMT->hidden.func_level == GMT_TOP_MODULE);   /* Only update the history for top level function */
 
 	for (opt = options; opt; opt = opt->next) {
 		if (opt->option == 'B') {	/* Do some initial counting of how many -B options and determine if there is just one with no args */
@@ -267,25 +274,51 @@ GMT_LOCAL int parse_complete_options (struct GMT_CTRL *GMT, struct GMT_OPTION *o
 			n_B++;
 		}
 	}
-	for (k = 0, B_id = -1; k < GMT_N_UNIQUE && B_id == -1; k++)
+	for (k = 0, B_id = GMT_NOTSET; k < GMT_N_UNIQUE && B_id == GMT_NOTSET; k++)
 		if (!strcmp (GMT_unique_option[k], "B")) B_id = k;	/* B_id === 0 but just in case this changes we do this search anyway */
 
+	check_B = (strncmp (GMT->init.module_name, "psscale", 7U) && strncmp (GMT->init.module_name, "docs", 4U));
+	if (GMT->current.setting.run_mode == GMT_MODERN && n_B && check_B) {	/* Write gmt.frame file unless module is psscale, overwriting any previous file */
+		char file[PATH_MAX] = {""};
+		FILE *fp = NULL;
+		snprintf (file, PATH_MAX, "%s/gmt%d.%s/gmt.frame", GMT->parent->session_dir, GMT_MAJOR_VERSION, GMT->parent->session_name);
+		if ((fp = fopen (file, "w")) == NULL) {
+			GMT_Report (GMT->parent, GMT_MSG_DEBUG, "Unable to create file %s\n", file);
+			return (-1);
+		}
+		for (k = 0, opt = options; opt; opt = opt->next) {
+			if (opt->option != 'B') continue;
+			if (k) fprintf (fp, "%s", B_delim);
+			fprintf (fp, "%s", opt->arg);
+			k++;
+		}
+		fprintf (fp, "\n");
+		fclose (fp);
+	}
 	for (opt = options; opt; opt = opt->next) {
 		if (!strchr (GMT_SHORTHAND_OPTIONS, opt->option)) continue;	/* Not one of the shorthand options */
 		if (GMT->current.setting.run_mode == GMT_MODERN && opt->option == 'B') continue;	/* The -B option is NOT a shorthand option under modern mode */
 		update = false;
-		GMT_Report (GMT->parent, GMT_MSG_DEBUG, "History: Process -%c%s.\n", opt->option, opt->arg);
+		GMT_Report (GMT->parent, GMT_MSG_DEBUG, "History: Process -%c%s\n", opt->option, opt->arg);
 
 		str[0] = opt->option; str[1] = str[2] = '\0';
 		if (opt->option == 'J') {               /* -J is special since it can be -J or -J<code> */
+
 			/* Always look up "J" first. It comes before "J?" and tells what the last -J was */
-			if ((id = gmtlib_get_option_id (0, str)) == -1) Return;	/* No -J found at all - nothing more to do */
+			if ((id = gmt_get_option_id (0, str)) == GMT_NOTSET) Return;	/* No -J found at all - nothing more to do */
 			if (opt->arg && opt->arg[0]) {      /* Gave -J<code>[<args>] so we either use or update history and continue */
 				str[1] = opt->arg[0];
 				/* Remember this last -J<code> for later use as -J, but do not remember it when -Jz|Z */
-				if (str[1] != 'Z' && str[1] != 'z' && remember) {
-					gmt_M_str_free (GMT->init.history[id]);
-					GMT->init.history[id] = strdup (&str[1]);
+				if (remember) {
+					if (str[1] == 'Z' || str[1] == 'z') {
+						int z_id = gmt_get_option_id (0, "Z");
+						gmt_M_str_free (GMT->init.history[z_id]);
+						GMT->init.history[z_id] = strdup (&str[1]);
+					}
+					else {
+						gmt_M_str_free (GMT->init.history[id]);
+						GMT->init.history[id] = strdup (&str[1]);
+					}
 				}
 				if (opt->arg[1]) update = true; /* Gave -J<code><args> so we want to update history and continue */
 			}
@@ -294,7 +327,8 @@ GMT_LOCAL int parse_complete_options (struct GMT_CTRL *GMT, struct GMT_OPTION *o
 				str[1] = GMT->init.history[id][0];
 			}
 			/* Continue looking for -J<code> */
-			if ((id = gmtlib_get_option_id (id + 1, str)) == -1) Return;	/* No -J<code> found */
+			if ((id = gmt_get_option_id (id + 1, str)) == GMT_NOTSET) Return;	/* No -J<code> found */
+			update_id = id;
 		}
 		else if (opt->option == 'B') {          /* -B is also special since there may be many of these, or just -B */
 			if (B_replace) {                    /* Only -B is given and we want to use the history */
@@ -319,23 +353,37 @@ GMT_LOCAL int parse_complete_options (struct GMT_CTRL *GMT, struct GMT_OPTION *o
 			}
 		}
 		else {	/* Gave -R[<args>], -V[<args>] etc., so we either use or update the history and continue */
-			if ((id = gmtlib_get_option_id (0, str)) == -1) Return;	/* Error: user gave shorthand option but there is no record in the history */
-			if (GMT->current.setting.run_mode == GMT_MODERN && opt->option == 'R' && !GMT->current.ps.active)	/* Check RG instead */
-				id++;	/* RG follows R in gmt_unique.h order [Modern mode only] */
+			update_id = id = gmt_get_option_id (0, str);	/* If -R then we get id for RP */
+			if (id == GMT_NOTSET) Return;	/* Error: user gave shorthand option but there is no record in the history */
+			if (GMT->current.setting.run_mode == GMT_MODERN && opt->option == 'R') {	/* Must deal with both RP and RG under modern mode */
+				if (GMT->current.ps.active || !strncmp (GMT->init.module_name, "subplot", 7U) || !strncmp (GMT->init.module_name, "pscoast", 7U) || !strncmp (GMT->init.module_name, "psbasemap", 9U)) {	/* Plotting module plus special options to pscoast and psbasemap: First check RP if it exists */
+					if (!GMT->init.history[id]) id++;	/* No RP in the history, try RG next */
+				}
+				else {	/* Only try RG for non-plotting modules. RG follows RP in gmt_unique.h order [Modern mode only] */
+					id++;	/* Go to RG */
+					update_id = id;	/* We want to update the RG history since it is not a plotter */
+				}
+			}
 			if (opt->arg && opt->arg[0]) update = true;	/* Gave -R<args>, -V<args> etc. so we we want to update history and continue */
 		}
 		if (opt->option != 'B') {               /* Do -B separately again after the loop so skip it here */
 			if (id < 0) Return;                 /* Error: user gave shorthand option but there is no record in the history */
 			if (update) {                       /* Gave -J<code><args>, -R<args>, -V<args> etc. so we update history and continue */
 				if (remember) {
-					gmt_M_str_free (GMT->init.history[id]);
-					GMT->init.history[id] = strdup (opt->arg);
+					gmt_M_str_free (GMT->init.history[update_id]);
+					GMT->init.history[update_id] = strdup (opt->arg);
 				}
 			}
 			else {	/* Gave -J<code>, -R, -J etc. so we complete the option and continue */
 				if (!GMT->init.history[id]) Return;
 				gmt_M_str_free (opt->arg);   /* Free previous pointer to arg */
 				opt->arg = strdup (GMT->init.history[id]);
+				if (id != update_id) {	/* Happens for -R when we need the RG settings as first-time RP setting via -R */
+					char *c = NULL;
+					if ((c = strstr (opt->arg, "+I")) || (c = strstr (opt->arg, "+G")))	/* Strip off increment/node setting when used for plot domain */
+						c[0] = '\0';
+					if (remember) GMT->init.history[update_id] = strdup (opt->arg);
+				}
 			}
 		}
 	}
@@ -385,24 +433,47 @@ int GMT_List_Args (void *V_API, struct GMT_OPTION *head) {
 }
 #endif
 
+/*! . */
+GMT_LOCAL struct GMT_OPTION *fix_gdal_files (struct GMT_OPTION *opt) {
+	char *pch = NULL;
+	if (((pch = strstr (opt->arg, "+b")) != NULL) && !strstr(opt->arg, "=gd")) {
+		/* Here we deal with the case that the filename has embedded a band request for gdalread, as in img.tif+b1
+		   However, the issue is that for these cases the machinery is set only to parse the request in the
+		   form of img.tif=gd+b1 so the trick is to insert the missing '=gd' in the filename and let t go.
+		   JL 29-November 2014
+		*/
+		char t[GMT_LEN256] = {""};
+		pch[0] = '\0';
+		strncpy (t, opt->arg, GMT_LEN256-1);
+		strcat (t, "=gd"); 
+		pch[0] = '+';			/* Restore what we have erased 2 lines above */
+		strncat(t, pch, GMT_LEN256-1);
+		gmt_M_str_free (opt->arg);	/* free it so that we can extend it */
+		opt->arg = strdup (t);
+	}
+	return opt;
+}
+
 /*! . */
 struct GMT_OPTION *GMT_Create_Options (void *V_API, int n_args_in, const void *in) {
 	/* This function will loop over the n_args_in supplied command line arguments (in) and
-	 * returns a linked list of GMT_OPTION structures for each program option.
-	 * These will in turn will be processed by the program-specific option parsers.
+	 * then creates a linked list of GMT_OPTION structures for each program option.
+	 * These will in turn will be processed by the module-specific option parsers.
 	 * What actually happens is controlled by n_args_in.  There are three cases:
-	 * n_args_in < 0 : This means that in is already a linked list and we just return it.
+	 * n_args_in < 0 : This means that in is already a linked list and we just duplicate and return a copy.
 	 * n_args_in == 0: in is a single text string with multiple options (e.g., "-R0/2/0/5 -Jx1 -O -K > file")
 	 *		   and we must first break this command string into separate words.
 	 * n_args_in > 0 : in is an array of text strings (e.g., argv[]).
 	 *
 	 * Note: 1. If argv[0] is the calling program name, make sure to pass argc-1, args+1 instead.
 	 *	 2. in == NULL is allowed only for n_args_in <= 0 (an empty list of options).
+	 *
+	 * The linked list returned by GMT_Create_Options should be freed by GMT_Destroy_Options().
 	 */
 
 	int error = GMT_OK;
 	unsigned int arg, first_char, append = 0, n_args;
-	char option, **args = NULL, **new_args = NULL, *pch = NULL, *this_arg = NULL, buffer[BUFSIZ] = {""};
+	char option, **args = NULL, **new_args = NULL, *this_arg = NULL, buffer[GMT_LEN1024] = {""};
 	struct GMT_OPTION *head = NULL, *new_opt = NULL;
 	struct GMT_CTRL *G = NULL;
 	struct GMTAPI_CTRL *API = NULL;
@@ -410,14 +481,25 @@ struct GMT_OPTION *GMT_Create_Options (void *V_API, int n_args_in, const void *i
 	if (V_API == NULL) return_null (V_API, GMT_NOT_A_SESSION);		/* GMT_Create_Session has not been called */
 	if (in == NULL && n_args_in > 1) return_null (V_API, GMT_ARGV_LIST_NULL);	/* Gave no argument pointer but said we had at least 1 */
 	if (in == NULL) return (NULL);	/* Gave no argument pointer so a null struct is returned */
-	if (n_args_in < 0) return ((struct GMT_OPTION *)in);	/* Already was to linked list */
-	API = parse_get_api_ptr (V_API);	/* Convert API to a GMTAPI_CTRL pointer */
+	API = parse_get_api_ptr (V_API);	/* Convert V_API to a GMTAPI_CTRL pointer */
+	if (n_args_in < 0) {	/* Already was given a linked list.  Duplicate and return the copy */
+		struct GMT_OPTION *head_in = (struct GMT_OPTION *)in;
+		while (head_in) {
+			if ((new_opt = GMT_Make_Option (API, head_in->option, head_in->arg)) == NULL)	/* Make option with the listing name flagged as option -= */
+				return_null (API, API->error);	/* Create the new option structure given the args, or return the error */
+			if ((head = GMT_Append_Option (API, new_opt, head)) == NULL)		/* Hook new option to the end of the list (or initiate list if new == NULL) */
+				return_null (API, API->error);	/* Create the new option structure given the args, or return the error */
+			head_in = head_in->next;	/* Go to next input option */
+		}
+		return (head);
+	}
+	
 	G = API->GMT;	/* GMT control structure */
 	if (n_args_in == 0) {	/* Check if a single command line, if so break into tokens */
 		unsigned int pos = 0, new_n_args = 0, k;
 		bool quoted;
 		size_t n_alloc = GMT_SMALL_CHUNK;
-		char p[GMT_BUFSIZ] = {""}, *txt_in = strdup (in);	/* Passed a single text string */
+		char p[GMT_LEN1024] = {""}, *txt_in = strdup (in);	/* Passed a single text string */
 		new_args = gmt_M_memory (G, NULL, n_alloc, char *);
 		/* txt_in can contain options that take multi-word text strings, e.g., -B+t"My title".  We avoid the problem of splitting
 		 * these items by temporarily replacing spaces inside quoted strings with ASCII 31 US (Unit Separator), do the strtok on
@@ -429,7 +511,8 @@ struct GMT_OPTION *GMT_Create_Options (void *V_API, int n_args_in, const void *i
 		}
 		while ((gmt_strtok (txt_in, " ", &pos, p))) {	/* Break up string into separate words, and strip off double quotes */
 			unsigned int i, o;
-			for (k = 0; p[k]; k++) if (p[k] == GMT_ASCII_GS) p[k] = '\t'; else if (p[k] == GMT_ASCII_US) p[k] = ' ';	/* Replace spaces and tabs masked above */
+			for (k = 0; p[k]; k++)
+				if (p[k] == GMT_ASCII_GS) p[k] = '\t'; else if (p[k] == GMT_ASCII_US) p[k] = ' ';	/* Replace spaces and tabs masked above */
 			for (i = o = 0; p[i]; i++) if (p[i] != '\"') p[o++] = p[i];	/* Ignore double quotes */
 			p[o] = '\0';
 			new_args[new_n_args++] = strdup (p);
@@ -457,7 +540,25 @@ struct GMT_OPTION *GMT_Create_Options (void *V_API, int n_args_in, const void *i
 		/* Note: The UNIX command line will never see redirections like >, >>, and < pass as arguments, so when we check for these
 		 * below it is because command-strings passed from external APIs may contain things like '-Fap -O -K >> plot.ps' */
 		
-		if (args[arg][0] == '<' && !args[arg][1] && (arg+1) < n_args && args[arg+1][0] != '-')	/* string command with "< file" for input */
+		if (args[arg][0] == '=' && args[arg][1] && !gmt_access (API->GMT, &args[arg][1], F_OK)) {	/* Gave a file list which we must expand into options */
+			char **flist = NULL;
+			uint64_t n_files, f;
+			n_files = gmtlib_read_list (API->GMT, &args[arg][1], &flist);
+			if ((new_opt = GMT_Make_Option (API, '=', &args[arg][1])) == NULL)	/* Make option with the listing name flagged as option -= */
+				return_null (API, error);	/* Create the new option structure given the args, or return the error */
+			head = GMT_Append_Option (API, new_opt, head);		/* Hook new option to the end of the list (or initiate list if head == NULL) */
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "GMT_Create_Options: Must expand list file %s\n", args[arg]);
+			for (f = 0; f < n_files; f++) {	/* Now expand all the listed files into options */
+				GMT_Report (API, GMT_MSG_DEBUG, "GMT_Create_Options: Adding input file: %s\n", flist[f]);
+				if ((new_opt = GMT_Make_Option (API, GMT_OPT_INFILE, flist[f])) == NULL)
+					return_null (API, error);	/* Create the new option structure given the args, or return the error */
+				new_opt = fix_gdal_files (new_opt);
+				head = GMT_Append_Option (API, new_opt, head);		/* Hook new option to the end of the list (or initiate list if head == NULL) */
+			}
+			gmtlib_free_list (API->GMT, flist, n_files);
+			continue;
+		}
+		else if (args[arg][0] == '<' && !args[arg][1] && (arg+1) < n_args && args[arg+1][0] != '-')	/* string command with "< file" for input */
 			first_char = 0, option = GMT_OPT_INFILE, arg++;
 		else if (args[arg][0] == '>' && !args[arg][1] && (arg+1) < n_args && args[arg+1][0] != '-')	/* string command with "> file" for output */
 			first_char = 0, option = GMT_OPT_OUTFILE, arg++;
@@ -487,31 +588,30 @@ struct GMT_OPTION *GMT_Create_Options (void *V_API, int n_args_in, const void *i
 				this_arg = &args[arg][first_char];
 			}
 			else {
-				sprintf (buffer, ">%s", &args[arg][first_char]);
+				snprintf (buffer, GMT_LEN1024, ">%s", &args[arg][first_char]);
 				this_arg = buffer;
 			}
 			append = 0;
 		}
 		else
 			this_arg = &args[arg][first_char];
-		if ((new_opt = GMT_Make_Option (API, option, this_arg)) == NULL)
-			return_null (API, error);	/* Create the new option structure given the args, or return the error */
 
-		if (option == GMT_OPT_INFILE && ((pch = strstr(new_opt->arg, "+b")) != NULL) && !strstr(new_opt->arg, "=gd")) {
-			/* Here we deal with the case that the filename has embedded a band request for gdalread, as in img.tif+b1
-			   However, the issue is that for these cases the machinery is set only to parse the request in the
-			   form of img.tif=gd+b1 so the trick is to insert the missing '=gd' in the filename and let go.
-			   JL 29-November 2014
-			*/
-			char t[GMT_LEN256] = {""};
-			pch[0] = '\0';
-			strcpy(t, new_opt->arg);
-			strcat(t, "=gd"); 
-			pch[0] = '+';			/* Restore what we have erased 2 lines above */
-			strcat(t, pch);
-			gmt_M_str_free (new_opt->arg);	/* free it so that we can extend it */
-			new_opt->arg = strdup(t);
+		/* To store -JEPSG:n or even -Jn in history we must prefix it with a '+' sign */
+		if (option == 'J' && (isdigit(this_arg[0]) || !strncmp(this_arg, "EPSG:", 5) || !strncmp(this_arg, "epsg:", 5))) {
+			char *t = malloc (strlen(this_arg)+2);
+			sprintf (t, "+%s", this_arg);
+			if ((new_opt = GMT_Make_Option (API, option, t)) == NULL) {
+				free (t);
+				return_null (API, error);	/* Create the new option structure given the args, or return the error */
+			}
+			free (t);
 		}
+		else
+			if ((new_opt = GMT_Make_Option (API, option, this_arg)) == NULL)
+				return_null (API, error);	/* Create the new option structure given the args, or return the error */
+
+		if (option == GMT_OPT_INFILE)	/* A special check on infiles to see if they refer to GDAL implicitly */
+			new_opt = fix_gdal_files (new_opt);
 
 		head = GMT_Append_Option (API, new_opt, head);		/* Hook new option to the end of the list (or initiate list if head == NULL) */
 	}
@@ -520,6 +620,24 @@ struct GMT_OPTION *GMT_Create_Options (void *V_API, int n_args_in, const void *i
 		gmt_M_free (G, new_args);
 	}
 
+	/* Check if we are in presence of a oneliner. If yes that implies MODERN mode. */
+	if (n_args_in == 0) {
+		int k;
+		bool code = false;
+		char figure[GMT_LEN512] = {""}, *c = NULL;
+		struct GMT_OPTION *opt = NULL;
+		for (opt = head; opt; opt = opt->next) {	/* Loop over all options */
+			if (opt->option == 'O' || opt->option == 'K') break;	/* Cannot be a one-liner if -O or -K are involved */
+			snprintf (figure, GMT_LEN512, "%c%s", opt->option, opt->arg);	/* So -png, which would be parse into -p and ng, are reunited to png */
+			if ((c = strchr (figure, ','))) c[0] = 0;	/* Chop of more than one format for the id test */
+			if ((k = gmt_get_graphics_id (API->GMT, figure)) == GMT_NOTSET) continue;	/* Not a quicky one-liner option */
+			if (opt->next && opt->next->option == GMT_OPT_INFILE) 	/* Found a -ext[,ext,ext,...] <prefix> pair */
+				code = true;
+		}
+		if (code)
+			API->GMT->current.setting.run_mode = GMT_MODERN;
+	}
+
 	return (head);		/* We return the linked list */
 }
 
@@ -590,7 +708,8 @@ char **GMT_Create_Args (void *V_API, int *argc, struct GMT_OPTION *head) {
 	 */
 
 	char **txt = NULL, buffer[GMT_BUFSIZ] = {""};
-	unsigned int arg = 0;
+	int arg = 0;
+	bool skip_infiles = false;
 	struct GMT_OPTION *opt = NULL;
 	struct GMT_CTRL *G = NULL;
 	struct GMTAPI_CTRL *API = NULL;
@@ -611,12 +730,18 @@ char **GMT_Create_Args (void *V_API, int *argc, struct GMT_OPTION *head) {
 		if (!opt->option) continue;			/* Skip all empty options */
 		if (opt->option == GMT_OPT_SYNOPSIS)		/* Produce special - option for synopsis */
 			sprintf (buffer, "-");
-		else if (opt->option == GMT_OPT_INFILE)		/* Option for input filename [or numbers] */
-			sprintf (buffer, "%s", opt->arg);
+		else if (opt->option == '=' && opt->arg[0]) {		/* Start of long list of file args initially given via -=<list> */
+			snprintf (buffer, GMT_BUFSIZ, "=%s", opt->arg);
+			skip_infiles = true;
+		}
+		else if (opt->option == GMT_OPT_INFILE)	{	/* Option for input filename [or numbers] */
+			if (skip_infiles) continue;
+			snprintf (buffer, GMT_BUFSIZ, "%s", opt->arg);
+		}
 		else if (opt->arg && opt->arg[0])			/* Regular -?arg commandline option with argument for some ? */
-			sprintf (buffer, "-%c%s", opt->option, opt->arg);
+			snprintf (buffer, GMT_BUFSIZ, "-%c%s", opt->option, opt->arg);
 		else							/* Regular -? commandline argument without argument */
-			sprintf (buffer, "-%c", opt->option);
+			snprintf (buffer, GMT_BUFSIZ, "-%c", opt->option);
 
 		txt[arg] = gmt_M_memory (G, NULL, strlen (buffer)+1, char);	/* Get memory for this item */
 
@@ -625,6 +750,10 @@ char **GMT_Create_Args (void *V_API, int *argc, struct GMT_OPTION *head) {
 		arg++;	/* One more option added */
 	}
 	/* OK, done processing all options */
+	if (arg < *argc) {	/* Shrink the list to fit */
+		*argc = arg;
+		txt = gmt_M_memory (G, txt, *argc, char *);
+	}
 	
 	return (txt);	/* Pass back the char** array to the calling module */
 }
@@ -652,8 +781,8 @@ char * GMT_Create_Cmd (void *V_API, struct GMT_OPTION *head) {
 	 * correspond to the linked options provided.
 	 */
 
-	char *txt = NULL, buffer[GMT_BUFSIZ] = {""};
-	bool first = true;
+	char *txt = NULL, *c = NULL, buffer[GMT_BUFSIZ] = {""};
+	bool first = true, skip_infiles = false;
 	size_t length = 0, inc, n_alloc = GMT_BUFSIZ;
 	struct GMT_OPTION *opt = NULL;
 	struct GMT_CTRL *G = NULL;
@@ -670,12 +799,30 @@ char * GMT_Create_Cmd (void *V_API, struct GMT_OPTION *head) {
 		if (!opt->option) continue;			/* Skip all empty options */
 		if (opt->option == GMT_OPT_SYNOPSIS)		/* Produce special - option for synopsis */
 			sprintf (buffer, "-");
-		else if (opt->option == GMT_OPT_INFILE)		/* Option for input filename [or numbers] */
-			sprintf (buffer, "%s", opt->arg);
-		else if (opt->arg && opt->arg[0])			/* Regular -?arg commandline option with argument for some ? */
-			sprintf (buffer, "-%c%s", opt->option, opt->arg);
+		else if (opt->option == '=' && opt->arg[0]) {		/* Special list file, add it but not the files that follow */
+			snprintf (buffer, GMT_BUFSIZ, "=%s", opt->arg);
+			skip_infiles = true;
+		}
+		else if (opt->option == GMT_OPT_INFILE)	{	/* Option for input filename [or numbers] */
+			if (skip_infiles) continue;
+			if (gmtlib_file_is_srtmlist (API, opt->arg))	/* Want to replace the srtm list with the original @earth_relief_xxx name instead */
+				snprintf (buffer, GMT_BUFSIZ, "@earth_relief_0%cs", opt->arg[strlen(opt->arg)-8]);
+			else if (gmt_M_file_is_remotedata (opt->arg) && (c = strstr (opt->arg, ".grd"))) {
+				c[0] = '\0';
+				snprintf (buffer, GMT_BUFSIZ, "%s", opt->arg);
+				c[0] = '.';
+			}
+			else
+				snprintf (buffer, GMT_BUFSIZ, "%s", opt->arg);
+		}
+		else if (opt->arg && opt->arg[0]) {			/* Regular -?arg commandline option with argument for some ? */
+			if (strchr (opt->arg, ' '))	/* Has a space in the argument, e.g., a title or similar */
+				snprintf (buffer, GMT_BUFSIZ, "-%c\"%s\"", opt->option, opt->arg);
+			else
+				snprintf (buffer, GMT_BUFSIZ, "-%c%s", opt->option, opt->arg);
+		}
 		else							/* Regular -? commandline argument without argument */
-			sprintf (buffer, "-%c", opt->option);
+			snprintf (buffer, GMT_BUFSIZ, "-%c", opt->option);
 
 		inc = strlen (buffer);
 		if (!first) inc++;	/* Count the space between args */
@@ -739,7 +886,6 @@ struct GMT_OPTION * GMT_Find_Option (void *V_API, char option, struct GMT_OPTION
 	struct GMT_OPTION *current = NULL;
 
 	if (V_API == NULL) return_null (V_API, GMT_NOT_A_SESSION);	/* GMT_Create_Session has not been called */
-	if (head == NULL)  return_null (V_API, GMT_OPTION_LIST_NULL);	/* Hard to find something in a non-existent list */
 	
 	for (current = head; current && current->option != option; current = current->next);	/* Linearly search for the specified option */
 	return (current);	/* NULL if not found */
@@ -771,7 +917,6 @@ int GMT_Expand_Option (void *V_API, struct GMT_OPTION *opt, const char *arg) {
 	s_length = strlen (arg); 
 	if ((s_length + strlen (opt->arg)) > BUFSIZ) return_error (V_API, GMT_DIM_TOO_LARGE);		/* Don't have room */
 
-	s_length = strlen(arg); 
 	while (opt->arg[in]) {
 		if (opt->arg[in] == '\"') quote = !quote;
 		if (opt->arg[in] == '?' && !quote) {	/* Found an unquoted questionmark */
