@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt_project.h 20055 2018-04-22 00:27:09Z pwessel $
  *
- *	Copyright (c) 1991-2018 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  * Include file for programs that use the map-projections functions.  Note
@@ -21,7 +20,7 @@
  *
  * Author:	Paul Wessel
  * Date:	1-JAN-2010
- * Version:	5 API
+ * Version:	6 API
  *
  */
 
@@ -30,8 +29,8 @@
  * \brief Include file for programs that use the map-projections functions.
  */
 
-#ifndef _GMT_PROJECT_H
-#define _GMT_PROJECT_H
+#ifndef GMT_PROJECT_H
+#define GMT_PROJECT_H
 
 #define HALF_DBL_MAX (DBL_MAX/2.0)
 
@@ -49,11 +48,12 @@ EXTERN_MSC double gmtmap_lat_swap_quick (struct GMT_CTRL *GMT, double lat, doubl
 #define gmt_M_lata_to_latg(C,lat) gmtmap_lat_swap_quick (C, lat, C->current.proj.lat_swap_vals.c[GMT_LATSWAP_A2G])
 
 /*! Macros returns true if the two coordinates are lon/lat; way should be GMT_IN or GMT_OUT */
-#define gmt_M_x_is_lon(C,way) (C->current.io.col_type[way][GMT_X] == GMT_IS_LON)
-#define gmt_M_y_is_lon(C,way) (C->current.io.col_type[way][GMT_Y] == GMT_IS_LON)
-#define gmt_M_x_is_lat(C,way) (C->current.io.col_type[way][GMT_X] == GMT_IS_LAT)
-#define gmt_M_y_is_lat(C,way) (C->current.io.col_type[way][GMT_Y] == GMT_IS_LAT)
+#define gmt_M_x_is_lon(C,way) (C->current.io.col_type[way][GMT_X] & GMT_IS_LON)
+#define gmt_M_y_is_lon(C,way) (C->current.io.col_type[way][GMT_Y] & GMT_IS_LON)
+#define gmt_M_x_is_lat(C,way) (C->current.io.col_type[way][GMT_X] & GMT_IS_LAT)
+#define gmt_M_y_is_lat(C,way) (C->current.io.col_type[way][GMT_Y] & GMT_IS_LAT)
 #define gmt_M_is_geographic(C,way) (gmt_M_x_is_lon(C,way) && gmt_M_y_is_lat(C,way))
+#define gmt_M_is_cartesian(C,way) (!gmt_M_is_geographic(C,way))
 
 #define GMT_N_PROJECTIONS	29	/* Total number of projections in GMT */
 
@@ -98,6 +98,7 @@ enum gmt_enum_conic {GMT_ALBERS = 200,
 
 /* Azimuthal projections tagged 300-399 */
 #define gmt_M_is_azimuthal(C) (C->current.proj.projection / 100 == 3)
+#define gmt_M_is_perspective(C) (C->current.proj.projection == GMT_ORTHO || C->current.proj.projection == GMT_GENPER)
 enum gmt_enum_azim {GMT_STEREO = 300,
 	GMT_LAMB_AZ_EQ,
 	GMT_ORTHO,
@@ -117,6 +118,9 @@ enum gmt_enum_misc {GMT_MOLLWEIDE = 400,
 	GMT_ECKERT6,
 	GMT_WINKEL};
 
+/* All projections from proj.4 lib */
+enum gmt_enum_allprojs {GMT_PROJ4_PROJS = 500};
+
 /*! The various GMT measurement units */
 enum gmt_enum_units {GMT_IS_METER = 0,
 	GMT_IS_KM,
@@ -176,7 +180,7 @@ struct GMT_LATSWAP_CONSTS {
 	double  c[GMT_LATSWAP_N][4];	/* Coefficients in 4-term series  */
 	double	ra;			/* Authalic   radius (sphere for equal-area)  */
 	double	rm;			/* Meridional radius (sphere for N-S distance)  */
-	bool spherical;		/* True if no conversions need to be done.  */
+	bool spherical;			/* True if no conversions need to be done.  */
 };
 
 struct GMT_THREE_D {
@@ -194,7 +198,7 @@ struct GMT_THREE_D {
 	unsigned int quadrant;	/* quadrant we're looking from */
 	unsigned int z_axis;	/* Which z-axis to draw. */
 	unsigned int face[3];	/* Tells if this facet has normal in pos direction */
-	bool draw[4];	/* axes to draw */
+	bool draw[4];		/* axes to draw */
 	bool fixed;		/* true if we want a given point to be fixed in the projection [for animations] */
 	bool world_given;	/* true if a fixed world point was given in -E ..+glon/lat/z */
 	bool view_given;	/* true if a fixed projected point was given in -E ..+cx0/y0 */
@@ -202,17 +206,18 @@ struct GMT_THREE_D {
 
 struct GMT_DATUM {	/* Main parameter for a particular datum */
 	double a, b, f, e_squared, ep_squared;
-	double xyz[3];
+	double xyz[7];
 	int ellipsoid_id;	/* Ellipsoid GMT ID number (or -1) */
 };
 
 struct GMT_DATUM_CONV {
-	bool h_given;	/* true if we have incoming height data [h = 0] */
+	bool h_given;		/* true if we have incoming height data [h = 0] */
 	double da;		/* Major semi-axis in meters */
 	double df;		/* Flattening */
 	double e_squared;	/* Eccentricity squared (e^2 = 2*f - f*f) */
 	double one_minus_f;	/* 1 - f */
 	double dxyz[3];		/* Ellipsoids offset in meter from Earth's center of mass for x,y, and z */
+	double bursa[7];	/* Bursa-Wolf seven parameters */
 	struct GMT_DATUM from, to;	/* The old and new datums */
 };
 
@@ -235,13 +240,17 @@ struct GMT_PROJ {
 	void (*inv_y) (struct GMT_CTRL *, double *, double);	/* Pointers to the selected linear y inverse function */
 	void (*inv_z) (struct GMT_CTRL *, double *, double);	/* Pointers to the selected linear z inverse function */
 
-	double pars[10];		/* Raw unprocessed map-projection parameters as passed on command line */
+	double pars[16];		/* Raw unprocessed map-projection parameters as passed on command line */
 	double z_pars[2];		/* Raw unprocessed z-projection parameters as passed on command line */
 
 	/* Common projection parameters */
 
 	int projection;		/* Gives the id number for the projection used (-1 if not set) */
+	int projection_GMT;	/* A copy of the above to use when using the Proj.4 lib for coord/map transforms */
+	int autoscl[2];		/* If +1 then set x/y-scale to y/x-scale and compute height/width, -1 reverses direction */
+	int corner[4];		/* In/on/out status of map corners w.r.t. current polygon for -R...+r regions */
 
+	bool search;		/* true if we are allowed to search along oblique boundary for min/max lon/lats */
 	bool units_pr_degree;	/* true if scale is given as inch (or cm)/degree.  false for 1:xxxxx */
 	bool north_pole;		/* true if projection is on northern hemisphere, false on southern */
 	bool edge[4];		/* true if the edge is a map boundary */
@@ -250,6 +259,7 @@ struct GMT_PROJ {
 	bool GMT_convert_latitudes;	/* true if using spherical code with authalic/conformal latitudes */
 	bool inv_coordinates;	/* true if -fp[unit] was given and we must first recover lon,lat during reading */
 	bool N_hemi;		/* true if we only allow northern hemisphere oblique Mercator poles */
+	bool sph_inside;	/* true is we must use the spherical in-on-out machinery [false = Cartesian] */
 	unsigned int n_antipoles;	/* Number of antipole coordinates so far [used for -JE only] */
 	struct GMT_LATSWAP_CONSTS lat_swap_vals;
 
@@ -378,27 +388,27 @@ struct GMT_PROJ {
 
 	double v_r, v_ir;
 
-        /* General Perspective parameters */
-        double g_H, g_R;
-        double g_P, g_P_inverse;
-        double g_lon0;
-        double g_sphi1, g_cphi1;
-        double g_phig, g_sphig, g_cphig;
-        double g_sdphi, g_cdphi;
-        double g_B, g_D, g_L, g_G, g_J;
-        double g_BLH, g_DG, g_BJ, g_DHJ, g_LH2, g_HJ;
-        double g_sin_tilt, g_cos_tilt;
-        double g_azimuth, g_sin_azimuth, g_cos_azimuth;
-        double g_sin_twist, g_cos_twist;
-        double g_width;
-        double g_yoffset;
-        double g_rmax;
-        double g_max_yt;
-        double g_xmin, g_xmax;
-        double g_ymin, g_ymax;
+	/* General Perspective parameters */
+	double g_H, g_R;
+	double g_P, g_P_inverse;
+	double g_lon0;
+	double g_sphi1, g_cphi1;
+	double g_phig, g_sphig, g_cphig;
+	double g_sdphi, g_cdphi;
+	double g_B, g_D, g_L, g_G, g_J;
+	double g_BLH, g_DG, g_BJ, g_DHJ, g_LH2, g_HJ;
+	double g_sin_tilt, g_cos_tilt;
+	double g_azimuth, g_sin_azimuth, g_cos_azimuth;
+	double g_sin_twist, g_cos_twist;
+	double g_width;
+	double g_yoffset;
+	double g_rmax;
+	double g_max_yt;
+	double g_xmin, g_xmax;
+	double g_ymin, g_ymax;
 
-        unsigned int g_debug;
-        int g_box, g_outside, g_longlat_set, g_sphere, g_radius, g_auto_twist;
+	unsigned int g_debug;
+	int g_box, g_outside, g_longlat_set, g_sphere, g_radius, g_auto_twist;
 	bool windowed;
 	
 	/* Polar (cylindrical) projection */
@@ -406,6 +416,9 @@ struct GMT_PROJ {
 	double p_base_angle;
 	bool got_azimuths, got_elevations, z_down;
 
+	/* PROJ4 variables */
+	double proj4_x0, proj4_y0, proj4_scl;
+	bool is_proj4;
 };
 
 enum GMT_enum_frame {GMT_IS_PLAIN = 0,	/* Plain baseframe */
@@ -419,7 +432,7 @@ enum GMT_enum_frame {GMT_IS_PLAIN = 0,	/* Plain baseframe */
 enum GMT_enum_tick {GMT_ANNOT_UPPER = 0,	/* Tick annotations closest to the axis */
 	GMT_ANNOT_LOWER,	/* Tick annotations farthest from the axis*/
 	GMT_TICK_UPPER,		/* Frame tick marks closest to the axis */
-	GMT_TICK_LOWER,		/* Frame tick marks closest to the axis */
+	GMT_TICK_LOWER,		/* Frame tick marks farthest to the axis */
 	GMT_GRID_UPPER,		/* Gridline spacing */
 	GMT_GRID_LOWER};	/* Gridline spacing */
 
@@ -463,11 +476,15 @@ struct GMT_PLOT_AXIS {		/* Information for one time axis */
 	unsigned int type;		/* GMT_LINEAR, GMT_LOG10, GMT_POW, GMT_TIME */
 	unsigned int special;		/* See gmt_enum_annot values */
 	unsigned int label_mode;	/* 0 = parallel to all axes, 1 = always horizontal on map */
-	struct GMT_PLOT_AXIS_ITEM item[6];	/* see above defines for which is which */
+	bool substitute_pi;		/* True if we need to plot fractions of pi on this axis */
+	bool use_angle;			/* True if we got +a<angle>|n|p for this axis */
+	struct GMT_PLOT_AXIS_ITEM item[8];	/* see above defines for which is which */
 	double phase;			/* Phase offset for strides: (knot-phase)%interval = 0  */
-	char label[GMT_LEN256];	/* Label of the axis */
-	char unit[GMT_LEN64];	/* Axis unit appended to annotations */
-	char prefix[GMT_LEN64];	/* Axis prefix starting all annotations */
+	double angle;			/* Annotations angle set by user */
+	char label[GMT_LEN256];		/* Label of the axis */
+	char secondary_label[GMT_LEN256];	/* Optionally use this label when axis is right or top */
+	char unit[GMT_LEN64];		/* Axis unit appended to annotations */
+	char prefix[GMT_LEN64];		/* Axis prefix starting all annotations */
 	char *file_custom;		/* File with custom annotations */
 };
 
@@ -479,6 +496,7 @@ struct GMT_PLOT_FRAME {		/* Various parameters for plotting of time axis boundar
 	bool init;			/* true if -B was used at all */
 	bool set;			/* true if -B was used to set any increments */
 	bool draw;			/* true if -B<int> was used, even -B0, as sign to draw axes */
+	bool drawz;			/* true if -B<int> was used, even -Bz0, as sign to draw z axes */
 	bool paint;			/* true if -B +g<fill> was used */
 	bool draw_box;			/* true if a 3-D Z-box is desired */
 	bool no_frame;			/* true if we just want gridlines but no frame, i.e +n was used */
@@ -492,4 +510,4 @@ struct GMT_PLOT_FRAME {		/* Various parameters for plotting of time axis boundar
 	unsigned int z_axis[4];		/* Which axes to use for the 3-D z-axis [auto] */
 };
 
-#endif /* _GMT_PROJECT_H */
+#endif /* GMT_PROJECT_H */
