@@ -1,5 +1,4 @@
-#!/bin/bash
-#	$Id$
+#!/usr/bin/env bash
 #
 # These functions can be used from any sh/bash script by specifying
 # . gmt_shell_functions.sh
@@ -65,7 +64,7 @@ gmt_get_region() {
 
 #	Return the w/e/s/n from the header in grd file
 gmt_get_gridregion() {
-	printf "%s/%s/%s/%s\n" `gmt grdinfo -C $* | cut -d'	' -f2-5`
+	printf "%s/%s/%s/%s\n" `gmt grdinfo -Cn $* | cut -d'	' -f1-4`
 }
 
 # Make output PostScript file name based on script base name
@@ -121,19 +120,20 @@ EOF
 	EOF
 	ls kml/*.kml > /tmp/$$.lis
 	while read file; do
-		cat <<- EOF >> doc.kml
+	        name=`basename $file .kml`
+		cat << EOF >> doc.kml
 			<NetworkLink>
-	        	<name>$file</name>
+	        	<name>$name</name>
 	        	<Link>
 				<href> $file </href>
 			</Link>
 			</NetworkLink>
-		EOF
+EOF
 	done < /tmp/$$.lis
-	cat <<- EOF >> doc.kml
+	cat << EOF >> doc.kml
 		</Document>
 	</kml>
-	EOF
+EOF
 	zip -rq9 $name.kmz doc.kml kml
 	if [ $remove -eq 0 ]; then
 		mv -f kml/* ..
@@ -285,7 +285,7 @@ EOF
 	now=`date`
 	you=`finger $LOGNAME | head -1 | awk -F': ' '{print $3}'`
 	cat << EOF > $name.sh
-#!/bin/bash
+#!/usr/bin/env bash
 # Author:	$you
 # Purpose:	Make an animation of ...
 # Date:		$now
@@ -490,7 +490,7 @@ EOF
 	# Create n_cpu empty files for execution
 	let cpu=0
 	while [ $cpu -lt $n_cpu ]; do
-		printf "#!/bin/bash\n# gmt_launch_jobs command file chunk # ${cpu}\n#---------------------------------\n" > /tmp/gmt_launch_jobs${tag}.$cpu.sh
+		printf "#!/usr/bin/env bash\n# gmt_launch_jobs command file chunk # ${cpu}\n#---------------------------------\n" > /tmp/gmt_launch_jobs${tag}.$cpu.sh
 		let cpu=cpu+1
 	done
 	# Distribute $n_lines from the commands across these core scripts
