@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *	Copyright (c) 1991-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  *  Misc statistical and special functions.
@@ -127,7 +126,7 @@ GMT_LOCAL double gmtstat_ln_gamma (struct GMT_CTRL *GMT, double xx) {
 }
 
 GMT_LOCAL int gmtstat_ln_gamma_r (struct GMT_CTRL *GMT, double x, double *lngam) {
-	/* Get natural logrithm of Gamma(x), x > 0.
+	/* Get natural logarithm of Gamma(x), x > 0.
 		To maintain full accuracy, this
 		routine uses Gamma(1 + x) / x when
 		x < 1.  This routine in turn calls
@@ -166,7 +165,7 @@ GMT_LOCAL void gmtstat_gamma_ser (struct GMT_CTRL *GMT, double *gamser, double a
 	}
 
 	if (x < 0.0) {
-		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "GMT DOMAIN ERROR:  x < 0 in gmtstat_gamma_ser(x)\n");
+		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "x < 0 in gmtstat_gamma_ser(x)\n");
 		*gamser = GMT->session.d_NaN;
 		return;
 	}
@@ -185,7 +184,7 @@ GMT_LOCAL void gmtstat_gamma_ser (struct GMT_CTRL *GMT, double *gamser, double a
 	 		return;
 	 	}
 	}
-	GMT_Report (GMT->parent, GMT_MSG_NORMAL, "GMT DOMAIN ERROR:  a too large, ITMAX too small in gmtstat_gamma_ser(x)\n");
+	GMT_Report (GMT->parent, GMT_MSG_NORMAL, "a too large, ITMAX too small in gmtstat_gamma_ser(x)\n");
 }
 
 GMT_LOCAL void gmtstat_gamma_cf (struct GMT_CTRL *GMT, double *gammcf, double a, double x, double *gln) {
@@ -219,7 +218,7 @@ GMT_LOCAL void gmtstat_gamma_cf (struct GMT_CTRL *GMT, double *gammcf, double a,
 			gold = g;
 		}
 	}
-	GMT_Report (GMT->parent, GMT_MSG_NORMAL, "GMT DOMAIN ERROR:  a too large, ITMAX too small in gmtstat_gamma_cf(x)\n");
+	GMT_Report (GMT->parent, GMT_MSG_NORMAL, "a too large, ITMAX too small in gmtstat_gamma_cf(x)\n");
 }
 
 GMT_LOCAL double gmtstat_gammq (struct GMT_CTRL *GMT, double a, double x) {
@@ -228,7 +227,7 @@ GMT_LOCAL double gmtstat_gammq (struct GMT_CTRL *GMT, double a, double x) {
 	double G = 0.0, gln;
 
 	if (x < 0.0 || a <= 0.0) {
-		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "GMT DOMAIN ERROR:  Invalid arguments to GMT_gammaq\n");
+		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Invalid arguments to GMT_gammaq\n");
 		return (GMT->session.d_NaN);
 	}
 
@@ -411,7 +410,7 @@ GMT_LOCAL int gmtstat_f_test_new (struct GMT_CTRL *GMT, double chisq1, uint64_t
 
 	if (chisq1 <= 0.0 || chisq2 <= 0.0 || nu1 < 1 || nu2 < 1) {
 		*prob = GMT->session.d_NaN;
-		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "gmtstat_f_test_new: Error: Bad argument(s).\n");
+		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "gmtstat_f_test_new: Bad argument(s).\n");
 		return (-1);
 	}
 
@@ -433,7 +432,7 @@ GMT_LOCAL double gmtstat_factln (struct GMT_CTRL *GMT, int n) {
 	/* returns log(n!) */
 	static double a[101];	/* Automatically initialized to zero */
 	if (n < 0) {
-		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Error: n < 0 in gmtstat_factln(n)\n");
+		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "n < 0 in gmtstat_factln(n)\n");
 		return (GMT->session.d_NaN);
 	}
 	if (n <= 1) return 0.0;
@@ -746,7 +745,7 @@ double gmt_ker (struct GMT_CTRL *GMT, double x) {
 	double t, rxsq, alpha, beta;
 
 	if (x <= 0.0) {
-		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "GMT DOMAIN ERROR:  x <= 0 in gmt_ker(x)\n");
+		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "x <= 0 in gmt_ker(x)\n");
 		return (GMT->session.d_NaN);
 	}
 
@@ -786,7 +785,7 @@ double gmt_kei (struct GMT_CTRL *GMT, double x) {
 		/* Zero is valid.  If near enough to zero, return kei(0)  */
 		if (x > -GMT_CONV8_LIMIT) return (-0.25 * M_PI);
 
-		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "GMT DOMAIN ERROR:  x < 0 in gmt_kei(x)\n");
+		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "x < 0 in gmt_kei(x)\n");
 		return (GMT->session.d_NaN);
 	}
 
@@ -952,12 +951,12 @@ double gmt_plm (struct GMT_CTRL *GMT, int l, int m, double x) {
 
 	/* x is cosine of colatitude and must be -1 <= x <= +1 */
 	if (fabs(x) > 1.0) {
-		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Error: |x| > 1.0 in gmt_plm\n");
+		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "|x| > 1.0 in gmt_plm\n");
 		return (GMT->session.d_NaN);
 	}
 
 	if (m < 0 || m > l) {
-		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Error: gmt_plm requires 0 <= m <= l.\n");
+		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "gmt_plm requires 0 <= m <= l.\n");
 		return (GMT->session.d_NaN);
 	}
 
@@ -1027,7 +1026,7 @@ double gmt_plm_bar (struct GMT_CTRL *GMT, int l, int m, double x, bool ortho) {
 	/* x is cosine of colatitude (sine of latitude) and must be -1 <= x <= +1 */
 
 	if (fabs (x) > 1.0) {
-		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Error: |x| > 1.0 in gmt_plm_bar\n");
+		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "|x| > 1.0 in gmt_plm_bar\n");
 		return (GMT->session.d_NaN);
 	}
 
@@ -1039,7 +1038,7 @@ double gmt_plm_bar (struct GMT_CTRL *GMT, int l, int m, double x, bool ortho) {
 	}
 
 	if (m > l) {
-		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Error: gmt_plm_bar requires 0 <= m <= l.\n");
+		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "gmt_plm_bar requires 0 <= m <= l.\n");
 		return (GMT->session.d_NaN);
 	}
 
@@ -1136,7 +1135,7 @@ void gmt_plm_bar_all (struct GMT_CTRL *GMT, int lmax, double x, bool ortho, doub
 	/* x is cosine of colatitude (sine of latitude) and must be -1 <= x <= +1 */
 
 	if (fabs (x) > 1.0) {
-		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Error: |x| > 1.0 in gmt_plm_bar_all\n");
+		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "|x| > 1.0 in gmt_plm_bar_all\n");
 		return;
 	}
 
@@ -1215,7 +1214,7 @@ double gmt_factorial (struct GMT_CTRL *GMT, int n) {
 	int i;
 
 	if (n < 0) {
-		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Error: n < 0 in gmt_factorial(n)\n");
+		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "n < 0 in gmt_factorial(n)\n");
 		return (GMT->session.d_NaN);
 		/* This could be set to return 0 without warning, to facilitate
 			sums over binomial coefficients, if desired.  -whfs  */
@@ -1231,7 +1230,7 @@ double gmt_factorial (struct GMT_CTRL *GMT, int n) {
 double gmt_permutation (struct GMT_CTRL *GMT, int n, int r) {
 	/* Compute Permutations n_P_r */
 	if (n < 0 || r < 0 || r > n) {
-		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Error: n < 0 or r < 0 or r > n in gmt_permutation(n,r)\n");
+		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "n < 0 or r < 0 or r > n in gmt_permutation(n,r)\n");
 		return (GMT->session.d_NaN);
 	}
 	return (floor (0.5 + exp (gmtstat_factln (GMT, n) - gmtstat_factln (GMT, n-r))));
@@ -1240,7 +1239,7 @@ double gmt_permutation (struct GMT_CTRL *GMT, int n, int r) {
 double gmt_combination (struct GMT_CTRL *GMT, int n, int r) {
 	/* Compute Combinations n_C_r */
 	if (n < 0 || r < 0 || r > n) {
-		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Error: n < 0 or r < 0 or r > n in gmt_combination(n,r)\n");
+		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "n < 0 or r < 0 or r > n in gmt_combination(n,r)\n");
 		return (GMT->session.d_NaN);
 	}
 	return (floor (0.5 + exp (gmtstat_factln (GMT, n) - gmtstat_factln (GMT, r) - gmtstat_factln (GMT, n-r))));
@@ -1343,7 +1342,7 @@ double gmt_f_pdf (struct GMT_CTRL *GMT, double F, uint64_t nu1, uint64_t nu2) {
 	/* Probability density distribution for F */
 	double y;
 
-	y = sqrt (pow (nu1 * F, (double)nu1) * pow ((double)nu2, (double)nu2) / pow (nu1 * F + nu2, (double)(nu1+nu2))) / (F * gmtstat_beta (GMT, 0.5*nu1, 0.5*nu2));
+	y = (F < GMT_CONV8_LIMIT) ? 0.0 : sqrt (pow (nu1 * F, (double)nu1) * pow ((double)nu2, (double)nu2) / pow (nu1 * F + nu2, (double)(nu1+nu2))) / (F * gmtstat_beta (GMT, 0.5*nu1, 0.5*nu2));
 	return (y);
 }
 
@@ -1351,6 +1350,7 @@ double gmt_f_cdf (struct GMT_CTRL *GMT, double F, uint64_t nu1, uint64_t nu2) {
 	/* Cumulative probability density distribution for F */
 	double y = 0.0;
 
+	if (F < GMT_CONV8_LIMIT) return 0.0;
 	gmtstat_inc_beta (GMT, 0.5*nu1, 0.5*nu2, F*nu1/(F*nu1+nu2), &y);
 
 	return (y);
@@ -1804,7 +1804,7 @@ int gmt_median (struct GMT_CTRL *GMT, double *x, uint64_t n, double xmin, double
 			m_guess = (temp < glb) ? temp : glb;	/* Move guess at least to glb  */
 		}
 		else {	/* If we get here, I made a mistake!  */
-			GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Error: Internal goof in gmt_median; please report to developers!\n");
+			GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Internal goof in gmt_median; please report to developers!\n");
 			GMT_exit (GMT, GMT_RUNTIME_ERROR); return GMT_RUNTIME_ERROR;
 		}
 
@@ -1960,7 +1960,7 @@ int gmt_mode (struct GMT_CTRL *GMT, double *x, uint64_t n, uint64_t j, bool sort
 	return (0);
 }
 
-int gmt_mode_f (struct GMT_CTRL *GMT, float *x, uint64_t n, uint64_t j, bool sort, int mode_selection, unsigned int *n_multiples, double *mode_est) {
+int gmt_mode_f (struct GMT_CTRL *GMT, gmt_grdfloat *x, uint64_t n, uint64_t j, bool sort, int mode_selection, unsigned int *n_multiples, double *mode_est) {
 	uint64_t i, istop;
 	unsigned int multiplicity;
 	double mid_point_sum = 0.0, length, short_length = FLT_MAX, this_mode;
@@ -2044,9 +2044,9 @@ void gmt_getmad (struct GMT_CTRL *GMT, double *x, uint64_t n, double location, d
 	*scale = MAD_NORMALIZE * med;
 }
 
-void gmt_getmad_f (struct GMT_CTRL *GMT, float *x, uint64_t n, double location, double *scale) {
+void gmt_getmad_f (struct GMT_CTRL *GMT, gmt_grdfloat *x, uint64_t n, double location, double *scale) {
 	uint64_t i;
-	float *dev = NULL;
+	gmt_grdfloat *dev = NULL;
 	double med;
 
 	if (n == 0) {	/* No data, so cannot define MAD */
@@ -2058,7 +2058,7 @@ void gmt_getmad_f (struct GMT_CTRL *GMT, float *x, uint64_t n, double location,
 		return;
 	}
 	dev = gmt_M_memory (GMT, NULL, n, double);
-	for (i = 0; i < n; i++) dev[i] = (float) fabs (x[i] - location);
+	for (i = 0; i < n; i++) dev[i] = (gmt_grdfloat) fabs (x[i] - location);
 	gmt_sort_array (GMT, dev, n, GMT_FLOAT);
 	for (i = n; i > 0 && gmt_M_is_fnan (dev[i-1]); i--);
 	if (i)
@@ -2071,9 +2071,9 @@ void gmt_getmad_f (struct GMT_CTRL *GMT, float *x, uint64_t n, double location,
 
 double gmt_extreme (struct GMT_CTRL *GMT, double x[], uint64_t n, double x_default, int kind, int way) {
 	/* Returns the extreme value in the x array according to:
-	*  kind: -1 means only consider negative values.
+	*  kind: -1 means only consider negative values, including zero.
 	*  kind:  0 means consider all values.
-	*  kind: +1 means only consider positive values.
+	*  kind: +1 means only consider positive values, including zero.
 	*  way:  -1 means look for minimum.
 	*  way:  +1 means look for maximum.
 	* If kind is non-zero we assign x_default is no values are found.
@@ -2166,7 +2166,7 @@ double gmt_corrcoeff (struct GMT_CTRL *GMT, double *x, double *y, uint64_t n, un
 	return (r);
 }
 
-double gmt_corrcoeff_f (struct GMT_CTRL *GMT, float *x, float *y, uint64_t n, unsigned int mode) {
+double gmt_corrcoeff_f (struct GMT_CTRL *GMT, gmt_grdfloat *x, gmt_grdfloat *y, uint64_t n, unsigned int mode) {
 	/* Returns plain correlation coefficient, r.
 	 * If mode = 1 we assume mean(x) = mean(y) = 0.
 	 */
@@ -2222,7 +2222,7 @@ double gmt_quantile (struct GMT_CTRL *GMT, double *x, double q, uint64_t n) {
 	return (p);
 }
 
-double gmt_quantile_f (struct GMT_CTRL *GMT, float *x, double q, uint64_t n) {
+double gmt_quantile_f (struct GMT_CTRL *GMT, gmt_grdfloat *x, double q, uint64_t n) {
 	/* Returns the q'th (q in percent) quantile of x (assumed sorted).
 	 * q is expected to be 0 < q < 100 */
 
@@ -2490,7 +2490,7 @@ double gmt_grd_mean (struct GMT_CTRL *GMT, struct GMT_GRID *G, struct GMT_GRID *
 			sum_zw += G->data[node];
 			n++;
 		}
-		sum_w = n;
+		sum_w = (double)n;
 	}
 	return (n == 0 || sum_w == 0.0) ? GMT->session.d_NaN : sum_zw / sum_w;
 }
@@ -2545,7 +2545,7 @@ double gmt_grd_rms (struct GMT_CTRL *GMT, struct GMT_GRID *G, struct GMT_GRID *W
 			n++;
 			sum_z2w += (G->data[node] * G->data[node]);
 		}
-		sum_w = n;
+		sum_w = (double)n;
 	}
 	rms = (sum_w > 0) ? sqrt (sum_z2w / sum_w) : GMT->session.d_NaN;
 	return (rms);
@@ -2600,7 +2600,7 @@ double gmt_grd_mad (struct GMT_CTRL *GMT, struct GMT_GRID *G, struct GMT_GRID *W
 			gmt_M_grd_loop (GMT, G, row, col, node) {
 				if (gmt_M_is_fnan (G->data[node]) || gmt_M_is_dnan (W->data[node]))
 					continue;
-				pair[n].value    = (float)fabs (G->data[node] - wmed);
+				pair[n].value    = (gmt_grdfloat)fabs (G->data[node] - wmed);
 				pair[n++].weight = W->data[node];
 			}
 		}
@@ -2615,7 +2615,7 @@ double gmt_grd_mad (struct GMT_CTRL *GMT, struct GMT_GRID *G, struct GMT_GRID *W
 			/* 2. Find the weighted median */
 			wmed = gmt_median_weighted (GMT, pair, n);
 			/* 3. Compute the absolute deviations from this median */
-			for (node = 0; node < n; node++) pair[node].value = (float)fabs (pair[node].value - wmed);
+			for (node = 0; node < n; node++) pair[node].value = (gmt_grdfloat)fabs (pair[node].value - wmed);
 		}
 		/* 4. Find the weighted median absolue deviation */
 		wmad = MAD_NORMALIZE * gmt_median_weighted (GMT, pair, n);
@@ -2659,7 +2659,7 @@ double gmt_grd_mode (struct GMT_CTRL *GMT, struct GMT_GRID *G, struct GMT_GRID *
 			pair[n++].weight = W->data[node];
 		}
 		/* 2. Find the weighted lms mode */
-		wmode = (float)gmt_mode_weighted (GMT, pair, n);
+		wmode = (gmt_grdfloat)gmt_mode_weighted (GMT, pair, n);
 		gmt_M_free (GMT, pair);
 	}
 	else {	/* Plain median */
@@ -2669,11 +2669,11 @@ double gmt_grd_mode (struct GMT_CTRL *GMT, struct GMT_GRID *G, struct GMT_GRID *
 		gmt_sort_array (GMT, Z->data, Z->header->nm, GMT_FLOAT);
 		for (n = Z->header->nm; n > 1 && gmt_M_is_fnan (Z->data[n-1]); n--);
 		if (n)
-			gmt_mode_f (GMT, G->data, n, n/2, 0, gmt_mode_selection, &GMT_n_multiples, &wmode);
+			gmt_mode_f (GMT, Z->data, n, n/2, 0, gmt_mode_selection, &GMT_n_multiples, &wmode);
 		else
 			wmode = GMT->session.d_NaN;
 		if (!overwrite) gmt_free_grid (GMT, &Z, true);
-		if (GMT_n_multiples > 0) GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Warning: %d Multiple modes found in the grid\n", GMT_n_multiples);
+		if (GMT_n_multiples > 0) GMT_Report (GMT->parent, GMT_MSG_VERBOSE, "%d Multiple modes found in the grid\n", GMT_n_multiples);
 	}
 	return wmode;
 }
@@ -2693,7 +2693,7 @@ double gmt_grd_lmsscl (struct GMT_CTRL *GMT, struct GMT_GRID *G, struct GMT_GRID
 			gmt_M_grd_loop (GMT, G, row, col, node) {
 				if (gmt_M_is_fnan (G->data[node]) || gmt_M_is_dnan (W->data[node]))
 					continue;
-				pair[n].value    = (float)fabs (G->data[node] - wmode);
+				pair[n].value    = (gmt_grdfloat)fabs (G->data[node] - wmode);
 				pair[n++].weight = W->data[node];
 			}
 		}
@@ -2706,9 +2706,9 @@ double gmt_grd_lmsscl (struct GMT_CTRL *GMT, struct GMT_GRID *G, struct GMT_GRID
 				pair[n++].weight = W->data[node];
 			}
 			/* 2. Find the weighted median */
-			wmode = (float)gmt_mode_weighted (GMT, pair, n);
+			wmode = (gmt_grdfloat)gmt_mode_weighted (GMT, pair, n);
 			/* 3. Compute the absolute deviations from this mode */
-			for (node = 0; node < n; node++) pair[node].value = (float)fabs (pair[node].value - wmode);
+			for (node = 0; node < n; node++) pair[node].value = (gmt_grdfloat)fabs (pair[node].value - wmode);
 		}
 		/* 4. Find the weighted median absolue deviation and scale it */
 		lmsscl = MAD_NORMALIZE * gmt_median_weighted (GMT, pair, n);
@@ -2726,13 +2726,13 @@ double gmt_grd_lmsscl (struct GMT_CTRL *GMT, struct GMT_GRID *G, struct GMT_GRID
 			gmt_sort_array (GMT, Z->data, Z->header->nm, GMT_FLOAT);
 			for (n = Z->header->nm; n > 1 && gmt_M_is_fnan (Z->data[n-1]); n--);
 			if (n)
-				gmt_mode_f (GMT, G->data, n, n/2, 0, gmt_mode_selection, &GMT_n_multiples, &wmode);
+				gmt_mode_f (GMT, Z->data, n, n/2, 0, gmt_mode_selection, &GMT_n_multiples, &wmode);
 			else
 				wmode = GMT->session.d_NaN;
 		}
-		gmt_getmad_f (GMT, G->data, n, wmode, &lmsscl);
+		gmt_getmad_f (GMT, Z->data, n, wmode, &lmsscl);
 		if (!overwrite) gmt_free_grid (GMT, &Z, true);
-		if (GMT_n_multiples > 0) GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Warning: %d Multiple modes found in the grid\n", GMT_n_multiples);
+		if (GMT_n_multiples > 0) GMT_Report (GMT->parent, GMT_MSG_VERBOSE, "%d Multiple modes found in the grid\n", GMT_n_multiples);
 	}
 	return lmsscl;
 }
@@ -2763,7 +2763,7 @@ GMT_LOCAL void get_geo_cellarea (struct GMT_CTRL *GMT, struct GMT_GRID *G) {
 		row_weight = 1.0 - cosd (f * G->header->inc[GMT_Y]);
 		gmt_M_col_loop (GMT, G, first_row, col, node) {
 			if (G->header->registration == GMT_GRID_NODE_REG) col_weight = (col == 0 || col == last_col) ? 0.5 : 1.0;
-			G->data[node] = (float)(row_weight * col_weight * area);
+			G->data[node] = (gmt_grdfloat)(row_weight * col_weight * area);
 		}
 		first_row++;
 	}
@@ -2771,7 +2771,7 @@ GMT_LOCAL void get_geo_cellarea (struct GMT_CTRL *GMT, struct GMT_GRID *G) {
 		row_weight = 1.0 - cosd (f * G->header->inc[GMT_Y]);
 		gmt_M_col_loop (GMT, G, last_row, col, node) {
 			if (G->header->registration == GMT_GRID_NODE_REG) col_weight = (col == 0 || col == last_col) ? 0.5 : 1.0;
-			G->data[node] = (float)(row_weight * col_weight * area);
+			G->data[node] = (gmt_grdfloat)(row_weight * col_weight * area);
 		}
 		last_row--;
 	}
@@ -2783,7 +2783,7 @@ GMT_LOCAL void get_geo_cellarea (struct GMT_CTRL *GMT, struct GMT_GRID *G) {
 		row_weight = cosd (lat);
 		gmt_M_col_loop (GMT, G, row, col, node) {	/* Loop over cols; always save the next left before we update the array at that col */
 			if (G->header->registration == GMT_GRID_NODE_REG) col_weight = (col == 0 || col == last_col) ? 0.5 : 1.0;
-			G->data[node] = (float)(row_weight * col_weight * area);
+			G->data[node] = (gmt_grdfloat)(row_weight * col_weight * area);
 		}
 	}
 }
@@ -2798,7 +2798,7 @@ GMT_LOCAL void get_cart_cellarea (struct GMT_CTRL *GMT, struct GMT_GRID *G) {
 		if (G->header->registration == GMT_GRID_NODE_REG) row_weight = (row == 0 || row == last_row) ? 0.5 : 1.0;	/* half-cells in y */
 		gmt_M_col_loop (GMT, G, row, col, node) {	/* Now loop over the columns */
 			if (G->header->registration == GMT_GRID_NODE_REG) col_weight = (col == 0 || col == last_col) ? 0.5 : 1.0;	/* half-cells in x */
-			G->data[node] = (float)(row_weight * col_weight * area);
+			G->data[node] = (gmt_grdfloat)(row_weight * col_weight * area);
 		}
 	}
 }
