@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt_texture.h 19521 2017-12-27 22:38:24Z pwessel $
  *
- *	Copyright (c) 1991-2018 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,14 +12,14 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  * gmt_texture.h contains definitions of structures for pens, fills, and fonts.
  *
  * Author:	Paul Wessel
  * Date:	01-OCT-2009
- * Version:	5 API
+ * Version:	6 API
  */
 
 /*!
@@ -28,8 +27,12 @@
  * \brief Definitions of structures for pens, fills, and fonts.
  */
 
-#ifndef _GMT_TEXTURE_H
-#define _GMT_TEXTURE_H
+#ifndef GMT_TEXTURE_H
+#define GMT_TEXTURE_H
+
+#ifndef PATH_MAX
+#	define PATH_MAX 1024
+#endif
 
 /*--------------------------------------------------------------------
  *			GMT TEXTURE STRUCTURE DEFINITIONS
@@ -66,7 +69,10 @@ struct GMT_FILL {
 	bool use_pattern;		/* true if pattern rather than rgb is set */
 	int pattern_no;			/* Number of a predefined pattern, or -1 if not set */
 	unsigned int dpi;		/* Desired dpi of image building-block if use_pattern is true */
-	char pattern[GMT_BUFSIZ];	/* Full filename of user-defined raster pattern */
+	unsigned int dim[3];		/* width, height, depth of image */
+	char pattern[PATH_MAX];		/* Full filename of user-defined raster pattern */
+	unsigned char *image;	/* Pointer to image array */
+	struct GMT_IMAGE *I;	/* The image container */
 };
 
 /*! Holds font attributes */
@@ -74,6 +80,7 @@ struct GMT_FONT {
 	double size;			/* Font size in points */
 	unsigned int id;		/* Font ID number from predefined list */
 	unsigned int form;		/* Combination of binary 1 = fill, 2 = outline, 4 = pattern fill [1] */
+	unsigned int set;		/* 1 if font fill set, 2 if font output pen set, 3 if both, 0 for default */
 	struct GMT_FILL fill;		/* Font fill [black] */
 	struct GMT_PEN pen;		/* Font outline pen [none] */
 };
@@ -92,4 +99,4 @@ struct GMT_MEDIA {
 	double height;		/* Height in points */
 };
 
-#endif  /* _GMT_TEXTURE_H */
+#endif  /* GMT_TEXTURE_H */
