@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *	Copyright (c) 1991-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,14 +12,14 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  * gmt_types.h contains definitions of special types used by GMT.
  *
  * Author:	Paul Wessel
  * Date:	01-OCT-2009
- * Version:	5 API
+ * Version:	6 API
  */
 
 /*!
@@ -28,8 +27,8 @@
  * \brief Definitions of special types used by GMT.
  */
 
-#ifndef _GMT_TYPES_H
-#define _GMT_TYPES_H
+#ifndef GMT_TYPES_H
+#define GMT_TYPES_H
 #include <stdbool.h>
 #include <stdint.h>
 
@@ -44,6 +43,14 @@ struct GMT_MATH_MACRO {
 	char **arg;	/* List of those commands */
 };
 
+#ifdef USE_GMT_KWD
+/*! Definition of GMT_KW_DICT used to expand keyword-value pairs to GMT options */
+struct GMT_KW_DICT {	/* Used for keyword-value lookup */
+	char code;	/* Single character GMT option code */
+	char name[31];	/* Name of corresponding keyword */
+};
+#endif
+
 /*! Definition of structure use for finding optimal n_columns/n_rows for surface */
 struct GMT_SURFACE_SUGGESTION {	/* Used to find top ten list of faster grid dimensions  */
 	unsigned int n_columns;
@@ -72,8 +79,90 @@ struct GMT_TEXT_SELECTION {	/* Used to hold array with items (0-n) that have bee
 
 /*! For weighted mean/mode */
 struct GMT_OBSERVATION {
-	float value;
-	float weight;
+	gmt_grdfloat value;
+	gmt_grdfloat weight;
+};
+
+/*! For finding tightest west/east range of multiple items */
+struct GMT_RANGE {
+	double west;
+	double east;
+	double center;	/* Forced to be 0-360 */
+};
+
+/*! For information on 1-D array */
+
+struct GMT_ARRAY {	/* Used by modules that needs to set up 1-D output/bin arrays */
+	bool temporal;	/* true if array will be in absolute time */ 
+	bool vartime;	/* true if <unit> implies a variable calendar unit */ 
+	bool count;	/* true if we got number of items rather than increment */
+	bool add;	/* true if we are asked to add a computed spatial distance column to output */
+	bool reverse;	/* true if we want to reverse the array to give high to low on output */
+	bool round;	/* true if we want to adjust increment to ensure min/max range is a multiple of inc */
+	bool exact_inc;	/* true if we want the increment to be exact and to adjust min/max instead */
+	bool logarithmic;	/* true if inc = 1,2,3 and we want logarithmic scale */
+	bool logarithmic2;	/* true if inc = integer and we want log2 scale */
+	bool delay[2];	/* true if min and/or max shall be set from data set extremes after read [false] */
+	unsigned int spatial;	/* 1 if <unit> implies a Cartesian and 2 if a geospatial distance */ 
+	unsigned int set;	/* 1 if inc set, 3 if min/max/in set, 0 otherwise */
+	unsigned int col;	/* The column that this array goes with */
+	int distmode;	/* Type of geospatial calculation mode for distances */
+	uint64_t n;	/* Number of elements in the array when complete */
+	double min, max, inc;	/* Equidistant items */
+	double *array;	/* This will eventually hold the array */
+	char *file;	/* In case a file is given with the values */
+	char *list;	/* In case a comma-separated list of values */
+	char unit;	/* To remind us what units the inc is in, if given */
+};
+
+/*! For keeping table,segment IDs in a 1-D array */
+struct GMT_TBLSEG {
+	uint64_t tbl, seg;
+};
+
+/*! For keeping track of GMT figures under modern mode */
+struct GMT_FIGURE {
+	int ID;	/* Figure number [0] */
+	char prefix[GMT_LEN256];	/* File prefix (no extension) */
+	char formats[GMT_LEN64];	/* List of comma-separated extensions(formats) */
+	char options[GMT_LEN256];	/* Optional arguments to psconvert (e.g., -A, -E, ...) */
+};
+
+struct GMT_INSET {
+	bool active;	/* true the first time we set up scaling for a map inset */
+	bool first;	/* true the first time we plot into the map inset */
+	double w, h;	/* Width and height of current inset */
+	double dx, dy;	/* offsets */
+};
+
+/*! For keeping track of GMT subplots under modern mode */
+struct GMT_SUBPLOT {
+	unsigned int active;	/* 1 if subplot is in effect */
+	unsigned int first;		/* 1 the first time we reach panel, 0 later */
+	unsigned int no_scaling;	/* 1 when we are plotting a scale, bar, etc and not map and dont want to auto-scale plot */
+	unsigned int parallel;	/* 1 for axis-parallel annotations [0 for standard] */
+	int row, col;			/* Current panel position e.g., 0,0 */
+	int nrows, ncolumns;	/* Panel arrangement for subplot window */
+	int dir[2];				/* Cartesian axis direction: +1 or -1 [1/1] */
+	double x, y;			/* LB corner of current panel */
+	double dx, dy;			/* Offset from LB when projection rescaling is required to center */
+	double w, h;			/* Width and height of current panel */
+	double dim[2];			/* Dimension of entire subplot */
+	double origin[2];		/* Location of lower left figure origin set via -X -Y */
+	double off[2];			/* Offset from justification point of panel tag */
+	double clearance[4];		/* Space around text for surrounding textbox */
+	double gap[4];			/* Shrink plottable region to make space for enhancements */
+	char refpoint[3];		/* Reference point for panel tag */
+	char justify[3];		/* Justification relative to refpoint */
+	char tag[GMT_LEN128];		/* Panel tag, e.g., a) */
+	char fill[GMT_LEN64];		/* Panel fill color */
+	char pen[GMT_LEN64];		/* Panel tag pen outline */
+	char Baxes[GMT_LEN128];		/* The -B setting for selected axes, including +color, tec */
+	char Btitle[GMT_LEN128];	/* The -B setting for any title */
+	char Bxlabel[GMT_LEN128];	/* The -Bx setting for x labels */
+	char Bylabel[GMT_LEN128];	/* The -By setting for x labels */
+	char Bxannot[GMT_LEN32];	/* The -Bx setting for annotations */
+	char Byannot[GMT_LEN32];	/* The -By setting for annotations */
 };
 
 /*! For trend-fitting models */
@@ -136,6 +225,8 @@ struct GMT_MAP {		/* Holds all map-related parameters */
 	unsigned int n_lon_nodes;		/* Somewhat arbitrary # of nodes for lines in longitude (may be reset in gmt_map.c) */
 	unsigned int n_lat_nodes;		/* Somewhat arbitrary # of nodes for lines in latitude (may be reset in gmt_map.c) */
 	unsigned int path_mode;		/* 0 if we should call gmt_fix_up_path to resample across gaps > path_step, 1 to leave alone */
+	double last_width;			/* Full width in inches of previous plot */
+	double last_height;			/* Full height in inches of previous plot */
 	double width;				/* Full width in inches of this world map */
 	double height;				/* Full height in inches of this world map */
 	double half_width;			/* Half width in inches of this world map */
@@ -220,6 +311,7 @@ struct GMT_INIT { /* Holds misc run-time parameters */
 	char *runtime_library;        /* Name of the main shared library at run-time */
 	char *runtime_plugindir;      /* Directory that contains the main supplemental plugins at run-time */
 	char *history[GMT_N_UNIQUE];  /* The internal gmt.history information */
+	char cpt[GMT_N_CPT][GMT_LEN16];	/* The default CPTs for different data types; see gmt_cpt_default() */
 	struct GMT_CUSTOM_SYMBOL **custom_symbol; /* For custom symbol plotting in psxy[z]. */
 };
 
@@ -227,6 +319,7 @@ struct GMT_PLOT {		/* Holds all plotting-related parameters */
 	uint64_t n;			/* Number of such points */
 	size_t n_alloc;			/* Size of allocated plot arrays */
 	bool r_theta_annot;		/* true for special r-theta map annotation (see gmtlib_get_annot_label) */
+	bool substitute_pi;		/* true when -R or -B was given with pi and we want to use pi in annotations if possible */
 	unsigned int mode_3D;		/* Determines if we draw fore and/or back 3-D box lines [Default is both] */
 	unsigned int *pen;		/* Pen (PSL_MOVE = up, PSL_DRAW = down) for these points */
 	struct GMT_PLOT_CALCLOCK calclock;
@@ -234,6 +327,8 @@ struct GMT_PLOT {		/* Holds all plotting-related parameters */
 	double *x;			/* Holds the x/y (inches) of a line to be plotted */
 	double *y;
 	char format[3][2][GMT_LEN256];	/* Keeps the 6 formats for dd:mm:ss plot output */
+	struct GMT_SUBPLOT panel;	/* Current subplot panel settings */
+	struct GMT_INSET inset;		/* Current inset settings */
 };
 
 struct GMT_CURRENT {
@@ -260,7 +355,8 @@ struct GMT_INTERNAL {
 	/* These are internal parameters that need to be passed around between
 	 * many GMT functions.  These may change during execution but are not
 	 * modified directly by user interaction. */
-	unsigned int func_level;	/* Keeps track of what level in a nested GMT_func calling GMT_func etc we are.  0 is top function */
+	unsigned int func_level;	/* Keeps track of what level in a nested GMT_func calling GMT_func etc we are.  GMT_CONTROLLER (0) is initiating process (e.g. gmt.c) */
+	bool mem_set;			/* true when we have initialized the tmp memory already */
 	size_t mem_cols;		/* Current number of allocated columns for temp memory */
 	size_t mem_rows;		/* Current number of allocated rows for temp memory */
 	size_t mem_txt_alloc;
@@ -285,7 +381,7 @@ struct GMT_SESSION {
 	 * are essentially read-only constants for the duration of the session */
 	FILE *std[3];			/* Pointers for standard input, output, and error */
 	void * (*input_ascii) (struct GMT_CTRL *, FILE *, uint64_t *, int *);	/* Pointer to function reading ASCII tables only */
-	int (*output_ascii) (struct GMT_CTRL *, FILE *, uint64_t, double *);	/* Pointer to function writing ASCII tables only */
+	int (*output_ascii) (struct GMT_CTRL *, FILE *, uint64_t, double *, char *);	/* Pointer to function writing ASCII tables only */
 	unsigned int n_fonts;		/* Total number of fonts returned by gmtinit_init_fonts */
 	unsigned int n_user_media;	/* Total number of user media returned by gmtinit_load_user_media */
 	size_t min_meminc;		/* with -DMEMDEBUG, sets min/max memory increments */
@@ -294,7 +390,7 @@ struct GMT_SESSION {
 	double d_NaN;			/* Holds the IEEE NaN for doubles */
 	double no_rgb[4];		/* To hold {-1, -1, -1, 0} when needed */
 	double u2u[4][4];		/* u2u is the 4x4 conversion matrix for cm, inch, m, pt */
-	char unit_name[4][8];		/* Full name of the 4 units cm, inch, m, pt */
+	char unit_name[4][GMT_LEN8];	/* Full name of the 4 units cm, inch, m, pt */
 	struct GMT_HASH rgb_hashnode[GMT_N_COLOR_NAMES];/* Used to translate colornames to r/g/b */
 	bool rgb_hashnode_init;		/* true once the rgb_hashnode array has been loaded; false otherwise */
 	unsigned int n_shorthands;			/* Length of arrray with shorthand information */
@@ -302,10 +398,10 @@ struct GMT_SESSION {
 	int (*readinfo[GMT_N_GRD_FORMATS]) (struct GMT_CTRL *, struct GMT_GRID_HEADER *);	/* Pointers to grid read header functions */
 	int (*updateinfo[GMT_N_GRD_FORMATS]) (struct GMT_CTRL *, struct GMT_GRID_HEADER *);	/* Pointers to grid update header functions */
 	int (*writeinfo[GMT_N_GRD_FORMATS]) (struct GMT_CTRL *, struct GMT_GRID_HEADER *);	/* Pointers to grid write header functions */
-	int (*readgrd[GMT_N_GRD_FORMATS]) (struct GMT_CTRL *, struct GMT_GRID_HEADER *, float *, double *, unsigned int *, unsigned int);	/* Pointers to grid read functions */
-	int (*writegrd[GMT_N_GRD_FORMATS]) (struct GMT_CTRL *, struct GMT_GRID_HEADER *, float *, double *, unsigned int *, unsigned int);	/* Pointers to grid read functions */
-	int (*fft1d[k_n_fft_algorithms]) (struct GMT_CTRL *, float *, unsigned int, int, unsigned int);	/* Pointers to available 1-D FFT functions (or NULL if not configured) */
-	int (*fft2d[k_n_fft_algorithms]) (struct GMT_CTRL *, float *, unsigned int, unsigned int, int, unsigned int);	/* Pointers to available 2-D FFT functions (or NULL if not configured) */
+	int (*readgrd[GMT_N_GRD_FORMATS]) (struct GMT_CTRL *, struct GMT_GRID_HEADER *, gmt_grdfloat *, double *, unsigned int *, unsigned int);	/* Pointers to grid read functions */
+	int (*writegrd[GMT_N_GRD_FORMATS]) (struct GMT_CTRL *, struct GMT_GRID_HEADER *, gmt_grdfloat *, double *, unsigned int *, unsigned int);	/* Pointers to grid read functions */
+	int (*fft1d[k_n_fft_algorithms]) (struct GMT_CTRL *, gmt_grdfloat *, unsigned int, int, unsigned int);	/* Pointers to available 1-D FFT functions (or NULL if not configured) */
+	int (*fft2d[k_n_fft_algorithms]) (struct GMT_CTRL *, gmt_grdfloat *, unsigned int, unsigned int, int, unsigned int);	/* Pointers to available 2-D FFT functions (or NULL if not configured) */
 	/* This part contains pointers that may point to additional memory outside this struct */
 	char *DCWDIR;			/* Path to the DCW directory */
 	char *GSHHGDIR;			/* Path to the GSHHG directory */
@@ -316,6 +412,7 @@ struct GMT_SESSION {
 	char *DATADIR;			/* Path to one or more directories with data sets */
 	char *TMPDIR;			/* Path to the directory directory for isolation mode */
 	char *CUSTOM_LIBS;		/* Names of one or more comma-separated GMT-compatible shared libraries */
+	char *DATASERVER;		/* URL where to get remote @files */
 	char **user_media_name;		/* Length of array with custom media dimensions */
 	struct GMT_FONTSPEC *font;		/* Array with font names and height specification */
 	struct GMT_MEDIA *user_media;		/* Array with custom media dimensions */
@@ -351,4 +448,4 @@ static inline void GMT_exit (struct GMT_CTRL *GMT, int code) {
 		exit (code);
 }
 
-#endif  /* _GMT_TYPES_H */
+#endif  /* GMT_TYPES_H */
