@@ -1,5 +1,6 @@
 "B",
 "J",
+"J+",
 "JA",
 "Ja",
 "JB",
@@ -56,9 +57,11 @@
 "RG",
 "X",
 "Y",
+"Z",
 "a",
 "b",
-"c",
+"d",
+"e",
 "f",
 "g",
 "h",
