@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt_vector.c 19521 2017-12-27 22:38:24Z pwessel $
  *
- *	Copyright (c) 1991-2018 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 
 /*
@@ -312,7 +311,7 @@ GMT_LOCAL uint64_t vector_fix_up_path_cartonly (struct GMT_CTRL *GMT, double **a
 	x = *a_x;	y = *a_y;	/* Default is to return the input unchanged */
 	if (n < 2 || mode == 0) return n;		/* Nothing to do */
 
-	gmt_prep_tmp_arrays (GMT, 1, 2);	/* Init or reallocate two tmp vectors */
+	gmt_prep_tmp_arrays (GMT, GMT_NOTSET, 1, 2);	/* Init or reallocate two tmp vectors */
 	/* Start at first point */
 	GMT->hidden.mem_coord[GMT_X][0] = x[0];	GMT->hidden.mem_coord[GMT_Y][0] = y[0];
 
@@ -355,7 +354,7 @@ GMT_LOCAL uint64_t vector_fix_up_path_cartesian (struct GMT_CTRL *GMT, double **
 
 	x = *a_x;	y = *a_y;
 
-	gmt_prep_tmp_arrays (GMT, 1, 2);	/* Init or reallocate tmp vectors */
+	gmt_prep_tmp_arrays (GMT, GMT_NOTSET, 1, 2);	/* Init or reallocate tmp vectors */
 	GMT->hidden.mem_coord[GMT_X][0] = x[0];	GMT->hidden.mem_coord[GMT_Y][0] = y[0];	n_new = 1;
 	if (step <= 0.0) step = 1.0;	/* Sanity valve; if step not given we set it to 1 */
 
@@ -364,7 +363,7 @@ GMT_LOCAL uint64_t vector_fix_up_path_cartesian (struct GMT_CTRL *GMT, double **
 			n_step = lrint (fabs (x[i] - x[i-1]) / step);
 			for (j = 1; j < n_step; j++) {
 				c = j / (double)n_step;
-				gmt_prep_tmp_arrays (GMT, n_new, 2);	/* Init or reallocate tmp read vectors */
+				gmt_prep_tmp_arrays (GMT, GMT_NOTSET, n_new, 2);	/* Init or reallocate tmp read vectors */
 				GMT->hidden.mem_coord[GMT_X][n_new] = x[i-1] * (1 - c) + x[i] * c;
 				GMT->hidden.mem_coord[GMT_Y][n_new] = y[i-1];
 				n_new++;
@@ -372,7 +371,7 @@ GMT_LOCAL uint64_t vector_fix_up_path_cartesian (struct GMT_CTRL *GMT, double **
 			n_step = lrint (fabs (y[i]-y[i-1]) / step);
 			for (j = k; j < n_step; j++) {	/* Start at 0 to make sure corner point is saved */
 				c = j / (double)n_step;
-				gmt_prep_tmp_arrays (GMT, n_new, 2);	/* Init or reallocate tmp read vectors */
+				gmt_prep_tmp_arrays (GMT, GMT_NOTSET, n_new, 2);	/* Init or reallocate tmp read vectors */
 				GMT->hidden.mem_coord[GMT_X][n_new] = x[i];
 				GMT->hidden.mem_coord[GMT_Y][n_new] = y[i-1] * (1 - c) + y[i] * c;
 				n_new++;
@@ -383,7 +382,7 @@ GMT_LOCAL uint64_t vector_fix_up_path_cartesian (struct GMT_CTRL *GMT, double **
 			n_step = lrint (fabs (y[i]-y[i-1]) / step);
 			for (j = 1; j < n_step; j++) {
 				c = j / (double)n_step;
-				gmt_prep_tmp_arrays (GMT, n_new, 2);	/* Init or reallocate tmp read vectors */
+				gmt_prep_tmp_arrays (GMT, GMT_NOTSET, n_new, 2);	/* Init or reallocate tmp read vectors */
 				GMT->hidden.mem_coord[GMT_X][n_new] = x[i-1];
 				GMT->hidden.mem_coord[GMT_Y][n_new] = y[i-1] * (1 - c) + y[i] * c;
 				n_new++;
@@ -391,7 +390,7 @@ GMT_LOCAL uint64_t vector_fix_up_path_cartesian (struct GMT_CTRL *GMT, double **
 			n_step = lrint (fabs (x[i]-x[i-1]) / step);
 			for (j = k; j < n_step; j++) {	/* Start at 0 to make sure corner point is saved */
 				c = j / (double)n_step;
-				gmt_prep_tmp_arrays (GMT, n_new, 2);	/* Init or reallocate tmp read vectors */
+				gmt_prep_tmp_arrays (GMT, GMT_NOTSET, n_new, 2);	/* Init or reallocate tmp read vectors */
 				GMT->hidden.mem_coord[GMT_X][n_new] = x[i-1] * (1 - c) + x[i] * c;
 				GMT->hidden.mem_coord[GMT_Y][n_new] = y[i];
 				n_new++;
@@ -402,13 +401,13 @@ GMT_LOCAL uint64_t vector_fix_up_path_cartesian (struct GMT_CTRL *GMT, double **
 		else if ((n_step = lrint (hypot (x[i]-x[i-1], y[i]-y[i-1]) / step)) > 1) {	/* Must insert (n_step - 1) points, i.e. create n_step intervals */
 			for (j = 1; j < n_step; j++) {
 				c = j / (double)n_step;
-				gmt_prep_tmp_arrays (GMT, n_new, 2);	/* Init or reallocate tmp read vectors */
+				gmt_prep_tmp_arrays (GMT, GMT_NOTSET, n_new, 2);	/* Init or reallocate tmp read vectors */
 				GMT->hidden.mem_coord[GMT_X][n_new] = x[i-1] * (1 - c) + x[i] * c;
 				GMT->hidden.mem_coord[GMT_Y][n_new] = y[i-1] * (1 - c) + y[i] * c;
 				n_new++;
 			}
 		}
-		gmt_prep_tmp_arrays (GMT, n_new, 2);	/* Init or reallocate tmp read vectors */
+		gmt_prep_tmp_arrays (GMT, GMT_NOTSET, n_new, 2);	/* Init or reallocate tmp read vectors */
 		GMT->hidden.mem_coord[GMT_X][n_new] = x[i];	GMT->hidden.mem_coord[GMT_Y][n_new] = y[i];	n_new++;
 	}
 
@@ -995,7 +994,7 @@ int gmt_gaussjordan (struct GMT_CTRL *GMT, double *a, unsigned int nu, double *b
 }
 
 #ifndef __APPLE__	/* Since it is already declared in Accelerate.h */
-	extern int dsyev_ (char* jobz, char* uplo, int* n, double* a, int* lda, double* w, double* work, int* lwork, int* info);
+extern int dsyev_ (char* jobz, char* uplo, int* n, double* a, int* lda, double* w, double* work, int* lwork, int* info);
 #endif
 
 int gmt_svdcmp (struct GMT_CTRL *GMT, double *a, unsigned int m_in, unsigned int n_in, double *w, double *v) {
@@ -1006,7 +1005,7 @@ int gmt_svdcmp (struct GMT_CTRL *GMT, double *a, unsigned int m_in, unsigned int
 	double wkopt, *work = NULL;
 	gmt_M_unused(n_in);	/* Since we are actually only doing square matrices... */
 	gmt_M_unused(v);
-	GMT_Report (GMT->parent, GMT_MSG_VERBOSE, "gmt_svdcmp: Using Lapack dsyev\n");
+	GMT_Report (GMT->parent, GMT_MSG_LONG_VERBOSE, "gmt_svdcmp: Using Lapack dsyev\n");
 	/* Query and allocate the optimal workspace */
 	lwork = -1;
 	dsyev_ ( "Vectors", "Upper", &n, a, &lda, w, &wkopt, &lwork, &info );
@@ -1025,7 +1024,7 @@ int gmt_svdcmp (struct GMT_CTRL *GMT, double *a, unsigned int m_in, unsigned int
 	v = a;
 	return (GMT_NOERROR);
 #else
-	GMT_Report (GMT->parent, GMT_MSG_VERBOSE, "gmt_svdcmp: Using GMT's NR-based SVD\n");
+	GMT_Report (GMT->parent, GMT_MSG_LONG_VERBOSE, "gmt_svdcmp: Using GMT's NR-based SVD\n");
 	return vector_svdcmp_nr (GMT, a, m_in, n_in, w, v);
 #endif
 }
@@ -1040,8 +1039,11 @@ int gmt_svdcmp (struct GMT_CTRL *GMT, double *a, unsigned int m_in, unsigned int
 
 */
 
-int gmt_solve_svd (struct GMT_CTRL *GMT, double *u, unsigned int m, unsigned int nu, double *v, double *w, double *b, unsigned int k, double *x, double *cutoff, unsigned int mode) {
-	double w_abs, sing_max, total_variance, variance = 0.0, limit = 0;
+int gmt_solve_svd (struct GMT_CTRL *GMT, double *u, unsigned int m, unsigned int nu, double *v, double *w, double *b, unsigned int k, double *x, double cutoff, unsigned int mode) {
+	/* Mode = 0: Use all singular values s_j for which s_j/s_0 > cutoff [0 = all]
+	 * mode = 1: Use the first cutoff singular values only.
+	 */
+	double w_abs, sing_max;
 	int i, j, n_use = 0, n = (int)nu;	/* Because OpenMP cannot handle unsigned loop variables */
 	double s, *tmp = gmt_M_memory (GMT, NULL, n, double);
 	gmt_M_unused(m);	/* Since we are actually only doing square matrices... */
@@ -1049,22 +1051,23 @@ int gmt_solve_svd (struct GMT_CTRL *GMT, double *u, unsigned int m, unsigned int
 #ifdef HAVE_LAPACK
 	gmt_M_unused(v);	/* Not used when we solve via Lapack */
 #endif
-	GMT_Report (GMT->parent, GMT_MSG_VERBOSE, "gmt_solve_svd: Evaluate solution\n");
-	/* find maximum singular value and total variance.  Assumes w[] may have negative eigenvalues */
+	GMT_Report (GMT->parent, GMT_MSG_LONG_VERBOSE, "gmt_solve_svd: Evaluate solution\n");
+	/* find maximum singular value.  Assumes w[] may have negative eigenvalues */
 
 	sing_max = fabs (w[0]);
-	total_variance = w[0] * w[0];
 	for (i = 1; i < n; i++) {
-		total_variance += (w[i] * w[i]);
 		w_abs = fabs (w[i]);
 		sing_max = MAX (sing_max, w_abs);
 	}
-	GMT_Report (GMT->parent, GMT_MSG_VERBOSE, "gmt_solve_svd: Total variance = %g\n", total_variance);
+
+	if (cutoff > 0.0 && cutoff <= 1.0) {	/* Gave desired fraction of eigenvalues to use instead; scale to # of values */
+		double was = cutoff;
+		cutoff = rint (n*cutoff);
+		GMT_Report (GMT->parent, GMT_MSG_LONG_VERBOSE, "gmt_solve_svd: Given fraction %g corresponds to %d eigenvalues\n", was, irint(cutoff));
+	}
 
 	if (mode) {
-		/* mode = 1: Find the m largest singular values needed to explain the specified variance level.
-		 * mode = 2: Find the m largest singular values, with m = limit.
-		 * mode = 3: Find the m largest singular values needed so residual rms is ~limit.
+		 /* mode = 1: Find the m largest singular values, with m = cutoff (if <1 it is the fraction of values).
 		 * Either case requires sorted singular values so we need to do some work first.
 		 * It also assumes that the matrix passed is a squared normal equation kind of matrix
 		 * so that the singular values are the individual variace contributions. */
@@ -1080,12 +1083,9 @@ int gmt_solve_svd (struct GMT_CTRL *GMT, double *u, unsigned int m, unsigned int
 		}
 		qsort (eigen, n, sizeof (struct GMT_SINGULAR_VALUE), vector_compare_singular_values);
 		
-		if (mode == 1) limit = (*cutoff) * total_variance * 0.01;		/* Desired variance level given cutoff in % */
-		else if (mode == 2) n_eigen = (unsigned int)lrint (*cutoff);	/* Desired number of eigenvalues to use instead */
-		else if (mode == 3) limit = total_variance - n * (*cutoff)*(*cutoff);	/* Unexplained variance desired given stated data rms */
+		n_eigen = (unsigned int)lrint (cutoff);	/* Desired number of eigenvalues to use instead */
 		for (i = 0; i < n; i++) {	/* Visit all singular values in decreasing magnitude */
-			if ((mode == 1 && variance < limit) || (mode == 2 && i < n_eigen) || (mode == 3 && variance < limit)) {	/* Still within specified limit so we add this singular value */
-				variance += (eigen[i].value * eigen[i].value);
+			if (i < n_eigen) {	/* Still within specified limit so we add this singular value */
 				w[eigen[i].order] = 1.0 / w[eigen[i].order];
 				n_use++;
 			}
@@ -1094,12 +1094,10 @@ int gmt_solve_svd (struct GMT_CTRL *GMT, double *u, unsigned int m, unsigned int
 		}
 		gmt_M_free (GMT, eigen);
 	}
-	else {	/* Loop through singular values removing small ones (if limit is nonzero) */
-		limit = *cutoff;
+	else {	/* Loop through singular values removing small ones (if cutoff is nonzero) */
 		for (i = 0; i < n; i++) {
 			w_abs = fabs (w[i]);
-			if ((w_abs/sing_max) > limit) {
-				variance += (w[i] * w[i]);
+			if ((w_abs/sing_max) > cutoff) {
 				w[i] = 1.0 / w[i];
 				n_use++;
 			}
@@ -1107,24 +1105,12 @@ int gmt_solve_svd (struct GMT_CTRL *GMT, double *u, unsigned int m, unsigned int
 				w[i] = 0.0;
 		}
 	}
-	if (mode == 3) limit = *cutoff;
-	*cutoff = (100.0 * variance / total_variance);	/* Actual explained variance level in % */
 	if (mode == 0)
 		GMT_Report (GMT->parent, GMT_MSG_LONG_VERBOSE,
-		            "gmt_solve_svd: Ratio limit %g gave %d singular values that explain %g %% of total variance %g\n",
-		            limit, n_use, *cutoff, total_variance);
-	if (mode == 1)
+		            "gmt_solve_svd: Ratio limit %g ratained %d singular values\n", cutoff, n_use);
+	if (mode == 2)
 		GMT_Report (GMT->parent, GMT_MSG_LONG_VERBOSE,
-		            "gmt_solve_svd: Found %d singular values needed to explain %g %% of total variance %g\n",
-		            n_use, *cutoff, total_variance);
-	if (mode == 3)
-		GMT_Report (GMT->parent, GMT_MSG_LONG_VERBOSE,
-		            "gmt_solve_svd: Found %d singular values needed for an approximate rms misfit of %g (Closest was %g)\n",
-		            n_use, limit, sqrt ((total_variance - variance) / n));
-	else
-		GMT_Report (GMT->parent, GMT_MSG_LONG_VERBOSE,
-		            "gmt_solve_svd: Selected %d singular values that explain %g %% of total variance %g\n",
-		            n_use, *cutoff, total_variance);
+		            "gmt_solve_svd: Selected first %d singular values\n", n_use);
 
 	/* Here w contains 1/eigenvalue so we multiply by w if w != 0*/
 
@@ -1231,6 +1217,43 @@ void gmt_matrix_vect_mult (struct GMT_CTRL *GMT, unsigned int dim, double a[3][3
 	for (i = 0; i < dim; i++) for (j = 0, c[i] = 0.0; j < dim; j++) c[i] += a[i][j] * b[j];
 }
 
+/* Things to use if figuring out blas calls to speed up these multiplications:
+ * Then, if LAPACK is true then gmt_matrix_matrix_mult should call dgemm_ instead of plain code.
+ */
+
+extern int dgemm_ (char* tra, char* trb, int* na, int* nb, int* nc, double* alpha, double* a, int *nd, double* b, int *ne, double* beta, double* c, int* nf);
+
+void gmt_matrix_matrix_mult (struct GMT_CTRL *GMT, double *A, double *B, uint64_t n_rowsA, uint64_t n_rowsB, uint64_t n_colsB, double *C) {
+#ifdef HAVE_LAPACK
+	double one = 1.0, zero = 0.0;
+	int na, nb, nc, nd, ne, nf;
+	char tr[2] = {'t', '\0'};	/* If B is a vector we must switch to n */
+	gmt_M_unused(GMT);
+	gmt_M_memset (C, n_rowsA * n_colsB, double);
+	na = (int)n_rowsA, nb = (int)n_colsB, nc = (int)n_rowsB, nd = (int)n_rowsB, ne = (int)n_colsB, nf = (int)n_colsB;
+	// cblas_dgemm (CblasRowMajor, CblasNoTrans, CblasNoTrans, (int)n_rowsA, (int)n_colsB, (int)n_rowsB, one, A, (int)n_rowsB, B, (int)n_colsB, zero, C, (int)n_colsB);
+	if (n_colsB == 1) {	/* Vector, so no transposing of the matrix */
+		tr[0] = 'n';
+		ne = nf = (int)n_rowsB;
+	}
+	dgemm_ ("t", tr, &na, &nb, &nc, &one, A, &nd, B, &ne, &zero, C, &nf);
+#else
+	/* Plain matrix multiplication, no speed up; space must exist */
+	uint64_t row, col, k, a_ij, b_ij, c_ij, n_colsA = n_rowsB;
+	gmt_M_unused(GMT);
+	for (row = 0; row < n_rowsA; row++) {
+		for (col = 0; col < n_colsB; col++) {
+			a_ij = row * n_colsA;		/* Start address of row in A */
+			b_ij = col;			/* Start address of col in B */
+			c_ij = row * n_colsB + col;	/* Address of C element to hold their dot-product */
+			C[c_ij] = 0.0;
+			for (k = 0; k < n_rowsB; k++)	/* Do the dot product */
+				C[c_ij] += A[a_ij+k]*B[b_ij+k*n_colsB];
+		}
+	}
+#endif
+}
+
 void gmt_make_rot_matrix2 (struct GMT_CTRL *GMT, double E[3], double w, double R[3][3]) {
 	/* Based on Cox and Hart, 1986 */
 /*	E	Euler pole in in cartesian coordinates
@@ -1374,17 +1397,17 @@ uint64_t gmt_fix_up_path (struct GMT_CTRL *GMT, double **a_lon, double **a_lat,
 	double c, d, fraction, theta, minlon, maxlon;
 	double dlon, lon_i, boost, f_lat_a, f_lat_b;
 
-	if (!gmt_M_is_geographic (GMT, GMT_IN)) return (vector_fix_up_path_cartonly (GMT, a_lon, a_lat, n, mode));	/* Stair case only */
+	if (gmt_M_is_cartesian (GMT, GMT_IN)) return (vector_fix_up_path_cartonly (GMT, a_lon, a_lat, n, mode));	/* Stair case only */
 
 	lon = *a_lon;	lat = *a_lat;	/* Input arrays */
 
 	if (gmt_M_is_dnan (lon[0]) || gmt_M_is_dnan (lat[0])) {	/* If user manages to pass NaN NaN records then we check on the first record and bail */
-		GMT_Report (GMT->parent, GMT_MSG_VERBOSE, "Warning: Your data contains NaNs - no resampling taken place!\n");
+		GMT_Report (GMT->parent, GMT_MSG_VERBOSE, "Your data array row 0 contains NaNs - no resampling taken place!\n");
 		return n;
 	}
 	
 	gmt_geo_to_cart (GMT, lat[0], lon[0], a, true);	/* Start point of current arc */
-	gmt_prep_tmp_arrays (GMT, 1, 2);	/* Init or reallocate tmp vectors */
+	gmt_prep_tmp_arrays (GMT, GMT_NOTSET, 1, 2);	/* Init or reallocate tmp vectors */
 	GMT->hidden.mem_coord[GMT_X][0] = lon[0];
 	GMT->hidden.mem_coord[GMT_Y][0] = lat[0];
 	n_new = 1;
@@ -1403,12 +1426,23 @@ uint64_t gmt_fix_up_path (struct GMT_CTRL *GMT, double **a_lon, double **a_lat,
 	boostable = !(gmt_M_is_linear (GMT) || gmt_M_pole_is_point (GMT));	/* Only boost for projections where poles are lines */
 	f_lat_a = fabs (lat[0]);
 	for (i = 1; i < n; i++) {
+		if (gmt_M_is_dnan (lon[i]) || gmt_M_is_dnan (lat[i])) {	/* If user manages to pass NaN NaN records then we check on the first record and bail */
+			GMT_Report (GMT->parent, GMT_MSG_VERBOSE, "Your data array row %" PRIu64 " contains NaNs - no resampling taken place!\n", i);
+			return n;
+		}
 		f_lat_b = fabs (lat[i]);
 
 		gmt_geo_to_cart (GMT, lat[i], lon[i], b, true);	/* End point of current arc */
-		if (boostable && MIN(f_lat_a, f_lat_b) > 75.0)	/* Enforce closer sampling close to poles */
-			boost = 1.0 + 10.0 * (MAX(f_lat_a, f_lat_b) - 75.0);	/* Crude way to get a boost from 1 at 75 to ~151 at the pole */
-		else
+		if (boostable) {	/* Enforce closer sampling close to poles or for near anti-meridian separation of points */
+			gmt_M_set_delta_lon (lon[i-1], lon[i], dlon);	/* Beware of jumps due to sign differences */
+			if (MIN (f_lat_a, f_lat_b) > 75.0)	/* Enforce closer sampling close to poles */
+				boost = 1.0 + 10.0 * (MAX(f_lat_a, f_lat_b) - 75.0);	/* Crude way to get a boost from 1 at 75 to ~151 at the pole */
+			else if ((c = fabs (fabs (dlon)-180.0)) < 5.0)	/* Enforce closer sampling if points are ~180 degrees apart in longitude */
+				boost = 1.0 + 10.0 * c;	/* Crude way to get a boost when we are close to points along antimeridian */
+			else
+				boost = 1.0;
+		}
+		else	/* No boost needed */
 			boost = 1.0;
 
 		if (mode == GMT_STAIRS_Y) {	/* First follow meridian, then parallel */
@@ -1418,7 +1452,7 @@ uint64_t gmt_fix_up_path (struct GMT_CTRL *GMT, double **a_lon, double **a_lat,
 			n_step = lrint (theta / step);
 			for (j = 1; j < n_step; j++) {
 				c = j / (double)n_step;
-				gmt_prep_tmp_arrays (GMT, n_new, 2);	/* Init or reallocate tmp vectors */
+				gmt_prep_tmp_arrays (GMT, GMT_NOTSET, n_new, 2);	/* Init or reallocate tmp vectors */
 				GMT->hidden.mem_coord[GMT_X][n_new] = lon[i-1] * (1 - c) + lon_i * c;
 				GMT->hidden.mem_coord[GMT_Y][n_new] = lat[i-1];
 				n_new++;
@@ -1427,7 +1461,7 @@ uint64_t gmt_fix_up_path (struct GMT_CTRL *GMT, double **a_lon, double **a_lat,
 			n_step = lrint (theta / step);
 			for (j = k; j < n_step; j++) {	/* Start at 0 to make sure corner point is saved */
 				c = j / (double)n_step;
-				gmt_prep_tmp_arrays (GMT, n_new, 2);	/* Init or reallocate tmp vectors */
+				gmt_prep_tmp_arrays (GMT, GMT_NOTSET, n_new, 2);	/* Init or reallocate tmp vectors */
 				GMT->hidden.mem_coord[GMT_X][n_new] = lon[i];
 				GMT->hidden.mem_coord[GMT_Y][n_new] = lat[i-1] * (1 - c) + lat[i] * c;
 				n_new++;
@@ -1440,7 +1474,7 @@ uint64_t gmt_fix_up_path (struct GMT_CTRL *GMT, double **a_lon, double **a_lat,
 			n_step = lrint (theta / step);
 			for (j = 1; j < n_step; j++) {
 				c = j / (double)n_step;
-				gmt_prep_tmp_arrays (GMT, n_new, 2);	/* Init or reallocate tmp read vectors */
+				gmt_prep_tmp_arrays (GMT, GMT_NOTSET, n_new, 2);	/* Init or reallocate tmp read vectors */
 				GMT->hidden.mem_coord[GMT_X][n_new] = lon[i-1];
 				GMT->hidden.mem_coord[GMT_Y][n_new] = lat[i-1] * (1 - c) + lat[i] * c;
 				n_new++;
@@ -1451,27 +1485,67 @@ uint64_t gmt_fix_up_path (struct GMT_CTRL *GMT, double **a_lon, double **a_lat,
 			n_step = lrint (theta / step);
 			for (j = k; j < n_step; j++) {	/* Start at 0 to make sure corner point is saved */
 				c = j / (double)n_step;
-				gmt_prep_tmp_arrays (GMT, n_new, 2);	/* Init or reallocate tmp read vectors */
+				gmt_prep_tmp_arrays (GMT, GMT_NOTSET, n_new, 2);	/* Init or reallocate tmp read vectors */
 				GMT->hidden.mem_coord[GMT_X][n_new] = lon[i-1] * (1 - c) + lon_i * c;
 				GMT->hidden.mem_coord[GMT_Y][n_new] = lat[i];
 				n_new++;
 			}
 			k = 0;
 		}
-
 		/* Follow great circle */
 		else if ((theta = d_acosd (gmt_dot3v (GMT, a, b))) == 180.0) {	/* trouble, no unique great circle */
 			if (gmt_M_is_spherical (GMT) || ((lat[i] + lat[i-1]) == 0.0)) {
-				GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Error: Two points in input list are antipodal - great circle resampling is not unique!\n");
-				GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Error: Fix input data or use project -A to generate the desired great circle by providing an azimuth.\n");
+				GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Two points in input list are antipodal - great circle resampling is not unique!\n");
+				GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Fix input data or use project -A to generate the desired great circle by providing an azimuth.\n");
 			}
 			else {
-				GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Error: Two points in input list are antipodal - great circle resampling is not unique!\n");
+				GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Two points in input list are antipodal - great circle resampling is not unique!\n");
 				GMT_Report (GMT->parent, GMT_MSG_NORMAL, "There are two possible geodesics but GMT does not currently calculate geodesics.\n");
 			}
 			return 0;
 		}
-
+		else if (doubleAlmostEqual (fabs (fmod (lon[i-1] - lon[i], 360)), 180.0)) {
+			/* Must march along the two meridians through the nearest pole */
+			double sL = lat[i-1] + lat[i], dy;
+			double Narc = 180.0 - sL, Sarc = 180.0 + sL;	/* Compute the arc lengths of the two pole passes */
+			if (Narc < Sarc) {	/* Shortest path is to connect through N pole */
+				n_step = lrint ((90.0 - lat[i-1]) / step);	/* Must insert (n_step - 1) points, i.e. create n_step intervals */
+				dy = (90.0 - lat[i-1]) / n_step;	/* Ensure we land exactly on N pole */
+				for (j = 1; j <= n_step; j++) {	/* Start at 1 since 0 is lon[i-1] and end exactly at pole */
+					gmt_prep_tmp_arrays (GMT, GMT_NOTSET, n_new, 2);	/* Init or reallocate tmp read vectors */
+					GMT->hidden.mem_coord[GMT_X][n_new] = lon[i-1];	/* Keep longitude constant */
+					GMT->hidden.mem_coord[GMT_Y][n_new] = lat[i-1] + j * dy;	/* March towards N pole */
+					n_new++;
+				}
+				n_step = lrint ((90.0 - lat[i]) / step);	/* Must insert (n_step - 1) points, i.e. create n_step intervals */
+				dy = (90.0 - lat[i]) / n_step;	/* Ensure we start exactly on N pole */
+				for (j = 0; j < n_step; j++) {	/* Start at 0 to pick up N pole */
+					gmt_prep_tmp_arrays (GMT, GMT_NOTSET, n_new, 2);	/* Init or reallocate tmp read vectors */
+					GMT->hidden.mem_coord[GMT_X][n_new] = lon[1];
+					GMT->hidden.mem_coord[GMT_Y][n_new] = 90.0 - j * dy;
+					n_new++;
+				}
+			}
+			else {	/* Must connect via S pole */
+				n_step = lrint ((lat[i-1] + 90.0) / step);	/* Must insert (n_step - 1) points, i.e. create n_step intervals */
+				dy = (lat[i-1] + 90.0) / n_step;	/* Ensure we land exactly on S pole */
+				for (j = 1; j <= n_step; j++) {	/* Start at 1 since 0 is lon[i-1] and end exactly at pole */
+					gmt_prep_tmp_arrays (GMT, GMT_NOTSET, n_new, 2);	/* Init or reallocate tmp read vectors */
+					GMT->hidden.mem_coord[GMT_X][n_new] = lon[i-1];
+					GMT->hidden.mem_coord[GMT_Y][n_new] = lat[i-1] - j * dy;
+					n_new++;
+				}
+				n_step = lrint ((lat[i] + 90.0) / step);	/* Must insert (n_step - 1) points, i.e. create n_step intervals */
+				dy = (lat[i] + 90.0) / n_step;	/* Ensure we start exactly on S pole */
+				for (j = 0; j < n_step; j++) {	/* Start at 0 to pick up S pole */
+					gmt_prep_tmp_arrays (GMT, GMT_NOTSET, n_new, 2);	/* Init or reallocate tmp read vectors */
+					GMT->hidden.mem_coord[GMT_X][n_new] = lon[1];
+					GMT->hidden.mem_coord[GMT_Y][n_new] = j * dy - 90.0;
+					n_new++;
+				}
+			}
+			/* Next point is now point i */
+		}
 		else if ((n_step = lrint (boost * theta / step)) > 1) {	/* Must insert (n_step - 1) points, i.e. create n_step intervals */
 			fraction = 1.0 / (double)n_step;
 			minlon = MIN (lon[i-1], lon[i]);
@@ -1482,7 +1556,7 @@ uint64_t gmt_fix_up_path (struct GMT_CTRL *GMT, double **a_lon, double **a_lat,
 				d = 1 - c;
 				for (k = 0; k < 3; k++) x[k] = a[k] * d + b[k] * c;
 				gmt_normalize3v (GMT, x);		/* Make unit vector */
-				gmt_prep_tmp_arrays (GMT, n_new, 2);	/* Init or reallocate tmp read vectors */
+				gmt_prep_tmp_arrays (GMT, GMT_NOTSET, n_new, 2);	/* Init or reallocate tmp read vectors */
 				gmt_cart_to_geo (GMT, &GMT->hidden.mem_coord[GMT_Y][n_new], &GMT->hidden.mem_coord[GMT_X][n_new], x, true);
 				if (meridian)
 					GMT->hidden.mem_coord[GMT_X][n_new] = minlon;
@@ -1493,7 +1567,7 @@ uint64_t gmt_fix_up_path (struct GMT_CTRL *GMT, double **a_lon, double **a_lat,
 				n_new++;
 			}
 		}
-		gmt_prep_tmp_arrays (GMT, n_new, 2);	/* Init or reallocate tmp read vectors */
+		gmt_prep_tmp_arrays (GMT, GMT_NOTSET, n_new, 2);	/* Init or reallocate tmp read vectors */
 		GMT->hidden.mem_coord[GMT_X][n_new] = lon[i];	GMT->hidden.mem_coord[GMT_Y][n_new] = lat[i];
 		n_new++;
 		gmt_M_cpy3v (a, b);
@@ -1503,7 +1577,7 @@ uint64_t gmt_fix_up_path (struct GMT_CTRL *GMT, double **a_lon, double **a_lat,
 	/* Destroy old allocated memory and put the new one in place */
 	gmt_M_free (GMT, lon);
 	gmt_M_free (GMT, lat);
-	gmtlib_eliminate_lon_jumps (GMT, GMT->hidden.mem_coord[GMT_X], n_new);	/* Ensure longitudes are in the same quadrants */
+	gmt_eliminate_lon_jumps (GMT, GMT->hidden.mem_coord[GMT_X], n_new);	/* Ensure longitudes are in the same quadrants */
 	*a_lon = gmtlib_assign_vector (GMT, n_new, GMT_X);
 	*a_lat = gmtlib_assign_vector (GMT, n_new, GMT_Y);
 
