@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *	Copyright (c) 1991-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,14 +12,14 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  * API functions to support the gmtconvert application.
  *
  * Author:	Paul Wessel
  * Date:	1-JAN-2010
- * Version:	5 API
+ * Version:	6 API
  *
  * Brief synopsis: Read one or more data tables and can concatenated them
  * vertically [Default] or horizontally (pasting), select certain columns,
@@ -30,7 +29,8 @@
 
 #include "gmt_dev.h"
 
-#define THIS_MODULE_NAME	"gmtconvert"
+#define THIS_MODULE_CLASSIC_NAME	"gmtconvert"
+#define THIS_MODULE_MODERN_NAME	"gmtconvert"
 #define THIS_MODULE_LIB		"core"
 #define THIS_MODULE_PURPOSE	"Convert, paste, or extract columns from data tables"
 #define THIS_MODULE_KEYS	"<D{,>D}"
@@ -67,19 +67,25 @@ struct GMTCONVERT_CTRL {
 	} D;
 	struct E {	/* -E */
 		bool active;
+		bool end;
 		int mode;	/* -3, -1, -1, 0, or increment stride */
 	} E;
 	struct F {	/* -F<mode> */
 		bool active;
 		struct GMT_SEGMENTIZE S;
 	} F;
-	struct L {	/* -L */
-		bool active;
-	} L;
 	struct I {	/* -I[ast] */
 		bool active;
 		unsigned int mode;
 	} I;
+	struct L {	/* -L */
+		bool active;
+	} L;
+	struct N {	/* -N<col>[+a|d] sorting */
+		bool active;
+		int dir;	/* +1 ascending [default], -1 descending */
+		uint64_t col;
+	} N;
 	struct Q {	/* -Q<selections> */
 		bool active;
 		struct GMT_INT_SELECTION *select;
@@ -88,9 +94,17 @@ struct GMTCONVERT_CTRL {
 		bool active;
 		struct GMT_TEXT_SELECTION *select;
 	} S;
-	struct T {	/* -T */
-		bool active;
+	struct T {	/* -T[sd] */
+		bool active[2];
 	} T;
+	struct W {	/* -W[+n] */
+		bool active;
+		unsigned int mode;
+	} W;
+	struct Z {	/* -Z[<first>]:[<last>] */
+		bool active;
+		int64_t first, last;
+	} Z;
 };
 
 GMT_LOCAL void *New_Ctrl (struct GMT_CTRL *GMT) {	/* Allocate and initialize a new control structure */
@@ -114,11 +128,11 @@ GMT_LOCAL void Free_Ctrl (struct GMT_CTRL *GMT, struct GMTCONVERT_CTRL *C) {	/*
 }
 
 GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
-	gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
+	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: gmtconvert [<table>] [-A] [-C[+l<min>][+u<max>][+i]] [-D[<template>[+o<orig>]]] [-E[f|l|m<stride>]] [-F<arg>] [-I[tsr]]\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t[-L] [-Q[~]<selection>] [-S[~]\"search string\"] [-T] [%s] [%s]\n\t[%s] [%s] [%s] [%s] [%s]\n", GMT_V_OPT, GMT_a_OPT, GMT_b_OPT, GMT_d_OPT, GMT_e_OPT, GMT_f_OPT, GMT_g_OPT);
-	GMT_Message (API, GMT_TIME_NONE, "\t[%s] [%s]\n\t[%s] [%s] [%s]\n\n", GMT_h_OPT, GMT_i_OPT, GMT_o_OPT, GMT_s_OPT, GMT_colon_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "usage: %s [<table>] [-A] [-C[+l<min>][+u<max>][+i]] [-D[<template>[+o<orig>]]] [-E[f|l|m|M<stride>]] [-F<arg>] [-I[tsr]]\n", name);
+	GMT_Message (API, GMT_TIME_NONE, "\t[-L] [-N<col>[+a|d]] [-Q[~]<selection>] [-S[~]\"search string\"] [-T[hd]] [%s] [-W[+n]] [-Z[<first>][/<last>]] [%s]\n\t[%s] [%s] [%s] [%s] [%s]\n", GMT_V_OPT, GMT_a_OPT, GMT_b_OPT, GMT_d_OPT, GMT_e_OPT, GMT_f_OPT, GMT_g_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "\t[%s] [%s]\n\t[%s] [%s] [%s] [%s]\n\n", GMT_h_OPT, GMT_i_OPT, GMT_o_OPT, GMT_s_OPT, GMT_colon_OPT, GMT_PAR_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
@@ -142,7 +156,7 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t   Use +o<t_orig>/<s_orig> to start numbering at <t_orig> for tables and <s_orig> for segments [0/0].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-E Extract first and last point per segment only [Output all points].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   Append f for first only or l for last only.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Append m<stride> to pass only 1 out of <stride> records.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   Append m<stride> to pass every <stride> records; use -EM to include the last point.\n");
 	gmt_segmentize_syntax (API->GMT, 'F', 0);
 	GMT_Message (API, GMT_TIME_NONE, "\t-I Invert output order of (t)ables, (s)egments, or (r)ecords.  Append any combination of:\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t     t: reverse the order of input tables on output.\n");
@@ -150,6 +164,8 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t     r: reverse the order of records within each segment on output [Default].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-L Output only segment headers and skip all data records.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   Requires ASCII input data [Output headers and data].\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t-N Numerically sort all records per segment based on data in column <col>.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   Append +a for ascending or +d for descending order [ascending].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-Q Only output specified segment numbers in <selection> [All].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   <selection> syntax is [~]<range>[,<range>,...] where each <range> of items is\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   either a single number, start-stop (for range), start:step:stop (for stepped range),\n");
@@ -158,12 +174,17 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t-S Only output segments whose headers contain the pattern \"string\".\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   Use -S~\"string\" to output segment that DO NOT contain this pattern.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   If your pattern begins with ~, escape it with \\~.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   To match OGR aspatial values, use name=value, and to match against\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   extended regular expressions use -S[~]/regexp/[i] (i for insensitive).\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   To match OGR aspatial values, use name=value, and to match headers against\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   extended regular expressions use -S[~]/regexp/[i] (i for case-insensitive).\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   Give +f<file> for a file list with such patterns, one per line.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   To give a single pattern starting with +f, escape it with \\+f.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-T Prevent the writing of segment headers.\n");
-	GMT_Option (API, "V,a,bi,bo,d,e,f,g,h,i,o,s,:,.");
+	GMT_Message (API, GMT_TIME_NONE, "\t-T Skip certain types of records.  Append one or both of d and h [h].\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   h: Prevent the writing of segment headers [Default].\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   d: Prevent the writing of duplicate data records.\n");
+	GMT_Option (API, "V");
+	GMT_Message (API, GMT_TIME_NONE, "\t-W Convert trailing text to numbers, if possible.  Append +n to suppress NaN columns.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t-Z Select range of output records.  If not set, <first> = 0 and <last> = last record [all records].\n");
+	GMT_Option (API, "a,bi,bo,d,e,f,g,h,i,o,s,:,.");
 
 	return (GMT_MODULE_USAGE);
 }
@@ -209,13 +230,13 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GMTCONVERT_CTRL *Ctrl, struct
 							Ctrl->C.invert = true;	break;
 						case 'l':	/* Set fewest records required */
 						 	if ((value = atol (&p[1])) < 0)
-								GMT_Report (API, GMT_MSG_NORMAL, "Error: The -C+l modifier was given negative record count!\n");
+								GMT_Report (API, GMT_MSG_NORMAL, "The -C+l modifier was given negative record count!\n");
 							else
 								Ctrl->C.min = (uint64_t)value;
 							break;
 						case 'u':	/* Set max records required */
 					 		if ((value = atol (&p[1])) < 0)
-								GMT_Report (API, GMT_MSG_NORMAL, "Error: The -C+u modifier was given negative record count!\n");
+								GMT_Report (API, GMT_MSG_NORMAL, "The -C+u modifier was given negative record count!\n");
 							else
 								Ctrl->C.max = (uint64_t)value;
 							break;
@@ -243,6 +264,8 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GMTCONVERT_CTRL *Ctrl, struct
 						Ctrl->E.mode = -1; break;
 					case 'l':		/* Get last point only */
 						Ctrl->E.mode = -2; break;
+					case 'M':		/* Set modulo step */
+						Ctrl->E.end = true;	/* Include last point; fall through on purpose to set the step */
 					case 'm':		/* Set modulo step */
 						Ctrl->E.mode = atoi (&opt->arg[1]); break;
 					default:		/* Get first and last point only */
@@ -254,7 +277,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GMTCONVERT_CTRL *Ctrl, struct
 				if (opt->arg[0] == '\0') {	/* No arguments, must be old GMT4 option -F */
 					if (gmt_M_compat_check (GMT, 4)) {
 						GMT_Report (API, GMT_MSG_COMPAT,
-						            "Warning: Option -F for output columns is deprecated; use -o instead\n");
+						            "Option -F for output columns is deprecated; use -o instead\n");
 						gmt_parse_o_option (GMT, opt->arg);
 					}
 					else
@@ -273,7 +296,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GMTCONVERT_CTRL *Ctrl, struct
 						case 'r': Ctrl->I.mode |= INV_ROWS; break;	/* Reverse record order */
 						default:
 							GMT_Report (API, GMT_MSG_NORMAL,
-							            "Error: The -I option does not recognize modifier %c\n", (int)opt->arg[k]);
+							            "The -I option does not recognize modifier %c\n", (int)opt->arg[k]);
 							n_errors++;
 							break;
 					}
@@ -283,6 +306,19 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GMTCONVERT_CTRL *Ctrl, struct
 			case 'L':	/* Only output segment headers */
 				Ctrl->L.active = true;
 				break;
+			case 'N':	/* Sort per segment on specified column */
+				Ctrl->N.active = true;
+				if ((c = strstr (opt->arg, "+a")) || (c = strstr (opt->arg, "+d"))) {	/* New syntax */
+					Ctrl->N.dir = (c[1] == 'd') ? -1 : +1;
+					c[0] = '\0';	/* Chop off modifier */
+				}
+				value = atol (opt->arg);
+				if (c)
+					c[0] = '+';	/* Restore modifier */
+				else
+					Ctrl->N.dir = (value < 0 || opt->arg[0] == '-') ? -1 : +1;
+				Ctrl->N.col = int64_abs (value);
+				break;
 			case 'Q':	/* Only report for specified segment numbers */
 				Ctrl->Q.active = true;
 				Ctrl->Q.select = gmt_set_int_selection (GMT, opt->arg);
@@ -291,8 +327,35 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GMTCONVERT_CTRL *Ctrl, struct
 				Ctrl->S.active = true;
 				Ctrl->S.select = gmt_set_text_selection (GMT, opt->arg);
 				break;
-			case 'T':	/* Do not write segment headers */
-				Ctrl->T.active = true;
+			case 'T':	/* -T[h]: Do not write segment headers, -Td: Skip duplicate records */
+				if (strchr (opt->arg, 'd')) /* Skip duplicates */
+					Ctrl->T.active[1] = true;
+				if (!opt->arg[0] || strchr (opt->arg, 'h')) /* Skip duplicates */
+					Ctrl->T.active[0] = true;	/* Skip segment headers */
+				break;
+			case 'W':
+				Ctrl->W.active = true;
+				if (!strncmp (opt->arg, "+n", 2U))
+					Ctrl->W.mode = 1;
+				break;
+			case 'Z':
+				Ctrl->Z.active = true;
+				if ((c = strchr (opt->arg, ':')) || (c = strchr (opt->arg, '/'))) {	/* Got [<first>]:[<last>] or [<first>]/[<last>] */
+					char div = c[0];	/* Either : or / */
+					if (opt->arg[0] == div) /* No first given, default to 0 */
+						Ctrl->Z.last = atol (&opt->arg[1]);
+					else { /* Gave first, and maybe last */
+						c[0] = '\0';	/* Chop off last */
+						Ctrl->Z.first = atol (opt->arg);
+						Ctrl->Z.last = (c[1]) ? (int64_t)atol (&c[1]) : INTMAX_MAX;	/* Last record if not given */
+						c[0] = div;	/* Restore */
+					}
+				}
+				else /* No colon means first is 0 and given value is last */
+					Ctrl->Z.last = atol (opt->arg);
+				/* Adjust to system where first record is 1 since we must increment n_in_rows before applying -Z check later */
+				Ctrl->Z.first++;	if (Ctrl->Z.last < INTMAX_MAX) Ctrl->Z.last++;
+				GMT_Report (API, GMT_MSG_DEBUG, "Output record numbers %" PRIu64 " through = %" PRIu64 "\n", Ctrl->Z.first, Ctrl->Z.last);
 				break;
 
 			default:	/* Report bad options */
@@ -344,11 +407,22 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GMTCONVERT_CTRL *Ctrl, struct
 	                                 "Syntax error: -D Output template must contain %%d\n");
 	n_errors += gmt_M_check_condition (GMT, Ctrl->Q.active && Ctrl->S.active,
 	                                 "Syntax error: Only one of -Q and -S can be used simultaneously\n");
+	n_errors += gmt_M_check_condition (GMT, Ctrl->N.active && Ctrl->F.active,
+	                                 "Syntax error: The -N option cannot be used with -F\n");
 	n_errors += gmt_M_check_condition (GMT, n_files > 1, "Syntax error: Only one output destination can be specified\n");
 
 	return (n_errors ? GMT_PARSE_ERROR : GMT_NOERROR);
 }
 
+GMT_LOCAL bool is_duplicate_row (struct GMT_DATASEGMENT *S, uint64_t row) {
+	uint64_t col;
+	/* Loop over all columns and compare the two records, if differ then return false.
+	 * If passes all columns then they are the same and we return true. */
+	for (col = 0; col < S->n_columns; col++)
+		if (!doubleAlmostEqualZero (S->data[col][row], S->data[col][row-1])) return false;
+	return true;
+}
+
 /* Must free allocated memory before returning */
 #define bailout(code) {gmt_M_free_options (mode); return (code);}
 #define Return(code) {Free_Ctrl (GMT, Ctrl); gmt_end_module (GMT, GMT_cpy); bailout (code);}
@@ -356,9 +430,10 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GMTCONVERT_CTRL *Ctrl, struct
 int GMT_gmtconvert (void *V_API, int mode, void *args) {
 	bool match = false, prevent_seg_headers = false;
 	int error = 0;
-	uint64_t out_col, col, n_cols_in, n_cols_out, tbl;
+	uint64_t out_col, col, n_cols_in = 0, n_cols_out, tbl, tlen;
 	uint64_t n_horizontal_tbls, n_vertical_tbls, tbl_ver, tbl_hor, use_tbl;
 	uint64_t last_row, n_rows, row, seg, n_out_seg = 0, out_seg = 0;
+	int64_t n_in_rows;
 
 	double *val = NULL;
 
@@ -366,6 +441,10 @@ int GMT_gmtconvert (void *V_API, int mode, void *args) {
 
 	struct GMTCONVERT_CTRL *Ctrl = NULL;
 	struct GMT_DATASET *D[2] = {NULL, NULL};	/* Pointer to GMT multisegment table(s) in and out */
+	struct GMT_DATASEGMENT *S = NULL;
+	struct GMT_DATASET_HIDDEN *DHi = NULL, *DHo = NULL;
+	struct GMT_DATATABLE_HIDDEN *THi = NULL, *THo = NULL;
+	struct GMT_DATASEGMENT_HIDDEN *SHi = NULL, *SHo = NULL;
 	struct GMT_CTRL *GMT = NULL, *GMT_cpy = NULL;
 	struct GMT_OPTION *options = NULL;
 	struct GMTAPI_CTRL *API = gmt_get_api_ptr (V_API);	/* Cast from void to GMTAPI_CTRL pointer */
@@ -376,21 +455,20 @@ int GMT_gmtconvert (void *V_API, int mode, void *args) {
 	if (mode == GMT_MODULE_PURPOSE) return (usage (API, GMT_MODULE_PURPOSE));	/* Return the purpose of program */
 	options = GMT_Create_Options (API, mode, args);	if (API->error) return (API->error);	/* Set or get option list */
 
-	if (options && options->option == GMT_OPT_USAGE) bailout (usage (API, GMT_USAGE));	/* Return the usage message */
-	if (options && options->option == GMT_OPT_SYNOPSIS) bailout (usage (API, GMT_SYNOPSIS));	/* Return the synopsis */
+	if ((error = gmt_report_usage (API, options, 1, usage)) != GMT_NOERROR) bailout (error);	/* Give usage if requested */
 
 	/* Parse the command-line arguments */
 
-	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
 	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
 	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
 	if ((error = parse (GMT, Ctrl, options)) != 0) Return (error);
 
 	/*---------------------------- This is the gmtconvert main code ----------------------------*/
 
-	GMT_Report (API, GMT_MSG_VERBOSE, "Processing input table data\n");
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Processing input table data\n");
 
-	if (GMT_Init_IO (API, GMT_IS_DATASET, GMT_IS_NONE, GMT_IN, GMT_ADD_DEFAULT, 0, options) != GMT_NOERROR) {
+	if (GMT_Init_IO (API, GMT_IS_DATASET, GMT_IS_POINT, GMT_IN, GMT_ADD_DEFAULT, 0, options) != GMT_NOERROR) {
 		Return (API->error);	/* Establishes data files or stdin */
 	}
 
@@ -401,19 +479,21 @@ int GMT_gmtconvert (void *V_API, int mode, void *args) {
 	}
 
 	if (D[GMT_IN]->n_records == 0) {
-		GMT_Report (API, GMT_MSG_NORMAL, "No data records provided\n");
+		GMT_Report (API, GMT_MSG_VERBOSE, "No data records provided\n");
 		Return (GMT_NOERROR);
 	}
 	if (GMT->common.a.active && D[GMT_IN]->n_tables > 1) {
 		GMT_Report (API, GMT_MSG_NORMAL, "The -a option requires a single table only.\n");
 		Return (GMT_RUNTIME_ERROR);
 	}
-	if (GMT->common.a.active && D[GMT_IN]->table[0]->ogr) {
+	DHi = gmt_get_DD_hidden (D[GMT_IN]);
+	THi = gmt_get_DT_hidden (D[GMT_IN]->table[0]);
+	if (GMT->common.a.active && THi->ogr) {
 		GMT_Report (API, GMT_MSG_NORMAL, "The -a option requires a single table without OGR metadata.\n");
 		Return (GMT_RUNTIME_ERROR);
 	}
 
-	if (Ctrl->T.active && !gmt_M_file_is_memory (Ctrl->Out.file))
+	if (Ctrl->T.active[0] && !gmt_M_file_is_memory (Ctrl->Out.file))
 		prevent_seg_headers = true;
 
 	if (prevent_seg_headers)	/* Turn off segment headers on file output */
@@ -426,13 +506,73 @@ int GMT_gmtconvert (void *V_API, int mode, void *args) {
 		}
 		if (D[GMT_OUT]->n_segments > 1) gmt_set_segmentheader (GMT, GMT_OUT, true);	/* Turn on segment headers on output */
 
-		if (GMT_Write_Data (API, GMT_IS_DATASET, GMT_IS_FILE, D[GMT_OUT]->geometry, D[GMT_OUT]->io_mode, NULL, Ctrl->Out.file, D[GMT_OUT]) != GMT_NOERROR) {
+		DHo = gmt_get_DD_hidden (D[GMT_OUT]);
+		if (GMT_Write_Data (API, GMT_IS_DATASET, GMT_IS_FILE, D[GMT_OUT]->geometry, DHo->io_mode, NULL, Ctrl->Out.file, D[GMT_OUT]) != GMT_NOERROR) {
 			Return (API->error);
 		}
 		if (prevent_seg_headers) GMT->current.io.skip_headers_on_outout = false;	/* Restore to default if it was changed */
 		Return (GMT_NOERROR);	/* We are done! */
 	}
 
+	if (Ctrl->W.active) {	/* Text to data happens here and then we are done */
+		double out[GMT_BUFSIZ], *tmp = NULL;
+		struct GMT_RECORD *Out = NULL;
+		char *nan = NULL;
+		uint64_t n_col = 0, k;
+		S = D[GMT_IN]->table[0]->segment[0];	/* Short-hand */
+		if (S->text) {	/* Has trailing text */
+			n_col = n_cols_in = GMT_Get_Values (API, S->text[0], out, GMT_BUFSIZ);
+			if (Ctrl->W.mode) {	/* Only wants non-NaN columns from trailing text */
+				nan = gmt_M_memory (GMT, NULL, n_col, char);
+				tmp = gmt_M_memory (GMT, NULL, n_col, double);
+				for (col = 0; col < n_col; col++) if (gmt_M_is_dnan (out[col])) nan[col] = 1, n_cols_in--;
+				GMT_Report (API, GMT_MSG_LONG_VERBOSE, "First record trailing text converted to %" PRIu64 " columns, with %" PRIu64 " of them yielding NaNs [skipped]\n", n_col, n_col - n_cols_in);
+			}
+			else
+				GMT_Report (API, GMT_MSG_LONG_VERBOSE, "First record trailing text converted to %" PRIu64 " columns\n", n_col);
+		}
+		else
+			GMT_Report (API, GMT_MSG_VERBOSE, "No trialing text found in first record; -W will not have any effect.\n");
+		if (GMT_Init_IO (API, GMT_IS_DATASET, GMT_IS_POINT, GMT_OUT, GMT_ADD_DEFAULT, 0, options) != GMT_NOERROR) {	/* Registers default output destination, unless already set */
+			Return (API->error);
+		}
+		if (GMT_Begin_IO (API, GMT_IS_DATASET, GMT_OUT, GMT_HEADER_ON) != GMT_NOERROR) {	/* Enables data output and sets access mode */
+			Return (API->error);
+		}
+		if ((error = GMT_Set_Columns (API, GMT_OUT, (unsigned int)(n_cols_in + S->n_columns), GMT_COL_FIX_NO_TEXT)) != GMT_NOERROR) {
+			Return (error);
+		}
+		Out = gmt_new_record (GMT, out, NULL);	/* Since we only need to worry about numerics in this module */
+		for (tbl = 0; tbl < D[GMT_IN]->n_tables; tbl++) {
+			for (seg = 0; seg < D[GMT_IN]->table[tbl]->n_segments; seg++) {	/* For each segment in the tables */
+				S = D[GMT_IN]->table[tbl]->segment[seg];	/* Short-hand */
+				if (D[GMT_IN]->n_segments > 1 || S->header) GMT_Put_Record (API, GMT_WRITE_SEGMENT_HEADER, S->header);
+				for (row = 0; row < S->n_rows; row++) {
+					for (col = 0; col < S->n_columns; col++)
+						out[col] = S->data[col][row];
+					if (S->text) {
+						if (Ctrl->W.mode) {	/* Exclude NaN columns */
+							n_cols_in = GMT_Get_Values (API, S->text[row], tmp, (int)n_col);
+							for (col = k = 0; col < n_cols_in; col++) if (!nan[col]) out[S->n_columns+k] = tmp[col], k++;
+						}
+						else
+							GMT_Get_Values (API, S->text[row], &out[S->n_columns], (int)n_col);
+					}
+					GMT_Put_Record (API, GMT_WRITE_DATA, Out);	/* Write this to output */
+				}
+			}
+		}
+		gmt_M_free (GMT, Out);
+		if (Ctrl->W.mode) {
+			gmt_M_free (GMT, nan);
+			gmt_M_free (GMT, tmp);
+		}
+		if (GMT_End_IO (API, GMT_OUT, 0) != GMT_NOERROR) {	/* Disables further data output */
+			Return (API->error);
+		}
+		Return (GMT_NOERROR);	/* We are done! */
+	}
+
 	/* Determine number of input and output columns for the selected options.
 	 * For -A, require all tables to have the same number of segments and records. */
 
@@ -450,12 +590,12 @@ int GMT_gmtconvert (void *V_API, int mode, void *args) {
 		GMT_Report (API, GMT_MSG_NORMAL, "Parsing requires files with same number of records.\n");
 		Return (GMT_RUNTIME_ERROR);
 	}
-	if (n_cols_out == 0) {
+	if (n_cols_out == 0 && !GMT->current.io.trailing_text[GMT_OUT]) {
 		GMT_Report (API, GMT_MSG_NORMAL, "Selection led to no output columns.\n");
 		Return (GMT_RUNTIME_ERROR);
 
 	}
-	if ((error = gmt_set_cols (GMT, GMT_OUT, n_cols_out)) != GMT_NOERROR) {
+	if ((error = GMT_Set_Columns (API, GMT_OUT, (unsigned int)n_cols_out, (D[GMT_IN]->type == GMT_READ_DATA) ? GMT_COL_FIX_NO_TEXT : GMT_COL_FIX)) != GMT_NOERROR) {
 		Return (error);
 	}
 
@@ -471,8 +611,9 @@ int GMT_gmtconvert (void *V_API, int mode, void *args) {
 	/* We now know the exact number of segments and columns and an upper limit on total records.
 	 * Allocate data set with a single table with those proportions. This copies headers as well */
 
-	D[GMT_IN]->dim[GMT_COL] = n_cols_out;	/* State we want a different set of columns on output */
+	DHi->dim[GMT_COL] = n_cols_out;	/* State we want a different set of columns on output */
 	D[GMT_OUT] = GMT_Duplicate_Data (API, GMT_IS_DATASET, GMT_DUPLICATE_ALLOC + ((Ctrl->A.active) ? GMT_ALLOC_HORIZONTAL : GMT_ALLOC_NORMAL), D[GMT_IN]);
+	DHo = gmt_get_DD_hidden (D[GMT_OUT]);
 
 	n_horizontal_tbls = (Ctrl->A.active) ? D[GMT_IN]->n_tables : 1;	/* Only with pasting do we go horizontally */
 	n_vertical_tbls   = (Ctrl->A.active) ? 1 : D[GMT_IN]->n_tables;	/* Only for concatenation do we go vertically */
@@ -480,33 +621,44 @@ int GMT_gmtconvert (void *V_API, int mode, void *args) {
 
 	for (tbl_ver = 0; tbl_ver < n_vertical_tbls; tbl_ver++) {	/* Number of tables to place vertically */
 		D[GMT_OUT]->table[tbl_ver]->n_records = 0;	/* Reset record count per table since we may return fewer than the original */
+		THo = gmt_get_DT_hidden (D[GMT_OUT]->table[tbl_ver]);
 		for (seg = 0; seg < D[GMT_IN]->table[tbl_ver]->n_segments; seg++) {	/* For each segment in the tables */
-			if (Ctrl->L.active) D[GMT_OUT]->table[tbl_ver]->segment[seg]->mode = GMT_WRITE_HEADER;	/* Only write segment header */
+			S = D[GMT_IN]->table[tbl_ver]->segment[seg];	/* Current segment */
+			SHi = gmt_get_DS_hidden (S);
+			SHo = gmt_get_DS_hidden (D[GMT_OUT]->table[tbl_ver]->segment[seg]);
+			if (Ctrl->L.active) SHo->mode = GMT_WRITE_HEADER;	/* Only write segment header */
 			if (Ctrl->S.active) {		/* See if the combined segment header has text matching our search string */
-				match = gmt_get_segtext_selection (GMT, Ctrl->S.select, D[GMT_IN]->table[tbl_ver]->segment[seg], match);
-				if (Ctrl->S.select->invert == match) D[GMT_OUT]->table[tbl_ver]->segment[seg]->mode = GMT_WRITE_SKIP;	/* Mark segment to be skipped */
+				match = gmt_get_segtext_selection (GMT, Ctrl->S.select, S, match);
+				if (Ctrl->S.select->invert == match) SHo->mode = GMT_WRITE_SKIP;	/* Mark segment to be skipped */
 			}
-			if (Ctrl->Q.active && !gmt_get_int_selection (GMT, Ctrl->Q.select, seg)) D[GMT_OUT]->table[tbl_ver]->segment[seg]->mode = GMT_WRITE_SKIP;	/* Mark segment to be skipped */
+			if (Ctrl->Q.active && !gmt_get_int_selection (GMT, Ctrl->Q.select, seg)) SHo->mode = GMT_WRITE_SKIP;	/* Mark segment to be skipped */
 			if (Ctrl->C.active) {	/* See if the number of records in this segment passes our test for output */
-				match = (D[GMT_IN]->table[tbl_ver]->segment[seg]->n_rows >= Ctrl->C.min && D[GMT_IN]->table[tbl_ver]->segment[seg]->n_rows <= Ctrl->C.max);
-				if (Ctrl->C.invert == match) D[GMT_OUT]->table[tbl_ver]->segment[seg]->mode = GMT_WRITE_SKIP;	/* Mark segment to be skipped */
+				match = (S->n_rows >= Ctrl->C.min && S->n_rows <= Ctrl->C.max);
+				if (Ctrl->C.invert == match) SHo->mode = GMT_WRITE_SKIP;	/* Mark segment to be skipped */
 			}
-			if (D[GMT_OUT]->table[tbl_ver]->segment[seg]->mode) continue;	/* No point copying values given segment content will be skipped */
+			if (SHo->mode) continue;	/* No point copying values given segment content will be skipped */
 			n_out_seg++;	/* Number of segments that passed the test */
-			last_row = D[GMT_IN]->table[tbl_ver]->segment[seg]->n_rows - 1;
-			for (row = n_rows = 0; row <= last_row; row++) {	/* Go down all the rows */
-				if (!Ctrl->E.active) { /* Skip this section */ }
+			last_row = S->n_rows - 1;
+			for (row = n_rows = 0, n_in_rows = 0; row <= last_row; row++) {	/* Go down all the rows */
+				n_in_rows++;
+				if (Ctrl->Z.active && (n_in_rows < Ctrl->Z.first || n_in_rows > Ctrl->Z.last)) continue;	/* Skip if outside limited record range */
+				if (!Ctrl->E.active) {
+					if (Ctrl->T.active[1] && row && is_duplicate_row (S, row)) continue;	/* Skip duplicate records */
+				}
 				else if (Ctrl->E.mode < 0) {	/* Only pass first or last or both of them, skipping all others */
 					if (row > 0 && row < last_row) continue;		/* Always skip the middle of the segment */
 					if (row == 0 && !(-Ctrl->E.mode & 1)) continue;		/* First record, but we are to skip it */
 					if (row == last_row && !(-Ctrl->E.mode & 2)) continue;	/* Last record, but we are to skip it */
 				}
-				else {	/* Only pass modulo E.mode records */
-					if (row % Ctrl->E.mode != 0) continue;
+				else {	/* Only pass modulo E.mode records (this always includes the first row), if -EM then make sure we also write the last row */
+					if ((row % Ctrl->E.mode) != 0) {	/* Check if last row and -EM was used */
+						if (!Ctrl->E.end || row < last_row) continue;
+					}
 				}
 				/* Pull out current virtual row (may consist of a single or many (-A) table rows) */
-				for (tbl_hor = out_col = 0; tbl_hor < n_horizontal_tbls; tbl_hor++) {	/* Number of tables to place horizontally */
+				for (tbl_hor = out_col = tlen = 0; tbl_hor < n_horizontal_tbls; tbl_hor++) {	/* Number of tables to place horizontally */
 					use_tbl = (Ctrl->A.active) ? tbl_hor : tbl_ver;
+					if (D[GMT_IN]->table[use_tbl]->segment[seg]->text && D[GMT_IN]->table[use_tbl]->segment[seg]->text[row]) tlen += strlen (D[GMT_IN]->table[use_tbl]->segment[seg]->text[row]) + 1;	/* String + separator */
 					for (col = 0; col < D[GMT_IN]->table[use_tbl]->segment[seg]->n_columns; col++, out_col++) {	/* Now go across all columns in current table */
 						val[out_col] = D[GMT_IN]->table[use_tbl]->segment[seg]->data[col][row];
 					}
@@ -515,15 +667,24 @@ int GMT_gmtconvert (void *V_API, int mode, void *args) {
 					if (col >= n_cols_in) continue;			/* This column is beyond end of this table */
 					D[GMT_OUT]->table[tbl_ver]->segment[seg]->data[col][n_rows] = val[col];
 				}
+				if (tlen) {
+					if (D[GMT_OUT]->table[tbl_ver]->segment[seg]->text == NULL) D[GMT_OUT]->table[tbl_ver]->segment[seg]->text = gmt_M_memory (GMT, NULL, S->n_rows, char *);
+					D[GMT_OUT]->table[tbl_ver]->segment[seg]->text[n_rows] = calloc (tlen+1, sizeof(char));	/* Space for trailing \0 */
+					for (tbl_hor = 0; tbl_hor < n_horizontal_tbls; tbl_hor++) {	/* Number of tables to place horizontally */
+						use_tbl = (Ctrl->A.active) ? tbl_hor : tbl_ver;
+						if (use_tbl) strcat (D[GMT_OUT]->table[tbl_ver]->segment[seg]->text[n_rows], GMT->current.setting.io_col_separator);
+						if (D[GMT_IN]->table[use_tbl]->segment[seg]->text[row]) strcat (D[GMT_OUT]->table[tbl_ver]->segment[seg]->text[n_rows], D[GMT_IN]->table[use_tbl]->segment[seg]->text[row]);
+					}
+				}
 				n_rows++;
 			}
-			D[GMT_OUT]->table[tbl_ver]->segment[seg]->id = out_seg++;
+			SHo->id = out_seg++;
 			D[GMT_OUT]->table[tbl_ver]->segment[seg]->n_rows = n_rows;	/* Possibly shorter than originally allocated if -E is used */
 			D[GMT_OUT]->table[tbl_ver]->n_records += n_rows;
 			D[GMT_OUT]->n_records = D[GMT_OUT]->table[tbl_ver]->n_records;
-			if (D[GMT_IN]->table[tbl_ver]->segment[seg]->ogr) gmt_duplicate_ogr_seg (GMT, D[GMT_OUT]->table[tbl_ver]->segment[seg], D[GMT_IN]->table[tbl_ver]->segment[seg]);
+			if (SHi->ogr) gmt_duplicate_ogr_seg (GMT, D[GMT_OUT]->table[tbl_ver]->segment[seg], S);
 		}
-		D[GMT_OUT]->table[tbl_ver]->id = tbl_ver;
+		THo->id = tbl_ver;
 	}
 	gmt_M_free (GMT, val);
 
@@ -566,26 +727,29 @@ int GMT_gmtconvert (void *V_API, int mode, void *args) {
 	if (Ctrl->D.active) {	/* Set composite name and io-mode */
 		gmt_M_str_free (Ctrl->Out.file);
 		Ctrl->Out.file = strdup (Ctrl->D.name);
-		D[GMT_OUT]->io_mode = Ctrl->D.mode;
+		DHo->io_mode = Ctrl->D.mode;
 		if (Ctrl->D.origin) {	/* Update IDs */
 			for (tbl = 0; tbl < D[GMT_OUT]->n_tables; tbl++) {	/* For each table */
-				D[GMT_OUT]->table[tbl]->id += Ctrl->D.t_orig;
-				for (seg = 0; seg < D[GMT_OUT]->table[tbl]->n_segments; seg++)	/* For each segment */
-					D[GMT_OUT]->table[tbl]->segment[seg]->id += Ctrl->D.s_orig;
+				THo = gmt_get_DT_hidden (D[GMT_OUT]->table[tbl]);
+				THo->id += Ctrl->D.t_orig;
+				for (seg = 0; seg < D[GMT_OUT]->table[tbl]->n_segments; seg++) {	/* For each segment */
+					SHo = gmt_get_DS_hidden (D[GMT_OUT]->table[tbl]->segment[seg]);
+					SHo->id += Ctrl->D.s_orig;
+				}
 			}
 		}
 	}
 	else {	/* Just register output to stdout or the given file via ->outfile */
 		if (GMT->common.a.output)	/* Must notify the machinery of this output type */
-			D[GMT_OUT]->io_mode = GMT_WRITE_OGR;
+			DHo->io_mode = GMT_WRITE_OGR;
 	}
 
-	if (Ctrl->T.active && gmt_M_file_is_memory (Ctrl->Out.file) && D[GMT_OUT]->n_segments > 1) {
+	if (Ctrl->T.active[0] && gmt_M_file_is_memory (Ctrl->Out.file) && D[GMT_OUT]->n_segments > 1) {
 		/* Since no file is written we must physically collate segments into a single segment per table first */
 		unsigned int flag[3] = {0, 0, GMT_WRITE_SEGMENT};
 		struct GMT_DATASET *D2 = NULL;
 		if ((D2 = GMT_Convert_Data (API, D[GMT_OUT], GMT_IS_DATASET, NULL, GMT_IS_DATASET, flag)) == NULL) {
-			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Error collating each table's segments into a single segment per table.\n");
+			GMT_Report (API, GMT_MSG_NORMAL, "Error collating each table's segments into a single segment per table.\n");
 			Return (API->error);
 		}
 		if (GMT_Destroy_Data (API, &D[GMT_OUT]) != GMT_NOERROR) {	/* Remove the previously registered output dataset */
@@ -593,15 +757,52 @@ int GMT_gmtconvert (void *V_API, int mode, void *args) {
 		}
 		D[GMT_OUT] = D2;	/* Hook up the reformatted dataset */
 	}
-
-	if (GMT_Write_Data (API, GMT_IS_DATASET, GMT_IS_FILE, D[GMT_IN]->geometry, D[GMT_OUT]->io_mode, NULL, Ctrl->Out.file, D[GMT_OUT]) != GMT_NOERROR) {
+	if (Ctrl->N.active) {	/* Sort the output data on selected column before writing */
+		struct GMT_ORDER *Z = NULL;
+		uint64_t max_len = 0;
+		bool do_it = true;
+		char *way[3] = {"descending", "", "ascending"};
+		if (Ctrl->N.col >= D[GMT_OUT]->n_columns) {
+			GMT_Report (API, GMT_MSG_VERBOSE, "Column selected (%d) as sorting key is outside range of valid columns [0-%d].  No sorting performed\n", (int)Ctrl->N.col, (int)(D[GMT_OUT]->n_columns - 1));
+			do_it = false;
+		}
+		else
+			GMT_Report (API, GMT_MSG_VERBOSE, "Sort data based on column %d in %s order\n", (int)Ctrl->N.col, way[Ctrl->N.dir+1]);
+		GMT->current.io.record_type[GMT_OUT] = GMT->current.io.record_type[GMT_IN];
+		for (tbl = 0; do_it && tbl < D[GMT_OUT]->n_tables; tbl++) {	/* Number of output tables */
+			for (seg = 0; seg < D[GMT_OUT]->table[tbl]->n_segments; seg++) {	/* For each segment in the tables */
+				S = D[GMT_OUT]->table[tbl]->segment[seg];	/* Current segment */
+				if (S->n_rows > max_len) {	/* (Re-)allocate memory for sort order array */
+					Z = gmt_M_memory (GMT, Z, S->n_rows, struct GMT_ORDER);
+					max_len =  S->n_rows;
+				}
+				for (row = 0; row < S->n_rows; row++) {	/* Load up value/order struct array */
+					Z[row].value = S->data[Ctrl->N.col][row];
+					Z[row].order = row;
+				}
+				gmt_sort_order (GMT, Z, S->n_rows, Ctrl->N.dir);	/* Sort per segment */
+				gmt_prep_tmp_arrays (GMT, GMT_OUT, S->n_rows, 1);	/* Init or reallocate tmp vector */
+				for (col = 0; col < S->n_columns; col++) {
+					for (row = 0; row < S->n_rows; row++) {	/* Do the shuffle via a temp vector */
+						GMT->hidden.mem_coord[GMT_X][row] = S->data[col][Z[row].order];
+						if (S->text) GMT->hidden.mem_txt[row] = S->text[Z[row].order];
+					}
+					gmt_M_memcpy (S->data[col], GMT->hidden.mem_coord[GMT_X], S->n_rows, double);
+					if (S->text) gmt_M_memcpy (S->text, GMT->hidden.mem_txt, S->n_rows, char *);
+				}
+			}
+		}
+		if (do_it) gmt_M_free (GMT, Z);
+	}
+	
+	if (GMT_Write_Data (API, GMT_IS_DATASET, GMT_IS_FILE, D[GMT_IN]->geometry, DHo->io_mode, NULL, Ctrl->Out.file, D[GMT_OUT]) != GMT_NOERROR) {
 		Return (API->error);
 	}
 	if (prevent_seg_headers) GMT->current.io.skip_headers_on_outout = false;	/* Restore to default if it was changed for file output */
 
-	GMT_Report (API, GMT_MSG_VERBOSE, "%" PRIu64 " tables %s, %" PRIu64 " records passed (input cols = %d; output cols = %d)\n",
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "%" PRIu64 " tables %s, %" PRIu64 " records passed (input cols = %d; output cols = %d)\n",
 		D[GMT_IN]->n_tables, method[Ctrl->A.active], D[GMT_OUT]->n_records, n_cols_in, n_cols_out);
-	if (Ctrl->Q.active || Ctrl->S.active) GMT_Report (API, GMT_MSG_VERBOSE, "Extracted %" PRIu64 " from a total of %" PRIu64 " segments\n", n_out_seg, D[GMT_OUT]->n_segments);
+	if (Ctrl->Q.active || Ctrl->S.active) GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Extracted %" PRIu64 " from a total of %" PRIu64 " segments\n", n_out_seg, D[GMT_OUT]->n_segments);
 
 	Return (GMT_NOERROR);
 }
