@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *    $Id$
  *
- *	Copyright (c) 1991-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  * Brief synopsis: gmtinfo.c will read ASCII or binary tables and report the
@@ -26,12 +25,13 @@
 
 #include "gmt_dev.h"
 
-#define THIS_MODULE_NAME	"gmtinfo"
+#define THIS_MODULE_CLASSIC_NAME	"gmtinfo"
+#define THIS_MODULE_MODERN_NAME	"gmtinfo"
 #define THIS_MODULE_LIB		"core"
 #define THIS_MODULE_PURPOSE	"Get information about data tables"
-#define THIS_MODULE_KEYS	"<D{,>T},>DC"
+#define THIS_MODULE_KEYS	"<D{,>D}"
 #define THIS_MODULE_NEEDS	""
-#define THIS_MODULE_OPTIONS "-:>Vbdefghiors" GMT_OPT("HMm")
+#define THIS_MODULE_OPTIONS "-:>Vabdefghiors" GMT_OPT("HMm")
 
 EXTERN_MSC int gmtlib_geo_C_format (struct GMT_CTRL *GMT);
 EXTERN_MSC unsigned int gmtlib_log_array (struct GMT_CTRL *GMT, double min, double max, double delta, double **array);
@@ -43,6 +43,7 @@ EXTERN_MSC unsigned int gmtlib_log_array (struct GMT_CTRL *GMT, double min, doub
 #define BEST_FOR_SURF	1
 #define BEST_FOR_FFT	2
 #define ACTUAL_BOUNDS	3
+#define BOUNDBOX		4
 
 #define GMT_INFO_TOTAL		1
 #define GMT_INFO_TABLEINFO	2
@@ -50,6 +51,7 @@ EXTERN_MSC unsigned int gmtlib_log_array (struct GMT_CTRL *GMT, double min, doub
 
 struct MINMAX_CTRL {	/* All control options for this program (except common args) */
 	/* active is true if the option has been activated */
+	unsigned int n_files;
 	struct A {	/* -A */
 		bool active;
 		unsigned int mode;	/* 0 reports range for all tables, 1 is per table, 2 is per segment */
@@ -63,7 +65,7 @@ struct MINMAX_CTRL {	/* All control options for this program (except common args
 		unsigned int mode;	/* 0 means center, 1 means use dx granularity */
 		double inc[GMT_MAX_COLUMNS];
 	} D;
-	struct E {	/* -E<L|l|H|h><col> */
+	struct E {	/* -E<L|l|H|h>[<col>] */
 		bool active;
 		bool abs;
 		int mode;	/* -1, 0, +1 */
@@ -73,7 +75,7 @@ struct MINMAX_CTRL {	/* All control options for this program (except common args
 		bool active;
 		int mode;	/*  */
 	} F;
-	struct I {	/* -I[f|p|s]dx[/dy[/<dz>..]] */
+	struct I {	/* -I[b|e|f|p|s]dx[/dy[/<dz>..]] */
 		bool active;
 		unsigned int ncol;
 		unsigned int mode;	/* Nominally 0, unless set to BEST_FOR_SURF, BEST_FOR_FFT or ACTUAL_BOUNDS */
@@ -119,11 +121,11 @@ GMT_LOCAL void Free_Ctrl (struct GMT_CTRL *GMT, struct MINMAX_CTRL *C) {	/* Deal
 }
 
 GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
-	gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
+	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: gmtinfo [<table>] [-Aa|f|s] [-C] [-D[<dx>[/<dy>]] [-E<L|l|H|h><col>] [-Fi|d|t] [-I[p|f|s]<dx>[/<dy>[/<dz>..]]\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t[-L] [-S[x][y]] [-T<dz>[+c<col>]] [%s] [%s] [%s] [%s]\n\t[%s] [%s]\n\t[%s] [%s]\n\t[%s] [%s] [%s] [%s]\n\n",
-		GMT_V_OPT, GMT_bi_OPT, GMT_d_OPT, GMT_e_OPT, GMT_f_OPT, GMT_g_OPT, GMT_h_OPT, GMT_i_OPT, GMT_o_OPT, GMT_r_OPT, GMT_s_OPT, GMT_colon_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "usage: %s [<table>] [-Aa|f|s] [-C] [-D[<dx>[/<dy>]] [-E<L|l|H|h>[<col>]] [-Fi|d|t] [-I[b|e|f|p|s]<dx>[/<dy>[/<dz>..]]\n", name);
+	GMT_Message (API, GMT_TIME_NONE, "\t[-L] [-S[x][y]] [-T<dz>[+c<col>]] [%s] [%s] [%s] [%s]\n\t[%s] [%s] [%s]\n\t[%s] [%s]\n\t[%s] [%s] [%s] [%s] [%s]\n\n",
+		GMT_V_OPT, GMT_a_OPT, GMT_bi_OPT, GMT_d_OPT, GMT_e_OPT, GMT_f_OPT, GMT_g_OPT, GMT_h_OPT, GMT_i_OPT, GMT_o_OPT, GMT_r_OPT, GMT_s_OPT, GMT_colon_OPT, GMT_PAR_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
@@ -141,13 +143,14 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t   d: Dataset: One record per segment with tbl_no, seg_no, nrows, start_rec, stop_rec.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   t: Tables:  Same as D but the counts resets per table.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-I Return textstring -Rw/e/s/n to nearest multiple of dx/dy (assumes at least 2 columns).\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Give -I- to just report the min/max extent in the -Rw/e/s/n string (no multiples).\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   Give -Ie to just report the min/max extent in the -Rw/e/s/n string (no multiples).\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   If -C is set then no -R string is issued.  Instead, the number of increments\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   given determines how many columns are rounded off to the nearest multiple.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   If only one increment is given we also use it for the second column (for backwards compatibility).\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   To override this behavior, use -Ip<dx>.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   If input data are regularly distributed we use observed phase shifts in determining -R [no phase shift]\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t     and allow -r to change from gridline-registration to pixel-registration.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   Use -Ib to report the bounding box polygon for the data files (or segments; see -A).\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   Use -If<dx>[/<dy>] to report an extended region optimized for fastest results in FFTs.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   Use -Is<dx>[/<dy>] to report an extended region optimized for fastest results in surface.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-L Determine limiting region. With -I it rounds inward so bounds are within data range.\n");
@@ -158,7 +161,9 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t   -S or -Sxy leaves space for both error bars using values in third&fourth (2&3) columns.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-T Return textstring -Tzmin/zmax/dz to nearest multiple of the given dz.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   Calculations are based on the first (0) column; append +c<col> to use another column.\n");
-	GMT_Option (API, "V,bi2,d,e,f,g,h,i,o,r,s,:,.");
+	GMT_Option (API, "V,a");
+	if (gmt_M_showusage (API)) GMT_Message (API, GMT_TIME_NONE, "\t   Reports the names and data types of the aspatial fields.\n");
+	GMT_Option (API, "bi2,d,e,f,g,h,i,o,r,s,:,.");
 	
 	return (GMT_MODULE_USAGE);
 }
@@ -182,6 +187,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct MINMAX_CTRL *Ctrl, struct GMT_
 
 			case '<':	/* Input files */
 				if (!gmt_check_filearg (GMT, '<', opt->arg, GMT_IN, GMT_IS_DATASET)) n_errors++;
+				Ctrl->n_files++;
 				break;
 
 			/* Processes program-specific parameters */
@@ -219,11 +225,13 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct MINMAX_CTRL *Ctrl, struct GMT_
 				switch (opt->arg[0]) {
 					case 'L':
 						Ctrl->E.abs = true;
+						/* fall through on purpose to 'l' */
 					case 'l':
 						Ctrl->E.mode = -1;
 						break;
 					case 'H':
 						Ctrl->E.abs = true;
+						/* fall through on purpose to 'h' */
 					case 'h':
 						Ctrl->E.mode = +1;
 						break;
@@ -253,10 +261,11 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct MINMAX_CTRL *Ctrl, struct GMT_
 					case 'p': special = true; break;
 					case 'f': Ctrl->I.mode = BEST_FOR_FFT; break;
 					case 's': Ctrl->I.mode = BEST_FOR_SURF; break;
-					case '-': Ctrl->I.mode = ACTUAL_BOUNDS; break;
+					case 'b': Ctrl->I.mode = BOUNDBOX; break;
+					case 'e': case '-': Ctrl->I.mode = ACTUAL_BOUNDS; break;	/* -I- is backwards compatible */
 					default: j = 0;	break;
 				}
-				Ctrl->I.ncol = (Ctrl->I.mode == ACTUAL_BOUNDS) ? 2 : gmt_getincn (GMT, &opt->arg[j], Ctrl->I.inc, GMT_MAX_COLUMNS);
+				Ctrl->I.ncol = (Ctrl->I.mode == ACTUAL_BOUNDS || Ctrl->I.mode == BOUNDBOX) ? 2 : gmt_getincn (GMT, &opt->arg[j], Ctrl->I.inc, GMT_MAX_COLUMNS);
 				break;
 			case 'L':	/* Detect limiting range */
 				Ctrl->L.active = true;
@@ -274,7 +283,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct MINMAX_CTRL *Ctrl, struct GMT_
 			case 'T':	/* makecpt inc string */
 				Ctrl->T.active = true;
 				if ((c = strchr (opt->arg, '/')) && gmt_M_compat_check (GMT, 5)) {	/* Let it slide for now */
-					GMT_Report (API, GMT_MSG_COMPAT, "Warning: Option -T<inc>[/<col>] syntax is deprecated; use -T<inc>[+c<col>] instead.\n");
+					GMT_Report (API, GMT_MSG_COMPAT, "Option -T<inc>[/<col>] syntax is deprecated; use -T<inc>[+c<col>] instead.\n");
 					j = sscanf (opt->arg, "%lf/%d", &Ctrl->T.inc, &Ctrl->T.col);
 					if (j == 1) Ctrl->T.col = 0;
 				}
@@ -294,6 +303,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct MINMAX_CTRL *Ctrl, struct GMT_
 
 			case 'b':	/* -b[i]c will land here */
 				if (gmt_M_compat_check (GMT, 4)) break;
+				/* Otherwise we fall through on purpose to get an error */
 
 			default:	/* Report bad options */
 				n_errors += gmt_default_error (GMT, opt->option);
@@ -314,13 +324,13 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct MINMAX_CTRL *Ctrl, struct GMT_
 		Ctrl->D.ncol = 2;
 	}
 	n_errors += gmt_M_check_condition (GMT, Ctrl->D.active && !Ctrl->I.active, "Syntax error: -D requires -I\n");
-	n_errors += gmt_M_check_condition (GMT, Ctrl->I.active && !Ctrl->C.active && Ctrl->I.ncol < 2,
+	n_errors += gmt_M_check_condition (GMT, Ctrl->I.active && Ctrl->I.mode != BOUNDBOX && !Ctrl->C.active && Ctrl->I.ncol < 2,
 	                                   "Syntax error: -Ip requires -C\n");
 	n_errors += gmt_M_check_condition (GMT, Ctrl->I.active && Ctrl->T.active,
 	                                   "Syntax error: Only one of -I and -T can be specified\n");
 	n_errors += gmt_M_check_condition (GMT, Ctrl->T.active && Ctrl->T.inc <= 0.0 ,
 	                                   "Syntax error -T option: Must specify a positive increment\n");
-	for (k = 0; Ctrl->I.active && k < Ctrl->I.ncol; k++) {
+	for (k = 0; Ctrl->I.active && Ctrl->I.mode != BOUNDBOX && k < Ctrl->I.ncol; k++) {
 		n_errors += gmt_M_check_condition (GMT, Ctrl->I.mode != ACTUAL_BOUNDS && Ctrl->I.inc[k] <= 0.0,
 		                                   "Syntax error -I option: Must specify positive increment(s)\n");
 	}
@@ -329,23 +339,27 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct MINMAX_CTRL *Ctrl, struct GMT_
 }
 
 #define bailout(code) {gmt_M_free_options (mode); return (code);}
-#define Return(code) {Free_Ctrl (GMT, Ctrl); gmt_M_free (GMT, xyzmin); gmt_M_free (GMT, xyzmax); gmt_M_free (GMT, xyzminL); gmt_M_free (GMT, xyzmaxL); gmt_M_free (GMT, Q); gmt_end_module (GMT, GMT_cpy); bailout (code);}
+#define Return(code) {Free_Ctrl (GMT, Ctrl); gmt_M_free (GMT, xyzmin); gmt_M_free (GMT, xyzmax); gmt_M_free (GMT, xyzminL); gmt_M_free (GMT, lonmin);  gmt_M_free (GMT, lonmax); gmt_M_free (GMT, xyzmaxL); gmt_M_free (GMT, Q); gmt_M_free (GMT, Z); gmt_M_free (GMT, Out); gmt_end_module (GMT, GMT_cpy); bailout (code);}
 
 int GMT_gmtinfo (void *V_API, int mode, void *args) {
 	bool got_stuff = false, first_data_record, give_r_string = false;
 	bool brackets = false, work_on_abs_value, do_report, done, full_range = false;
 	int i, j, error = 0, col_type[GMT_MAX_COLUMNS];
-	unsigned int fixed_phase[2] = {1, 1}, min_cols, o_mode, save_range;
-	uint64_t col, ncol = 0, n = 0;
+	unsigned int fixed_phase[2] = {1, 1}, min_cols, save_range, n_items = 0;
+	uint64_t col, ncol = 0, n = 0, n_alloc = GMT_BIG_CHUNK;
 
-	char file[GMT_BUFSIZ] = {""}, chosen[GMT_BUFSIZ] = {""}, record[GMT_BUFSIZ] = {""}, buffer[GMT_BUFSIZ] = {""}, delimiter[2] = {""};
+	char file[PATH_MAX] = {""}, chosen[GMT_BUFSIZ] = {""}, record[GMT_BUFSIZ] = {""};
+	char buffer[GMT_BUFSIZ] = {""}, delimiter[2] = {""}, *t_ptr = NULL;
 
 	double *xyzmin = NULL, *xyzmax = NULL, *in = NULL, *dchosen = NULL, phase[2] = {0.0, 0.0}, this_phase, off;
 	double west = 0.0, east = 0.0, south = 0.0, north = 0.0, low, high, value, e_min = DBL_MAX, e_max = -DBL_MAX;
-	double *xyzminL = NULL, *xyzmaxL = NULL;
+	double *xyzminL = NULL, *xyzmaxL = NULL, *d_ptr = NULL;
+	double out[5] = {0.0, 0.0, 0.0, 0.0, 0.0}, *lonmin = NULL, *lonmax = NULL;
 
 	struct GMT_QUAD *Q = NULL;
+	struct GMT_RANGE **Z = NULL;
 	struct MINMAX_CTRL *Ctrl = NULL;
+	struct GMT_RECORD *In = NULL, *Out = NULL;
 	struct GMT_CTRL *GMT = NULL, *GMT_cpy = NULL;
 	struct GMT_OPTION *options = NULL;
 	struct GMTAPI_CTRL *API = gmt_get_api_ptr (V_API);	/* Cast from void to GMTAPI_CTRL pointer */
@@ -356,41 +370,46 @@ int GMT_gmtinfo (void *V_API, int mode, void *args) {
 	if (mode == GMT_MODULE_PURPOSE) return (usage (API, GMT_MODULE_PURPOSE));	/* Return the purpose of program */
 	options = GMT_Create_Options (API, mode, args);	if (API->error) return (API->error);	/* Set or get option list */
 
-	if (options && options->option == GMT_OPT_USAGE) bailout (usage (API, GMT_USAGE));	/* Return the usage message */
-	if (options && options->option == GMT_OPT_SYNOPSIS) bailout (usage (API, GMT_SYNOPSIS));	/* Return the synopsis */
+	if ((error = gmt_report_usage (API, options, 1, usage)) != GMT_NOERROR) bailout (error);	/* Give usage if requested */
 
 	/* Parse the command-line arguments */
 
-	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
 	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
 	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
 	if ((error = parse (GMT, Ctrl, options)) != 0) Return (error);
 
 	/*---------------------------- This is the gmtinfo main code ----------------------------*/
 
-	GMT_Report (API, GMT_MSG_VERBOSE, "Processing input table data\n");
+	if (Ctrl->n_files > 1 && GMT->common.a.active) {
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Cannot give multiple files when -a is selected!\n");
+		Return (GMT_DIM_TOO_LARGE);
+	}
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Processing input table data\n");
 	
 	if (Ctrl->F.active) {	/* Special case of reporting on record numbers */
 		/* This is best done by reading the whole thing in */
 		struct GMT_DATASET *D = NULL;
 		struct GMT_DATATABLE *T = NULL;
 		struct GMT_DATASEGMENT *S = NULL;
-		double out[5] = {0.0, 0.0, 0.0, 0.0, 0.0};
 		uint64_t tbl, seg, start_rec = 0;
-		if (GMT_Init_IO (API, GMT_IS_DATASET, GMT_IS_NONE, GMT_IN, GMT_ADD_DEFAULT, 0, options) != GMT_NOERROR) {
+		
+		if (GMT_Init_IO (API, GMT_IS_DATASET, GMT_IS_PLP, GMT_IN, GMT_ADD_DEFAULT, 0, options) != GMT_NOERROR) {
 			Return (API->error);	/* Establishes data files or stdin */
 		}
 		if ((D = GMT_Read_Data (API, GMT_IS_DATASET, GMT_IS_FILE, 0, GMT_READ_NORMAL, NULL, NULL, NULL)) == NULL) {
 			Return (API->error);
 		}
-		if (GMT_Init_IO (API, GMT_IS_DATASET, GMT_IS_NONE, GMT_OUT, GMT_ADD_DEFAULT, 0, options) != GMT_NOERROR) {	/* Establishes data output */
+		if (GMT_Init_IO (API, GMT_IS_DATASET, GMT_IS_PLP, GMT_OUT, GMT_ADD_DEFAULT, 0, options) != GMT_NOERROR) {	/* Establishes data output */
 			Return (API->error);
 		}
 		if (GMT_Begin_IO (API, GMT_IS_DATASET, GMT_OUT, GMT_HEADER_OFF) != GMT_NOERROR) {
 			Return (API->error);
 		}
-		if ((error = gmt_set_cols (GMT, GMT_OUT, 5)) != 0) Return (error);
+		if ((error = GMT_Set_Columns (API, GMT_OUT, 5, GMT_COL_FIX_NO_TEXT)) != GMT_NOERROR) Return (error);
 		for (col = 0; col < 5; col++) GMT->current.io.col_type[GMT_OUT][col] = GMT_IS_FLOAT;
+		
+		Out = gmt_new_record (GMT, out, NULL);	/* Since we only need to worry about numerics in this module */
 
 		switch (Ctrl->F.mode) {
 			case GMT_INFO_TOTAL:	/* Report total number of tables */
@@ -401,7 +420,7 @@ int GMT_gmtinfo (void *V_API, int mode, void *args) {
 					out[3] += (double)(D->table[tbl]->n_headers);
 				if (D->n_segments == 1) out[4] -= 1.0;	/* Still unsure about this.  How do we know file actually had a segment header? */
 				out[4] = out[3] + out[1] + out[2];
-				GMT_Put_Record (API, GMT_WRITE_DOUBLE, out);
+				GMT_Put_Record (API, GMT_WRITE_DATA, Out);
 				break;
 			case GMT_INFO_TABLEINFO:	/* Virtual data set or individual tables */
 			case GMT_INFO_DATAINFO:
@@ -420,7 +439,7 @@ int GMT_gmtinfo (void *V_API, int mode, void *args) {
 						out[2] = (double)S->n_rows;
 						out[3] = (double)start_rec;
 						out[4] = (double)(start_rec + S->n_rows - 1);
-						GMT_Put_Record (API, GMT_WRITE_DOUBLE, out);
+						GMT_Put_Record (API, GMT_WRITE_DATA, Out);
 						start_rec += S->n_rows;
 					}
 				}
@@ -435,8 +454,8 @@ int GMT_gmtinfo (void *V_API, int mode, void *args) {
 		Return (GMT_NOERROR);	/* We are done */
 	}
 	
-	give_r_string = (Ctrl->I.active && !Ctrl->C.active);
-	delimiter[0] = (Ctrl->C.active) ? '\t' : '/';
+	give_r_string = (Ctrl->I.active && !(Ctrl->I.mode == BOUNDBOX || Ctrl->C.active));
+	delimiter[0] = (Ctrl->C.active || Ctrl->I.mode == BOUNDBOX) ? '\t' : '/';
 	delimiter[1] = '\0';
 	off = (GMT->common.R.active[GSET]) ? 0.5 : 0.0;
 
@@ -446,53 +465,68 @@ int GMT_gmtinfo (void *V_API, int mode, void *args) {
 		if (!strcmp (GMT->current.setting.format_geo_out, "D")) {
 			strcpy (GMT->current.setting.format_geo_out, "+D");
 			gmt_M_err_fail (GMT, gmtlib_geo_C_format (GMT), "");
-			GMT_Report (API, GMT_MSG_VERBOSE, "Warning: FORMAT_GEO_OUT reset from D to %s to ensure east > west\n",
+			GMT_Report (API, GMT_MSG_VERBOSE, "FORMAT_GEO_OUT reset from D to %s to ensure east > west\n",
 			            GMT->current.setting.format_geo_out);
 		}
 		else if (!strcmp (GMT->current.setting.format_geo_out, "ddd:mm:ss")) {
 			strcpy (GMT->current.setting.format_geo_out, "ddd:mm:ssF");
 			gmt_M_err_fail (GMT, gmtlib_geo_C_format (GMT), "");
-			GMT_Report (API, GMT_MSG_VERBOSE, "Warning: FORMAT_GEO_OUT reset from ddd:mm:ss to %s to ensure east > west\n",
+			GMT_Report (API, GMT_MSG_VERBOSE, "FORMAT_GEO_OUT reset from ddd:mm:ss to %s to ensure east > west\n",
 			            GMT->current.setting.format_geo_out);
 		}
 	}
 
-	if ((error = gmt_set_cols (GMT, GMT_IN, 0)) != 0) Return (error);
-	o_mode = (Ctrl->C.active) ? GMT_IS_DATASET : GMT_IS_TEXTSET;
-	if (GMT_Init_IO (API, GMT_IS_DATASET, GMT_IS_NONE, GMT_IN,  GMT_ADD_DEFAULT, 0, options) != GMT_NOERROR) {	/* Establishes data input */
+	if ((error = GMT_Set_Columns (API, GMT_IN, 0, GMT_COL_FIX)) != GMT_NOERROR) Return (error);
+
+	if (GMT_Init_IO (API, GMT_IS_DATASET, GMT_IS_PLP, GMT_IN,  GMT_ADD_DEFAULT, 0, options) != GMT_NOERROR) {	/* Establishes data input */
 		Return (API->error);
 	}
-	if (GMT_Init_IO (API, o_mode, GMT_IS_NONE, GMT_OUT, GMT_ADD_DEFAULT, 0, options) != GMT_NOERROR) {	/* Establishes data output */
+	if (GMT_Init_IO (API, GMT_IS_DATASET, GMT_IS_PLP, GMT_OUT, GMT_ADD_DEFAULT, 0, options) != GMT_NOERROR) {	/* Establishes data output */
 		Return (API->error);
 	}
 
-	if (GMT_Begin_IO (API, GMT_IS_DATASET,  GMT_IN, GMT_HEADER_ON) != GMT_NOERROR) {	/* Enables data input and sets access mode */
+	if (GMT_Begin_IO (API, GMT_IS_DATASET, GMT_IN, GMT_HEADER_ON) != GMT_NOERROR) {	/* Enables data input and sets access mode */
 		Return (API->error);	/* Enables data output and sets access mode */
 	}
-	if (GMT_Begin_IO (API, o_mode, GMT_OUT, GMT_HEADER_OFF) != GMT_NOERROR) {
-		Return (API->error);
-	}
-	if (GMT_Set_Geometry (API, GMT_OUT, GMT_IS_NONE) != GMT_NOERROR) {	/* Sets output geometry */
+	if (GMT_Begin_IO (API, GMT_IS_DATASET, GMT_OUT, GMT_HEADER_OFF) != GMT_NOERROR) {
 		Return (API->error);
 	}
 
-	if (Ctrl->C.active) {	/* Must set output column types since each input col will produce two output cols. */
-		gmt_M_memcpy (col_type, GMT->current.io.col_type[GMT_OUT], GMT_MAX_COLUMNS, int);	/* Save previous output col types */
-		for (col = 0; col < GMT_MAX_COLUMNS/2; col++)
-			GMT->current.io.col_type[GMT_OUT][2*col] = GMT->current.io.col_type[GMT_OUT][2*col+1] = GMT->current.io.col_type[GMT_IN][col];
-	}
-		
 	save_range = GMT->current.io.geo.range;
+	GMT->current.io.geo.range = GMT_IGNORE_RANGE;	/* Ensure no adjustment will happen since we control it here */
+	d_ptr = (Ctrl->C.active || Ctrl->E.active) ? GMT->current.io.curr_rec : ((Ctrl->I.mode == BOUNDBOX) ? out : NULL);
+	t_ptr = (d_ptr && !Ctrl->E.active) ? NULL : record;
+	Out = gmt_new_record (GMT, d_ptr, t_ptr);
 	first_data_record = true;
 	done = false;
 	while (!done) {	/* Keep returning records until we reach EOF of last file */
-		in = GMT_Get_Record (API, GMT_READ_DATA | GMT_READ_FILEBREAK, NULL);
+		In = GMT_Get_Record (API, GMT_READ_DATA | GMT_READ_FILEBREAK, NULL);
 
 		if (gmt_M_rec_is_error (GMT)) Return (GMT_RUNTIME_ERROR);
 		if (gmt_M_rec_is_table_header (GMT)) continue;	/* Skip table headers */
-		if ((gmt_M_rec_is_segment_header (GMT) && Ctrl->A.mode != REPORT_PER_SEGMENT)) continue;	/* Since we are not reporting per segment they are just headers as far as we are concerned */
+		if (gmt_M_rec_is_segment_header (GMT) || gmt_M_rec_is_file_break (GMT) || gmt_M_rec_is_eof (GMT)) {	/* Need per segment scrutiny of longitudes */
+			bool alloc_more = (n_items == n_alloc);
+			if (alloc_more) n_alloc <<= 1;	/* Double the memory */
+			for (col = 0; col < ncol; col++) if (GMT->current.io.col_type[GMT_IN][col] == GMT_IS_LON) {	/* Must finalize longitudes first */
+				j = gmt_quad_finalize (GMT, &Q[col]);
+				if (alloc_more)
+					Z[col] = gmt_M_memory (GMT, Z[col], n_alloc, struct GMT_RANGE);
+				Z[col][n_items].west = Q[col].min[j];	Z[col][n_items].east = Q[col].max[j];
+				n_items++;
+			}
+			gmt_quad_reset (GMT, Q, ncol);
+			if (gmt_M_rec_is_segment_header (GMT) && Ctrl->A.mode != REPORT_PER_SEGMENT) continue;	/* Since we are not reporting per segment they are just headers as far as we are concerned */
+		}
 		
 		if (gmt_M_rec_is_segment_header (GMT) || (Ctrl->A.mode == REPORT_PER_TABLE && gmt_M_rec_is_file_break (GMT)) || gmt_M_rec_is_eof (GMT)) {	/* Time to report */
+			if (Ctrl->A.active) {
+				if (Ctrl->A.mode == REPORT_PER_TABLE && gmt_M_rec_is_file_break (GMT)) {
+					e_min = DBL_MAX; e_max = -DBL_MAX;
+				}
+				else if (Ctrl->A.mode == REPORT_PER_SEGMENT && gmt_M_rec_is_segment_header (GMT)) {
+					e_min = DBL_MAX; e_max = -DBL_MAX;
+				}
+			}
 			if (gmt_M_rec_is_segment_header (GMT) && GMT->current.io.seg_no == 0) continue;	/* Very first segment header means there is no prior segment to report on yet */
 			if (gmt_M_rec_is_eof (GMT)) {	/* We are done after this since we hit EOF */
 				done = true;
@@ -504,10 +538,9 @@ int GMT_gmtinfo (void *V_API, int mode, void *args) {
 			
 			do_report = true;
 			if (Ctrl->L.active && !gmt_M_rec_is_eof (GMT)) do_report = false;	/* Only final report for -L */
- 			for (col = 0; col < ncol; col++) if (GMT->current.io.col_type[GMT_IN][col] == GMT_IS_LON) {	/* Must finalize longitudes first */
-				j = gmt_quad_finalize (GMT, &Q[col]);
-				GMT->current.io.geo.range = Q[col].range[j];		/* Override this setting explicitly */
-				xyzmin[col] = Q[col].min[j];	xyzmax[col] = Q[col].max[j];
+ 			for (col = 0; col < ncol; col++) if (GMT->current.io.col_type[GMT_IN][col] == GMT_IS_LON) {	/* Must update longitudes separately */
+				gmt_find_range (GMT, Z[col], n_items, &xyzmin[col], &xyzmax[col]);
+				n_items = 0;
 			}
 			if (Ctrl->I.active) {	/* Must report multiples of dx/dy etc */
 				if (n > 1 && fixed_phase[GMT_X] && fixed_phase[GMT_Y]) {	/* Got xy[z] data that lined up on a grid, so use the common phase shift */
@@ -525,6 +558,21 @@ int GMT_gmtinfo (void *V_API, int mode, void *args) {
 					west  = xyzmin[GMT_X];	east  = xyzmax[GMT_X];
 					south = xyzmin[GMT_Y];	north = xyzmax[GMT_Y];
 				}
+				else if (Ctrl->I.mode == BOUNDBOX) {	/* Write out bounding box */
+					sprintf (buffer, "Bounding box for table data");
+					GMT_Put_Record (API, GMT_WRITE_SEGMENT_HEADER, buffer);
+					out[GMT_X] = xyzmin[GMT_X];	out[GMT_Y] = xyzmin[GMT_Y];
+					GMT_Put_Record (API, GMT_WRITE_DATA, Out);
+					out[GMT_X] = xyzmax[GMT_X];
+					GMT_Put_Record (API, GMT_WRITE_DATA, Out);
+					out[GMT_Y] = xyzmax[GMT_Y];
+					GMT_Put_Record (API, GMT_WRITE_DATA, Out);
+					out[GMT_X] = xyzmin[GMT_X];
+					GMT_Put_Record (API, GMT_WRITE_DATA, Out);
+					out[GMT_Y] = xyzmin[GMT_Y];
+					GMT_Put_Record (API, GMT_WRITE_DATA, Out);
+					do_report = false;
+				}
 				else if (Ctrl->L.active) { /* Round down to nearest inc for this segment or table */
 					west  = (ceil ((xyzmin[GMT_X] - phase[GMT_X]) / Ctrl->I.inc[GMT_X]) - off) * Ctrl->I.inc[GMT_X] + phase[GMT_X];
 					east  = (floor  ((xyzmax[GMT_X] - phase[GMT_X]) / Ctrl->I.inc[GMT_X]) + off) * Ctrl->I.inc[GMT_X] + phase[GMT_X];
@@ -549,15 +597,15 @@ int GMT_gmtinfo (void *V_API, int mode, void *args) {
 					if (gmt_M_is_geographic (GMT, GMT_IN)) {	/* Must make sure we don't get outside valid bounds */
 						if (south < -90.0) {
 							south = -90.0;
-							GMT_Report (API, GMT_MSG_VERBOSE, "Warning: Using -I caused south to become < -90. Reset to -90.\n");
+							GMT_Report (API, GMT_MSG_VERBOSE, "Using -I caused south to become < -90. Reset to -90.\n");
 						}
 						if (north > 90.0) {
 							north = 90.0;
-							GMT_Report (API, GMT_MSG_VERBOSE, "Warning: Using -I caused north to become > +90. Reset to +90.\n");
+							GMT_Report (API, GMT_MSG_VERBOSE, "Using -I caused north to become > +90. Reset to +90.\n");
 						}
 						if (fabs (east - west) > 360.0) {
 							GMT_Report (API, GMT_MSG_VERBOSE,
-							            "Warning: Using -I caused longitude range to exceed 360. Reset to a range of 360.\n");
+							            "Using -I caused longitude range to exceed 360. Reset to a range of 360.\n");
 							west = (west < 0.0) ? -180.0 : 0.0;
 							east = (west < 0.0) ? +180.0 : 360.0;
 							full_range = true;
@@ -581,7 +629,7 @@ int GMT_gmtinfo (void *V_API, int mode, void *args) {
 					west  -= sub * Ctrl->I.inc[GMT_X];		east  += add * Ctrl->I.inc[GMT_X];
 					sub = (out_dim[GMT_Y] - in_dim[GMT_Y]) / 2;	add = out_dim[GMT_Y] - in_dim[GMT_Y] - sub;
 					south -= sub * Ctrl->I.inc[GMT_Y];		north += add * Ctrl->I.inc[GMT_Y];
-					GMT_Report (API, GMT_MSG_VERBOSE,
+					GMT_Report (API, GMT_MSG_LONG_VERBOSE,
 					            "Initial -R: %g/%g/%g/%g [n_columns = %u n_rows = %u] --> Suggested -R:  %g/%g/%g/%g [n_columns = %u n_rows = %u].\n",
 						ww, ee, ss, nn, in_dim[GMT_X], in_dim[GMT_Y], west, east, south, north, out_dim[GMT_X], out_dim[GMT_Y]);
 				}
@@ -606,18 +654,15 @@ int GMT_gmtinfo (void *V_API, int mode, void *args) {
 				i = strip_blanks_and_output (GMT, buffer, Ctrl->T.inc, Ctrl->T.col);	strcat (record, &buffer[i]);
 			}
 			else if (Ctrl->E.active) {	/* Return extreme record */
-				int sep = 0;
-				if (GMT->common.b.active[GMT_IN]) {	/* Make ASCII record */
-					gmt_M_memset (chosen, GMT_BUFSIZ, char);
-					for (col = 0; col < ncol; col++) {	/* Report min/max for each column in the format controlled by -C */
-						gmt_add_to_record (GMT, chosen, dchosen[col], col, GMT_OUT, sep);
-						sep = 1;
-					}
-				}
-				strncpy (record, chosen, GMT_BUFSIZ);
+				gmt_M_memcpy (Out->data, dchosen, ncol, double);
+				if (Out->text) strncpy (record, chosen, GMT_BUFSIZ);
 			}
 			else {				/* Return min/max for each column */
 				if (!Ctrl->C.active) {	/* Want info about each item */
+					if (GMT->common.a.active) {	/* Write text record with name[type] of the aspatial fields */
+						gmt_list_aspatials (GMT, record);
+						GMT_Put_Record (API, GMT_WRITE_DATA, Out);
+					}
 					record[0] = '\0';	/* Start with blank slate */
 					if (Ctrl->A.mode == REPORT_PER_DATASET && GMT->current.io.tbl_no > 1)	/* More than one table given */
 						strcpy (record, "dataset");
@@ -675,35 +720,38 @@ int GMT_gmtinfo (void *V_API, int mode, void *args) {
 					}
 				}
 			}
-			if (do_report) {
-				if (Ctrl->C.active) {	/* Plain data record */
-					GMT_Put_Record (API, GMT_WRITE_DATA, GMT->current.io.curr_rec);	/* Write data record to output destination */
-				}
-				else {
-					GMT_Put_Record (API, GMT_WRITE_TEXT, record);	/* Write text record to output destination */
-				}
-			}
+			if (do_report)
+				GMT_Put_Record (API, GMT_WRITE_DATA, Out);	/* Write data record to output destination */
 			got_stuff = true;		/* We have at least reported something */
 			for (col = 0; col < ncol; col++) {	/* Reset counters for next block */
 				xyzmin[col] = DBL_MAX;
 				xyzmax[col] = -DBL_MAX;
+				lonmin[col] = lonmax[col] = 0.0;
 			}
-			gmt_quad_reset (GMT, Q, ncol);
 			n = 0;
 			file[0] = '\0';
 			fixed_phase[GMT_X] = fixed_phase[GMT_Y] = 1;	/* Get ready for next batch */
-			if (done || do_report || gmt_M_rec_is_file_break (GMT)) continue;	/* We are done OR have no data record to process yet */
+			if (done || do_report || Ctrl->I.mode == BOUNDBOX || gmt_M_rec_is_file_break (GMT)) continue;	/* We are done OR have no data record to process yet */
 		}
 		if (gmt_M_rec_is_file_break (GMT)) continue;
 
 		/* We get here once we have read a data record */
-		
+		if ((in = In->data) == NULL) {	/* Only need to process numerical part here */
+			GMT_Report (API, GMT_MSG_NORMAL, "No data columns to work with - exiting\n");
+			Return (GMT_DIM_TOO_SMALL);
+		}
 		if (first_data_record) {	/* First time we read data, we must allocate arrays based on the number of columns */
 
+			if (Ctrl->C.active) {	/* Must set output column types since each input col will produce two output cols. */
+				gmt_M_memcpy (col_type, GMT->current.io.col_type[GMT_OUT], GMT_MAX_COLUMNS, int);	/* Save previous output col types */
+				for (col = 0; col < GMT_MAX_COLUMNS/2; col++)
+					GMT->current.io.col_type[GMT_OUT][2*col] = GMT->current.io.col_type[GMT_OUT][2*col+1] = GMT->current.io.col_type[GMT_IN][col];
+			}
+
 			ncol = gmt_get_cols (GMT, GMT_IN);
 			if (Ctrl->E.active) {
 				if (Ctrl->E.col == UINT_MAX) Ctrl->E.col = ncol - 1;	/* Default is last column */
-				if (GMT->common.b.active[GMT_IN]) dchosen = gmt_M_memory (GMT, NULL, ncol, double);
+				dchosen = gmt_M_memory (GMT, NULL, ncol, double);
 			}
 			min_cols = 2;	if (Ctrl->S.xbar) min_cols++;	if (Ctrl->S.ybar) min_cols++;
 			if (Ctrl->S.active && min_cols > ncol) {
@@ -720,23 +768,47 @@ int GMT_gmtinfo (void *V_API, int mode, void *args) {
 			}
 			if (Ctrl->T.active) ncol = Ctrl->T.col + 1;
 			if (give_r_string) ncol = 2;	/* Since we only will concern ourselves with lon/lat or x/y */
-
+			if (ncol == 0) {
+				GMT_Report (API, GMT_MSG_NORMAL, "Something went wrong, ncol is still = 0. Maybe a call from an external program? Have to abort here.\n");
+				Return (GMT_RUNTIME_ERROR);
+			}
+			
 			/* Now we know number of columns, so allocate memory */
 
 			Q = gmt_quad_init (GMT, ncol);
+			Z = gmt_M_memory (GMT, NULL, ncol, struct GMT_RANGE *);
 			xyzmin = gmt_M_memory (GMT, NULL, ncol, double);
 			xyzmax = gmt_M_memory (GMT, NULL, ncol, double);
 			xyzminL = gmt_M_memory (GMT, NULL, ncol, double);
 			xyzmaxL = gmt_M_memory (GMT, NULL, ncol, double);
+			lonmin = gmt_M_memory (GMT, NULL, ncol, double);
+			lonmax = gmt_M_memory (GMT, NULL, ncol, double);
 
 			for (col = 0; col < ncol; col++) {	/* Initialize */
 				xyzmin[col] = xyzmaxL[col] = DBL_MAX;
 				xyzmax[col] = xyzminL[col] = -DBL_MAX;
+				if (GMT->current.io.col_type[GMT_IN][col] == GMT_IS_LON)
+					Z[col] = gmt_M_memory (GMT, NULL, n_alloc, struct GMT_RANGE);
 			}
 			n = 0;
 			if (Ctrl->I.active && ncol < 2 && !Ctrl->C.active) Ctrl->I.active = false;
 			first_data_record = false;
-			if (Ctrl->C.active && (error = gmt_set_cols (GMT, GMT_OUT, 2*ncol)) != 0) Return (error);
+			if (Ctrl->C.active) {
+				if ((error = GMT_Set_Columns (API, GMT_OUT, (unsigned int)(2*ncol), GMT_COL_FIX_NO_TEXT)) != GMT_NOERROR)
+					Return (error);
+			}
+			else if (Ctrl->E.active) {
+				if ((error = GMT_Set_Columns (API, GMT_OUT, (unsigned int)ncol, (In->text) ? GMT_COL_FIX : GMT_COL_FIX_NO_TEXT)) != GMT_NOERROR)
+					Return (error);
+			}
+			else if (Ctrl->I.mode == BOUNDBOX) {
+				if ((error = GMT_Set_Columns (API, GMT_OUT, 2, GMT_COL_FIX_NO_TEXT)) != GMT_NOERROR)
+					Return (error);
+			}
+			else {
+				if ((error = GMT_Set_Columns (API, GMT_OUT, 0, GMT_COL_FIX)) != GMT_NOERROR)
+					Return (error);
+			}
 		}
 
 		/* Process all columns and update the corresponding minmax arrays */
@@ -746,17 +818,13 @@ int GMT_gmtinfo (void *V_API, int mode, void *args) {
 				value = (work_on_abs_value) ? fabs (in[Ctrl->E.col]) : in[Ctrl->E.col];
 				if (Ctrl->E.mode == -1 && value < e_min) {	/* Lower than previous low */
 					e_min = value;
-					if (GMT->common.b.active[GMT_IN])
-						gmt_M_memcpy (dchosen, in, ncol, double);
-					else
-						strncpy (chosen, GMT->current.io.record, GMT_BUFSIZ);
+					gmt_M_memcpy (dchosen, in, ncol, double);
+					if (In->text) strncpy (chosen, In->text, GMT_BUFSIZ-1);
 				}
 				else if (Ctrl->E.mode == +1 && value > e_max) {	/* Higher than previous high */
 					e_max = value;
-					if (GMT->common.b.active[GMT_IN])
-						gmt_M_memcpy (dchosen, in, ncol, double);
-					else
-						strncpy (chosen, GMT->current.io.record, GMT_BUFSIZ);
+					gmt_M_memcpy (dchosen, in, ncol, double);
+					if (In->text) strncpy (chosen, In->text, GMT_BUFSIZ-1);
 				}
 			}
 		}
@@ -788,9 +856,13 @@ int GMT_gmtinfo (void *V_API, int mode, void *args) {
 			}
 		}
 		n++;	/* Number of records processed in current block (all/table/segment; see -A) */
-		if (file[0] == 0) strncpy (file, GMT->current.io.filename[GMT_IN], GMT_BUFSIZ);	/* Grab name of current file while we can */
+		if (file[0] == 0) strncpy (file, GMT->current.io.filename[GMT_IN], PATH_MAX-1);	/* Grab name of current file while we can */
 		
 	}
+	if (GMT->common.a.active) {
+		gmt_list_aspatials (GMT, buffer);
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, buffer);
+	}
 	if (GMT_End_IO (API, GMT_IN,  0) != GMT_NOERROR) {	/* Disables further data input */
 		Return (API->error);
 	}
@@ -804,8 +876,10 @@ int GMT_gmtinfo (void *V_API, int mode, void *args) {
 	if (Ctrl->C.active) {	/* Restore previous output col types */
 		gmt_M_memcpy (GMT->current.io.col_type[GMT_OUT], col_type, GMT_MAX_COLUMNS, int);
 	}
-	if (Ctrl->E.active && GMT->common.b.active[GMT_IN])
+	if (Ctrl->E.active)
 		gmt_M_free (GMT, dchosen);
 
+	for (col = 0; col < ncol; col++) gmt_M_free (GMT, Z[col]);
+
 	Return (GMT_NOERROR);
 }
