@@ -1,7 +1,6 @@
 /*
- *	$Id$
  *
- *	Copyright (c) 1991-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  * Brief synopsis: gmtregress fits y = a + bx linear regression to x,y[,w] data
@@ -21,12 +20,13 @@
  *
  * Author:	Paul Wessel
  * Date:	5-JAN-2015
- * Version:	5 API
+ * Version:	6 API
  */
 
 #include "gmt_dev.h"
 
-#define THIS_MODULE_NAME	"gmtregress"
+#define THIS_MODULE_CLASSIC_NAME	"gmtregress"
+#define THIS_MODULE_MODERN_NAME	"gmtregress"
 #define THIS_MODULE_LIB		"core"
 #define THIS_MODULE_PURPOSE	"Linear regression of 1-D data sets"
 #define THIS_MODULE_KEYS	"<D{,>D}"
@@ -53,7 +53,9 @@ enum GMT_enum_regress {
 	GMTREGRESS_SIGIC	= 5,
 	GMTREGRESS_XMEAN	= 6,
 	GMTREGRESS_YMEAN	= 7,
-	GMTREGRESS_NPAR		= 8,
+	GMTREGRESS_R		= 8,
+	GMTREGRESS_CORR		= 9,
+	GMTREGRESS_NPAR		= 10,
 	GMTREGRESS_NPAR_MAIN	= 4,
 	GMTREGRESS_OUTPUT_GOOD  = 1,
 	GMTREGRESS_OUTPUT_BAD   = 2};
@@ -95,11 +97,10 @@ struct GMTREGRESS_CTRL {
 		bool active;
 		unsigned int mode;
 	} S;
-	struct T {	/* 	-T<min>/<max>/<inc> or -T<n> */
+	struct T {	/* 	-T[<min>/<max>/]<inc>[+n] */
 		bool active;
-		bool reset;	/* True if -T<n> was given */
-		uint64_t n;
-		double min, max, inc;
+		bool no_eval;
+		struct GMT_ARRAY T;
 	} T;
 	struct W {	/* 	-W[s]x|y|r */
 		bool active;
@@ -124,15 +125,16 @@ GMT_LOCAL void *New_Ctrl (struct GMT_CTRL *GMT) {	/* Allocate and initialize a n
 GMT_LOCAL void Free_Ctrl (struct GMT_CTRL *GMT, struct GMTREGRESS_CTRL *C) {	/* Deallocate control structure */
 	if (!C) return;
 	gmt_M_str_free (C->Out.file);	
+	gmt_free_array (GMT, &(C->T.T));
 	gmt_M_free (GMT, C);	
 }
 
 GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
-	gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
+	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: gmtregress [<table>] [-A[<min>/<max>/<inc>]] [-C<level>] [-Ex|y|o|r] [-F<flags>] [-N1|2|r|w]\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t[-S[r]] [-T<min>/<max>/<inc> | -T<n>] [%s] [-W[w][x][y][r]] [%s]\n", GMT_V_OPT, GMT_a_OPT);
-	GMT_Message (API, GMT_TIME_NONE, "\t[%s] [%s] [%s] [%s]\n\t[%s] [%s] [%s]\n\n", GMT_b_OPT, GMT_d_OPT, GMT_e_OPT, GMT_g_OPT, GMT_h_OPT, GMT_i_OPT, GMT_o_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "usage: %s [<table>] [-A[<min>/<max>/<inc>]] [-C<level>] [-Ex|y|o|r] [-F<flags>] [-N1|2|r|w]\n", name);
+	GMT_Message (API, GMT_TIME_NONE, "\t[-S[r]] [-T[<min>/<max>/]<inc>[+n] [%s] [-W[w][x][y][r]] [%s]\n", GMT_V_OPT, GMT_a_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "\t[%s] [%s] [%s] [%s]\n\t[%s] [%s] [%s] [%s]\n\n", GMT_b_OPT, GMT_d_OPT, GMT_e_OPT, GMT_g_OPT, GMT_h_OPT, GMT_i_OPT, GMT_o_OPT, GMT_PAR_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
@@ -168,8 +170,10 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t-S Skip records identified as outliers on output. Append r to reverse mode and\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   only output the outlier records. Cannot be used with -T [output all records].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-T Evaluate model at the equidistant points implied by the arguments.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   If -T<n> is given instead we reset <min> and <max> to the extreme x-values\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   for each segment and recalculate <inc> to give <n> output values per segment.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   If only -T<inc>[+n] is given we reset <min> and <max> to the extreme x-values\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   for each segment.  Append +n if <inc> is the number of t-values to produce instead.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   For absolute time data, append a valid time unit (%s) to the increment.\n", GMT_TIME_UNITS_DISPLAY);
+	GMT_Message (API, GMT_TIME_NONE, "\t   Alternatively, give a file with output times in the first column, or a comma-separated list.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   Use -T0 to bypass model evaluation entirely.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   [Default uses locations of input data to evaluate the model].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-W Supply individual 1-sigma uncertainties for data points [no weights].\n");
@@ -273,14 +277,10 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GMTREGRESS_CTRL *Ctrl, struct
 				break;
 			case 'T':	/* Output lattice or length */
 				Ctrl->T.active = true;
-				if (strchr (opt->arg, '/')) {
-					n = sscanf (opt->arg, "%lf/%lf/%lf", &Ctrl->T.min, &Ctrl->T.max, &Ctrl->T.inc);
-					n_errors += gmt_M_check_condition (GMT, n < 2, "Syntax error -T option: Must specify min/max/inc\n");
-				}
-				else {
-					Ctrl->T.reset = true;
-					Ctrl->T.n = atol (opt->arg);
-				}
+				if (!strcmp (opt->arg, "0"))	/* -T0 means no model evaluation */
+					Ctrl->T.no_eval = true;
+				else
+					n_errors += gmt_parse_array (GMT, 'T', opt->arg, &(Ctrl->T.T), GMT_ARRAY_TIME | GMT_ARRAY_DIST, 0);
 				break;
 			case 'W':	/* Weights or not */
 				Ctrl->W.active = true;
@@ -329,6 +329,24 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GMTREGRESS_CTRL *Ctrl, struct
 	return (n_errors ? GMT_PARSE_ERROR : GMT_NOERROR);
 }
 
+/* Some of the inspiration for this module came from the following two papers:
+ *
+ * Hartmann, C., P. Venkeerberghen, J. Smeyers-Verbeke, and D. L. Massart (1997),
+ *   Robust orthogonal regression for the outlier detection when comparing two
+ *   serious of measurement results, Analytica Chimica Acta, 344, 17–28.
+ * York, D., N. M. Evensen, M. L. Martínez, and J. De Basebe Delgado (2004),
+ *   Unified equations for the slope, intercept, and standard errors of the
+ *   best straight line, Am. J. Phys., 72(3), 367–375.
+ *
+ * as well as the standard book on regular and robust regression:
+ *
+ * Draper, N. R., and H. Smith (1998), Applied regression analysis,
+ *   3rd ed., 736 pp., John Wiley and Sons, New York.
+ *
+ * Rousseeuw, P. J., and A. M. Leroy (1987), Robust regression and outlier
+ *   detection, 329 pp., John Wiley and Sons, New York.
+ */
+
 GMT_LOCAL double model (double x, double *par) {
 	/* Evaluate the model given the parameters in par */
 	return (par[GMTREGRESS_SLOPE] * x + par[GMTREGRESS_ICEPT]);
@@ -377,8 +395,8 @@ GMT_LOCAL double icept_weighted (struct GMT_CTRL *GMT, double *e, double *W, uin
 		uint64_t k;
 		ee = gmt_M_memory (GMT, NULL, n, struct GMT_OBSERVATION);
 		for (k = 0; k < n; k++) {
-			ee[k].weight = (float)W[k];
-			ee[k].value  = (float)e[k];
+			ee[k].weight = (gmt_grdfloat)W[k];
+			ee[k].value  = (gmt_grdfloat)e[k];
 		}
 	}
 	switch (norm) {
@@ -458,8 +476,8 @@ GMT_LOCAL double L1_scale (struct GMT_CTRL *GMT, double *ey, double *W, uint64_t
 	/* Compute the (weighted) MAD */
 	ee = gmt_M_memory (GMT, NULL, n, struct GMT_OBSERVATION);
 	for (k = 0; k < n; k++) {
-		ee[k].weight = (float)W[k];
-		ee[k].value  = (float)fabs (ey[k]);
+		ee[k].weight = (gmt_grdfloat)W[k];
+		ee[k].value  = (gmt_grdfloat)fabs (ey[k]);
 	}
 	MAD = gmt_median_weighted (GMT, ee, n);
 	gmt_M_free (GMT, ee);
@@ -517,6 +535,50 @@ GMT_LOCAL void ones (double *x, uint64_t n) {
 	for (k = 0; k < n; k++) x[k] = 1.0;
 }
 
+GMT_LOCAL void get_correlation (struct GMT_CTRL *GMT, double *X, double *Y, double *w[], uint64_t n, double *par) {
+	/* standard r = s_xy / (s_x * s_y), using the weighted expressions for these terms.
+	 */
+
+	uint64_t k;
+	double sx = 0.0, sy = 0.0, sxy = 0.0, wx = 1.0, wy = 1.0, swx = 0.0, swy = 0.0, swxy = 0.0;
+	gmt_M_unused(GMT);
+	
+	for (k = 0; k < n; k++) {
+		if (w[GMT_X]) wx = w[GMT_X][k];	/* Was given x-weights */
+		if (w[GMT_Y]) wy = w[GMT_Y][k];	/* Was given y-weights */
+		sx  += wx * pow (X[k] - par[GMTREGRESS_XMEAN], 2.0);
+		sy  += wy * pow (Y[k] - par[GMTREGRESS_YMEAN], 2.0);
+		sxy += wx * wy * (X[k] - par[GMTREGRESS_XMEAN]) * (Y[k] - par[GMTREGRESS_YMEAN]);
+		swx += wx;	swy += wy;	swxy += wx * wy;
+	}
+	par[GMTREGRESS_CORR] = (sxy / swxy) / sqrt ((sx / swx) * (sy / swy));
+}
+
+GMT_LOCAL void get_coeffR (struct GMT_CTRL *GMT, double *X, double *Y, double *w[], uint64_t n, unsigned int regression, double *par) {
+	/* Compute coefficient of determination, R ( = r^2 for LSY Pearsonian correlation).
+	 * Currently only set up to do standard y on x (weights on y) only.
+	 * Compute both coefficient of determination (R) and the correlation coefficient (r).
+	 *   R = 1 - SSR/SST, with
+	 *   SSR is the sum of squared residuals: sum (y_i - y(x_i))^2
+	 *   and SST is the sum of total variance of the model : sum (y_i - mean(y))^2
+	 *   We augment these with weights w_i as well, if given.
+	 */
+
+	uint64_t k;
+	double SSR = 0.0, SST = 0.0, y_hat, ww = 1.0, f;
+	gmt_M_unused(GMT);
+	
+	f = get_scale_factor (regression, par[GMTREGRESS_SLOPE]);
+	f *= f;	/* Since working on squared misfits */
+	for (k = 0; k < n; k++) {
+		if (w[GMT_Y]) ww = w[GMT_Y][k];	/* Was given weights */
+		y_hat = par[GMTREGRESS_SLOPE] * X[k] + par[GMTREGRESS_ICEPT];
+		SSR += ww * pow (Y[k] - y_hat, 2.0);
+		SST += ww * pow (Y[k] - par[GMTREGRESS_YMEAN], 2.0);
+	}
+	par[GMTREGRESS_R] = 1.0 - f * SSR / SST;
+}
+
 GMT_LOCAL double gmt_demeaning (struct GMT_CTRL *GMT, double *X, double *Y, double *w[], uint64_t n, double *par, double *U, double *V, double *W, double *alpha, double *beta) {
 	/* Compute weighted X and Y means, return these via par, and calculate residuals U and V and weights W
 	 * (and alpha, beta if orthogonal).  If orthogonal regression we expect a preliminary estimate of the
@@ -696,7 +758,7 @@ GMT_LOCAL void regress1D_sub (struct GMT_CTRL *GMT, double *x, double *y, double
 		case GMTREGRESS_NORM_L2:  misfit = L2_misfit;  break;
 		case GMTREGRESS_NORM_LMS: misfit = LMS_misfit; break;
 		default:
-			GMT_Report (GMT->parent, GMT_MSG_VERBOSE, "Internal error: misfit norm not specified?\n");
+			GMT_Report (GMT->parent, GMT_MSG_VERBOSE, "Internal error: misfit norm not specified? - set to L2\n");
 			misfit = L2_misfit;
 			break;
 	}
@@ -746,7 +808,7 @@ GMT_LOCAL double regress1D (struct GMT_CTRL *GMT, double *x, double *y, double *
 		case GMTREGRESS_NORM_L2:  scl_func = L2_scale;  break;
 		case GMTREGRESS_NORM_LMS: scl_func = LMS_scale; break;
 		default:
-			GMT_Report (GMT->parent, GMT_MSG_VERBOSE, "Internal error: misfit norm not specified?\n");
+			GMT_Report (GMT->parent, GMT_MSG_VERBOSE, "Internal error: misfit norm not specified? - set to L2\n");
 			scl_func = L2_scale;
 			break;
 	}
@@ -782,11 +844,11 @@ GMT_LOCAL double regress1D (struct GMT_CTRL *GMT, double *x, double *y, double *
 			double a = par[GMTREGRESS_ICEPT] + (par[GMTREGRESS_YMEAN] - par[GMTREGRESS_SLOPE] * par[GMTREGRESS_XMEAN]);
 			snprintf (buffer, GMT_LEN256, "Robust iteration %u: N: %" PRIu64 " x0: %g y0: %g angle: %g E: %g slope: %g icept: %g sig_slope: --N/A-- sig_icept: --N/A--",
 				n_iter, n, par[GMTREGRESS_XMEAN], par[GMTREGRESS_YMEAN], par[GMTREGRESS_ANGLE], par[GMTREGRESS_MISFT], par[GMTREGRESS_SLOPE], a);
-			GMT_Report (GMT->parent, GMT_MSG_VERBOSE, "%s\n", buffer);
+			GMT_Report (GMT->parent, GMT_MSG_LONG_VERBOSE, "%s\n", buffer);
 		}
 	}
 	par[GMTREGRESS_ICEPT] += (par[GMTREGRESS_YMEAN] - par[GMTREGRESS_SLOPE] * par[GMTREGRESS_XMEAN]);
-	GMT_Report (GMT->parent, GMT_MSG_VERBOSE, "Robust regression algorithm convergence required %d iterations\n", n_iter);
+	GMT_Report (GMT->parent, GMT_MSG_LONG_VERBOSE, "Robust regression algorithm convergence required %d iterations\n", n_iter);
 	scale = scl_func (GMT, e, W, n, par);	/* Get final regression-scale for residuals */
 	
 	gmt_M_free (GMT, U);
@@ -844,9 +906,9 @@ GMT_LOCAL double LSxy_regress1D_york (struct GMT_CTRL *GMT, double *X, double *Y
 		n_iter++;
 		snprintf (buffer, GMT_LEN256, "York iteration %u: N: %" PRIu64 " x0: %g y0: %g angle: %g E: %g slope: %g icept: %g sig_slope: %g sig_icept: %g",
 			n_iter, n, par[GMTREGRESS_XMEAN], par[GMTREGRESS_YMEAN], atand (b), misfit, b, a, sigma_b, sigma_a);
-		GMT_Report (GMT->parent, GMT_MSG_VERBOSE, "%s\n", buffer);
+		GMT_Report (GMT->parent, GMT_MSG_LONG_VERBOSE, "%s\n", buffer);
 	} while (fabs (b - b_old) > GMT_CONV15_LIMIT && n_iter < GMTREGRESS_MAX_YORK_ITERATIONS);
-	GMT_Report (GMT->parent, GMT_MSG_VERBOSE, "York orthogonal algorithm convergence required %d iterations\n", n_iter);
+	GMT_Report (GMT->parent, GMT_MSG_LONG_VERBOSE, "York orthogonal algorithm convergence required %d iterations\n", n_iter);
 	/* Pass out the final result via the par array */
 	par[GMTREGRESS_SLOPE] = b;
 	par[GMTREGRESS_ICEPT] = a;
@@ -869,7 +931,7 @@ GMT_LOCAL double LSxy_regress1D_york (struct GMT_CTRL *GMT, double *X, double *Y
 }
 
 GMT_LOCAL double LSxy_regress1D (struct GMT_CTRL *GMT, double *x, double *y, double *w[], uint64_t n, double *par) {
-	/* Front to calling LSxy_regress1D_york or LSxy_regress1D_basic, depening on weights */
+	/* Front to calling LSxy_regress1D_york or LSxy_regress1D_basic, depending on weights */
 	double scale;
 	if (w && w[GMT_X] && w[GMT_Y])	/* Have weights in x and y [and possibly correlation coefficients as well] */
 		scale = LSxy_regress1D_york (GMT, x, y, w, n, par);
@@ -985,6 +1047,9 @@ GMT_LOCAL double *do_regression (struct GMT_CTRL *GMT, double *x_in, double *y_i
 		for (col = first_col; col <= GMT_Y; col++)	/* Free any arrays we allocated */
 			if (made[col]) gmt_M_free (GMT, www[col]);
 	}
+	get_correlation (GMT, x_in, y_in, w, n, par);	/* Evaluate r */
+	get_coeffR (GMT, x_in, y_in, w, n, regression, par);	/* Evaluate R */
+	
 	return (z);	/* Return those z-scores, calling unit must free this array when done */
 }
 
@@ -1000,12 +1065,13 @@ int GMT_gmtregress (void *V_API, int mode, void *args) {
 	unsigned geometry = GMT_IS_NONE;
 	
 	double *x = NULL, *U = NULL, *V = NULL, *W = NULL, *e = NULL, *w[3] = {NULL, NULL, NULL};
-	double t_scale = 0.0, par[GMTREGRESS_NPAR], out[9], *t = NULL;
+	double t_scale = 0.0, par[GMTREGRESS_NPAR], out[9];
 	
 	char buffer[GMT_LEN256];
 	
 	struct GMT_DATASET *Din = NULL;
 	struct GMT_DATASEGMENT *S = NULL, *Sa = NULL;
+	struct GMT_RECORD *Out = NULL;
 	struct GMTREGRESS_CTRL *Ctrl = NULL;
 	struct GMT_CTRL *GMT = NULL, *GMT_cpy = NULL;
 	struct GMT_OPTION *options = NULL;
@@ -1021,12 +1087,11 @@ int GMT_gmtregress (void *V_API, int mode, void *args) {
 	if (mode == GMT_MODULE_PURPOSE) return (usage (API, GMT_MODULE_PURPOSE));	/* Return the purpose of program */
 	options = GMT_Create_Options (API, mode, args);	if (API->error) return (API->error);	/* Set or get option list */
 
-	if (options && options->option == GMT_OPT_USAGE) bailout (usage (API, GMT_USAGE));/* Return the usage message */
-	if (options && options->option == GMT_OPT_SYNOPSIS) bailout (usage (API, GMT_SYNOPSIS));	/* Return the synopsis */
+	if ((error = gmt_report_usage (API, options, 1, usage)) != GMT_NOERROR) bailout (error);	/* Give usage if requested */
 
 	/* Parse the command-line arguments */
 
-	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
 	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
 	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
 	if ((error = parse (GMT, Ctrl, options)) != 0) Return (error);
@@ -1037,7 +1102,7 @@ int GMT_gmtregress (void *V_API, int mode, void *args) {
 		n_try = lrint ((Ctrl->A.max - Ctrl->A.min) / Ctrl->A.inc) + 1;	/* Number of angles to explore */
 		n_columns = GMTREGRESS_NPAR_MAIN;	/* Hardwired to return angle, misfit, slope, intercept */
 		/* Allocate fixed temp space hold the result of the experiment */
-		Sa = GMT_Alloc_Segment (GMT->parent, GMT_IS_DATASET, n_try, n_columns, NULL, NULL);
+		Sa = GMT_Alloc_Segment (GMT->parent, GMT_NO_STRINGS, n_try, n_columns, NULL, NULL);
 	}
 	else {	/* Work up best regression solution per input segment */
 		if (Ctrl->F.param)
@@ -1053,30 +1118,19 @@ int GMT_gmtregress (void *V_API, int mode, void *args) {
 		}
 	}
 	
-	if (Ctrl->T.active) {	/* Evaluate solution for equidistant spacing instead of at the input locations */
-		unsigned int bad = 0;	/* Must chech for conflict between -T and -F settings */
-		if (!Ctrl->T.reset)	/* Compute number of points given fixed range and increment */
-			Ctrl->T.n = gmt_M_get_n (GMT, Ctrl->T.min, Ctrl->T.max, Ctrl->T.inc, 0);
-		for (col = 0; col < n_columns; col++) if (Ctrl->T.n && strchr ("yrzw", (int)Ctrl->F.col[col])) {
-			GMT_Report (API, GMT_MSG_NORMAL, "Syntax error -F: Cannot include %c when -T is in effect\n", (int)Ctrl->F.col[col]);
-			bad++;
-		}
-		if (bad) Return (GMT_RUNTIME_ERROR);
-		if (Ctrl->T.n) t = gmt_M_memory (GMT, NULL, Ctrl->T.n, double);	/* Allocate space for output x-values (unless when -T0 is given) */
-		geometry = GMT_IS_LINE;
-	}
-
 	/* Allocate memory and read in all the files; each file can have many records */
 	
 	if (GMT_Init_IO (API, GMT_IS_DATASET, GMT_IS_POINT, GMT_IN, GMT_ADD_DEFAULT, 0, options) != GMT_NOERROR) {
 		Return (API->error);	/* Establishes data files or stdin */
 	}
-	if ((error = gmt_set_cols (GMT, GMT_IN, 2 + Ctrl->W.n_weights)) != 0) Return (error);
+	if ((error = GMT_Set_Columns (API, GMT_IN, 2 + Ctrl->W.n_weights, GMT_COL_FIX_NO_TEXT)) != GMT_NOERROR) {
+		Return (error);
+	}
 	if ((Din = GMT_Read_Data (API, GMT_IS_DATASET, GMT_IS_FILE, 0, GMT_READ_NORMAL, NULL, NULL, NULL)) == NULL) {
 		Return (API->error);
 	}
 	if (Din->n_columns < (2 + Ctrl->W.n_weights)) {
-		GMT_Report (API, GMT_MSG_VERBOSE, "Dataset only has %" PRIu64 " columns but %d are required by your settings!\n", Din->n_columns, 2 + Ctrl->W.n_weights);
+		GMT_Report (API, GMT_MSG_NORMAL, "Dataset only has %" PRIu64 " columns but %d are required by your settings!\n", Din->n_columns, 2 + Ctrl->W.n_weights);
 		Return (GMT_RUNTIME_ERROR);
 	}
 	
@@ -1089,10 +1143,24 @@ int GMT_gmtregress (void *V_API, int mode, void *args) {
 	if (GMT_Set_Geometry (API, GMT_OUT, geometry) != GMT_NOERROR) {	/* Sets output geometry */
 		Return (API->error);
 	}
-	if ((error = gmt_set_cols (GMT, GMT_OUT, n_columns)) != 0) Return (error);
+	if ((error = GMT_Set_Columns (API, GMT_OUT, (unsigned int)n_columns, GMT_COL_FIX_NO_TEXT)) != GMT_NOERROR) {
+		Return (error);
+	}
+
+	if (Ctrl->T.active) {	/* Evaluate solution for equidistant spacing instead of at the input locations */
+		unsigned int bad = 0;	/* Must check for conflict between -T and -F settings */
+		for (col = 0; col < n_columns; col++) if (!Ctrl->T.no_eval && strchr ("yrzw", (int)Ctrl->F.col[col])) {
+			GMT_Report (API, GMT_MSG_NORMAL, "Syntax error -F: Cannot include %c when -T is in effect\n", (int)Ctrl->F.col[col]);
+			bad++;
+		}
+		if (bad) Return (GMT_RUNTIME_ERROR);
+		if (Ctrl->T.T.set == 3 && gmt_create_array (GMT, 'T', &(Ctrl->T.T), NULL, NULL))
+			Return (GMT_RUNTIME_ERROR);
+		geometry = GMT_IS_LINE;
+	}
 
 	gmt_set_segmentheader (GMT, GMT_OUT, true);	/* To write segment headers regardless of input */
-	
+	Out = gmt_new_record (GMT, out, NULL);	/* Since we only need to worry about numerics in this module */
 	/* Process all tables and their segments */
 	for (tbl = 0; tbl < Din->n_tables; tbl++) {
 		for (seg = 0; seg < Din->table[tbl]->n_segments; seg++) {
@@ -1143,14 +1211,14 @@ int GMT_gmtregress (void *V_API, int mode, void *args) {
 					}
 					for (k = 0; k < GMTREGRESS_NPAR_MAIN; k++) Sa->data[k][row] = par[k];	/* Save the results into this row */
 				}
-				/* Make segment header with the findings forthe overall best regression */
+				/* Make segment header with the findings for the overall best regression */
 				snprintf (buffer, GMT_LEN256, "Best regression: N: %" PRIu64 " x0: %g y0: %g angle: %g E: %g slope: %g icept: %g sig_slope: --N/A-- sig_icept: --N/A--", S->n_rows, par[GMTREGRESS_XMEAN], par[GMTREGRESS_YMEAN], Sa->data[0][min_row],
 					Sa->data[1][min_row], Sa->data[2][min_row], Sa->data[3][min_row]);
-				GMT_Report (API, GMT_MSG_VERBOSE, "%s\n", buffer);	/* Report results if verbose */
+				GMT_Report (API, GMT_MSG_LONG_VERBOSE, "%s\n", buffer);	/* Report results if verbose */
 				GMT_Put_Record (API, GMT_WRITE_SEGMENT_HEADER, buffer);	/* Also include result in segment header */
 				for (row = 0; row < n_try; row++) {	/* Write the saved results of the experiment */
 					for (k = 0; k < GMTREGRESS_NPAR_MAIN; k++) out[k] = Sa->data[k][row];
-					GMT_Put_Record (API, GMT_WRITE_DATA, out);	/* Write this record to output */
+					GMT_Put_Record (API, GMT_WRITE_DATA, Out);	/* Write this record to output */
 				}
 			}
 			else {	/* Here we are solving for the best regression */
@@ -1166,27 +1234,31 @@ int GMT_gmtregress (void *V_API, int mode, void *args) {
 					out[6] = par[GMTREGRESS_ICEPT];
 					out[7] = par[GMTREGRESS_SIGSL];
 					out[8] = par[GMTREGRESS_SIGIC];
-					GMT_Put_Record (API, GMT_WRITE_DATA, out);	/* Write this record to output */
+					GMT_Put_Record (API, GMT_WRITE_DATA, Out);	/* Write this record to output */
 				}
 				else {
 					/* Make segment header with the findings for best regression */
-					snprintf (buffer, GMT_LEN256, "Best regression: N: %" PRIu64 " x0: %g y0: %g angle: %g E: %g slope: %g icept: %g sig_slope: %g sig_icept: %g", S->n_rows, par[GMTREGRESS_XMEAN], par[GMTREGRESS_YMEAN],
-						par[GMTREGRESS_ANGLE], par[GMTREGRESS_MISFT], par[GMTREGRESS_SLOPE], par[GMTREGRESS_ICEPT], par[GMTREGRESS_SIGSL], par[GMTREGRESS_SIGIC]);
-					GMT_Report (API, GMT_MSG_VERBOSE, "%s\n", buffer);	/* Report results if verbose */
+					snprintf (buffer, GMT_LEN256, "Best regression: N: %" PRIu64 " x0: %g y0: %g angle: %g E: %g slope: %g icept: %g sig_slope: %g sig_icept: %g corr: %g R: %g", S->n_rows, par[GMTREGRESS_XMEAN], par[GMTREGRESS_YMEAN],
+						par[GMTREGRESS_ANGLE], par[GMTREGRESS_MISFT], par[GMTREGRESS_SLOPE], par[GMTREGRESS_ICEPT], par[GMTREGRESS_SIGSL], par[GMTREGRESS_SIGIC], par[GMTREGRESS_CORR], par[GMTREGRESS_R]);
+					GMT_Report (API, GMT_MSG_LONG_VERBOSE, "%s\n", buffer);	/* Report results if verbose */
 					GMT_Put_Record (API, GMT_WRITE_SEGMENT_HEADER, buffer);	/* Also include in segment header */
 
 					if (Ctrl->F.band)	/* For confidence band we need the student-T scale given the significance level and degrees of freedom */
 						t_scale = fabs (gmt_tcrit (GMT, 0.5 * (1.0 - Ctrl->C.value), S->n_rows - 2.0));
 
-					if (Ctrl->T.active) {	/* Evaluate the model at the chosen equidistant output points */
-						if (Ctrl->T.reset) {	/* Got -T<n>, so must recompute a new increment for the present segment's x-range */
-							Ctrl->T.min = S->min[GMT_X];	Ctrl->T.max = S->max[GMT_X];
-							if (Ctrl->T.n) Ctrl->T.inc = gmt_M_get_inc (GMT, Ctrl->T.min, Ctrl->T.max, Ctrl->T.n, 0);	/* Protect against -T0 */
+					if (Ctrl->T.no_eval)	/* No model evaluation */
+						n_t = 0;
+					else if (Ctrl->T.active) {	/* Evaluate the model at the chosen equidistant output points */
+						if (Ctrl->T.T.set == 1) {	/* Must update the output t array to be nultiples of inc, possibly starting and ending outside range */
+							double min, max;
+							gmt_M_free (GMT, Ctrl->T.T.array);
+							min = floor (S->min[GMT_X] / Ctrl->T.T.inc) * Ctrl->T.T.inc;
+							max = ceil  (S->max[GMT_X] / Ctrl->T.T.inc) * Ctrl->T.T.inc ;
+							if (gmt_create_array (GMT, 'T', &(Ctrl->T.T), &min, &max))
+								Return (GMT_RUNTIME_ERROR);
 						}
-						for (k = 0; k < Ctrl->T.n; k++)	/* Build new output x-coordinates */
-							t[k] = gmt_M_col_to_x (GMT, k, Ctrl->T.min, Ctrl->T.max, Ctrl->T.inc, 0, Ctrl->T.n);
-						x = t;	/* Pass these coordinates as our "x" */
-						n_t = Ctrl->T.n;
+						x = Ctrl->T.T.array;	/* Pass these coordinates as our "x" */
+						n_t = Ctrl->T.T.n;
 					}
 					else {	/* Use the given data abscissae instead */
 						x = S->data[GMT_X];
@@ -1226,7 +1298,7 @@ int GMT_gmtregress (void *V_API, int mode, void *args) {
 									break;
 							}
 						}
-						GMT_Put_Record (API, GMT_WRITE_DATA, out);	/* Write this record to output */
+						GMT_Put_Record (API, GMT_WRITE_DATA, Out);	/* Write this record to output */
 					}
 				}
 				gmt_M_free (GMT, z_score);	/* Done with this array */
@@ -1234,9 +1306,10 @@ int GMT_gmtregress (void *V_API, int mode, void *args) {
 		}
 	}
 
-	if (GMT_End_IO (API, GMT_OUT, 0) != GMT_NOERROR) {	/* Disables further data output */
-		Return (API->error);
-	}
+	gmt_M_free (GMT, Out);
+	error = GMT_NOERROR;
+	if (GMT_End_IO (API, GMT_OUT, 0) != GMT_NOERROR) 	/* Disables further data output */
+		error = API->error;
 	
 	if (Ctrl->A.active) {	/* Free special arrays and segment used for -A experiment */
 		gmt_free_segment (GMT, &Sa);
@@ -1247,8 +1320,6 @@ int GMT_gmtregress (void *V_API, int mode, void *args) {
 	}
 	for (k = GMT_X; k <= GMT_Z; k++)	/* Free arrays used for weights */
 		if (Ctrl->W.col[k]) gmt_M_free (GMT, w[k]);
-	if (Ctrl->T.n)	/* Finally, free output locations for -T */
-		gmt_M_free (GMT, t);
 	
-	Return (GMT_NOERROR);
+	Return (error);
 }
