@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *	Copyright (c) 1991-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,14 +12,14 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  * API functions to support the gmtselect application.
  *
  * Author:	Paul Wessel
  * Date:	1-JAN-2010
- * Version:	5 API
+ * Version:	6 API
  *
  * Brief synopsis: gmtselect is a general-purpose spatial filter.  Data pass
  * or fail basedon one or more conditions.  Seven conditions may be set:
@@ -33,7 +32,7 @@
  *	6. Only data with z-values within specified range may pass
  *  7. Only data with (x,y) locations inside a mask grid may pass
  *
- * Distances are calculated in the users units using Euclidian geometry
+ * Distances are calculated in the users units using Euclidean geometry
  * unless a map projection and region (-R -J) are used.  Then, distances
  * are calculated using spherical geometry and converted to km, and any
  * distances given in options or via headers are assumed to be in km.
@@ -44,7 +43,8 @@
  
 #include "gmt_dev.h"
 
-#define THIS_MODULE_NAME	"gmtselect"
+#define THIS_MODULE_CLASSIC_NAME	"gmtselect"
+#define THIS_MODULE_MODERN_NAME	"gmtselect"
 #define THIS_MODULE_LIB		"core"
 #define THIS_MODULE_PURPOSE	"Select data table subsets based on multiple spatial criteria"
 #define THIS_MODULE_KEYS	"<D{,CD(=,FD(,LD(=,>D},GG("
@@ -74,6 +74,7 @@ struct GMTSELECT_DATA {	/* Used for temporary storage when sorting data on x coo
 struct GMTSELECT_ZLIMIT {	/* Used to hold info for each -Z option given */
 	unsigned int col;	/* Column to test */
 	bool equal;	/* Just check if z == min within 5 ULps */
+	bool invert;	/* Just check if the test should be negative != etc */
 	double min;	/* Smallest z-value to pass through, for this column */
 	double max;	/* Largest z-value to pass through, for this column */
 };
@@ -91,10 +92,10 @@ struct GMTSELECT_CTRL {	/* All control options for this program (except common a
 		char unit;	/* Unit name */
 		char *file;	/* Name of file with points */
 	} C;
-	struct GMTSELECT_D {	/* -D<resolution> */
+	struct GMTSELECT_D {	/* -D<resolution>[+f] */
 		bool active;
 		bool force;	/* if true, select next highest level if current set is not available */
-		char set;	/* One of f, h, i, l, c */
+		char set;	/* One of f, h, i, l, c, or a for auto */
 	} D;
 	struct GMTSELECT_E {	/* -E<operators> , <op> = combination or f,n */
 		bool active;
@@ -125,8 +126,9 @@ struct GMTSELECT_CTRL {	/* All control options for this program (except common a
 		unsigned int mode;	/* 1 if dry/wet only, 0 if 5 mask levels */
 		bool mask[GMTSELECT_N_CLASSES];	/* Mask for each level */
 	} N;
-	struct GMTSELECT_Z {	/* -Z<min>/<max>[+c<col>] */
+	struct GMTSELECT_Z {	/* -Z<min>/<max>[+c<col>][+i][+a] */
 		bool active;
+		bool or;		/* Logical or.  If any test passes then we pass the record [all tests must pass] */
 		unsigned int n_tests;	/* How many of these tests did we get */
 		unsigned int max_col;	/* The largest column we encountered */
 		struct GMTSELECT_ZLIMIT *limit;
@@ -173,69 +175,16 @@ GMT_LOCAL int compare_x (const void *point_1, const void *point_2) {
 	return (0);
 }
 
-GMT_LOCAL void ogr_to_text (struct GMT_CTRL *GMT, struct GMT_OGR *G, char *out) {
-	unsigned int col, k, n_aspatial = 0;
-	size_t len;
-	char text[GMT_LEN64] = {""};
-	static int ogr[MAX_ASPATIAL];
-	
-	out[0] = '\0';	/* Start with nothing */
-	
-	if (n_aspatial == 0) {	/* First time */
-		for (col = 0; col < GMT->common.a.n_aspatial; col++) {	/* Loop over requested aspatial items via -a */
-			for (k = 0; k < G->n_aspatial; k++) {	/* Given the actual data... */
-				if (strcmp (GMT->common.a.name[col], G->name[k])) continue;	/* Not one of the specified items */
-				ogr[n_aspatial++] = k;
-			}
-		}
-	}
-	for (col = 0; col < n_aspatial; col++) {	/* Loop over requested aspatial items via -a */
-		k = ogr[col];
-		switch (G->type[k]) {
-			case GMT_TEXT:
-				len = strlen (G->tvalue[k]);
-				if (G->tvalue[k][0] == '\"' && G->tvalue[k][len-1] == '\"') {	/* Skip opening and closing quotes */
-					strncpy (text, &G->tvalue[k][1], len-2);
-					text[len-2] = '\0';
-				}
-				else
-					strcpy (text, G->tvalue[k]);
-				break;
-			case GMT_DOUBLE:
-			case GMT_FLOAT:
-				gmt_ascii_format_col (GMT, text, G->dvalue[k], GMT_OUT, GMT_Z);
-				break;
-			case GMT_CHAR:
-			case GMT_UCHAR:
-			case GMT_INT:
-			case GMT_UINT:
-			case GMT_LONG:
-			case GMT_ULONG:
-				snprintf (text, GMT_LEN64, "%ld", lrint (G->dvalue[k]));
-				break;
-			case GMT_DATETIME:
-				gmt_format_abstime_output (GMT, G->dvalue[k], text);
-				break;
-			default:
-				GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Bad type passed to gmtio_write_formatted_ogr_value - assumed to be double\n");
-				gmt_ascii_format_col (GMT, text, G->dvalue[k], GMT_OUT, GMT_Z);
-				break;
-		}
-		strcat (out, GMT->current.setting.io_col_separator);
-		strcat (out, text);
-	}
-}
-
 GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
-	gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
+	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: gmtselect [<table>] [%s]\n", GMT_A_OPT);
-	GMT_Message (API, GMT_TIME_NONE, "\t[-C<ptfile>+d%s] [-D<resolution>][+] [-E[f][n]] [-F<polygon>] [-G<gridmask>] [%s]\n",
+	GMT_Message (API, GMT_TIME_NONE, "usage: %s [<table>] [%s]\n", name, GMT_A_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "\t[-C<ptfile>+d%s] [-D<resolution>][+f] [-E[f][n]] [-F<polygon>] [-G<gridmask>] [%s]\n",
 	             GMT_DIST_OPT, GMT_J_OPT);
-	GMT_Message (API, GMT_TIME_NONE, "\t[-I[cfglrsz] [-L<lfile>+d%s[+p]] [-N<info>] [%s]\n\t[%s] [-Z<min>[/<max>][+c<col>]] [%s] "
-	             "[%s]\n\t[%s] [%s] [%s] [%s]\n\t[%s] [%s]\n\t[%s] [%s] [%s]\n\n",
+	GMT_Message (API, GMT_TIME_NONE, "\t[-I[cfglrsz] [-L<lfile>+d%s[+p]] [-N<info>] [%s]\n\t[%s] [-Z<min>[/<max>][+c<col>][+a][+i]] [%s] "
+	             "[%s]\n\t[%s] [%s] [%s] [%s]\n\t[%s] [%s]\n\t[%s] [%s] [%s] [%s]\n\n",
 	             GMT_DIST_OPT, GMT_Rgeo_OPT, GMT_V_OPT, GMT_a_OPT, GMT_b_OPT, GMT_d_OPT, GMT_e_OPT, GMT_f_OPT,
-				 GMT_g_OPT, GMT_h_OPT, GMT_i_OPT, GMT_o_OPT, GMT_s_OPT, GMT_colon_OPT);
+				 GMT_g_OPT, GMT_h_OPT, GMT_i_OPT, GMT_o_OPT, GMT_s_OPT, GMT_colon_OPT, GMT_PAR_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
@@ -253,7 +202,7 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t     i - intermediate resolution.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t     l - low resolution [Default].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t     c - crude resolution, for tasks that need crude continent outlines only.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Append + to use a lower resolution should the chosen one not be available [abort].\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   Append +f to use a lower resolution should the chosen one not be available [abort].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-E Indicate if points exactly on a polygon boundary are inside or outside.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   Append f and/or n to modify the -F option or -N option, respectively,\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   to consider such points to be outside the feature [inside].\n");
@@ -267,7 +216,7 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t     l will pass locations beyond the minimum distance to the lines in -L.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t     r will pass locations outside the region given in -R [and -J].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t     s will pass locations that otherwise would be skipped in -N.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     z will pass locations outside the range given in -Z.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t     z will pass locations outside the range given in -Z (single -Z required).\n");
 	GMT_Option (API, "J");
 	gmt_dist_syntax (API->GMT, 'L', "Pass locations that are within <dist> of any line in ASCII <linefile>.");
 	GMT_Message (API, GMT_TIME_NONE, "\t   Give distance as 0 if 2nd column of segment headers have individual distances.\n");
@@ -284,8 +233,11 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t-Z Assume the 3rd data column contains z-values and we want to keep records with\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   <min> <= z <= <max>.  Use - for <min> or <max> if there is no lower/upper limit.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   Append +c<col> to select another column than the third [2].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   If <max> is not given we pass records whose z equal <min>.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   The -Z option is repeatable.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   If <max> is not given we pass records whose z equal <min> within 5 ulps.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   The -Z option is repeatable.  If given more than once then these modifiers may be useful:\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t     +a: Pass the record if any of the tests are true [all tests must be true in order to pass].\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t     +i: Reverse an individual test since -Iz only applies to a single test.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t     If +c is not given then it is incremented automatically, starting at 2.\n");
 	GMT_Option (API, "a,bi0");
 	if (gmt_M_showusage (API)) GMT_Message (API, GMT_TIME_NONE, "\t   Default is 2 input columns (3 if -Z is used).\n");
 	GMT_Option (API, "bo,d,e,f,g,h,i");
@@ -305,7 +257,7 @@ GMT_LOCAL int old_C_parse (struct GMTAPI_CTRL *API, char *arg, struct GMTSELECT_
 	}
 	if (arg[0] == 'f') {
 		if (gmt_M_compat_check (API->GMT, 4)) {	/* Allow old-style quick-mode specification */
-			GMT_Report (API, GMT_MSG_COMPAT, "Warning: Option -Cf is deprecated; use -C<file>+d-<distance> instead\n");
+			GMT_Report (API, GMT_MSG_COMPAT, "Option -Cf is deprecated; use -C<file>+d-<distance> instead\n");
 			arg[0] = '-';
 			fix = true;
 		}
@@ -365,8 +317,10 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GMTSELECT_CTRL *Ctrl, struct G
 	 * returned when registering these sources/destinations with the API.
 	 */
 
-	unsigned int n_errors = 0, pos, j, col, n_z_alloc = 0;
-	char ptr[GMT_BUFSIZ] = {""}, buffer[GMT_BUFSIZ] = {""}, za[GMT_LEN64] = {""}, zb[GMT_LEN64] = {""}, *c = NULL;
+	unsigned int n_errors = 0, pos, j, col, n_z_alloc = 0, z_col = GMT_Z;
+	bool invert = false;
+	char buffer[GMT_BUFSIZ] = {""}, za[GMT_LEN64] = {""}, zb[GMT_LEN64] = {""}, p[GMT_LEN256] = {""}, *c = NULL;
+	enum gmt_col_enum ctype;
 	struct GMT_OPTION *opt = NULL;
 	struct GMTAPI_CTRL *API = GMT->parent;
 
@@ -401,7 +355,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GMTSELECT_CTRL *Ctrl, struct G
 			case 'D':	/* Set GSHHS resolution */
 				Ctrl->D.active = true;
 				Ctrl->D.set = opt->arg[0];
-				if (opt->arg[1] == '+') Ctrl->D.force = true;
+				Ctrl->D.force = (opt->arg[1] == '+' && (opt->arg[2] == 'f' || opt->arg[2] == '\0'));
 				break;
 			case 'E':	/* On-boundary selection */
 				Ctrl->E.active = true;
@@ -453,6 +407,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GMTSELECT_CTRL *Ctrl, struct G
 					break;
 				}
 				/* Here we perform new syntax parsing */
+				if (gmt_validate_modifiers (GMT, opt->arg, 'L', "dp")) n_errors++;
 				if (opt->arg[0] == 0) {
 					GMT_Report (API, GMT_MSG_NORMAL, "Syntax error -L option: No file given\n");
 					n_errors++;
@@ -469,14 +424,14 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GMTSELECT_CTRL *Ctrl, struct G
 				Ctrl->N.active = true;
 				strncpy (buffer, opt->arg, GMT_BUFSIZ);
 				if (buffer[strlen(buffer)-1] == 'o' && gmt_M_compat_check (GMT, 4)) { /* Edge is considered outside */
-					GMT_Report (API, GMT_MSG_COMPAT, "Warning: Option -N...o is deprecated; use -E instead\n");
+					GMT_Report (API, GMT_MSG_COMPAT, "Option -N...o is deprecated; use -E instead\n");
 					Ctrl->E.active = true;
 					Ctrl->E.inside[N_ITEM] = GMT_INSIDE;
 					buffer[strlen(buffer)-1] = 0;
 				}
 				j = pos = 0;
-				while (j < GMTSELECT_N_CLASSES && (gmt_strtok (buffer, "/", &pos, ptr))) {
-					switch (ptr[0]) {
+				while (j < GMTSELECT_N_CLASSES && (gmt_strtok (buffer, "/", &pos, p))) {
+					switch (p[0]) {
 						case 's':	/* Skip points in this level */
 							Ctrl->N.mask[j] = false;
 							break;
@@ -497,10 +452,21 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GMTSELECT_CTRL *Ctrl, struct G
 				break;
 			case 'Z':	/* Test column-ranges */
 				Ctrl->Z.active = true;
-				col = ((c = strstr (opt->arg, "+c"))) ? atoi (&c[2]) : GMT_Z;
-				if (c) c[0] = '\0';
+				invert = false;
+				col = z_col++;	/* If no +c<col> given the we march from GMT_Z outwards */
+				if ((c = gmt_first_modifier (GMT, opt->arg, "aci"))) {	/* Process any modifiers */
+					pos = 0;	/* Reset to start of new word */
+					while (gmt_getmodopt (GMT, 'Z', c, "aci", &pos, p, &n_errors) && n_errors == 0) {
+						switch (p[0]) {
+							case 'a': Ctrl->Z.or = true; break;	/* Logical or: pass if any z-test is true */
+							case 'c': col = atoi (&p[1]); break;	/* Set z column # */
+							case 'i': invert = true; break;		/* Reverse this test */
+							default: break;	/* These are caught in gmt_getmodopt so break is just for Coverity */
+						}
+					}
+					c[0] = '\0';
+				}
 				j = sscanf (opt->arg, "%[^/]/%s", za, zb);
-				if (c) c[0] = '+';
 				if (j < 1) {
 					GMT_Report (API, GMT_MSG_NORMAL, "Syntax error -Z option: Specify z_min [and z_max]\n");
 					n_errors++;
@@ -508,18 +474,24 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GMTSELECT_CTRL *Ctrl, struct G
 				if (Ctrl->Z.n_tests == n_z_alloc) Ctrl->Z.limit = gmt_M_memory (GMT, Ctrl->Z.limit, n_z_alloc += 8, struct GMTSELECT_ZLIMIT);
 				Ctrl->Z.limit[Ctrl->Z.n_tests].min = -DBL_MAX;
 				Ctrl->Z.limit[Ctrl->Z.n_tests].max = +DBL_MAX;
-				if (!(za[0] == '-' && za[1] == '\0')) n_errors += gmt_verify_expectations (GMT, GMT->current.io.col_type[GMT_IN][GMT_Z],
-					gmt_scanf_arg (GMT, za, GMT->current.io.col_type[GMT_IN][GMT_Z], &Ctrl->Z.limit[Ctrl->Z.n_tests].min), za);
+				Ctrl->Z.limit[Ctrl->Z.n_tests].invert = invert;
+				if (!(za[0] == '-' && za[1] == '\0')) {
+					ctype = (strchr (za, 'T')) ? GMT_IS_ABSTIME : gmt_M_type (GMT, GMT_IN, col);
+					n_errors += gmt_verify_expectations (GMT, ctype, gmt_scanf_arg (GMT, za, ctype, false, &Ctrl->Z.limit[Ctrl->Z.n_tests].min), za);
+				}
 				if (j == 1)
 					Ctrl->Z.limit[Ctrl->Z.n_tests].equal = true;
 				else {
-					if (!(zb[0] == '-' && zb[1] == '\0')) n_errors += gmt_verify_expectations (GMT, GMT->current.io.col_type[GMT_IN][GMT_Z],
-						gmt_scanf_arg (GMT, zb, GMT->current.io.col_type[GMT_IN][GMT_Z], &Ctrl->Z.limit[Ctrl->Z.n_tests].max), zb);
+					if (!(zb[0] == '-' && zb[1] == '\0')) {
+						ctype = (strchr (zb, 'T')) ? GMT_IS_ABSTIME : gmt_M_type (GMT, GMT_IN, col);
+						n_errors += gmt_verify_expectations (GMT, ctype, gmt_scanf_arg (GMT, zb, ctype, false, &Ctrl->Z.limit[Ctrl->Z.n_tests].max), zb);
+					}
 				}
 				n_errors += gmt_M_check_condition (GMT, Ctrl->Z.limit[Ctrl->Z.n_tests].max <= Ctrl->Z.limit[Ctrl->Z.n_tests].min, "Syntax error: -Z must have zmax > zmin!\n");
 				Ctrl->Z.limit[Ctrl->Z.n_tests].col = col;
 				if (col > Ctrl->Z.max_col) Ctrl->Z.max_col = col;
 				Ctrl->Z.n_tests++;
+				if (c) c[0] = '+';
 				break;
 #ifdef DEBUG
 			case '+':	/* Undocumented option to increase path-fix resolution */
@@ -563,25 +535,26 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GMTSELECT_CTRL *Ctrl, struct G
 
 int GMT_gmtselect (void *V_API, int mode, void *args) {
 	int err;	/* Required by gmt_M_err_fail */
-	unsigned int base = 3, np[2] = {0, 0}, r_mode;
+	unsigned int base = 3, np[2] = {0, 0};
 	unsigned int side, col, id;
 	int n_fields, ind, wd[2] = {0, 0}, n_minimum = 2, bin, last_bin = INT_MAX, error = 0;
-	bool inside = false, need_header = false, shuffle = false, just_copy_record = false, pt_cartesian = false;
+	bool inside = false, need_header = false, pt_cartesian = false;
 	bool output_header = false, do_project = false, no_resample = false, keep;
 
 	uint64_t k, row, seg, n_read = 0, n_pass = 0, n_output = 0;
 
-	double xx, yy, *in = NULL;
+	double xx, yy;
 	double west_border = 0.0, east_border = 0.0, xmin, xmax, ymin, ymax, lon;
 
 	char *shore_resolution[5] = {"full", "high", "intermediate", "low", "crude"};
-	char extra[GMT_BUFSIZ] = {""};
 
 	struct GMT_DATATABLE *pol = NULL, *line = NULL, *point = NULL;
 	struct GMT_GSHHS_POL *p[2] = {NULL, NULL};
 	struct GMT_SHORE c;
 	struct GMT_DATASET *Cin = NULL, *Lin = NULL, *Fin = NULL;
+	struct GMT_DATASEGMENT_HIDDEN *SH = NULL;
 	struct GMT_GRID *G = NULL;
+	struct GMT_RECORD *In = NULL;
 	struct GMTSELECT_CTRL *Ctrl = NULL;
 	struct GMT_CTRL *GMT = NULL, *GMT_cpy = NULL;
 	struct GMT_OPTION *options = NULL;
@@ -593,23 +566,21 @@ int GMT_gmtselect (void *V_API, int mode, void *args) {
 	if (mode == GMT_MODULE_PURPOSE) return (usage (API, GMT_MODULE_PURPOSE));	/* Return the purpose of program */
 	options = GMT_Create_Options (API, mode, args);	if (API->error) return (API->error);	/* Set or get option list */
 
-	if (!options || options->option == GMT_OPT_USAGE) bailout (usage (API, GMT_USAGE));/* Return the usage message */
-	if (options->option == GMT_OPT_SYNOPSIS) bailout (usage (API, GMT_SYNOPSIS));	/* Return the synopsis */
+	if ((error = gmt_report_usage (API, options, 0, usage)) != GMT_NOERROR) bailout (error);	/* Give usage if requested */
 
 	/* Parse the command-line arguments */
 
-	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
 	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
 	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
 	if ((error = parse (GMT, Ctrl, options)) != 0) Return (error);
 
 	/*---------------------------- This is the gmtselect main code ----------------------------*/
 
-	GMT_Report (API, GMT_MSG_VERBOSE, "Processing input table data\n");
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Processing input table data\n");
 
-	if (Ctrl->C.active && !gmt_M_is_geographic (GMT, GMT_IN)) pt_cartesian = true;
+	if (Ctrl->C.active && gmt_M_is_cartesian (GMT, GMT_IN)) pt_cartesian = true;
 
-	shuffle = (GMT->current.setting.io_lonlat_toggle[GMT_IN] != GMT->current.setting.io_lonlat_toggle[GMT_OUT]);	/* Must rewrite output record */
 	n_minimum = Ctrl->Z.max_col;	/* Minimum number of columns in ASCII input */
 	
 	if (!GMT->common.R.active[RSET] && Ctrl->N.active) {	/* If we use coastline data or used -fg but didn't give -R we implicitly set -Rg */
@@ -620,7 +591,7 @@ int GMT_gmtselect (void *V_API, int mode, void *args) {
 	if (GMT->common.R.active[RSET]) {	/* -R was set directly or indirectly; hence must set -J if not supplied */
 		if (!GMT->common.J.active) {	/* -J not specified, set one implicitly */
 			/* Supply dummy linear proj */
-			GMT->current.proj.projection = GMT->current.proj.xyz_projection[GMT_X] = GMT->current.proj.xyz_projection[GMT_Y] = GMT_LINEAR;
+			GMT->current.proj.projection = GMT->current.proj.projection_GMT = GMT->current.proj.xyz_projection[GMT_X] = GMT->current.proj.xyz_projection[GMT_Y] = GMT_LINEAR;
 			GMT->current.proj.pars[0] = GMT->current.proj.pars[1] = 1.0;
 			GMT->common.J.active = no_resample = true;
 		}
@@ -633,11 +604,11 @@ int GMT_gmtselect (void *V_API, int mode, void *args) {
 				GMT->common.R.wesn[XHI] += 360.0;
 			}
 		}
-		if (gmt_M_err_pass (GMT, gmt_map_setup (GMT, GMT->common.R.wesn), "")) Return (GMT_PROJECTION_ERROR);
+		if (gmt_M_err_pass (GMT, gmt_proj_setup (GMT, GMT->common.R.wesn), "")) Return (GMT_PROJECTION_ERROR);
 		if (no_resample) GMT->current.map.parallel_straight = GMT->current.map.meridian_straight = 2;	/* No resampling along bin boundaries */
 	}
 
-	if (do_project) GMT_Report (API, GMT_MSG_VERBOSE, "Warning: -J means all data will be projected before tests are applied\n");
+	if (do_project) GMT_Report (API, GMT_MSG_VERBOSE, "-J means all data will be projected before tests are applied\n");
 	 
 	if (Ctrl->N.active) {	/* Set up GSHHS */
 		if (Ctrl->D.force) Ctrl->D.set = gmt_shore_adjust_res (GMT, Ctrl->D.set);
@@ -685,7 +656,8 @@ int GMT_gmtselect (void *V_API, int mode, void *args) {
 		}
 		point = Cin->table[0];	/* Can only be one table since we read a single file */
 		for (seg = 0; seg < point->n_segments; seg++) {
-			if (Cin->n_columns == 2) point->segment[seg]->dist = Ctrl->C.dist;
+			SH = gmt_get_DS_hidden (point->segment[seg]);
+			if (Cin->n_columns == 2) SH->dist = Ctrl->C.dist;
 			if (do_project) {	/* Convert all the points using the map projection */
 				for (row = 0; row < point->segment[seg]->n_rows; row++) {
 					gmt_geo_to_xy (GMT, point->segment[seg]->data[GMT_X][row], point->segment[seg]->data[GMT_Y][row], &xx, &yy);
@@ -734,7 +706,8 @@ int GMT_gmtselect (void *V_API, int mode, void *args) {
 		}
 		line = Lin->table[0];	/* Can only be one table since we read a single file */
 		for (seg = 0; seg < line->n_segments; seg++) {
-			if (Ctrl->L.dist > 0.0) line->segment[seg]->dist = Ctrl->L.dist;	/* Only override when nonzero */
+			SH = gmt_get_DS_hidden (line->segment[seg]);
+			if (Ctrl->L.dist > 0.0) SH->dist = Ctrl->L.dist;	/* Only override when nonzero */
 			if (do_project) {	/* Convert all the line points using the map projection */
 				for (row = 0; row < line->segment[seg]->n_rows; row++) {
 					gmt_geo_to_xy (GMT, line->segment[seg]->data[GMT_X][row], line->segment[seg]->data[GMT_Y][row], &xx, &yy);
@@ -764,7 +737,10 @@ int GMT_gmtselect (void *V_API, int mode, void *args) {
 					pol->segment[seg]->data[GMT_Y][row] = yy;
 				}
 			}
+			gmt_set_inside_mode (GMT, NULL, GMT_IOO_CARTESIAN);
 		}
+		else
+			gmt_set_inside_mode (GMT, Fin, GMT_IOO_UNKNOWN);
 	}
 
 	if (Ctrl->G.active) {	/* Grid mask */
@@ -776,7 +752,7 @@ int GMT_gmtselect (void *V_API, int mode, void *args) {
 	gmt_reenable_bhi_opts (GMT);	/* Recover settings provided by user (if -b -h -i were used at all) */
 
 	/* Specify input and output expected columns */
-	if ((error = gmt_set_cols (GMT, GMT_IN,  0)) != 0) Return (error);
+	if ((error = GMT_Set_Columns (API, GMT_IN, 0, GMT_COL_FIX)) != GMT_NOERROR) Return (error);
 
 	/* Gather input/output  file names (or stdin/out) and enable i/o */
 	
@@ -798,13 +774,11 @@ int GMT_gmtselect (void *V_API, int mode, void *args) {
 
 	/* Now we are ready to take on some input values */
 
-	just_copy_record = (gmt_is_ascii_record (GMT, options) && !shuffle && !GMT->common.s.active);
 	GMT->common.b.ncol[GMT_OUT] = UINT_MAX;	/* Flag to have it reset to GMT->common.b.ncol[GMT_IN] when writing */
-	r_mode = (just_copy_record) ? GMT_READ_MIXED : GMT_READ_DATA;
 	gmt_set_segmentheader (GMT, GMT_OUT, false);	/* Since processing of -C|L|F files might have turned it on [should be determined below] */
 	
 	do {	/* Keep returning records until we reach EOF */
-		if ((in = GMT_Get_Record (API, r_mode, &n_fields)) == NULL) {	/* Read next record, get NULL if special case */
+		if ((In = GMT_Get_Record (API, GMT_READ_DATA, &n_fields)) == NULL) {	/* Read next record, get NULL if special case */
 			if (gmt_M_rec_is_error (GMT)) {		/* Bail if there are any read errors */
 				Return (GMT_RUNTIME_ERROR);
 			}
@@ -823,12 +797,12 @@ int GMT_gmtselect (void *V_API, int mode, void *args) {
 		/* Data record to process */
 
 		if (n_output == 0) {
-			gmt_set_cols (GMT, GMT_OUT, gmt_get_cols (GMT, GMT_IN));
+			GMT_Set_Columns (API, GMT_OUT, (unsigned int)gmt_get_cols (GMT, GMT_IN), (In->text) ? GMT_COL_FIX : GMT_COL_FIX_NO_TEXT);
 			n_output = gmt_get_cols (GMT, GMT_OUT);
 		}
 
 		n_read++;
-		if (n_read%1000 == 0) GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Read %" PRIu64 " records, passed %" PRIu64 "records\r", n_read, n_pass);
+		if (n_read%1000 == 0) GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Read %" PRIu64 " records, passed %" PRIu64 " records\n", n_read, n_pass);
 
 		if (n_fields < n_minimum) {	/* Bad number of columns */
 			if (Ctrl->Z.active)
@@ -841,28 +815,34 @@ int GMT_gmtselect (void *V_API, int mode, void *args) {
 		if (Ctrl->Z.active) {	/* Apply z-range test(s) */
 			for (k = 0, keep = true; keep && k < Ctrl->Z.n_tests; k++) {
 				col = Ctrl->Z.limit[k].col;			/* Shorthand notation */
-				if (gmt_M_is_dnan (in[col]))
+				if (gmt_M_is_dnan (In->data[col]))
 					keep = true;		/* Make no decision on NaNs here; see -s instead */
-				else if (Ctrl->Z.limit[k].equal)
-					inside = doubleAlmostEqualZero (in[col], Ctrl->Z.limit[k].min);
-				else
-					inside = (in[col] >= Ctrl->Z.limit[k].min && in[col] <= Ctrl->Z.limit[k].max); 
+				else if (Ctrl->Z.limit[k].equal) {
+					inside = doubleAlmostEqualZero (In->data[col], Ctrl->Z.limit[k].min);
+					if (Ctrl->Z.limit[k].invert) inside = !inside;	/* Flip the result for the test below */
+				}
+				else {
+					inside = (In->data[col] >= Ctrl->Z.limit[k].min && In->data[col] <= Ctrl->Z.limit[k].max); 
+					if (Ctrl->Z.limit[k].invert) inside = !inside;	/* Flip the result for the test below */
+				}
 				if (inside != Ctrl->I.pass[GMT_SELECT_Z]) keep = false;
+				if (Ctrl->Z.or && keep)
+					break;	/* At least one test passed so we are done */
 			}
 			if (!keep) { output_header = need_header; continue;}
 		}
 
-		lon = in[GMT_X];	/* Use copy since we may have to wrap 360 */
+		lon = In->data[GMT_X];	/* Use copy since we may have to wrap 360 */
 		if (GMT->common.R.active[RSET]) {	/* Apply region test */
-			inside = !gmt_map_outside (GMT, lon, in[GMT_Y]);
+			inside = !gmt_map_outside (GMT, lon, In->data[GMT_Y]);
 			if (inside != Ctrl->I.pass[GMT_SELECT_R]) { output_header = need_header; continue;}
 		}
 
 		if (do_project)	/* First project the input point */
-			gmt_geo_to_xy (GMT, lon, in[GMT_Y], &xx, &yy);
+			gmt_geo_to_xy (GMT, lon, In->data[GMT_Y], &xx, &yy);
 		else {
 			xx = lon;
-			yy = in[GMT_Y];
+			yy = In->data[GMT_Y];
 		}
 		
 		if (Ctrl->C.active) {	/* Check for distance to points */
@@ -880,7 +860,7 @@ int GMT_gmtselect (void *V_API, int mode, void *args) {
 				gmt_set_cartesian (GMT, GMT_IN);
 			inside = 0;
 			for (seg = 0; seg < pol->n_segments && !inside; seg++) {	/* Check each polygon until we find that our point is inside */
-				if (gmt_M_polygon_is_hole (pol->segment[seg])) continue;	/* Holes are handled within gmt_inonout */
+				if (gmt_polygon_is_hole (GMT, pol->segment[seg])) continue;	/* Holes are handled within gmt_inonout */
 				inside = (gmt_inonout (GMT, xx, yy, pol->segment[seg]) >= Ctrl->E.inside[F_ITEM]);
 			}
 			if (do_project)	/* Reset input type for gmt_inonout to do Cartesian mode */
@@ -890,13 +870,13 @@ int GMT_gmtselect (void *V_API, int mode, void *args) {
 
 		if (Ctrl->G.active) {	/* Check if we are in/out-side mask cell */
 			unsigned int row, col;
-			if (gmt_M_y_is_outside (GMT, in[GMT_Y], G->header->wesn[YLO], G->header->wesn[YHI]) ||	/* Outside y-range */
-				gmt_x_is_outside (GMT, &in[GMT_X], G->header->wesn[XLO], G->header->wesn[XHI])) {	/* Outside x-range */
+			if (gmt_M_y_is_outside (GMT, In->data[GMT_Y], G->header->wesn[YLO], G->header->wesn[YHI]) ||	/* Outside y-range */
+				gmt_x_is_outside (GMT, &In->data[GMT_X], G->header->wesn[XLO], G->header->wesn[XHI])) {	/* Outside x-range */
 				inside = false;	/* Outside both y- and x-range (or periodic longitude) for this grid */
 			}
 			else
-				inside = !gmt_row_col_out_of_bounds (GMT, in, G->header, &row, &col);
-			if (inside) {	/* Inside -R so check the node value */
+				inside = !gmt_row_col_out_of_bounds (GMT, In->data, G->header, &row, &col);
+			if (inside) {	/* Inside grid so check the node value */
 				uint64_t node = gmt_M_ijp (G->header, row, col);
 				inside = !(gmt_M_is_fnan (G->data[node]) || G->data[node] == 0);
 			}
@@ -907,7 +887,7 @@ int GMT_gmtselect (void *V_API, int mode, void *args) {
 			int brow, i, this_point_level;
 			xx = lon;
 			while (xx < 0.0) xx += 360.0;
-			brow = irint (floor ((90.0 - in[GMT_Y]) / c.bsize));
+			brow = irint (floor ((90.0 - In->data[GMT_Y]) / c.bsize));
 			if (brow >= c.bin_ny) brow = c.bin_ny - 1;	/* Presumably only kicks in for south pole */
 			col = urint (floor (xx / c.bsize));
 			bin = brow * c.bin_nx + col;
@@ -936,7 +916,7 @@ int GMT_gmtselect (void *V_API, int mode, void *args) {
 			}
 			else {
 				this_point_level = 0;
-				gmt_geo_to_xy (GMT, lon, in[GMT_Y], &xx, &yy);
+				gmt_geo_to_xy (GMT, lon, In->data[GMT_Y], &xx, &yy);
 				for (id = 0; id < 2; id++) {
 
 					for (k = 0; k < np[id]; k++) {
@@ -979,15 +959,7 @@ int GMT_gmtselect (void *V_API, int mode, void *args) {
 			output_header = false;
 		}
 
-		if (just_copy_record) {	/* Want to (or can) do text output */
-			if (GMT->common.a.active) {	/* Add selected aspatial fields to output record */
-				ogr_to_text (GMT, GMT->current.io.OGR, extra);
-				strcat (API->GMT->current.io.record, extra);
-			}
-			GMT_Put_Record (API, GMT_WRITE_TEXT, NULL);
-		}
-		else
-			GMT_Put_Record (API, GMT_WRITE_DATA, in);
+		GMT_Put_Record (API, GMT_WRITE_DATA, In);
 		n_pass++;
 	} while (true);
 	
@@ -998,7 +970,7 @@ int GMT_gmtselect (void *V_API, int mode, void *args) {
 		Return (API->error);
 	}
 
-	GMT_Report (API, GMT_MSG_VERBOSE, "Read %" PRIu64 " records, passed %" PRIu64" records\n", n_read, n_pass);
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Read %" PRIu64 " records, passed %" PRIu64" records\n", n_read, n_pass);
 
 	if (Ctrl->N.active) {
 		gmt_free_shore (GMT, &c);
