@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *	Copyright (c) 1991-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  * Brief synopsis: grd2cpt reads a 2d binary gridded grdfile and creates a continuous-color-
@@ -29,18 +28,19 @@
  *
  * Author:	Walter H. F. Smith
  * Date:	1-JAN-2010
- * Version:	5 API
+ * Version:	6 API
  *
  */
 
 #include "gmt_dev.h"
 
-#define THIS_MODULE_NAME	"grd2cpt"
+#define THIS_MODULE_CLASSIC_NAME	"grd2cpt"
+#define THIS_MODULE_MODERN_NAME	"grd2cpt"
 #define THIS_MODULE_LIB		"core"
 #define THIS_MODULE_PURPOSE	"Make linear or histogram-equalized color palette table from grid"
 #define THIS_MODULE_KEYS	"<G{+,>C}"
 #define THIS_MODULE_NEEDS	""
-#define THIS_MODULE_OPTIONS "->RVh"
+#define THIS_MODULE_OPTIONS	"->RVh"
 
 #define GRD2CPT_N_LEVELS	11	/* The default number of levels if nothing is specified */
 
@@ -52,7 +52,7 @@ struct GRD2CPT_CTRL {
 		bool active;
 		char *file;
 	} Out;
-	struct A {	/* -A+ */
+	struct A {	/* -A<transp>[+a] */
 		bool active;
 		unsigned int mode;
 		double value;
@@ -78,6 +78,9 @@ struct GRD2CPT_CTRL {
 		bool active;
 		double z_low, z_high;
 	} G;
+	struct H {	/* -H */
+		bool active;
+	} H;
 	struct I {	/* -I[z][c] */
 		bool active;
 		unsigned int mode;
@@ -96,17 +99,17 @@ struct GRD2CPT_CTRL {
 		bool active;
 		unsigned int mode;
 	} Q;
-	struct S {	/* -S<z_start>/<z_stop>/<z_inc> or -S<n_levels> */
+	struct T {	/* -T<start>/<stop>/<inc> or -T<n_levels> */
 		bool active;
-		unsigned int mode;	/* 0 or 1 (-Sn) */
+		unsigned int mode;	/* 0 or 1 (-Tn) */
 		unsigned int n_levels;
 		double low, high, inc;
 		char *file;
-	} S;
-	struct T {	/* -T<kind> */
+	} T;
+	struct S {	/* -S<kind> */
 		bool active;
 		int kind; /* -1 symmetric +-zmin, +1 +-zmax, -2 = +-Minx(|zmin|,|zmax|), +2 = +-Max(|zmin|,|zmax|), 0 = min to max [Default] */
-	} T;
+	} S;
 	struct W {	/* -W[w] */
 		bool active;
 		bool wrap;
@@ -130,21 +133,23 @@ GMT_LOCAL void Free_Ctrl (struct GMT_CTRL *GMT, struct GRD2CPT_CTRL *C) {	/* Dea
 	if (!C) return;
 	gmt_M_str_free (C->Out.file);
 	gmt_M_str_free (C->C.file);
-	gmt_M_str_free (C->S.file);
+	gmt_M_str_free (C->T.file);
 	gmt_M_free (GMT, C);
 }
 
 GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
-	gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
+	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
+	const char *H_OPT = (API->GMT->current.setting.run_mode == GMT_MODERN) ? " [-H]" : "";
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: grd2cpt <grid> [-A[+]<transparency>] [-C<cpt>] [-D[i]] [-E[<nlevels>]] [-F[R|r|h|c][+c]]\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t[-G<zlo>/<zhi>] [-I[c][z]] [-L<min_limit>/<max_limit>] [-M] [-N] [-Q[i|o]]\n\t[%s] [-S<z_start>/<z_stop>/<z_inc> or -S<n>]\n\t[-T<-|+|=|_>] [%s] [-W[w]] [-Z]\n\n", GMT_Rgeo_OPT, GMT_V_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "usage: %s <grid> [-A<transparency>[+a]] [-C<cpt>] [-D[i]] [-E[<nlevels>]] [-F[R|r|h|c][+c]]\n", name);
+	GMT_Message (API, GMT_TIME_NONE, "\t[-G<zlo>/<zhi>]%s [-I[c][z]] [-L<min_limit>/<max_limit>] [-M] [-N] [-Q[i|o]]\n", H_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "\t[%s] [-T<start>/<stop>/<inc> or -T<n>]\n\t[-Sh|l|m|u] [%s] [-W[w]] [-Z] [%s]\n\n", GMT_Rgeo_OPT, GMT_V_OPT, GMT_PAR_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
 	GMT_Message (API, GMT_TIME_NONE, "\t<grid> is name of one or more grid files.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\n\tOPTIONS:\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-A Set constant transparency for all colors; prepend + to also include back-, for-, and nan-colors [0].\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t-A Set constant transparency for all colors; append +a to also include back-, for-, and nan-colors [0]\n");
 	if (gmt_list_cpt (API->GMT, 'C')) return (GMT_CPT_READ_ERROR);	/* Display list of available color tables */
 	GMT_Message (API, GMT_TIME_NONE, "\t-D Set back- and foreground color to match the bottom/top limits\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   in the output CPT [Default uses color table]. Append i to match the\n");
@@ -156,6 +161,8 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t   Append +c to output a discrete CPT in categorical CPT format.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-G Truncate incoming CPT to be limited to the z-range <zlo>/<zhi>.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   To accept one of the incoming limits, set that limit to NaN.\n");
+	if (API->GMT->current.setting.run_mode == GMT_MODERN)
+		GMT_Message (API, GMT_TIME_NONE, "\t-H Also write CPT to stdout [Default just saves as current CPT].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-I Reverse sense of CPT in one or two ways:\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   -Ic Reverse sense of color table as well as back- and foreground color [Default].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   -Iz Reverse sign of z-values in the color table (takes affect before -G, T are consulted).\n");
@@ -167,15 +174,15 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t   -Qi: z-values are actually log10(z). Assign colors and write z [Default].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   -Qo: z-values are z, but take log10(z), assign colors and write z.\n");
 	GMT_Option (API, "R");
-	GMT_Message (API, GMT_TIME_NONE, "\t-S CDF sample points should Step from <z_start> to <z_stop> by <z_inc>.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Use -S<n> to select <n> points from a cumulative normal distribution [%d].\n", GRD2CPT_N_LEVELS);
-	GMT_Message (API, GMT_TIME_NONE, "\t   <z_start> maps to data min and <z_stop> maps to data max (but see -L).\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t-S Force color tables to be symmetric about 0. Append one modifier:\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   l (low)   for values symmetric about zero from -|zmin| to +|zmin|.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   u (upper) for values symmetric about zero from -|zmax| to +|zmax|.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   m (min)   for values symmetric about zero -+min(|zmin|,|zmax|).\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   h (high)  for values symmetric about zero -+max(|zmin|,|zmax|).\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t-T CDF sample points should range from <start> to <stop> by <inc>.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   Use -T<n> to select <n> points from a cumulative normal distribution [%d].\n", GRD2CPT_N_LEVELS);
+	GMT_Message (API, GMT_TIME_NONE, "\t   <start> maps to data min and <stop> maps to data max (but see -L).\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   [Default uses equidistant steps for a Gaussian CDF].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-T Force color tables to be symmetric about 0. Append one modifier:\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   - for values symmetric about zero from -|zmin| to +|zmin|.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   + for values symmetric about zero from -|zmax| to +|zmax|.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   _ for values symmetric about zero -+min(|zmin|,|zmax|).\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   = for values symmetric about zero -+max(|zmin|,|zmax|).\n");
 	GMT_Option (API, "V");
 	GMT_Message (API, GMT_TIME_NONE, "\t-W Do not interpolate color palette. Alternatively, append w for a wrapped CPT.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-Z Create a continuous color palette [Default is discontinuous, i.e., constant color intervals].\n");
@@ -193,8 +200,8 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRD2CPT_CTRL *Ctrl, struct GMT
 
 	int n;
 	unsigned int n_errors = 0, n_files[2] = {0, 0};
-	char txt_a[GMT_LEN32] = {""}, txt_b[GMT_LEN32] = {""};
-	char kind;
+	char txt_a[GMT_LEN32] = {""}, txt_b[GMT_LEN32] = {""}, *c;
+	char *T_arg = NULL, *S_arg = NULL;
 	struct GMT_OPTION *opt = NULL;
 	struct GMTAPI_CTRL *API = GMT->parent;
 
@@ -212,10 +219,20 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRD2CPT_CTRL *Ctrl, struct GMT
 
 			/* Processes program-specific parameters */
 
-			case 'A':	/* Sets transparency */
+			case 'A':	/* Sets transparency [-A<transp>[+a]] */
 				Ctrl->A.active = true;
-				if (opt->arg[0] == '+') Ctrl->A.mode = 1;
-				Ctrl->A.value = 0.01 * atof (&opt->arg[Ctrl->A.mode]);
+				if (opt->arg[0] == '+') {	/* Old syntax */
+					Ctrl->A.mode = 1;
+					Ctrl->A.value = 0.01 * atof (&opt->arg[1]);
+				}
+				else if ((c = strstr (opt->arg, "+a"))) {
+					Ctrl->A.mode = 1;
+					c[0] = '\0';
+					Ctrl->A.value = 0.01 * atof (opt->arg);
+					c[0] = '+';
+				}
+				else
+					Ctrl->A.value = 0.01 * atof (opt->arg);
 				break;
 			case 'C':	/* Get CPT */
 				Ctrl->C.active = true;
@@ -234,6 +251,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRD2CPT_CTRL *Ctrl, struct GMT
 				}
 				break;
 			case 'F':	/* Set color model for output */
+				if (gmt_validate_modifiers (GMT, opt->arg, 'F', "c")) n_errors++;
 				if (gmt_get_modifier (opt->arg, 'c', txt_a)) {
 					Ctrl->F.cat = true;
 					if (txt_a[0] == '\0') break;
@@ -255,6 +273,9 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRD2CPT_CTRL *Ctrl, struct GMT
 				n_errors += gmt_M_check_condition (GMT, gmt_M_is_dnan (Ctrl->G.z_low) && gmt_M_is_dnan (Ctrl->G.z_high),
 								"Syntax error -G option: Both of z_low/z_high cannot be NaN\n");
 				break;
+			case 'H':	/* Modern mode only: write CPT to stdout */
+				Ctrl->H.active = true;
+				break;
 			case 'I':	/* Invert table */
 				Ctrl->I.active = true;
 				if ((Ctrl->I.mode = gmt_parse_inv_cpt (GMT, opt->arg)) == UINT_MAX)
@@ -280,37 +301,13 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRD2CPT_CTRL *Ctrl, struct GMT
 				else			/* Input is log10(z) */
 					Ctrl->Q.mode = 1;
 				break;
-			case 'S':	/* Sets sample range */
-				Ctrl->S.active = true;
-				if (strchr (opt->arg, '/')) {	/* Gave low/high/inc */
-					if (sscanf (opt->arg, "%lf/%lf/%lf", &Ctrl->S.low, &Ctrl->S.high, &Ctrl->S.inc) != 3) {
-						GMT_Report (API, GMT_MSG_NORMAL, "Syntax error -S option: Cannot decode values\n");
-						n_errors++;
-					}
-					Ctrl->S.mode = 0;
-				}
-				else if (opt->arg[0]) {	/* Gave -S<nlevels> */
-					Ctrl->S.n_levels = atoi (opt->arg);
-					Ctrl->S.mode = 1;
-				}
-				break;
-			case 'T':	/* Force symmetry */
+			case 'T':	/* Sets sample range */
 				Ctrl->T.active = true;
-				kind = '\0';
-				if (sscanf (opt->arg, "%c", &kind) != 1) {
-					GMT_Report (API, GMT_MSG_NORMAL, "Syntax error -T option: Cannot decode option\n");
-					n_errors++;
-				}
-				switch (kind) {
-					case '-': Ctrl->T.kind = -1; break; /* Symmetric with |zmin| range */
-					case '+': Ctrl->T.kind = +1; break; /* Symmetric with |zmax| range */
-					case '_': Ctrl->T.kind = -2; break; /* Symmetric with min(|zmin|,|zmax|) range */
-					case '=': Ctrl->T.kind = +2; break; /* Symmetric with max(|zmin|,|zmax|) range */
-					default:
-						GMT_Report (API, GMT_MSG_NORMAL, "Syntax error -T option: Must append modifier -, +, _, or =\n");
-						n_errors++;
-						break;
-				}
+				T_arg = opt->arg;
+				break;
+			case 'S':	/* Force symmetry */
+				Ctrl->S.active = true;
+				S_arg = opt->arg;
 				break;
 			case 'W':	/* Do not interpolate colors */
 				if (opt->arg[0] == 'w')
@@ -328,6 +325,75 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRD2CPT_CTRL *Ctrl, struct GMT
 		}
 	}
 
+	if (T_arg) {	/* Must handle old or new syntax */
+		if (strchr ("-+_=", T_arg[0]) && T_arg[1] == '\0') {	/* Old -Targs for symmetry given */
+			if (gmt_M_compat_check (GMT, 5)) {	/* OK to parse that? */
+				GMT_Report (API, GMT_MSG_COMPAT, "Option -T-|+|_|= is deprecated; use -Sl|u|m|h instead.\n");
+				Ctrl->S.active = true;
+				if (!S_arg) Ctrl->T.active = false;
+				switch (T_arg[0]) {
+					case '-': Ctrl->S.kind = -1; break; /* Symmetric with |zmin| range */
+					case '+': Ctrl->S.kind = +1; break; /* Symmetric with |zmax| range */
+					case '_': Ctrl->S.kind = -2; break; /* Symmetric with min(|zmin|,|zmax|) range */
+					case '=': Ctrl->S.kind = +2; break; /* Symmetric with max(|zmin|,|zmax|) range */
+					default: break;
+				}
+			}
+			else {
+				GMT_Report (API, GMT_MSG_NORMAL, "Syntax error -T option: Cannot decode values %s\n", T_arg);
+				n_errors++;
+			}
+		}
+		else {	/* Got correct modern args */
+			if (strchr (T_arg, '/')) {	/* Gave low/high/inc */
+				if (sscanf (T_arg, "%lf/%lf/%lf", &Ctrl->T.low, &Ctrl->T.high, &Ctrl->T.inc) != 3) {
+					GMT_Report (API, GMT_MSG_NORMAL, "Syntax error -T option: Cannot decode values\n");
+					n_errors++;
+				}
+				Ctrl->T.mode = 0;
+			}
+			else if (T_arg[0]) {	/* Gave -T<nlevels> */
+				Ctrl->T.n_levels = atoi (T_arg);
+				Ctrl->T.mode = 1;
+			}
+		}
+	}
+	if (S_arg) {	/* Must handle old or new syntax */
+		if (strchr ("hlmu", S_arg[0]) && S_arg[1] == '\0') {	/* New -S for symmetry */
+			switch (S_arg[0]) {
+				case 'l': Ctrl->S.kind = -1; break; /* Symmetric with |zmin| range */
+				case 'u': Ctrl->S.kind = +1; break; /* Symmetric with |zmax| range */
+				case 'm': Ctrl->S.kind = -2; break; /* Symmetric with min(|zmin|,|zmax|) range */
+				case 'h': Ctrl->S.kind = +2; break; /* Symmetric with max(|zmin|,|zmax|) range */
+				default: break;
+			}
+		}
+		else if (gmt_M_compat_check (GMT, 5)) {	/* Old-style -S range */
+			GMT_Report (API, GMT_MSG_COMPAT, "Option -S<start>/<stop>/<inc> or -S<n> is deprecated; use -T instead.\n");
+			if (strchr (S_arg, '/')) {	/* Gave low/high/inc */
+				if (sscanf (S_arg, "%lf/%lf/%lf", &Ctrl->T.low, &Ctrl->T.high, &Ctrl->T.inc) != 3) {
+					GMT_Report (API, GMT_MSG_NORMAL, "Syntax error -T option: Cannot decode values %s\n", S_arg);
+					n_errors++;
+				}
+				Ctrl->T.mode = 0;
+			}
+			else if (S_arg[0]) {	/* Gave -S<nlevels> */
+				Ctrl->T.n_levels = atoi (S_arg);
+				Ctrl->T.mode = 1;
+			}
+			if (!T_arg) Ctrl->S.active = false;
+			Ctrl->T.active = true;
+		}
+		else {
+			GMT_Report (API, GMT_MSG_NORMAL, "Syntax error -S option: Cannot decode values %s\n", S_arg);
+			n_errors++;
+		}
+	}
+	
+	if (Ctrl->H.active && GMT->current.setting.run_mode == GMT_CLASSIC) {
+		n_errors++;
+		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Unrecognized option -H\n");
+	}
 	n_errors += gmt_M_check_condition (GMT, n_files[GMT_IN] < 1, "Error: No grid name(s) specified.\n");
 	n_errors += gmt_M_check_condition (GMT, Ctrl->W.active && Ctrl->Z.active,
 					"Syntax error: -W and -Z cannot be used simultaneously\n");
@@ -335,12 +401,12 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRD2CPT_CTRL *Ctrl, struct GMT
 	                                "Syntax error: -F+c and -Z cannot be used simultaneously\n");
 	n_errors += gmt_M_check_condition (GMT, Ctrl->L.active && Ctrl->L.min >= Ctrl->L.max,
 					"Syntax error -L option: min_limit must be less than max_limit.\n");
-	n_errors += gmt_M_check_condition (GMT, Ctrl->S.active && Ctrl->S.mode == 0 && (Ctrl->S.high <= Ctrl->S.low || Ctrl->S.inc <= 0.0),
+	n_errors += gmt_M_check_condition (GMT, Ctrl->T.active && Ctrl->T.mode == 0 && (Ctrl->T.high <= Ctrl->T.low || Ctrl->T.inc <= 0.0),
 					"Syntax error -S option: Bad arguments\n");
-	n_errors += gmt_M_check_condition (GMT, Ctrl->S.active && Ctrl->S.mode == 1  && Ctrl->S.n_levels == 0,
+	n_errors += gmt_M_check_condition (GMT, Ctrl->T.active && Ctrl->T.mode == 1  && Ctrl->T.n_levels == 0,
 					"Syntax error -S option: Bad arguments\n");
-	n_errors += gmt_M_check_condition (GMT, Ctrl->S.active && (Ctrl->T.active || Ctrl->E.active),
-					"Syntax error -S option: Cannot be combined with -E nor -T option.\n");
+	n_errors += gmt_M_check_condition (GMT, Ctrl->T.active && (Ctrl->S.active || Ctrl->E.active),
+					"Syntax error -T option: Cannot be combined with -E nor -S option.\n");
 	n_errors += gmt_M_check_condition (GMT, n_files[GMT_OUT] > 1, "Syntax error: Only one output destination can be specified\n");
 	n_errors += gmt_M_check_condition (GMT, Ctrl->A.active && (Ctrl->A.value < 0.0 || Ctrl->A.value > 1.0),
 					"Syntax error -A: Transparency must be n 0-100 range [0 or opaque]\n");
@@ -367,6 +433,7 @@ int GMT_grd2cpt (void *V_API, int mode, void *args) {
 	unsigned int row, col, j, cpt_flags = 0;
 	int signed_levels, error = 0;
 	size_t n_alloc = GMT_TINY_CHUNK;
+	bool write = false;
 
 	char format[GMT_BUFSIZ] = {""}, *l = NULL, **grdfile = NULL;
 
@@ -391,12 +458,11 @@ int GMT_grd2cpt (void *V_API, int mode, void *args) {
 	if (mode == GMT_MODULE_PURPOSE) return (usage (API, GMT_MODULE_PURPOSE));	/* Return the purpose of program */
 	options = GMT_Create_Options (API, mode, args);	if (API->error) return (API->error);	/* Set or get option list */
 
-	if (!options || options->option == GMT_OPT_USAGE) bailout (usage (API, GMT_USAGE));	/* Return the usage message */
-	if (options->option == GMT_OPT_SYNOPSIS) bailout (usage (API, GMT_SYNOPSIS));	/* Return the synopsis */
+	if ((error = gmt_report_usage (API, options, 0, usage)) != GMT_NOERROR) bailout (error);	/* Give usage if requested */
 
 	/* Parse the command-line arguments */
 
-	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
 	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
 	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
 	if ((error = parse (GMT, Ctrl, options)) != 0) Return (error);
@@ -404,14 +470,14 @@ int GMT_grd2cpt (void *V_API, int mode, void *args) {
 	/*---------------------------- This is the grd2cpt main code ----------------------------*/
 
 	if (Ctrl->C.active) {
-		if ((l = strstr (Ctrl->C.file, ".cpt")) != NULL) *l = 0;	/* Strip off .cpt if used */
+		if (Ctrl->C.file[0] != '@' && (l = strstr (Ctrl->C.file, ".cpt")) != NULL) *l = 0;	/* Strip off .cpt if used */
 	}
-	else {	/* No table specified; set default rainbow table */
+	else {	/* No table specified; set GMT_DEFAULT_CPT_NAME table */
 		Ctrl->C.active = true;
-		Ctrl->C.file = strdup ("rainbow");
+		Ctrl->C.file = strdup (GMT->init.cpt[0]);
 	}
 
-	if (!Ctrl->E.active) Ctrl->E.levels = (Ctrl->S.n_levels > 0) ? Ctrl->S.n_levels : GRD2CPT_N_LEVELS;	/* Default number of levels */
+	if (!Ctrl->E.active) Ctrl->E.levels = (Ctrl->T.n_levels > 0) ? Ctrl->T.n_levels : GRD2CPT_N_LEVELS;	/* Default number of levels */
 	if (Ctrl->M.active) cpt_flags |= GMT_CPT_NO_BNF;		/* bit 0 controls if BFN is determined by parameters */
 	if (Ctrl->D.mode == 2) cpt_flags |= GMT_CPT_EXTEND_BNF;		/* bit 1 controls if BF will be set to equal bottom/top rgb value */
 	if (Ctrl->Z.active) cpt_flags |= GMT_CPT_CONTINUOUS;	/* Controls if final CPT should be continuous in case input is a list of colors */
@@ -430,7 +496,9 @@ int GMT_grd2cpt (void *V_API, int mode, void *args) {
 	}
 	if (Ctrl->W.wrap) Pin->is_wrapping = true;	/* A cyclic CPT has been requested */
 
-	GMT_Report (API, GMT_MSG_VERBOSE, "Processing input grid(s)\n");
+	write = (GMT->current.setting.run_mode == GMT_CLASSIC || Ctrl->H.active);	/* Only output to stdout in classic mode and with -H in modern mode */
+
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Processing input grid(s)\n");
 
 	gmt_M_memset (wesn, 4, double);
 	if (GMT->common.R.active[RSET]) gmt_M_memcpy (wesn, GMT->common.R.wesn, 4, double);	/* Subset */
@@ -449,7 +517,7 @@ int GMT_grd2cpt (void *V_API, int mode, void *args) {
 		}
 		grdfile[k] = strdup (opt->arg);
 		if (k && !(G[k]->header->n_columns == G[k-1]->header->n_columns && G[k]->header->n_rows == G[k-1]->header->n_rows)) {
-			GMT_Report (API, GMT_MSG_NORMAL, "Error: Grids do not have the same domain!\n");
+			GMT_Report (API, GMT_MSG_NORMAL, "Grids do not have the same domain!\n");
 			error = free_them_grids (API, G, grdfile, k);
 			gmt_M_free (GMT, G);
 			gmt_M_free (GMT, grdfile);
@@ -529,9 +597,10 @@ int GMT_grd2cpt (void *V_API, int mode, void *args) {
 		if (Ctrl->D.mode == 1) cpt_flags |= GMT_CPT_EXTEND_BNF;	/* bit 1 controls if BF will be set to equal bottom/top rgb value */
 		if (Ctrl->F.active) Pout->model = Ctrl->F.model;
 		if (Ctrl->F.cat) Pout->categorical = 1;
-		if (GMT_Write_Data (API, GMT_IS_PALETTE, GMT_IS_FILE, GMT_IS_NONE, cpt_flags, NULL, Ctrl->Out.file, Pout) != GMT_NOERROR) {
+		if (write && GMT_Write_Data (API, GMT_IS_PALETTE, GMT_IS_FILE, GMT_IS_NONE, cpt_flags, NULL, Ctrl->Out.file, Pout) != GMT_NOERROR) {
 			Return (API->error);
 		}
+		if (!write) gmt_save_current_cpt (GMT, Pout);	/* Save for use by session, if modern */
 		free_them_grids (API, G, grdfile, ngrd);
 		gmt_M_free (GMT, G);
 		gmt_M_free (GMT, grdfile);
@@ -546,37 +615,37 @@ int GMT_grd2cpt (void *V_API, int mode, void *args) {
 	if (gmt_M_is_verbose (GMT, GMT_MSG_VERBOSE)) {
 		sprintf (format, "Mean and S.D. of data are %s %s\n",
 		         GMT->current.setting.format_float_out, GMT->current.setting.format_float_out);
-		GMT_Report (API, GMT_MSG_VERBOSE, format, mean, sd);
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, format, mean, sd);
 	}
 
 	/* Decide how to make steps in z.  */
-	if (Ctrl->S.active && Ctrl->S.mode == 0) {	/* Use predefined levels and interval */
+	if (Ctrl->T.active && Ctrl->T.mode == 0) {	/* Use predefined levels and interval */
 		unsigned int i, j;
 
-		Ctrl->E.levels = (G[0]->header->z_min < Ctrl->S.low) ? 1 : 0;
-		Ctrl->E.levels += urint (floor((Ctrl->S.high - Ctrl->S.low)/Ctrl->S.inc)) + 1;
-		if (G[0]->header->z_max > Ctrl->S.high) Ctrl->E.levels++;
+		Ctrl->E.levels = (G[0]->header->z_min < Ctrl->T.low) ? 1 : 0;
+		Ctrl->E.levels += urint (floor((Ctrl->T.high - Ctrl->T.low)/Ctrl->T.inc)) + 1;
+		if (G[0]->header->z_max > Ctrl->T.high) Ctrl->E.levels++;
 		cdf_cpt = gmt_M_memory (GMT, NULL, Ctrl->E.levels, struct CDF_CPT);
-		if (G[0]->header->z_min < Ctrl->S.low) {
+		if (G[0]->header->z_min < Ctrl->T.low) {
 			cdf_cpt[0].z = G[0]->header->z_min;
-			cdf_cpt[1].z = Ctrl->S.low;
+			cdf_cpt[1].z = Ctrl->T.low;
 			i = 2;
 		}
 		else {
-			cdf_cpt[0].z = Ctrl->S.low;
+			cdf_cpt[0].z = Ctrl->T.low;
 			i = 1;
 		}
-		j = (G[0]->header->z_max > Ctrl->S.high) ? Ctrl->E.levels - 1 : Ctrl->E.levels;
+		j = (G[0]->header->z_max > Ctrl->T.high) ? Ctrl->E.levels - 1 : Ctrl->E.levels;
 		while (i < j) {
-			cdf_cpt[i].z = cdf_cpt[i-1].z + Ctrl->S.inc;
+			cdf_cpt[i].z = cdf_cpt[i-1].z + Ctrl->T.inc;
 			i++;
 		}
 		if (j == Ctrl->E.levels-1) cdf_cpt[j].z = G[0]->header->z_max;
 	}
-	else if (Ctrl->T.active || Ctrl->E.active) {	/* Make a equaldistant color map from G[k]->header->z_min to G[k]->header->z_max */
+	else if (Ctrl->S.active || Ctrl->E.active) {	/* Make a equaldistant color map from G[k]->header->z_min to G[k]->header->z_max */
 		double start, range;
 
-		switch (Ctrl->T.kind) {
+		switch (Ctrl->S.kind) {
 			case -1:
 				start = -fabs (G[0]->header->z_min);
 				break;
@@ -593,12 +662,12 @@ int GMT_grd2cpt (void *V_API, int mode, void *args) {
 				start = G[0]->header->z_min;
 				break;
 		}
-		range = (Ctrl->T.kind) ? 2.0 * fabs (start) : G[0]->header->z_max - G[0]->header->z_min;
+		range = (Ctrl->S.kind) ? 2.0 * fabs (start) : G[0]->header->z_max - G[0]->header->z_min;
 		range *= (1.0 + GMT_CONV8_LIMIT);	/* To ensure the max grid values do not exceed the CPT limit due to round-off issues */
 		start -= fabs (start) * GMT_CONV8_LIMIT;	/* To ensure the start of cpt is less than min value due to roundoff  */
-		Ctrl->S.inc = range / (double)(Ctrl->E.levels - 1);
+		Ctrl->T.inc = range / (double)(Ctrl->E.levels - 1);
 		cdf_cpt = gmt_M_memory (GMT, NULL, Ctrl->E.levels, struct CDF_CPT);
-		for (j = 0; j < Ctrl->E.levels; j++) cdf_cpt[j].z = start + j * Ctrl->S.inc;
+		for (j = 0; j < Ctrl->E.levels; j++) cdf_cpt[j].z = start + j * Ctrl->T.inc;
 	}
 
 	else {	/* This is completely ad-hoc.  It chooses z based on equidistant steps [of 0.1 unless -Sn set] for a Gaussian CDF:  */
@@ -610,7 +679,8 @@ int GMT_grd2cpt (void *V_API, int mode, void *args) {
 			mean = 0.5 * (G[0]->header->z_min + G[0]->header->z_max);
 			sd = (G[0]->header->z_max - mean) / 1.5;	/* This factor of 1.5 probably needs to change since z_inc is no longer fixed at 0.1 */
 			if (sd <= 0.0) {
-				GMT_Report (API, GMT_MSG_NORMAL, "Error: Min and Max data values are equal.\n");
+				GMT_Report (API, GMT_MSG_NORMAL, "Min and Max data values are equal.\n");
+				gmt_M_free (GMT, cdf_cpt);
 				Return (GMT_RUNTIME_ERROR);
 			}
 		}	/* End of stupid bug fix  */
@@ -659,13 +729,17 @@ int GMT_grd2cpt (void *V_API, int mode, void *args) {
 
 	if (Ctrl->A.active) gmt_cpt_transparency (GMT, Pout, Ctrl->A.value, Ctrl->A.mode);	/* Set transparency */
 
-	if (GMT_Write_Data (API, GMT_IS_PALETTE, GMT_IS_FILE, GMT_IS_NONE, cpt_flags, NULL, Ctrl->Out.file, Pout) != GMT_NOERROR) {
-		Return (API->error);
-	}
+	if (write && GMT_Write_Data (API, GMT_IS_PALETTE, GMT_IS_FILE, GMT_IS_NONE, cpt_flags, NULL, Ctrl->Out.file, Pout) != GMT_NOERROR)
+		error = API->error;
+
+	if (!write) gmt_save_current_cpt (GMT, Pout);	/* Save for use by session, if modern */
 
 	gmt_M_free (GMT, cdf_cpt);
 	gmt_M_free (GMT, z);
-	error = free_them_grids (API, G, grdfile, ngrd);
+	if (error == GMT_NOERROR)
+		error = free_them_grids (API, G, grdfile, ngrd);
+	else
+		free_them_grids (API, G, grdfile, ngrd);
 	gmt_M_free (GMT, G);
 	gmt_M_free (GMT, grdfile);
 
