@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *	Copyright (c) 1991-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  * Brief synopsis: grd2xyz.c reads a grid file and prints out the x,y,z values to
@@ -21,14 +20,15 @@
  *
  * Author:	Paul Wessel
  * Date:	1-JAN-2010
- * Version:	5 API
+ * Version:	6 API
  */
 
 #include "gmt_dev.h"
 
-#define THIS_MODULE_NAME	"grd2xyz"
+#define THIS_MODULE_CLASSIC_NAME	"grd2xyz"
+#define THIS_MODULE_MODERN_NAME	"grd2xyz"
 #define THIS_MODULE_LIB		"core"
-#define THIS_MODULE_PURPOSE	"Convert grid file to data table"
+#define THIS_MODULE_PURPOSE	"Convert grid to data table"
 #define THIS_MODULE_KEYS	"<G{+,>D}"
 #define THIS_MODULE_NEEDS	""
 #define THIS_MODULE_OPTIONS "-:>RVbdfhos" GMT_OPT("H")
@@ -72,11 +72,11 @@ GMT_LOCAL void Free_Ctrl (struct GMT_CTRL *GMT, struct GRD2XYZ_CTRL *C) {	/* Dea
 }
 
 GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
-	gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
+	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: grd2xyz <grid> [-C[f]] [%s] [%s]\n", GMT_Rgeo_OPT, GMT_V_OPT);
-	GMT_Message (API, GMT_TIME_NONE, "\t[-W[a|<weight>]] [-Z[<flags>]] [%s] [%s] [%s]\n\t[%s] [%s] [%s] [%s]\n\n",
-		GMT_bo_OPT, GMT_d_OPT, GMT_f_OPT, GMT_ho_OPT, GMT_o_OPT, GMT_s_OPT, GMT_colon_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "usage: %s <grid> [-C[f]] [%s] [%s]\n", name, GMT_Rgeo_OPT, GMT_V_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "\t[-W[a|<weight>]] [-Z[<flags>]] [%s] [%s] [%s]\n\t[%s] [%s] [%s] [%s] [%s]\n\n",
+		GMT_bo_OPT, GMT_d_OPT, GMT_f_OPT, GMT_ho_OPT, GMT_o_OPT, GMT_s_OPT, GMT_colon_OPT, GMT_PAR_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
@@ -146,7 +146,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRD2XYZ_CTRL *Ctrl, struct GMT
 			case 'E':	/* Old ESRI option */
 				if (gmt_M_compat_check (GMT, 4)) {
 					Ctrl->E.active = true;
-					GMT_Report (API, GMT_MSG_COMPAT, "Warning: Option -E is deprecated; use grdconvert instead.\n");
+					GMT_Report (API, GMT_MSG_COMPAT, "Option -E is deprecated; use grdconvert instead.\n");
 					if (opt->arg[0] == 'f') Ctrl->E.floating = true;
 					if (opt->arg[Ctrl->E.floating]) Ctrl->E.nodata = atof (&opt->arg[Ctrl->E.floating]);
 				}
@@ -155,7 +155,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRD2XYZ_CTRL *Ctrl, struct GMT
 				break;
 			case 'S':	/* Suppress/no-suppress NaNs on output */
 				if (gmt_M_compat_check (GMT, 4)) {
-					GMT_Report (API, GMT_MSG_COMPAT, "Warning: Option -S is deprecated; use -s instead.\n");
+					GMT_Report (API, GMT_MSG_COMPAT, "Option -S is deprecated; use -s instead.\n");
 					gmt_M_memset (GMT->current.io.io_nan_col, GMT_MAX_COLUMNS, int);
 					GMT->current.io.io_nan_col[0] = GMT_Z;	/* The default is to examine the z-column */
 					GMT->current.io.io_nan_ncols = GMT_IO_NAN_SKIP;		/* Default is that single z column */
@@ -168,7 +168,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRD2XYZ_CTRL *Ctrl, struct GMT
 				break;
 			case 'N':	/* Nan-value */
 				if (gmt_M_compat_check (GMT, 4)) {	/* Honor old -N[i]<value> option */
-					GMT_Report (API, GMT_MSG_COMPAT, "Warning: Option -N is deprecated; use GMT common option -d[i|o]<nodata> instead.\n");
+					GMT_Report (API, GMT_MSG_COMPAT, "Option -N is deprecated; use GMT common option -d[i|o]<nodata> instead.\n");
 					if (opt->arg[0]) {
 						if (opt->arg[0] == 'i')	/* Simulate -di<nodata> */
 							n_errors += gmt_parse_d_option (GMT, opt->arg);
@@ -228,6 +228,7 @@ int GMT_grd2xyz (void *V_API, int mode, void *args) {
 	double wesn[4], d_value, out[4], *x = NULL, *y = NULL;
 
 	struct GMT_GRID *G = NULL, *W = NULL;
+	struct GMT_RECORD *Out = NULL;
 	struct GMT_Z_IO io;
 	struct GMT_OPTION *opt = NULL;
 	struct GRD2XYZ_CTRL *Ctrl = NULL;
@@ -241,29 +242,28 @@ int GMT_grd2xyz (void *V_API, int mode, void *args) {
 	if (mode == GMT_MODULE_PURPOSE) return (usage (API, GMT_MODULE_PURPOSE));	/* Return the purpose of program */
 	options = GMT_Create_Options (API, mode, args);	if (API->error) return (API->error);	/* Set or get option list */
 
-	if (!options || options->option == GMT_OPT_USAGE) bailout (usage (API, GMT_USAGE));	/* Return the usage message */
-	if (options->option == GMT_OPT_SYNOPSIS) bailout (usage (API, GMT_SYNOPSIS));	/* Return the synopsis */
+	if ((error = gmt_report_usage (API, options, 0, usage)) != GMT_NOERROR) bailout (error);	/* Give usage if requested */
 
 	/* Parse the command-line arguments */
 
-	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
 	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
 	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
 	if ((error = parse (GMT, Ctrl, &io, options)) != 0) Return (error);
 	
 	/*---------------------------- This is the grd2xyz main code ----------------------------*/
 
-	GMT_Report (API, GMT_MSG_VERBOSE, "Processing input grid(s)\n");
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Processing input grid(s)\n");
 	
 	gmt_M_memcpy (wesn, GMT->common.R.wesn, 4, double);	/* Current -R setting, if any */
 
 	if (GMT->common.b.active[GMT_OUT]) {
 		if (Ctrl->Z.active && !io.binary) {
-			GMT_Report (API, GMT_MSG_NORMAL, "Warning: -Z overrides -bo\n");
+			GMT_Report (API, GMT_MSG_NORMAL, "-Z overrides -bo\n");
 			GMT->common.b.active[GMT_OUT] = false;
 		}
 		if (Ctrl->E.active && gmt_M_compat_check (GMT, 4)) {
-			GMT_Report (API, GMT_MSG_NORMAL, "Warning: -E overrides -bo\n");
+			GMT_Report (API, GMT_MSG_NORMAL, "-E overrides -bo\n");
 			GMT->common.b.active[GMT_OUT] = false;
 		}
 	}
@@ -277,7 +277,7 @@ int GMT_grd2xyz (void *V_API, int mode, void *args) {
 		if (Ctrl->W.active) n_output++;
 		if (Ctrl->C.mode == 2) w_col = 2;
 	}
-	if ((error = gmt_set_cols (GMT, GMT_OUT, n_output)) != GMT_NOERROR) Return (error);
+	if ((error = GMT_Set_Columns (API, GMT_OUT, n_output, GMT_COL_FIX_NO_TEXT)) != GMT_NOERROR) Return (error);
 
 	if (GMT_Init_IO (API, GMT_IS_DATASET, GMT_IS_POINT, GMT_OUT, GMT_ADD_DEFAULT, 0, options) != GMT_NOERROR) {	/* Registers stdout, unless already set */
 		Return (API->error);
@@ -299,8 +299,6 @@ int GMT_grd2xyz (void *V_API, int mode, void *args) {
 			Return (API->error);
 		}
 
-		GMT_Report (API, GMT_MSG_VERBOSE, "Working on file %s\n", G->header->name);
-
 		if (gmt_M_is_subset (GMT, G->header, wesn))	/* Subset requested; make sure wesn matches header spacing */
 			gmt_M_err_fail (GMT, gmt_adjust_loose_wesn (GMT, wesn, G->header), "");
 
@@ -314,8 +312,9 @@ int GMT_grd2xyz (void *V_API, int mode, void *args) {
 
 		if (Ctrl->Z.active) {	/* Write z-values only to stdout */
 			bool previous = GMT->common.b.active[GMT_OUT], rst = false;
-			int (*save) (struct GMT_CTRL *, FILE *, uint64_t, double *);
+			int (*save) (struct GMT_CTRL *, FILE *, uint64_t, double *, char *);
 			save = GMT->current.io.output;
+			Out = gmt_new_record (GMT, &d_value, NULL);	/* Since we only need to worry about numerics in this module */
 			
 			GMT->current.io.output = gmt_z_output;		/* Override and use chosen output mode */
 			GMT->common.b.active[GMT_OUT] = io.binary;	/* May have to set binary as well */
@@ -330,9 +329,10 @@ int GMT_grd2xyz (void *V_API, int mode, void *args) {
 					d_value = GMT->common.d.nan_proxy[GMT_OUT];
 				else if (gmt_input_is_nan_proxy (GMT, d_value))	/* The inverse: Grid node is nan-proxy and -di was set, so change to NaN */
 					d_value = GMT->session.d_NaN;
-				write_error = GMT_Put_Record (API, GMT_WRITE_DATA, &d_value);
+				write_error = GMT_Put_Record (API, GMT_WRITE_DATA, Out);
 				if (write_error == GMT_NOTSET) n_suppressed++;	/* Bad value caught by -s[r] */
 			}
+			gmt_M_free (GMT, Out);
 			GMT->current.io.output = save;			/* Reset pointer */
 			GMT->common.b.active[GMT_OUT] = previous;	/* Reset binary */
 			if (rst) GMT->current.io.io_nan_col[0] = GMT_Z;	/* Reset to what it was */
@@ -341,42 +341,44 @@ int GMT_grd2xyz (void *V_API, int mode, void *args) {
 			double slop;
 			char *record = NULL, item[GMT_BUFSIZ];
 			size_t n_alloc, len, rec_len;
+			struct GMT_RECORD *Out = NULL;
 			slop = 1.0 - (G->header->inc[GMT_X] / G->header->inc[GMT_Y]);
 			if (!gmt_M_is_zero (slop)) {
-				GMT_Report (API, GMT_MSG_NORMAL, "Error: x_inc must equal y_inc when writing to ESRI format\n");
+				GMT_Report (API, GMT_MSG_NORMAL, "x_inc must equal y_inc when writing to ESRI format\n");
 				Return (GMT_RUNTIME_ERROR);
 			}
 			n_alloc = G->header->n_columns * 8;	/* Assume we only need 8 bytes per item (but we will allocate more if needed) */
 			record = gmt_M_memory (GMT, NULL, G->header->n_columns, char);
+			Out = gmt_new_record (GMT, NULL, record);
 			
 			sprintf (record, "ncols %d\nnrows %d", G->header->n_columns, G->header->n_rows);
-			GMT_Put_Record (API, GMT_WRITE_TEXT, record);	/* Write a text record */
+			GMT_Put_Record (API, GMT_WRITE_DATA, Out);	/* Write a text record */
 			if (G->header->registration == GMT_GRID_PIXEL_REG) {	/* Pixel format */
 				sprintf (record, "xllcorner ");
 				sprintf (item, GMT->current.setting.format_float_out, G->header->wesn[XLO]);
 				strcat  (record, item);
-				GMT_Put_Record (API, GMT_WRITE_TEXT, record);	/* Write a text record */
+				GMT_Put_Record (API, GMT_WRITE_DATA, Out);	/* Write a text record */
 				sprintf (record, "yllcorner ");
 				sprintf (item, GMT->current.setting.format_float_out, G->header->wesn[YLO]);
 				strcat  (record, item);
-				GMT_Put_Record (API, GMT_WRITE_TEXT, record);	/* Write a text record */
+				GMT_Put_Record (API, GMT_WRITE_DATA, Out);	/* Write a text record */
 			}
 			else {	/* Gridline format */
 				sprintf (record, "xllcenter ");
 				sprintf (item, GMT->current.setting.format_float_out, G->header->wesn[XLO]);
 				strcat  (record, item);
-				GMT_Put_Record (API, GMT_WRITE_TEXT, record);	/* Write a text record */
+				GMT_Put_Record (API, GMT_WRITE_DATA, Out);	/* Write a text record */
 				sprintf (record, "yllcenter ");
 				sprintf (item, GMT->current.setting.format_float_out, G->header->wesn[YLO]);
 				strcat  (record, item);
-				GMT_Put_Record (API, GMT_WRITE_TEXT, record);	/* Write a text record */
+				GMT_Put_Record (API, GMT_WRITE_DATA, Out);	/* Write a text record */
 			}
 			sprintf (record, "cellsize ");
 			sprintf (item, GMT->current.setting.format_float_out, G->header->inc[GMT_X]);
 			strcat  (record, item);
-			GMT_Put_Record (API, GMT_WRITE_TEXT, record);	/* Write a text record */
+			GMT_Put_Record (API, GMT_WRITE_DATA, Out);	/* Write a text record */
 			sprintf (record, "nodata_value %ld", lrint (Ctrl->E.nodata));
-			GMT_Put_Record (API, GMT_WRITE_TEXT, record);	/* Write a text record */
+			GMT_Put_Record (API, GMT_WRITE_DATA, Out);	/* Write a text record */
 			gmt_M_row_loop (GMT, G, row) {	/* Scanlines, starting in the north (ymax) */
 				rec_len = 0;
 				gmt_M_col_loop (GMT, G, row, col, ij) {
@@ -395,9 +397,10 @@ int GMT_grd2xyz (void *V_API, int mode, void *args) {
 					rec_len += len;
 					if (col < (G->header->n_columns-1)) { strcat (record, " "); rec_len++;}
 				}
-				GMT_Put_Record (API, GMT_WRITE_TEXT, record);	/* Write a whole y line */
+				GMT_Put_Record (API, GMT_WRITE_DATA, Out);	/* Write a whole y line */
 			}
 			gmt_M_free (GMT, record);
+			gmt_M_free (GMT, Out);
 		}
 		else {	/* Regular x,y,z[,w], col,row,z[,w] or index,z[,w] output */
 			if (first && GMT_Init_IO (API, GMT_IS_DATASET, GMT_IS_POINT, GMT_OUT, GMT_ADD_STDIO_IF_NONE, 0, options) != GMT_NOERROR) {	/* Establishes data output */
@@ -413,6 +416,7 @@ int GMT_grd2xyz (void *V_API, int mode, void *args) {
 
 			x = gmt_grd_coord (GMT, G->header, GMT_X);
 			y = gmt_grd_coord (GMT, G->header, GMT_Y);
+			Out = gmt_new_record (GMT, out, NULL);	/* Since we only need to worry about numerics in this module */
 			if (Ctrl->C.active) {	/* Replace x,y with col,row */
 				if (Ctrl->C.mode < 2) {
 					gmt_M_row_loop  (GMT, G, row) y[row] = row + Ctrl->C.mode;
@@ -432,9 +436,9 @@ int GMT_grd2xyz (void *V_API, int mode, void *args) {
 				}
 				if (!G->header->z_units[0]) strcpy (G->header->z_units, "z");
 				if (GMT->current.setting.io_lonlat_toggle[GMT_IN])
-					sprintf (header, "# %s%s%s%s%s", G->header->y_units, GMT->current.setting.io_col_separator, G->header->x_units, GMT->current.setting.io_col_separator, G->header->z_units);
+					sprintf (header, "%s%s%s%s%s", G->header->y_units, GMT->current.setting.io_col_separator, G->header->x_units, GMT->current.setting.io_col_separator, G->header->z_units);
 				else
-					sprintf (header, "# %s%s%s%s%s", G->header->x_units, GMT->current.setting.io_col_separator, G->header->y_units, GMT->current.setting.io_col_separator, G->header->z_units);
+					sprintf (header, "%s%s%s%s%s", G->header->x_units, GMT->current.setting.io_col_separator, G->header->y_units, GMT->current.setting.io_col_separator, G->header->z_units);
 				if (Ctrl->W.active) {
 					strcat (header, GMT->current.setting.io_col_separator);
 					strcat (header, "weight");
@@ -460,11 +464,12 @@ int GMT_grd2xyz (void *V_API, int mode, void *args) {
 						out[GMT_Z] = GMT->session.d_NaN;
 				}
 				if (Ctrl->W.area) out[w_col] = W->data[ij];
-				write_error = GMT_Put_Record (API, GMT_WRITE_DATA, out);		/* Write this to output */
+				write_error = GMT_Put_Record (API, GMT_WRITE_DATA, Out);		/* Write this to output */
 				if (write_error == GMT_NOTSET) n_suppressed++;	/* Bad value caught by -s[r] */
 			}
 			gmt_M_free (GMT, x);
 			gmt_M_free (GMT, y);
+			gmt_M_free (GMT, Out);
 			if (W) gmt_free_grid (GMT, &W, true);
 		}
 	}
@@ -473,7 +478,7 @@ int GMT_grd2xyz (void *V_API, int mode, void *args) {
 		Return (API->error);
 	}
 
-	GMT_Report (API, GMT_MSG_VERBOSE, "%" PRIu64 " values extracted\n", n_total - n_suppressed);
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "%" PRIu64 " values extracted\n", n_total - n_suppressed);
 	if (n_suppressed) {
 		if (GMT->current.setting.io_nan_mode == GMT_IO_NAN_KEEP)
 			GMT_Report (API, GMT_MSG_VERBOSE, "%" PRIu64 " finite values suppressed\n", n_suppressed);
