@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *	Copyright (c) 1991-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  *  Brief synopsis: grdfft.c is a program to do various operations on
@@ -21,17 +20,19 @@
  *
  * Author:	W.H.F. Smith
  * Date:	1-JAN-2010
- * Version:	5 API
+ * Version:	6 API
  * Note:	PW: As of 2/14/2013 the various setup and init functions for FFT use
  *		have been generalized and made available GMT-wide via new functions
  *		in gmt_fft.c, called GMT_fft_*.
  */
 
 #include "gmt_dev.h"
+EXTERN_MSC unsigned int gmtlib_count_char (struct GMT_CTRL *GMT, char *txt, char it);
 
-#define THIS_MODULE_NAME	"grdfft"
+#define THIS_MODULE_CLASSIC_NAME	"grdfft"
+#define THIS_MODULE_MODERN_NAME	"grdfft"
 #define THIS_MODULE_LIB		"core"
-#define THIS_MODULE_PURPOSE	"Mathematical operations on grids in the wavenumber (or frequency) domain"
+#define THIS_MODULE_PURPOSE	"Mathematical operations on grids in the spectral domain"
 #define THIS_MODULE_KEYS	"<G{+,GG},GDE"
 #define THIS_MODULE_NEEDS	""
 #define THIS_MODULE_OPTIONS "-Vfh" GMT_OPT("T")
@@ -156,7 +157,7 @@ GMT_LOCAL void Free_Ctrl (struct GMT_CTRL *GMT, struct GRDFFT_CTRL *C) {	/* Deal
 GMT_LOCAL unsigned int do_differentiate (struct GMT_GRID *Grid, double *par, struct GMT_FFT_WAVENUMBER *K) {
 	uint64_t k;
 	double scale, fact;
-	float *datac = Grid->data;	/* Shorthand */
+	gmt_grdfloat *datac = Grid->data;	/* Shorthand */
 
 	/* Differentiate in frequency domain by multiplying by kr [scale optional] */
 
@@ -164,8 +165,8 @@ GMT_LOCAL unsigned int do_differentiate (struct GMT_GRID *Grid, double *par, str
 	datac[0] = datac[1] = 0.0f;	/* Derivative of the mean is zero */
 	for (k = 2; k < Grid->header->size; k += 2) {
 		fact = scale * gmt_fft_get_wave (k, K);
-		datac[k]   *= (float)fact;
-		datac[k+1] *= (float)fact;
+		datac[k]   *= (gmt_grdfloat)fact;
+		datac[k+1] *= (gmt_grdfloat)fact;
 	}
 	return (1);	/* Number of parameters used */
 }
@@ -174,26 +175,26 @@ GMT_LOCAL unsigned int do_integrate (struct GMT_GRID *Grid, double *par, struct
 	/* Integrate in frequency domain by dividing by kr [scale optional] */
 	uint64_t k;
 	double fact, scale;
-	float *datac = Grid->data;	/* Shorthand */
+	gmt_grdfloat *datac = Grid->data;	/* Shorthand */
 
 	scale = (*par != 0.0) ? *par : 1.0;
 	datac[0] = datac[1] = 0.0f;
 	for (k = 2; k < Grid->header->size; k += 2) {
 		fact = 1.0 / (scale * gmt_fft_get_wave (k, K));
-		datac[k]   *= (float)fact;
-		datac[k+1] *= (float)fact;
+		datac[k]   *= (gmt_grdfloat)fact;
+		datac[k+1] *= (gmt_grdfloat)fact;
 	}
 	return (1);	/* Number of parameters used */
 }
 
 GMT_LOCAL unsigned int do_continuation (struct GMT_GRID *Grid, double *zlevel, struct GMT_FFT_WAVENUMBER *K) {
 	uint64_t k;
-	float tmp, *datac = Grid->data;	/* Shorthand */
+	gmt_grdfloat tmp, *datac = Grid->data;	/* Shorthand */
 
 	/* If z is positive, the field will be upward continued using exp[- k z].  */
 
 	for (k = 2; k < Grid->header->size; k += 2) {
-		tmp = (float)exp (-(*zlevel) * gmt_fft_get_wave (k, K));
+		tmp = (gmt_grdfloat)exp (-(*zlevel) * gmt_fft_get_wave (k, K));
 		datac[k]   *= tmp;
 		datac[k+1] *= tmp;
 	}
@@ -202,14 +203,14 @@ GMT_LOCAL unsigned int do_continuation (struct GMT_GRID *Grid, double *zlevel, s
 
 GMT_LOCAL unsigned int do_azimuthal_derivative (struct GMT_GRID *Grid, double *azim, struct GMT_FFT_WAVENUMBER *K) {
 	uint64_t k;
-	float tempr, tempi, fact, *datac = Grid->data;	/* Shorthand */
+	gmt_grdfloat tempr, tempi, fact, *datac = Grid->data;	/* Shorthand */
 	double cos_azim, sin_azim;
 
 	sincosd (*azim, &sin_azim, &cos_azim);
 
 	datac[0] = datac[1] = 0.0f;
 	for (k = 2; k < Grid->header->size; k += 2) {
-		fact = (float)(sin_azim * gmt_fft_any_wave (k, GMT_FFT_K_IS_KX, K) + cos_azim * gmt_fft_any_wave (k, GMT_FFT_K_IS_KY, K));
+		fact = (gmt_grdfloat)(sin_azim * gmt_fft_any_wave (k, GMT_FFT_K_IS_KX, K) + cos_azim * gmt_fft_any_wave (k, GMT_FFT_K_IS_KY, K));
 		tempr = -(datac[k+1] * fact);
 		tempi =  (datac[k]   * fact);
 		datac[k]   = tempr;
@@ -238,7 +239,7 @@ GMT_LOCAL unsigned int do_isostasy (struct GMT_GRID *Grid, struct GRDFFT_CTRL *C
 	double rw;	/* Water density, SI units  */
 	double ri;	/* Infill density, SI units  */
 
-	float *datac = Grid->data;	/* Shorthand */
+	gmt_grdfloat *datac = Grid->data;	/* Shorthand */
 
 	te = par[0];	rl = par[1];	rm = par[2];	rw = par[3];	ri = par[4];
 	airy_ratio = -(rl - rw)/(rm - ri);
@@ -256,8 +257,8 @@ GMT_LOCAL unsigned int do_isostasy (struct GMT_GRID *Grid, struct GRDFFT_CTRL *C
 		k2 = mk * mk;
 		k4 = k2 * k2;
 		transfer_fn = airy_ratio / ((d_over_restoring_force * k4) + 1.0);
-		datac[k]   *= (float)transfer_fn;
-		datac[k+1] *= (float)transfer_fn;
+		datac[k]   *= (gmt_grdfloat)transfer_fn;
+		datac[k+1] *= (gmt_grdfloat)transfer_fn;
 	}
 	return (5);	/* Number of parameters used */
 }
@@ -298,10 +299,10 @@ GMT_LOCAL double get_filter_weight (uint64_t k, struct F_INFO *f_info, struct GM
 
 GMT_LOCAL void do_filter (struct GMT_GRID *Grid, struct F_INFO *f_info, struct GMT_FFT_WAVENUMBER *K) {
 	uint64_t k;
-	float weight, *datac = Grid->data;	/* Shorthand */
+	gmt_grdfloat weight, *datac = Grid->data;	/* Shorthand */
 
 	for (k = 0; k < Grid->header->size; k += 2) {
-		weight = (float) get_filter_weight (k, f_info, K);
+		weight = (gmt_grdfloat) get_filter_weight (k, f_info, K);
 		datac[k]   *= weight;
 		datac[k+1] *= weight;
 	}
@@ -317,7 +318,7 @@ GMT_LOCAL int do_spectrum (struct GMT_CTRL *GMT, struct GMT_GRID *GridX, struct
 	uint64_t dim[GMT_DIM_SIZE] = {1, 1, 0, 0};	/* One table and one segment, with either 1 + 1*2 = 3 or 1 + 8*2 = 17 columns and yet unknown rows */
 	uint64_t k, nk, ifreq, *nused = NULL;
 	unsigned int col;
-	float *X = GridX->data, *Y = NULL;	/* Short-hands */
+	gmt_grdfloat *X = GridX->data, *Y = NULL;	/* Short-hands */
 	double delta_k, r_delta_k, freq, coh_k, sq_norm, powfactor, k_pow_factor, tmp, eps_pow;
 	double *X_pow = NULL, *Y_pow = NULL, *co_spec = NULL, *quad_spec = NULL;
 	struct GMT_DATASET *D = NULL;
@@ -388,6 +389,7 @@ GMT_LOCAL int do_spectrum (struct GMT_CTRL *GMT, struct GMT_GRID *GridX, struct
 		return (GMT->parent->error);
 	}
 	S = D->table[0]->segment[0];	/* Only one table with one segment here, with 17 cols and nk rows */
+	S->n_rows = nk;
 	if (give_wavelength && km) delta_k *= 1000.0;	/* Wanted distances measured in km */
 
 	k_pow_factor = powfactor;	/* Standard normalization */
@@ -448,11 +450,11 @@ GMT_LOCAL int do_spectrum (struct GMT_CTRL *GMT, struct GMT_GRID *GridX, struct
 	if (GMT->common.h.add_colnames) {
 		char header[GMT_BUFSIZ] = {""}, *name[2] = {"freq", "wlength"};
 		if (GridY) {	/* Long header record - number in [] is GMT column; useful for -i option */
-			sprintf (header, "#%s[0]\txpow[1]\tstd_xpow[2]\typow[3]\tstd_ypow[4]\tcpow[5]\tstd_cpow[6]\tnpow[7]\tstd_npow[8]\t" \
+			sprintf (header, "%s[0]\txpow[1]\tstd_xpow[2]\typow[3]\tstd_ypow[4]\tcpow[5]\tstd_cpow[6]\tnpow[7]\tstd_npow[8]\t" \
 			"phase[9]\tstd_phase[10]\tadm[11]\tstd_ad[12]\tgain[13]\tstd_gain[14]\tcoh[15]\tstd_coh[16]", name[give_wavelength]);
 		}
 		else
-			sprintf (header, "#%s[0]\tpow[1]\tstd_pow[2]", name[give_wavelength]);
+			sprintf (header, "%s[0]\tpow[1]\tstd_pow[2]", name[give_wavelength]);
 		if (GMT_Set_Comment (GMT->parent, GMT_IS_DATASET, GMT_COMMENT_IS_COLNAMES, header, D)) return (GMT->parent->error);
 	}
 
@@ -470,12 +472,6 @@ GMT_LOCAL int do_spectrum (struct GMT_CTRL *GMT, struct GMT_GRID *GridX, struct
 	return (1);	/* Number of parameters used */
 }
 
-GMT_LOCAL unsigned int count_slashes (char *txt) {
-	unsigned int i, n;
-	for (i = n = 0; txt[i]; i++) if (txt[i] == '/') n++;
-	return (n);
-}
-
 GMT_LOCAL bool parse_f_string (struct GMT_CTRL *GMT, struct F_INFO *f_info, char *c) {
 	unsigned int i, j, n_tokens, pos;
 	bool descending;
@@ -569,12 +565,12 @@ GMT_LOCAL bool parse_f_string (struct GMT_CTRL *GMT, struct F_INFO *f_info, char
 }
 
 GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
-	gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
+	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: grdfft <ingrid> [<ingrid2>] [-G<outgrid>|<table>] [-A<azimuth>] [-C<zlevel>]\n");
+	GMT_Message (API, GMT_TIME_NONE, "usage: %s <ingrid> [<ingrid2>] [-G<outgrid>|<table>] [-A<azimuth>] [-C<zlevel>]\n", name);
 	GMT_Message (API, GMT_TIME_NONE, "\t[-D[<scale>|g]] [-E[r|x|y][+w[k]][+n]] [-F[r|x|y]<parameters>] [-I[<scale>|g]]\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t[-N%s] [-S<scale>]\n", GMT_FFT_OPT);
-	GMT_Message (API, GMT_TIME_NONE, "\t[%s] [-fg] [%s]\n\n", GMT_V_OPT, GMT_ho_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "\t[%s] [-fg] [%s] [%s]\n\n", GMT_V_OPT, GMT_ho_OPT, GMT_PAR_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
@@ -729,7 +725,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRDFFT_CTRL *Ctrl, struct F_IN
 			case 'F':	/* Filter */
 				Ctrl->F.active = true;
 				if (!(f_info->set_already)) {
-					filter_type = count_slashes (opt->arg);
+					filter_type = gmtlib_count_char (GMT, opt->arg, '/');
 					f_info->kind = GRDFFT_FILTER_EXP + (filter_type - 1);
 					f_info->set_already = true;
 					add_operation (GMT, Ctrl, f_info->kind, 0, NULL);
@@ -748,14 +744,14 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRDFFT_CTRL *Ctrl, struct F_IN
 				break;
 			case 'L':	/* Leave trend alone */
 				if (gmt_M_compat_check (GMT, 4))
-					GMT_Report (API, GMT_MSG_COMPAT, "Warning: Option -L is deprecated; use -N modifiers in the future.\n");
+					GMT_Report (API, GMT_MSG_COMPAT, "Option -L is deprecated; use -N modifiers in the future.\n");
 				else
 					n_errors += gmt_default_error (GMT, opt->option);
 				break;
 			case 'M':	/* Geographic data */
 				if (gmt_M_compat_check (GMT, 4)) {
-					GMT_Report (API, GMT_MSG_COMPAT, "Warning: Option -M is deprecated; -fg was set instead, use this in the future.\n");
-					if (!gmt_M_is_geographic (GMT, GMT_IN))
+					GMT_Report (API, GMT_MSG_COMPAT, "Option -M is deprecated; -fg was set instead, use this in the future.\n");
+					if (gmt_M_is_cartesian (GMT, GMT_IN))
 						gmt_parse_common_options (GMT, "f", 'f', "g"); /* Set -fg unless already set */
 				}
 				else
@@ -778,7 +774,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRDFFT_CTRL *Ctrl, struct F_IN
 			case 'T':	/* Flexural isostasy */
 				if (gmt_M_compat_check (GMT, 4)) {
 					GMT_Report (API, GMT_MSG_COMPAT,
-					            "Warning: Option -T is deprecated; see gravfft for isostasy and gravity calculations.\n");
+					            "Option -T is deprecated; see gravfft for isostasy and gravity calculations.\n");
 					Ctrl->T.active = true;
 					n_scan = sscanf (opt->arg, "%lf/%lf/%lf/%lf/%lf", &par[0], &par[1], &par[2], &par[3], &par[4]);
 					for (j = 1, k = 0; j < 5; j++) if (par[j] < 0.0) k++;
@@ -838,19 +834,18 @@ int GMT_grdfft (void *V_API, int mode, void *args) {
 	if (mode == GMT_MODULE_PURPOSE) return (usage (API, GMT_MODULE_PURPOSE));	/* Return the purpose of program */
 	options = GMT_Create_Options (API, mode, args);	if (API->error) return (API->error);	/* Set or get option list */
 
-	if (!options || options->option == GMT_OPT_USAGE) bailout (usage (API, GMT_USAGE));	/* Return the usage message */
-	if (options->option == GMT_OPT_SYNOPSIS) bailout (usage (API, GMT_SYNOPSIS));	/* Return the synopsis */
+	if ((error = gmt_report_usage (API, options, 0, usage)) != GMT_NOERROR) bailout (error);	/* Give usage if requested */
 
 	/* Parse the command-line arguments */
 
-	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
 	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
 	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
 	if ((error = parse (GMT, Ctrl, &f_info, options)) != 0) Return (error);
 
 	/*---------------------------- This is the grdfft main code ----------------------------*/
 
-	GMT_Report (API, GMT_MSG_VERBOSE, "Processing input grid(s)\n");
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Processing input grid(s)\n");
 	for (k = 0; k < Ctrl->In.n_grids; k++) {	/* First read the grid header(s) */
 		if ((Orig[k] = GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_CONTAINER_ONLY, NULL, Ctrl->In.file[k], NULL)) == NULL)
 			Return (API->error);
@@ -899,7 +894,7 @@ int GMT_grdfft (void *V_API, int mode, void *args) {
 #endif
 
 	for (k = 0; k < Ctrl->In.n_grids; k++) {	/* Call the forward FFT, once per grid, optionally save raw FFT output */
-		GMT_Report (API, GMT_MSG_VERBOSE, "forward FFT...\n");
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "forward FFT...\n");
 		if (GMT_FFT (API, Grid[k], GMT_FFT_FWD, GMT_FFT_COMPLEX, FFT_info[k]))
 			Return (GMT_RUNTIME_ERROR);
 	}
