@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *	Copyright (c) 1991-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  * Brief synopsis: grdgradient read a grid file and compute gradient in azim direction:
@@ -27,12 +26,13 @@
  *
  * Author:	W.H.F. Smith
  * Date: 	1-JAN-2010
- * Version:	5 API
+ * Version:	6 API
  */
  
 #include "gmt_dev.h"
 
-#define THIS_MODULE_NAME	"grdgradient"
+#define THIS_MODULE_CLASSIC_NAME	"grdgradient"
+#define THIS_MODULE_MODERN_NAME	"grdgradient"
 #define THIS_MODULE_LIB		"core"
 #define THIS_MODULE_PURPOSE	"Compute directional gradients from a grid"
 #define THIS_MODULE_KEYS	"<G{,AG(,GG},SG)"
@@ -59,7 +59,7 @@ struct GRDGRADIENT_CTRL {
 		bool active;
 		unsigned int mode;
 	} D;
-	struct E {	/* -E[s|p]<azim>/<elev[ambient/diffuse/specular/shine]> */
+	struct E {	/* -E[s|p]<azim>/<elev[+a<ambient>][+d<diffuse>][+p<specular>][+<shine>] */
 		bool active;
 		unsigned int mode;
 		double azimuth, elevation;
@@ -69,11 +69,17 @@ struct GRDGRADIENT_CTRL {
 		bool active;
 		char *file;
 	} G;
-	struct N {	/* -N[t_or_e][<amp>[/<sigma>[/<offset>]]] */
+	struct N {	/* -N[t_or_e][<amp>][+o<offset>][+s<sigma>] */
 		bool active;
+		unsigned int set[3];	/* 1 if values are specified for amp, offset and sigma, 2 means we want last-run values */
 		unsigned int mode;	/* 1 = atan, 2 = exp */
 		double norm, sigma, offset;
 	} N;
+	struct Q {	/* -Qc|r|R */
+		/* Note: If -Qc is set with -N then -G is not required. GMT_Encode_Options turns off the primary output */
+		bool active;
+		unsigned int mode;
+	} Q;
 	struct S {	/* -S<slopefile> */
 		bool active;
 		char *file;
@@ -122,11 +128,11 @@ GMT_LOCAL double specular (double n_columns, double n_rows, double nz, double *s
 }
 
 GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
-	gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
+	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: grdgradient <ingrid> -G<outgrid> [-A<azim>[/<azim2>]] [-D[a][c][o][n]]\n");
+	GMT_Message (API, GMT_TIME_NONE, "usage: %s <ingrid> -G<outgrid> [-A<azim>[/<azim2>]] [-D[a][c][o][n]]\n", name);
 	GMT_Message (API, GMT_TIME_NONE, "\t[-E[s|p|m]<azim>/<elev>[+a<ambient>][+d<diffuse>][+p<specular>][+s<shine>]]\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t[-N[t|e][<amp>][+s<sigma>][+o<offset>]] [%s]\n\t[-S<slopegrid>] [%s] [-fg] [%s]\n\n", GMT_Rgeo_OPT, GMT_V_OPT, GMT_n_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "\t[-N[t|e][<amp>][+s<sigma>][+o<offset>]] [-Qc|r|R] [%s]\n\t[-S<slopegrid>] [%s] [-fg] [%s] [%s]\n\n", GMT_Rgeo_OPT, GMT_V_OPT, GMT_n_OPT, GMT_PAR_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
@@ -156,7 +162,13 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t  -Nt will make atan transform, then scale to <amp> [1.0].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t  -Ne will make exp  transform, then scale to <amp> [1.0].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t    For -Nt|e, optionally append +s<sigma> and/or +o<offset> to set\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t    sigma and offset for transform [Default estimates from the data].\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t    sigma and offset for the transform [Default estimates from the data].\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t    See -Q to use the same offset, sigma for multiple grid calculations.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t-Q Control handling of -N arguments from previous calculations. Append code:\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t     c: Create stat file and write the offset and sigma of this run.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t     r: Read offset and sigma of the previous run from stat file.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t     R: Remove & read.  As r but also removes the stat file after use.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   The values obtained are used if +o and/or +s modifiers in -N are given no argument.\n");
 	GMT_Option (API, "R");
 	GMT_Message (API, GMT_TIME_NONE, "\t-S Output file for |grad z|; requires -D.\n");
 	GMT_Option (API, "V");
@@ -175,7 +187,6 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRDGRADIENT_CTRL *Ctrl, struct
 	 */
 
 	unsigned int n_errors = 0, n_files = 0, j, entry, pos;
-	int n_opt_args = 0;
 	char p[GMT_BUFSIZ] = {""}, *c = NULL;
 	struct GMT_OPTION *opt = NULL;
 	struct GMTAPI_CTRL *API = GMT->parent;
@@ -193,13 +204,13 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRDGRADIENT_CTRL *Ctrl, struct
 
 			/* Processes program-specific parameters */
 
-			case 'A':	/* Set azimuth */
+			case 'A':	/* Set azimuth(s) */
 				Ctrl->A.active = true;
-				if (!gmt_access (GMT, opt->arg, F_OK)) {	/* file exists */
+				if (!gmt_access (GMT, opt->arg, F_OK)) {	/* file with variable azimuths exists */
 					Ctrl->A.file = strdup (opt->arg);
 					Ctrl->A.mode = GRDGRADIENT_VAR;
 				}
-				else {
+				else {	/* Get one or two fixed azimuths */
 					j = sscanf (opt->arg, "%lf/%lf", &Ctrl->A.azimuth[0], &Ctrl->A.azimuth[1]);
 					Ctrl->A.two = (j == 2);
 					Ctrl->A.mode = GRDGRADIENT_FIX;
@@ -234,7 +245,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRDGRADIENT_CTRL *Ctrl, struct
 						break;
 					case 'm':	/* Nice algorithm from an old program called manipRaster by Tierry Souriot */
 						Ctrl->E.mode = 4;
-						j = sscanf(&opt->arg[1], "%lf/%lf", &Ctrl->E.azimuth, &Ctrl->E.elevation);
+						j = sscanf (&opt->arg[1], "%lf/%lf", &Ctrl->E.azimuth, &Ctrl->E.elevation);
 						if (j == 0) {				/* Use default values */
 							Ctrl->E.azimuth = 360 - 45;
 							Ctrl->E.elevation = 45;
@@ -278,11 +289,11 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRDGRADIENT_CTRL *Ctrl, struct
 								}
 								entry++;
 							}
+							GMT_Report (API, GMT_MSG_DEBUG, "Settings are %g/%g+a%g+d%g+p%g+s%g\n",
+								Ctrl->E.azimuth, Ctrl->E.elevation, Ctrl->E.ambient, Ctrl->E.diffuse, Ctrl->E.specular, Ctrl->E.shine);
 						}
 						break;
 				}
-				GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Settings are %g/%g+a%g+d%g+p%g+s%g\n",
-					Ctrl->E.azimuth, Ctrl->E.elevation, Ctrl->E.ambient, Ctrl->E.diffuse, Ctrl->E.specular, Ctrl->E.shine);
 				break;
 			case 'G':	/* Output grid */
 				if ((Ctrl->G.active = gmt_check_filearg (GMT, 'G', opt->arg, GMT_OUT, GMT_IS_GRID)))
@@ -292,7 +303,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRDGRADIENT_CTRL *Ctrl, struct
 				break;
 			case 'L':	/* GMT4 BCs */
 				if (gmt_M_compat_check (GMT, 4)) {
-					GMT_Report (API, GMT_MSG_COMPAT, "Warning: Option -L is deprecated; -n+b%s was set instead, use this in the future.\n", opt->arg);
+					GMT_Report (API, GMT_MSG_COMPAT, "Option -L is deprecated; -n+b%s was set instead, use this common GMT option in the future.\n", opt->arg);
 					gmt_strncpy (GMT->common.n.BC, opt->arg, 4U);
 					/* We turn on geographic coordinates if -Lg is given by faking -fg */
 					/* But since GMT_parse_f_option is private to gmt_init and all it does */
@@ -308,15 +319,15 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRDGRADIENT_CTRL *Ctrl, struct
 
 			case 'M':	/* Geographic data */
 				if (gmt_M_compat_check (GMT, 4)) {
-					GMT_Report (API, GMT_MSG_COMPAT, "Warning: Option -M is deprecated; -fg was set instead, use this in the future.\n");
-					if (!gmt_M_is_geographic (GMT, GMT_IN)) gmt_parse_common_options (GMT, "f", 'f', "g"); /* Set -fg unless already set */
+					GMT_Report (API, GMT_MSG_COMPAT, "Option -M is deprecated; -fg was set instead, use this common GMT option in the future.\n");
+					if (gmt_M_is_cartesian (GMT, GMT_IN)) gmt_parse_common_options (GMT, "f", 'f', "g"); /* Set -fg unless already set */
 				}
 				else
 					n_errors += gmt_default_error (GMT, opt->option);
 				break;
 			case 'N':	/* Normalization */
 				Ctrl->N.active = true;	j = 0;
-				if (opt->arg[0] && strchr ("et", opt->arg[0])) {
+				if (opt->arg[0] && strchr ("et", opt->arg[0])) {	/* Got -Ne or -Nt, possibly with arguments */
 					Ctrl->N.mode = (opt->arg[0] == 't') ? 1 : 2;
 					j = 1;
 				}
@@ -324,18 +335,36 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRDGRADIENT_CTRL *Ctrl, struct
 					pos = 0;	/* Reset to start of new word */
 					while (gmt_getmodopt (GMT, 'N', c, "os", &pos, p, &n_errors) && n_errors == 0) {
 						switch (p[0]) {
-							case 'o': Ctrl->N.offset = atof (&p[1]); break;
-							case 's': Ctrl->N.sigma  = atof (&p[1]); break;
+							case 'o': Ctrl->N.set[1] = 1; if (p[1]) Ctrl->N.offset = atof (&p[1]); else Ctrl->N.set[1] = 2; break;
+							case 's': Ctrl->N.set[2] = 1; if (p[1]) Ctrl->N.sigma  = atof (&p[1]); else Ctrl->N.set[2] = 2; break;
 							default: break;	/* These are caught in gmt_getmodopt so break is just for Coverity */
 						}
 					}
-					c[0] = '\0';	/* Chop off all modifiers so range can be determined */
-					if (opt->arg[j]) Ctrl->N.norm = atof (&opt->arg[j]);
+					c[0] = '\0';	/* Chop off all modifiers so amplitude can be determined (if given) */
+					if (opt->arg[j]) {
+						Ctrl->N.set[0] = true;
+						Ctrl->N.norm = atof (&opt->arg[j]);
+					}
+					c[0] = '+';	/* Restore modifiers */
+				}
+				else if (opt->arg[j]) {	/* Old-style args */
+					int n_opt_args = sscanf (&opt->arg[j], "%lf/%lf/%lf", &Ctrl->N.norm, &Ctrl->N.sigma, &Ctrl->N.offset);
+					Ctrl->N.set[0] = (n_opt_args >= 1);	/* Had to set the first two to set the amplitude */
+					Ctrl->N.set[2] = (n_opt_args >= 2);	/* Had to set the first two to set the sigma */
+					Ctrl->N.set[1] = (n_opt_args == 3);	/* Had to set all three to set the offset */
+				}
+				break;
+			case 'Q':	/* Read/write normalization values */
+				Ctrl->Q.active = true;
+				switch (opt->arg[0]) {
+					case 'r': Ctrl->Q.mode = 1; break;
+					case 'c': Ctrl->Q.mode = 2; break;
+					case 'R': Ctrl->Q.mode = 3; break;
+					default:
+						GMT_Report (API, GMT_MSG_NORMAL, "Syntax error -Q option: Unrecognized directive %s\n", opt->arg);
+						n_errors++;
+						break;
 				}
-				else if (opt->arg[j])	/* Old-style args */
-					n_opt_args = sscanf (&opt->arg[j], "%lf/%lf/%lf", &Ctrl->N.norm, &Ctrl->N.sigma, &Ctrl->N.offset);
-				GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Settings are %g+o%g+s%g\n",
-					Ctrl->N.norm, Ctrl->N.offset, Ctrl->N.sigma);
 				break;
 			case 'S':	/* Slope grid */
 				if ((Ctrl->S.active = gmt_check_filearg (GMT, 'S', opt->arg, GMT_OUT, GMT_IS_GRID)))
@@ -352,13 +381,16 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRDGRADIENT_CTRL *Ctrl, struct
 
 	n_errors += gmt_M_check_condition (GMT, !(Ctrl->A.active || Ctrl->D.active || Ctrl->E.active), "Syntax error: Must specify -A, -D, or -E\n");
 	n_errors += gmt_M_check_condition (GMT, Ctrl->S.active && !Ctrl->S.file, "Syntax error -S option: Must specify output file\n");
-	n_errors += gmt_M_check_condition (GMT, !Ctrl->G.file && !Ctrl->S.active, "Syntax error -G option: Must specify output file\n");
+	n_errors += gmt_M_check_condition (GMT, !(Ctrl->N.active && Ctrl->Q.mode == 2) && !Ctrl->G.file && !Ctrl->S.active, "Syntax error -G option: Must specify output file\n");
 	n_errors += gmt_M_check_condition (GMT, !Ctrl->In.file, "Syntax error: Must specify input file\n");
-	n_errors += gmt_M_check_condition (GMT, Ctrl->N.active && (n_opt_args && Ctrl->N.norm <= 0.0), "Syntax error -N option: Normalization amplitude must be > 0\n");
-	n_errors += gmt_M_check_condition (GMT, Ctrl->N.active && (n_opt_args > 1 && Ctrl->N.sigma <= 0.0) , "Syntax error -N option: Sigma must be > 0\n");
+	n_errors += gmt_M_check_condition (GMT, Ctrl->N.active && (Ctrl->N.set[0] && Ctrl->N.norm <= 0.0), "Syntax error -N option: Normalization amplitude must be > 0\n");
+	n_errors += gmt_M_check_condition (GMT, Ctrl->N.active && (Ctrl->N.set[2] == 1 && Ctrl->N.sigma <= 0.0) , "Syntax error -N option: Sigma must be > 0\n");
 	n_errors += gmt_M_check_condition (GMT, Ctrl->E.active && Ctrl->E.mode > 1 && (Ctrl->E.elevation < 0.0 || Ctrl->E.elevation > 90.0), "Syntax error -E option: Use 0-90 degree range for elevation\n");
+	n_errors += gmt_M_check_condition (GMT, Ctrl->Q.active && !Ctrl->N.active, "Syntax error -Q option: Requires -N\n");
+	n_errors += gmt_M_check_condition (GMT, Ctrl->N.set[1] == 2 && !(Ctrl->Q.mode & 1), "Syntax error: Must specify -Q if -N+o is given no value\n");
+	n_errors += gmt_M_check_condition (GMT, Ctrl->N.set[2] == 2 && !(Ctrl->Q.mode & 1), "Syntax error: Must specify -Q if -N+s is given no value\n");
 	if (Ctrl->E.active && (Ctrl->A.active || Ctrl->D.active || Ctrl->S.active)) {
-		GMT_Report (API, GMT_MSG_NORMAL, "Warning: -E option overrides -A, -D or -S\n");
+		GMT_Report (API, GMT_MSG_VERBOSE, "-E option overrides -A, -D or -S\n");
 		Ctrl->A.active = Ctrl->D.active = Ctrl->S.active = false;
 	}
 
@@ -369,7 +401,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRDGRADIENT_CTRL *Ctrl, struct
 #define Return(code) {Free_Ctrl (GMT, Ctrl); gmt_end_module (GMT, GMT_cpy); bailout (code);}
 
 int GMT_grdgradient (void *V_API, int mode, void *args) {
-	bool sigma_set = false, offset_set = false, bad, new_grid = false, separate = false;
+	bool bad, new_grid = false, separate = false;
 	int p[4], mx, error = 0;
 	unsigned int row, col, n;
 	uint64_t ij, ij0, index, n_used = 0;
@@ -381,6 +413,7 @@ int GMT_grdgradient (void *V_API, int mode, void *args) {
 	double x_factor2 = 0.0, x_factor2_set = 0.0, y_factor2 = 0.0, dzdx2 = 0.0, dzdy2 = 0.0, dzds1, dzds2;
 	double p0 = 0.0, q0 = 0.0, p0q0_cte = 1.0, norm_z, mag, s[3], lim_x, lim_y, lim_z;
 	double k_ads = 0.0, diffuse, spec, r_min = DBL_MAX, r_max = -DBL_MAX, scale, sin_Az[2] = {0.0, 0.0};
+	double def_offset = 0.0, def_sigma = 0.0;
 	
 	struct GMT_GRID *Surf = NULL, *Slope = NULL, *Out = NULL, *A = NULL;
 	struct GRDGRADIENT_CTRL *Ctrl = NULL;
@@ -394,30 +427,62 @@ int GMT_grdgradient (void *V_API, int mode, void *args) {
 	if (mode == GMT_MODULE_PURPOSE) return (usage (API, GMT_MODULE_PURPOSE));	/* Return the purpose of program */
 	options = GMT_Create_Options (API, mode, args);	if (API->error) return (API->error);	/* Set or get option list */
 
-	if (!options || options->option == GMT_OPT_USAGE) bailout (usage (API, GMT_USAGE));/* Return the usage message */
-	if (options->option == GMT_OPT_SYNOPSIS) bailout (usage (API, GMT_SYNOPSIS));	/* Return the synopsis */
+	if ((error = gmt_report_usage (API, options, 0, usage)) != GMT_NOERROR) bailout (error);	/* Give usage if requested */
 
 	/* Parse the command-line arguments */
 
-	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
 	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
 	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
 	if ((error = parse (GMT, Ctrl, options)) != 0) Return (error);
 
 	/*---------------------------- This is the grdgradient main code ----------------------------*/
 
-	GMT_Report (API, GMT_MSG_VERBOSE, "Processing input grid\n");
+	if (Ctrl->Q.mode & 1) {	/* Read in previous statistics */
+		char sfile[PATH_MAX] = {""};
+		FILE *fp = NULL;
+		GMT_Report (API, GMT_MSG_DEBUG, "Read statistics file [%s] with offset and sigma\n", sfile);
+		if (GMT->session.TMPDIR)
+			sprintf (sfile, "%s/grdgradient.stat", GMT->session.TMPDIR);
+		else if (API->tmp_dir)
+			sprintf (sfile, "%s/grdgradient.stat", API->tmp_dir);
+		else
+			sprintf (sfile, "grdgradient.stat");
+		if (access (sfile, F_OK)) {
+			GMT_Report (API, GMT_MSG_NORMAL, "Unable to find statistics file from last run [%s]!\n", sfile);
+			Return (GMT_FILE_NOT_FOUND);
+		}
+		if ((fp = fopen (sfile, "r")) == NULL) {
+			GMT_Report (API, GMT_MSG_NORMAL, "Cannot open statistics file from last run [%s]!\n", sfile);
+			Return (GMT_ERROR_ON_FOPEN);
+		}
+		if (fscanf (fp, "%lg %lg", &def_offset, &def_sigma) != 2) {
+			GMT_Report (API, GMT_MSG_NORMAL, "Unable to read record from statistics file from last run [%s]!\n", sfile);
+			fclose (fp);
+			Return (GMT_RUNTIME_ERROR);
+		}
+		fclose (fp);
+		if (Ctrl->Q.mode == 3) {	/* Gave -FR to delete after we have read the file */
+			GMT_Report (API, GMT_MSG_DEBUG, "Remove statistics file [%s]\n", sfile);
+			if (Ctrl->Q.mode == 3 && gmt_remove_file (GMT, sfile)) {	/* Gave -FR to read and delete */
+				GMT_Report (API, GMT_MSG_NORMAL, "Cannot remove statistics file from last run [%s]!\n", sfile);
+				Return (GMT_RUNTIME_ERROR);
+			}
+		}
+	}
+	
+	if (Ctrl->N.active) {	/* Report what was set if debug is enabled */
+		char *answer = "NY";
+		if (Ctrl->N.set[1] == 2) Ctrl->N.offset = def_offset, Ctrl->N.set[1] = 1;
+		if (Ctrl->N.set[2] == 2) Ctrl->N.sigma  = def_sigma,  Ctrl->N.set[2] = 1;
+		GMT_Report (API, GMT_MSG_DEBUG, "amplitude_set = %c offset_set = %c sigma_set = %c\n", answer[Ctrl->N.set[0]], answer[Ctrl->N.set[1]], answer[Ctrl->N.set[2]]);
+	}
+
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Processing input grid\n");
 	gmt_M_memset (s, 3, double);
 	gmt_M_memset (wesn, 4, double);
 	gmt_set_pad (GMT, 2U);	/* Ensure space for BCs in case an API passed pad == 0 */
 	
-	if (Ctrl->N.active) {
-		char *answer = "NT";
-		if (Ctrl->N.sigma  != 0.0) sigma_set  = true;
-		if (Ctrl->N.offset != 0.0) offset_set = true;
-		GMT_Report (API, GMT_MSG_LONG_VERBOSE, " sigma_set = %c  offset_set = %c\n", answer[sigma_set], answer[offset_set]);
-	}
-
 	if (Ctrl->A.active) {	/* Get azimuth in 0-360 range */
 		if (Ctrl->A.mode == GRDGRADIENT_VAR) {	/* Got variable azimuth(s) */
 			if ((A = GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_CONTAINER_AND_DATA, NULL, Ctrl->A.file, NULL)) == NULL) {
@@ -602,7 +667,7 @@ int GMT_grdgradient (void *V_API, int mode, void *args) {
 					if (Ctrl->D.mode & 2 && azim >= 180) azim -= 180.0;
 				}
 				output = azim;
-				if (Ctrl->S.active) Slope->data[ij] = (float)hypot (dzdx, dzdy);
+				if (Ctrl->S.active) Slope->data[ij] = (gmt_grdfloat)hypot (dzdx, dzdy);
 			}
 			else {	/* Ctrl->E.active */
 				if (Ctrl->E.mode == 2)
@@ -624,7 +689,7 @@ int GMT_grdgradient (void *V_API, int mode, void *args) {
 				r_max = MAX (r_max, output);
 			}
 			index = (new_grid) ? ij : ij0;
-			Out->data[index] = (float)output;
+			Out->data[index] = (gmt_grdfloat)output;
 			n_used++;
 		}
 	}
@@ -641,12 +706,12 @@ int GMT_grdgradient (void *V_API, int mode, void *args) {
 		if (Out->header->wesn[YLO] == -90.0 && Out->header->registration == GMT_GRID_NODE_REG) {	/* Average all the multiple N pole estimates */
 			for (col = 0, ij = gmt_M_ijp (Out->header, 0, 0), sum = 0.0; col < Out->header->n_columns; col++, ij++) sum += Out->data[ij];
 			sum /= Out->header->n_columns;	/* Average gradient */
-			for (col = 0, ij = gmt_M_ijp (Out->header, 0, 0); col < Out->header->n_columns; col++, ij++) Out->data[ij] = (float)sum;
+			for (col = 0, ij = gmt_M_ijp (Out->header, 0, 0); col < Out->header->n_columns; col++, ij++) Out->data[ij] = (gmt_grdfloat)sum;
 		}
 		if (Out->header->wesn[YLO] == -90.0 && Out->header->registration == GMT_GRID_NODE_REG) {	/* Average all the multiple S pole estimates */
 			for (col = 0, ij = gmt_M_ijp (Out->header, Out->header->n_rows - 1, 0), sum = 0.0; col < Out->header->n_columns; col++, ij++) sum += Out->data[ij];
 			sum /= Out->header->n_columns;	/* Average gradient */
-			for (col = 0, ij = gmt_M_ijp (Out->header, Out->header->n_rows - 1, 0); col < Out->header->n_columns; col++, ij++) Out->data[ij] = (float)sum;
+			for (col = 0, ij = gmt_M_ijp (Out->header, Out->header->n_rows - 1, 0); col < Out->header->n_columns; col++, ij++) Out->data[ij] = (gmt_grdfloat)sum;
 		}
 	}
 	
@@ -654,11 +719,11 @@ int GMT_grdgradient (void *V_API, int mode, void *args) {
 		scale = 1.0 / (r_max - r_min);
 		gmt_M_grd_loop (GMT, Out, row, col, ij) {
 			if (gmt_M_is_fnan (Out->data[ij])) continue;
-			Out->data[ij] = (float)((-1.0 + 2.0 * ((Out->data[ij] - r_min) * scale)) * 0.95);
+			Out->data[ij] = (gmt_grdfloat)((-1.0 + 2.0 * ((Out->data[ij] - r_min) * scale)) * 0.95);
 		}
 	}
 
-	if (offset_set)
+	if (Ctrl->N.set[1])
 		ave_gradient = Ctrl->N.offset;
 	else
 		ave_gradient /= n_used;
@@ -667,7 +732,7 @@ int GMT_grdgradient (void *V_API, int mode, void *args) {
 
 		if (Ctrl->N.active) {	/* Chose normalization */
 			if (Ctrl->N.mode == 1) {	/* atan transformation */
-				if (sigma_set)
+				if (Ctrl->N.set[2])
 					denom = 1.0 / Ctrl->N.sigma;
 				else {
 					denom = 0.0;
@@ -679,16 +744,20 @@ int GMT_grdgradient (void *V_API, int mode, void *args) {
 				}
 				rpi = 2.0 * Ctrl->N.norm / M_PI;
 				gmt_M_grd_loop (GMT, Out, row, col, ij) {
-					if (!gmt_M_is_fnan (Out->data[ij])) Out->data[ij] = (float)(rpi * atan ((Out->data[ij] - ave_gradient) * denom));
+					if (!gmt_M_is_fnan (Out->data[ij])) Out->data[ij] = (gmt_grdfloat)(rpi * atan ((Out->data[ij] - ave_gradient) * denom));
 				}
 				Out->header->z_max = rpi * atan ((max_gradient - ave_gradient) * denom);
 				Out->header->z_min = rpi * atan ((min_gradient - ave_gradient) * denom);
 			}
 			else if (Ctrl->N.mode == 2) {	/* Exp transformation */
-				if (!sigma_set) {
+				if (!Ctrl->N.set[2]) {
 					Ctrl->N.sigma = 0.0;
 					gmt_M_grd_loop (GMT, Out, row, col, ij) {
+#ifdef DOUBLE_PRECISION_GRID
+						if (!gmt_M_is_fnan (Out->data[ij])) Ctrl->N.sigma += fabs (Out->data[ij]);
+#else
 						if (!gmt_M_is_fnan (Out->data[ij])) Ctrl->N.sigma += fabsf (Out->data[ij]);
+#endif
 					}
 					Ctrl->N.sigma = M_SQRT2 * Ctrl->N.sigma / n_used;
 				}
@@ -696,10 +765,10 @@ int GMT_grdgradient (void *V_API, int mode, void *args) {
 				gmt_M_grd_loop (GMT, Out, row, col, ij) {
 					if (gmt_M_is_fnan (Out->data[ij])) continue;
 					if (Out->data[ij] < ave_gradient) {
-						Out->data[ij] = (float)(-Ctrl->N.norm * (1.0 - exp ( (Out->data[ij] - ave_gradient) * denom)));
+						Out->data[ij] = (gmt_grdfloat)(-Ctrl->N.norm * (1.0 - exp ( (Out->data[ij] - ave_gradient) * denom)));
 					}
 					else {
-						Out->data[ij] = (float)( Ctrl->N.norm * (1.0 - exp (-(Out->data[ij] - ave_gradient) * denom)));
+						Out->data[ij] = (gmt_grdfloat)( Ctrl->N.norm * (1.0 - exp (-(Out->data[ij] - ave_gradient) * denom)));
 					}
 				}
 				Out->header->z_max =  Ctrl->N.norm * (1.0 - exp (-(max_gradient - ave_gradient) * denom));
@@ -711,7 +780,7 @@ int GMT_grdgradient (void *V_API, int mode, void *args) {
 				else
 					denom = Ctrl->N.norm / (ave_gradient - min_gradient);
 				gmt_M_grd_loop (GMT, Out, row, col, ij) {
-					if (!gmt_M_is_fnan (Out->data[ij])) Out->data[ij] = (float)((Out->data[ij] - ave_gradient) * denom);
+					if (!gmt_M_is_fnan (Out->data[ij])) Out->data[ij] = (gmt_grdfloat)((Out->data[ij] - ave_gradient) * denom);
 				}
 				Out->header->z_max = (max_gradient - ave_gradient) * denom;
 				Out->header->z_min = (min_gradient - ave_gradient) * denom;
@@ -723,37 +792,62 @@ int GMT_grdgradient (void *V_API, int mode, void *args) {
 
 	/* Now we write out: */
 
-	if (Ctrl->A.active) {
-		if (Ctrl->N.active)
-			strcpy (buffer, "Normalized directional derivative(s)");
-		else
-			strcpy (buffer, "Directional derivative(s)");
-		sprintf (format, "\t%s\t%s\t%s\t%s\n", GMT->current.setting.format_float_out, GMT->current.setting.format_float_out, GMT->current.setting.format_float_out, GMT->current.setting.format_float_out);
-		GMT_Report (API, GMT_MSG_VERBOSE, " Min Mean Max sigma intensities:");
-		GMT_Report (API, GMT_MSG_VERBOSE, format, min_gradient, ave_gradient, max_gradient, Ctrl->N.sigma);
-	}
-	else {
-		if (Ctrl->E.mode > 1)
-			strcpy (buffer, "Lambertian radiance");
-		else if (Ctrl->E.mode == 1)
-			strcpy (buffer, "Peucker piecewise linear radiance");
-		else
-			strcpy (buffer, "Directions of grad (z) [uphill direction]");
-	}
+	if (!(Ctrl->Q.mode == 2 && !Ctrl->G.active)) {	/* Not the special case when we don't really need a grid out */
+		if (Ctrl->A.active) {
+			if (Ctrl->N.active)
+				strcpy (buffer, "Normalized directional derivative(s)");
+			else
+				strcpy (buffer, "Directional derivative(s)");
+			sprintf (format, "\t%s\t%s\t%s\t%s\n", GMT->current.setting.format_float_out, GMT->current.setting.format_float_out, GMT->current.setting.format_float_out, GMT->current.setting.format_float_out);
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, " Min Mean Max sigma intensities:");
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, format, min_gradient, ave_gradient, max_gradient, Ctrl->N.sigma);
+		}
+		else {
+			if (Ctrl->E.mode > 1)
+				strcpy (buffer, "Lambertian radiance");
+			else if (Ctrl->E.mode == 1)
+				strcpy (buffer, "Peucker piecewise linear radiance");
+			else
+				strcpy (buffer, "Directions of grad (z) [uphill direction]");
+		}
 
-	if (GMT_Set_Comment (API, GMT_IS_GRID, GMT_COMMENT_IS_OPTION | GMT_COMMENT_IS_COMMAND, options, Out)) Return (API->error);
-	if (GMT_Set_Comment (API, GMT_IS_GRID, GMT_COMMENT_IS_REMARK, buffer, Out)) Return (API->error);
-	if (Ctrl->G.active && GMT_Write_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_CONTAINER_AND_DATA, NULL, Ctrl->G.file, Out) != GMT_NOERROR) {
-		Return (API->error);
-	}
-
-	if (Ctrl->S.active) {
-		if (GMT_Set_Comment (API, GMT_IS_GRID, GMT_COMMENT_IS_OPTION | GMT_COMMENT_IS_COMMAND, options, Slope)) Return (API->error);
-		if (GMT_Set_Comment (API, GMT_IS_GRID, GMT_COMMENT_IS_REMARK, "Magnitude of grad (z)", Slope)) Return (API->error);
-		if (GMT_Write_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_CONTAINER_AND_DATA, NULL, Ctrl->S.file, Slope) != GMT_NOERROR) {
+		if (GMT_Set_Comment (API, GMT_IS_GRID, GMT_COMMENT_IS_OPTION | GMT_COMMENT_IS_COMMAND, options, Out)) Return (API->error);
+		if (GMT_Set_Comment (API, GMT_IS_GRID, GMT_COMMENT_IS_REMARK, buffer, Out)) Return (API->error);
+		if (Ctrl->G.active && GMT_Write_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_CONTAINER_AND_DATA, NULL, Ctrl->G.file, Out) != GMT_NOERROR) {
 			Return (API->error);
 		}
+
+		if (Ctrl->S.active) {
+			if (GMT_Set_Comment (API, GMT_IS_GRID, GMT_COMMENT_IS_OPTION | GMT_COMMENT_IS_COMMAND, options, Slope)) Return (API->error);
+			if (GMT_Set_Comment (API, GMT_IS_GRID, GMT_COMMENT_IS_REMARK, "Magnitude of grad (z)", Slope)) Return (API->error);
+			if (GMT_Write_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_CONTAINER_AND_DATA, NULL, Ctrl->S.file, Slope) != GMT_NOERROR) {
+				Return (API->error);
+			}
+		}
 	}
 
+	if (Ctrl->Q.mode == 2) {	/* Write the new statistics */
+		char sfile[PATH_MAX] = {""};
+		FILE *fp = NULL;
+		GMT_Report (API, GMT_MSG_DEBUG, "Create statistics file [%s] with offset and sigma\n", sfile);
+		if (GMT->session.TMPDIR)
+			sprintf (sfile, "%s/grdgradient.stat", GMT->session.TMPDIR);
+		else if (API->tmp_dir)
+			sprintf (sfile, "%s/grdgradient.stat", API->tmp_dir);
+		else
+			sprintf (sfile, "grdgradient.stat");
+		if ((fp = fopen (sfile, "w")) == NULL) {
+			GMT_Report (API, GMT_MSG_NORMAL, "Cannot create statistics file from this run [%s]!\n", sfile);
+			Return (GMT_ERROR_ON_FOPEN);
+		}
+		if (fprintf (fp, "%.16lg %.16lg\n", ave_gradient, Ctrl->N.sigma) < 0) {
+			GMT_Report (API, GMT_MSG_NORMAL, "Unable to write record to statistics file from this run [%s]!\n", sfile);
+			fclose (fp);
+			Return (GMT_RUNTIME_ERROR);
+		}
+		fclose (fp);
+	}
+	
+
 	Return (GMT_NOERROR);
 }
