@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *	Copyright (c) 1991-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  * Brief synopsis: grdlandmask defines a grid based on region and xinc/yinc values,
@@ -23,19 +22,26 @@
  *
  * Author:	P. Wessel
  * Date:	1-JAN-2010
- * Version:	5 API
+ * Version:	6 API
  */
  
 #include "gmt_dev.h"
 
-#define THIS_MODULE_NAME	"grdlandmask"
+#define THIS_MODULE_CLASSIC_NAME	"grdlandmask"
+#define THIS_MODULE_MODERN_NAME	"grdlandmask"
 #define THIS_MODULE_LIB		"core"
 #define THIS_MODULE_PURPOSE	"Create a \"wet-dry\" mask grid from shoreline data base"
 #define THIS_MODULE_KEYS	"GG}"
 #define THIS_MODULE_NEEDS	"R"
 #define THIS_MODULE_OPTIONS "-VRr" GMT_ADD_x_OPT GMT_OPT("F")
 
-#define GRDLANDMASK_N_CLASSES	(GSHHS_MAX_LEVEL + 1)	/* Number of bands separated by the levels */
+#define GRDLANDMASK_N_CLASSES	(2*GSHHS_MAX_LEVEL + 1)	/* Number of bands separated by the levels */
+/* There are 4 GSHHG levels, but we may find points exactly on the border between levels.
+ * To handle these cases we must double the number of levels.  The N.mask[] array will hold the
+ * data values we wish to assign to each of these 9 potential cases.  As we determine where a
+ * node fits into this hierarchy we assign values 0-9 to the grid.  Then, at the end we replace
+ * these values with the corresponding entries in N.mask[].
+ */
 
 struct GRDLANDMASK_CTRL {	/* All control options for this program (except common args) */
 	/* ctive is true if the option has been activated */
@@ -43,13 +49,14 @@ struct GRDLANDMASK_CTRL {	/* All control options for this program (except common
 		bool active;
 		struct GMT_SHORE_SELECT info;
 	} A;
-	struct GRDLNDM_D {	/* -D<resolution> */
+	struct GRDLNDM_D {	/* -D<resolution>[+f] */
 		bool active;
 		bool force;	/* if true, select next highest level if current set is not available */
-		char set;	/* One of f, h, i, l, c */
+		char set;	/* One of f, h, i, l, c, or a for auto */
 	} D;
-	struct GRDLNDM_E {	/* -E */
+	struct GRDLNDM_E {	/* -E[<border> | <cborder>/<lborder>/<iborder>/<pborder>] */
 		bool active;
+		bool linetrace;	/* True if -E was given arguments as we must trace the lines through the grid */
 		unsigned int inside;	/* if 2, then a point exactly on a polygon boundary is considered OUTSIDE, else 1 */
 	} E;
 	struct GRDLNDM_G {	/* -G<maskfile> */
@@ -58,17 +65,23 @@ struct GRDLANDMASK_CTRL {	/* All control options for this program (except common
 	} G;
 	struct GRDLNDM_N {	/* -N<maskvalues> */
 		bool active;
-		unsigned int mode;	/* 1 if dry/wet only, 0 if 5 mask levels */
-		float mask[GRDLANDMASK_N_CLASSES];	/* values for each level */
+		unsigned int wetdry;	/* 1 if dry/wet only, 0 if 5 mask levels */
+		gmt_grdfloat mask[GRDLANDMASK_N_CLASSES];	/* values for each level */
 	} N;
-#ifdef DEBUG
-	struct DBG {	/* -+<bin> */
-		bool active;
-		unsigned int bin;
-	} debug;
-#endif
 };
 
+struct BINCROSS {
+	double x, y, d;
+};
+
+GMT_LOCAL int comp_bincross (const void *p1, const void *p2) {
+	const struct BINCROSS *a = p1, *b = p2;
+
+	if (a->d < b->d) return (-1);
+	if (a->d > b->d) return (+1);
+	return (0);
+}
+
 GMT_LOCAL void *New_Ctrl (struct GMT_CTRL *GMT) {	/* Allocate and initialize a new control structure */
 	struct GRDLANDMASK_CTRL *C;
 	
@@ -79,8 +92,8 @@ GMT_LOCAL void *New_Ctrl (struct GMT_CTRL *GMT) {	/* Allocate and initialize a n
 	C->A.info.high = GSHHS_MAX_LEVEL;				/* Include all GSHHS levels */
 	C->D.set = 'l';							/* Low-resolution coastline data */
 	C->E.inside = GMT_ONEDGE;					/* Default is that points on a boundary are inside */
-	gmt_M_memset (C->N.mask, GRDLANDMASK_N_CLASSES, float);		/* Default "wet" value = 0 */
-	C->N.mask[1] = C->N.mask[3] = 1.0f;				/* Default for "dry" areas = 1 (inside) */
+	gmt_M_memset (C->N.mask, GRDLANDMASK_N_CLASSES, gmt_grdfloat);		/* Default "wet" value = 0 */
+	C->N.mask[2] = C->N.mask[6] = 1.0f;				/* Default for "dry" areas = 1 (inside) */
 	
 	return (C);
 }
@@ -92,14 +105,10 @@ GMT_LOCAL void Free_Ctrl (struct GMT_CTRL *GMT, struct GRDLANDMASK_CTRL *C) {	/*
 }
 
 GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
-	gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
+	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: grdlandmask -G<outgrid> %s %s\n", GMT_I_OPT, GMT_Rgeo_OPT);
-	GMT_Message (API, GMT_TIME_NONE, "\t[%s] [-D<resolution>][+] [-E]\n\t[-N<maskvalues>] [%s] [%s]%s", GMT_A_OPT, GMT_V_OPT, GMT_r_OPT, GMT_x_OPT);
-#ifdef DEBUG
-	GMT_Message (API, GMT_TIME_NONE, " [-+<bin>]");
-#endif
-	GMT_Message (API, GMT_TIME_NONE, "\n\n");
+	GMT_Message (API, GMT_TIME_NONE, "usage: %s -G<outgrid> %s %s\n", name, GMT_I_OPT, GMT_Rgeo_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "\t[%s] [-D<resolution>][+f] [-E[<bordervalues>]]\n\t[-N<maskvalues>] [%s] [%s]%s [%s]\n\n", GMT_A_OPT, GMT_V_OPT, GMT_r_OPT, GMT_x_OPT, GMT_PAR_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
@@ -114,17 +123,18 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t     i - intermediate resolution.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t     l - low resolution [Default].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t     c - crude resolution, for tasks that need crude continent outlines only.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Append + to use a lower resolution should the chosen one not be available [abort].\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   Append +f to use a lower resolution should the chosen one not be available [abort].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-E Indicate that nodes exactly on a polygon boundary are outside [inside].\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   Optionally append <border> or <cborder>/<lborder>/<iborder>/<pborder>.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   We will then trace lines through the grid and reset the cells crossed by\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   the lines to the indicated values [Default is no line tracing].\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   This is a new option and is experimental.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-N Give values to use if a node is outside or inside a feature.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   Specify this information using 1 of 2 formats:\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t     -N<wet>/<dry>.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t     -N<ocean>/<land>/<lake>/<island>/<pond>.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   NaN is a valid entry.  Default values are 0/1/0/1/0 (i.e., 0/1).\n");
 	GMT_Option (API, "V,r,x.");
-#ifdef DEBUG
-	GMT_Message (API, GMT_TIME_NONE, "\t-+ Print only a single bin (debug option).\n");
-#endif
 	
 	return (GMT_MODULE_USAGE);
 }
@@ -157,11 +167,27 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRDLANDMASK_CTRL *Ctrl, struct
 			case 'D':	/* Set GSHHS resolution */
 				Ctrl->D.active = true;
 				Ctrl->D.set = opt->arg[0];
-				Ctrl->D.force = (opt->arg[1] == '+');
+				Ctrl->D.force = (opt->arg[1] == '+' && (opt->arg[2] == 'f' || opt->arg[2] == '\0'));
 				break;
 			case 'E':	/* On-boundary setting */
 				Ctrl->E.active = true;
-				Ctrl->E.inside = GMT_INSIDE;
+				if (opt->arg[0]) {	/* Trace lines through grid */
+					GMT_Report (API, GMT_MSG_VERBOSE, "-E<values> is presently being tested and is considered experimental\n");
+					Ctrl->E.linetrace = true;
+					j = pos = 0;
+					strncpy (line, opt->arg,  GMT_LEN256-1);
+					while (j < 4 && (gmt_strtok (line, "/", &pos, ptr))) {
+						Ctrl->N.mask[2*j+1] = (ptr[0] == 'N' || ptr[0] == 'n') ? GMT->session.f_NaN : (gmt_grdfloat)atof (ptr);
+						j++;
+					}
+					if (!(j == 1 || j == 4)) {
+						GMT_Report (API, GMT_MSG_NORMAL, "Syntax error -E option: Specify 1 or 4 border values\n");
+						n_errors++;
+					}
+					if (j == 1) Ctrl->N.mask[3] = Ctrl->N.mask[5] = Ctrl->N.mask[7] = Ctrl->N.mask[1];	/* Duplicate border values */
+				}
+				else
+					Ctrl->E.inside = GMT_INSIDE;
 				break;
 			case 'G':	/* Output filename */
 				if ((Ctrl->G.active = gmt_check_filearg (GMT, 'G', opt->arg, GMT_OUT, GMT_IS_GRID)))
@@ -176,28 +202,22 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRDLANDMASK_CTRL *Ctrl, struct
 				Ctrl->N.active = true;
 				strncpy (line, opt->arg,  GMT_LEN256);
 				if (line[strlen(line)-1] == 'o' && gmt_M_compat_check (GMT, 4)) { /* Edge is considered outside */
-					GMT_Report (API, GMT_MSG_COMPAT, "Warning: Option -N...o is deprecated; use -E instead\n");
+					GMT_Report (API, GMT_MSG_COMPAT, "Option -N...o is deprecated; use -E instead\n");
 					Ctrl->E.active = true;
 					Ctrl->E.inside = GMT_INSIDE;
 					line[strlen(line)-1] = 0;
 				}
 				j = pos = 0;
 				while (j < 5 && (gmt_strtok (line, "/", &pos, ptr))) {
-					Ctrl->N.mask[j] = (ptr[0] == 'N' || ptr[0] == 'n') ? GMT->session.f_NaN : (float)atof (ptr);
+					Ctrl->N.mask[2*j] = (ptr[0] == 'N' || ptr[0] == 'n') ? GMT->session.f_NaN : (gmt_grdfloat)atof (ptr);
 					j++;
 				}
 				if (!(j == 2 || j == 5)) {
-					GMT_Report (API, GMT_MSG_NORMAL, "Syntax error -N option: Specify 2 or 5 arguments\n");
+					GMT_Report (API, GMT_MSG_NORMAL, "Syntax error -N option: Specify 2 or 5 mask values\n");
 					n_errors++;
 				}
-				Ctrl->N.mode = (j == 2);
+				Ctrl->N.wetdry = (j == 2);
 				break;
-#ifdef DEBUG
-			case '+':
-				Ctrl->debug.active = true;
-				Ctrl->debug.bin = atoi (opt->arg);
-				break;
-#endif
 			default:	/* Report bad options */
 				n_errors += gmt_default_error (GMT, opt->option);
 				break;
@@ -212,6 +232,29 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRDLANDMASK_CTRL *Ctrl, struct
 	return (n_errors ? GMT_PARSE_ERROR : GMT_NOERROR);
 }
 
+GMT_LOCAL bool inside (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *C, double x, double y) {
+	/* Return true if this point is inside the grid */
+	int row, col;
+	gmt_M_unused(GMT);
+	row = gmt_M_grd_y_to_row (GMT, y, C);
+	if (row < 0 || row > (int)C->n_rows) return false;
+	col = gmt_M_grd_x_to_col (GMT, x, C);
+	if (col < 0 || col > (int)C->n_columns) return false;
+	return true;
+}
+
+GMT_LOCAL void assign_node (struct GMT_CTRL *GMT, struct GMT_GRID *G, struct GMT_GRID_HEADER *C, double x, double y, gmt_grdfloat f_level, int64_t *ij) {
+	/* Set node value and return node index if this point is inside the grid */
+	int row, col;
+	gmt_M_unused(GMT);
+	row = gmt_M_grd_y_to_row (GMT, y, C);
+	if (row < 0 || row > (int)C->n_rows) return;
+	col = gmt_M_grd_x_to_col (GMT, x, C);
+	if (col < 0 || col > (int)C->n_columns) return;
+	*ij = gmt_M_ijp (G->header, row, col);
+	G->data[*ij] = f_level;
+}
+
 #define bailout(code) {gmt_M_free_options (mode); return (code);}
 #define Return(code) {Free_Ctrl (GMT, Ctrl); gmt_end_module (GMT, GMT_cpy); bailout (code);}
 
@@ -220,19 +263,25 @@ int GMT_grdlandmask (void *V_API, int mode, void *args) {
 	unsigned int base = 3, k, bin, np, side, np_new;
 	int row, row_min, row_max, ii, col, col_min, col_max, i, direction, err, ind, nx1, ny1, error = 0;
 	
-	uint64_t ij, count[GRDLANDMASK_N_CLASSES];
+	int64_t ij;
+	uint64_t count[GRDLANDMASK_N_CLASSES];
+	
+	size_t nx_alloc = GMT_SMALL_CHUNK;
 
 	char line[GMT_LEN256] = {""};
 	char *shore_resolution[5] = {"full", "high", "intermediate", "low", "crude"};
 
-	double xmin, xmax, ymin, ymax, west_border, east_border, i_dx_inch, i_dy_inch;
+	double xmin, xmax, ymin, ymax, west_border, east_border, i_dx_inch, i_dy_inch, inc_inch[2];
 	double dummy, *x = NULL, *y = NULL;
 	
-	float f_level = 0.0f;
+	gmt_grdfloat f_level = 0.0f;
 
 	struct GMT_SHORE c;
-	struct GMT_GRID *Grid = NULL;
+	struct GMT_GRID *Grid = NULL, *Cart = NULL;
+	struct GMT_GRID_HEADER *C = NULL;
+	struct GMT_GRID_HEADER_HIDDEN *HH = NULL;
 	struct GMT_GSHHS_POL *p = NULL;
+	struct BINCROSS *X = NULL;
 	struct GRDLANDMASK_CTRL *Ctrl = NULL;
 	struct GMT_CTRL *GMT = NULL, *GMT_cpy = NULL;
 	struct GMT_OPTION *options = NULL;
@@ -244,21 +293,20 @@ int GMT_grdlandmask (void *V_API, int mode, void *args) {
 	if (mode == GMT_MODULE_PURPOSE) return (usage (API, GMT_MODULE_PURPOSE));	/* Return the purpose of program */
 	options = GMT_Create_Options (API, mode, args);	if (API->error) return (API->error);	/* Set or get option list */
 
-	if (!options || options->option == GMT_OPT_USAGE) bailout (usage (API, GMT_USAGE));	/* Return the usage message */
-	if (options->option == GMT_OPT_SYNOPSIS) bailout (usage (API, GMT_SYNOPSIS));	/* Return the synopsis */
+	if ((error = gmt_report_usage (API, options, 0, usage)) != GMT_NOERROR) bailout (error);	/* Give usage if requested */
 
 	/* Parse the command-line arguments */
 
-	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
-	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+    if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
 	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
 	if ((error = parse (GMT, Ctrl, options)) != 0) Return (error);
 
 	/*---------------------------- This is the grdlandmask main code ----------------------------*/
 
-	gmt_enable_threads (GMT);	/* Set number of active threads, if supported */
-	/* We know coastline data are geographic so we hardwire this here: */
+	/* We know coastline data are geographic so we hardwire this here so parsing of -R works: */
 	gmt_set_geographic (GMT, GMT_IN);
+	gmt_enable_threads (GMT);	/* Set number of active threads, if supported */
 
 	/* Create the empty grid and allocate space */
 	if ((Grid = GMT_Create_Data (API, GMT_IS_GRID, GMT_IS_SURFACE, GMT_CONTAINER_AND_DATA, NULL, NULL, NULL, \
@@ -269,54 +317,70 @@ int GMT_grdlandmask (void *V_API, int mode, void *args) {
 		Grid->header->wesn[XLO] += 360.0;
 		Grid->header->wesn[XHI] += 360.0;
 	}
-
+	HH = gmt_get_H_hidden (Grid->header);
+	
 	if (Ctrl->D.force) Ctrl->D.set = gmt_shore_adjust_res (GMT, Ctrl->D.set);
 	base = gmt_set_resolution (GMT, &Ctrl->D.set, 'D');
 	gmt_M_memset (count, GRDLANDMASK_N_CLASSES, uint64_t);		/* Counts of each level */
 	
-	if (Ctrl->N.mode) {
-		Ctrl->N.mask[3] = Ctrl->N.mask[1];
-		Ctrl->N.mask[2] = Ctrl->N.mask[4] = Ctrl->N.mask[0];
+	if (Ctrl->N.wetdry) {	/* Must duplicate wet/dry settings */
+		Ctrl->N.mask[6] = Ctrl->N.mask[2];
+		Ctrl->N.mask[4] = Ctrl->N.mask[8] = Ctrl->N.mask[0];
 	}
 
 	if ((err = gmt_init_shore (GMT, Ctrl->D.set, &c, Grid->header->wesn, &Ctrl->A.info))) {
 		GMT_Report (API, GMT_MSG_NORMAL, "%s [GSHHG %s resolution shorelines]\n", GMT_strerror(err), shore_resolution[base]);
 		Return (GMT_RUNTIME_ERROR);
 	}
-	if (gmt_M_is_verbose (GMT, GMT_MSG_VERBOSE)) {
-		GMT_Report (API, GMT_MSG_VERBOSE, "GSHHG version %s\n%s\n%s\n", c.version, c.title, c.source);
+	if (gmt_M_is_verbose (GMT, GMT_MSG_LONG_VERBOSE)) {
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "GSHHG version %s\n", c.version);
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "%s\n", c.title);
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "%s\n", c.source);
 
 		sprintf (line, "%s\n", GMT->current.setting.format_float_out);
-		if (Ctrl->N.mode) {
-			GMT_Report (API, GMT_MSG_VERBOSE, "Nodes in water will be set to ");
+		if (Ctrl->N.wetdry) {
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Nodes in water will be set to ");
 			(gmt_M_is_fnan (Ctrl->N.mask[0])) ? GMT_Message (API, GMT_TIME_NONE, "NaN\n") : GMT_Message (API, GMT_TIME_NONE, line, Ctrl->N.mask[0]);
-			GMT_Report (API, GMT_MSG_VERBOSE, "Nodes on land will be set to ");
-			(gmt_M_is_fnan (Ctrl->N.mask[1])) ? GMT_Message (API, GMT_TIME_NONE, "NaN\n") : GMT_Message (API, GMT_TIME_NONE, line, Ctrl->N.mask[1]);
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Nodes on land will be set to ");
+			(gmt_M_is_fnan (Ctrl->N.mask[2])) ? GMT_Message (API, GMT_TIME_NONE, "NaN\n") : GMT_Message (API, GMT_TIME_NONE, line, Ctrl->N.mask[2]);
 		}
 		else {
-			GMT_Report (API, GMT_MSG_VERBOSE, "Nodes in the oceans will be set to ");
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Nodes in the oceans will be set to ");
 			(gmt_M_is_fnan (Ctrl->N.mask[0])) ? GMT_Message (API, GMT_TIME_NONE, "NaN\n") : GMT_Message (API, GMT_TIME_NONE, line, Ctrl->N.mask[0]);
-			GMT_Report (API, GMT_MSG_VERBOSE, "Nodes on land will be set to ");
-			(gmt_M_is_fnan (Ctrl->N.mask[1])) ? GMT_Message (API, GMT_TIME_NONE, "NaN\n") : GMT_Message (API, GMT_TIME_NONE, line, Ctrl->N.mask[1]);
-			GMT_Report (API, GMT_MSG_VERBOSE, "Nodes in lakes will be set to ");
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Nodes on land will be set to ");
 			(gmt_M_is_fnan (Ctrl->N.mask[2])) ? GMT_Message (API, GMT_TIME_NONE, "NaN\n") : GMT_Message (API, GMT_TIME_NONE, line, Ctrl->N.mask[2]);
-			GMT_Report (API, GMT_MSG_VERBOSE, "Nodes in islands will be set to ");
-			(gmt_M_is_fnan (Ctrl->N.mask[3])) ? GMT_Message (API, GMT_TIME_NONE, "NaN\n") : GMT_Message (API, GMT_TIME_NONE, line, Ctrl->N.mask[3]);
-			GMT_Report (API, GMT_MSG_VERBOSE, "Nodes in ponds will be set to ");
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Nodes in lakes will be set to ");
 			(gmt_M_is_fnan (Ctrl->N.mask[4])) ? GMT_Message (API, GMT_TIME_NONE, "NaN\n") : GMT_Message (API, GMT_TIME_NONE, line, Ctrl->N.mask[4]);
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Nodes in islands will be set to ");
+			(gmt_M_is_fnan (Ctrl->N.mask[6])) ? GMT_Message (API, GMT_TIME_NONE, "NaN\n") : GMT_Message (API, GMT_TIME_NONE, line, Ctrl->N.mask[6]);
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Nodes in ponds will be set to ");
+			(gmt_M_is_fnan (Ctrl->N.mask[8])) ? GMT_Message (API, GMT_TIME_NONE, "NaN\n") : GMT_Message (API, GMT_TIME_NONE, line, Ctrl->N.mask[8]);
+		}
+		if (Ctrl->E.linetrace) {
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Nodes near shoreline will be set to ");
+			(gmt_M_is_fnan (Ctrl->N.mask[1])) ? GMT_Message (API, GMT_TIME_NONE, "NaN\n") : GMT_Message (API, GMT_TIME_NONE, line, Ctrl->N.mask[1]);
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Nodes near lakeline will be set to ");
+			(gmt_M_is_fnan (Ctrl->N.mask[3])) ? GMT_Message (API, GMT_TIME_NONE, "NaN\n") : GMT_Message (API, GMT_TIME_NONE, line, Ctrl->N.mask[3]);
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Nodes near islandline will be set to ");
+			(gmt_M_is_fnan (Ctrl->N.mask[5])) ? GMT_Message (API, GMT_TIME_NONE, "NaN\n") : GMT_Message (API, GMT_TIME_NONE, line, Ctrl->N.mask[5]);
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Nodes near pondline will be set to ");
+			(gmt_M_is_fnan (Ctrl->N.mask[7])) ? GMT_Message (API, GMT_TIME_NONE, "NaN\n") : GMT_Message (API, GMT_TIME_NONE, line, Ctrl->N.mask[7]);
 		}
 	}
 
-	gmt_parse_common_options (GMT, "J", 'J', "x1d");	/* Fake linear projection so the shore machinery will work */
-	if (gmt_M_err_pass (GMT, gmt_map_setup (GMT, Grid->header->wesn), "")) Return (GMT_PROJECTION_ERROR);
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Apply linear scale to avoid wrap-around: -Jx100id; this temporarily changes the domain\n");
+	gmt_parse_common_options (GMT, "J", 'J', "x100id");	/* Fake linear projection so the shore machinery will work. Used bo be -Jx1id but had trouble with roundoff */
+	if (gmt_M_err_pass (GMT, gmt_proj_setup (GMT, Grid->header->wesn), "")) Return (GMT_PROJECTION_ERROR);
 	GMT->current.map.parallel_straight = GMT->current.map.meridian_straight = 2;	/* No resampling along bin boundaries */
-	wrap = GMT->current.map.is_world = gmt_M_grd_is_global (GMT, Grid->header);
+	wrap = GMT->current.map.is_world = gmt_grd_is_global (GMT, Grid->header);
 	double_dip = (wrap && Grid->header->registration == GMT_GRID_NODE_REG);	/* Must duplicate the west nodes to east */
 	/* Using -Jx1d means output is Cartesian but we want to force geographic */
 	gmt_set_geographic (GMT, GMT_OUT);
 	/* All data nodes are thus initialized to 0 */
 	x = gmt_M_memory (GMT, NULL, Grid->header->n_columns, double);
 	y = gmt_M_memory (GMT, NULL, Grid->header->n_rows, double);
+	if (Ctrl->E.linetrace)
+		X = gmt_M_memory (GMT, X, nx_alloc, struct BINCROSS);
 
 	nx1 = Grid->header->n_columns - 1;	ny1 = Grid->header->n_rows - 1;
 
@@ -324,19 +388,22 @@ int GMT_grdlandmask (void *V_API, int mode, void *args) {
 
 	for (col = 0; col <= nx1; col++) gmt_geo_to_xy (GMT, gmt_M_grd_col_to_x (GMT, col, Grid->header), 0.0, &x[col], &dummy);
 	for (row = 0; row <= ny1; row++) gmt_geo_to_xy (GMT, 0.0, gmt_M_grd_row_to_y (GMT, row, Grid->header), &dummy, &y[row]);
-	i_dx_inch = 1.0 / fabs (x[1] - x[0]);
-	i_dy_inch = 1.0 / fabs (y[1] - y[0]);
+	inc_inch[GMT_X] = fabs (x[1] - x[0]);
+	inc_inch[GMT_Y] = fabs (y[1] - y[0]);
+	i_dx_inch = 1.0 / inc_inch[GMT_X];
+	i_dy_inch = 1.0 / inc_inch[GMT_Y];
 
+	if ((Cart = GMT_Create_Data (API, GMT_IS_GRID, GMT_IS_SURFACE, GMT_CONTAINER_ONLY, NULL, GMT->current.proj.rect, inc_inch, \
+		Grid->header->registration, GMT_NOTSET, NULL)) == NULL) Return (API->error);
+	C = Cart->header;
+	
 	west_border = floor (GMT->common.R.wesn[XLO] / c.bsize) * c.bsize;
 	east_border =  ceil (GMT->common.R.wesn[XHI] / c.bsize) * c.bsize;
 	
 	for (ind = 0; ind < c.nb; ind++) {	/* Loop over necessary bins only */
 
 		bin = c.bins[ind];
-#ifdef DEBUG
-		if (Ctrl->debug.active && bin != Ctrl->debug.bin) continue;
-#endif
-		GMT_Report (API, GMT_MSG_VERBOSE, "Working on block # %5ld\r", bin);
+		GMT_Report (API, GMT_MSG_DEBUG, "Working on block # %5ld\n", bin);
 
 		if ((err = gmt_get_shore_bin (GMT, ind, &c))) {
 			GMT_Report (API, GMT_MSG_NORMAL, "%s [%s resolution shoreline]\n", GMT_strerror(err), shore_resolution[base]);
@@ -383,7 +450,7 @@ int GMT_grdlandmask (void *V_API, int mode, void *args) {
 				/* So row_min is in range [0,?] */
 				row_max = MIN (ny1, irint (floor ((GMT->current.proj.rect[YHI] - ymin) * i_dy_inch - Grid->header->xy_off + GMT_CONV8_LIMIT)));
 				/* So row_max is in range [?,ny1] */
-				f_level = (float)p[k].level;
+				f_level = (gmt_grdfloat)(2*p[k].level);
 
 #ifdef _OPENMP
 #pragma omp parallel for private(row,col,side,ij) shared(row_min,row_max,col_min,col_max,GMT,x,y,p,k,Ctrl,Grid,f_level)
@@ -397,13 +464,121 @@ int GMT_grdlandmask (void *V_API, int mode, void *args) {
 						/* Here, point is inside, we must assign value */
 
 						ij = gmt_M_ijp (Grid->header, row, col);
-						if (p[k].level > Grid->data[ij]) Grid->data[ij] = f_level;
+						if (f_level > Grid->data[ij]) Grid->data[ij] = f_level;
 					}
 				}
 			}
 
 			gmt_free_shore_polygons (GMT, p, np_new);
 			gmt_M_free (GMT, p);
+
+			if (Ctrl->E.linetrace) {	/* Deal with tracing the lines and resetting gridnodes */
+				int64_t last_ij;
+				uint64_t nx;
+				unsigned int curr_x_pt, prev_x_pt, pt;
+				int last_col, last_row, start_col, end_col, brow, bcol;
+				double dx, del_x, del_y, xc, yc, xb, yb;
+				bool last_not_set;
+				
+				if ((np = gmt_assemble_shore (GMT, &c, 1, false, west_border, east_border, &p)) == 0) {	/* Just get segments */
+					gmt_free_shore (GMT, &c);
+					continue;
+				}
+				for (k = 0; k < np; k++) {
+					if (p[k].n == 0) continue;
+					for (pt = 0; pt < (unsigned int)p[k].n; pt++) {	/* Convert geo coordinates to inches */
+						gmt_geo_to_xy (GMT, p[k].lon[pt], p[k].lat[pt], &xc, &yc);
+						p[k].lon[pt] = xc;		p[k].lat[pt] = yc;
+					}
+					f_level = (gmt_grdfloat)(2*p[k].level-1);	/* Border index is half-way between the two levels */
+					last_ij = UINT_MAX;
+					last_not_set = true;
+					last_col = last_row = -1;
+
+					/* To handle lines that exit the grid we pursue the entire line even if outside.
+					 * We only check if (row,col) is inside when filling in between points and assigning nodes */
+					
+					for (pt = 0; pt < (unsigned int)p[k].n; pt++) {
+						/* Get (row,col) and index to nearest node for this point */
+						row = gmt_M_grd_y_to_row (GMT, p[k].lat[pt], C);
+						col = gmt_M_grd_x_to_col (GMT, p[k].lon[pt], C);
+						ij = gmt_M_ijp (Grid->header, row, col);
+
+						if (last_not_set) {	/* Initialize last node to this node the first time */
+							last_ij = ij;
+							last_not_set = false;
+						}
+
+						if (ij != last_ij) {	/* Crossed into another cell */
+							start_col = MIN (last_col, col) + 1;
+							end_col = MAX (last_col, col);
+							dx = p[k].lon[pt] - p[k].lon[pt-1];
+
+							/* Find all the bin-line intersections - modified from x2sys_binlist */
+
+							/* Add the start and stop coordinates to the xc/yc arrays so we can get mid-points of each interval */
+
+							nx = 0;
+							if (inside (GMT, C, last_row, last_col)) {	/* Previous point is inside, add it to our list with zero distance */
+								X[nx].x = p[k].lon[pt-1];	X[nx].y = p[k].lat[pt-1];	X[nx++].d = 0.0;
+							}
+							if (inside (GMT, C, row, col)) {	/* This point is inside, add it to our list with max distance */
+								X[nx].x = p[k].lon[pt];	X[nx].y = p[k].lat[pt];	X[nx++].d = hypot (dx, p[k].lat[pt] - p[k].lat[pt-1]);
+							}
+							
+							/* Now add all crossings between this line segment and the gridlines outlining the cells centered on the nodes */
+							
+							for (brow = MIN (last_row, row) + 1; brow <= MAX (last_row, row); brow++) {	/* If we go in here we know dy is non-zero */
+								if (brow < 0 || brow > (int)C->n_rows) continue;	/* Outside grid */
+								/* Determine intersections between the line segment and parallels */
+								yb = gmt_M_grd_row_to_y (GMT, brow, C) + 0.5 * inc_inch[GMT_Y];
+								del_y = yb - p[k].lat[pt-1];
+								del_x = del_y * dx / (p[k].lat[pt] - p[k].lat[pt-1]);
+								xc = p[k].lon[pt-1] + del_x;
+								X[nx].x = xc;	X[nx].y = yb;	X[nx].d = hypot (del_x , del_y);
+								nx++;
+								if (nx == nx_alloc) {
+									nx_alloc <<= 1;
+									X = gmt_M_memory (GMT, X, nx_alloc, struct BINCROSS);
+								}
+							}
+							for (bcol = start_col; bcol <= end_col; bcol++) {	/* If we go in here we think dx is non-zero (we do a last-ditch dx check just in case) */
+								if (bcol < 0 || bcol > (int)C->n_columns) continue;
+								/* Determine intersections between the line segment and meridians */
+								xb = gmt_M_grd_col_to_x (GMT, bcol, C) - 0.5 * inc_inch[GMT_X];
+								del_x = xb - p[k].lon[pt-1];
+								del_y = (dx == 0.0) ? 0.5 * (p[k].lat[pt] - p[k].lat[pt-1]) : del_x * (p[k].lat[pt] - p[k].lat[pt-1]) / dx;
+								yc = p[k].lat[pt-1] + del_y;
+								X[nx].x = xb;	X[nx].y = yc;	X[nx].d = hypot (del_x, del_y);
+								nx++;
+								if (nx == nx_alloc) {
+									nx_alloc <<= 1;
+									X = gmt_M_memory (GMT, X, nx_alloc, struct BINCROSS);
+								}
+							}
+
+							if (nx) {	/* Here we have intersections */
+								qsort (X, nx, sizeof (struct BINCROSS), comp_bincross);	/* Sort on distances along the line segment */
+								assign_node (GMT, Grid, C, X[0].x, X[0].y, f_level, &ij);	/* Possibly assign f_level to nearest node */
+								for (curr_x_pt = 1, prev_x_pt = 0; curr_x_pt < nx; curr_x_pt++, prev_x_pt++) {	/* Process the intervals, getting mid-points and using that to get bin */
+									assign_node (GMT, Grid, C, X[curr_x_pt].x, X[curr_x_pt].y, f_level, &ij);	/* Possibly assign f_level to nearest node */
+									/* Add mid-point between the cuts */
+									xc = 0.5 * (X[curr_x_pt].x + X[prev_x_pt].x);
+									yc = 0.5 * (X[curr_x_pt].y + X[prev_x_pt].y);
+									assign_node (GMT, Grid, C, xc, yc, f_level, &ij);	/* Possibly assign f_level to nearest node */
+								}
+							}
+						}
+						/* Update last row/col and index */
+						last_ij = ij;
+						last_col = col;
+						last_row = row;
+					}
+
+				}
+				gmt_free_shore_polygons (GMT, p, np);
+				gmt_M_free (GMT, p);
+			}
 		}
 
 		if (!used_polygons) {	/* Lack of polygons or clipping etc resulted in no polygons after all, must deal with background */
@@ -418,15 +593,15 @@ int GMT_grdlandmask (void *V_API, int mode, void *args) {
 			/* If k is still INT_MAX we must assume this patch should have the min level of the bin */
 
 			if (k == INT_MAX) k = MIN (MIN (c.node_level[0], c.node_level[1]) , MIN (c.node_level[2], c.node_level[3]));
-			f_level = (float)k;
+			f_level = (gmt_grdfloat)(2*k);
 
 			/* Determine nodes to initialize */
 
-			row_min = MAX (0, irint (ceil ((Grid->header->wesn[YHI] - c.lat_sw - c.bsize) * Grid->header->r_inc[GMT_Y] - Grid->header->xy_off)));
-			row_max = MIN (ny1, irint (floor ((Grid->header->wesn[YHI] - c.lat_sw) * Grid->header->r_inc[GMT_Y] - Grid->header->xy_off)));
+			row_min = MAX (0, irint (ceil ((Grid->header->wesn[YHI] - c.lat_sw - c.bsize) * HH->r_inc[GMT_Y] - Grid->header->xy_off)));
+			row_max = MIN (ny1, irint (floor ((Grid->header->wesn[YHI] - c.lat_sw) * HH->r_inc[GMT_Y] - Grid->header->xy_off)));
 			if (wrap) {	/* Handle jumps */
-				col_min = irint (ceil (fmod (c.lon_sw - Grid->header->wesn[XLO], 360.0) * Grid->header->r_inc[GMT_X] - Grid->header->xy_off));
-				col_max = irint (floor (fmod (c.lon_sw + c.bsize - Grid->header->wesn[XLO], 360.0) * Grid->header->r_inc[GMT_X] - Grid->header->xy_off));
+				col_min = irint (ceil (fmod (c.lon_sw - Grid->header->wesn[XLO], 360.0) * HH->r_inc[GMT_X] - Grid->header->xy_off));
+				col_max = irint (floor (fmod (c.lon_sw + c.bsize - Grid->header->wesn[XLO], 360.0) * HH->r_inc[GMT_X] - Grid->header->xy_off));
 				if (col_max < col_min) col_max += Grid->header->n_columns;
 			}
 			else {	/* Make sure we are inside our grid */
@@ -438,8 +613,8 @@ int GMT_grdlandmask (void *V_API, int mode, void *args) {
 				else if (lon_e > Grid->header->wesn[XHI] && (lon_e-360.0) > Grid->header->wesn[XLO]) {
 					lon_w -= 360.0;	lon_e -= 360.0;
 				}
-				col_min = irint (ceil (lon_w * Grid->header->r_inc[GMT_X] - Grid->header->xy_off));
-				col_max = irint (floor (lon_e * Grid->header->r_inc[GMT_X] - Grid->header->xy_off));
+				col_min = irint (ceil (lon_w * HH->r_inc[GMT_X] - Grid->header->xy_off));
+				col_max = irint (floor (lon_e * HH->r_inc[GMT_X] - Grid->header->xy_off));
 				if (col_min < 0) col_min = 0;
 				if (col_max > nx1) col_max = nx1;
 			}
@@ -462,6 +637,10 @@ int GMT_grdlandmask (void *V_API, int mode, void *args) {
 	gmt_shore_cleanup (GMT, &c);
 	gmt_M_free (GMT, x);
 	gmt_M_free (GMT, y);
+	if (Ctrl->E.linetrace) gmt_M_free (GMT, X);
+	if (GMT_Destroy_Data (API, &Cart) != GMT_NOERROR) {
+		Return (API->error);
+	}
 
 #ifdef _OPENMP
 #pragma omp parallel for private(row,col,k,ij) shared(GMT,Grid,Ctrl)
@@ -471,7 +650,7 @@ int GMT_grdlandmask (void *V_API, int mode, void *args) {
 		k = urint (Grid->data[ij]);
 		Grid->data[ij] = Ctrl->N.mask[k];
 		count[k]++;
-		if (col == 0 && double_dip) count[k]++;	/* CoOunt these guys twice */
+		if (col == 0 && double_dip) count[k]++;	/* Count these guys twice */
 	}
 
 	if (double_dip) { /* Copy over values to the repeating right column */
@@ -492,10 +671,14 @@ int GMT_grdlandmask (void *V_API, int mode, void *args) {
 	}
 
 	if (gmt_M_is_verbose (GMT, GMT_MSG_VERBOSE)) {
-		for (k = 0; k < GRDLANDMASK_N_CLASSES; k++)
-			if (count[k]) GMT_Report (API, GMT_MSG_VERBOSE, "Level %d contained %" PRIu64 " nodes\n", k, count[k]);
+		for (k = 0; k < GRDLANDMASK_N_CLASSES; k++) {
+			if (count[k] == 0) continue;
+			if (k%2 == 0)
+				GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Level %d set for %" PRIu64 " nodes\n", k/2, count[k]);
+			else
+				GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Border between Levels %d-%d set for %" PRIu64 " nodes\n", (k-1)/2, (k+1)/2, count[k]);
+		}
 	}
-	GMT_Report (API, GMT_MSG_VERBOSE, "Done!\n");
 
 	Return (GMT_NOERROR);
 }
