@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *	Copyright (c) 1991-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  * Brief synopsis: grdpaste.c reads two grid files and writes a new file with
@@ -21,12 +20,13 @@
  *
  * Author:	Walter Smith
  * Date:	1-JAN-2010
- * Version:	5 API
+ * Version:	6 API
  */
 
 #include "gmt_dev.h"
 
-#define THIS_MODULE_NAME	"grdpaste"
+#define THIS_MODULE_CLASSIC_NAME	"grdpaste"
+#define THIS_MODULE_MODERN_NAME	"grdpaste"
 #define THIS_MODULE_LIB		"core"
 #define THIS_MODULE_PURPOSE	"Join two grids along their common edge"
 #define THIS_MODULE_KEYS	"<G{2,GG}"
@@ -63,9 +63,9 @@ GMT_LOCAL void Free_Ctrl (struct GMT_CTRL *GMT, struct GRDPASTE_CTRL *C) {	/* De
 }
 
 GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
-	gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
+	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: grdpaste <grid1> <grid2> -G<outgrid> [%s] [%s]\n\n", GMT_V_OPT, GMT_f_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "usage: %s <grid1> <grid2> -G<outgrid> [%s] [%s] [%s]\n\n", name, GMT_V_OPT, GMT_f_OPT, GMT_PAR_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
@@ -150,6 +150,7 @@ int GMT_grdpaste (void *V_API, int mode, void *args) {
 	double x_noise, y_noise;
 
 	struct GMT_GRID *A = NULL, *B = NULL, *C = NULL;
+	struct GMT_GRID_HEADER_HIDDEN *AH = NULL, *BH = NULL;
 	struct GRDPASTE_CTRL *Ctrl = NULL;
 	struct GMT_CTRL *GMT = NULL, *GMT_cpy = NULL;
 	struct GMT_OPTION *options = NULL;
@@ -161,19 +162,18 @@ int GMT_grdpaste (void *V_API, int mode, void *args) {
 	if (mode == GMT_MODULE_PURPOSE) return (usage (API, GMT_MODULE_PURPOSE));	/* Return the purpose of program */
 	options = GMT_Create_Options (API, mode, args);	if (API->error) return (API->error);	/* Set or get option list */
 
-	if (!options || options->option == GMT_OPT_USAGE) bailout (usage (API, GMT_USAGE));	/* Return the usage message */
-	if (options->option == GMT_OPT_SYNOPSIS) bailout (usage (API, GMT_SYNOPSIS));	/* Return the synopsis */
+	if ((error = gmt_report_usage (API, options, 0, usage)) != GMT_NOERROR) bailout (error);	/* Give usage if requested */
 
 	/* Parse the command-line arguments */
 
-	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
 	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
 	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
 	if ((error = parse (GMT, Ctrl, options)) != 0) Return (error);
 
 	/*---------------------------- This is the grdpaste main code ----------------------------*/
 
-	GMT_Report (API, GMT_MSG_VERBOSE, "Processing input grids\n");
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Processing input grids\n");
 	gmt_set_pad (GMT, 0); /* No padding */
 
 	/* Try to find a common side to join on  */
@@ -203,6 +203,9 @@ int GMT_grdpaste (void *V_API, int mode, void *args) {
 	x_noise = GMT_CONV4_LIMIT * C->header->inc[GMT_X] * 10;
 	y_noise = GMT_CONV4_LIMIT * C->header->inc[GMT_Y] * 10;
 
+	AH = gmt_get_H_hidden (A->header);
+	BH = gmt_get_H_hidden (B->header);
+	
 	common_y = (fabs (A->header->wesn[YLO] - B->header->wesn[YLO]) < y_noise && fabs (A->header->wesn[YHI] - B->header->wesn[YHI]) < y_noise);
 	
 	if (gmt_M_is_geographic (GMT, GMT_IN)) {	/* Must be careful in determining a match since grids may differ by +/-360 in x */
@@ -327,12 +330,12 @@ int GMT_grdpaste (void *V_API, int mode, void *args) {
 
 	/* Now we can do it  */
 
-	if (gmt_M_is_verbose (GMT, GMT_MSG_VERBOSE)) {
+	if (gmt_M_is_verbose (GMT, GMT_MSG_LONG_VERBOSE)) {
 		sprintf (format, "%%s\t%s\t%s\t%s\t%s\t%s\t%s\t%%d\t%%d\n", GMT->current.setting.format_float_out, GMT->current.setting.format_float_out, GMT->current.setting.format_float_out, GMT->current.setting.format_float_out, GMT->current.setting.format_float_out, GMT->current.setting.format_float_out);
-		GMT_Report (API, GMT_MSG_VERBOSE, "File\tW\tE\tS\tN\tdx\tdy\tnx\tny\n");
-		GMT_Report (API, GMT_MSG_VERBOSE, format, Ctrl->In.file[0], A->header->wesn[XLO], A->header->wesn[XHI], A->header->wesn[YLO], A->header->wesn[YHI], A->header->inc[GMT_X], A->header->inc[GMT_Y], A->header->n_columns, A->header->n_rows);
-		GMT_Report (API, GMT_MSG_VERBOSE, format, Ctrl->In.file[1], B->header->wesn[XLO], B->header->wesn[XHI], B->header->wesn[YLO], B->header->wesn[YHI], B->header->inc[GMT_X], B->header->inc[GMT_Y], B->header->n_columns, B->header->n_rows);
-		GMT_Report (API, GMT_MSG_VERBOSE, format, Ctrl->G.file, C->header->wesn[XLO], C->header->wesn[XHI], C->header->wesn[YLO], C->header->wesn[YHI], C->header->inc[GMT_X], C->header->inc[GMT_Y], C->header->n_columns, C->header->n_rows);
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "File\tW\tE\tS\tN\tdx\tdy\tnx\tny\n");
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, format, Ctrl->In.file[0], A->header->wesn[XLO], A->header->wesn[XHI], A->header->wesn[YLO], A->header->wesn[YHI], A->header->inc[GMT_X], A->header->inc[GMT_Y], A->header->n_columns, A->header->n_rows);
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, format, Ctrl->In.file[1], B->header->wesn[XLO], B->header->wesn[XHI], B->header->wesn[YLO], B->header->wesn[YHI], B->header->inc[GMT_X], B->header->inc[GMT_Y], B->header->n_columns, B->header->n_rows);
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, format, Ctrl->G.file, C->header->wesn[XLO], C->header->wesn[XHI], C->header->wesn[YLO], C->header->wesn[YHI], C->header->inc[GMT_X], C->header->inc[GMT_Y], C->header->n_columns, C->header->n_rows);
 	}
 
 	gmt_set_grddim (GMT, C->header);
@@ -340,18 +343,18 @@ int GMT_grdpaste (void *V_API, int mode, void *args) {
 		NULL, 0, 0, C) == NULL) Return (API->error);	/* Note: 0 for pad since no BC work needed */
 	A->data = B->data = C->data;	/* A and B share the same final matrix declared for C */
 	A->header->size = B->header->size = C->header->size;	/* Set A & B's size to the same as C */
-	A->header->no_BC = B->header->no_BC = true;	/* We must disable the BC machinery */
+	AH->no_BC = BH->no_BC = true;	/* We must disable the BC machinery */
 
 	switch (way) {    /* How A and B are positioned relative to each other */
 		case 1:         /* B is on top of A */
 		case 10:		/* B is on top of A but their grid reg limits underlap by one cell */
 		case 11:        /* B is on top of A but their pixel reg limits overlap by one cell */
 			if (is_nc_grid(A)) {
-				A->header->data_offset = B->header->n_columns * (B->header->n_rows - one_or_zero);
+				AH->data_offset = B->header->n_columns * (B->header->n_rows - one_or_zero);
 				if (way == 11)
-					A->header->data_offset -= B->header->n_columns;
+					AH->data_offset -= B->header->n_columns;
 				else if (way == 10)
-					A->header->data_offset += B->header->n_columns;
+					AH->data_offset += B->header->n_columns;
 			}
 			else {
 				GMT->current.io.pad[YHI] = B->header->n_rows - one_or_zero;
@@ -399,11 +402,11 @@ int GMT_grdpaste (void *V_API, int mode, void *args) {
 			}
 			if (is_nc_grid(B)) {
 				gmt_set_pad (GMT, 0U); /* Reset padding */
-				B->header->data_offset = A->header->n_columns * (A->header->n_rows - one_or_zero);
+				BH->data_offset = A->header->n_columns * (A->header->n_rows - one_or_zero);
 				if (way == 22)
-					B->header->data_offset -= A->header->n_columns;
+					BH->data_offset -= A->header->n_columns;
 				else if (way == 21)
-					B->header->data_offset += A->header->n_columns;
+					BH->data_offset += A->header->n_columns;
 			}
 			else {
 				GMT->current.io.pad[YLO] = 0;
@@ -423,12 +426,12 @@ int GMT_grdpaste (void *V_API, int mode, void *args) {
 		case 32:        /* A is on right of B but their grid reg limits underlap by one cell */
 		case 33:        /* A is on right of B but their pixel reg limits overlap by one cell */
 			if (is_nc_grid(A)) {
-				A->header->stride = C->header->n_columns;
-				A->header->data_offset = B->header->n_columns - one_or_zero;
+				AH->stride = C->header->n_columns;
+				AH->data_offset = B->header->n_columns - one_or_zero;
 				if (way == 33)
-					A->header->data_offset--;
+					AH->data_offset--;
 				else if (way == 32)
-					A->header->data_offset++;
+					AH->data_offset++;
 			}
 			else {
 				GMT->current.io.pad[XLO] = B->header->n_columns - one_or_zero;
@@ -444,7 +447,7 @@ int GMT_grdpaste (void *V_API, int mode, void *args) {
 			}
 			if (is_nc_grid(B)) {
 				gmt_set_pad (GMT, 0U); /* Reset padding */
-				B->header->stride = C->header->n_columns;
+				BH->stride = C->header->n_columns;
 			}
 			else {
 				GMT->current.io.pad[XLO] = 0; GMT->current.io.pad[XHI] = A->header->n_columns - one_or_zero;
@@ -463,7 +466,7 @@ int GMT_grdpaste (void *V_API, int mode, void *args) {
 		case 43:        /* A is on left of B but their grid reg limits underlap by one cell */
 		case 44:        /* A is on left of B but their pixel reg limits overlap by one cell */
 			if (is_nc_grid(A)) {
-				A->header->stride = C->header->n_columns;
+				AH->stride = C->header->n_columns;
 			}
 			else {
 				GMT->current.io.pad[XHI] = B->header->n_columns - one_or_zero;
@@ -479,12 +482,12 @@ int GMT_grdpaste (void *V_API, int mode, void *args) {
 			}
 			if (is_nc_grid(B)) {
 				gmt_set_pad (GMT, 0U); /* Reset padding */
-				B->header->stride = C->header->n_columns;
-				B->header->data_offset = A->header->n_columns - one_or_zero;
+				BH->stride = C->header->n_columns;
+				BH->data_offset = A->header->n_columns - one_or_zero;
 				if (way == 44)
-					B->header->data_offset--;
+					BH->data_offset--;
 				else if (way == 43)
-					B->header->data_offset++;
+					BH->data_offset++;
 			}
 			else {
 				GMT->current.io.pad[XHI] = 0;
