@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *	Copyright (c) 1991-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  * Brief synopsis: Reads a grid file and fits a trend surface.  Trend surface
@@ -27,7 +26,7 @@
  *
  * Author:		W. H. F. Smith
  * Date:		1 JAN, 2010
- * Version:	5 API
+ * Version:	6 API
  *
  * Explanations:
  *
@@ -83,33 +82,39 @@
 
 #include "gmt_dev.h"
 
-#define THIS_MODULE_NAME	"grdtrend"
+#define THIS_MODULE_CLASSIC_NAME	"grdtrend"
+#define THIS_MODULE_MODERN_NAME	"grdtrend"
 #define THIS_MODULE_LIB		"core"
 #define THIS_MODULE_PURPOSE	"Fit trend surface to grids and compute residuals"
-#define THIS_MODULE_KEYS	"<G{,DG),TG),WG)"
+#define THIS_MODULE_KEYS	"<G{,DG),TG),WG(,WG)"
 #define THIS_MODULE_NEEDS	""
 #define THIS_MODULE_OPTIONS "-RV"
 
+#ifdef DOUBLE_PRECISION_GRID
+#define fabsf(x) fabs(x)
+#endif
+
 struct GRDTREND_CTRL {	/* All control options for this program (except common args) */
-	struct In {
+	struct GRDTREND_In {
 		bool active;
 		char *file;
 	} In;
-	struct D {	/* -D<diffgrid> */
+	struct GRDTREND_D {	/* -D<diffgrid> */
 		bool active;
 		char *file;
 	} D;
-	struct N {	/* -N[r]<n_model> */
+	struct GRDTREND_N {	/* -N[r]<n_model> */
 		bool active;
 		bool robust;
 		unsigned int value;
 	} N;
-	struct T {	/* -T<trend.grd> */
+	struct GRDTREND_T {	/* -T<trend.grd> */
 		bool active;
 		char *file;
 	} T;
-	struct W {	/* -W<weight.grd> */
+	struct GRDTREND_W {	/* -W<weight.grd>[+s] */
 		bool active;
+		unsigned int mode;
 		char *file;
 	} W;
 };
@@ -134,10 +139,10 @@ GMT_LOCAL void Free_Ctrl (struct GMT_CTRL *GMT, struct GRDTREND_CTRL *C) {	/* De
 }
 
 GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
-	gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
+	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: grdtrend <ingrid> -N<n_model>[+r] [-D<diffgrid>] [%s]\n", GMT_Rgeo_OPT);
-	GMT_Message (API, GMT_TIME_NONE, "\t[-T<trendgrid>] [%s] [-W<weightgrid>]\n\n", GMT_V_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "usage: %s <ingrid> -N<n_model>[+r] [-D<diffgrid>] [%s]\n", name, GMT_Rgeo_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "\t[-T<trendgrid>] [%s] [-W<weightgrid>[+s]] [%s]\n\n", GMT_V_OPT, GMT_PAR_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
@@ -153,6 +158,7 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t-W Supply filename if you want to [read and] write grid file of weights.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   If <weightgrid> can be read at run, and if robust = false, weighted problem will be solved.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   If robust = true, weights used for robust fit will be written to <weightgrid>.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   Append +s to read standard deviations from file and compute weights as 1/s^2.\n");
 	GMT_Option (API, ".");
 
 	return (GMT_MODULE_USAGE);
@@ -167,6 +173,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRDTREND_CTRL *Ctrl, struct GM
 	 * returned when registering these sources/destinations with the API.
 	 */
 
+	char *c = NULL;
 	unsigned int n_errors = 0, n_files = 0, j;
 	struct GMT_OPTION *opt = NULL;
 
@@ -205,6 +212,12 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRDTREND_CTRL *Ctrl, struct GM
 				break;
 			case 'W':
 				Ctrl->W.active = true;
+				if (opt->arg[0] != '+' && (c = strstr (opt->arg, "+s"))) {	/* Gave a file arg plus a +s modifier */
+					Ctrl->W.mode = 2;
+					c[0] = '\0';	/* Chop off modifier */
+				}
+				else
+					Ctrl->W.mode = 1;
 				/* OK if this file doesn't exist; we always write to that file on output */
 				if (gmt_check_filearg (GMT, 'W', opt->arg, GMT_IN, GMT_IS_GRID) || gmt_check_filearg (GMT, 'W', opt->arg, GMT_OUT, GMT_IS_GRID))
 					Ctrl->W.file = strdup (opt->arg);
@@ -248,7 +261,8 @@ GMT_LOCAL void set_up_vals (double *val, unsigned int nval, double vmin, double
 
 GMT_LOCAL void load_pstuff (double *pstuff, unsigned int n_model, double x, double y, unsigned int newx, unsigned int newy) {
 	/* Compute Legendre polynomials of x[i],y[j] as needed  */
-	/* If either x or y has changed, compute new Legendre polynomials as needed  */
+	/* If either x or y has changed, compute new Legendre polynomials as needed.
+	 * Remember: pstuff[0] == 1 throughout.  */
 
 	if (newx) {
 		if (n_model >= 2) pstuff[1] = x;
@@ -278,7 +292,7 @@ GMT_LOCAL void compute_trend (struct GMT_CTRL *GMT, struct GMT_GRID *T, double *
 	gmt_M_grd_loop (GMT, T, row, col, ij) {
 		load_pstuff (pstuff, n_model, xval[col], yval[row], 1, (!(col)));
 		T->data[ij] = 0.0f;
-		for (k = 0; k < n_model; k++) T->data[ij] += (float)(pstuff[k]*gtd[k]);
+		for (k = 0; k < n_model; k++) T->data[ij] += (gmt_grdfloat)(pstuff[k]*gtd[k]);
 	}
 }
 
@@ -354,7 +368,7 @@ GMT_LOCAL double compute_robust_weight (struct GMT_CTRL *GMT, struct GMT_GRID *R
 	/* Find weights from residuals  */
 	unsigned int row, col;
 	uint64_t j = 0, j2, ij;
-	float r, mad, scale;
+	gmt_grdfloat r, mad, scale;
 
 	gmt_M_grd_loop (GMT, R, row, col, ij) {
 		if (gmt_M_is_fnan (R->data[ij])) continue;
@@ -372,7 +386,7 @@ GMT_LOCAL double compute_robust_weight (struct GMT_CTRL *GMT, struct GMT_GRID *R
 
 	/* Use weight according to Huber (1981), but squared */
 
-	gmt_M_memset (W->data, W->header->size, float);	/* Wipe W clean */
+	gmt_M_memset (W->data, W->header->size, gmt_grdfloat);	/* Wipe W clean */
 	gmt_M_grd_loop (GMT, R, row, col, ij) {
 		if (gmt_M_is_fnan (R->data[ij])) {
 			W->data[ij] = R->data[ij];
@@ -440,8 +454,8 @@ GMT_LOCAL void load_gtg_and_gtd (struct GMT_CTRL *GMT, struct GMT_GRID *G, doubl
 
 			if (weighted) {
 				/* Loop over all gtg and gtd elements */
-				gtd[0] += (G->data[ij] * W->data[ij]);
-				gtg[0] += W->data[ij];
+				gtd[0] += (G->data[ij] * W->data[ij]);	/* Implicitly multiply by pstuff[0] which is 1 */
+				gtg[0] += W->data[ij];			/* Implicitly multiply by pstuff[0] which is 1 */
 				for (k = 1; k < n_model; k++) {
 					gtd[k] += (G->data[ij] * W->data[ij] * pstuff[k]);
 					gtg[k] += (W->data[ij] * pstuff[k]);
@@ -450,7 +464,7 @@ GMT_LOCAL void load_gtg_and_gtd (struct GMT_CTRL *GMT, struct GMT_GRID *G, doubl
 			}
 			else {	/* If !weighted  */
 				/* Loop over all gtg and gtd elements */
-				gtd[0] += G->data[ij];
+				gtd[0] += G->data[ij];	/* Implicitly multiply by pstuff[0] which is 1 */
 				for (k = 1; k < n_model; k++) {
 					gtd[k] += (G->data[ij] * pstuff[k]);
 					gtg[k] += pstuff[k];
@@ -478,9 +492,9 @@ GMT_LOCAL void load_gtg_and_gtd (struct GMT_CTRL *GMT, struct GMT_GRID *G, doubl
 int GMT_grdtrend (void *V_API, int mode, void *args) {
 	/* High-level function that implements the grdcontour task */
 
-	bool trivial, weighted,iterations, set_ones = true;
+	bool trivial, weighted, set_ones = true;
 	int error = 0;
-	unsigned int row, col;
+	unsigned int row, col, iterations;
 
 	uint64_t ij;
 
@@ -507,17 +521,16 @@ int GMT_grdtrend (void *V_API, int mode, void *args) {
 	if (mode == GMT_MODULE_PURPOSE) return (usage (API, GMT_MODULE_PURPOSE));	/* Return the purpose of program */
 	options = GMT_Create_Options (API, mode, args);	if (API->error) return (API->error);	/* Set or get option list */
 
-	if (!options || options->option == GMT_OPT_USAGE) bailout (usage (API, GMT_USAGE));	/* Return the usage message */
-	if (options->option == GMT_OPT_SYNOPSIS) bailout (usage (API, GMT_SYNOPSIS));	/* Return the synopsis */
+	if ((error = gmt_report_usage (API, options, 0, usage)) != GMT_NOERROR) bailout (error);	/* Give usage if requested */
 
-	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
 	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
 	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
 	if ((error = parse (GMT, Ctrl, options)) != 0) Return (error);
 
 	/*---------------------------- This is the grdtrend main code ----------------------------*/
 
-	GMT_Report (API, GMT_MSG_VERBOSE, "Processing input grid\n");
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Processing input grid\n");
 	weighted = (Ctrl->N.robust || Ctrl->W.active);
 	trivial = (Ctrl->N.value < 5 && !weighted);
 
@@ -529,7 +542,7 @@ int GMT_grdtrend (void *V_API, int mode, void *args) {
 	if (GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_DATA_ONLY, wesn, Ctrl->In.file, G) == NULL) {	/* Get subset */
 		Return (API->error);
 	}
-
+	
 	/* Check for NaNs (we include the pad for simplicity)  */
 	ij = 0;
 	while (trivial && ij < G->header->size) if (gmt_M_is_fnan (G->data[ij++])) trivial = false;
@@ -553,22 +566,30 @@ int GMT_grdtrend (void *V_API, int mode, void *args) {
 	if (weighted) {
 		if (!gmt_access (GMT, Ctrl->W.file, R_OK)) {	/* We have weights on input  */
 			if ((W = GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_CONTAINER_ONLY, NULL, Ctrl->W.file, NULL)) == NULL) {	/* Get header only */
-				gmt_M_free (GMT, gtd);		gmt_M_free (GMT, gtg);
-				gmt_M_free (GMT, xval);		gmt_M_free (GMT, yval);
-				gmt_M_free (GMT, old);		gmt_M_free (GMT, pstuff);
-				Return (API->error);
+				error = API->error;
+				goto END;
 			}
 			if (W->header->n_columns != G->header->n_columns || W->header->n_rows != G->header->n_rows)
-				GMT_Report (API, GMT_MSG_NORMAL, "Error: Input weight file does not match input data file.  Ignoring.\n");
+				GMT_Report (API, GMT_MSG_NORMAL, "Input weight file does not match input data file.  Ignoring.\n");
 			else {
 				if (GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_DATA_ONLY, NULL, Ctrl->W.file, W) == NULL) {	/* Get data */
-					Return (API->error);
+					error = API->error;
+					goto END;
+				}
+				gmt_M_str_free (Ctrl->W.file);	/* Prevent that the weights grid is overwritten later down */
+				if (Ctrl->W.mode == 2) {	/* Convert sigmas to weights */
+					gmt_M_grd_loop (GMT, W, row, col, ij) {
+						W->data[ij] = (gmt_grdfloat)(1.0 / (W->data[ij] * W->data[ij]));
+					}
 				}
 				set_ones = false;
 			}
 		}
 		if (set_ones) {
-			if ((W = GMT_Duplicate_Data (API, GMT_IS_GRID, GMT_DUPLICATE_ALLOC, G)) == NULL) Return (API->error);	/* Pointer for grid with unit weights  */
+			if ((W = GMT_Duplicate_Data (API, GMT_IS_GRID, GMT_DUPLICATE_ALLOC, G)) == NULL) { /* Pointer for grid with unit weights  */
+				error = API->error;
+				goto END;
+			}
 			gmt_M_setnval (W->data, W->header->size, 1.0f);
 		}
 	}
@@ -596,7 +617,8 @@ int GMT_grdtrend (void *V_API, int mode, void *args) {
 		ierror = gmt_gauss (GMT, gtg, gtd, Ctrl->N.value, Ctrl->N.value, true);
 		if (ierror) {
 			GMT_Report (API, GMT_MSG_NORMAL, "Gauss returns error code %d\n", ierror);
-			return (GMT_RUNTIME_ERROR);
+			error = GMT_RUNTIME_ERROR;
+			goto END;
 		}
 		compute_trend (GMT, T, xval, yval, gtd, Ctrl->N.value, pstuff);
 		if (Ctrl->D.active || Ctrl->N.robust) compute_resid (GMT, G, T, R);
@@ -613,12 +635,13 @@ int GMT_grdtrend (void *V_API, int mode, void *args) {
 				ierror = gmt_gauss (GMT, gtg, gtd, Ctrl->N.value, Ctrl->N.value, true);
 				if (ierror) {
 					GMT_Report (API, GMT_MSG_NORMAL, "Gauss returns error code %d\n", ierror);
-					return (GMT_RUNTIME_ERROR);
+					error = GMT_RUNTIME_ERROR;
+					goto END;
 				}
 				compute_trend (GMT, T, xval, yval, gtd, Ctrl->N.value, pstuff);
 				compute_resid (GMT, G, T, R);
 				chisq = compute_chisq (GMT, R, W, scale);
-				GMT_Report (API, GMT_MSG_VERBOSE, format, iterations, old_chisq, chisq);
+				GMT_Report (API, GMT_MSG_LONG_VERBOSE, format, iterations, old_chisq, chisq);
 				iterations++;
 			} while (old_chisq / chisq > 1.0001);
 
@@ -639,7 +662,8 @@ int GMT_grdtrend (void *V_API, int mode, void *args) {
 		if (GMT_Set_Comment (API, GMT_IS_GRID, GMT_COMMENT_IS_REMARK, "trend surface", T)) Return (API->error);
 		if (GMT_Set_Comment (API, GMT_IS_GRID, GMT_COMMENT_IS_OPTION | GMT_COMMENT_IS_COMMAND, options, T)) Return (API->error);
 		if (GMT_Write_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_CONTAINER_AND_DATA, NULL, Ctrl->T.file, T) != GMT_NOERROR) {
-			Return (API->error);
+			error = API->error;
+			goto END;
 		}
 	}
 	else if (GMT_Destroy_Data (API, &T) != GMT_NOERROR) {
@@ -649,7 +673,8 @@ int GMT_grdtrend (void *V_API, int mode, void *args) {
 		if (GMT_Set_Comment (API, GMT_IS_GRID, GMT_COMMENT_IS_REMARK, "trend residuals", R)) Return (API->error);
 		if (GMT_Set_Comment (API, GMT_IS_GRID, GMT_COMMENT_IS_OPTION | GMT_COMMENT_IS_COMMAND, options, R)) Return (API->error);
 		if (GMT_Write_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_CONTAINER_AND_DATA, NULL, Ctrl->D.file, R) != GMT_NOERROR) {
-			Return (API->error);
+			error = API->error;
+			goto END;
 		}
 	}
 	else if (Ctrl->D.active || Ctrl->N.robust) {
@@ -661,7 +686,8 @@ int GMT_grdtrend (void *V_API, int mode, void *args) {
 		if (GMT_Set_Comment (API, GMT_IS_GRID, GMT_COMMENT_IS_REMARK, "trend weights", W)) Return (API->error);
 		if (GMT_Set_Comment (API, GMT_IS_GRID, GMT_COMMENT_IS_OPTION | GMT_COMMENT_IS_COMMAND, options, W)) Return (API->error);
 		if (GMT_Write_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_CONTAINER_AND_DATA, NULL, Ctrl->W.file, W) != GMT_NOERROR) {
-			Return (API->error);
+			error = API->error;
+			goto END;
 		}
 	}
 	else if (set_ones && GMT_Destroy_Data (API, &W) != GMT_NOERROR) {
@@ -670,7 +696,7 @@ int GMT_grdtrend (void *V_API, int mode, void *args) {
 
 	/* That's all, folks!  */
 
-
+END:
 	gmt_M_free (GMT, pstuff);
 	gmt_M_free (GMT, gtd);
 	gmt_M_free (GMT, gtg);
@@ -678,7 +704,5 @@ int GMT_grdtrend (void *V_API, int mode, void *args) {
 	gmt_M_free (GMT, yval);
 	gmt_M_free (GMT, xval);
 
-	GMT_Report (API, GMT_MSG_VERBOSE, "Done!\n");
-
-	Return (GMT_NOERROR);
+	Return (error);
 }
