@@ -1,8 +1,7 @@
-/*	$Id$
- *
+/*
  * Include file defining macros, functions and structures used in gshhg.c
  *
- *	Copyright (c) 1996-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1996-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -14,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/pwessel
+ *	Contact info: www.generic-mapping-tools.org
  *
  *	12-APR-2016.  For use with GSHHG version 2.3.5
  */
