@@ -1,6 +1,5 @@
-/* $Id$
- *
- * Copyright (c) 1991-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+/*
+ * Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  * See LICENSE.TXT file for copying and redistribution conditions.
  *
  * img2grd.c
@@ -49,13 +48,16 @@
  **  PW 18 Oct 1999 Use WORDS_BIGENDIAN macro set by configure.
  *   PW 12 Apr 2006 Replaced -x -y with -W -D
  *   PW 28 Nov 2006 Added -C for setting origin to main img origin (0,0)
+ *   PW 13 Apr 2018 Get ready for grids with < 1min spacing
+ *   PW 12 May 2019 Make -C the default, add backwards option -F for old default
  *
  */
 
 #include "gmt_dev.h"
 #include "common_byteswap.h"
 
-#define THIS_MODULE_NAME	"img2grd"
+#define THIS_MODULE_CLASSIC_NAME	"img2grd"
+#define THIS_MODULE_MODERN_NAME	"img2grd"
 #define THIS_MODULE_LIB		"img"
 #define THIS_MODULE_PURPOSE	"Extract a subset from an img file in Mercator or Geographic format"
 #define THIS_MODULE_KEYS	"<G{,GG}"
@@ -108,9 +110,6 @@ struct IMG2GRD_CTRL {
 		bool active;
 		char *file;	/* Input file name */
 	} In;
-	struct C {	/* -C */
-		bool active;
-	} C;
 	struct D {	/* -D[<minlat>/<maxlat>] */
 		bool active;
 		double min, max;
@@ -118,6 +117,9 @@ struct IMG2GRD_CTRL {
 	struct E {	/* -E */
 		bool active;
 	} E;
+	struct F {	/* -F */
+		bool active;
+	} F;
 	struct G {	/* -G<output_grdfile> */
 		bool active;
 		char *file;
@@ -171,10 +173,10 @@ GMT_LOCAL void Free_Ctrl (struct GMT_CTRL *GMT, struct IMG2GRD_CTRL *C) {	/* Dea
 }
 
 GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
-	gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
+	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: img2grd <world_image_filename> %s -G<outgrid> -T<type> [-C]\n", GMT_Rgeo_OPT);
-	GMT_Message (API, GMT_TIME_NONE, "\t[-D[<minlat>/<maxlat>]] [-E] [-I<min>] [-M] [-N<navg>] [-S[<scale>]] [%s]\n\t[-W<maxlon>] [%s]\n\n", GMT_V_OPT, GMT_n_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "usage: %s <world_image_filename> %s -G<outgrid> -T<type>\n", name, GMT_Rgeo_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "\t[-D[<minlat>/<maxlat>]] [-E] [-F] [-I<min>[m|s]] [-M] [-N<navg>] [-S[<scale>]] [%s]\n\t[-W<maxlon>] [%s] [%s]\n\n", GMT_V_OPT, GMT_n_OPT, GMT_PAR_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
@@ -182,13 +184,12 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t-G Set filename for the output grid file.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-R Specify the region in decimal degrees or degrees:minutes.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\n\tOPTIONS:\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-C Refer Mercator coordinates to img source origin and requires -M\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   [Default sets lower left to 0,0].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-D Set input img file bottom and top latitudes [%.3f/%.3f].\n", GMT_IMG_MINLAT, GMT_IMG_MAXLAT);
 	GMT_Message (API, GMT_TIME_NONE, "\t   If no latitudes are given it is taken to mean %.3f/%.3f.\n", GMT_IMG_MINLAT_80, GMT_IMG_MAXLAT_80);
 	GMT_Message (API, GMT_TIME_NONE, "\t   Without -D we automatically determine the extent from the file size.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-E Resample geographic grid to the specified -R.  Cannot be used with -M .\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   (Default gives the exact -R of the Mercator grid).\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t-F Translate Mercator coordinates so lower left is (0,0); requires -M\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-I Set input img pixels to be <min> minutes of longitude wide [2.0].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   Without -I we automatically determine the pixel size from the file size.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-M Write a Mercator grid [Default writes a geographic grid].\n");
@@ -216,6 +217,8 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct IMG2GRD_CTRL *Ctrl, struct GMT
 	 */
 
 	unsigned int n_errors = 0, n_files = 0;
+	bool sec = false, min = false;
+	size_t L = 0;
 	struct GMT_OPTION *opt = NULL;
 	struct GMTAPI_CTRL *API = GMT->parent;
 
@@ -233,8 +236,8 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct IMG2GRD_CTRL *Ctrl, struct GMT
 
 			/* Processes program-specific parameters */
 
-			case 'C':
-				Ctrl->C.active = true;
+			case 'F':	/* Backwards helper for old non-C behavior */
+				Ctrl->F.active = true;
 				break;
 			case 'D':
 				Ctrl->D.active = true;
@@ -257,18 +260,30 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct IMG2GRD_CTRL *Ctrl, struct GMT
 					n_errors++;
 				break;
 			case 'm':
-				if (gmt_M_compat_check (GMT, 4))	/* Warn and fall through */
-					GMT_Report (API, GMT_MSG_COMPAT, "Warning: -m<inc> is deprecated; use -I<inc> instead.\n");
+				if (gmt_M_compat_check (GMT, 4))	/* Warn and fall through to 'I' on purpose */
+					GMT_Report (API, GMT_MSG_COMPAT, "-m<inc> is deprecated; use -I<inc> instead.\n");
 				else {
 					n_errors += gmt_default_error (GMT, opt->option);
 					break;
 				}
 			case 'I':
 				Ctrl->I.active = true;
+				L = strlen (opt->arg);
+				if (strchr ("ms", opt->arg[L])) {	/* Valid minute or second unit */
+					if (opt->arg[L] == 's') sec = true;
+					if (opt->arg[L] == 'm') min = true;
+					opt->arg[L] = '\0';
+				}
 				if ((sscanf (opt->arg, "%lf", &Ctrl->I.value)) != 1) {
 					n_errors++;
 					GMT_Report (API, GMT_MSG_NORMAL, "Syntax error: -I requires a positive value.\n");
 				}
+				if (sec) {
+					Ctrl->I.value /= 60.0;
+					opt->arg[L] = 's';
+				}
+				else if (min)
+					opt->arg[L] = 'm';
 				break;
 			case 'M':
 				Ctrl->M.active = true;
@@ -310,7 +325,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct IMG2GRD_CTRL *Ctrl, struct GMT
 	n_errors += gmt_M_check_condition (GMT, Ctrl->T.value < 0 || Ctrl->T.value > 3, "Syntax error: Must specify output type in the range 0-3 with -T.\n");
 	n_errors += gmt_M_check_condition (GMT, Ctrl->W.active && Ctrl->W.value < 360.0, "Syntax error: Requires a maximum longitude >= 360.0 with -W.\n");
 	n_errors += gmt_M_check_condition (GMT, Ctrl->I.active && Ctrl->I.value <= 0.0, "Syntax error: Requires a positive value with -I.\n");
-	n_errors += gmt_M_check_condition (GMT, Ctrl->C.active && !Ctrl->M.active, "Syntax error: -C requires -M.\n");
+	n_errors += gmt_M_check_condition (GMT, Ctrl->F.active && !Ctrl->M.active, "Syntax error: -F requires -M.\n");
 	n_errors += gmt_M_check_condition (GMT, Ctrl->E.active && Ctrl->M.active, "Syntax error: -E cannot be used with -M.\n");
 	n_errors += gmt_M_check_condition (GMT, Ctrl->N.active && Ctrl->N.value < 1, "Syntax error: -N requires an integer > 1.\n");
 
@@ -391,8 +406,8 @@ int GMT_img2grd (void *V_API, int mode, void *args) {
 	int16_t *row = NULL;
 	uint16_t *u2 = NULL;
 
-	char infile[GMT_BUFSIZ] = {""}, cmd[GMT_BUFSIZ] = {""}, input[GMT_STR16] = {""}, output[GMT_LEN256] = {""};
-	char z_units[GMT_GRID_UNIT_LEN80] = {""};
+	char infile[PATH_MAX] = {""}, cmd[GMT_BUFSIZ] = {""}, input[GMT_STR16] = {""}, output[PATH_MAX] = {""};
+	char z_units[GMT_GRID_UNIT_LEN80] = {""}, exact_R[GMT_LEN256] = {""};
 
 	FILE *fp = NULL;
 
@@ -412,14 +427,11 @@ int GMT_img2grd (void *V_API, int mode, void *args) {
 	if (API->error)
 		return (API->error); /* Set or get option list */
 
-	if (!options || options->option == GMT_OPT_USAGE)
-		bailout (usage (API, GMT_USAGE));	/* Return the usage message */
-	if (options->option == GMT_OPT_SYNOPSIS)
-		bailout (usage (API, GMT_SYNOPSIS));	/* Return the synopsis */
+	if ((error = gmt_report_usage (API, options, 0, usage)) != GMT_NOERROR) bailout (error);	/* Give usage if requested */
 
 	/* Parse the command-line arguments */
 
-	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
 	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
 	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
 	if ((error = parse (GMT, Ctrl, options)) != 0) Return (error);
@@ -437,7 +449,7 @@ int GMT_img2grd (void *V_API, int mode, void *args) {
 		imgrange.maxlat = Ctrl->D.max;
 	}
 
-	first = gmt_download_file_if_not_found (GMT, Ctrl->In.file);	/* Deal with downloadable GMT data sets first */
+	first = gmt_download_file_if_not_found (GMT, Ctrl->In.file, 0);	/* Deal with downloadable GMT data sets first */
 
 	if (!gmt_getdatapath (GMT, &Ctrl->In.file[first], infile, R_OK)) {
 		GMT_Report (API, GMT_MSG_NORMAL, "img file %s not found\n", &Ctrl->In.file[first]);
@@ -448,7 +460,7 @@ int GMT_img2grd (void *V_API, int mode, void *args) {
 		int min;
 		double lat = 0.0;
 		struct stat buf;
-		if (stat (infile, &buf)) return (GMT_GRDIO_STAT_FAILED);	/* Inquiry about file failed somehow */
+		if (stat (infile, &buf)) Return (GMT_GRDIO_STAT_FAILED);	/* Inquiry about file failed somehow */
 
 		switch (buf.st_size) {	/* Known sizes are 1 or 2 min at lat_max = ~72 or 80 */
 			case GMT_IMG_NLON_1M*GMT_IMG_NLAT_1M_80*GMT_IMG_ITEMSIZE:
@@ -467,6 +479,10 @@ int GMT_img2grd (void *V_API, int mode, void *args) {
 				if (lat == 0.0) lat = GMT_IMG_MAXLAT_72;
 				min = 2;
 				break;
+			case GMT_IMG_NLON_4M*GMT_IMG_NLAT_4M_72*GMT_IMG_ITEMSIZE:	/* Test grid only */
+				if (lat == 0.0) lat = GMT_IMG_MAXLAT_72;
+				min = 4;
+				break;
 			default:
 				if (lat == 0.0) return (GMT_GRDIO_BAD_IMG_LAT);
 				min = (buf.st_size > GMT_IMG_NLON_2M*GMT_IMG_NLAT_2M_80*GMT_IMG_ITEMSIZE) ? 1 : 2;
@@ -483,22 +499,22 @@ int GMT_img2grd (void *V_API, int mode, void *args) {
 		if (strstr (infile, "age")) {
 			Ctrl->S.value = 0.01;
 			strcpy (z_units, "Myr");
-			GMT_Report (API, GMT_MSG_VERBOSE, "img file %s determined to be crustal ages with scale 0.01.\n", infile);
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "img file %s determined to be crustal ages with scale 0.01.\n", infile);
 		}
 		if (strstr (infile, "topo")) {
 			Ctrl->S.value = 1.0;
 			strcpy (z_units, "meter");
-			GMT_Report (API, GMT_MSG_VERBOSE, "img file %s determined to be bathymetry with scale 1.\n", infile);
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "img file %s determined to be bathymetry with scale 1.\n", infile);
 		}
 		else if (strstr (infile, "grav")) {
 			Ctrl->S.value = 0.1;
 			strcpy (z_units, "mGal");
-			GMT_Report (API, GMT_MSG_VERBOSE, "img file %s determined to be free-air anomalies with scale 0.1.\n", infile);
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "img file %s determined to be free-air anomalies with scale 0.1.\n", infile);
 		}
 		else if (strstr (infile, "curv") || strstr (infile, "vgg")) {
 			Ctrl->S.value = (Ctrl->I.value == 2.0) ? 0.05 : 0.02;
 			strcpy (z_units, "Eotvos");
-			GMT_Report (API, GMT_MSG_VERBOSE, "img file %s determined to be VGG anomalies with scale %g.\n", infile, Ctrl->S.value);
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "img file %s determined to be VGG anomalies with scale %g.\n", infile, Ctrl->S.value);
 		}
 		else {
 			GMT_Report (API, GMT_MSG_VERBOSE, "Unable to determine data type for img file %s; scale not used.\n", infile);
@@ -516,7 +532,7 @@ int GMT_img2grd (void *V_API, int mode, void *args) {
 		Return (GMT_RUNTIME_ERROR);
 	}
 
-	if ((fp = fopen (infile, "rb")) == NULL) {
+	if ((fp = gmt_fopen (GMT, infile, "rb")) == NULL) {
 		GMT_Report (API, GMT_MSG_NORMAL, "Syntax error: Cannot open %s for binary read.\n", infile);
 		Return (GMT_RUNTIME_ERROR);
 	}
@@ -540,14 +556,14 @@ int GMT_img2grd (void *V_API, int mode, void *args) {
 	dx = 1.0 / ((double)imgcoord.nx360 / 360.0);
 	inc[GMT_X] = inc[GMT_Y] = dx * navg;
 
-	GMT_Report (API, GMT_MSG_VERBOSE, "Expects %s to be %d by %d pixels spanning 0/%5.1f/%.8g/%.8g.\n", infile, imgcoord.nxcol, imgcoord.nyrow, dx*imgcoord.nxcol, botlat, toplat);
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Expects %s to be %d by %d pixels spanning 0/%5.1f/%.8g/%.8g.\n", infile, imgcoord.nxcol, imgcoord.nyrow, dx*imgcoord.nxcol, botlat, toplat);
 
 	if (toplat < wesn[YHI]) {
-		GMT_Report (API, GMT_MSG_VERBOSE, "Warning: Your top latitude (%.12g) lies outside top latitude of input (%.12g) - now truncated.\n", wesn[YHI], toplat);
+		GMT_Report (API, GMT_MSG_VERBOSE, "Your top latitude (%.12g) lies outside top latitude of input (%.12g) - now truncated.\n", wesn[YHI], toplat);
 		wesn[YHI] = toplat - GMT_CONV8_LIMIT;	/* To ensure proper round-off in calculating n_rows */
 	}
 	if (botlat > wesn[YLO]) {
-		GMT_Report (API, GMT_MSG_VERBOSE, "Warning: Your bottom latitude (%.12g) lies outside bottom latitude of input (%.12g) - now truncated.\n", wesn[YLO], botlat);
+		GMT_Report (API, GMT_MSG_VERBOSE, "Your bottom latitude (%.12g) lies outside bottom latitude of input (%.12g) - now truncated.\n", wesn[YLO], botlat);
 		wesn[YLO] = botlat + GMT_CONV8_LIMIT;	/* To ensure proper round-off in calculating n_rows */
 	}
 
@@ -568,8 +584,9 @@ int GMT_img2grd (void *V_API, int mode, void *args) {
 	wesn[XHI] = iinstop  * dx;
 	n_columns = (int)(iinstop - iinstart) / navg;
 
-	GMT_Report (API, GMT_MSG_VERBOSE, "To fit [averaged] input, your %s is adjusted to -R%.12g/%.12g/%.12g/%.12g.\n", Ctrl->In.file, wesn[XLO], wesn[XHI], wesn[YLO], wesn[YHI]);
-	GMT_Report (API, GMT_MSG_VERBOSE, "The output grid size will be %d by %d pixels.\n", n_columns, n_rows);
+	sprintf (exact_R, "%.16g/%.16g/%.16g/%.16g", wesn[XLO], wesn[XHI], wesn[YLO], wesn[YHI]);
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "To fit [averaged] input, your %s is adjusted to the exact region -R%s.\n", Ctrl->In.file, exact_R);
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "The output grid size will be %d by %d pixels.\n", n_columns, n_rows);
 
 	/* Set iinstart so that it is non-negative, for use to index pixels.  */
 	while (iinstart < 0) iinstart += imgcoord.nx360;
@@ -577,11 +594,18 @@ int GMT_img2grd (void *V_API, int mode, void *args) {
 	/* Set navgsq, rnavgsq, for the averaging */
 	navgsq = navg * navg;
 	rnavgsq = 1.0 / navgsq;
-
-	/* Set up header with Mercatorized dimensions assuming -Jm1  */
-	if (Ctrl->C.active) {
-		int equator;
-		equator = irint (img_lat_to_ypix (0.0, &imgcoord));
+	
+	/* Set up header with Mercatorized dimensions assuming -Jm1i  */
+	if (Ctrl->F.active || !Ctrl->M.active) {	/* Backwards support for old behavior in -M, or being internally consistent with central meridian */
+		wesn[XLO] = 0.0;
+		wesn[XHI] = n_columns * inc[GMT_X];
+		wesn[YLO] = 0.0;
+		wesn[YHI] = n_rows * inc[GMT_Y];
+		left = wesn[XLO];
+		bottom = wesn[YLO];
+	}
+	else {
+		int equator = irint (img_lat_to_ypix (0.0, &imgcoord));
 		wesn[XLO] = iinstart * inc[GMT_X];
 		wesn[XHI] = wesn[XLO] + n_columns * inc[GMT_X];
 		wesn[YHI] = (imgcoord.nyrow - (int)jinstart - equator) * inc[GMT_Y];
@@ -591,19 +615,11 @@ int GMT_img2grd (void *V_API, int mode, void *args) {
 			wesn[XHI] -= 360.0;
 			wesn[XLO] -= 360.0;
 		}
-		else if (west < 0.0 && east < 0.0 && west >= -180.0 && wesn[XLO] > 0.0) {	/* Gave reasonable negative region, honor that for -C */
+		else if (west < 0.0 && east < 0.0 && west >= -180.0 && wesn[XLO] > 0.0) {	/* Gave reasonable negative region, honor it */
 			wesn[XHI] -= 360.0;
 			wesn[XLO] -= 360.0;
 		}
 	}
-	else {
-		wesn[XLO] = 0.0;
-		wesn[XHI] = n_columns * inc[GMT_X];
-		wesn[YLO] = 0.0;
-		wesn[YHI] = n_rows * inc[GMT_Y];
-		left = wesn[XLO];
-		bottom = wesn[YLO];
-	}
 	GMT_Report (API, GMT_MSG_DEBUG, "Allocate Grid container for Mercator data\n");
 	if ((Merc = GMT_Create_Data (API, GMT_IS_GRID, GMT_IS_SURFACE, GMT_CONTAINER_AND_DATA, NULL, wesn, inc, \
 		GMT_GRID_PIXEL_REG, GMT_NOTSET, NULL)) == NULL) {
@@ -663,7 +679,7 @@ int GMT_img2grd (void *V_API, int mode, void *args) {
 			continue;
 		}
 		if ((fread (row, sizeof (int16_t), n_expected, fp) ) != n_expected) {
-			GMT_Report (API, GMT_MSG_NORMAL, "Error: Read failure at jin = %d.\n", jin);
+			GMT_Report (API, GMT_MSG_NORMAL, "Read failure at jin = %d.\n", jin);
 			gmt_M_free (GMT, ix);	gmt_M_free (GMT, row);
 			fclose (fp);
 			GMT_exit (GMT, GMT_DATA_READ_ERROR); return GMT_DATA_READ_ERROR;
@@ -705,13 +721,13 @@ int GMT_img2grd (void *V_API, int mode, void *args) {
 			switch (Ctrl->T.value) {
 				case 0:
 				case 1:
-					Merc->data[ij] = (float) dsum;
+					Merc->data[ij] = (gmt_grdfloat) dsum;
 					break;
 				case 2:
-					Merc->data[ij] = (float)((csum >= 0.5) ? dsum : GMT->session.f_NaN);
+					Merc->data[ij] = (gmt_grdfloat)((csum >= 0.5) ? dsum : GMT->session.f_NaN);
 					break;
 				case 3:
-					Merc->data[ij] = (float)csum;
+					Merc->data[ij] = (gmt_grdfloat)csum;
 					break;
 			}
 
@@ -727,9 +743,21 @@ int GMT_img2grd (void *V_API, int mode, void *args) {
 	gmt_M_free (GMT, ix);
 	gmt_M_free (GMT, row);
 
+	if (!Ctrl->E.active) {	/* Update R history with exact region found above */
+		/* Because the Mercator grid's equidistant y-nodes are not equidistant when converted back to geogrpaohic coordinates,
+		 * the corresponding south and north coordinates for the outside of the pixels will generally not match the given
+		 * south and north boundaries in the specified -R setting.  Because this is only an issue in this program we replace
+		 * the given -R with the exact -R in the gmt.history file so that any subsequent module call using -R shorthand will
+		 * get the actual (exect) region and not the requested (initial, approximate) region. */
+		int id = gmt_get_option_id (0, "R");			/* The -R[P] item in the history array */
+		if (GMT->current.setting.run_mode == GMT_MODERN) id++;	/* Instead pick the -RG item under modern mode since this is not a plotting tool */
+		gmt_M_str_free (GMT->init.history[id]);			/* Free the previous history string set during parsing */
+		GMT->init.history[id] = strdup (exact_R);		/* Replace it with the exact region */
+	}
+	
 	/* We now have the Mercator grid in Grid. */
 
-	GMT_Report (API, GMT_MSG_VERBOSE, "Created %d by %d Mercatorized grid file.  Min, Max values are %.8g  %.8g\n", Merc->header->n_columns, Merc->header->n_rows, Merc->header->z_min, Merc->header->z_max);
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Created %d by %d Mercatorized grid file.  Min, Max values are %.8g  %.8g\n", Merc->header->n_columns, Merc->header->n_rows, Merc->header->z_min, Merc->header->z_max);
 	if (Ctrl->M.active) {	/* Write out the Mercator grid and return, no projection needed */
 		gmt_set_pad (GMT, API->pad);	/* Reset to session default pad before output */
 		if (GMT_Set_Comment (API, GMT_IS_GRID, GMT_COMMENT_IS_OPTION | GMT_COMMENT_IS_COMMAND, options, Merc)) Return (API->error);
@@ -741,7 +769,7 @@ int GMT_img2grd (void *V_API, int mode, void *args) {
 
 	/* Here we need to reproject the data, and Merc becomes a temporary grid */
 
-	GMT_Report (API, GMT_MSG_VERBOSE, "Undo the implicit spherical Mercator -Jm1i projection.\n");
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Undo the implicit spherical Mercator -Jm1i projection.\n");
 	/* Preparing source and destination for GMT_grdproject */
 	/* a. Register the Mercator grid to be the source read by GMT_grdproject by passing a pointer */
 	GMT_Report (API, GMT_MSG_DEBUG, "Open Mercator Grid as grdproject virtual input\n");
@@ -756,7 +784,7 @@ int GMT_img2grd (void *V_API, int mode, void *args) {
 		}
 	}
 	else	/* The output here is the final result */
-		strncpy (output, Ctrl->G.file, GMT_LEN256-1);
+		strncpy (output, Ctrl->G.file, PATH_MAX-1);
 	sprintf (cmd, "-R%g/%g/%g/%g -Jm1i -I %s -G%s --PROJ_ELLIPSOID=Sphere --PROJ_LENGTH_UNIT=inch --GMT_HISTORY=false", west, east, south2, north2, input, output);
 	GMT_Report (API, GMT_MSG_DEBUG, "Calling grdproject %s.\n", cmd);
 	if (GMT_Call_Module (API, "grdproject", GMT_MODULE_CMD, cmd)!= GMT_NOERROR) {	/* Inverse project the grid or fail */
@@ -791,7 +819,7 @@ int GMT_img2grd (void *V_API, int mode, void *args) {
 			Return (API->error);
 		}
 		sprintf (cmd, "-R%g/%g/%g/%g -I%gm %s -G%s -fg --GMT_HISTORY=false", west, east, south, north, Ctrl->I.value, input, Ctrl->G.file);
-		GMT_Report (API, GMT_MSG_VERBOSE, "Calling grdsample with args %s\n", cmd);
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Calling grdsample with args %s\n", cmd);
 		if (GMT_Call_Module (API, "grdsample", GMT_MODULE_CMD, cmd) != GMT_NOERROR) {	/* Resample the grid or fail */
 			Return (API->error);
 		}
