@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *	Copyright (c) 1991-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  * Brief synopsis: kml2gmt is a reformatter that takes KML files and extracts GMT tables;
@@ -21,15 +20,16 @@
  *
  * Author:	Paul Wessel
  * Date:	1-JAN-2010
- * Version:	5 API
+ * Version:	6 API
  */
  
 #include "gmt_dev.h"
 
-#define THIS_MODULE_NAME	"kml2gmt"
+#define THIS_MODULE_CLASSIC_NAME	"kml2gmt"
+#define THIS_MODULE_MODERN_NAME	"kml2gmt"
 #define THIS_MODULE_LIB		"core"
 #define THIS_MODULE_PURPOSE	"Extract GMT table data from Google Earth KML files"
-#define THIS_MODULE_KEYS	"<T{,>D}"
+#define THIS_MODULE_KEYS	">D}"
 #define THIS_MODULE_NEEDS	""
 #define THIS_MODULE_OPTIONS "-:Vbdh" GMT_OPT("HMm")
 
@@ -38,16 +38,19 @@
 #define POLYGON			2
 
 struct KML2GMT_CTRL {
-	struct In {	/* in file */
+	struct KML2GMT_In {	/* in file */
 		bool active;
 		char *file;
 	} In;
-	struct F {	/* -F */
+	struct KML2GMT_E {	/* -E */
+		bool active;
+	} E;
+	struct KML2GMT_F {	/* -F */
 		bool active;
 		unsigned int mode;
 		unsigned int geometry;
 	} F;
-	struct Z {	/* -Z */
+	struct KML2GMT_Z {	/* -Z */
 		bool active;
 	} Z;
 };
@@ -69,9 +72,10 @@ GMT_LOCAL void Free_Ctrl (struct GMT_CTRL *GMT, struct KML2GMT_CTRL *C) {	/* Dea
 }
 
 GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
-	gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
+	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: kml2gmt [<kmlfiles>] [-Fs|l|p] [%s] [-Z] [%s] [%s]\n\t[%s] [%s]\n\n", GMT_V_OPT, GMT_bo_OPT, GMT_do_OPT, GMT_ho_OPT, GMT_colon_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "usage: %s [<kmlfiles>] [-E] [-Fs|l|p] [%s] [-Z] [%s] [%s]\n\t[%s] [%s] [%s]\n\n",
+		name, GMT_V_OPT, GMT_bo_OPT, GMT_do_OPT, GMT_ho_OPT, GMT_colon_OPT, GMT_PAR_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
@@ -80,6 +84,7 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\n\tOPTIONS:\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t<kmlfiles> is one or more KML files from Google Earth or similar.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   If no files are given, standard input is read.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t-E Get Z from the ExtendData property (only single <SimpleData name=\"string\"> implemented so far).\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-F Restrict feature type; choose from (s)symbol, (l)ine, or (p)olygon.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   Use to only output data for the selected feature type [all].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-Z Output the z-column from the KML file [Only lon,lat is output].\n");
@@ -105,7 +110,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct KML2GMT_CTRL *Ctrl, struct GMT
 
 			case '<':	/* Input files */
 				if (n_files++ > 0) break;
-				if ((Ctrl->In.active = gmt_check_filearg (GMT, '<', opt->arg, GMT_IN, GMT_IS_TEXTSET)))
+				if ((Ctrl->In.active = gmt_check_filearg (GMT, '<', opt->arg, GMT_IN, GMT_IS_DATASET)))
 					Ctrl->In.file = strdup (opt->arg);
 				else
 					n_errors++;
@@ -113,6 +118,10 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct KML2GMT_CTRL *Ctrl, struct GMT
 
 			/* Processes program-specific parameters */
 
+			case 'E':	/* Feature type */
+		 		Ctrl->E.active = true;
+ 				Ctrl->Z.active = true;	/* Needs this too */
+				break;
 			case 'F':	/* Feature type */
 		 		Ctrl->F.active = true;
 				switch (opt->arg[0]) {
@@ -156,16 +165,17 @@ int GMT_kml2gmt (void *V_API, int mode, void *args) {
 	unsigned int i, start, fmode = POINT, n_features = 0, pos;
 	int error = 0, n_scan;
 	size_t length;
-	bool scan = true, first = true, skip, single = false;
+	bool scan = true, first = true, skip, single = false, extended = false;
 	
-	char line[GMT_BUFSIZ] = {""}, buffer[GMT_BUFSIZ] = {""}, name[GMT_BUFSIZ] = {""};
+	char buffer[GMT_BUFSIZ] = {""}, name[GMT_BUFSIZ] = {""};
 	char word[GMT_LEN128] = {""}, description[GMT_BUFSIZ] = {""};
-	char *gm[3] = {"Point", "Line", "Polygon"};
+	char *gm[3] = {"Point", "Line", "Polygon"}, *line = NULL;
 
-	double out[3];
+	double out[3], elev;
 	
 	FILE *fp = NULL;
 
+	struct GMT_RECORD *Out = NULL;
 	struct KML2GMT_CTRL *Ctrl = NULL;
 	struct GMT_CTRL *GMT = NULL, *GMT_cpy = NULL;		/* General GMT internal parameters */
 	struct GMT_OPTION *options = NULL;
@@ -177,26 +187,23 @@ int GMT_kml2gmt (void *V_API, int mode, void *args) {
 	if (mode == GMT_MODULE_PURPOSE) return (usage (API, GMT_MODULE_PURPOSE));	/* Return the purpose of program */
 	options = GMT_Create_Options (API, mode, args);	if (API->error) return (API->error);	/* Set or get option list */
 
-	if (!options || options->option == GMT_OPT_USAGE) bailout (usage (API, GMT_USAGE));/* Return the usage message */
-	if (options->option == GMT_OPT_SYNOPSIS) bailout (usage (API, GMT_SYNOPSIS));	/* Return the synopsis */
+	if ((error = gmt_report_usage (API, options, 0, usage)) != GMT_NOERROR) bailout (error);	/* Give usage if requested */
 
 	/* Parse the command-line arguments */
 
-	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
 	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
 	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
 	if ((error = parse (GMT, Ctrl, options)) != 0) Return (error);
 
 	/*---------------------------- This is the kml2gmt main code ----------------------------*/
 
-	GMT_Report (API, GMT_MSG_VERBOSE, "Processing input KML data\n");
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Processing input KML data\n");
 	gmt_set_geographic (GMT, GMT_IN);
 	gmt_set_geographic (GMT, GMT_OUT);
 	gmt_set_segmentheader (GMT, GMT_OUT, true);	/* Turn on segment headers on output */
 	
-	if ((error = gmt_set_cols (GMT, GMT_OUT, 2 + Ctrl->Z.active)) != GMT_NOERROR) {
-		Return (error);
-	}
+	GMT_Set_Columns (API, GMT_OUT, 2 + Ctrl->Z.active, GMT_COL_FIX_NO_TEXT);
 	if (GMT_Init_IO (API, GMT_IS_DATASET, GMT_IS_PLP, GMT_OUT, GMT_ADD_DEFAULT, 0, options) != GMT_NOERROR) {	/* Registers default output destination, unless already set */
 		Return (API->error);
 	}
@@ -214,28 +221,32 @@ int GMT_kml2gmt (void *V_API, int mode, void *args) {
 	 * P. Wessel, April 2013. */
 	
 	if (Ctrl->In.active) {
-		if ((fp = fopen (Ctrl->In.file, "r")) == NULL) {
+		if ((fp = gmt_fopen (GMT, Ctrl->In.file, "r")) == NULL) {
 			GMT_Report (API, GMT_MSG_NORMAL, "Cannot open file %s\n", Ctrl->In.file);
 			Return (GMT_ERROR_ON_FOPEN);
 		}
-		GMT_Report (API, GMT_MSG_VERBOSE, "Processing %s\n", Ctrl->In.file);
-		sprintf (buffer, "# kml2gmt: KML read from %s\n", Ctrl->In.file);
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Processing %s\n", Ctrl->In.file);
+		sprintf (buffer, "kml2gmt: KML read from %s", Ctrl->In.file);
 	}
 	else {     /* Just read standard input */
 		fp = stdin;
-		GMT_Report (API, GMT_MSG_VERBOSE, "Reading from standard input\n");
-		sprintf (buffer, "# kml2gmt: KML read from standard input\n");
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Reading from standard input\n");
+		sprintf (buffer, "kml2gmt: KML read from standard input");
 	}
 	if (Ctrl->F.active)
-		GMT_Report (API, GMT_MSG_VERBOSE, "Only output features with geometry: %s\n", gm[Ctrl->F.mode]);
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Only output features with geometry: %s\n", gm[Ctrl->F.mode]);
+
+	Out = gmt_new_record (GMT, out, buffer);
 
 	/* Now we are ready to take on some input values */
 
 	strcpy (GMT->current.setting.format_float_out, "%.12g");	/* Get enough decimals */
 	
-	GMT_Put_Record (API, GMT_WRITE_TABLE_HEADER, buffer);	/* Write this to output */
+	GMT_Put_Record (API, GMT_WRITE_TABLE_HEADER, Out);	/* Write this to output */
+	Out->text = NULL;
+	line = gmt_M_memory (GMT, NULL, GMT_INITIAL_MEM_ROW_ALLOC, char);
 
-	while (fgets (line, GMT_BUFSIZ, fp)) {
+	while (fgets (line, GMT_INITIAL_MEM_ROW_ALLOC, fp)) {
 		if (strstr (line, "<Placemark")) {	/* New Placemark, reset name and description */
 			scan = true;
 			name[0] = description[0] = 0;
@@ -255,9 +266,11 @@ int GMT_kml2gmt (void *V_API, int mode, void *args) {
 			strncpy (name, &line[start], GMT_BUFSIZ-1);
 			gmt_chop (name);
 			if (first && !skip) {
-				sprintf (buffer, "# %s\n", &line[start]);
-				GMT_Put_Record (API, GMT_WRITE_TABLE_HEADER, buffer);	/* Write this to output */
+				sprintf (buffer, "%s", &line[start]);
+				Out->text = buffer;
+				GMT_Put_Record (API, GMT_WRITE_TABLE_HEADER, Out);	/* Write this to output */
 				first = false;
+				Out->text = NULL;
 			}
 		}
 		if (strstr (line, "<description>")) {
@@ -268,8 +281,10 @@ int GMT_kml2gmt (void *V_API, int mode, void *args) {
 			strncpy (description, &line[start], GMT_BUFSIZ-1);
 			gmt_chop (description);
 			if (first && !skip) {
-				sprintf (buffer, "# %s\n", &line[start]);
-				GMT_Put_Record (API, GMT_WRITE_TABLE_HEADER, buffer);	/* Write this to output */
+				sprintf (buffer, "%s", &line[start]);
+				Out->text = buffer;
+				GMT_Put_Record (API, GMT_WRITE_TABLE_HEADER, Out);	/* Write this to output */
+				Out->text = NULL;
 				first = false;
 			}
 		}
@@ -281,6 +296,24 @@ int GMT_kml2gmt (void *V_API, int mode, void *args) {
 			if (description[0]) { strcat (GMT->current.io.segment_header, "-D\""); strcat (GMT->current.io.segment_header, description); strcat (GMT->current.io.segment_header, "\""); }
 		}
 		
+		if (Ctrl->E.active && strstr (line, "<ExtendedData>")) {
+			/* https://developers.google.com/kml/documentation/kmlreference#extendeddata
+			   But only a single <SimpleData name="string" is implemented here. */
+			fgets (line, GMT_INITIAL_MEM_ROW_ALLOC, fp);
+			if (strstr (line, "<SchemaData")) {
+				fgets (line, GMT_INITIAL_MEM_ROW_ALLOC, fp);
+				if (strstr (line, "<SimpleData")) {
+					char *p1, *p2;
+					p1 = strchr(line, '>');		p1++;           /* Find end of <SimpleData */
+					p2 = strchr (&p1[0], '<');	p2[0] = '\0';   /* Find begin of </SimpleData>*/
+					elev = atof(p1);
+					extended = true;
+				}
+				else
+					extended = false;
+			}
+		}
+
 		if (!strstr (line, "<coordinates>")) continue;
 		/* We get here when the line says coordinates */
 		
@@ -297,22 +330,25 @@ int GMT_kml2gmt (void *V_API, int mode, void *args) {
 		if (fmode == POINT && single) {	/* Process the single point from current record */
 			for (i = 0; i < length && line[i] != '>'; i++);		/* Find end of <coordinates> */
 			sscanf (&line[i+1], "%lg,%lg,%lg", &out[GMT_X], &out[GMT_Y], &out[GMT_Z]);
-			GMT_Put_Record (API, GMT_WRITE_DATA, out);	/* Write this to output */
+			if (extended) out[GMT_Z] = elev;
+			GMT_Put_Record (API, GMT_WRITE_DATA, Out);	/* Write this to output */
 		}
 		else if (single) {	/* Process multiple points from current single record */
 			for (i = 0; i < length && line[i] != '>'; i++);		/* Find end of <coordinates> */
 			pos = i + 1;
 			while (gmt_strtok (line, " \t", &pos, word)) {	/* Look for clusters of x,y,z separated by whitespace */
 				n_scan = sscanf (word, "%lg,%lg,%lg", &out[GMT_X], &out[GMT_Y], &out[GMT_Z]);
+				if (extended) out[GMT_Z] = elev;
 				if (n_scan == 2 || n_scan == 3)
-					GMT_Put_Record (API, GMT_WRITE_DATA, out);	/* Write this to output */
+					GMT_Put_Record (API, GMT_WRITE_DATA, Out);	/* Write this to output */
 				else
 					break;
 			}
 		}
 		else {	/* Processes points from separate lines */
 			while (fscanf (fp, "%lg,%lg,%lg", &out[GMT_X], &out[GMT_Y], &out[GMT_Z])) {
-				GMT_Put_Record (API, GMT_WRITE_DATA, out);	/* Write this to output */
+				if (extended) out[GMT_Z] = elev;	/* Replave Z by the extended data value */
+				GMT_Put_Record (API, GMT_WRITE_DATA, Out);	/* Write this to output */
 			}
 		}
 		n_features++;
@@ -321,8 +357,10 @@ int GMT_kml2gmt (void *V_API, int mode, void *args) {
 	if (GMT_End_IO (API, GMT_OUT, 0) != GMT_NOERROR) {	/* Disables further data output */
 		Return (API->error);
 	}
+	gmt_M_free (GMT, Out);
+	gmt_M_free (GMT, line);
 	
-	GMT_Report (API, GMT_MSG_VERBOSE, "Found %u features with selected geometry\n", n_features);
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Found %u features with selected geometry\n", n_features);
 	
 	Return (GMT_NOERROR);
 }
