@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *    Copyright (c) 2004-2018 by P. Wessel
+ *    Copyright (c) 2004-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *    See README file for copying and redistribution conditions.
  *--------------------------------------------------------------------*/
 /*
@@ -28,6 +27,7 @@
  *		31-MAR-2006: Changed -X to -L to avoid GMT collision
  *		23-MAY-2006: Added -Q for limits on speed/azimuths
  *		21-FEB-2008: Added -Ga|b<rec> for limits on rec range
+ *		22-MAR-2018: Added -am8|16 for recompute mag using diur
  *
  *
  */
@@ -35,12 +35,13 @@
 #include "gmt_dev.h"
 #include "mgd77.h"
 
-#define THIS_MODULE_NAME	"mgd77list"
+#define THIS_MODULE_CLASSIC_NAME	"mgd77list"
+#define THIS_MODULE_MODERN_NAME	"mgd77list"
 #define THIS_MODULE_LIB		"mgd77"
 #define THIS_MODULE_PURPOSE	"Extract data from MGD77 files"
-#define THIS_MODULE_KEYS	">?}"
+#define THIS_MODULE_KEYS	">D}"
 #define THIS_MODULE_NEEDS	""
-#define THIS_MODULE_OPTIONS "-:RVbdh"
+#define THIS_MODULE_OPTIONS "-:RVbdhj"
 
 #define MGD77_FMT  "drt,id,tz,year,month,day,hour,dmin,lat,lon,ptc,twt,depth,bcc,btc,mtf1,mtf2,mag,msens,diur,msd,gobs,eot,faa,nqc,sln,sspn"
 #define MGD77_ALL  "drt,id,time,lat,lon,ptc,twt,depth,bcc,btc,mtf1,mtf2,mag,msens,diur,msd,gobs,eot,faa,nqc,sln,sspn"
@@ -48,6 +49,7 @@
 #define MGD77T_ALL "id,time,lat,lon,ptc,nqc,twt,depth,bcc,btc,bqc,mtf1,mtf2,mag,msens,diur,msd,mqc,gobs,eot,faa,gqc,sln,sspn"
 #define MGD77_GEO  "time,lat,lon,twt,depth,mtf1,mtf2,mag,gobs,faa"
 #define MGD77_AUX  "dist,azim,cc,vel,weight"
+#define MGD77_DAT  "drt,tz,year,month,day,hour,dmin,lat,lon,ptc,twt,depth,bcc,btc,mtf1,mtf2,mag,msens,diur,msd,gobs,eot,faa,nqc,id,sln,sspn"
 
 #define ADJ_CT	0
 #define ADJ_DP	1
@@ -55,26 +57,28 @@
 #define ADJ_MG	3
 
 /* Carter adjustment options */
-#define CT_U_MINUS_DEPTH				1
-#define CT_U_MINUS_CARTER				2
-#define CT_UCORR_MINUS_CARTER_TU		4
+#define CT_U_MINUS_DEPTH		1
+#define CT_U_MINUS_CARTER		2
+#define CT_UCORR_MINUS_CARTER_TU	4
 #define CT_UCORR_CARTER_TU_MINUS_DEPTH	8
 
 /* FAA adjustment options */
-#define GR_FAA_STORED					1
-#define GR_OBS_MINUS_NGRAV				2
-#define GR_OBS_PLUS_EOT_MINUS_NGRAV		4
+#define GR_FAA_STORED			1
+#define GR_OBS_MINUS_NGRAV		2
+#define GR_OBS_PLUS_EOT_MINUS_NGRAV	4
 #define GR_OBS_PLUS_CEOT_MINUS_NGRAV	8
 
 /* Depth adjustment options */
-#define DP_DEPTH_STORED				1
-#define DP_TWT_X_V					2
+#define DP_DEPTH_STORED			1
+#define DP_TWT_X_V			2
 #define DP_TWT_X_V_MINUS_CARTER		4
 
 /* Mag adjustment options */
 #define MG_MAG_STORED			1
 #define MG_MTF1_MINUS_IGRF		2
 #define MG_MTF2_MINUS_IGRF		4
+#define MG_MTF1_PLUS_DIUR_MINUS_IGRF	8
+#define MG_MTF2_PLUS_DIUR_MINUS_IGRF	16
 
 #define N_D	0	/* These are indices for -N subsets */
 #define N_S	1
@@ -95,10 +99,6 @@ struct MGD77LIST_CTRL {	/* All control options for this program (except common a
 		double sound_speed;
 		double sensor_offset;
 	} A;
-	struct MGD77LIST_C {	/* -C */
-		bool active;
-		unsigned int mode;
-	} C;
 	struct MGD77LIST_D {	/* -D */
 		bool active;
 		bool mode;	/* true to skip recs with time == NaN */
@@ -148,7 +148,7 @@ struct MGD77LIST_CTRL {	/* All control options for this program (except common a
 		bool active;
 		double value;
 	} W;
-	struct MGD77LIST_Z {	/* -Z[-|+] */
+	struct MGD77LIST_Z {	/* -Z[n|p] */
 		bool active;
 		bool mode;
 	} Z;
@@ -162,7 +162,6 @@ GMT_LOCAL void *New_Ctrl (struct GMT_CTRL *GMT) {	/* Allocate and initialize a n
 	/* Initialize values whose defaults are not 0/false/NULL */
 	
 	C->A.GF_version = MGD77_NOT_SET;
-	C->C.mode = 2;
 	C->D.stop = C->S.stop = DBL_MAX;	/* No upper limit on time and distance */
 	C->G.stop = UINTMAX_MAX;		/* No limit on stop record */
 	C->N.unit[N_D][0] = 'k';	/* Default is -Ndk */
@@ -183,11 +182,11 @@ GMT_LOCAL void Free_Ctrl (struct GMT_CTRL *GMT, struct MGD77LIST_CTRL *C) {	/* D
 }
 
 GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
-	gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
+	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: mgd77list <cruise(s)> -F<dataflags>[,<tests>] [-A[+]c|d|f|m|t[<code>]] [-Cf|g|e]\n");
+	GMT_Message (API, GMT_TIME_NONE, "usage: %s <cruise(s)> -F<dataflags>[,<tests>] [-Ac|d|f|m|t[<code>][+f]]\n", name);
 	GMT_Message (API, GMT_TIME_NONE, "\t[-Da<startdate>] [-Db<stopdate>] [-E] [-Ga<startrec>] [-Gb<stoprec>] [-I<code>]\n\t[-L[<corrtable.txt>]] [-N[s|p][<unit>]]] [-Qa|v<min>/<max>]\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t[%s] [-Sa<startdist>[<unit>]] [-Sb<stopdist>[<unit>]]\n\t[-T[m|e]] [%s] [-W<Weight>] [-Z[+|-] [%s] [%s] [-h] [%s]\n\n", GMT_Rgeo_OPT, GMT_V_OPT, GMT_bo_OPT, GMT_do_OPT, GMT_colon_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "\t[%s] [-Sa<startdist>[<unit>]] [-Sb<stopdist>[<unit>]]\n\t[-T[m|e]] [%s] [-W<Weight>] [-Z[n|p] [%s] [%s] [-h] [%s] [%s] [%s]\n\n", GMT_Rgeo_OPT, GMT_V_OPT, GMT_bo_OPT, GMT_do_OPT, GMT_j_OPT, GMT_colon_OPT, GMT_PAR_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
@@ -215,7 +214,7 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t     ngdcid:  NGDC ID [TEXTSTRING].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t     recno:   Record number.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   >Derived navigational information:\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     dist:    Along-track distances (see -C for method and -N for units).\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t     dist:    Along-track distances (see -j for method and -N for units).\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t     azim:    Track azimuth (Degrees east from north).\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t     cc:      Course change, i.e., change in azimuth (Degrees east from north).\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t     vel:     Ship velocity (m/s).\n");
@@ -251,7 +250,8 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t     mgd77t:  The full set of all 26 columns in the MGD77T specification.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t     geo:     time,lon,lat + the 7 geophysical observations.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t     all:     As mgd77 but with time items written as a date-time string.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     allt:     As mgd77t but with time items written as a date-time string.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t     allt:    As mgd77t but with time items written as a date-time string.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t     dat:     As mgd77t but in plain table file order.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t    Append + to include the 5 derived quantities dist, azim, cc, vel, and weight [see -W]\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t    [Default is all].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t  Abbreviations in UPPER CASE will suppress records where any such column is NaN.\n");
@@ -298,17 +298,15 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t       f4 return difference gobs + eot - ngrav.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t       f8 return difference gobs + ceot - ngrav.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   m<code> Adjust field mag.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t       m1 return mag as stored in file [Default].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t       m2 return difference mtfx - igrf, where x = msens (or 1 if undefined).\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t       m4 return difference mtfx - igrf, where x != msens (or 2 if undefined).\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t       m1  return mag as stored in file [Default].\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t       m2  return difference mtfx - igrf, where x = msens (or 1 if undefined).\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t       m4  return difference mtfx - igrf, where x != msens (or 2 if undefined).\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t       m8  return difference mtfx + diur - igrf, where x = msens (or 1 if undefined).\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t       m16 return difference mtfx + diur - igrf, where x != msens (or 2 if undefined).\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t       mc<offset>[unit] Apply cable tow distance correction to mtf1.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   t will compute fake times for cruises with known duration but lacking record times.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   The optional -A+ means selected anomalies will be recalculated even when the original\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   Append +f to force selected anomalies to be recalculated even when the original\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   anomaly is NaN [Default honors NaNs in existing anomalies].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-C Select procedure for along-track distance and azimuth calculations:\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   f Flat Earth.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   g Great circle [Default].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   e Ellipsoidal (geodesic) using current ellipsoid.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-D List from a<date> (given as yyyy-mm-ddT[hh:mm:ss]) [Start of cruise]\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   up to b<date> (given as yyyy-mm-ddT[hh:mm:ss]) [End of cruise].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   If A|B is used instead or a|b then records with no time are excluded from output.\n");
@@ -342,10 +340,10 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t   For controlling application of point bit flags, see -F and the : modifier discussion.\n");
 	GMT_Option (API, "V");
 	GMT_Message (API, GMT_TIME_NONE, "\t-W Set weight for these data [1].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-Z Append - to report bathymetry & msd as negative depths [Default is positive -Z+].\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t-Z Append n to report bathymetry and msd as negative depths [Default is positive -Zp].\n");
 	GMT_Option (API, "bo,do");
-	GMT_Message (API, GMT_TIME_NONE, "\t-h Write header record with column information [Default is no header].\n");
-	GMT_Option (API, ":,.");
+	if (gmt_M_showusage (API)) GMT_Message (API, GMT_TIME_NONE, "\t-h Write header record with column information [Default is no header].\n");
+	GMT_Option (API, "j,:,.");
 	
 	return (GMT_MODULE_USAGE);
 }
@@ -359,7 +357,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct MGD77LIST_CTRL *Ctrl, struct G
 
 	unsigned int n_errors = 0, k;
 	int code;
-	char *t = NULL, buffer[GMT_BUFSIZ] = {""};
+	char *t = NULL, buffer[GMT_BUFSIZ] = {""}, *c = NULL;
 	double dist_scale;
 	struct GMT_OPTION *opt = NULL;
 	struct GMTAPI_CTRL *API = GMT->parent;
@@ -380,6 +378,10 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct MGD77LIST_CTRL *Ctrl, struct G
 					Ctrl->A.force = true;
 					k++;
 				}
+				else if ((c = strstr (opt->arg, "+f"))) {
+					Ctrl->A.force = true;
+					c[0] = '\0';	/* Chop of modifier */
+				}
 				switch (opt->arg[k]) {
 					case 'c':	/* Carter correction adjustment */
 						code = opt->arg[k+1] - '0';
@@ -438,8 +440,8 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct MGD77LIST_CTRL *Ctrl, struct G
 						}
 						else {
 							code = atoi (&opt->arg[k+1]);
-							if (code < 1 || code > 7) {
-								GMT_Report (API, GMT_MSG_NORMAL, "Syntax error -Am<code>.  <code> must be 1,2,4 or binary combination.\n");
+							if (code < 1 || code > 33) {
+								GMT_Report (API, GMT_MSG_NORMAL, "Syntax error -Am<code>.  <code> must be 1,2,4,8,16 or binary combination.\n");
 								n_errors++;
 							}
 							Ctrl->A.code[ADJ_MG] |= code;
@@ -453,15 +455,21 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct MGD77LIST_CTRL *Ctrl, struct G
 						n_errors++;
 						break;
 				}
+				if (c) c[0] = '+';	/* Restore modifier */
 				break;
 
 			case 'C':	/* Distance calculation flag */
-				Ctrl->C.active = true;
-				if (opt->arg[0] == 'f') Ctrl->C.mode = 1;
-				if (opt->arg[0] == 'g') Ctrl->C.mode = 2;
-				if (opt->arg[0] == 'e') Ctrl->C.mode = 3;
-				if (Ctrl->C.mode < 1 || Ctrl->C.mode > 3) {
-					GMT_Report (API, GMT_MSG_NORMAL, "Syntax error -C: Flag must be f, g, or e\n");
+				if (gmt_M_compat_check (API->GMT, 6)) {
+					GMT_Report (API, GMT_MSG_COMPAT, "The -C option is deprecated; use the GMT common option -j<mode> instead\n");
+					if (!strchr ("cefg", (int)opt->arg[0])) {
+						GMT_Report (API, GMT_MSG_NORMAL, "Syntax error -C: Flag must be c, f, g, or e\n");
+						n_errors++;
+					}
+					else
+						gmt_parse_j_option (GMT, opt->arg);
+				}
+				else {
+					GMT_Report (API, GMT_MSG_NORMAL, "Unrecognized option -C\n");
 					n_errors++;
 				}
 				break;
@@ -471,6 +479,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct MGD77LIST_CTRL *Ctrl, struct G
 				switch (opt->arg[0]) {
 				 	case 'A':		/* Start date, skip records with time = NaN */
 						Ctrl->D.mode = true;
+						/* Fall through on purpose to 'a' */
 				 	case 'a':		/* Start date */
 						t = &opt->arg[1];
 						if (t && gmt_verify_expectations (GMT, GMT_IS_ABSTIME, gmt_scanf (GMT, t, GMT_IS_ABSTIME, &Ctrl->D.start), t)) {
@@ -480,6 +489,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct MGD77LIST_CTRL *Ctrl, struct G
 						break;
 					case 'B':		/* Stop date, skip records with time = NaN */
 						Ctrl->D.mode = true;
+						/* Fall through on purpose to 'b' */
 					case 'b':		/* Stop date */
 						t = &opt->arg[1];
 						if (t && gmt_verify_expectations (GMT, GMT_IS_ABSTIME, gmt_scanf (GMT, t, GMT_IS_ABSTIME, &Ctrl->D.stop), t)) {
@@ -530,6 +540,12 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct MGD77LIST_CTRL *Ctrl, struct G
 					strcat (buffer, ",");
 					strcat (buffer, MGD77_AUX);
 				}
+				if (!strcmp (buffer, "dat")) strncpy (buffer, MGD77_DAT, GMT_BUFSIZ);
+				if (!strcmp (buffer, "dat+")) {
+					strncpy (buffer, MGD77_DAT, GMT_BUFSIZ);
+					strcat (buffer, ",");
+					strcat (buffer, MGD77_AUX);
+				}
 				Ctrl->F.flags = strdup (buffer);
 				break;
 
@@ -609,6 +625,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct MGD77LIST_CTRL *Ctrl, struct G
 						break;
 					case 'C':	/* Course change min/max using absolute value of cc */
 						Ctrl->Q.c_abs = true;
+						/* Fall through on purpose to 'c' */
 					case 'c':	/* Course change min/max */
 						if (sscanf (&opt->arg[1], "%lf/%lf", &Ctrl->Q.min[Q_C], &Ctrl->Q.max[Q_C]) != 2) {
 							GMT_Report (API, GMT_MSG_NORMAL, "Syntax error -Qc: append min/max course change limits [-360/+360]\n");
@@ -670,9 +687,16 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct MGD77LIST_CTRL *Ctrl, struct G
 				Ctrl->W.value = (!strcmp (opt->arg, "NaN")) ? GMT->session.d_NaN : atof (opt->arg);
 				break;
 
-			case 'Z':		/* -Z- is negative down for depths */
+			case 'Z':		/* -Zn is negative down for depths */
 				Ctrl->Z.active = true;
-				Ctrl->Z.mode = (opt->arg[0] == '-') ? true : false;
+				switch (opt->arg[0]) {
+					case '-':	case 'n':	Ctrl->Z.mode = true;	break;
+					case '+':	case 'p':	Ctrl->Z.mode = false;	break;
+					default:
+						GMT_Report (API, GMT_MSG_NORMAL, "Syntax error -Z: append n or p\n");
+						n_errors++;
+						break;
+				}
 				break;
 			default:	/* Report bad options */
 				n_errors += gmt_default_error (GMT, opt->option);
@@ -741,7 +765,7 @@ GMT_LOCAL int augment_aux_columns (int n_items, char **item_name, struct MGD77_A
 int GMT_mgd77list (void *V_API, int mode, void *args) {
 	int i, c, id, k, time_column, lon_column, lat_column, error = 0;
 	int t_col, x_col, y_col, z_col, e_col = 0, m_col = 0, f_col = 0;
-	int ms_col = 0, twt_col = 0, g_col = 0, m1_col = 0, m2_col = 0;
+	int ms_col = 0, md_col = 0, twt_col = 0, g_col = 0, m1_col = 0, m2_col = 0;
 	int sep_flag, n_paths;
 	
 	unsigned int select_option, n_out = 0, argno, n_cruises = 0, kx, n_items = 0;
@@ -749,7 +773,6 @@ int GMT_mgd77list (void *V_API, int mode, void *args) {
 	
 	uint64_t rec, prevrec;
 	
-	enum GMT_enum_family family;
 	enum GMT_enum_geometry geometry;
 
 	bool negative_depth = false, negative_msd = false, need_distances, need_time;
@@ -796,6 +819,7 @@ int GMT_mgd77list (void *V_API, int mode, void *args) {
 		{ "recno",   MGD77_AUX_RN, true,  false, "recno"},
 		{ "ngdcid",  MGD77_AUX_ID, true,  false, "NGDC-ID"}
 	};
+	struct GMT_RECORD *Out = NULL;
 	struct MGD77LIST_CTRL *Ctrl = NULL;
 	struct GMT_CTRL *GMT = NULL, *GMT_cpy = NULL;
 	struct GMT_OPTION *options = NULL;
@@ -807,12 +831,11 @@ int GMT_mgd77list (void *V_API, int mode, void *args) {
 	if (mode == GMT_MODULE_PURPOSE) return (usage (API, GMT_MODULE_PURPOSE));	/* Return the purpose of program */
 	options = GMT_Create_Options (API, mode, args);	if (API->error) return (API->error);	/* Set or get option list */
 
-	if (!options || options->option == GMT_OPT_USAGE) bailout (usage (API, GMT_USAGE));	/* Return the usage message */
-	if (options->option == GMT_OPT_SYNOPSIS) bailout (usage (API, GMT_SYNOPSIS));	/* Return the synopsis */
+	if ((error = gmt_report_usage (API, options, 0, usage)) != GMT_NOERROR) bailout (error);	/* Give usage if requested */
 
 	/* Parse the command-line arguments */
 
-	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
 	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
 	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
 	if ((error = parse (GMT, Ctrl, options)) != 0) Return (error);
@@ -827,6 +850,7 @@ int GMT_mgd77list (void *V_API, int mode, void *args) {
 	aux_dvalue[MGD77_AUX_RT] = 5.0;					/* Default record type */
 	if (!Ctrl->F.active) Ctrl->F.flags = strdup (MGD77_ALL);	/* Default is a full MGD77 record */
 	negative_msd = negative_depth = Ctrl->Z.mode;			/* Follow the -Z option to start with */
+	if (!GMT->common.j.active) GMT->common.j.mode = GMT_GREATCIRCLE;
 
 	if (Ctrl->T.active) {	/* Turn off automatic corrections */
 		if (Ctrl->T.mode == MGD77_NOT_SET)	/* Both sets */
@@ -842,18 +866,19 @@ int GMT_mgd77list (void *V_API, int mode, void *args) {
 	n_paths = MGD77_Path_Expand (GMT, &M, options, &list);	/* Get list of requested IDs */
 
 	if (n_paths <= 0) {
-		GMT_Report (API, GMT_MSG_NORMAL, "Error: No cruises given\n");
+		GMT_Report (API, GMT_MSG_NORMAL, "No cruises given\n");
 		Return (GMT_NO_INPUT);
 	}
 
 	if (M.adjust_time) Ctrl->D.start = MGD77_time2utime (GMT, &M, Ctrl->D.start);	/* Convert to Unix time if need be */
 	if (M.adjust_time) Ctrl->D.stop  = MGD77_time2utime (GMT, &M, Ctrl->D.stop);
 	if (Ctrl->L.active) {	/* Scan the ephemeral correction table for needed auxiliary columns */
-		char path[GMT_BUFSIZ] = {""};
+		char path[PATH_MAX] = {""};
 		if (!Ctrl->L.file) {	/* Try default correction table */
 			sprintf (path, "%s/mgd77_corrections.txt", M.MGD77_HOME);
 			if (access (path, R_OK)) {
 				GMT_Report (API, GMT_MSG_NORMAL, "No default MGD77 Correction table (%s) found!\n", path);
+				MGD77_Path_Free (GMT, (uint64_t)n_paths, list);
 				Return (GMT_FILE_NOT_FOUND);
 			}
 			Ctrl->L.file = path;
@@ -891,33 +916,49 @@ int GMT_mgd77list (void *V_API, int mode, void *args) {
 	}
 	need_distances = (Ctrl->S.active || auxlist[MGD77_AUX_SP].requested || auxlist[MGD77_AUX_DS].requested || auxlist[MGD77_AUX_AZ].requested || auxlist[MGD77_AUX_CC].requested);	/* Distance is requested */
 	need_lonlat = (auxlist[MGD77_AUX_MG].requested || auxlist[MGD77_AUX_GR].requested || auxlist[MGD77_AUX_CT].requested || Ctrl->A.code[ADJ_MG] > MG_MAG_STORED || Ctrl->A.code[ADJ_DP] & DP_TWT_X_V_MINUS_CARTER || Ctrl->A.code[ADJ_CT] > CT_U_MINUS_DEPTH || Ctrl->A.code[ADJ_GR] > GR_FAA_STORED || Ctrl->A.fake_times || Ctrl->A.cable_adjust);	/* Need lon, lat to calculate reference fields or Carter correction */
-	need_time = (auxlist[MGD77_AUX_YR].requested || auxlist[MGD77_AUX_MO].requested || auxlist[MGD77_AUX_DY].requested || auxlist[MGD77_AUX_HR].requested || auxlist[MGD77_AUX_MI].requested || auxlist[MGD77_AUX_SC].requested \
-		|| auxlist[MGD77_AUX_DM].requested || auxlist[MGD77_AUX_HM].requested || auxlist[MGD77_AUX_DA].requested || auxlist[MGD77_AUX_MG].requested || Ctrl->A.code[ADJ_MG] > MG_MAG_STORED);
+	need_time = (auxlist[MGD77_AUX_YR].requested || auxlist[MGD77_AUX_MO].requested || auxlist[MGD77_AUX_DY].requested ||
+	             auxlist[MGD77_AUX_HR].requested || auxlist[MGD77_AUX_MI].requested || auxlist[MGD77_AUX_SC].requested ||
+	             auxlist[MGD77_AUX_DM].requested || auxlist[MGD77_AUX_HM].requested || auxlist[MGD77_AUX_DA].requested ||
+	             auxlist[MGD77_AUX_MG].requested || Ctrl->A.code[ADJ_MG] > MG_MAG_STORED);
 	n_sub = 0;	/* This value will hold the number of columns that we will NOT printout (they are only needed to calculate auxiliary values) */
 	if (need_distances || need_lonlat) {	/* Must make sure we get lon,lat if they are not already requested */
-		 if (MGD77_Get_Column (GMT, "lat", &M) == MGD77_NOT_SET) strcat (fx_setting, ",lat"), n_sub++;	/* Append lat to requested list */
-		 if (MGD77_Get_Column (GMT, "lon", &M) == MGD77_NOT_SET) strcat (fx_setting, ",lon"), n_sub++;	/* Append lon to requested list */
+		 if (MGD77_Get_Column (GMT, "lat", &M) == MGD77_NOT_SET)
+		 	strcat (fx_setting, ",lat"), n_sub++;	/* Append lat to requested list */
+		 if (MGD77_Get_Column (GMT, "lon", &M) == MGD77_NOT_SET)
+		 	strcat (fx_setting, ",lon"), n_sub++;	/* Append lon to requested list */
 	}
 	if ((Ctrl->D.active || need_time || auxlist[MGD77_AUX_SP].requested) && MGD77_Get_Column (GMT, "time", &M) == MGD77_NOT_SET) strcat (fx_setting, ",time"), n_sub++;	/* Append time to requested list */
-	need_twt = (auxlist[MGD77_AUX_CT].requested || (Ctrl->A.code[ADJ_CT] > 0 && Ctrl->A.code[ADJ_CT] <= CT_U_MINUS_CARTER) || (Ctrl->A.code[ADJ_DP] > DP_DEPTH_STORED));
+	need_twt = (auxlist[MGD77_AUX_CT].requested || (Ctrl->A.code[ADJ_CT] > 0 && Ctrl->A.code[ADJ_CT] <= CT_U_MINUS_CARTER) ||
+	            (Ctrl->A.code[ADJ_DP] > DP_DEPTH_STORED));
 	if (need_twt) {	/* Want to estimate Carter corrections */
-		 if (MGD77_Get_Column (GMT, "twt", &M) == MGD77_NOT_SET) strcat (fx_setting, ",twt"), n_sub++;	/* Must append twt to requested list */
+		 if (MGD77_Get_Column (GMT, "twt", &M) == MGD77_NOT_SET)
+		 	strcat (fx_setting, ",twt"), n_sub++;	/* Must append twt to requested list */
 		MGD77_carter_init (GMT, &Carter);	/* Initialize Carter machinery */
 	}
-	need_depth = ((Ctrl->A.code[ADJ_CT] & (CT_U_MINUS_DEPTH | CT_UCORR_MINUS_CARTER_TU | CT_UCORR_CARTER_TU_MINUS_DEPTH)) || (Ctrl->A.code[ADJ_DP] & DP_DEPTH_STORED));
+	need_depth = ((Ctrl->A.code[ADJ_CT] & (CT_U_MINUS_DEPTH | CT_UCORR_MINUS_CARTER_TU | CT_UCORR_CARTER_TU_MINUS_DEPTH)) ||
+	              (Ctrl->A.code[ADJ_DP] & DP_DEPTH_STORED));
 	if (need_depth) {                   /* Need depth*/
-		 if (MGD77_Get_Column (GMT, "depth", &M) == MGD77_NOT_SET) strcat (fx_setting, ",depth"), n_sub++;	/* Must append depth to requested list */
+		if (MGD77_Get_Column (GMT, "depth", &M) == MGD77_NOT_SET)
+		 	strcat (fx_setting, ",depth"), n_sub++;	/* Must append depth to requested list */
 	}
 	if (Ctrl->A.code[ADJ_GR] > GR_FAA_STORED) {     /* Need gobs */
-		 if (MGD77_Get_Column (GMT, "gobs", &M) == MGD77_NOT_SET) strcat (fx_setting, ",gobs"), n_sub++;	/* Must append gobs to requested list */
+		if (MGD77_Get_Column (GMT, "gobs", &M) == MGD77_NOT_SET)
+		 	strcat (fx_setting, ",gobs"), n_sub++;	/* Must append gobs to requested list */
 	}
 	if (Ctrl->A.code[ADJ_GR] & GR_OBS_PLUS_EOT_MINUS_NGRAV) {    /* Need stored eot */
-		 if (MGD77_Get_Column (GMT, "eot", &M) == MGD77_NOT_SET) strcat (fx_setting, ",eot"), n_sub++;	/* Must append eot to requested list */
+		if (MGD77_Get_Column (GMT, "eot", &M) == MGD77_NOT_SET) strcat (fx_setting, ",eot"), n_sub++;	/* Must append eot to requested list */
 	}
 	if (Ctrl->A.code[ADJ_MG] > MG_MAG_STORED) {     /* Need mtf1,2, and msens */
-		 if (MGD77_Get_Column (GMT, "mtf1", &M) == MGD77_NOT_SET) strcat (fx_setting, ",mtf1"), n_sub++;	/* Must append mtf1 to requested list */
-		 if (MGD77_Get_Column (GMT, "mtf2", &M) == MGD77_NOT_SET) strcat (fx_setting, ",mtf2"), n_sub++;	/* Must append mtf2 to requested list */
-		 if (MGD77_Get_Column (GMT, "msens", &M) == MGD77_NOT_SET) strcat (fx_setting, ",msens"), n_sub++;	/* Must append msens to requested list */
+		if (MGD77_Get_Column (GMT, "mtf1", &M) == MGD77_NOT_SET)
+		 	strcat (fx_setting, ",mtf1"), n_sub++;	/* Must append mtf1 to requested list */
+		if (MGD77_Get_Column (GMT, "mtf2", &M) == MGD77_NOT_SET)
+		 	strcat (fx_setting, ",mtf2"), n_sub++;	/* Must append mtf2 to requested list */
+		if (MGD77_Get_Column (GMT, "msens", &M) == MGD77_NOT_SET)
+		 	strcat (fx_setting, ",msens"), n_sub++;	/* Must append msens to requested list */
+		if (Ctrl->A.code[ADJ_MG] >= 8) {     /* Need mtf1,2, msens and diur */
+			if (MGD77_Get_Column (GMT, "diur", &M) == MGD77_NOT_SET)
+		 		strcat (fx_setting, ",diur"), n_sub++;	/* Must append diur to requested list */
+		}
 	}
 	else if (Ctrl->A.cable_adjust)
 		 if (MGD77_Get_Column (GMT, "mtf1", &M) == MGD77_NOT_SET) strcat (fx_setting, ",mtf1"), n_sub++;	/* Must append mtf1 to requested list */
@@ -983,14 +1024,14 @@ int GMT_mgd77list (void *V_API, int mode, void *args) {
 			break;
 	}
 
-	gmt_init_distaz (GMT, GMT_MAP_DIST_UNIT, Ctrl->C.mode, GMT_MAP_DIST);
+	gmt_init_distaz (GMT, GMT_MAP_DIST_UNIT, GMT->common.j.mode, GMT_MAP_DIST);
 
 	Ctrl->S.start *= dist_scale;	Ctrl->S.stop *= dist_scale;	/* Convert the meters to the same units used for cumulative distances */
 	if (Ctrl->A.cable_adjust) Ctrl->A.sensor_offset *= dist_scale;
 	
 
 	if (Ctrl->L.active) {	/* Load an ephemeral correction table */
-		char path[GMT_BUFSIZ] = {""};
+		char path[PATH_MAX] = {""};
 		if (!Ctrl->L.file) {	/* Try default correction table */
 			sprintf (path, "%s/mgd77_corrections.txt", M.MGD77_HOME);
 			if (access (path, R_OK)) {
@@ -1006,33 +1047,31 @@ int GMT_mgd77list (void *V_API, int mode, void *args) {
 	if (strstr (Ctrl->F.flags, "id") || strstr (Ctrl->F.flags, "ngdcid") || strstr (Ctrl->F.flags, "sln")
 	    || strstr (Ctrl->F.flags, "sspn") || strstr (Ctrl->F.flags, "date") || strstr (Ctrl->F.flags, "recno"))
 		string_output = true;
-
-	family = (string_output) ? GMT_IS_TEXTSET : GMT_IS_DATASET;
 	
 	geometry = (string_output) ? GMT_IS_NONE : GMT_IS_POINT;
-	if (GMT_Init_IO (API, family, geometry, GMT_OUT, GMT_ADD_DEFAULT, 0, options) != GMT_NOERROR) {	/* Establishes data output */
+	if (GMT_Init_IO (API, GMT_IS_DATASET, geometry, GMT_OUT, GMT_ADD_DEFAULT, 0, options) != GMT_NOERROR) {	/* Establishes data output */
 		Return (API->error);
 	}
-	if (GMT_Begin_IO (API, family, GMT_OUT, GMT_HEADER_ON) != GMT_NOERROR) {	/* Enables data output and sets access mode */
+	if (GMT_Begin_IO (API, GMT_IS_DATASET, GMT_OUT, GMT_HEADER_ON) != GMT_NOERROR) {	/* Enables data output and sets access mode */
 		Return (API->error);
 	}
 	if (GMT_Set_Geometry (API, GMT_OUT, GMT_IS_POINT) != GMT_NOERROR) {	/* Sets output geometry */
 		Return (API->error);
 	}
-	if (!string_output) gmt_set_cols (GMT, GMT_OUT, n_out_columns);
+	if (!string_output) GMT_Set_Columns (API, GMT_OUT, n_out_columns, GMT_COL_FIX_NO_TEXT);
 
 	for (argno = 0; argno < (unsigned int)n_paths; argno++) {		/* Process each ID */
 	
 		if (MGD77_Open_File (GMT, list[argno], &M, MGD77_READ_MODE)) continue;
 
-		GMT_Report (API, GMT_MSG_VERBOSE, "Now processing cruise %s\n", list[argno]);
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Now processing cruise %s\n", list[argno]);
 		
 		D = MGD77_Create_Dataset (GMT);
 
 		error = MGD77_Read_Header_Record (GMT, list[argno], &M, &D->H);
 		if (error) {
 			if (error == MGD77_ERROR_NOSUCHCOLUMN)
-				GMT_Report (API, GMT_MSG_NORMAL, "One or more requested columns not present in cruise %s - skipping\n", list[argno]);
+				GMT_Report (API, GMT_MSG_VERBOSE, "One or more requested columns not present in cruise %s - skipping\n", list[argno]);
 			else
 				GMT_Report (API, GMT_MSG_NORMAL, "Error reading header sequence for cruise %s - skipping\n", list[argno]);
 			MGD77_Free_Dataset (GMT, &D);
@@ -1048,12 +1087,16 @@ int GMT_mgd77list (void *V_API, int mode, void *args) {
 			}
 			if (auxlist[MGD77_AUX_ID].requested || auxlist[MGD77_AUX_DA].requested) string_output = true;
 			if (string_output && GMT->common.b.active[1]) {
-				GMT_Report (API, GMT_MSG_NORMAL, "Error: Cannot specify binary output with text fields\n");
+				GMT_Report (API, GMT_MSG_NORMAL, "Cannot specify binary output with text fields\n");
 				MGD77_Free_Dataset (GMT, &D);
 				Return (GMT_RUNTIME_ERROR);
 			}
-			if (!string_output) out = gmt_M_memory (GMT, NULL, n_out_columns, double);
-
+			if (string_output)
+				Out = gmt_new_record (GMT, NULL, record);
+			else {
+				out = gmt_M_memory (GMT, NULL, n_out_columns, double);
+				Out = gmt_new_record (GMT, out, NULL);
+			}
 		}
 		
 		if (MGD77_Read_Data (GMT, list[argno], &M, D)) {
@@ -1069,26 +1112,25 @@ int GMT_mgd77list (void *V_API, int mode, void *args) {
 		lat_column  = ((i = MGD77_Get_Column (GMT, "lat",  &M)) != MGD77_NOT_SET && M.order[i].set == MGD77_M77_SET) ? M.order[i].item : 3 * MGD77_NOT_SET;
 		
 		if (time_column != MGD77_NOT_SET && GMT->common.b.active[GMT_OUT] && gmt_M_is_verbose (GMT, GMT_MSG_VERBOSE) && first_warning) {	/* Warn that binary time output is in Unix secs */
-			GMT_Report (API, GMT_MSG_VERBOSE, "Warning: For binary output, time is stored as seconds since 1970 (Use TIME_SYSTEM=Unix to decode)\n");
+			GMT_Report (API, GMT_MSG_VERBOSE, "For binary output, time is stored as seconds since 1970 (Use TIME_SYSTEM=Unix to decode)\n");
 			first_warning = false;
 		}
 		for (kk = kx = pos = 0; pos < n_out_columns; kk++, pos++) {	/* Prepare GMT output formatting machinery */
 			while (kx < n_aux && aux[kx].pos == kk) {	/* Insert formatting for auxiliary column (none are special) */
-				GMT->current.io.col_type[GMT_OUT][pos] = GMT_IS_FLOAT;
+				gmt_set_column (GMT, GMT_OUT, pos, GMT_IS_FLOAT);
 				pos++, kx++;
 			}
 			if (kk >= n_cols_to_process) continue;	/* Don't worry about helper columns that won't be printed */
 			c  = M.order[kk].set;
 			id = M.order[kk].item;
-			if (c == MGD77_M77_SET && id == time_column)	{	/* Special time formatting */
-				GMT->current.io.col_type[GMT_OUT][pos] = M.time_format;
-			}
+			if (c == MGD77_M77_SET && id == time_column)	/* Special time formatting */
+				gmt_set_column (GMT, GMT_OUT, pos, M.time_format);
 			else if (c == MGD77_M77_SET && id == lon_column)	/* Special lon formatting */
-				GMT->current.io.col_type[GMT_OUT][pos] = GMT_IS_LON ;
+				gmt_set_column (GMT, GMT_OUT, pos, GMT_IS_LON);
 			else if (c == MGD77_M77_SET && id == lat_column)	/* Special lat formatting */
-				GMT->current.io.col_type[GMT_OUT][pos] = GMT_IS_LAT;
+				gmt_set_column (GMT, GMT_OUT, pos, GMT_IS_LAT);
 			else 		/* Everything else is float (not true for the 3 strings though but dealt with separately) */
-				GMT->current.io.col_type[GMT_OUT][pos] = GMT_IS_FLOAT;
+				gmt_set_column (GMT, GMT_OUT, pos, GMT_IS_FLOAT);
 		}
 		
 		if (first_cruise && !GMT->common.b.active[GMT_OUT] && GMT->current.setting.io_header[GMT_OUT]) {	/* Write out header record */
@@ -1129,11 +1171,12 @@ int GMT_mgd77list (void *V_API, int mode, void *args) {
 			m1_col = MGD77_Get_Column (GMT, "mtf1",  &M);
 			m2_col = MGD77_Get_Column (GMT, "mtf2",  &M);
 			ms_col = MGD77_Get_Column (GMT, "msens",  &M);
+			md_col = MGD77_Get_Column (GMT, "diur",  &M);
 		}
 		if ((auxlist[MGD77_AUX_GR].requested || (Ctrl->A.code[ADJ_GR] > GR_FAA_STORED)) && Ctrl->A.GF_version == MGD77_NOT_SET) {
 			Ctrl->A.GF_version = D->H.mgd77[use]->Gravity_Theoretical_Formula_Code - '0';
 			if (Ctrl->A.GF_version < MGD77_IGF_HEISKANEN || Ctrl->A.GF_version > MGD77_IGF_1980) {
-				GMT_Report (API, GMT_MSG_NORMAL, "Invalid Gravity Theoretical Formula Code (%c) - default to %d\n", D->H.mgd77[use]->Gravity_Theoretical_Formula_Code, MGD77_IGF_1980);
+				GMT_Report (API, GMT_MSG_VERBOSE, "Invalid Gravity Theoretical Formula Code (%c) - default to %d\n", D->H.mgd77[use]->Gravity_Theoretical_Formula_Code, MGD77_IGF_1980);
 				Ctrl->A.GF_version = MGD77_IGF_1980;
 			}
 		}
@@ -1147,11 +1190,11 @@ int GMT_mgd77list (void *V_API, int mode, void *args) {
 			bool faked = false;
 			if (Ctrl->A.fake_times) {	/* Try to make fake times based on duration and distances */
 				faked = MGD77_fake_times (GMT, &M, &(D->H), dvalue[x_col], dvalue[y_col], dvalue[t_col], D->H.n_records);
-				if (faked) GMT_Report (API, GMT_MSG_VERBOSE, "Warning: Time column for cruise %s created from distances and duration\n", list[argno]);
+				if (faked) GMT_Report (API, GMT_MSG_VERBOSE, "Time column for cruise %s created from distances and duration\n", list[argno]);
 			}
 			if (!faked) {
-				GMT_Report (API, GMT_MSG_VERBOSE, "Warning: Time column not present in cruise %s - set to NaN\n", list[argno]);
-				if (this_limit_on_time) GMT_Report (API, GMT_MSG_VERBOSE, "Warning: -D limits cannot be used for cruise %s\n", list[argno]);
+				GMT_Report (API, GMT_MSG_VERBOSE, "Time column not present in cruise %s - set to NaN\n", list[argno]);
+				if (this_limit_on_time) GMT_Report (API, GMT_MSG_VERBOSE, "-D limits cannot be used for cruise %s\n", list[argno]);
 			}
 			if (!faked && !Ctrl->D.mode) this_limit_on_time = false;	/* To avoid pointless tests against NaN in loop */
 		}
@@ -1399,6 +1442,24 @@ int GMT_mgd77list (void *V_API, int mode, void *args) {
 					k = (i == 2) ? m1_col : m2_col;
 					m = MGD77_Recalc_Mag_Anomaly_IGRF (GMT, &M, date, dvalue[x_col][rec], dvalue[y_col][rec], dvalue[k][rec], false);
 				}
+				if ((Ctrl->A.code[ADJ_MG] & MG_MTF1_PLUS_DIUR_MINUS_IGRF) && gmt_M_is_dnan (m)) {	/* Try mtf 1st + diur - igrf */
+					if (need_date) {	/* Did not get computed already */
+						date = MGD77_time_to_fyear (GMT, &M, dvalue[t_col][rec]);
+						need_date = false;
+					}
+					i = irint (dvalue[ms_col][rec]);
+					k = (i == 2) ? m2_col : m1_col;
+					m = dvalue[md_col][rec] + MGD77_Recalc_Mag_Anomaly_IGRF (GMT, &M, date, dvalue[x_col][rec], dvalue[y_col][rec], dvalue[k][rec], false);
+				}
+				if ((Ctrl->A.code[ADJ_MG] & MG_MTF2_PLUS_DIUR_MINUS_IGRF) && gmt_M_is_dnan (m)) {	/* Try mtf 2nd + diur - igrf */
+					if (need_date) {	/* Did not get computed already */
+						date = MGD77_time_to_fyear (GMT, &M, dvalue[t_col][rec]);
+						need_date = false;
+					}
+					i = irint (dvalue[ms_col][rec]);
+					k = (i == 2) ? m1_col : m2_col;
+					m = dvalue[md_col][rec] + MGD77_Recalc_Mag_Anomaly_IGRF (GMT, &M, date, dvalue[x_col][rec], dvalue[y_col][rec], dvalue[k][rec], false);
+				}
 				if (Ctrl->A.force || !gmt_M_is_dnan(dvalue[m_col][rec])) dvalue[m_col][rec] = m;
 			}
 
@@ -1505,7 +1566,7 @@ int GMT_mgd77list (void *V_API, int mode, void *args) {
 						gmt_cat_to_record (GMT, record, word, GMT_OUT, sep_flag);	/* Format our output x value */
 					}
 					else if (c == MGD77_M77_SET && id == time_column) {	/* Time */
-						if (GMT->current.io.col_type[GMT_OUT][pos] == GMT_IS_FLOAT) {	/* fractional year */
+						if (gmt_M_type (GMT, GMT_OUT, pos) == GMT_IS_FLOAT) {	/* fractional year */
 							if (need_date) {	/* Did not get computed already */
 								date = MGD77_time_to_fyear (GMT, &M, dvalue[t_col][rec]);
 								need_date = false;
@@ -1523,7 +1584,7 @@ int GMT_mgd77list (void *V_API, int mode, void *args) {
 					}
 					sep_flag = 1;
 				}
-				GMT_Put_Record (API, GMT_WRITE_TEXT, record);	/* Write this to output */
+				GMT_Put_Record (API, GMT_WRITE_DATA, Out);	/* Write this to output */
 			}
 			else {	/* Use GMT output machinery which can handle binary output, if requested */
 				for (kk = kx = pos = 0; pos < n_out_columns; kk++, pos++) {
@@ -1535,7 +1596,7 @@ int GMT_mgd77list (void *V_API, int mode, void *args) {
 					c  = M.order[kk].set;
 					id = M.order[kk].item;
 					if (c == MGD77_M77_SET && id == time_column) {	/* This is the time column */
-						if (GMT->current.io.col_type[GMT_OUT][pos] == GMT_IS_FLOAT) {	/* fractional year */
+						if (gmt_M_type (GMT, GMT_OUT, pos) == GMT_IS_FLOAT) {	/* fractional year */
 							if (need_date) {	/* Did not get computed already */
 								date = MGD77_time_to_fyear (GMT, &M, dvalue[t_col][rec]);
 								need_date = false;
@@ -1552,7 +1613,7 @@ int GMT_mgd77list (void *V_API, int mode, void *args) {
 						out[pos] = dvalue[kk][rec] - correction;
 					}
 				}
-				GMT_Put_Record (API, GMT_WRITE_DATA, out);	/* Write this to output */
+				GMT_Put_Record (API, GMT_WRITE_DATA, Out);	/* Write this to output */
 			}
 			n_out++;
 		}
@@ -1574,8 +1635,9 @@ int GMT_mgd77list (void *V_API, int mode, void *args) {
 	if (!string_output) gmt_M_free (GMT, out);
 	gmt_M_free (GMT, aux_tvalue[MGD77_AUX_ID]);
 	gmt_M_free (GMT, aux_tvalue[MGD77_AUX_DA]);
+	gmt_M_free (GMT, Out);
 	
-	GMT_Report (API, GMT_MSG_VERBOSE, "Returned %d output records from %d cruises\n", n_out, n_cruises);
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Returned %d output records from %d cruises\n", n_out, n_cruises);
 	
 	MGD77_Path_Free (GMT, (uint64_t)n_paths, list);
 	if (Ctrl->L.active) MGD77_Free_Correction (GMT, CORR, n_paths);
