@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *    Copyright (c) 2005-2018 by P. Wessel
+ *    Copyright (c) 2005-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  * mgd77manage is used to (1) remove data columns from mgd77+ files
  * or (2) add a new data column to mgd77+ files.  Data can be added
  * from data tables, created from reference field formulas, or by
@@ -22,12 +21,13 @@
 #include "mgd77.h"
 #include "mgd77_e77.h"	/* E77 Header Errata Codes */
 
-#define THIS_MODULE_NAME	"mgd77manage"
+#define THIS_MODULE_CLASSIC_NAME	"mgd77manage"
+#define THIS_MODULE_MODERN_NAME	"mgd77manage"
 #define THIS_MODULE_LIB		"mgd77"
 #define THIS_MODULE_PURPOSE	"Manage the content of MGD77+ files"
 #define THIS_MODULE_KEYS	""
 #define THIS_MODULE_NEEDS	""
-#define THIS_MODULE_OPTIONS "-RVbn"
+#define THIS_MODULE_OPTIONS "-RVbjn"
 
 #define N_PAR		7
 #define COL_SCALE	0
@@ -74,10 +74,6 @@ struct MGD77MANAGE_CTRL {	/* All control options for this program (except common
 		char *file;
 		double parameters[N_PAR];
 	} A;
-	struct MGD77MANAGE_C {	/* -C */
-		bool active;
-		unsigned int mode;
-	} C;
 	struct MGD77MANAGE_D {	/* -D */
 		bool active;
 		char *file;
@@ -113,7 +109,6 @@ GMT_LOCAL void *New_Ctrl (struct GMT_CTRL *GMT) {	/* Allocate and initialize a n
 	C->A.kind = GMT_IS_FLOAT;
 	C->A.parameters[COL_SCALE] = 1.0;	/* Output column scaling */
 	C->A.parameters[IMG_SCALE] = 1.0;	/* IMG data scaling */
-	C->C.mode = 2;
 	C->E.value = '9';
 	C->N.code[0] = 'k';			/* km is default distance unit */
  	return (C);
@@ -127,20 +122,21 @@ GMT_LOCAL void Free_Ctrl (struct GMT_CTRL *GMT, struct MGD77MANAGE_CTRL *C) {	/*
 }
 
 GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
-	gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
+	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: mgd77manage <cruise(s)> [-A[+]a|c|d|D|e|E|g|i|n|t|T<info>] [-Cf|g|e] [-D<name1>,<name2>,...]\n");
+	GMT_Message (API, GMT_TIME_NONE, "usage: %s <cruise(s)> [-Aa|c|d|D|e|E|g|i|n|t|T<info>[+f]] [-D<name1>,<name2>,...]\n", name);
 	GMT_Message (API, GMT_TIME_NONE, "\t[-E<no_char>] [-F] [-I<abbrev>/<name>/<units>/<size>/<scale>/<offset>/\"comment\"]\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t[-N%s[+|-]] [%s] [%s] [%s]\n\t[%s]\n\n",
-	             GMT_LEN_UNITS2_DISPLAY, GMT_Rgeo_OPT, GMT_V_OPT, GMT_bi_OPT, GMT_n_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "\t[-N%s] [%s] [%s] [%s]\n\t[%s] [%s] [%s]\n\n",
+	             GMT_LEN_UNITS2_DISPLAY, GMT_Rgeo_OPT, GMT_V_OPT, GMT_bi_OPT, GMT_j_OPT, GMT_n_OPT, GMT_PAR_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
 	MGD77_Cruise_Explain (API->GMT);
 	GMT_Message (API, GMT_TIME_NONE, "\n\tOPTIONS:\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-A Append a new data column to the given files.  The code letters are:\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   +: Optional.  Will overwrite an existing column with same name with new data.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   [Default will refuse if an existing column has the same abbreviation as the new data].\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t-A Append a new data column to the given files.  Append +f to overwrite an\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   existing column with same name with new data [Default will refuse if an\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   existing column has the same abbreviation as the new data].\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   The code letters are:\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   a: Give filename with a new column to add.  We expect a single-column file\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t      with the same number of records as the MGD77 file.  Only one cruise can be set.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t      If filename is - we read from stdin.\n");
@@ -177,12 +173,8 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t      with dateTclock strings in first column and data values in 2nd.  Only one cruise can be set.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t      If filename is - we read from stdin.  Only records with matching times will have data assigned.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   T: Same as t but we interpolate between the time, data pairs to fill in all data records.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-C Append code for distance calculation procedure (when -Ad|D is set):\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     f Flat Earth.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     g Great circle [Default].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     e Ellipsoidal (geodesic) using current GMT ellipsoid.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-D Delete the columns listed from all the cruise data files.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   The columns are removed before any data are added.  It is not a substitute for -A+.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   The columns are removed before any data are added.  It is not a substitute for -A...+f.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   However, sometimes the shape of new data demands the old to be deleted first (you will be told).\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-E Give character used to fill empty/missing string columns [9]\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-F Force mode.  This allows you to even replace the standard MGD77 columns [only extended columns can be changed].\n");
@@ -198,8 +190,8 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t   Note for text: Interpolation is not allowed, and \"not-a-string\" is created from -E.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-N Append your choice for distance unit (if -Ad|D are set). Choose among:\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   m(e)ter, (f)oot, (k)m, (M)ile, (n)autical mile, or s(u)rvey foot [Default is -Nk].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t    See -C for selecting distance calculation procedure.\n");
-	GMT_Option (API, "Rg,V,bi,di,n,.");
+	GMT_Message (API, GMT_TIME_NONE, "\t    See -j for selecting distance calculation procedure.\n");
+	GMT_Option (API, "Rg,V,bi,di,j,n,.");
 	
 	return (GMT_MODULE_USAGE);
 }
@@ -208,16 +200,19 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 
 GMT_LOCAL int decode_A_options (int mode, char *line, char *file, double parameters[]) {
 	int error = 0, n;
+	char *c = strstr (line, "+f");
 	
+	if (c) c[0] = '\0';	/* Chop off modifier */
 	if (mode == 1) {	/* For *.img files since we need to know data scale and grid mode */
-		/* -A[+]i<filename>,<scale>/<mode>[/<lat>] */
+		/* -Ai<filename>,<scale>/<mode>[/<lat>][+f] */
 		n = sscanf (line, "%[^,],%lf,%lf,%lf", file, &parameters[IMG_SCALE], &parameters[IMG_MODE], &parameters[IMG_LAT]);
 		if (n < 3) error = 1;
 	}
 	else {	/* GMT grid or table: No data scale and mode to worry about */
-		/* -A[+]a|c|d|D|e|g|n|t|T<filename> */
+		/* -Aa|c|d|D|e|g|n|t|T<filename>[+f] */
 		strcpy (file, line);
 	}
+	if (c) c[0] = '+';	/* Chop off modifier */
 	
 	return (error);
 }
@@ -327,7 +322,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct MGD77MANAGE_CTRL *Ctrl, struct
 	unsigned int n_errors = 0, k, n_cruises = 0;
 	bool got_table, got_grid, strings;
 	nc_type c_nc_type;
-	char file[GMT_BUFSIZ] = {""};
+	char file[PATH_MAX] = {""}, *c = NULL;
 	struct GMT_OPTION *opt = NULL;
 	struct GMTAPI_CTRL *API = GMT->parent;
 	
@@ -344,10 +339,14 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct MGD77MANAGE_CTRL *Ctrl, struct
 			case 'A':	/* Adding a new column */
 				Ctrl->A.active = true;
 				k = 0;
-				if (opt->arg[k] == '+') {
+				if (opt->arg[k] == '+') {	/* Deprecated way of doing _f */
 					Ctrl->A.replace = true;
 					k++;
 				}
+				else if ((c = strstr (opt->arg, "+f"))) {
+					Ctrl->A.replace = true;
+					c[0] = '\0';	/* Chop off modifier */
+				}
 				switch (opt->arg[k]) {
 					case 'a':	/* Plain column data file of exact same # of records */
 						Ctrl->A.mode = MODE_a;
@@ -359,12 +358,14 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct MGD77MANAGE_CTRL *Ctrl, struct
 						break;
 					case 'D':	/* dist,val data file - interpolate to get values at all records */
 						Ctrl->A.interpolate = true;
+						/* Fall through on purpose to 'd' */
 					case 'd':	/* dist,val data file - only update records with matching distances */
 						Ctrl->A.mode = MODE_d;
 						n_errors += decode_A_options (0, &opt->arg[k+1], file, Ctrl->A.parameters);
 						break;
 					case 'E':	/* Plain E77 error flag file from mgd77sniffer */
 						Ctrl->A.ignore_verify = true;	/* Process raw e77 files that have not been verified */
+						/* Fall through on purpose to 'e' */
 					case 'e':	/* Plain E77 error flag file from mgd77sniffer */
 						Ctrl->A.mode = MODE_e;
 						while (opt->arg[++k]) {
@@ -385,7 +386,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct MGD77MANAGE_CTRL *Ctrl, struct
 									Ctrl->A.e77_skip_mode[E77_SLOPES_MODE] = true;
 									break;
 								default:
-									GMT_Report (API, GMT_MSG_NORMAL, "Error: -Ae modifiers must be combination of hfnvs\n");
+									GMT_Report (API, GMT_MSG_NORMAL, "-Ae modifiers must be combination of hfnvs\n");
 									n_errors++;
 									break;
 							}
@@ -405,26 +406,35 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct MGD77MANAGE_CTRL *Ctrl, struct
 						break;
 					case 'T':	/* abstime,val data file - interpolate to get values at all records */
 						Ctrl->A.interpolate = true;
+						/* Fall through on purpose to 't' */
 					case 't':	/* abstime,val data file - only update records with matching times */
 						Ctrl->A.mode = MODE_t;
 						Ctrl->A.kind = GMT_IS_ABSTIME;
 						n_errors += decode_A_options (0, &opt->arg[k+1], file, Ctrl->A.parameters);
 						break;
 					default:
-						GMT_Report (API, GMT_MSG_NORMAL, "Error: -A modifier must be a|c|d|D|e|g|i|n|t|T\n");
+						GMT_Report (API, GMT_MSG_NORMAL, "-A modifier must be a|c|d|D|e|g|i|n|t|T\n");
 						n_errors++;
 						break;
 				}
 				if (strlen (file)) Ctrl->A.file = strdup (file);
+				if (c) c[0] = '+';	/* Restore modifier */
 				break;
 
 			case 'C':	/* Distance calculation method */
-				Ctrl->C.active = true;
-				if (opt->arg[0] == 'f') Ctrl->C.mode = 1;
-				if (opt->arg[0] == 'g') Ctrl->C.mode = 2;
-				if (opt->arg[0] == 'e') Ctrl->C.mode = 3;
-				if (Ctrl->C.mode < 1 || Ctrl->C.mode > 3) {
-					GMT_Report (API, GMT_MSG_NORMAL, "Error -C: Flag must be f, g, or e\n");
+				if (gmt_M_compat_check (API->GMT, 6)) {
+					GMT_Report (API, GMT_MSG_COMPAT, "The -C option is deprecated; use -j<mode> instead\n");
+					GMT->common.j.active = true;
+					if (opt->arg[0] == 'f') GMT->common.j.mode = GMT_FLATEARTH;
+					if (opt->arg[0] == 'g') GMT->common.j.mode = GMT_GREATCIRCLE;
+					if (opt->arg[0] == 'e') GMT->common.j.mode = GMT_GEODESIC;
+					if (GMT->common.j.mode < 1 || GMT->common.j.mode > 3) {
+						GMT_Report (API, GMT_MSG_NORMAL, "Error -C: Flag must be f, g, or e\n");
+						n_errors++;
+					}
+				}
+				else {
+					GMT_Report (API, GMT_MSG_NORMAL, "Unrecognized option -C\n");
 					n_errors++;
 				}
 				break;
@@ -537,12 +547,11 @@ int GMT_mgd77manage (void *V_API, int mode, void *args) {
 	if (mode == GMT_MODULE_PURPOSE) return (usage (API, GMT_MODULE_PURPOSE));	/* Return the purpose of program */
 	options = GMT_Create_Options (API, mode, args);	if (API->error) return (API->error);	/* Set or get option list */
 
-	if (!options || options->option == GMT_OPT_USAGE) bailout (usage (API, GMT_USAGE));	/* Return the usage message */
-	if (options->option == GMT_OPT_SYNOPSIS) bailout (usage (API, GMT_SYNOPSIS));	/* Return the synopsis */
+	if ((error = gmt_report_usage (API, options, 0, usage)) != GMT_NOERROR) bailout (error);	/* Give usage if requested */
 
 	/* Parse the command-line arguments */
 
-	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
 	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
 	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
 	if ((error = parse (GMT, Ctrl, options)) != 0) Return (error);
@@ -564,12 +573,13 @@ int GMT_mgd77manage (void *V_API, int mode, void *args) {
 	n_paths = MGD77_Path_Expand (GMT, &In, options, &list);	/* Get list of requested IDs */
 
 	if (n_paths <= 0) {
-		GMT_Report (API, GMT_MSG_NORMAL, "Error: No cruises given\n");
+		GMT_Report (API, GMT_MSG_NORMAL, "No cruises given\n");
+		MGD77_Path_Free (GMT, (uint64_t)n_paths, list);
 		Return (GMT_NO_INPUT);
 	}
 
 	if (got_table && n_paths != 1) {
-		GMT_Report (API, GMT_MSG_NORMAL, "Error: With -Aa|d|D|n|t|T you can only select one cruise at the time.\n");
+		GMT_Report (API, GMT_MSG_NORMAL, "With -Aa|d|D|n|t|T you can only select one cruise at the time.\n");
 		MGD77_Path_Free (GMT, (uint64_t)n_paths, list);
 		Return (GMT_PARSE_ERROR);
 	}
@@ -599,7 +609,8 @@ int GMT_mgd77manage (void *V_API, int mode, void *args) {
 			MTF_col = mfield;
 		}
 		else {
-			GMT_Report (API, GMT_MSG_NORMAL, "Error: -Ac expects m, c, or g[1-4]\n");
+			GMT_Report (API, GMT_MSG_NORMAL, "-Ac expects m, c, or g[1-4]\n");
+			MGD77_Path_Free (GMT, (uint64_t)n_paths, list);
 			Return (GMT_PARSE_ERROR);
 		}
 	}
@@ -612,6 +623,7 @@ int GMT_mgd77manage (void *V_API, int mode, void *args) {
 		gmt_M_memset (wesn, 4, double);
 		if (GMT->common.R.active[RSET]) gmt_M_memcpy (wesn, GMT->common.R.wesn, 4, double);	/* Current -R setting for subset */
 		if ((G = GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_DATA_ONLY, wesn, Ctrl->A.file, NULL)) == NULL) {	/* Get data */
+			MGD77_Path_Free (GMT, (uint64_t)n_paths, list);
 			Return (API->error);
 		}
 		interpolate = (GMT->common.n.threshold > 0.0);
@@ -643,6 +655,7 @@ int GMT_mgd77manage (void *V_API, int mode, void *args) {
 			}
 		}
 
+		GMT->current.io.first_rec = true;
 		if (GMT->current.setting.io_header[GMT_IN]) {	/* Skip any header records */
 			for (i = 0; i < (int)GMT->current.setting.io_n_header_items; i++) {
 				if (!gmt_fgets (GMT, line, GMT_BUFSIZ, fp)) {
@@ -661,13 +674,13 @@ int GMT_mgd77manage (void *V_API, int mode, void *args) {
 		colvalue = gmt_M_memory (GMT, NULL, n_alloc, double);
 		if (two_cols) {	/* Got an abscissae column as well (dnt: d = dist, n = rec number, t = time) */
 			coldnt = gmt_M_memory (GMT, NULL, n_alloc, double);
-			GMT->current.io.col_type[GMT_IN][GMT_X] = Ctrl->A.kind;
+			gmt_set_column (GMT, GMT_IN, GMT_X, Ctrl->A.kind);
 		}
 		if (strings && !two_cols) {	/* Must read strings directly from file since GMT->current.io.input would barf */
 			ok_to_read = false;
 			tmp_string = gmt_M_memory (GMT, NULL, n_alloc, char *);
 			while (gmt_fgets (GMT, word, GMT_BUFSIZ, fp)) {
-				if (word[0] == '#') continue;
+				if (word[0] == GMT->current.setting.io_head_marker[GMT_IN]) continue;
 				width = (int)strlen (word);
 				tmp_string[n] = gmt_M_memory (GMT, NULL, width + 1, char);
 				strcpy (tmp_string[n], word);
@@ -702,7 +715,7 @@ int GMT_mgd77manage (void *V_API, int mode, void *args) {
 
 			if (strings) {	/* number in col1, string in col2 */
 				coldnt[n]   = in[0];
-				sscanf (GMT->current.io.record, "%*s %s", word);
+				sscanf (GMT->current.io.curr_text, "%*s %s", word);
 				tmp_string[n] = gmt_M_memory (GMT, NULL, strlen(word) + 1, char);
 				strcpy (tmp_string[n], word);
 			}
@@ -751,7 +764,7 @@ int GMT_mgd77manage (void *V_API, int mode, void *args) {
 	
 		if (MGD77_Open_File (GMT, list[argno], &In, MGD77_READ_MODE)) continue;
 				
-		GMT_Report (API, GMT_MSG_VERBOSE, "Now processing cruise %s\n", list[argno]);
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Now processing cruise %s\n", list[argno]);
 		
 		D = MGD77_Create_Dataset (GMT);
 		In.n_out_columns = 0;
@@ -785,7 +798,7 @@ int GMT_mgd77manage (void *V_API, int mode, void *args) {
 		if (Ctrl->D.active) {	/* Must create a new file with everything except the fields to be deleted */
 			int id, c;
 			bool reset_column = false;
-			char oldfile[GMT_BUFSIZ] = {""};
+			char oldfile[PATH_MAX+4] = {""};
 			
 			if (column != MGD77_NOT_SET) {	/* Get info about this existing column to see if it is compatible with new data */
 				n_dims = (D->H.info[In.order[column].set].col[In.order[column].item].constant) ? 0 : 1;
@@ -831,7 +844,7 @@ int GMT_mgd77manage (void *V_API, int mode, void *args) {
 			/* Rename the old file for now */
 			
 			sprintf (oldfile, "%s.old", In.path);
-			if (gmt_rename_file (GMT, In.path, oldfile)) {
+			if (gmt_rename_file (GMT, In.path, oldfile, GMT_RENAME_FILE)) {
 				GMT_exit (GMT, GMT_RUNTIME_ERROR); return GMT_RUNTIME_ERROR;
 			}
 			
@@ -891,7 +904,7 @@ int GMT_mgd77manage (void *V_API, int mode, void *args) {
 				colvalue[rec] = (MGD77_igrf10syn (GMT, 0, date, 1, 0.0, xvar[rec], yvar[rec], IGRF)) ? GMT->session.d_NaN : IGRF[MGD77_IGRF_F];
 				n_sampled++;
 			}
-			GMT_Report (API, GMT_MSG_VERBOSE, "Estimated IGRF at %d locations out of %d for cruise %s\n", n_sampled, D->H.n_records, list[argno]);
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Estimated IGRF at %d locations out of %d for cruise %s\n", n_sampled, D->H.n_records, list[argno]);
 		}
 		else if (c_kind == ADD_GRAV) {	/* Append IGF column */
 			int ix, iy, use;
@@ -904,7 +917,7 @@ int GMT_mgd77manage (void *V_API, int mode, void *args) {
 				use = (In.original) ? MGD77_ORIG : MGD77_REVISED;
 				GF_version = D->H.mgd77[use]->Gravity_Theoretical_Formula_Code - '0';
 				if (GF_version < MGD77_IGF_HEISKANEN || GF_version > MGD77_IGF_1980) {
-					GMT_Report (API, GMT_MSG_NORMAL, "Invalid Gravity Theoretical Formula Code (%c) - default to %d\n",
+					GMT_Report (API, GMT_MSG_VERBOSE, "Invalid Gravity Theoretical Formula Code (%c) - default to %d\n",
 					            D->H.mgd77[use]->Gravity_Theoretical_Formula_Code, MGD77_IGF_1980);
 					GF_version = MGD77_IGF_1980;
 				}
@@ -914,7 +927,7 @@ int GMT_mgd77manage (void *V_API, int mode, void *args) {
 			colvalue = gmt_M_memory (GMT, NULL, D->H.n_records, double);
 			
 			for (rec = 0; rec < D->H.n_records; rec++) colvalue[rec] = MGD77_Theoretical_Gravity (GMT, xvar[rec], yvar[rec], GF_version);
-			GMT_Report (API, GMT_MSG_VERBOSE, "Estimated IGRF at %d locations out of %d for cruise %s\n",
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Estimated IGRF at %d locations out of %d for cruise %s\n",
 			            D->H.n_records, D->H.n_records, list[argno]);
 		}
 		else if (c_kind == ADD_CARTER) {	/* Append Carter correction column */
@@ -931,7 +944,7 @@ int GMT_mgd77manage (void *V_API, int mode, void *args) {
 			colvalue = gmt_M_memory (GMT, NULL, D->H.n_records, double);
 			
 			for (rec = 0; rec < D->H.n_records; rec++) colvalue[rec] = MGD77_carter_correction (GMT, xvar[rec], yvar[rec], 1000.0 * tvar[rec], &Carter);
-			GMT_Report (API, GMT_MSG_VERBOSE, "Estimated IGRF at %d locations out of %d for cruise %s\n", D->H.n_records, D->H.n_records, list[argno]);
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Estimated IGRF at %d locations out of %d for cruise %s\n", D->H.n_records, D->H.n_records, list[argno]);
 		}
 		else if (c_kind == ADD_RMAG) {	/* Append recomputed residual mag column */
 			int ix, iy, it, im;
@@ -956,12 +969,13 @@ int GMT_mgd77manage (void *V_API, int mode, void *args) {
 				colvalue[rec] = (check) ? GMT->session.d_NaN : mvar[rec] - IGRF[MGD77_IGRF_F];
 				n_sampled++;
 			}
-			GMT_Report (API, GMT_MSG_VERBOSE, "Estimated recomputed magnetic anomaly at %d locations out of %d for cruise %s\n",
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Estimated recomputed magnetic anomaly at %d locations out of %d for cruise %s\n",
 			            n_sampled, D->H.n_records, list[argno]);
 		}
 		else if (got_grid) {	/* Sample grid along track (or Mercator-projected) track */
 			int ix, iy;
 			double *xvar = NULL, *yvar = NULL;
+			struct GMT_GRID_HEADER_HIDDEN *HH = gmt_get_H_hidden (G->header);
 			
 			if ((ix = skip_if_missing (GMT, "lon", list[argno], &In, &D)) == MGD77_NOT_SET) continue;
 			if ((iy = skip_if_missing (GMT, "lat", list[argno], &In, &D)) == MGD77_NOT_SET) continue;
@@ -983,10 +997,10 @@ int GMT_mgd77manage (void *V_API, int mode, void *args) {
 				}
 				if (y < G->header->wesn[YLO] || y > G->header->wesn[YHI]) continue;
 
-				while ((x < G->header->wesn[XLO]) && (G->header->nxp > 0)) x += (G->header->inc[GMT_X] * G->header->nxp);
+				while ((x < G->header->wesn[XLO]) && (HH->nxp > 0)) x += (G->header->inc[GMT_X] * HH->nxp);
 				if (x < G->header->wesn[XLO]) continue;
 
-				while ((x > G->header->wesn[XHI]) && (G->header->nxp > 0)) x -= (G->header->inc[GMT_X] * G->header->nxp);
+				while ((x > G->header->wesn[XHI]) && (HH->nxp > 0)) x -= (G->header->inc[GMT_X] * HH->nxp);
 				if (x > G->header->wesn[XHI]) continue;
 
 				if (interpolate) {	/* IMG has been corrected, and GRD is good to go */
@@ -999,7 +1013,7 @@ int GMT_mgd77manage (void *V_API, int mode, void *args) {
 				}
 				n_sampled++;
 			}
-			GMT_Report (API, GMT_MSG_VERBOSE, "Sampled grid at %d locations out of %d for cruise %s\n", n_sampled, D->H.n_records, list[argno]);
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Sampled grid at %d locations out of %d for cruise %s\n", n_sampled, D->H.n_records, list[argno]);
 		}
 		else if (Ctrl->A.mode == MODE_a) {	/* Just got a single column to paste in, assuming the row numbers match */
 			if (n != D->H.n_records) {
@@ -1007,7 +1021,7 @@ int GMT_mgd77manage (void *V_API, int mode, void *args) {
 				            n, D->H.n_records, list[argno]);
 				GMT_exit (GMT, GMT_RUNTIME_ERROR); return GMT_RUNTIME_ERROR;
 			}
-			GMT_Report (API, GMT_MSG_VERBOSE, "Appended column data for all %d records for cruise %s\n", D->H.n_records, list[argno]);
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Appended column data for all %d records for cruise %s\n", D->H.n_records, list[argno]);
 		}
 		else if (Ctrl->A.mode == MODE_d || Ctrl->A.mode == MODE_n || Ctrl->A.mode == MODE_t) {	/* Got either (time,data) or (dist,data) */
 			int ix, iy, it;
@@ -1019,7 +1033,7 @@ int GMT_mgd77manage (void *V_API, int mode, void *args) {
 				if ((iy = skip_if_missing (GMT, "lat", list[argno], &In, &D)) == MGD77_NOT_SET) continue;
 				x = D->values[ix];
 				y = D->values[iy];
-				if ((d = gmt_dist_array_2 (GMT, x, y, D->H.n_records, dist_scale, Ctrl->C.mode)) == NULL)
+				if ((d = gmt_dist_array_2 (GMT, x, y, D->H.n_records, dist_scale, GMT->common.h.mode)) == NULL)
 					gmt_M_err_fail (GMT, GMT_MAP_BAD_DIST_FLAG, "");
 				x = d;
 			}
@@ -1049,14 +1063,14 @@ int GMT_mgd77manage (void *V_API, int mode, void *args) {
 					}
 				}
 				gmt_M_free (GMT, tmp_string);
-				GMT_Report (API, GMT_MSG_VERBOSE, "Appended column data for %d locations out of %d for cruise %s\n",
+				GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Appended column data for %d locations out of %d for cruise %s\n",
 				            n_sampled, D->H.n_records, list[argno]);
 			}
 			else if (strings) {	/* One to one match */
 				text = gmt_M_memory (GMT, NULL, D->H.n_records * LEN_size, char);
 				for (rec = 0; rec < n; rec++) strncpy (&text[rec*LEN_size], tmp_string[rec], LEN_size);
 				gmt_M_free (GMT, tmp_string);
-				GMT_Report (API, GMT_MSG_VERBOSE, "Appended column data for %d locations out of %d for cruise %s\n",
+				GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Appended column data for %d locations out of %d for cruise %s\n",
 				            n_sampled, D->H.n_records, list[argno]);
 			}
 			else {	/* Only update the exact matching records */
@@ -1071,7 +1085,7 @@ int GMT_mgd77manage (void *V_API, int mode, void *args) {
 					}
 				}
 				gmt_M_memcpy (colvalue, y, D->H.n_records, double);
-				GMT_Report (API, GMT_MSG_VERBOSE, "Appended column data for %d locations out of %d for cruise %s\n",
+				GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Appended column data for %d locations out of %d for cruise %s\n",
 				            n_sampled, D->H.n_records, list[argno]);
 				gmt_M_free (GMT, y);
 			}
@@ -1084,7 +1098,7 @@ int GMT_mgd77manage (void *V_API, int mode, void *args) {
 			 */
 			FILE *fp_e = NULL;
 			int cdf_var_id, cdf_adjust;
-			char ID[16] = {""}, date[16] = {""}, field[GMT_LEN64] = {""}, efile[GMT_BUFSIZ] = {""};
+			char ID[16] = {""}, date[16] = {""}, field[GMT_LEN64] = {""}, efile[PATH_MAX] = {""};
 			char E77[256] = {""}, timestamp[GMT_LEN64] = {""}, answer[GMT_BUFSIZ] = {""}, code[GMT_BUFSIZ] = {""}, kind, YorN;
 			int number, type, it, id, key, n_E77_flags, day, month, year, item;
 			int n_E77_headers, n_E77_scales, n_E77_offsets, n_E77_recalcs, n_unprocessed, e_error = 0, old_flags;
@@ -1105,7 +1119,7 @@ int GMT_mgd77manage (void *V_API, int mode, void *args) {
 			if ((fp_e = gmt_fopen (GMT, efile, "r")) == NULL) {	/* Not in current directory, try MGD77_HOME/E77 */
 				sprintf (efile, "%s/E77/%s.e77", In.MGD77_HOME, list[argno]);
 				if ((fp_e = gmt_fopen (GMT, efile, "r")) == NULL) {	/* Not here either */
-					GMT_Report (API, GMT_MSG_NORMAL, "Error: The file %s.e77 could not be found in current directory or in MGD77_HOME/E77 - skipped\n", list[argno]);
+					GMT_Report (API, GMT_MSG_NORMAL, "The file %s.e77 could not be found in current directory or in MGD77_HOME/E77 - skipped\n", list[argno]);
 					MGD77_Free_Dataset (GMT, &D);	/* Free memory allocated by MGD77_Read_File for this aborted effort */
 					continue;
 				}
@@ -1117,12 +1131,12 @@ int GMT_mgd77manage (void *V_API, int mode, void *args) {
 			 
 			P = D->H.mgd77[MGD77_ORIG];	/* Because E77 is absolute and not incremental we start from original settings */
 			if (!gmt_fgets (GMT, line, GMT_BUFSIZ, fp_e)) {
-				GMT_Report (API, GMT_MSG_NORMAL, "Error: Could not read record #1 from %s.e77 - aborting\n", list[argno]);
+				GMT_Report (API, GMT_MSG_NORMAL, "Could not read record #1 from %s.e77 - aborting\n", list[argno]);
 				e_error++;
 			}
 			sscanf (&line[1], "%*s %s %*s %*s %*s %*s %*s %s %*s %" SCNu64, ID, date, &n_recs);
 			if (strcmp (In.NGDC_id, ID)) {
-				GMT_Report (API, GMT_MSG_NORMAL, "Error: E77 Conflict %s : ID = %s versus %s - aborting\n", efile, ID, In.NGDC_id);
+				GMT_Report (API, GMT_MSG_NORMAL, "E77 Conflict %s : ID = %s versus %s - aborting\n", efile, ID, In.NGDC_id);
 				e_error++;
 			}
 			/* Make sure the File creation dates from the data file and the E77 match */
@@ -1133,12 +1147,12 @@ int GMT_mgd77manage (void *V_API, int mode, void *args) {
 			year = atoi (date);
 			
 			if (!(year == atoi (P->File_Creation_Year) && month == atoi (P->File_Creation_Month) && day == atoi (P->File_Creation_Day))) {
-				GMT_Report (API, GMT_MSG_NORMAL, "Error: E77 Conflict %s: File Creation Date: %s versus %s%s%s - aborting\n", efile, date,
+				GMT_Report (API, GMT_MSG_NORMAL, "E77 Conflict %s: File Creation Date: %s versus %s%s%s - aborting\n", efile, date,
 					P->File_Creation_Year, P->File_Creation_Month, P->File_Creation_Day);
 				e_error++;
 			}
 			if (n_recs != D->H.n_records) {
-				GMT_Report (API, GMT_MSG_NORMAL, "Error: E77 Conflict %s: n_recs = %d versus %d = aborting\n", efile, n_recs, D->H.n_records);
+				GMT_Report (API, GMT_MSG_NORMAL, "E77 Conflict %s: n_recs = %d versus %d = aborting\n", efile, n_recs, D->H.n_records);
 				e_error++;
 			}
 			verified = false;
@@ -1148,12 +1162,12 @@ int GMT_mgd77manage (void *V_API, int mode, void *args) {
 				if (!strncmp (line, "Y Errata table verification status", 34U)) verified = true;
 			}
 			if (!verified && !Ctrl->A.ignore_verify) {
-				GMT_Report (API, GMT_MSG_NORMAL, "Error: E77 file %s not yet verified.  E77 not applied\n", efile);
+				GMT_Report (API, GMT_MSG_NORMAL, "E77 file %s not yet verified.  E77 not applied\n", efile);
 				e_error++;
 			}
 			
 			if (e_error) {
-				GMT_Report (API, GMT_MSG_NORMAL, "Error: The file %s has too many errors.  E77 not applied\n", efile);
+				GMT_Report (API, GMT_MSG_NORMAL, "The file %s has too many errors.  E77 not applied\n", efile);
 				MGD77_Free_Dataset (GMT, &D);	/* Free memory allocated by MGD77_Read_File for this aborted effort */
 				continue;
 			}
@@ -1176,19 +1190,19 @@ int GMT_mgd77manage (void *V_API, int mode, void *args) {
 				else				/* Data record */
 					sscanf (line, "%c %s %s %" SCNu64 " %s", &YorN, ID, timestamp, &rec, code);
 				if (strcmp (In.NGDC_id, ID)) {
-					GMT_Report (API, GMT_MSG_NORMAL, "Error: E77 Conflict %s : ID = %s versus %s in header records!\n", efile, ID, In.NGDC_id);
+					GMT_Report (API, GMT_MSG_NORMAL, "E77 Conflict %s : ID = %s versus %s in header records!\n", efile, ID, In.NGDC_id);
 					e_error++;
 				}
 			}
 			
 			if (e_error) {
-				GMT_Report (API, GMT_MSG_NORMAL, "Error: The file %s has too many errors.  E77 not applied\n", efile);
+				GMT_Report (API, GMT_MSG_NORMAL, "The file %s has too many errors.  E77 not applied\n", efile);
 				gmt_fclose (GMT, fp_e);
 				MGD77_Free_Dataset (GMT, &D);	/* Free memory allocated by MGD77_Read_File for this aborted effort */
 				continue;
 			}
 			if (n_unprocessed) {
-				GMT_Report (API, GMT_MSG_NORMAL, "Error: The file %s has unprocessed E77 recommendations.  E77 not applied\n", efile);
+				GMT_Report (API, GMT_MSG_NORMAL, "The file %s has unprocessed E77 recommendations.  E77 not applied\n", efile);
 				gmt_fclose (GMT, fp_e);
 				MGD77_Free_Dataset (GMT, &D);	/* Free memory allocated by MGD77_Read_File for this aborted effort */
 				continue;
@@ -1213,7 +1227,7 @@ int GMT_mgd77manage (void *V_API, int mode, void *args) {
 				*/
 				sscanf (line, "%c-%c-%[^-]-%[^-]-%d", &YorN, &kind, ID, field, &item);
 				if (strcmp (In.NGDC_id, ID)) {
-					GMT_Report (API, GMT_MSG_NORMAL, "Error: E77 Conflict %s : ID = %s versus %s in header records - skipped\n", efile, ID, In.NGDC_id);
+					GMT_Report (API, GMT_MSG_NORMAL, "E77 Conflict %s : ID = %s versus %s in header records - skipped\n", efile, ID, In.NGDC_id);
 					e_error++;
 					continue;
 				}
@@ -1273,7 +1287,7 @@ int GMT_mgd77manage (void *V_API, int mode, void *args) {
 									GMT_Message (API, GMT_TIME_NONE, "Warning: Correction implied for %s which is not in this cruise?\n", field);
 									break;
 								}
-								/* no break - we want to fall through and also set depth adjustment */
+								/* No break! - we want to fall through and also set depth adjustment */
 							case E77_HDR_CARTER:	/* Recalculate Carter depth from twt */
 								cdf_adjust = MGD77_COL_ADJ_DEPTH;
 								MGD77_nc_status (GMT, nc_put_att_int (In.nc_id, D->H.info[set].col[id].var_id, "adjust", NC_INT, 1U, &cdf_adjust));
@@ -1314,7 +1328,7 @@ int GMT_mgd77manage (void *V_API, int mode, void *args) {
 							case E77_HDR_FLAGRANGE:		/* Range of bad values - set flags to BAD  */
 								sscanf (answer, "%" SCNu64 "-%" SCNu64, &from, &to);
 								if (from < 1 || from > D->H.n_records || to < 1 || to > D->H.n_records || to < from) {
-									GMT_Report (API, GMT_MSG_NORMAL, "Error: Record range %s is invalid.  Correction skipped\n", answer);
+									GMT_Report (API, GMT_MSG_NORMAL, "Record range %s is invalid.  Correction skipped\n", answer);
 									break;
 								}
 								pattern = mgd77_set_bit (id);
@@ -1336,7 +1350,7 @@ int GMT_mgd77manage (void *V_API, int mode, void *args) {
 			while (gmt_fgets (GMT, line, GMT_BUFSIZ, fp_e)) {	/* Read until EOF */
 				sscanf (line, "%c %s %s %" SCNu64 " %s", &YorN, ID, timestamp, &rec, code);
 				if (strcmp (In.NGDC_id, ID)) {
-					GMT_Report (API, GMT_MSG_NORMAL, "Error: E77 Conflict %s : ID = %s versus %s in data records - skipped\n", efile, ID, In.NGDC_id);
+					GMT_Report (API, GMT_MSG_NORMAL, "E77 Conflict %s : ID = %s versus %s in data records - skipped\n", efile, ID, In.NGDC_id);
 					e_error++;
 					continue;
 				}
@@ -1353,12 +1367,12 @@ int GMT_mgd77manage (void *V_API, int mode, void *args) {
 					}
 					else {	/* Must try to interpret the timestamp */
 						if (gmt_verify_expectations (GMT, GMT_IS_ABSTIME, gmt_scanf (GMT, timestamp, GMT_IS_ABSTIME, &rec_time), timestamp)) {
-							GMT_Report (API, GMT_MSG_NORMAL, "Error: %s: E77 time stamp (%s) in wrong format? - skipped\n", ID, timestamp);
+							GMT_Report (API, GMT_MSG_NORMAL, "%s: E77 time stamp (%s) in wrong format? - skipped\n", ID, timestamp);
 							continue;
 						}
 						del_t = fabs (tvar[rec] - rec_time);
 						if (del_t > (0.06 + GMT_CONV8_LIMIT)) {	/* 0.06 is finest time step in MGD77 file so we allow that much slop */
-							GMT_Report (API, GMT_MSG_NORMAL, "Error: %s: E77 time stamp and record number do not match record time (del_t = %g s) - skipped\n", ID, del_t);
+							GMT_Report (API, GMT_MSG_NORMAL, "%s: E77 time stamp and record number do not match record time (del_t = %g s) - skipped\n", ID, del_t);
 							continue;
 						}
 					}
@@ -1443,7 +1457,7 @@ int GMT_mgd77manage (void *V_API, int mode, void *args) {
 				if (old_flags) {	/* Flag variable exists already - simply replace existing flags with the new ones */
 					if (D->flags[0])	/* Was allocated and read */
 						gmt_M_memcpy (D->flags[0], flags, D->H.n_records, int);
-					else	/* Was not allcoated */
+					else	/* Was not allocated */
 						D->flags[0] = flags;
 				}
 				else {	/* We need to define the flags for the first time */
@@ -1603,24 +1617,24 @@ int GMT_mgd77manage (void *V_API, int mode, void *args) {
 		if (n_bad) {	/* Report what we found */
 			if (In.verbose_level & 1)
 				fprintf (fp_err, "%s: %s [%s] had %" PRIu64 " values outside valid range <%g,%g> for the chosen type (set to NaN = %g)\n",
-				THIS_MODULE_NAME, In.NGDC_id, Ctrl->I.c_abbrev, n_bad, MGD77_Low_val[c_nc_type], MGD77_High_val[c_nc_type], MGD77_NaN_val[c_nc_type]);
+				THIS_MODULE_CLASSIC_NAME, In.NGDC_id, Ctrl->I.c_abbrev, n_bad, MGD77_Low_val[c_nc_type], MGD77_High_val[c_nc_type], MGD77_NaN_val[c_nc_type]);
 		}
 
 		MGD77_Close_File (GMT, &In);
 		MGD77_Free_Dataset (GMT, &D);
 		n_changed++;
-		GMT_Report (API, GMT_MSG_NORMAL, "Data column %s added to %s\n", Ctrl->I.c_abbrev, list[argno]);
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Data column %s added to %s\n", Ctrl->I.c_abbrev, list[argno]);
 	}
 
 	gmt_M_free (GMT, colvalue);
 	if (two_cols) gmt_M_free (GMT, coldnt);
 
 	if (Ctrl->D.active)
-		GMT_Report (API, GMT_MSG_VERBOSE, "Removed %d data columns from %d MGD77 files\n", n_delete, n_changed);
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Removed %d data columns from %d MGD77 files\n", n_delete, n_changed);
 	else if (Ctrl->A.mode == MODE_e)
-		GMT_Report (API, GMT_MSG_VERBOSE, "E77 corrections applied to %d MGD77 files\n", n_changed);
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "E77 corrections applied to %d MGD77 files\n", n_changed);
 	else
-		GMT_Report (API, GMT_MSG_VERBOSE, "Sampled data for %d MGD77 files\n", n_changed);
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Sampled data for %d MGD77 files\n", n_changed);
 	
 	MGD77_Path_Free (GMT, (uint64_t)n_paths, list);
 	MGD77_end (GMT, &In);
