@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *	Copyright (c) 2009-2018 by P. Wessel and R. Scharroo
+ *	Copyright (c) 2009-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU Lesser General Public License as published by
@@ -20,8 +19,8 @@
  *			   pwessel@hawaii.edu
  *		Remko Scharroo, Altimetrics
  *			   remko@altimetrics.com
- * Version:	5.3 [64-bit enabled API edition, decoupled from GMT]
- * Date:	13-NOV-2015
+ * Version:	6.0 [64-bit enabled API edition, decoupled from GMT]
+ * Date:	13-OCT-2017
  */
 
 /*!
@@ -130,6 +129,7 @@ enum PSL_enum_const {PSL_CM	= 0,
 	PSL_FINALIZE		= 1,
 	PSL_OVERLAY		= 1,
 	PSL_MEMORY		= 2,
+	PSL_CHANGESET		= 4,
 	PSL_MEM_ALLOC		= 2097152, /* 2 Mb of memory */
 	PSL_INIT		= 0,
 	PSL_LANDSCAPE		= 0,
@@ -149,10 +149,10 @@ enum PSL_enum_const {PSL_CM	= 0,
 	PSL_LINEAR		= 0,
 	PSL_BEZIER		= 1,
 	PSL_MAX_EPS_FONTS	= 6,
-	PSL_MAX_DIMS		= 12,		/* Max number of dim arguments to PSL_plot_symbol */
+	PSL_MAX_DIMS		= 13,		/* Max number of dim arguments to PSL_plot_symbol */
 	PSL_N_PATTERNS		= 91,		/* Current number of predefined patterns + 1, # 91 is user-supplied */
 	PSL_NAME_LEN		= 32,		/* Max length of font names */
-	PSL_BUFSIZ		= 256U};
+	PSL_BUFSIZ		= 4096U};
 
 /* PSL codes for pen movements (used by PSL_plotpoint, PSL_plotline, PSL_plotarc) */
 
@@ -160,7 +160,8 @@ enum PSL_enum_move {PSL_DRAW	= 0,
 	PSL_MOVE		= 1,
 	PSL_STROKE		= 2,
 	PSL_REL			= 4,
-	PSL_CLOSE		= 8};
+	PSL_CLOSE		= 8,
+	PSL_CLIP		= 16};
 
 /* PSL codes for text and paragraph justification */
 
@@ -205,6 +206,11 @@ enum PSL_enum_txt {PSL_TXT_INIT	= 1,
 	PSL_TXT_FILLBOX		= 128,
 	PSL_TXT_DRAWBOX		= 256};
 
+/* PSL codes for text hyphen substitution (PSL_settextmode) */
+
+enum PSL_enum_txtmode {PSL_TXTMODE_HYPHEN	= 0,
+	PSL_TXTMODE_MINUS			= 1};
+	
 /* Verbosity levels */
 
 enum PSL_enum_verbose {PSL_MSG_QUIET = 0,	/* No messages whatsoever */
@@ -279,6 +285,7 @@ struct PSL_CTRL {
 		double linewidth;		/* Current pen thickness			*/
 		double rgb[3][4];		/* Current stroke, fill, and fs fill rgb	*/
 		double offset;			/* Current setdash offset			*/
+		double transparency;		/* Current transparency				*/
 		double fontsize;		/* Current font size				*/
 		double subsupsize;		/* Fractional size of super/sub-scripts		*/
 		double scapssize;		/* Fractional size of small caps		*/
@@ -288,6 +295,8 @@ struct PSL_CTRL {
 		int nclip;			/* Clip depth 					*/
 		int font_no;			/* Current font number				*/
 		int outline;			/* Current outline				*/
+		int complete;			/* true for executing a custom PSL_plot_completion procedure once */
+		int use_minus;			/* true for replacing hyphen code with minus codes */
 	} current;
 	struct INTERNAL {	/* Variables used internally only */
 		char *SHAREDIR;			/* Pointer to path of directory with postscriptlight subdirectory */
@@ -370,6 +379,7 @@ EXTERN_MSC int PSL_beginclipping (struct PSL_CTRL *PSL, double *x, double *y, in
 EXTERN_MSC int PSL_beginlayer (struct PSL_CTRL *PSL, int layer);
 EXTERN_MSC int PSL_beginplot (struct PSL_CTRL *PSL, FILE *fp, int orientation, int overlay, int color_mode, char origin[], double offset[], double page_size[], char *title, int font_no[]);
 EXTERN_MSC int PSL_beginsession (struct PSL_CTRL *PSL, unsigned int search, char *sharedir, char *userdir);
+EXTERN_MSC int PSL_copy (struct PSL_CTRL *PSL, const char *txt);
 EXTERN_MSC int PSL_endaxes (struct PSL_CTRL *PSL);
 EXTERN_MSC int PSL_endclipping (struct PSL_CTRL *PSL, int mode);
 EXTERN_MSC int PSL_endlayer (struct PSL_CTRL *PSL);
@@ -378,6 +388,7 @@ EXTERN_MSC int PSL_endsession (struct PSL_CTRL *PSL);
 EXTERN_MSC char * PSL_getplot (struct PSL_CTRL *PSL);
 EXTERN_MSC char * PSL_makecolor (struct PSL_CTRL *PSL, double rgb[]);
 EXTERN_MSC char * PSL_makefont (struct PSL_CTRL *PSL, double size, double rgb[]);
+EXTERN_MSC char * PSL_makefontsize (struct PSL_CTRL *PSL, double size);
 EXTERN_MSC char * PSL_makepen (struct PSL_CTRL *PSL, double linewidth, double rgb[], char *pattern, double offset);
 EXTERN_MSC int PSL_plotarc (struct PSL_CTRL *PSL, double x, double y, double radius, double az1, double az2, int type);
 EXTERN_MSC int PSL_plotaxis (struct PSL_CTRL *PSL, double annotation_int, char *label, double annotfontsize, int side);
@@ -397,13 +408,17 @@ EXTERN_MSC int PSL_plottext (struct PSL_CTRL *PSL, double x, double y, double fo
 EXTERN_MSC int PSL_plottextbox (struct PSL_CTRL *PSL, double x, double y, double fontsize, char *text, double angle, int justify, double offset[], int mode);
 EXTERN_MSC int PSL_plottextline (struct PSL_CTRL *PSL, double x[], double y[], int np[], int n_segments, void *arg1, void *arg2, char *label[], double angle[], int nlabel_per_seg[], double fontsize, int justify, double offset[], int mode);
 EXTERN_MSC int PSL_loadimage (struct PSL_CTRL *PSL, char *file, struct imageinfo *header, unsigned char **image);
+EXTERN_MSC int PSL_loadeps (struct PSL_CTRL *PSL, char *file, struct imageinfo *h, unsigned char **picture);
 EXTERN_MSC int PSL_setcolor (struct PSL_CTRL *PSL, double rgb[], int mode);
+EXTERN_MSC int PSL_setcurrentpoint (struct PSL_CTRL *PSL, double x, double y);
+EXTERN_MSC int PSL_setexec (struct PSL_CTRL *PSL, int action);
 EXTERN_MSC int PSL_setdefaults (struct PSL_CTRL *PSL, double xyscales[], double page_rgb[], char *encoding);
 EXTERN_MSC int PSL_setdash (struct PSL_CTRL *PSL, char *pattern, double offset);
 EXTERN_MSC int PSL_setfill (struct PSL_CTRL *PSL, double rgb[], int outline);
 EXTERN_MSC int PSL_setfont (struct PSL_CTRL *PSL, int font_no);
 EXTERN_MSC int PSL_setfontdims (struct PSL_CTRL *PSL, double supsub, double scaps, double sup_lc, double sup_uc, double sdown);
 EXTERN_MSC int PSL_setformat (struct PSL_CTRL *PSL, int n_decimals);
+EXTERN_MSC int PSL_setimage (struct PSL_CTRL *PSL, int image_no, char *imagefile, unsigned char *image, int image_dpi, unsigned int dim[], double f_rgb[], double b_rgb[]);
 EXTERN_MSC int PSL_setlinecap (struct PSL_CTRL *PSL, int cap);
 EXTERN_MSC int PSL_setlinejoin (struct PSL_CTRL *PSL, int join);
 EXTERN_MSC int PSL_setlinewidth (struct PSL_CTRL *PSL, double linewidth);
@@ -411,6 +426,8 @@ EXTERN_MSC int PSL_setmiterlimit (struct PSL_CTRL *PSL, int limit);
 EXTERN_MSC int PSL_setorigin (struct PSL_CTRL *PSL, double x, double y, double angle, int mode);
 EXTERN_MSC int PSL_setparagraph (struct PSL_CTRL *PSL, double line_space, double par_width, int par_just);
 EXTERN_MSC int PSL_setpattern (struct PSL_CTRL *PSL, int image_no, char *imagefile, int image_dpi, double f_rgb[], double b_rgb[]);
+EXTERN_MSC int PSL_settextmode (struct PSL_CTRL *PSL, int mode);
+EXTERN_MSC int PSL_settransparency (struct PSL_CTRL *PSL, double transparency);
 EXTERN_MSC int PSL_settransparencymode (struct PSL_CTRL *PSL, const char *mode);
 EXTERN_MSC int PSL_definteger (struct PSL_CTRL *PSL, const char *param, int value);
 EXTERN_MSC int PSL_defpen (struct PSL_CTRL *PSL, const char *param, double width, char *style, double offset, double rgb[]);
@@ -422,6 +439,7 @@ EXTERN_MSC int PSL_defunits (struct PSL_CTRL *PSL, const char *param, double val
 EXTERN_MSC unsigned char *psl_gray_encode (struct PSL_CTRL *PSL, size_t *nbytes, unsigned char *input);
 EXTERN_MSC void psl_set_txt_array (struct PSL_CTRL *PSL, const char *param, char *array[], int n);
 EXTERN_MSC void psl_set_int_array (struct PSL_CTRL *PSL, const char *param, int *array, int n);
+EXTERN_MSC char *psl_prepare_text (struct PSL_CTRL *PSL, char *text);
 
 /* Used indirectly by FORTRAN wrapper PSL_free_ . */
 EXTERN_MSC int PSL_free_nonmacro (void *addr);
@@ -442,7 +460,7 @@ EXTERN_MSC int PSL_fclose (struct PSL_CTRL *C);
 EXTERN_MSC void psl_vector_v4 (struct PSL_CTRL *PSL, double x, double y, double param[], double rgb[], int outline);
 #endif
 
-/*! Macro for free that excplicitly checks for NULL pointer and sets freed pointer to NULL */
+/*! Macro for free that explicitly checks for NULL pointer and sets freed pointer to NULL */
 #define PSL_free(ptr) (free((void *)(ptr)),(ptr)=NULL)
 
 #ifdef __cplusplus
