@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *	Copyright (c) 1991-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  * Author:      Paul Wessel
@@ -26,7 +25,8 @@
 
 #include "gmt_dev.h"
 
-#define THIS_MODULE_NAME	"gmtflexure"
+#define THIS_MODULE_CLASSIC_NAME	"gmtflexure"
+#define THIS_MODULE_MODERN_NAME	"gmtflexure"
 #define THIS_MODULE_LIB		"potential"
 #define THIS_MODULE_PURPOSE	"Compute flexural deformation of 2-D loads, forces, and bending moments"
 #define THIS_MODULE_KEYS	"ED(,QD(,TD(,>D}"
@@ -84,6 +84,9 @@ struct GMTFLEXURE_CTRL {
 		bool active;
 		double force;
 	} F;
+	struct L {	/* Use variable restoring force [constant] */
+		bool active;
+	} L;
 	struct M {	/* -Mx|z  */
 		bool active[2];	/* True if km, else m */
 	} M;
@@ -91,10 +94,10 @@ struct GMTFLEXURE_CTRL {
 		bool active;
 		bool set_x;
 		unsigned int mode;
-		double min, max, inc;
+		struct GMT_ARRAY T;
 		char *file;
 	} Q;
-	struct S {	/* Variable restoring force */
+	struct S {	/* Compute second derivatives (curvatures) */
 		bool active;
 	} S;
 	struct T {	/* Pre-existing deformation */
@@ -144,6 +147,7 @@ GMT_LOCAL void Free_Ctrl (struct GMT_CTRL *GMT, struct GMTFLEXURE_CTRL *C) {	/*
 	gmt_M_str_free (C->E.file);
 	gmt_M_str_free (C->Q.file);
 	gmt_M_str_free (C->T.file);
+	gmt_free_array (GMT, &(C->Q.T));
 	gmt_M_free (GMT, C);
 }
 
@@ -230,6 +234,9 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GMTFLEXURE_CTRL *Ctrl, struct
 				Ctrl->F.active = true;
 				Ctrl->F.force = atof (opt->arg);
 				break;
+			case 'L':	/* Variable restoring force */
+				Ctrl->L.active = true;
+				break;
 			case 'M':	/* Length units */
 				both = false;	side = 0;
 				switch (opt->arg[0]) {
@@ -253,13 +260,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GMTFLEXURE_CTRL *Ctrl, struct
 					case 'n':	Ctrl->Q.mode = NO_LOAD;
 						if (opt->arg[1]) {	/* Gave domain info */
 							Ctrl->Q.set_x = true;
-							if (sscanf (&opt->arg[1], "%lf/%lf/%lf", &Ctrl->Q.min, &Ctrl->Q.max, &Ctrl->Q.inc) != 3) {
-								GMT_Report (API, GMT_MSG_NORMAL, "Syntax error: Unable to decode distance arguments for -Q\n");
-								n_errors++;
-							}
-							if (opt->arg[strlen(opt->arg)-1] == '+') {	/* Gave number of points instead; calculate inc */
-								Ctrl->Q.inc = (Ctrl->Q.max - Ctrl->Q.min) / (Ctrl->Q.inc - 1.0);
-							}
+							n_errors += gmt_parse_array (GMT, 'Q', &opt->arg[1], &(Ctrl->Q.T), GMT_ARRAY_DIST, 0);
 						}
 						break;
 					case 'q':	Ctrl->Q.mode = F_LOAD; break;
@@ -294,10 +295,11 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GMTFLEXURE_CTRL *Ctrl, struct
 }
 
 GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
-	gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
+	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: gmtflexure -D<rhom>/<rhol>[/<rhoi>]/<rhow> -E<te> -Q<loadinfo> [-A[l|r]<bc>[/<args>]]\n");
-	GMT_Message (API, GMT_TIME_NONE,"\t[-C[p|y]<value] [-F<force>] [-S] [-T<wpre>] [%s] [-W<w0>] [-Z<zm>]\n\t[%s] [%s] [%s] [%s] [%s]\n\n", GMT_V_OPT, GMT_e_OPT, GMT_e_OPT, GMT_h_OPT, GMT_i_OPT, GMT_o_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "usage: %s -D<rhom>/<rhol>[/<rhoi>]/<rhow> -E<te> -Q<loadinfo> [-A[l|r]<bc>[/<args>]]\n", name);
+	GMT_Message (API, GMT_TIME_NONE,"\t[-C[p|y]<value] [-F<force>] [-L] [-S] [-T<wpre>] [%s] [-W<w0>] [-Z<zm>]\n\t[%s] [%s] [%s]\n\t[%s] [%s] [%s]\n\n",
+		GMT_V_OPT, GMT_e_OPT, GMT_e_OPT, GMT_h_OPT, GMT_i_OPT, GMT_o_OPT, GMT_PAR_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
@@ -308,7 +310,7 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t-Q Input load option. Choose among these options:\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   -Qn indicates there is no load (only -A and -L contribute to deformation).\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t      If no file is given via -E<file> then append <min/max/inc> to set an equidistant profile.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t      Append + to inc to indicate the number of points instead.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t      Append +n to inc to indicate the number of points instead.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   -Qq[<load>] is a file (or stdin) with (x,load in Pa) for all points.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   -Qt[<load>] is a file (or stdin) with (x,load in m or km) for all points (see -M).\n");
 	GMT_Message (API, GMT_TIME_NONE, "\n\tOPTIONS:\n");
@@ -319,14 +321,14 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t   -Al2/w0 or -Ar2/w0 :   \"Clamped\", w at end = w0 [0], w' = 0\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   -Al3/m/f or -Ar3/m/f : \"Free\" condition, specify (m)oment and (f)orce at end [0/0].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   Default BCs are -Al0 -Ar0.  Use SI units for any optional arguments.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-C use -Cy<Young> or -Cp<poisson> to change Young's modulus [%g] or Poisson's ratio [%g].\n", YOUNGS_MODULUS, POISSONS_RATIO);
-	GMT_Message (API, GMT_TIME_NONE, "\t-F specifies the uniform, horizontal stress in the plate [Pa m].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-L uses variable restoring force k(x) that depends on w(x).\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-M sets units used, as follows:\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t-C Use -Cy<Young> or -Cp<poisson> to change Young's modulus [%g] or Poisson's ratio [%g].\n", YOUNGS_MODULUS, POISSONS_RATIO);
+	GMT_Message (API, GMT_TIME_NONE, "\t-F Specifies the uniform, horizontal stress in the plate [Pa m].\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t-L Use variable restoring force k(x) that depends on w(x).\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t-M Sets units used, as follows:\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   -Mx indicates all x-distances are in km [meters]\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   -Mz indicates all z-deflections are in km [meters]\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-S Also compute second derivatives (curvatures) on output.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-T to use file <wpre> with pre-existing deflections [none].\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t-T To use file <wpre> with pre-existing deflections [none].\n");
 	GMT_Option (API, "V");
 	GMT_Message (API, GMT_TIME_NONE, "\t-W Specify water depth in m; append k for km.  Must be positive.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   Subaerial topography will be scaled by -D to account for density differences.\n");
@@ -974,7 +976,7 @@ GMT_LOCAL int flx1dw0 (struct GMT_CTRL *GMT, double *w, double *w0, double *d, d
  *	d2/dr2 (Dr * d2/dr2 w(r)) + k * w(r) = p (r)
  * BC's at r = Inf are w(r) = w'(r) = 0
  *
- * Finite difference equation taken from J. Bodine's Techincal Report.
+ * Finite difference equation taken from J. Bodine's Technical Report.
  * The deflections are solved by forward/backward substitution to solve
  * the 5-diagonal matrix problem A*w = p using a LU-transformation (lu_solver)
  * The parameters passed are:
@@ -1244,14 +1246,11 @@ int GMT_gmtflexure (void *V_API, int mode, void *args) {
 	options = GMT_Create_Options (API, mode, args);
 	if (API->error) return (API->error);	/* Set or get option list */
 
-	if (!options || options->option == GMT_OPT_USAGE)
-		bailout (usage (API, GMT_USAGE));	/* Return the usage message */
-	if (options->option == GMT_OPT_SYNOPSIS)
-		bailout (usage (API, GMT_SYNOPSIS));	/* Return the synopsis */
+	if ((error = gmt_report_usage (API, options, 0, usage)) != GMT_NOERROR) bailout (error);	/* Give usage if requested */
 
 	/* Parse the command-line arguments */
 
-	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
 	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
 	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
 	if ((error = parse (GMT, Ctrl, options)) != 0) Return (error);
@@ -1264,7 +1263,7 @@ int GMT_gmtflexure (void *V_API, int mode, void *args) {
 	}
 	else {	/* Load file given */
 		char *type = (Ctrl->Q.mode == T_LOAD) ? "topography" : "pressure";
-		GMT_Report (API, GMT_MSG_VERBOSE, "Processing input %s table data\n", type);
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Processing input %s table data\n", type);
 		if ((Q = GMT_Read_Data (API, GMT_IS_DATASET, GMT_IS_FILE, GMT_IS_LINE, GMT_READ_NORMAL, NULL, Ctrl->Q.file, NULL)) == NULL) {
 			Return (API->error);
 		}
@@ -1296,7 +1295,7 @@ int GMT_gmtflexure (void *V_API, int mode, void *args) {
 	if (Ctrl->E.file) {	/* Gave file with elastic thicknesses or rigidities */
 		double scale = (Ctrl->M.active[1]) ? 1000.0 : 1.0;	/* Either got Te in km or m */
 		double d_min = DBL_MAX, d_max = 0.0;
-		GMT_Report (API, GMT_MSG_VERBOSE, "Processing input Te or Rigidity table data\n");
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Processing input Te or Rigidity table data\n");
 		gmt_disable_bhi_opts (GMT);	/* Do not want any -b -h -i to affect the reading from -C,-F,-L files */
 		if ((E = GMT_Read_Data (API, GMT_IS_DATASET, GMT_IS_FILE, GMT_IS_LINE, GMT_READ_NORMAL, NULL, Ctrl->E.file, NULL)) == NULL) {
 			Return (API->error);
@@ -1318,9 +1317,9 @@ int GMT_gmtflexure (void *V_API, int mode, void *args) {
 			}
 		}
 		if (d_min == d_max)
-			GMT_Report (API, GMT_MSG_VERBOSE, "Constant rigidity: %g \n", d_min);
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Constant rigidity: %g \n", d_min);
 		else
-			GMT_Report (API, GMT_MSG_VERBOSE, "Range of rigidities: %g to %g\n", d_min, d_max);
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Range of rigidities: %g to %g\n", d_min, d_max);
 	}
 	if (Ctrl->Q.mode == NO_LOAD)	{	/* No load file given */
 		if (Ctrl->E.file) {	/* Use info from elastic thickness file instead */
@@ -1335,20 +1334,21 @@ int GMT_gmtflexure (void *V_API, int mode, void *args) {
 		}
 		else {	/* No input files given, create single equidistant profile */
 			uint64_t dim[GMT_DIM_SIZE] = {1, 1, 0, 2};
-			dim[2] = urint ((Ctrl->Q.max - Ctrl->Q.min)/Ctrl->Q.inc) + 1;
-			GMT_Report (API, GMT_MSG_VERBOSE, "Create empty load table data\n");
+			if (gmt_create_array (GMT, 'Q', &(Ctrl->Q.T), NULL, NULL))
+				Return (GMT_RUNTIME_ERROR);
+			dim[2] = Ctrl->Q.T.n;
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Create empty load table data\n");
 			if ((Q = GMT_Create_Data (API, GMT_IS_DATASET, GMT_IS_LINE, 0, dim, NULL, NULL, 0, 0, NULL)) == NULL) {
 				Return (API->error);	/* An empty table */
 			}
 			S = Q->table[0]->segment[0];	/* Only a single segment here */
-			for (row = 0; row < dim[GMT_ROW]; row++) {	/* Fill in x values */
-				S->data[GMT_X][row] = (row == (S->n_rows-1)) ? Ctrl->Q.max: Ctrl->Q.min + row * Ctrl->Q.inc;
-			}
+			S->n_rows = dim[GMT_ROW];
+			gmt_M_memcpy (S->data[GMT_X], Ctrl->Q.T.array, Ctrl->Q.T.n, double);
 		}
 	}
 	if (!Ctrl->E.file) {	/* Got a constant Te in m instead */
 		double d = te_2_d (Ctrl, Ctrl->E.te);
-		GMT_Report (API, GMT_MSG_VERBOSE, "Constant rigidity: %g \n", d);
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Constant rigidity: %g \n", d);
 		E = GMT_Duplicate_Data (API, GMT_IS_DATASET, GMT_DUPLICATE_DATA, Q);
 		/* Overwrite 2nd column with constant d below */
 		for (tbl = 0; tbl < E->n_tables; tbl++) {
@@ -1418,12 +1418,12 @@ int GMT_gmtflexure (void *V_API, int mode, void *args) {
 				sprintf (txt, "plate free with Moment = %g and Force = %g at right edge.\n", Ctrl->A.moment[RIGHT], Ctrl->A.force[RIGHT]);
 				strcat (msg, txt);
 			}
-			GMT_Report (API, GMT_MSG_VERBOSE, msg);
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, msg);
 			for (row = 0, airy = true; airy && row < S->n_rows; row++)
 				if (rigidity[row] > 0.0) airy = false;
 
 			if (airy) {	/* Airy compensation */
-				GMT_Report (API, GMT_MSG_VERBOSE, "Calculate flexure using Airy compensation\n");
+				GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Calculate flexure using Airy compensation\n");
 				for (row = 0; row < S->n_rows; row++) deflection[row] = load[row] / restore;
 			}
 			x_inc = S->data[GMT_X][1] - S->data[GMT_X][0];
@@ -1431,20 +1431,20 @@ int GMT_gmtflexure (void *V_API, int mode, void *args) {
 
 			if (Ctrl->T.active) {	/* Plate has pre-existing deflection */
 				double *w0 = T->table[tbl]->segment[seg]->data[GMT_Y];
-				GMT_Report (API, GMT_MSG_VERBOSE, "Calculate flexure of pre-deformed surface\n");
+				GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Calculate flexure of pre-deformed surface\n");
 				error = flx1dw0 (GMT, deflection, w0, rigidity, load, (int)S->n_rows, x_inc, &restore, 0, Ctrl->F.force, Ctrl->A.bc[LEFT], Ctrl->A.bc[RIGHT]);
 			}
-			else if (Ctrl->S.active) {
-				GMT_Report (API, GMT_MSG_VERBOSE, "Calculate flexure with variable restoring force\n");
+			else if (Ctrl->L.active) {
+				GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Calculate flexure with variable restoring force\n");
 				error = flx1dk (GMT, deflection, rigidity, load, (int)S->n_rows, x_inc, Ctrl->D.rhom, Ctrl->D.rhol, Ctrl->D.rhoi, Ctrl->D.rhow, Ctrl->F.force, Ctrl->A.bc[LEFT], Ctrl->A.bc[RIGHT]);
 			}
 			else {	/* Constant restoring force */
-				GMT_Report (API, GMT_MSG_VERBOSE, "Calculate flexure for constant restoring force\n");
+				GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Calculate flexure for constant restoring force\n");
 				error = flx1d (GMT, deflection, rigidity, load, (int)S->n_rows, x_inc, &restore, 0, Ctrl->F.force, Ctrl->A.bc[LEFT], Ctrl->A.bc[RIGHT]);
 			}
 
 			if (error) {
-				GMT_Report (API, GMT_MSG_VERBOSE, "Flexure sub-function returned error = %d!\n", error);
+				GMT_Report (API, GMT_MSG_NORMAL, "Flexure sub-function returned error = %d!\n", error);
 				Return (API->error);
 			}
 
