@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *	Copyright (c) 1991-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  * Author:      Joaquim Luis
@@ -34,9 +33,10 @@
 
 #include "gmt_dev.h"
 
-#define THIS_MODULE_NAME	"gravfft"
+#define THIS_MODULE_CLASSIC_NAME	"gravfft"
+#define THIS_MODULE_MODERN_NAME	"gravfft"
 #define THIS_MODULE_LIB		"potential"
-#define THIS_MODULE_PURPOSE	"Compute gravitational attraction of 3-D surfaces and a little more (ATTENTION z positive up)"
+#define THIS_MODULE_PURPOSE	"Spectral calculations of gravity, isostasy, admittance, and coherence for grids"
 #define THIS_MODULE_KEYS	"<G{+,GG},DG(,GDC,GDI"
 #define THIS_MODULE_NEEDS	""
 #define THIS_MODULE_OPTIONS "-Vf"
@@ -74,7 +74,7 @@ struct GRAVFFT_CTRL {
 		bool active;
 		unsigned int n_terms;
 	} E;
-	struct GRVF_F {	/* -F[f[+]|g|e|n|v] */
+	struct GRVF_F {	/* -F[f[+s]|b|g|e|n|v] */
 		bool active;
 		bool slab;
 		bool bouger;
@@ -162,14 +162,14 @@ static double	scale_out = 1.0;
 static double	earth_rad = 6371008.7714;	/* GRS-80 sphere */
 
 GMT_LOCAL void do_parker (struct GMT_CTRL *GMT, struct GMT_GRID *Grid, struct GRAVFFT_CTRL *Ctrl, struct GMT_FFT_WAVENUMBER *K,
-                          float *raised, uint64_t n, double rho);
+                          gmt_grdfloat *raised, uint64_t n, double rho);
 GMT_LOCAL void do_isostasy(struct GMT_CTRL *GMT, struct GMT_GRID *Grid, struct GRAVFFT_CTRL *Ctrl, struct GMT_FFT_WAVENUMBER *K);
 GMT_LOCAL void load_from_below_admitt(struct GMT_CTRL *GMT, struct GRAVFFT_CTRL *Ctrl, struct GMT_FFT_WAVENUMBER *K, double *z_below);
 GMT_LOCAL void load_from_top_admitt(struct GMT_CTRL *GMT, struct GRAVFFT_CTRL *Ctrl, struct GMT_FFT_WAVENUMBER *K, double *z_top);
 GMT_LOCAL void load_from_top_grid(struct GMT_CTRL *GMT, struct GMT_GRID *Grid, struct GRAVFFT_CTRL *Ctrl,
-                                  struct GMT_FFT_WAVENUMBER *K, float *raised, unsigned int n);
+                                  struct GMT_FFT_WAVENUMBER *K, gmt_grdfloat *raised, unsigned int n);
 GMT_LOCAL void load_from_below_grid(struct GMT_CTRL *GMT, struct GMT_GRID *Grid, struct GRAVFFT_CTRL *Ctrl,
-                                    struct GMT_FFT_WAVENUMBER *K, float *raised, unsigned int n);
+                                    struct GMT_FFT_WAVENUMBER *K, gmt_grdfloat *raised, unsigned int n);
 GMT_LOCAL void compute_only_admitts(struct GMT_CTRL *GMT, struct GRAVFFT_CTRL *Ctrl, struct GMT_FFT_WAVENUMBER *K,
                                     double *z_top_or_bot, double delta_pt);
 GMT_LOCAL int do_admittance(struct GMT_CTRL *GMT, struct GMT_GRID *Grid, struct GMT_GRID *GridB, struct GRAVFFT_CTRL *Ctrl,
@@ -231,7 +231,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRAVFFT_CTRL *Ctrl, struct GMT
 			case 'D':
 				if (!opt->arg) {
 					GMT_Report (API, GMT_MSG_NORMAL,
-					            "Syntax error -D option: must give constant density contrast or grid with density constrasts\n");
+					            "Syntax error -D option: must give constant density contrast or grid with density contrasts\n");
 					n_errors++;
 				}
 				else {
@@ -269,8 +269,9 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRAVFFT_CTRL *Ctrl, struct GMT
 						Ctrl->F.slab   = true;
 						Ctrl->F.bouger = true;
 						break;
-					default:  Ctrl->F.mode = GRAVFFT_FAA; 	   /* FAA */
-					 	if (opt->arg[1] == '+') Ctrl->F.slab = true;
+					case 'f': default:
+						Ctrl->F.mode = GRAVFFT_FAA; 	   /* FAA */
+					 	if (opt->arg[1] == '+' && (opt->arg[2] == 's' || opt->arg[2] == '\0')) Ctrl->F.slab = true;
 						break;
 				}
 				break;
@@ -308,7 +309,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRAVFFT_CTRL *Ctrl, struct GMT
 				break;
 			case 'L':	/* Leave trend alone */
 				if (gmt_M_compat_check (GMT, 4))
-					GMT_Report (API, GMT_MSG_COMPAT, "Warning: Option -L is deprecated; use -N modifiers in the future.\n");
+					GMT_Report (API, GMT_MSG_COMPAT, "Option -L is deprecated; use -N modifiers in the future.\n");
 				else
 					n_errors += gmt_default_error (GMT, opt->option);
 				break;
@@ -415,12 +416,12 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRAVFFT_CTRL *Ctrl, struct GMT
 }
 
 GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
-	gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
+	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: gravfft <topo_grd> [<ingrid2>] -G<outgrid> [-C<n/wavelength/mean_depth/tbw>]\n");
-	GMT_Message (API, GMT_TIME_NONE,"\t[-D<density|grid>] [-E<n_terms>] [-F[f[+]|g|v|n|e]] [-I<wbctk>]\n");
+	GMT_Message (API, GMT_TIME_NONE, "usage: %s <topo_grd> [<ingrid2>] -G<outgrid> [-C<n/wavelength/mean_depth/tbw>]\n", name);
+	GMT_Message (API, GMT_TIME_NONE,"\t[-D<density|grid>] [-E<n_terms>] [-F[f[+s]|b|g|v|n|e]] [-I<wbctk>]\n");
 	GMT_Message (API, GMT_TIME_NONE,"\t[-N%s] [-Q]\n", GMT_FFT_OPT);
-	GMT_Message (API, GMT_TIME_NONE,"\t[-T<te/rl/rm/rw>[/<ri>][+m]] [%s] [-W<wd>] [-Z<zm>[/<zl>]] [-fg]\n\n", GMT_V_OPT);
+	GMT_Message (API, GMT_TIME_NONE,"\t[-T<te/rl/rm/rw>[/<ri>][+m]] [%s] [-W<wd>] [-Z<zm>[/<zl>]] [-fg] [%s]\n\n", GMT_V_OPT, GMT_PAR_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
@@ -452,8 +453,9 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE,"\t       theoretical admittance.\n");
 	GMT_Message (API, GMT_TIME_NONE,"\t-E Number of terms used in Parker's expansion [Default = 3].\n");
 	GMT_Message (API, GMT_TIME_NONE,"\t-F Specify desired geopotential field:\n");
+	GMT_Message (API, GMT_TIME_NONE,"\t   b = Bouguer anomalies (mGal).\n");
 	GMT_Message (API, GMT_TIME_NONE,"\t   f = Free-air anomalies (mGal) [Default].\n");
-	GMT_Message (API, GMT_TIME_NONE,"\t       Append + to adjust for implied slab correction [none].\n");
+	GMT_Message (API, GMT_TIME_NONE,"\t       Append +s to adjust for implied slab correction [none].\n");
 	GMT_Message (API, GMT_TIME_NONE,"\t   g = Geoid anomalies (m).\n");
 	GMT_Message (API, GMT_TIME_NONE,"\t   v = Vertical Gravity Gradient (VGG; 1 Eovtos = 0.1 mGal/km).\n");
 	GMT_Message (API, GMT_TIME_NONE,"\t   e = East deflections of the vertical (micro-radian).\n");
@@ -488,7 +490,7 @@ int GMT_gravfft (void *V_API, int mode, void *args) {
 	unsigned int k, n;
 	int      error = 0;
 	uint64_t m;
-	float    slab_gravity = 0.0f, *topo = NULL, *raised = NULL;
+	gmt_grdfloat    slab_gravity = 0.0f, *topo = NULL, *raised = NULL;
 	double   delta_pt, freq;
 
 	struct GMT_GRID *Grid[2] = {NULL, NULL}, *Orig[2] = {NULL, NULL}, *Rho = NULL;
@@ -505,14 +507,11 @@ int GMT_gravfft (void *V_API, int mode, void *args) {
 	options = GMT_Create_Options (API, mode, args);
 	if (API->error) return (API->error);	/* Set or get option list */
 
-	if (!options || options->option == GMT_OPT_USAGE)
-		bailout (usage (API, GMT_USAGE));		/* Return the usage message */
-	if (options->option == GMT_OPT_SYNOPSIS)
-		bailout (usage (API, GMT_SYNOPSIS));	/* Return the synopsis */
+	if ((error = gmt_report_usage (API, options, 0, usage)) != GMT_NOERROR) bailout (error);	/* Give usage if requested */
 
 	/* Parse the command-line arguments */
 
-	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
 	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
 	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
 	if ((error = parse (GMT, Ctrl, options)) != 0) Return (error);
@@ -545,6 +544,7 @@ int GMT_gravfft (void *V_API, int mode, void *args) {
 			Return (API->error);
 		}
 		S = D->table[0]->segment[0];	/* Only one table with one segment here */
+		S->n_rows = Ctrl->C.n_pt;
 
 		for (k = 0; k < Ctrl->C.n_pt; k++) {
 			freq = (k + 1) * delta_pt;
@@ -563,14 +563,14 @@ int GMT_gravfft (void *V_API, int mode, void *args) {
 
 	if (!Ctrl->Q.active) {
 		if (((Ctrl->T.active && !Ctrl->T.moho) && Ctrl->E.n_terms > 1) || (Ctrl->S.active && Ctrl->E.n_terms > 1)) {
-			GMT_Report (API, GMT_MSG_NORMAL, "Warning: Due to a bug, or a method limitation (I didn't figure that yet)\n"
-				"with the selected options, the number of terms in Parker expansion is reset to one\n"
+			GMT_Report (API, GMT_MSG_VERBOSE, "Due to a bug, or a method limitation (I didn't figure that yet)\n" \
+				"with the selected options, the number of terms in Parker expansion is reset to one\n" \
 				"See examples in the manual if you want to compute with higher order expansion\n\n");
 			Ctrl->E.n_terms = 1;
 		}
 	}
 
-	GMT_Report (API, GMT_MSG_VERBOSE, "Allocates memory and read data file\n");
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Allocates memory and read data file\n");
 
 	for (k = 0; k < Ctrl->In.n_grids; k++) {	/* First read the grid header(s) */
 		if ((Orig[k] = GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_CONTAINER_ONLY, NULL, Ctrl->In.file[k], NULL)) == NULL)
@@ -633,11 +633,11 @@ int GMT_gravfft (void *V_API, int mode, void *args) {
 
 	if (Ctrl->W.active) {	/* Need to adjust for a different observation level relative to topo.grd */
 		unsigned int row, col;
-		GMT_Report (API, GMT_MSG_VERBOSE, "Remove %g m from topography grid %s\n", Ctrl->W.water_depth, Ctrl->In.file[0]);
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Remove %g m from topography grid %s\n", Ctrl->W.water_depth, Ctrl->In.file[0]);
 		gmt_M_grd_loop (GMT, Grid[0], row, col, m)
-			Grid[0]->data[m] -= (float)Ctrl->W.water_depth;
-		Grid[0]->header->z_min -= (float)Ctrl->W.water_depth;
-		Grid[0]->header->z_max -= (float)Ctrl->W.water_depth;
+			Grid[0]->data[m] -= (gmt_grdfloat)Ctrl->W.water_depth;
+		Grid[0]->header->z_min -= (gmt_grdfloat)Ctrl->W.water_depth;
+		Grid[0]->header->z_max -= (gmt_grdfloat)Ctrl->W.water_depth;
 	}
 
 	for (k = 0; k < Ctrl->In.n_grids; k++) {	/* Read, and check that no NaNs are present in either grid */
@@ -654,14 +654,14 @@ int GMT_gravfft (void *V_API, int mode, void *args) {
 	K = FFT_info[0];	/* We only need one of these anyway; K is a shorthand */
 
 	Ctrl->misc.z_level = fabs (FFT_info[0]->coeff[0]);	/* Need absolute value or level removed for uppward continuation */
-	GMT_Report (API, GMT_MSG_VERBOSE, "Level used for upward continuation: %g\n", Ctrl->misc.z_level);
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Level used for upward continuation: %g\n", Ctrl->misc.z_level);
 
 	if (Ctrl->I.active) {		/* Compute admittance or coherence from data and exit */
 
-		GMT_Report (API, GMT_MSG_VERBOSE, "Processing gravity file %s\n", Ctrl->In.file[1]);
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Processing gravity file %s\n", Ctrl->In.file[1]);
 
 		for (k = 0; k < Ctrl->In.n_grids; k++) {	/* Call the forward FFT, once per grid */
-			GMT_Report (API, GMT_MSG_VERBOSE, "forward FFT...\n");
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "forward FFT...\n");
 			if (GMT_FFT (API, Grid[k], GMT_FFT_FWD, GMT_FFT_COMPLEX, FFT_info[k]))
 				Return (GMT_RUNTIME_ERROR);
 		}
@@ -678,19 +678,19 @@ int GMT_gravfft (void *V_API, int mode, void *args) {
 		}
 	}
 
-	topo   = gmt_M_memory (GMT, NULL, Grid[0]->header->size, float);
-	raised = gmt_M_memory (GMT, NULL, Grid[0]->header->size, float);
+	topo   = gmt_M_memory (GMT, NULL, Grid[0]->header->size, gmt_grdfloat);
+	raised = gmt_M_memory (GMT, NULL, Grid[0]->header->size, gmt_grdfloat);
 
 	if (Ctrl->Q.active || Ctrl->T.moho) {
 		double coeff[3];
-		GMT_Report (API, GMT_MSG_VERBOSE, "Forward FFT...\n");
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Forward FFT...\n");
 		if (GMT_FFT (API, Grid[0], GMT_FFT_FWD, GMT_FFT_COMPLEX, FFT_info[0])) {
 			Return (GMT_RUNTIME_ERROR);
 		}
 
 		do_isostasy (GMT, Grid[0], Ctrl, K);
 
-		GMT_Report (API, GMT_MSG_VERBOSE, "Inverse FFT...\n");
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Inverse FFT...\n");
 		if (GMT_FFT (API, Grid[0], GMT_FFT_INV, GMT_FFT_COMPLEX, K))
 			Return (GMT_RUNTIME_ERROR);
 
@@ -725,7 +725,7 @@ int GMT_gravfft (void *V_API, int mode, void *args) {
 		}
 	}
 
-	gmt_M_memcpy (topo, Grid[0]->data, Grid[0]->header->size, float);
+	gmt_M_memcpy (topo, Grid[0]->data, Grid[0]->header->size, gmt_grdfloat);
 	/* Manually interleave this copy of topo [and hence raised] since we will call FFT repeatedly */
 	gmt_grd_mux_demux (API->GMT, Grid[0]->header, topo, GMT_GRID_IS_INTERLEAVED);
 	if (Ctrl->D.variable) {	/* Also interleave manually the rho grid */
@@ -734,17 +734,17 @@ int GMT_gravfft (void *V_API, int mode, void *args) {
 			raised[m] = topo[m] * Rho->data[m];
 	}
 	else	/* Constant density contrast passed into do_parker */
-		gmt_M_memcpy (raised,  topo, Grid[0]->header->size, float);
+		gmt_M_memcpy (raised,  topo, Grid[0]->header->size, gmt_grdfloat);
 
-	gmt_M_memset (Grid[0]->data, Grid[0]->header->size, float);
+	gmt_M_memset (Grid[0]->data, Grid[0]->header->size, gmt_grdfloat);
 
 	for (n = 1; n <= Ctrl->E.n_terms; n++) {
 
-		GMT_Report (API, GMT_MSG_VERBOSE, "Evaluating Parker for term = %d\n", n);
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Evaluating Parker for term = %d\n", n);
 
 		if (n > 1)	/* n == 1 was initialized via the gmt_M_memcpy or loop above */
 			for (m = 0; m < Grid[0]->header->size; m++) {
-				raised[m] = (float)pow(topo[m], (double)n);
+				raised[m] = (gmt_grdfloat)pow(topo[m], (double)n);
 				if (Ctrl->D.variable) raised[m] *= Rho->data[m];
 			}
 
@@ -779,9 +779,9 @@ int GMT_gravfft (void *V_API, int mode, void *args) {
 			strcpy (Grid[0]->header->title, "Gravity anomalies");
 			strcpy (Grid[0]->header->z_units, "mGal");
 			if (Ctrl->F.slab) {	/* Do the slab adjustment */
-				slab_gravity = (float) (1.0e5 * 2 * M_PI * Ctrl->misc.rho * GRAVITATIONAL_CONST *
+				slab_gravity = (gmt_grdfloat) (1.0e5 * 2 * M_PI * Ctrl->misc.rho * GRAVITATIONAL_CONST *
 				                        fabs (Ctrl->W.water_depth - Ctrl->misc.z_level));
-				GMT_Report (API, GMT_MSG_VERBOSE, "Add %g mGal to predicted FAA grid to account for implied slab\n", slab_gravity);
+				GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Add %g mGal to predicted FAA grid to account for implied slab\n", slab_gravity);
 				if (Ctrl->F.bouger)		/* The complete bouger contribution */
 					for (m = 0; m < Grid[0]->header->size; m++) Grid[0]->data[m] = slab_gravity - Grid[0]->data[m];
 				else
@@ -808,7 +808,7 @@ int GMT_gravfft (void *V_API, int mode, void *args) {
 
 	snprintf (Grid[0]->header->remark, GMT_GRID_REMARK_LEN160, "Parker expansion of order %d", Ctrl->E.n_terms);
 
-	GMT_Report (API, GMT_MSG_VERBOSE, "Write Output...\n");
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Write Output...\n");
 
 	for (k = 0; k < Ctrl->In.n_grids; k++)
 		GMT_FFT_Destroy (API, &(FFT_info[k]));
@@ -828,8 +828,6 @@ int GMT_gravfft (void *V_API, int mode, void *args) {
 
 	gmt_M_free (GMT, topo);
 
-	GMT_Report (API, GMT_MSG_VERBOSE, "Done!\n");
-
 	Return (GMT_NOERROR);
 }
 
@@ -841,7 +839,7 @@ GMT_LOCAL void do_isostasy (struct GMT_CTRL *GMT, struct GMT_GRID *Grid, struct
 	force on the plate (rm - ri)*gravity if ri = rw; so use zero for topo in air (ri changed to rl).  */
 	uint64_t k;
 	double  A = 1.0, rho_load, airy_ratio, rigidity_d, d_over_restoring_force, mk, k2, k4, transfer_fn;
-	float *datac = Grid->data;
+	gmt_grdfloat *datac = Grid->data;
 	gmt_M_unused(GMT);
 
 	/*   te	 Elastic thickness, SI units (m)  */
@@ -856,7 +854,7 @@ GMT_LOCAL void do_isostasy (struct GMT_CTRL *GMT, struct GMT_GRID *Grid, struct
 	rho_load = Ctrl->T.rhol;
 	if (Ctrl->T.approx) {	/* Do approximate calculation when both rhol and rhoi were set */
 		char way = (Ctrl->T.rhoi < Ctrl->T.rhol) ? '<' : '>';
-		GMT_Report (GMT->parent, GMT_MSG_VERBOSE, "Warning: Approximate FFT-solution to flexure since rho_i (%g) %c rho_l (%g)\n", Ctrl->T.rhoi, way, Ctrl->T.rhol);
+		GMT_Report (GMT->parent, GMT_MSG_VERBOSE, "Approximate FFT-solution to flexure since rho_i (%g) %c rho_l (%g)\n", Ctrl->T.rhoi, way, Ctrl->T.rhol);
 		rho_load = Ctrl->T.rhoi;
 		A = sqrt ((Ctrl->T.rhom - Ctrl->T.rhoi)/(Ctrl->T.rhom - Ctrl->T.rhol));
 	}
@@ -874,16 +872,16 @@ GMT_LOCAL void do_isostasy (struct GMT_CTRL *GMT, struct GMT_GRID *Grid, struct
 		mk = gmt_fft_get_wave (k, K);
 		k2 = mk * mk;	k4 = k2 * k2;
 		transfer_fn = airy_ratio / ( (d_over_restoring_force * k4) + 1.0);
-		datac[k] *= (float)transfer_fn;
-		datac[k+1] *= (float)transfer_fn;
+		datac[k] *= (gmt_grdfloat)transfer_fn;
+		datac[k+1] *= (gmt_grdfloat)transfer_fn;
 	}
 }
 
 #define	MGAL_AT_45	980619.9203 	/* Moritz's 1980 IGF value for gravity in mGal at 45 degrees latitude */
-GMT_LOCAL void do_parker (struct GMT_CTRL *GMT, struct GMT_GRID *Grid, struct GRAVFFT_CTRL *Ctrl, struct GMT_FFT_WAVENUMBER *K, float *raised, uint64_t n, double rho) {
+GMT_LOCAL void do_parker (struct GMT_CTRL *GMT, struct GMT_GRID *Grid, struct GRAVFFT_CTRL *Ctrl, struct GMT_FFT_WAVENUMBER *K, gmt_grdfloat *raised, uint64_t n, double rho) {
 	uint64_t i, k;
 	double f, p, t, mk, kx, ky, v, c;
-	float *datac = Grid->data;
+	gmt_grdfloat *datac = Grid->data;
 	gmt_M_unused(GMT);
 
 	f = 1.0;
@@ -904,34 +902,34 @@ GMT_LOCAL void do_parker (struct GMT_CTRL *GMT, struct GMT_GRID *Grid, struct GR
 		v = c * exp (-mk * Ctrl->misc.z_level) * t;
 		switch (Ctrl->F.mode) {
 			case GRAVFFT_FAA:
-				datac[k]   += (float) (v * raised[k]);
-				datac[k+1] += (float) (v * raised[k+1]);
+				datac[k]   += (gmt_grdfloat) (v * raised[k]);
+				datac[k+1] += (gmt_grdfloat) (v * raised[k+1]);
 				break;
 			case GRAVFFT_GEOID:
 				if (mk > 0.0) v /= (MGAL_AT_45 * mk);
-				datac[k]   += (float) (v * raised[k]);
-				datac[k+1] += (float) (v * raised[k+1]);
+				datac[k]   += (gmt_grdfloat) (v * raised[k]);
+				datac[k+1] += (gmt_grdfloat) (v * raised[k+1]);
 				break;
 			case GRAVFFT_VGG:
 			 	v *= 1.0e4 * mk;	/* Scale mGal/m to 0.1 mGal/km = 1 Eotvos */
-				datac[k]   += (float) (v * raised[k]);
-				datac[k+1] += (float) (v * raised[k+1]);
+				datac[k]   += (gmt_grdfloat) (v * raised[k]);
+				datac[k+1] += (gmt_grdfloat) (v * raised[k+1]);
 				break;
 			case GRAVFFT_DEFL_EAST:
 				if (mk > 0.0) {	/* Scale tan (xslope) ~ slope to microradians */
 					kx = gmt_fft_any_wave (k, GMT_FFT_K_IS_KX, K);
 					v *= 1.e6 * (-kx / (MGAL_AT_45 * mk));
 				}
-				datac[k]   += (float) (-v * raised[k+1]);
-				datac[k+1] += (float) ( v * raised[k]);
+				datac[k]   += (gmt_grdfloat) (-v * raised[k+1]);
+				datac[k+1] += (gmt_grdfloat) ( v * raised[k]);
 				break;
 			case GRAVFFT_DEFL_NORTH:
 				if (mk > 0.0) {	/* Scale tan (yslope) ~ slope to microradians */
 					ky = gmt_fft_any_wave (k, GMT_FFT_K_IS_KY, K);
 					v *= 1.e6 * (-ky / (MGAL_AT_45 * mk));
 				}
-				datac[k]   += (float) ( v * raised[k+1]);
-				datac[k+1] += (float) (-v * raised[k]);
+				datac[k]   += (gmt_grdfloat) ( v * raised[k+1]);
+				datac[k+1] += (gmt_grdfloat) (-v * raised[k]);
 				break;
 		}
 	}
@@ -962,8 +960,8 @@ GMT_LOCAL int do_admittance (struct GMT_CTRL *GMT, struct GMT_GRID *GridA, struc
 	size_t   n_alloc;
 	double   delta_k, r_delta_k, freq;
 	double  *out = NULL, *err_bar = NULL, *coh = NULL, *b_pow = NULL, *g_pow = NULL, *co_spec = NULL, *quad = NULL;
-	float   *datac = GridA->data;
-	float   *in_grv = GridB->data;
+	gmt_grdfloat   *datac = GridA->data;
+	gmt_grdfloat   *in_grv = GridB->data;
 	double  *z_from_below = NULL, *z_from_top = NULL;
 	struct   GMT_DATASET *D = NULL;
 	struct   GMT_DATASEGMENT *S = NULL;
@@ -1040,6 +1038,7 @@ GMT_LOCAL int do_admittance (struct GMT_CTRL *GMT, struct GMT_GRID *GridA, struc
 		goto Lfree;		/* So that we can free what it need to be */
 	}
 	S = D->table[0]->segment[0];	/* Only one table with one segment here, with 17 cols and nk rows */
+	S->n_rows = nk;
 
 	if (Ctrl->misc.give_wavelength && Ctrl->misc.give_km) delta_k *= 1000.0;	/* Wanted wavelength in km */
 
@@ -1160,7 +1159,7 @@ GMT_LOCAL void load_from_top_admitt (struct GMT_CTRL *GMT, struct GRAVFFT_CTRL *
 	}
 }
 
-GMT_LOCAL void load_from_top_grid (struct GMT_CTRL *GMT, struct GMT_GRID *Grid, struct GRAVFFT_CTRL *Ctrl, struct GMT_FFT_WAVENUMBER *K, float *raised, unsigned int n) {
+GMT_LOCAL void load_from_top_grid (struct GMT_CTRL *GMT, struct GMT_GRID *Grid, struct GRAVFFT_CTRL *Ctrl, struct GMT_FFT_WAVENUMBER *K, gmt_grdfloat *raised, unsigned int n) {
 
 	/* Computes the gravity|geoid grid due to the effect of the bathymetry using the theoretical
 	admittance for the "loading from top" model --  M. McNutt & Shure (1986)  */
@@ -1168,7 +1167,7 @@ GMT_LOCAL void load_from_top_grid (struct GMT_CTRL *GMT, struct GMT_GRID *Grid,
 	unsigned int i;
 	uint64_t k;
 	double	earth_curvature, alfa, D, twopi, t1, t2, f, p, t, mk;
-	float *datac = Grid->data;
+	gmt_grdfloat *datac = Grid->data;
 	gmt_M_unused(GMT);
 
 	f = 1.0;
@@ -1196,12 +1195,12 @@ GMT_LOCAL void load_from_top_grid (struct GMT_CTRL *GMT, struct GMT_GRID *Grid,
 		else                 /* Must be the GEOID case */
 			t1 /= (NORMAL_GRAVITY * mk * twopi);
 		t2 = exp(-twopi * mk * Ctrl->misc.z_level) - exp(-twopi * mk * Ctrl->Z.zm) / (1 + alfa*pow(mk,4.));
-		datac[k] += (float) ((Ctrl->T.rho_cw * t1 * t2) * t / f * raised[k]);
-		datac[k+1] += (float) ((Ctrl->T.rho_cw * t1 * t2) * t / f * raised[k+1]);
+		datac[k] += (gmt_grdfloat) ((Ctrl->T.rho_cw * t1 * t2) * t / f * raised[k]);
+		datac[k+1] += (gmt_grdfloat) ((Ctrl->T.rho_cw * t1 * t2) * t / f * raised[k+1]);
 	}
 }
 
-GMT_LOCAL void load_from_below_grid (struct GMT_CTRL *GMT, struct GMT_GRID *Grid, struct GRAVFFT_CTRL *Ctrl, struct GMT_FFT_WAVENUMBER *K, float *raised, unsigned int n) {
+GMT_LOCAL void load_from_below_grid (struct GMT_CTRL *GMT, struct GMT_GRID *Grid, struct GRAVFFT_CTRL *Ctrl, struct GMT_FFT_WAVENUMBER *K, gmt_grdfloat *raised, unsigned int n) {
 
 	/* Computes the gravity|geoid grid due to the effect of the bathymetry using the theoretical
 	admittance for the "loading from below" model --  M. McNutt & Shure (1986)  */
@@ -1209,7 +1208,7 @@ GMT_LOCAL void load_from_below_grid (struct GMT_CTRL *GMT, struct GMT_GRID *Grid
 	unsigned int i;
 	uint64_t k;
 	double	earth_curvature, alfa, D, twopi, t1, t2, t3, f, p, t, mk;
-	float *datac = Grid->data;
+	gmt_grdfloat *datac = Grid->data;
 	gmt_M_unused(GMT);
 
 	f = 1.0;
@@ -1239,7 +1238,7 @@ GMT_LOCAL void load_from_below_grid (struct GMT_CTRL *GMT, struct GMT_GRID *Grid
 		t2 = Ctrl->T.rho_cw * exp(-twopi * mk * Ctrl->misc.z_level) +
 			 Ctrl->T.rho_mc * exp(-twopi * mk * Ctrl->Z.zm);
 		t3 = -(Ctrl->T.rho_mw + Ctrl->T.rho_mc * pow(mk,4.) * alfa) * exp(-twopi * mk * Ctrl->Z.zl);
-		datac[k] += (float) ((t1 * (t2 + t3)) * t / f * raised[k]);
-		datac[k+1] += (float) ((t1 * (t2 + t3)) * t / f * raised[k+1]);
+		datac[k] += (gmt_grdfloat) ((t1 * (t2 + t3)) * t / f * raised[k]);
+		datac[k+1] += (gmt_grdfloat) ((t1 * (t2 + t3)) * t / f * raised[k+1]);
 	}
 }
