@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *	Copyright (c) 1991-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  * Brief synopsis: project.c reads (x,y,[z]) data and writes some combination of (x,y,z,p,q,u,v),
@@ -23,17 +22,18 @@
  *
  * Author: 	Walter H. F. Smith
  * Date:	1 JAN 2010
- * Version:	5 API
+ * Version:	6 API
  */
 
 #include "gmt_dev.h"
 
-#define THIS_MODULE_NAME	"project"
+#define THIS_MODULE_CLASSIC_NAME	"project"
+#define THIS_MODULE_MODERN_NAME	"project"
 #define THIS_MODULE_LIB		"core"
-#define THIS_MODULE_PURPOSE	"Project table data onto lines or great circles, generate tracks, or translate coordinates"
+#define THIS_MODULE_PURPOSE	"Project data onto lines or great circles, or generate tracks"
 #define THIS_MODULE_KEYS	"<D{,>D},G-("
 #define THIS_MODULE_NEEDS	""
-#define THIS_MODULE_OPTIONS "-:>Vbdefghis" GMT_OPT("HMm")
+#define THIS_MODULE_OPTIONS "-:>Vbdefghios" GMT_OPT("HMm")
 
 #define PROJECT_N_FARGS	7
 
@@ -57,7 +57,7 @@ struct PROJECT_CTRL {	/* All control options for this program (except common arg
 	} F;
 	struct G {	/* -G<inc>[/<colat>][+h] */
 		bool active;
-		unsigned int header;
+		bool header;
 		unsigned int mode;
 		double inc;
 		double colat;
@@ -84,6 +84,11 @@ struct PROJECT_CTRL {	/* All control options for this program (except common arg
 		bool active;
 		double min, max;
 	} W;
+	struct Z {	/* -Z<major/minor/azimuth>[+e] */
+		bool active;
+		bool exact;
+		double major, minor, azimuth;
+	} Z;
 };
 
 struct PROJECT_DATA {
@@ -278,9 +283,9 @@ GMT_LOCAL void sphere_project_setup (struct GMT_CTRL *GMT, double alat, double a
 }
 
 GMT_LOCAL void flat_project_setup (double alat, double alon, double blat, double blon, double plat, double plon, double *azim, double *e, bool two_pts, bool pole_set) {
-	/* Sets up stuff for rotation of cartesian 2-vectors, analogous
+	/* Sets up stuff for rotation of Cartesian 2-vectors, analogous
 	   to the spherical three vector stuff above.
-	   Output is the negative cartesian azimuth in degrees.
+	   Output is the negative Cartesian azimuth in degrees.
 	   Latitudes and longitudes are in degrees. */
 
 	if (two_pts)
@@ -294,18 +299,6 @@ GMT_LOCAL void flat_project_setup (double alat, double alon, double blat, double
 	e[2] = -e[1];
 }
 
-GMT_LOCAL void copy_text_from_col3 (char *line, char *z_cols) {
-	/* returns the input line starting at the 3rd column */
-
-	unsigned int i;
-
-	/* First replace any commas with spaces */
-
-	for (i = 0; line[i]; i++) if (line[i] == ',') line[i] = ' ';
-
-	sscanf (line, "%*s %*s %[^\n]", z_cols);
-}
-
 GMT_LOCAL void *New_Ctrl (struct GMT_CTRL *GMT) {	/* Allocate and initialize a new control structure */
 	struct PROJECT_CTRL *C;
 
@@ -323,11 +316,12 @@ GMT_LOCAL void Free_Ctrl (struct GMT_CTRL *GMT, struct PROJECT_CTRL *C) {	/* Dea
 }
 
 GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
-	gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
+	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: project [<table>] -C<ox>/<oy> [-A<azimuth>] [-E<bx>/<by>] [-F<flags>] [-G<dist>[/<colat>][+h]]\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t[-L[w|<l_min>/<l_max>]] [-N] [-Q] [-S] [-T<px>/<py>] [%s] [-W<w_min>/<w_max>]\n", GMT_V_OPT);
-	GMT_Message (API, GMT_TIME_NONE, "\t[%s] [%s] [%s] [%s] [%s]\n\t[%s] [%s]\n\t[%s] [%s]\n\n", GMT_b_OPT, GMT_d_OPT, GMT_e_OPT, GMT_f_OPT, GMT_g_OPT, GMT_h_OPT, GMT_i_OPT, GMT_s_OPT, GMT_colon_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "usage: %s [<table>] -C<ox>/<oy> [-A<azimuth>] [-E<bx>/<by>] [-F<flags>] [-G<dist>[/<colat>][+h]]\n", name);
+	GMT_Message (API, GMT_TIME_NONE, "\t[-L[w|<l_min>/<l_max>]] [-N] [-Q] [-S] [-T<px>/<py>] [%s] [-W<w_min>/<w_max>] [-Z<major>/<minor>/<azimuth>[+e]]\n", GMT_V_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "\t[%s] [%s] [%s] [%s] [%s]\n\t[%s] [%s]\n\t[%s] [%s] [%s] [%s]\n\n",
+		GMT_b_OPT, GMT_d_OPT, GMT_e_OPT, GMT_f_OPT, GMT_g_OPT, GMT_h_OPT, GMT_i_OPT, GMT_o_OPT, GMT_s_OPT, GMT_colon_OPT, GMT_PAR_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
@@ -341,7 +335,7 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t   of the (p,q) system (line q = 0) passes through -C and makes an angle\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   <azimuth> with North (case 1), or passes through -E (case 2), or is\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   determined by the pole -T (case 3).  In (3), point -C need not be on equator.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   In a cartesian [-N option] projection, p = q = 0 at -O in all cases;\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   In a Cartesian [-N option] projection, p = q = 0 at -O in all cases;\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   (1) and (2) orient the p axis, while (3) orients the q axis.\n\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-C Set the location of the center to be <ox>/<oy>.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\n\tOPTIONS:\n");
@@ -367,7 +361,7 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t   -Lw will use only those points Within the span from C to E (Must have set -E).\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   -L<l_min>/<l_max> will only use points whose p is [l_min <= p <= l_max].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   Default uses all points.  Note p = 0 at C and increases toward E in <azimuth> direction.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-N Flat Earth mode; a cartesian projection is made.  Default is spherical.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t-N Flat Earth mode; a Cartesian projection is made.  Default is spherical.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-Q Convert to Map units, so x,y,r,s are degrees,\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   while p,q,dist,l_min,l_max,w_min,w_max are km.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   If not set, then p,q,dist,l_min,l_max,w_min,w_max are assumed to be in same units as x,y,r,s.\n");
@@ -377,7 +371,9 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t-W Check the width across the projected track and use only certain points.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   This will use only those points whose q is [w_min <= q <= w_max].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   Note that q is positive to your LEFT as you walk from C toward E in <azimuth> direction.\n");
-	GMT_Option (API, "bi2,bo,d,e,f,g,h,i,s,:,.");
+	GMT_Message (API, GMT_TIME_NONE, "\t-Z With -G and -C, generate an ellipse of given major and minor axes (in km if geographic) and azimuth\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   of major axis. Append +e for adjusting increment to fix perimeter exactly [use increment as given in -G].\n");
+	GMT_Option (API, "bi2,bo,d,e,f,g,h,i,o,s,:,.");
 
 	return (GMT_MODULE_USAGE);
 }
@@ -391,7 +387,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct PROJECT_CTRL *Ctrl, struct GMT
 
 	unsigned int n_errors = 0, j, k;
 	size_t len;
-	char txt_a[GMT_LEN64] = {""}, txt_b[GMT_LEN64] = {""};
+	char txt_a[GMT_LEN64] = {""}, txt_b[GMT_LEN64] = {""}, *ce = NULL, *ch = NULL;
 	struct GMT_OPTION *opt = NULL;
 	struct GMTAPI_CTRL *API = GMT->parent;
 
@@ -417,14 +413,14 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct PROJECT_CTRL *Ctrl, struct GMT
 					n_errors++;
 				}
 				else {
-					n_errors += gmt_verify_expectations (GMT, GMT->current.io.col_type[GMT_IN][GMT_X], gmt_scanf_arg (GMT, txt_a, GMT->current.io.col_type[GMT_IN][GMT_X], &Ctrl->C.x), txt_a);
-					n_errors += gmt_verify_expectations (GMT, GMT->current.io.col_type[GMT_IN][GMT_Y], gmt_scanf_arg (GMT, txt_b, GMT->current.io.col_type[GMT_IN][GMT_Y], &Ctrl->C.y), txt_b);
+					n_errors += gmt_verify_expectations (GMT, gmt_M_type (GMT, GMT_IN, GMT_X), gmt_scanf_arg (GMT, txt_a, gmt_M_type (GMT, GMT_IN, GMT_X), false, &Ctrl->C.x), txt_a);
+					n_errors += gmt_verify_expectations (GMT, gmt_M_type (GMT, GMT_IN, GMT_Y), gmt_scanf_arg (GMT, txt_b, gmt_M_type (GMT, GMT_IN, GMT_Y), false, &Ctrl->C.y), txt_b);
 					if (n_errors) GMT_Report (API, GMT_MSG_NORMAL, "Syntax error -C option: Undecipherable argument %s\n", opt->arg);
 				}
 				break;
 			case 'D':
 				if (gmt_M_compat_check (GMT, 4)) {
-					GMT_Report (API, GMT_MSG_COMPAT, "Warning: Option -D is deprecated; use --FORMAT_GEO_OUT instead\n");
+					GMT_Report (API, GMT_MSG_COMPAT, "Option -D is deprecated; use --FORMAT_GEO_OUT instead\n");
 					if (opt->arg[0] == 'g') GMT->current.io.geo.range = GMT_IS_0_TO_P360_RANGE;
 					if (opt->arg[0] == 'd') GMT->current.io.geo.range = GMT_IS_M180_TO_P180_RANGE;
 				}
@@ -438,8 +434,8 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct PROJECT_CTRL *Ctrl, struct GMT
 					n_errors++;
 				}
 				else {
-					n_errors += gmt_verify_expectations (GMT, GMT->current.io.col_type[GMT_IN][GMT_X], gmt_scanf_arg (GMT, txt_a, GMT->current.io.col_type[GMT_IN][GMT_X], &Ctrl->E.x), txt_a);
-					n_errors += gmt_verify_expectations (GMT, GMT->current.io.col_type[GMT_IN][GMT_Y], gmt_scanf_arg (GMT, txt_b, GMT->current.io.col_type[GMT_IN][GMT_Y], &Ctrl->E.y), txt_b);
+					n_errors += gmt_verify_expectations (GMT, gmt_M_type (GMT, GMT_IN, GMT_X), gmt_scanf_arg (GMT, txt_a, gmt_M_type (GMT, GMT_IN, GMT_X), false, &Ctrl->E.x), txt_a);
+					n_errors += gmt_verify_expectations (GMT, gmt_M_type (GMT, GMT_IN, GMT_Y), gmt_scanf_arg (GMT, txt_b, gmt_M_type (GMT, GMT_IN, GMT_Y), false, &Ctrl->E.y), txt_b);
 					if (n_errors) GMT_Report (API, GMT_MSG_NORMAL, "Syntax error -E option: Undecipherable argument %s\n", opt->arg);
 				}
 				break;
@@ -461,23 +457,24 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct PROJECT_CTRL *Ctrl, struct GMT
 				break;
 			case 'G':
 				Ctrl->G.active = true;
-				len = strlen (opt->arg) - 1;
-				if (len > 1 && opt->arg[len-1] == '+' && opt->arg[len] == 'h') {
-					Ctrl->G.header = (int)len-1;	/* Wish to place a segment header on output via +h */
-					opt->arg[len-1] = 0;	/* Temporarily remove the trailing + sign */
+				if ((ch = strstr (opt->arg, "+h"))) {
+					Ctrl->G.header = true;
+					ch[0] = '\0';	/* Chop off +h */
 				}
-				else if (len > 0 && opt->arg[len] == '+') {
+				len = strlen (opt->arg) - 1;
+				if (len > 0 && opt->arg[len] == '+') {	/* Obsolete way to say +h */
 					Ctrl->G.header = true;	/* Wish to place a segment header on output */
 					opt->arg[len] = 0;	/* Temporarily remove the trailing + sign */
+					ch = &opt->arg[len];
 				}
 				if (sscanf (opt->arg, "%[^/]/%s", txt_a, txt_b) == 2) {	/* Got dist/colat */
 					Ctrl->G.mode = 1;
 					Ctrl->G.inc = atof (txt_a);
-					n_errors += gmt_verify_expectations (GMT, GMT->current.io.col_type[GMT_IN][GMT_Y], gmt_scanf_arg (GMT, txt_b, GMT->current.io.col_type[GMT_IN][GMT_Y], &Ctrl->G.colat), txt_b);
+					n_errors += gmt_verify_expectations (GMT, gmt_M_type (GMT, GMT_IN, GMT_Y), gmt_scanf_arg (GMT, txt_b, gmt_M_type (GMT, GMT_IN, GMT_Y), false, &Ctrl->G.colat), txt_b);
 				}
 				else
 					Ctrl->G.inc = atof (opt->arg);
-				if (Ctrl->G.header) opt->arg[Ctrl->G.header] = '+';	/* Restore the plus-sign */
+				if (ch) ch[0] = '+';	/* Restore the plus-sign */
 				break;
 			case 'L':
 				Ctrl->L.active = true;
@@ -503,8 +500,8 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct PROJECT_CTRL *Ctrl, struct GMT
 					n_errors++;
 				}
 				else {
-					n_errors += gmt_verify_expectations (GMT, GMT->current.io.col_type[GMT_IN][GMT_X], gmt_scanf_arg (GMT, txt_a, GMT->current.io.col_type[GMT_IN][GMT_X], &Ctrl->T.x), txt_a);
-					n_errors += gmt_verify_expectations (GMT, GMT->current.io.col_type[GMT_IN][GMT_Y], gmt_scanf_arg (GMT, txt_b, GMT->current.io.col_type[GMT_IN][GMT_Y], &Ctrl->T.y), txt_b);
+					n_errors += gmt_verify_expectations (GMT, gmt_M_type (GMT, GMT_IN, GMT_X), gmt_scanf_arg (GMT, txt_a, gmt_M_type (GMT, GMT_IN, GMT_X), false, &Ctrl->T.x), txt_a);
+					n_errors += gmt_verify_expectations (GMT, gmt_M_type (GMT, GMT_IN, GMT_Y), gmt_scanf_arg (GMT, txt_b, gmt_M_type (GMT, GMT_IN, GMT_Y), false, &Ctrl->T.y), txt_b);
 					if (n_errors) GMT_Report (API, GMT_MSG_NORMAL, "Syntax error -T option: Undecipherable argument %s\n", opt->arg);
 				}
 				break;
@@ -513,7 +510,15 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct PROJECT_CTRL *Ctrl, struct GMT
 				n_errors += gmt_M_check_condition (GMT, sscanf (opt->arg, "%lf/%lf", &Ctrl->W.min, &Ctrl->W.max) != 2,
 				                                 "Syntax error: Expected -W<min>/<max>\n");
 				break;
-
+			case 'Z': /* Parameters of ellipse */
+				Ctrl->Z.active = true;
+				if ((ce = strstr (opt->arg, "+e"))) {
+					Ctrl->Z.exact = true;
+					ce[0] = '\0';	/* Chop off +e */
+				}
+				n_errors += gmt_M_check_condition (GMT, sscanf(opt->arg, "%lf/%lf/%lf", &Ctrl->Z.major, &Ctrl->Z.minor, &Ctrl->Z.azimuth) != 3, "Syntax error: Expected -Z<major/minor/azimuth>\n");
+				if (ce) ce[0] = '+';	/* Restore the plus-sign */
+				break;
 			default:	/* Report bad options */
 				n_errors += gmt_default_error (GMT, opt->option);
 				break;
@@ -533,7 +538,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct PROJECT_CTRL *Ctrl, struct GMT
 	                                 "Syntax error: Specify only one of -A, -E, and -T\n");
 	n_errors += gmt_M_check_condition (GMT, Ctrl->E.active && (Ctrl->C.x == Ctrl->E.x) && (Ctrl->C.y == Ctrl->E.y),
 	                                 "Syntax error -E option: Second point must differ from origin!\n");
-	n_errors += gmt_M_check_condition (GMT, Ctrl->G.active && Ctrl->L.min == Ctrl->L.max && !Ctrl->E.active,
+	n_errors += gmt_M_check_condition (GMT, Ctrl->G.active && Ctrl->L.min == Ctrl->L.max && !(Ctrl->E.active || Ctrl->Z.active),
 	                                 "Syntax error -G option: Must also specify -Lmin/max or use -E instead\n");
 	n_errors += gmt_M_check_condition (GMT, Ctrl->G.active && Ctrl->F.active,
 	                                 "Syntax error -G option: -F not allowed [Defaults to rsp]\n");
@@ -550,11 +555,10 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct PROJECT_CTRL *Ctrl, struct GMT
 	return (n_errors ? GMT_PARSE_ERROR : GMT_NOERROR);
 }
 
-GMT_LOCAL int write_one_segment (struct GMT_CTRL *GMT, struct PROJECT_CTRL *Ctrl, double theta, struct PROJECT_DATA *p_data, bool pure_ascii, struct PROJECT_INFO *P) {
-	int error;
+GMT_LOCAL int write_one_segment (struct GMT_CTRL *GMT, struct PROJECT_CTRL *Ctrl, double theta, struct PROJECT_DATA *p_data, struct PROJECT_INFO *P) {
 	uint64_t col, n_items, rec, k;
 	double sin_theta, cos_theta, e[9], x[3], xt[3], *out = NULL;
-	char record[GMT_BUFSIZ] = {""}, text[GMT_BUFSIZ] = {""};
+	struct GMT_RECORD *Out = NULL;
 
 	if (Ctrl->S.active) qsort (p_data, P->n_used, sizeof (struct PROJECT_DATA), compare_distances);
 
@@ -588,44 +592,26 @@ GMT_LOCAL int write_one_segment (struct GMT_CTRL *GMT, struct PROJECT_CTRL *Ctrl
 		}
 	}
 
-	n_items = P->n_outputs + ((P->want_z_output && P->n_z) ? P->n_z - 1 : 0);
+	n_items = gmt_get_cols (GMT, GMT_OUT);
 	out = gmt_M_memory (GMT, NULL, n_items, double);
+	Out = gmt_new_record (GMT, out, NULL);	/* text will be set below */
 
-	if (P->first && (error = gmt_set_cols (GMT, GMT_OUT, n_items)) != 0) return (error);
+	/* Now output this segment */
 
-	/* Now output  */
-
-	/* Special case for pure ASCII since we may pass text */
-
-	if (P->n_z && pure_ascii) {
-		for (rec = 0; rec < P->n_used; rec++) {
-			record[0] = 0;
-			for (col = 0; col < P->n_outputs; col++) {
-				if (P->output_choice[col] == -1)	/* Output all z columns as one string */
-					strncat (record, p_data[rec].t, GMT_BUFSIZ-1);
-				else {
-					sprintf (text, GMT->current.setting.format_float_out, p_data[rec].a[P->output_choice[col]]);
-					strcat (record, text);
-				}
-				(col == (P->n_outputs - 1)) ? strcat (record, "\n") : strcat (record, GMT->current.setting.io_col_separator);
+	for (rec = 0; rec < P->n_used; rec++) {
+		for (col = k = 0; col < P->n_outputs; col++) {
+			if (P->output_choice[col] == -1) {	/* Copy over all z columns */
+				gmt_M_memcpy (&out[k], p_data[rec].z, P->n_z, double);
+				k += P->n_z;
 			}
-			GMT_Put_Record (GMT->parent, GMT_WRITE_TEXT, record);	/* Write this to output */
-		}
-	}
-	else {	/* Any other i/o combination */
-		for (rec = 0; rec < P->n_used; rec++) {
-			for (col = k = 0; col < P->n_outputs; col++) {
-				if (P->output_choice[col] == -1) {	/* Copy over all z columns */
-					gmt_M_memcpy (&out[k], p_data[rec].z, P->n_z, double);
-					k += P->n_z;
-				}
-				else
-					out[k++] = p_data[rec].a[P->output_choice[col]];
-			}
-			GMT_Put_Record (GMT->parent, GMT_WRITE_DATA, out);	/* Write this to output */
+			else
+				out[k++] = p_data[rec].a[P->output_choice[col]];
 		}
+		Out->text = p_data[rec].t;	/* The trailing text (may be NULL) */
+		GMT_Put_Record (GMT->parent, GMT_WRITE_DATA, Out);	/* Write this to output */
 	}
 	gmt_M_free (GMT, out);
+	gmt_M_free (GMT, Out);
 	return (0);
 }
 
@@ -634,8 +620,7 @@ GMT_LOCAL int write_one_segment (struct GMT_CTRL *GMT, struct PROJECT_CTRL *Ctrl
 
 int GMT_project (void *V_API, int mode, void *args) {
 	uint64_t rec, n_total_read, col, n_total_used = 0;
-	unsigned int rmode;
-	bool skip, z_first = true;
+	bool skip, z_first = true, z_set_auto = false;
 	int error = 0;
 
 	size_t n_alloc = GMT_CHUNK;
@@ -657,12 +642,11 @@ int GMT_project (void *V_API, int mode, void *args) {
 	if (mode == GMT_MODULE_PURPOSE) return (usage (API, GMT_MODULE_PURPOSE));	/* Return the purpose of program */
 	options = GMT_Create_Options (API, mode, args);	if (API->error) return (API->error);	/* Set or get option list */
 
-	if (!options || options->option == GMT_OPT_USAGE) bailout (usage (API, GMT_USAGE));/* Return the usage message */
-	if (options->option == GMT_OPT_SYNOPSIS) bailout (usage (API, GMT_SYNOPSIS));	/* Return the synopsis */
+	if ((error = gmt_report_usage (API, options, 0, usage)) != GMT_NOERROR) bailout (error);	/* Give usage if requested */
 
 	/* Parse the command-line arguments */
 
-	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
 	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
 	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
 	if ((error = parse (GMT, Ctrl, options)) != 0) Return (error);
@@ -689,7 +673,7 @@ int GMT_project (void *V_API, int mode, void *args) {
 	/* Convert user's -F choices to internal parameters */
 	for (col = P.n_outputs = 0; col < PROJECT_N_FARGS && Ctrl->F.col[col]; col++) {
 		switch (Ctrl->F.col[col]) {
-			case 'z':	/* Special flag, can mean any number of z columns */
+			case 'z':	/* Special flag, can mean any number of z columns and trailing text */
 				P.output_choice[col] = -1;
 				P.want_z_output = true;
 				break;
@@ -719,25 +703,26 @@ int GMT_project (void *V_API, int mode, void *args) {
 		P.n_outputs++;
 	}
 
-	if (P.n_outputs == 0 && !Ctrl->G.active) {	/* Generate default -F setting (all) */
+	if (P.n_outputs == 0 && !Ctrl->G.active) {	/* Generate default -F setting (xyzpqrs) */
 		P.n_outputs = PROJECT_N_FARGS;
-		for (col = 0; col < 2; col++) P.output_choice[col] = (int)col;
-		P.output_choice[2] = -1;
-		for (col = 3; col < P.n_outputs; col++) P.output_choice[col] = (int)col - 1;
+		for (col = 0; col < 2; col++) P.output_choice[col] = (int)col;	/* Do xy */
+		P.output_choice[2] = -1;	/* Do z as col 2 */
+		P.want_z_output = z_set_auto = true;
+		for (col = 3; col < P.n_outputs; col++) P.output_choice[col] = (int)col - 1;	/* Do pqrs */
 		P.find_new_point = true;
 	}
 	if (Ctrl->G.active) {	/* Hardwire 3 output columns and set their types */
 		P.n_outputs = 3;
-		GMT->current.io.col_type[GMT_OUT][GMT_X] = (Ctrl->N.active) ? GMT_IS_FLOAT : GMT_IS_LON;
-		GMT->current.io.col_type[GMT_OUT][GMT_Y] = (Ctrl->N.active) ? GMT_IS_FLOAT : (Ctrl->N.active) ? GMT_IS_FLOAT : GMT_IS_LAT;
-		GMT->current.io.col_type[GMT_OUT][GMT_Z] = GMT_IS_FLOAT;
+		gmt_set_column (GMT, GMT_OUT, GMT_X, (Ctrl->N.active) ? GMT_IS_FLOAT : GMT_IS_LON);
+		gmt_set_column (GMT, GMT_OUT, GMT_Y, (Ctrl->N.active) ? GMT_IS_FLOAT : GMT_IS_LAT);
+		gmt_set_column (GMT, GMT_OUT, GMT_Z, GMT_IS_FLOAT);
 	}
 	else {	/* Decode and set the various output column types */
 		for (col = 0; col < P.n_outputs; col++) {
 			switch (P.output_choice[col]) {
-				case 0: case 4: GMT->current.io.col_type[GMT_OUT][col] = GMT_IS_LON;	break;
-				case 1: case 5: GMT->current.io.col_type[GMT_OUT][col] = GMT_IS_LAT;	break;
-				default: 	GMT->current.io.col_type[GMT_OUT][col] = GMT_IS_FLOAT;	break;
+				case 0: case 4: gmt_set_column (GMT, GMT_OUT, (unsigned int)col, GMT_IS_LON);		break;
+				case 1: case 5: gmt_set_column (GMT, GMT_OUT, (unsigned int)col, GMT_IS_LAT);		break;
+				default: 	    gmt_set_column (GMT, GMT_OUT, (unsigned int)col, GMT_IS_FLOAT);	break;
 			}
 		}
 	}
@@ -750,7 +735,7 @@ int GMT_project (void *V_API, int mode, void *args) {
 	if (Ctrl->N.active) {	/* Flat Earth mode */
 		theta = Ctrl->A.azimuth;
 		flat_project_setup (Ctrl->C.y, Ctrl->C.x, Ctrl->E.y, Ctrl->E.x, Ctrl->T.y, Ctrl->T.x, &theta, e, Ctrl->E.active, Ctrl->T.active);
-		/* Azimuth (theta) is now cartesian in degrees */
+		/* Azimuth (theta) is now Cartesian in degrees */
 		if (Ctrl->L.constrain) {
 			Ctrl->L.min = 0.0;
 			xx = Ctrl->E.x - Ctrl->C.x;
@@ -834,13 +819,24 @@ int GMT_project (void *V_API, int mode, void *args) {
 
 	P.n_used = n_total_read = 0;
 
-	if (Ctrl->G.active) {	/* Not input data expected, just generate x,y,d track from arguments given */
+	if (Ctrl->G.active) {	/* No input data expected, just generate x,y,d track from arguments given */
 		double out[3];
+		struct GMT_RECORD *Out = gmt_new_record (GMT, out, NULL);
+		char *z_header = NULL;
 		P.output_choice[0] = 4;
 		P.output_choice[1] = 5;
 		P.output_choice[2] = 2;
+		if (Ctrl->Z.active) {	/* Do the full ellipse */
+			double h = pow (Ctrl->Z.major - Ctrl->Z.minor, 2.0) / pow (Ctrl->Z.major + Ctrl->Z.minor, 2.0);
+			Ctrl->L.min = 0.0;
+			Ctrl->L.max = M_PI * (Ctrl->Z.major + Ctrl->Z.minor) * (1.0 + (3.0 * h)/(10.0 + sqrt (4.0 - 3.0 * h)));	/* Ramanujan approximation of ellipse circumference */
+			if (Ctrl->Z.exact) {	/* Adjust inc to fit the ellipse perimeter exactly */
+				double f = rint (Ctrl->L.max / Ctrl->G.inc);
+				Ctrl->G.inc = Ctrl->L.max / f;
+			}
+		}
 
-		GMT_Report (API, GMT_MSG_VERBOSE, "Generate table data\n");
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Generate table data\n");
 		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Go from min dist = %g to max dist = %g\n", Ctrl->L.min, Ctrl->L.max);
 		d_along = Ctrl->L.min;
 		while ((Ctrl->L.max - d_along) > (GMT_CONV8_LIMIT*Ctrl->G.inc)) {
@@ -863,7 +859,35 @@ int GMT_project (void *V_API, int mode, void *args) {
 
 		/* We need to find r,s  */
 
-		if (Ctrl->N.active) {
+		if (Ctrl->Z.active) {
+			uint64_t ne = urint (Ctrl->L.max / Ctrl->G.inc);
+			if (Ctrl->N.active) {	/* Cartesian ellipse */
+				double r, ca, sa, d_azim = TWO_PI / ne, e2 = 1.0 - pow (Ctrl->Z.minor/Ctrl->Z.major, 2.0);
+				sincosd (Ctrl->Z.azimuth - 90.0, &sin_theta, &cos_theta);
+				e[0] = e[3] = cos_theta;
+				e[1] = sin_theta;
+				e[2] = -e[1];
+				for (rec = 0; rec < P.n_used; rec++) {
+					sincos (d_azim*rec, &sa, &ca);
+					r = Ctrl->Z.minor / sqrt (1.0 - e2 * pow (ca, 2.0));
+					x[0] = r * ca;	x[1] = r * sa;
+					matrix_2v (e, x, xt);
+					p_data[rec].a[4] = Ctrl->C.x + xt[GMT_X];
+					p_data[rec].a[5] = Ctrl->C.y + xt[GMT_Y];
+				}
+				z_header = strdup ("Testing Cartesian ellipse");
+			}
+			else {	/* Geographic ellipse */
+				struct GMT_DATASEGMENT *S = gmt_get_geo_ellipse (GMT, Ctrl->C.x, Ctrl->C.y, Ctrl->Z.major, Ctrl->Z.minor, Ctrl->Z.azimuth, ne);
+				for (rec = 0; rec < P.n_used; rec++) {
+					p_data[rec].a[4] = S->coord[GMT_X][rec];
+					p_data[rec].a[5] = S->coord[GMT_Y][rec];
+				}
+				z_header = strdup (S->header);
+				gmt_free_segment (GMT, &S);
+			}
+		}
+		else if (Ctrl->N.active) {
 			sincosd (90.0 + theta, &sin_theta, &cos_theta);
 			for (rec = 0; rec < P.n_used; rec++) {
 				p_data[rec].a[4] = Ctrl->C.x + p_data[rec].a[2] * cos_theta;
@@ -898,23 +922,23 @@ int GMT_project (void *V_API, int mode, void *args) {
 
 		/* Now output generated track */
 
-		if ((error = gmt_set_cols (GMT, GMT_OUT, P.n_outputs)) != GMT_NOERROR) {
+		if ((error = GMT_Set_Columns (API, GMT_OUT, (unsigned int)P.n_outputs, GMT_COL_FIX_NO_TEXT)) != GMT_NOERROR) {
 			Return (error);
 		}
-		if (GMT_Init_IO (API, GMT_IS_DATASET, GMT_IS_POINT, GMT_OUT, GMT_ADD_DEFAULT, 0, options) != GMT_NOERROR) {	/* Registers data output */
+		if (GMT_Init_IO (API, GMT_IS_DATASET, GMT_IS_POINT, GMT_OUT, GMT_ADD_DEFAULT, 0, options) != GMT_NOERROR) {	/* Registers data output failed */
 			for (rec = 0; rec < P.n_used; rec++) {
-				gmt_M_free (GMT, p_data[rec].t);	gmt_M_free (GMT, p_data[rec].z);
+				gmt_M_str_free (p_data[rec].t);	gmt_M_free (GMT, p_data[rec].z);
 			}
 			gmt_M_free (GMT, p_data);
 			Return (API->error);
 		}
-		if (GMT_Begin_IO (API, GMT_IS_DATASET, GMT_OUT, GMT_HEADER_ON) != GMT_NOERROR) {	/* Enables data output and sets access mode */
+		if (GMT_Begin_IO (API, GMT_IS_DATASET, GMT_OUT, GMT_HEADER_ON) != GMT_NOERROR) {	/* Failed to enable data output and set access mode */
 			if (GMT_Set_Geometry (API, GMT_OUT, GMT_IS_LINE) != GMT_NOERROR) {	/* Sets output geometry */
 				gmt_M_free (GMT, p_data);
 				Return (API->error);
 			}
 			for (rec = 0; rec < P.n_used; rec++) {
-				gmt_M_free (GMT, p_data[rec].t);	gmt_M_free (GMT, p_data[rec].z);
+				gmt_M_str_free (p_data[rec].t);	gmt_M_free (GMT, p_data[rec].z);
 			}
 			gmt_M_free (GMT, p_data);
 			Return (API->error);
@@ -924,22 +948,25 @@ int GMT_project (void *V_API, int mode, void *args) {
 			int kind = (doubleAlmostEqualZero (Ctrl->G.colat, 90.0)) ? 0 : 1;
 			char *type[2] = {"Great", "Small"};
 			gmt_set_segmentheader (GMT, GMT_OUT, true);	/* Turn on segment headers on output */
-			sprintf (GMT->current.io.segment_header, "%s-circle Pole at %g %g", type[kind], P.plon, P.plat);
+			if (Ctrl->Z.active)
+				sprintf (GMT->current.io.segment_header, "%s", z_header);
+			else
+				sprintf (GMT->current.io.segment_header, "%s-circle Pole at %g %g", type[kind], P.plon, P.plat);
 			GMT_Put_Record (API, GMT_WRITE_SEGMENT_HEADER, NULL);	/* Write segment header */
 		}
+		if (Ctrl->Z.active) gmt_M_str_free (z_header);
 		for (rec = 0; rec < P.n_used; rec++) {
 			for (col = 0; col < P.n_outputs; col++) out[col] = p_data[rec].a[P.output_choice[col]];
-			GMT_Put_Record (API, GMT_WRITE_DATA, out);
+			GMT_Put_Record (API, GMT_WRITE_DATA, Out);
 		}
+		gmt_M_free (GMT, Out);
 	}
 	else {	/* Must read input file */
-		bool pure_ascii = false;
-		enum GMT_enum_family family;
-		enum GMT_enum_geometry geometry;
+		struct GMT_RECORD *In = NULL;
 
-		GMT_Report (API, GMT_MSG_VERBOSE, "Processing input table data\n");
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Processing input table data\n");
 		/* Specify input and output expected columns */
-		if ((error = gmt_set_cols (GMT, GMT_IN, 0)) != GMT_NOERROR) {
+		if ((error = GMT_Set_Columns (API, GMT_IN, 0, GMT_COL_FIX)) != GMT_NOERROR) {
 			Return (error);
 		}
 
@@ -950,19 +977,11 @@ int GMT_project (void *V_API, int mode, void *args) {
 		if (GMT_Begin_IO (API, GMT_IS_DATASET, GMT_IN, GMT_HEADER_ON) != GMT_NOERROR) {	/* Enables data input and sets access mode */
 			Return (API->error);
 		}
-		pure_ascii = gmt_is_ascii_record (GMT, options);
-
-		rmode = (pure_ascii && gmt_get_cols (GMT, GMT_IN) >= 2) ? GMT_READ_MIXED : GMT_READ_DATA;
-		family = (pure_ascii) ? GMT_IS_TEXTSET : GMT_IS_DATASET;
-		geometry = (pure_ascii) ? GMT_IS_NONE : GMT_IS_POINT;
 		
-		if ((error = gmt_set_cols (GMT, GMT_OUT, P.n_outputs)) != GMT_NOERROR) {
-			Return (error);
-		}
-		if (GMT_Init_IO (API, family, geometry, GMT_OUT, GMT_ADD_DEFAULT, 0, options) != GMT_NOERROR) {	/* Registers data output */
+		if (GMT_Init_IO (API, GMT_IS_DATASET, GMT_IS_POINT, GMT_OUT, GMT_ADD_DEFAULT, 0, options) != GMT_NOERROR) {	/* Registers data output */
 			Return (API->error);
 		}
-		if (GMT_Begin_IO (API, family, GMT_OUT, GMT_HEADER_ON) != GMT_NOERROR) {	/* Enables data output and sets access mode */
+		if (GMT_Begin_IO (API, GMT_IS_DATASET, GMT_OUT, GMT_HEADER_ON) != GMT_NOERROR) {	/* Enables data output and sets access mode */
 			Return (API->error);
 		}
 		if (GMT_Set_Geometry (API, GMT_OUT, GMT_IS_POINT) != GMT_NOERROR) {	/* Sets output geometry */
@@ -970,7 +989,7 @@ int GMT_project (void *V_API, int mode, void *args) {
 		}
 
 		do {	/* Keep returning records until we reach EOF */
-			if ((in = GMT_Get_Record (API, rmode, NULL)) == NULL) {	/* Read next record, get NULL if special case */
+			if ((In = GMT_Get_Record (API, GMT_READ_MIXED, NULL)) == NULL) {	/* Read next record, get NULL if special case */
 				if (gmt_M_rec_is_error (GMT)) {		/* Bail if there are any read errors */
 					Return (GMT_RUNTIME_ERROR);
 				}
@@ -978,7 +997,7 @@ int GMT_project (void *V_API, int mode, void *args) {
 					GMT_Put_Record (API, GMT_WRITE_TABLE_HEADER, NULL);
 				else if (gmt_M_rec_is_segment_header (GMT)) {			/* Echo segment headers */
 					if (P.n_used) {	/* Write out previous segment */
-						if ((error = write_one_segment (GMT, Ctrl, theta, p_data, pure_ascii, &P)) != 0) Return (error);
+						if ((error = write_one_segment (GMT, Ctrl, theta, p_data, &P)) != 0) Return (error);
 						n_total_used += P.n_used;
 						P.n_used = 0;
 					}
@@ -990,16 +1009,53 @@ int GMT_project (void *V_API, int mode, void *args) {
 			}
 
 			/* Data record to process */
+			in = In->data;	/* Only need to process numerical part here */
 
 			if (z_first) {
-				uint64_t n_cols = gmt_get_cols (GMT, GMT_IN);
-				if (n_cols == 2 && P.want_z_output) {
-					GMT_Report (API, GMT_MSG_NORMAL, "No data columns, cannot use z flag in -F\n");
+				uint64_t n_cols = gmt_get_cols (GMT, GMT_IN), n_tot_cols;
+				if (n_cols == 2 && P.want_z_output && In->text == NULL) {
+					if (z_set_auto) {	/* Implicitly set -Fxyzpqrs earlier but input file has no z values so roll back to -Fxypqrs */
+						P.want_z_output = z_set_auto = false;
+						for (col = 3; col < P.n_outputs; col++) P.output_choice[col-1] = P.output_choice[col];	/* Shuffle pqrs to the left */
+						P.n_outputs--;
+						for (col = 0; col < P.n_outputs; col++) {
+							switch (P.output_choice[col]) {
+								case 0: case 4: gmt_set_column (GMT, GMT_OUT, (unsigned int)col, GMT_IS_LON);	break;
+								case 1: case 5: gmt_set_column (GMT, GMT_OUT, (unsigned int)col, GMT_IS_LAT);	break;
+								default: gmt_set_column (GMT, GMT_OUT, (unsigned int)col, GMT_IS_FLOAT);	break;
+							}
+						}
+					}
+					else {
+						GMT_Report (API, GMT_MSG_NORMAL, "No data columns or trailing text after leading coordinates, cannot use z flag in -F\n");
+						Return (GMT_RUNTIME_ERROR);
+					}
+				}
+				else if (n_cols < 2) {
+					GMT_Report (API, GMT_MSG_NORMAL, "Input file must at least have x,y or lon,lat columns\n");
+					gmt_M_free (GMT, p_data);
 					Return (GMT_RUNTIME_ERROR);
 				}
-				else
+				else {	/* Must update col types since # of z values will need to be considered */
+					unsigned int k, kk;
 					P.n_z = n_cols - 2;
+					for (col = kk = 0; col < P.n_outputs; col++, kk++) {
+						switch (P.output_choice[col]) {
+							case -1: /* Need to set float type for all the nz columns based on what the input z column types are */
+								for (k = 0; k < P.n_z; k++, kk++) gmt_set_column (GMT, GMT_OUT, kk, GMT->current.io.col_type[GMT_IN][k+GMT_Z]);
+								kk--;	/* Since we also do this at end of loop */
+								break;
+							case 0: case 4: gmt_set_column (GMT, GMT_OUT, kk, GMT_IS_LON);		break;
+							case 1: case 5: gmt_set_column (GMT, GMT_OUT, kk, GMT_IS_LAT);		break;
+							default: 	gmt_set_column (GMT, GMT_OUT, kk, GMT_IS_FLOAT);	break;
+						}
+					}
+				}
 				z_first = false;
+				n_tot_cols = (P.want_z_output) ? P.n_outputs - 1 + P.n_z : P.n_outputs;
+				if ((error = GMT_Set_Columns (API, GMT_OUT, (unsigned int)n_tot_cols, gmt_M_colmode (In->text))) != GMT_NOERROR) {
+					Return (error);
+				}
 			}
 
 			xx = in[GMT_X];
@@ -1025,15 +1081,11 @@ int GMT_project (void *V_API, int mode, void *args) {
 			p_data[P.n_used].a[3] = xt[1];
 			p_data[P.n_used].t = NULL;	/* Initialize since that is not done by realloc */
 			p_data[P.n_used].z = NULL;	/* Initialize since that is not done by realloc */
+			if (P.want_z_output && In->text)	/* Must store all trailing text */
+				p_data[P.n_used].t = strdup (In->text);
 			if (P.n_z) {	/* Copy over z column(s) */
-				if (pure_ascii) {	/* Must store all text beyond x,y columns */
-					p_data[P.n_used].t = gmt_M_memory (GMT, NULL, strlen (GMT->current.io.record), char);
-					copy_text_from_col3 (GMT->current.io.record, p_data[P.n_used].t);
-				}
-				else {
-					p_data[P.n_used].z = gmt_M_memory (GMT, NULL, P.n_z, double);
-					gmt_M_memcpy (p_data[P.n_used].z, &in[GMT_Z], P.n_z, double);
-				}
+				p_data[P.n_used].z = gmt_M_memory (GMT, NULL, P.n_z, double);
+				gmt_M_memcpy (p_data[P.n_used].z, &in[GMT_Z], P.n_z, double);
 			}
 			P.n_used++;
 			if (P.n_used == n_alloc) {
@@ -1047,7 +1099,7 @@ int GMT_project (void *V_API, int mode, void *args) {
 		if (P.n_used < n_alloc) p_data = gmt_M_memory (GMT, p_data, P.n_used, struct PROJECT_DATA);
 
 		if (P.n_used) {	/* Finish last segment output */
-			if ((error = write_one_segment (GMT, Ctrl, theta, p_data, pure_ascii, &P)) != 0) Return (error);
+			if ((error = write_one_segment (GMT, Ctrl, theta, p_data, &P)) != 0) Return (error);
 			n_total_used += P.n_used;
 		}
 
@@ -1059,10 +1111,10 @@ int GMT_project (void *V_API, int mode, void *args) {
 		Return (API->error);
 	}
 
-	GMT_Report (API, GMT_MSG_VERBOSE, "%" PRIu64 " read, %" PRIu64 " used\n", n_total_read, n_total_used);
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "%" PRIu64 " read, %" PRIu64 " used\n", n_total_read, n_total_used);
 
 	for (rec = 0; rec < P.n_used; rec++) {
-		gmt_M_free (GMT, p_data[rec].t);
+		gmt_M_str_free (p_data[rec].t);
 		gmt_M_free (GMT, p_data[rec].z);
 	}
 
