@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *	Copyright (c) 1991-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,25 +12,25 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
- * Brief synopsis: psimage reads a 1, 8, 24, or 32 bit Sun rasterfile and plots it on the page
- * Other raster formats are supported if GraphicsMagick's/ImageMagick's convert is found in the
- * system path.
+ * Brief synopsis: psimage reads an EPS file or a a 1, 8, 24, or 32 bit image and plots it on the page
+ * Images are only supported if GMT was built with GDAL support.
  *
  * Author:	Paul Wessel
  * Date:	1-JAN-2010
- * Version:	5 API
+ * Version:	6 API
  */
 
 #include "gmt_dev.h"
 
-#define THIS_MODULE_NAME	"psimage"
+#define THIS_MODULE_CLASSIC_NAME	"psimage"
+#define THIS_MODULE_MODERN_NAME	"image"
 #define THIS_MODULE_LIB		"core"
-#define THIS_MODULE_PURPOSE	"Place images or EPS files on maps"
+#define THIS_MODULE_PURPOSE	"Plot raster or EPS images"
 #define THIS_MODULE_KEYS	"<I{,>X}"
-#define THIS_MODULE_NEEDS	"rj"
+#define THIS_MODULE_NEEDS	"jr"
 #define THIS_MODULE_OPTIONS "->BJKOPRUVXYptxy" GMT_OPT("c")
 
 struct PSIMAGE_CTRL {
@@ -53,11 +52,9 @@ struct PSIMAGE_CTRL {
 		bool active;
 		struct GMT_MAP_PANEL *panel;
 	} F;
-	struct PSIMG_G {	/* -G[f|b|t]<rgb> */
+	struct PSIMG_G {	/* -G<rgb>[+b|+f|+t] */
 		bool active;
-		double f_rgb[4];
-		double b_rgb[4];
-		double t_rgb[4];
+		double rgb[3][4];
 	} G;
 	struct PSIMG_I {	/* -I */
 		bool active;
@@ -67,13 +64,17 @@ struct PSIMAGE_CTRL {
 	} M;
 };
 
+#define PSIMG_BGD	0
+#define PSIMG_FGD	1
+#define PSIMG_TRA	2
+
 GMT_LOCAL void *New_Ctrl (struct GMT_CTRL *GMT) {	/* Allocate and initialize a new control structure */
 	struct PSIMAGE_CTRL *C;
 
 	C = gmt_M_memory (GMT, NULL, 1, struct PSIMAGE_CTRL);
 
 	/* Initialize values whose defaults are not 0/false/NULL */
-	C->G.f_rgb[0] = C->G.b_rgb[0] = C->G.t_rgb[0] = -2;
+	C->G.rgb[PSIMG_FGD][0] = C->G.rgb[PSIMG_BGD][0] = C->G.rgb[PSIMG_TRA][0] = -2;	/* All turned off */
 	C->D.n_columns = C->D.n_rows = 1;
 	return (C);
 }
@@ -89,17 +90,17 @@ GMT_LOCAL void Free_Ctrl (struct GMT_CTRL *GMT, struct PSIMAGE_CTRL *C) {	/* Dea
 GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	/* This displays the psimage synopsis and optionally full usage information */
 
-	gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
+	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: psimage <imagefile> [%s] [-D%s+w[-]<width>[/<height>][+n<n_columns>[/<n_rows>]]%s+r<dpi>]\n", GMT_B_OPT, GMT_XYANCHOR, GMT_OFFSET);
-	GMT_Message (API, GMT_TIME_NONE, "\t[%s]\n", GMT_PANEL);
-	GMT_Message (API, GMT_TIME_NONE, "\t[-G[b|f|t]<color>] [-I] [%s] [%s] [-K] [-M] [-O]\n", GMT_J_OPT, GMT_Jz_OPT);
-	GMT_Message (API, GMT_TIME_NONE, "\t[-P] [%s] [%s]\n", GMT_Rgeoz_OPT, GMT_U_OPT);
-	GMT_Message (API, GMT_TIME_NONE, "\t[%s] [%s] [%s]\n\t[%s] [%s]\n\n", GMT_V_OPT, GMT_X_OPT, GMT_Y_OPT, GMT_p_OPT, GMT_t_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "usage: %s <imagefile> [%s] [-D%s+w[-]<width>[/<height>][+n<n_columns>[/<n_rows>]]%s+r<dpi>]\n", name, GMT_B_OPT, GMT_XYANCHOR, GMT_OFFSET);
+	GMT_Message (API, GMT_TIME_NONE, "\t[-F%s]\n", GMT_PANEL);
+	GMT_Message (API, GMT_TIME_NONE, "\t[-G[<color>][+b|+f|+t]] [-I] [%s] %s[-M] %s\n", GMT_J_OPT, API->K_OPT, API->O_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "\t%s[%s] [%s]\n", API->P_OPT, GMT_Rgeoz_OPT, GMT_U_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "\t[%s] [%s] [%s]\n\t%s[%s] [%s] [%s]\n\n", GMT_V_OPT, GMT_X_OPT, GMT_Y_OPT, API->c_OPT, GMT_p_OPT, GMT_t_OPT, GMT_PAR_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
-	GMT_Message (API, GMT_TIME_NONE, "\t<imagefile> is an EPS or image file.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t<imagefile> is an EPS or raster image file.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\n\tOPTIONS:\n");
 	GMT_Option (API, "B");
 	gmt_refpoint_syntax (API->GMT, "D", "Specify reference point for the image", GMT_ANCHOR_IMAGE, 3);
@@ -109,15 +110,15 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t   Alternatively, set image dpi (dots per inch) with +r.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   Use +n to replicate image <n_columns> by <n_rows> times [Default is no replication].\n");
 	gmt_mappanel_syntax (API->GMT, 'F', "Specify a rectangular panel behind the image", 1);
-	GMT_Message (API, GMT_TIME_NONE, "\t-Gb and -Gf (1-bit images only) set the background and foreground color,\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   respectively. Use <color> = - for transparency [Default is black and white].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-Gt (not for 1-bit images) indicate which color should be made transparent\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   [Default no transparency].\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t-G Change some pixels to given <color> depending on selected modifier (repeatable).\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   Append +b for replacing background and +f for foreground color (1-bit images only).\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   If no color was given we make those pixels transparent [Default is black and white].\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   Append +t to indicate the given <color> should be made transparent [no transparency].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-I Invert 1-bit images (does not affect 8 or 24-bit images).\n");
 	GMT_Option (API, "J-Z,K");
-	GMT_Message (API, GMT_TIME_NONE, "\t-M Force color -> monochrome image using gmt_M_yiq-transformation.\n");
-	GMT_Option (API, "O,P,R,U,V,X,p");
-	GMT_Message (API, GMT_TIME_NONE, "\t   (Requires -R and -J for proper functioning).\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t-M Force color -> monochrome image using YIQ-transformation.\n");
+	GMT_Option (API, "O,P,R,U,V,X,c,p");
+	if (gmt_M_showusage (API)) GMT_Message (API, GMT_TIME_NONE, "\t   (Requires -R and -J for proper functioning).\n");
 	GMT_Option (API, "t,.");
 
 	return (GMT_MODULE_USAGE);
@@ -131,10 +132,10 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct PSIMAGE_CTRL *Ctrl, struct GMT
 	 * returned when registering these sources/destinations with the API.
 	 */
 
-	unsigned int n_errors = 0, n_files = 0;
+	unsigned int n_errors = 0, n_files = 0, ind = PSIMG_FGD, k = 0;
 	int n;
 	bool p_fail = false;
-	char string[GMT_LEN256] = {""}, letter, *p = NULL;
+	char string[GMT_LEN256] = {""}, *p = NULL;
 	char txt_a[GMT_LEN256] = {""}, txt_b[GMT_LEN256] = {""}, txt_c[4] = {""};
 	struct GMT_OPTION *opt = NULL;
 
@@ -152,7 +153,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct PSIMAGE_CTRL *Ctrl, struct GMT
 			/* Processes program-specific parameters */
 
 			case 'C':	/* Image placement (old syntax) */
-				GMT_Report (GMT->parent, GMT_MSG_COMPAT, "Warning: -C option is deprecated, use -Dx instead.\n");
+				GMT_Report (GMT->parent, GMT_MSG_COMPAT, "-C option is deprecated, use -Dx instead.\n");
 				n = sscanf (opt->arg, "%[^/]/%[^/]/%2s", txt_a, txt_b, txt_c);
 				sprintf (string, "x%s/%s", txt_a, txt_b);
 				if (n == 3) {
@@ -195,7 +196,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct PSIMAGE_CTRL *Ctrl, struct GMT
 				}
 				break;
 			case 'E':	/* Specify image dpi */
-				GMT_Report (GMT->parent, GMT_MSG_COMPAT, "Warning: The -E option is deprecated but is accepted.\n");
+				GMT_Report (GMT->parent, GMT_MSG_COMPAT, "The -E option is deprecated but is accepted.\n");
 				GMT_Report (GMT->parent, GMT_MSG_COMPAT, "For the current -D syntax you should use -D modifier +r instead.\n");
 				GMT_Report (GMT->parent, GMT_MSG_COMPAT, "Note you cannot mix new-style modifiers (+r) with the old-style -C option.\n");
 				Ctrl->D.dpi = atof (opt->arg);
@@ -213,42 +214,46 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct PSIMAGE_CTRL *Ctrl, struct GMT
 				break;
 			case 'G':	/* Background/foreground color for 1-bit images */
 				Ctrl->G.active = true;
-				letter = (gmt_colorname2index (GMT, opt->arg) >= 0) ? 'x' : opt->arg[0];	/* If we have -G<colorname>, the x is used to bypass the case f|b|t switching below */
-				switch (letter) {
-					case 'f':
-						/* Set color for foreground pixels */
-						if (opt->arg[1] == '-' && opt->arg[2] == '\0')
-							Ctrl->G.f_rgb[0] = -1;
-						else if (gmt_getrgb (GMT, &opt->arg[1], Ctrl->G.f_rgb)) {
-							gmt_rgb_syntax (GMT, 'G', " ");
-							n_errors++;
-						}
-						break;
-					case 'b':
-						/* Set color for background pixels */
-						if (opt->arg[1] == '-' && opt->arg[2] == '\0')
-							Ctrl->G.b_rgb[0] = -1;
-						else if (gmt_getrgb (GMT, &opt->arg[1], Ctrl->G.b_rgb)) {
-							gmt_rgb_syntax (GMT, 'G', " ");
-							n_errors++;
-						}
-						break;
-					case 't':
-						/* Set transparent color */
-						if (gmt_getrgb (GMT, &opt->arg[1], Ctrl->G.t_rgb)) {
-							gmt_rgb_syntax (GMT, 'G', " ");
-							n_errors++;
-						}
-						break;
-					default:	/* Gave either -G<r/g/b>, -G-, or -G<colorname>; all treated as -Gf */
-						if (opt->arg[0] == '-' && opt->arg[1] == '\0')
-							Ctrl->G.f_rgb[0] = -1;
-						else if (gmt_getrgb (GMT, opt->arg, Ctrl->G.f_rgb)) {
-							gmt_rgb_syntax (GMT, 'G', " ");
-							n_errors++;
-						}
-						break;
+				if ((p = strstr (opt->arg, "+b")))	/* Background color (or transparency) selected */
+					ind = PSIMG_BGD, k = 0, p[0] = '\0';
+				else if ((p = strstr (opt->arg, "+f")))	/* Foreground color (or transparency) selected */
+					ind = PSIMG_FGD, k = 0, p[0] = '\0';
+				else if ((p = strstr (opt->arg, "+t"))) {	/* Transparency color specified */
+					ind = PSIMG_TRA;	k = 0; p[0] = '\0';
+					if (opt->arg[0] == '\0') {
+						GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Syntax error -G: Must specify a color when +t is used\n");
+						n_errors++;
+					}
 				}
+				else if (gmt_M_compat_check (GMT, 5)) {	/* Must check old-style arguments */
+					if (opt->arg[0] == '-' || gmt_colorname2index (GMT, opt->arg) >= 0 || !gmt_getrgb (GMT, opt->arg, Ctrl->G.rgb[PSIMG_FGD]))	/* Foreground color selected as default */
+						ind = PSIMG_FGD, k = 0;
+					else if (opt->arg[0] == 'f')	/* Foreground color selected */
+						ind = PSIMG_FGD, k = 1;
+					else if (opt->arg[0] == 'b')	/* Background color (or transparency) selected */
+						ind = PSIMG_BGD, k = 1;
+					else if (opt->arg[0] == 't')	/* Transparency color specified */
+						ind = PSIMG_TRA, k = 1;
+				}
+				else	/* Just -G[<color>] for foreground */
+					ind = PSIMG_FGD, k = 0;
+				if (opt->arg[k] == '\0')	/* No color given means set transparency */
+					Ctrl->G.rgb[ind][0] = -1;
+				else if (opt->arg[k] == '-') {	/* - means set transparency but only in GMT 5 and earlier */
+					if (gmt_M_compat_check (GMT, 5)) {	/* - means set transparency in GMT 5 and earlier */
+						Ctrl->G.rgb[ind][0] = -1;
+						GMT_Report (GMT->parent, GMT_MSG_COMPAT, "-G with color - for transparency is deprecated; give no <color> instead.\n");
+					}
+					else {
+						GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Syntax error -G: - is not a color\n");
+						n_errors++;
+					}
+				}
+				else if (gmt_getrgb (GMT, &opt->arg[k], Ctrl->G.rgb[ind])) {
+					gmt_rgb_syntax (GMT, 'G', " ");
+					n_errors++;
+				}
+				if (p) p[0] = '+';	/* Restore modifier */
 				break;
 			case 'I':	/* Invert 1-bit images */
 				Ctrl->I.active = true;
@@ -257,13 +262,13 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct PSIMAGE_CTRL *Ctrl, struct GMT
 				Ctrl->M.active = true;
 				break;
 			case 'N':	/* Replicate image */
-				GMT_Report (GMT->parent, GMT_MSG_COMPAT, "Warning: -N option is deprecated; use -D modifier +n instead.\n");
+				GMT_Report (GMT->parent, GMT_MSG_COMPAT, "-N option is deprecated; use -D modifier +n instead.\n");
 				n = sscanf (opt->arg, "%d/%d", &Ctrl->D.n_columns, &Ctrl->D.n_rows);
 				if (n == 1) Ctrl->D.n_rows = Ctrl->D.n_columns;
 				n_errors += gmt_M_check_condition (GMT, n < 1, "Syntax error -N option: Must give values for replication\n");
 				break;
 			case 'W':	/* Image width */
-				GMT_Report (GMT->parent, GMT_MSG_COMPAT, "Warning: -W option is deprecated; use -D modifier +w instead.\n");
+				GMT_Report (GMT->parent, GMT_MSG_COMPAT, "-W option is deprecated; use -D modifier +w instead.\n");
 				if ((n = gmt_get_pair (GMT, opt->arg, GMT_PAIR_DIM_NODUP, Ctrl->D.dim)) < 0) n_errors++;
 				if (Ctrl->D.dim[GMT_X] < 0.0) {
 					Ctrl->D.dim[GMT_X] = -Ctrl->D.dim[GMT_X];
@@ -279,8 +284,8 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct PSIMAGE_CTRL *Ctrl, struct GMT
 
 	/* If not done previously, set foreground to black, background to white */
 
-	if (Ctrl->G.f_rgb[0] == -2) { Ctrl->G.f_rgb[0] = Ctrl->G.f_rgb[1] = Ctrl->G.f_rgb[2] = 0.0; }
-	if (Ctrl->G.b_rgb[0] == -2) { Ctrl->G.b_rgb[0] = Ctrl->G.b_rgb[1] = Ctrl->G.b_rgb[2] = 1.0; }
+	if (Ctrl->G.rgb[PSIMG_FGD][0] == -2) { Ctrl->G.rgb[PSIMG_FGD][0] = Ctrl->G.rgb[PSIMG_FGD][1] = Ctrl->G.rgb[PSIMG_FGD][2] = 0.0; }
+	if (Ctrl->G.rgb[PSIMG_BGD][0] == -2) { Ctrl->G.rgb[PSIMG_BGD][0] = Ctrl->G.rgb[PSIMG_BGD][1] = Ctrl->G.rgb[PSIMG_BGD][2] = 1.0; }
 
 	if (!Ctrl->D.active) {	/* Old syntax without reference point implies -Dx0/0 */
 		Ctrl->D.refpoint = gmt_get_refpoint (GMT, "x0/0", 'D');	/* Default if no -D given */
@@ -297,39 +302,42 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct PSIMAGE_CTRL *Ctrl, struct GMT
 	n_errors += gmt_M_check_condition (GMT, !p_fail && Ctrl->D.dim[GMT_X] <= 0.0 && Ctrl->D.dpi <= 0.0, "Syntax error -D option: Must specify image width (+w) or dpi (+r)\n");
 	n_errors += gmt_M_check_condition (GMT, Ctrl->D.n_columns < 1 || Ctrl->D.n_rows < 1,
 			"Syntax error -D option: Must specify positive values for replication with +n\n");
-	n_errors += gmt_M_check_condition (GMT, Ctrl->G.f_rgb[0] < 0 && Ctrl->G.b_rgb[0] < 0,
+	n_errors += gmt_M_check_condition (GMT, Ctrl->G.rgb[PSIMG_FGD][0] < 0 && Ctrl->G.rgb[PSIMG_BGD][0] < 0,
 			"Syntax error -G option: Only one of fore/back-ground can be transparent for 1-bit images\n");
 
 	return (n_errors ? GMT_PARSE_ERROR : GMT_NOERROR);
 }
 
-GMT_LOCAL int file_is_known (struct GMT_CTRL *GMT, char *file) {	/* Returns 1 if it is an EPS file, 2 if a Sun rasterfile; 0 for any other file.
+GMT_LOCAL int file_is_eps (struct GMT_CTRL *GMT, char **file) {	/* Returns 1 if it is an EPS file; 0 for any other file.
        Returns -1 on read error */
 	FILE *fp = NULL;
-	unsigned char c[4], magic_ras[4] = {0x59, 0xa6, 0x6a, 0x95}, magic_ps[4] = {'%', '!', 'P', 'S'};
-	int j;
+	unsigned char c[4], magic_ps[4] = {'%', '!', 'P', 'S'};
+	char *F = *file;
 
-	if (file == NULL || file[0] == '\0') return -1;	/* Nothing given */
-	if (gmt_M_file_is_memory (file)) return (0);	/* Special passing of image */
-	j = (int)strlen(file) - 1;
-	while (j && file[j] && file[j] != '+') j--;	/* See if we have a band request */
-	if (j && file[j+1] == 'b') file[j] = '\0';			/* Temporarily strip the band request string so that the opening test doesn't fail */
+	if (F == NULL || F[0] == '\0') return GMT_NOTSET;	/* Nothing given */
+	if (gmt_M_file_is_memory (F)) return (0);	/* Special passing of image */
+	if (gmt_M_file_is_cache (F)) {	/* Must download, then modify the name */
+		char *tmp = strdup (&F[1]);
+		(void)gmt_download_file_if_not_found (GMT, F, 0);
+		gmt_M_str_free (*file);
+		*file = F = tmp;
+	}
+	if (strstr (F, "=gd")) return (0);		/* Passing an image via GDAL cannot be EPS */
+	if (strstr (F, "+b"))  return (0);		/* Band request of an image cannot be EPS */
 
-	if ((fp = gmt_fopen (GMT, file, "rb")) == NULL) {
-		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Cannot open file %s\n", file);
-		return (-1);
+	if ((fp = gmt_fopen (GMT, F, "rb")) == NULL) {
+		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Cannot open file %s\n", F);
+		return (GMT_NOTSET);
 	}
 	if (gmt_M_fread (c, 1U, 4U, fp) != 4U) {
-		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Could not read 4 bytes from file %s\n", file);
+		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Could not read 4 bytes from file %s\n", F);
 		gmt_fclose (GMT, fp);
-		return (-1);
+		return (GMT_NOTSET);
 	}
 	gmt_fclose (GMT, fp);
-	if (j) file[j] = '+';			/* Reset the band request string */
 	/* Note: cannot use gmt_M_same_rgb here, because that requires doubles */
 	if (c[0] == magic_ps[0] && c[1] == magic_ps[1] && c[2] == magic_ps[2] && c[3] == magic_ps[3]) return(1);
-	if (c[0] == magic_ras[0] && c[1] == magic_ras[1] && c[2] == magic_ras[2] && c[3] == magic_ras[3]) return(2);
-	return (0);	/* Neither */
+	return (0);
 }
 
 #define Return(code) {gmt_M_free (GMT, table); return (code);}
@@ -380,8 +388,18 @@ GMT_LOCAL int find_unique_color (struct GMT_CTRL *GMT, unsigned char *rgba, size
 #define Return(code) {Free_Ctrl (GMT, Ctrl); gmt_end_module (GMT, GMT_cpy); bailout (code);}
 EXTERN_MSC unsigned char *psl_gray_encode (struct PSL_CTRL *PSL, size_t *nbytes, unsigned char *input);
 
+int GMT_image (void *V_API, int mode, void *args) {
+	/* This is the GMT6 modern mode name */
+	struct GMTAPI_CTRL *API = gmt_get_api_ptr (V_API);	/* Cast from void to GMTAPI_CTRL pointer */
+	if (API->GMT->current.setting.run_mode == GMT_CLASSIC && !API->usage) {
+		GMT_Report (API, GMT_MSG_NORMAL, "Shared GMT module not found: image\n");
+		return (GMT_NOT_A_VALID_MODULE);
+	}
+	return GMT_psimage (V_API, mode, args);
+}
+
 int GMT_psimage (void *V_API, int mode, void *args) {
-	int i, j, PS_interpolate = 1, PS_transparent = 1, known = 0, error = 0;
+	int i, j, PS_interpolate = 1, PS_transparent = 1, is_eps = 0, error = 0, is_gdal = 0;
 	unsigned int row, col;
 	size_t n;
 	bool free_GMT = false, did_gray = false;
@@ -390,7 +408,7 @@ int GMT_psimage (void *V_API, int mode, void *args) {
 
 	unsigned char *picture = NULL, *buffer = NULL;
 
-	char path[GMT_BUFSIZ] = {""}, *format[2] = {"EPS", "Sun raster"}, *c = NULL;
+	char path[PATH_MAX] = {""}, *file = NULL, *c = NULL;
 
 	struct imageinfo header;
 
@@ -411,12 +429,11 @@ int GMT_psimage (void *V_API, int mode, void *args) {
 	if (mode == GMT_MODULE_PURPOSE) return (usage (API, GMT_MODULE_PURPOSE));	/* Return the purpose of program */
 	options = GMT_Create_Options (API, mode, args);	if (API->error) return (API->error);	/* Set or get option list */
 
-	if (!options || options->option == GMT_OPT_USAGE) bailout (usage (API, GMT_USAGE));	/* Return the usage message */
-	if (options->option == GMT_OPT_SYNOPSIS) bailout (usage (API, GMT_SYNOPSIS));	/* Return the synopsis */
+	if ((error = gmt_report_usage (API, options, 0, usage)) != GMT_NOERROR) bailout (error);	/* Give usage if requested */
 
 	/* Parse the command-line arguments; return if errors are encountered */
 
-	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
 	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
 	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
 	if ((error = parse (GMT, Ctrl, options)) != 0) Return (error);
@@ -425,30 +442,42 @@ int GMT_psimage (void *V_API, int mode, void *args) {
 
 	PS_interpolate = (Ctrl->D.interpolate) ? -1 : +1;
 
-	if ((c = strstr (Ctrl->In.file, "=gd"))) c[0] = '\0';	/* Chop off unnecessary =gd mandate */ 
-	known = file_is_known (GMT, Ctrl->In.file);	/* Determine if this is an EPS file, Sun rasterfile, or other */
-	if (known < 0) {
-		GMT_Report (API, GMT_MSG_NORMAL, "Cannot find/open/read file %s\n", Ctrl->In.file);
+	file = strdup (Ctrl->In.file);
+	if ((c = strstr (file, "=gd"))) {
+		is_gdal = true;
+		c[0] = '\0';	/* Chop off unnecessary =gd mandate */
+	}
+	is_eps = file_is_eps (GMT, &file);	/* Determine if this is an EPS file or other */
+	if (is_eps < 0) {
+		GMT_Report (API, GMT_MSG_NORMAL, "Cannot find/open/read file %s\n", file);
+		gmt_M_str_free (file);
 		Return (GMT_RUNTIME_ERROR);
 	}
 
 	memset (&header, 0, sizeof(struct imageinfo)); /* initialize struct */
-	if (known) {	/* Read an EPS or Sun raster file */
-		GMT_Report (API, GMT_MSG_VERBOSE, "Processing input EPS or Sun rasterfile\n");
-		if (gmt_getdatapath (GMT, Ctrl->In.file, path, R_OK) == NULL) {
-			GMT_Report (API, GMT_MSG_NORMAL, "Cannot find/open file %s.\n", Ctrl->In.file);
+	if (is_eps) {	/* Read an EPS file */
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Processing input EPS file\n");
+		if (gmt_getdatapath (GMT, file, path, R_OK) == NULL) {
+			GMT_Report (API, GMT_MSG_NORMAL, "Cannot find/open file %s.\n", file);
+			gmt_M_str_free (file);
 			Return (GMT_FILE_NOT_FOUND);
 		}
-		if (PSL_loadimage (PSL, path, &header, &picture)) {
-			GMT_Report (API, GMT_MSG_NORMAL, "Trouble loading %s file %s!\n", format[known-1], path);
+		if (PSL_loadeps (PSL, path, &header, &picture)) {
+			GMT_Report (API, GMT_MSG_NORMAL, "Trouble loading EPS file %s!\n", path);
+			gmt_M_str_free (file);
 			Return (GMT_IMAGE_READ_ERROR);
 		}
 	}
 #ifdef HAVE_GDAL
 	else  {	/* Read a raster image */
-		GMT_Report (API, GMT_MSG_VERBOSE, "Processing input raster via GDAL\n");
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Processing input raster via GDAL\n");
+		if (is_gdal) {	/* Need full name since there may be band requests */
+			gmt_M_str_free (file);
+			file = strdup (Ctrl->In.file);
+		}
 		gmt_set_pad (GMT, 0U);	/* Temporary turn off padding (and thus BC setting) since we will use image exactly as is */
-		if ((I = GMT_Read_Data (API, GMT_IS_IMAGE, GMT_IS_FILE, GMT_IS_SURFACE, GMT_CONTAINER_AND_DATA, NULL, Ctrl->In.file, NULL)) == NULL) {
+		if ((I = GMT_Read_Data (API, GMT_IS_IMAGE, GMT_IS_FILE, GMT_IS_SURFACE, GMT_CONTAINER_AND_DATA, NULL, file, NULL)) == NULL) {
+			gmt_M_str_free (file);
 			Return (API->error);
 		}
 		gmt_set_pad (GMT, API->pad);	/* Reset to GMT default */
@@ -486,9 +515,9 @@ int GMT_psimage (void *V_API, int mode, void *args) {
 
 		/* If a transparent color was found, we replace it with a unique one */
 		if (has_trans) {
-			Ctrl->G.t_rgb[0] = r / 255.;
-			Ctrl->G.t_rgb[1] = g / 255.;
-			Ctrl->G.t_rgb[2] = b / 255.;
+			Ctrl->G.rgb[PSIMG_TRA][0] = r / 255.;
+			Ctrl->G.rgb[PSIMG_TRA][1] = g / 255.;
+			Ctrl->G.rgb[PSIMG_TRA][2] = b / 255.;
 		}
 
 		picture = (unsigned char *)I->data;
@@ -497,8 +526,9 @@ int GMT_psimage (void *V_API, int mode, void *args) {
 		header.depth = (int)I->header->n_bands * 8;
 	}
 #else
-	else {	/* Without GDAL we can only read EPS and Sun raster */
-		GMT_Report (API, GMT_MSG_NORMAL, "Unsupported file format for file %s!\n", Ctrl->In.file);
+	else {	/* Without GDAL we can only read EPS files */
+		GMT_Report (API, GMT_MSG_NORMAL, "Unsupported file format for file %s!\n", file);
+		gmt_M_str_free (file);
 		Return (GMT_RUNTIME_ERROR);
 	}
 #endif
@@ -508,10 +538,11 @@ int GMT_psimage (void *V_API, int mode, void *args) {
 		n = 3 * header.width * header.height;
 		buffer = psl_gray_encode (PSL, &n, picture);
 		header.depth = 8;
-		if (known) PSL_free (picture); /* EPS or Sun raster file */
+		if (is_eps) PSL_free (picture); /* EPS ile */
 #ifdef HAVE_GDAL
 		else {	/* Got it via GMT_Read_Data */
 			if (GMT_Destroy_Data (API, &I) != GMT_NOERROR) {
+				gmt_M_str_free (file);
 				Return (API->error);
 			}
 		}
@@ -520,17 +551,18 @@ int GMT_psimage (void *V_API, int mode, void *args) {
 	}
 
 	/* Add transparent color at beginning, if requested */
-	if (Ctrl->G.t_rgb[0] < 0)
+	if (Ctrl->G.rgb[PSIMG_TRA][0] < 0)
 		PS_transparent = 1;
 	else if (header.depth >= 8) {
 		PS_transparent = -1;
 		j = header.depth / 8;
 		n = j * (header.width * header.height + 1);
 		buffer = gmt_M_memory (GMT, NULL, n, unsigned char);
-		for (i = 0; i < j; i++) buffer[i] = (unsigned char)rint(255 * Ctrl->G.t_rgb[i]);
+		for (i = 0; i < j; i++) buffer[i] = (unsigned char)rint(255 * Ctrl->G.rgb[PSIMG_TRA][i]);
 		gmt_M_memcpy (&(buffer[j]), picture, n - j, unsigned char);
 #ifdef HAVE_GDAL
 		if (GMT_Destroy_Data (API, &I) != GMT_NOERROR) {	/* If I is NULL then nothing is done */
+			gmt_M_str_free (file);
 			Return (API->error);
 		}
 #else
@@ -558,15 +590,17 @@ int GMT_psimage (void *V_API, int mode, void *args) {
 		if (gmt_M_err_pass (GMT, gmt_map_setup (GMT, wesn), "")) {
 			if (free_GMT)
 				gmt_M_free (GMT, picture);
-			else if (known || did_gray)
+			else if (is_eps || did_gray)
 				PSL_free (picture);
+			gmt_M_str_free (file);
 			Return (GMT_PROJECTION_ERROR);
 		}
 		if ((PSL = gmt_plotinit (GMT, options)) == NULL) {
 			if (free_GMT)
 				gmt_M_free (GMT, picture);
-			else if (known || did_gray)
+			else if (is_eps || did_gray)
 				PSL_free (picture);
+			gmt_M_str_free (file);
 			Return (GMT_RUNTIME_ERROR);
 		}
 		gmt_plane_perspective (GMT, GMT->current.proj.z_project.view_plane, GMT->current.proj.z_level);
@@ -575,8 +609,9 @@ int GMT_psimage (void *V_API, int mode, void *args) {
 		if (gmt_M_err_pass (GMT, gmt_map_setup (GMT, GMT->common.R.wesn), "")) {
 			if (free_GMT)
 				gmt_M_free (GMT, picture);
-			else if (known || did_gray)
+			else if (is_eps || did_gray)
 				PSL_free (picture);
+			gmt_M_str_free (file);
 			Return (GMT_PROJECTION_ERROR);
 		}
 		gmt_set_refpoint (GMT, Ctrl->D.refpoint);	/* Finalize reference point plot coordinates, if needed */
@@ -584,8 +619,9 @@ int GMT_psimage (void *V_API, int mode, void *args) {
 		if ((PSL = gmt_plotinit (GMT, options)) == NULL) {
 			if (free_GMT)
 				gmt_M_free (GMT, picture);
-			else if (known || did_gray)
+			else if (is_eps || did_gray)
 				PSL_free (picture);
+			gmt_M_str_free (file);
 			Return (GMT_RUNTIME_ERROR);
 		}
 		gmt_plane_perspective (GMT, GMT->current.proj.z_project.view_plane, GMT->current.proj.z_level);
@@ -599,8 +635,9 @@ int GMT_psimage (void *V_API, int mode, void *args) {
 		if (gmt_M_err_pass (GMT, gmt_map_setup (GMT, wesn), "")) {
 			if (free_GMT)
 				gmt_M_free (GMT, picture);
-			else if (known || did_gray)
+			else if (is_eps || did_gray)
 				PSL_free (picture);
+			gmt_M_str_free (file);
 			Return (GMT_PROJECTION_ERROR);
 		}
 	}
@@ -612,7 +649,7 @@ int GMT_psimage (void *V_API, int mode, void *args) {
 
 	for (row = 0; row < Ctrl->D.n_rows; row++) {
 		y = Ctrl->D.refpoint->y + row * Ctrl->D.dim[GMT_Y];
-		if (Ctrl->D.n_rows > 1) GMT_Report (API, GMT_MSG_VERBOSE, "Replicating image %d times for row %d\n", Ctrl->D.n_columns, row);
+		if (Ctrl->D.n_rows > 1) GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Replicating image %d times for row %d\n", Ctrl->D.n_columns, row);
 		for (col = 0; col < Ctrl->D.n_columns; col++) {
 			x = Ctrl->D.refpoint->x + col * Ctrl->D.dim[GMT_X];
 			if (header.depth == 0)
@@ -621,10 +658,10 @@ int GMT_psimage (void *V_API, int mode, void *args) {
 				/* Invert is opposite from what is expected. This is to match the behaviour of -Gp */
 				if (Ctrl->I.active)
 					PSL_plotbitimage (PSL, x, y, Ctrl->D.dim[GMT_X], Ctrl->D.dim[GMT_Y], PSL_BL, picture,
-							header.width, header.height, Ctrl->G.f_rgb, Ctrl->G.b_rgb);
+							header.width, header.height, Ctrl->G.rgb[PSIMG_FGD], Ctrl->G.rgb[PSIMG_BGD]);
 				else
 					PSL_plotbitimage (PSL, x, y, Ctrl->D.dim[GMT_X], Ctrl->D.dim[GMT_Y], PSL_BL, picture,
-							header.width, header.height, Ctrl->G.b_rgb, Ctrl->G.f_rgb);
+							header.width, header.height, Ctrl->G.rgb[PSIMG_BGD], Ctrl->G.rgb[PSIMG_FGD]);
 			}
 			else
 				 PSL_plotcolorimage (PSL, x, y, Ctrl->D.dim[GMT_X], Ctrl->D.dim[GMT_Y], PSL_BL, picture,
@@ -636,6 +673,7 @@ int GMT_psimage (void *V_API, int mode, void *args) {
 
 	gmt_plane_perspective (GMT, -1, 0.0);
 	gmt_plotend (GMT);
+	gmt_M_str_free (file);
 
 #ifdef HAVE_GDAL
 	if (I && GMT_Destroy_Data (API, &I) != GMT_NOERROR) {
@@ -645,7 +683,7 @@ int GMT_psimage (void *V_API, int mode, void *args) {
 	if (free_GMT) {
 		gmt_M_free (GMT, picture);
 	}
-	else if (known || did_gray)
+	else if (is_eps || did_gray)
 		PSL_free (picture);
 
 	Return (GMT_NOERROR);
