@@ -1,5 +1,4 @@
-/* $Id$
- *
+/*
  * psldemo.c: Test all PSL functions at least once
  */
 
@@ -30,8 +29,8 @@ page of text or graphics.  It uses a reverse Polish notation that puts and gets
 text, and images and even perform calculations.\r \t@%5%PSL@%% is a @_self-contained@_ library that presents a series of functions \
 that can be used to create plots.  The resulting @%6%PostScript@%% code is ASCII text.";
 	int i, k;
+	unsigned int dim[3] = {0, 0, 0};
 	struct imageinfo h;
-	unsigned char *picture;
 
 	if (argc > 1) fprintf (stderr, "psldemo: No argument required\n");
 	
@@ -67,18 +66,18 @@ that can be used to create plots.  The resulting @%6%PostScript@%% code is ASCII
 	PSL_plotaxis (PSL, 20.0, "Some Axis", 10.0, 0);
 	PSL_plotaxis (PSL, 25.0, "A Reversed Axis", 10.0, 3);
 	PSL_setlinewidth (PSL, 5.0);
-	PSL_plotline (PSL, x, y, 6, PSL_MOVE + PSL_STROKE);
+	PSL_plotline (PSL, x, y, 6, PSL_MOVE|PSL_STROKE);
 	PSL_setlinewidth (PSL, 1.0);
 	PSL_setfill (PSL, rgb[0], outline[0]);
 	for (i = 0; i < 6; i++) PSL_plotsymbol (PSL, x[i], y[i], size, PSL_STAR);
 
 	/* Plot some patterns within the x-y axis */
-	rgb[5][1] = (double) PSL_setpattern (PSL, -1, PSL_PATTERN, 100, rgb[2], rgb[2]);
+	rgb[5][1] = (double) PSL_setimage (PSL, -1, "circuit", circuit, 100, circuit_dim, rgb[2], rgb[3]);
 	PSL_setfill (PSL, rgb[5], PSL_NO);
 	PSL_plotbox (PSL, 80.0, 30.0, 140, 50);
 
 	size[0] = 0.5;
-	rgb[5][1] = (double) PSL_setpattern (PSL, 13, "", 100, rgb[3], rgb[2]);
+	rgb[5][1] = (double) PSL_setimage (PSL, 13, "13", NULL, 100, dim, rgb[3], rgb[2]);
 	PSL_setcolor (PSL, rgb[5], PSL_IS_STROKE);
 	PSL_plotsymbol (PSL, 80.0, 30.0, size, PSL_DOT);
 
@@ -90,15 +89,15 @@ that can be used to create plots.  The resulting @%6%PostScript@%% code is ASCII
 	PSL_plotsymbol (PSL, 95.0, 30.0, size, PSL_SQUARE);
 
 	size[0] = 100.0; size[1] = 0.5; size[2] = 0.4;
-	rgb[5][1] = (double) PSL_setpattern (PSL, 13, "", 100, rgb[2], rgb[3]);
+	rgb[5][1] = (double) PSL_setimage (PSL, 13, "13", NULL, 100, dim, rgb[2], rgb[3]);
 	PSL_plotsymbol (PSL, 110.0, 30.0, size, PSL_ROTRECT);
 
-	rgb[5][1] = (double) PSL_setpattern (PSL, 14, "", 100, rgb[1], rgb[2]);
+	rgb[5][1] = (double) PSL_setimage (PSL, 14, "14", NULL, 100, dim, rgb[1], rgb[2]);
 	PSL_setfill (PSL, rgb[5], PSL_OUTLINE);
 	PSL_plotsymbol (PSL, 125.0, 30.0, size, PSL_ELLIPSE);
 
-	size[0] = 0.3; size[1] = 45.0; size[2] = 315.0;
-	rgb[5][1] = (double) PSL_setpattern (PSL, 14, "", 100, rgb[1], rgb[0]);
+	size[0] = 0.3; size[1] = 45.0; size[2] = 315.0;	size[3] = 0.0;	size[7] = 3.0;
+	rgb[5][1] = (double) PSL_setimage (PSL, 14, "14", NULL, 100, dim, rgb[1], rgb[0]);
 	PSL_plotsymbol (PSL, 140.0, 30.0, size, PSL_WEDGE);
 
 	/* Return to normal coordinates */
@@ -155,9 +154,7 @@ that can be used to create plots.  The resulting @%6%PostScript@%% code is ASCII
 	PSL_plotparagraph (PSL, 2.2, 8.1, 12.0, NULL, 0.0, PSL_TL);
 
 	/* Plot an image by itself */
-	PSL_loadimage (PSL, PSL_IMAGE, &h, &picture);
-	PSL_plotcolorimage (PSL, 5.0, 8.2, 1.5, 0.0, PSL_TL, picture, h.width, h.height, h.depth);
-	PSL_free (picture);
+	PSL_plotcolorimage (PSL, 5.0, 8.2, 1.5, 0.0, PSL_TL, vader, vader_dim[0], vader_dim[1], vader_dim[2]);
 
 	PSL_endplot (PSL, PSL_FINALIZE);
 	PSL_endsession (PSL);
