@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *	Copyright (c) 1991-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  * Brief synopsis: psmask tries to achieve masking using one of two different approaches:
@@ -33,18 +32,25 @@
  *
  * Author:	Paul Wessel
  * Date:	1-JAN-2010
- * Version:	5 API
+ * Version:	6 API
  */
 
 #include "gmt_dev.h"
 
-#define THIS_MODULE_NAME	"psmask"
+#define THIS_MODULE_CLASSIC_NAME	"psmask"
+#define THIS_MODULE_MODERN_NAME	"mask"
 #define THIS_MODULE_LIB		"core"
 #define THIS_MODULE_PURPOSE	"Clip or mask map areas with no data table coverage"
-#define THIS_MODULE_KEYS	"<D{,DDD,C-(,>X},LG)"
-#define THIS_MODULE_NEEDS	"dJ"
+#define THIS_MODULE_KEYS	"<D{,DDD,C-(,>X},LG)@<D{,DD),C-(,LG)"
+#define THIS_MODULE_NEEDS	"Jd"
 #define THIS_MODULE_OPTIONS "-:>BJKOPRUVXYbdehiprstxy" GMT_OPT("Ec")
 
+enum Mask_Modes {
+	PSMASK_INSIDE  = -1,	/* Set inside nodes to NaN */
+	PSMASK_DEFAULT =  0,	/* Leave nodes as they are */
+	PSMASK_OUTSIDE = +1	/* Set outside nodes to NaN */
+};
+
 struct PSMASK_CTRL {
 	struct C {	/* -C */
 		bool active;
@@ -61,9 +67,9 @@ struct PSMASK_CTRL {
 		bool active;
 		struct GMT_FILL fill;
 	} G;
-	struct L {	/* -L[+|-]<file> */
+	struct L {	/* -L<file>[+i|o] */
 		bool active;
-		int mode;	/* -1 = set inside node to NaN, 0 as is, +1 set outside node to NaN */
+		int mode;	/* -1 = set inside node to NaN (+i), 0 as is, +1 set outside node to NaN (+o) */
 		char *file;
 	} L;
 	struct N {	/* -N */
@@ -325,13 +331,14 @@ GMT_LOCAL void orient_contours (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *h,
 	bool reverse;
 	uint64_t i, j, ij_ul, ij_ur, ij_ll, ij_lr;
 	double fx[2], fy[2], dx, dy;
+	struct GMT_GRID_HEADER_HIDDEN *HH = gmt_get_H_hidden (h);
 
 	if (orient == 0) return;	/* Nothing to be done when no orientation specified */
 	if (n < 2) return;		/* Cannot work on a single point */
 
 	for (k = 0; k < 2; k++) {	/* Calculate fractional node numbers from left/top */
-		fx[k] = (x[k] - h->wesn[XLO]) * h->r_inc[GMT_X] - h->xy_off;
-		fy[k] = (h->wesn[YHI] - y[k]) * h->r_inc[GMT_Y] - h->xy_off;
+		fx[k] = (x[k] - h->wesn[XLO]) * HH->r_inc[GMT_X] - h->xy_off;
+		fy[k] = (h->wesn[YHI] - y[k]) * HH->r_inc[GMT_Y] - h->xy_off;
 	}
 
 	/* Get(i,j) of the lower left node in the rectangle containing this contour segment.
@@ -423,13 +430,13 @@ GMT_LOCAL void Free_Ctrl (struct GMT_CTRL *GMT, struct PSMASK_CTRL *C) {	/* Deal
 }
 
 GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
-	gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
+	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: psmask <table> %s %s\n", GMT_I_OPT, GMT_J_OPT);
-	GMT_Message (API, GMT_TIME_NONE, "\t%s [%s] [-C] [-D<template>] [-G<fill>]\n\t[%s] [-K] [-L<grid>] [-N] [-O] [-P] [-Q<min>] [-S%s] [-T]\n", GMT_Rgeoz_OPT, GMT_B_OPT, GMT_Jz_OPT, GMT_RADIUS_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "usage: %s <table> %s %s\n", name, GMT_I_OPT, GMT_J_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "\t%s [%s] [-C] [-D<template>] [-F[l|r]] [-G<fill>]\n\t%s[-L<grid>[+i|o]] [-N] %s%s[-Q<min>] [-S%s] [-T]\n", GMT_Rgeoz_OPT, GMT_B_OPT, API->K_OPT, API->O_OPT, API->P_OPT, GMT_RADIUS_OPT);
 	GMT_Message (API, GMT_TIME_NONE, "\t[%s] [%s] [%s]\n", GMT_U_OPT, GMT_V_OPT, GMT_X_OPT);
-	GMT_Message (API, GMT_TIME_NONE, "\t[%s] [%s] [%s] [%s]\n\t[%s] [%s]\n", GMT_Y_OPT, GMT_b_OPT, GMT_d_OPT, GMT_e_OPT, GMT_h_OPT, GMT_i_OPT);
-	GMT_Message (API, GMT_TIME_NONE, "\t[%s] [%s]\n\t[%s] [%s] [%s]\n\n", GMT_p_OPT, GMT_r_OPT, GMT_s_OPT, GMT_t_OPT, GMT_colon_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "\t[%s] [%s] %s[%s] [%s]\n\t[%s] [%s]\n", GMT_Y_OPT, GMT_b_OPT, API->c_OPT, GMT_d_OPT, GMT_e_OPT, GMT_h_OPT, GMT_i_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "\t[%s] [%s]\n\t[%s] [%s] [%s] [%s]\n\n", GMT_p_OPT, GMT_r_OPT, GMT_s_OPT, GMT_t_OPT, GMT_colon_OPT, GMT_PAR_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
@@ -444,10 +451,13 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t   If an integer format (e.g., %%06d) is found we substitute a running segment count\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   and write all polygons to individual files; see manual page for more examples.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   Cannot be used with -T; see -Q to eliminate small polygons.\n");
-	gmt_fill_syntax (API->GMT, 'G', "Select fill color/pattern [Default is no fill].");
+	GMT_Message (API, GMT_TIME_NONE, "\t-F Force clip contours to be oriented so that the higher z-values\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   are to the left (-Fl [Default]) or right (-Fr) as we move along\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   the contour lines [Default is not oriented].\n");
+	gmt_fill_syntax (API->GMT, 'G', NULL, "Select fill color/pattern [Default is no fill].");
 	GMT_Option (API, "K");
 	GMT_Message (API, GMT_TIME_NONE, "\t-L Save internal on/off node grid to <grid> for testing [no grid saved].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Use -L+ to change inside nodes to NaNs or -L- to change outside nodes to NaNs.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   Append +i to change inside nodes to NaNs or +o to change outside nodes to NaNs.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-N Invert the sense of the clipping [or tiling].\n");
 	GMT_Option (API, "O,P");
 	GMT_Message (API, GMT_TIME_NONE, "\t-Q Do not dump contours with less than <cut> points [Dump all contours].\n");
@@ -458,7 +468,7 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t   Default is -S0, i.e., only the nearest node is considered reliable.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-T Paint tiles [Default will trace data outline].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   If set you must also specify a color/fill with -G.\n");
-	GMT_Option (API, "U,V,X,bi2,bo,d,e,h,i,p,r,s,t,:,.");
+	GMT_Option (API, "U,V,X,bi2,bo,c,d,e,h,i,p,r,s,t,:,.");
 	
 	return (GMT_MODULE_USAGE);
 }
@@ -475,6 +485,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct PSMASK_CTRL *Ctrl, struct GMT_
 	unsigned int n_errors = 0;
 	struct GMT_OPTION *opt = NULL;
 	struct GMTAPI_CTRL *API = GMT->parent;
+	char *c = NULL;
 
 	for (opt = options; opt; opt = opt->next) {	/* Process all the options given */
 
@@ -495,7 +506,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct PSMASK_CTRL *Ctrl, struct GMT_
 				if (gmt_M_compat_check (GMT, 4)) {
 					for (n_plus = -1, k = 0; opt->arg[k]; k++) {
 						if (opt->arg[k] == '+' && opt->arg[k+1] == 'n') {
-							GMT_Report (API, GMT_MSG_COMPAT, "Warning: Option -D..+n<min> is deprecated; use -Q instead.\n");
+							GMT_Report (API, GMT_MSG_COMPAT, "Option -D..+n<min> is deprecated; use -Q instead.\n");
 							Ctrl->Q.min = atoi (&opt->arg[k + 2]);
 							Ctrl->Q.active = true;
 							n_plus = k;
@@ -525,7 +536,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct PSMASK_CTRL *Ctrl, struct GMT_
 			case 'G':
 				Ctrl->G.active = true;
 				if (gmt_getfill (GMT, opt->arg, &Ctrl->G.fill)) {
-					gmt_fill_syntax (GMT, 'G', " ");
+					gmt_fill_syntax (GMT, 'G', NULL, " ");
 					n_errors++;
 				}
 				break;
@@ -534,13 +545,21 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct PSMASK_CTRL *Ctrl, struct GMT_
 				break;
 			case 'L':
 				Ctrl->L.active = true;
-				k = 1;
-				switch (opt->arg[0]) {
-					case '-': Ctrl->L.mode = -1;	break;
-					case '+': Ctrl->L.mode = +1;	break;
-					default: k = 0;	break;
+				k = 0;
+				if (opt->arg[0] == '-')	/* Old style leading -<name> */
+					Ctrl->L.mode = PSMASK_INSIDE, k = 1;
+				else if (opt->arg[0] == '+')	/* Old style leading +<name> */
+					Ctrl->L.mode = PSMASK_OUTSIDE, k = 1;
+				else if ((c = strstr (opt->arg, "+i"))) {	/* Appended modifier +i */
+					c[0] = '\0';	/* Temporarily chop off modifier */
+					Ctrl->L.mode = PSMASK_INSIDE;
+				}
+				else if ((c = strstr (opt->arg, "+o"))) {	/* Appended modifier +o */
+					c[0] = '\0';	/* Temporarily chop off modifier */
+					Ctrl->L.mode = PSMASK_OUTSIDE;
 				}
 				if (opt->arg[k]) Ctrl->L.file = strdup (&opt->arg[k]);
+				if (c) c[0] = '+';	/* Restore modifier */
 				break;
 			case 'N':
 				Ctrl->N.active = true;
@@ -587,6 +606,16 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct PSMASK_CTRL *Ctrl, struct GMT_
 #define bailout(code) {gmt_M_free_options (mode); return (code);}
 #define Return(code) {Free_Ctrl (GMT, Ctrl); gmt_end_module (GMT, GMT_cpy); bailout (code);}
 
+int GMT_mask (void *V_API, int mode, void *args) {
+	/* This is the GMT6 modern mode name */
+	struct GMTAPI_CTRL *API = gmt_get_api_ptr (V_API);	/* Cast from void to GMTAPI_CTRL pointer */
+	if (API->GMT->current.setting.run_mode == GMT_CLASSIC && !API->usage) {
+		GMT_Report (API, GMT_MSG_NORMAL, "Shared GMT module not found: mask\n");
+		return (GMT_NOT_A_VALID_MODULE);
+	}
+	return GMT_psmask (V_API, mode, args);
+}
+
 int GMT_psmask (void *V_API, int mode, void *args) {
 	unsigned int section, k, row, col, n_edges, *d_col = NULL, d_row = 0;
 	unsigned int io_mode = GMT_WRITE_SET, max_d_col = 0, ii, jj, i_start, j_start, first = 1;
@@ -606,6 +635,7 @@ int GMT_psmask (void *V_API, int mode, void *args) {
 	struct GMT_DATASET *D = NULL;
 	struct GMT_DATASEGMENT *S = NULL;
 	struct GMT_GRID *Grid = NULL;
+	struct GMT_RECORD *In = NULL;
 	struct PSMASK_INFO info;
 	struct PSMASK_CTRL *Ctrl = NULL;
 	struct GMT_CTRL *GMT = NULL, *GMT_cpy = NULL;		/* General GMT internal parameters */
@@ -619,12 +649,11 @@ int GMT_psmask (void *V_API, int mode, void *args) {
 	if (mode == GMT_MODULE_PURPOSE) return (usage (API, GMT_MODULE_PURPOSE));	/* Return the purpose of program */
 	options = GMT_Create_Options (API, mode, args);	if (API->error) return (API->error);	/* Set or get option list */
 
-	if (!options || options->option == GMT_OPT_USAGE) bailout (usage (API, GMT_USAGE));	/* Return the usage message */
-	if (options->option == GMT_OPT_SYNOPSIS) bailout (usage (API, GMT_SYNOPSIS));	/* Return the synopsis */
+	if ((error = gmt_report_usage (API, options, 0, usage)) != GMT_NOERROR) bailout (error);	/* Give usage if requested */
 
 	/* Parse the command-line arguments; return if errors are encountered */
 
-	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
 	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
 	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
 	if ((error = parse (GMT, Ctrl, options)) != 0) Return (error);
@@ -655,7 +684,7 @@ int GMT_psmask (void *V_API, int mode, void *args) {
 			if (fmt[1]) io_mode = GMT_WRITE_SEGMENT;	/* d: Want individual files with running numbers */
 		}
 		if ((D = GMT_Create_Data (API, GMT_IS_DATASET, GMT_IS_POLY, 0, dim, NULL, NULL, 0, 0, NULL)) == NULL) Return (API->error);	/* An empty table */
-		if ((error = gmt_set_cols (GMT, GMT_OUT, 2)) != 0) Return (error);
+		if ((error = GMT_Set_Columns (API, GMT_OUT, 2, GMT_COL_FIX_NO_TEXT)) != GMT_NOERROR) Return (error);
 	}
 	
 	if (make_plot) {
@@ -670,7 +699,7 @@ int GMT_psmask (void *V_API, int mode, void *args) {
 	if (Ctrl->C.active) {	/* Just undo previous polygon clip-path */
 		PSL_endclipping (PSL, 1);
 		gmt_map_basemap (GMT);
-		GMT_Report (API, GMT_MSG_VERBOSE, "Clipping off!\n");
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Clipping off!\n");
 	}
 	else {	/* Start new clip_path */
 		gmt_M_memset (inc2, 2, double);
@@ -688,7 +717,7 @@ int GMT_psmask (void *V_API, int mode, void *args) {
 			gmt_plotcanvas (GMT);	/* Fill canvas if requested */
 		}
 
-		GMT_Report (API, GMT_MSG_VERBOSE, "Allocate memory, read and process data file\n");
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Allocate memory, read and process data file\n");
 
 		/* Enlarge region by 1 row/column */
 
@@ -696,7 +725,7 @@ int GMT_psmask (void *V_API, int mode, void *args) {
 		gmt_set_pad (GMT, 0U);		/* Change default pad to 0 only */
 		gmt_M_grd_setpad (GMT, Grid->header, GMT->current.io.pad);	/* Change header pad to 0 */
 		gmt_set_grddim (GMT, Grid->header);	/* Recompute dimensions of array */
-		/* We allocate a 1-byte array separately instead of the 4-byte float array that GMT_Create_Data would have given us */
+		/* We allocate a 1-byte array separately instead of the 4-byte gmt_grdfloat array that GMT_Create_Data would have given us */
 		grd = gmt_M_memory (GMT, NULL, Grid->header->size, char);	/* Only need char array to store 0 and 1 */
 
 		if (Ctrl->S.active) {	/* Need distance calculations in correct units, and the d_row/d_col machinery */
@@ -710,10 +739,10 @@ int GMT_psmask (void *V_API, int mode, void *args) {
 		
 		node_only = (max_d_col == 0 && d_row == 0);
 		if (node_only && Ctrl->S.radius > 0.0) {
-			GMT_Report (API, GMT_MSG_NORMAL, "Warning: Your search radius is too small to have any effect and is ignored.\n");
+			GMT_Report (API, GMT_MSG_VERBOSE, "Your search radius is too small to have any effect and is ignored.\n");
 		}
 		
-		if ((error = gmt_set_cols (GMT, GMT_IN, 2)) != GMT_NOERROR) {
+		if ((error = GMT_Set_Columns (API, GMT_IN, 2, GMT_COL_FIX_NO_TEXT)) != GMT_NOERROR) {
 			Return (error);
 		}
 		if (GMT_Init_IO (API, GMT_IS_DATASET, GMT_IS_POINT, GMT_IN, GMT_ADD_DEFAULT, 0, options) != GMT_NOERROR) {	/* Establishes data input */
@@ -723,19 +752,19 @@ int GMT_psmask (void *V_API, int mode, void *args) {
 			Return (API->error);
 		}
 
-		GMT_Report (API, GMT_MSG_VERBOSE, "Processing input table data\n");
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Processing input table data\n");
 		n_read = 0;
 		do {	/* Keep returning records until we reach EOF */
 			n_read++;
-			if ((in = GMT_Get_Record (API, GMT_READ_DATA, NULL)) == NULL) {	/* Read next record, get NULL if special case */
+			if ((In = GMT_Get_Record (API, GMT_READ_DATA, NULL)) == NULL) {	/* Read next record, get NULL if special case */
 				if (gmt_M_rec_is_error (GMT)) 		/* Bail if there are any read errors */
 					Return (GMT_RUNTIME_ERROR);
 				if (gmt_M_rec_is_any_header (GMT)) 	/* Skip all table and segment headers */
 					continue;
 				if (gmt_M_rec_is_eof (GMT)) 		/* Reached end of file */
 					break;
-				assert (in != NULL);						/* Should never get here */
 			}
+			in = In->data;	/* Only need to process numerical part here */
 
 			if (gmt_M_y_is_outside (GMT, in[GMT_Y], Grid->header->wesn[YLO], Grid->header->wesn[YHI])) continue;	/* Outside y-range */
 			if (gmt_x_is_outside (GMT, &in[GMT_X], Grid->header->wesn[XLO], Grid->header->wesn[XHI])) continue;	/* Outside x-range (or longitude) */
@@ -802,7 +831,7 @@ int GMT_psmask (void *V_API, int mode, void *args) {
 			Return (API->error);
 		}
 
-		GMT_Report (API, GMT_MSG_VERBOSE, "Read %" PRIu64 " data points\n", n_read);
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Read %" PRIu64 " data points\n", n_read);
 
 		if (Ctrl->N.active) for (ij = 0; ij < Grid->header->size; ij++) grd[ij] = 1 - grd[ij];	/* Reverse sense of test */
 
@@ -813,19 +842,19 @@ int GMT_psmask (void *V_API, int mode, void *args) {
 
 		if (Ctrl->L.active) {	/* Save a copy of the grid to file */
 			struct GMT_GRID *G = NULL;
-			GMT_Report (API, GMT_MSG_VERBOSE, "Saving internal inside|outside grid to file %s\n", Ctrl->L.file);
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Saving internal inside|outside grid to file %s\n", Ctrl->L.file);
 			if ((G = GMT_Create_Data (API, GMT_IS_GRID, GMT_IS_SURFACE, GMT_CONTAINER_AND_DATA, NULL, Grid->header->wesn, Grid->header->inc, \
 				Grid->header->registration, 0, NULL)) == NULL) Return (API->error);
 			for (ij = 0; ij < Grid->header->size; ij++) {	/* Copy over the 0/1 grid */
 				switch (Ctrl->L.mode) {
 					case 0:	/* As is */
-						G->data[ij] = (float)grd[ij];
+						G->data[ij] = (gmt_grdfloat)grd[ij];
 						break;
 					case +1:	/* Replace zeros by NaNs */
-						G->data[ij] = (grd[ij] == 0) ? GMT->session.f_NaN : (float)grd[ij];
+						G->data[ij] = (grd[ij] == 0) ? GMT->session.f_NaN : (gmt_grdfloat)grd[ij];
 						break;
 					case -1:	/* Replace ones by NaNs */
-						G->data[ij] = (grd[ij] == 1) ? GMT->session.f_NaN : (float)grd[ij];
+						G->data[ij] = (grd[ij] == 1) ? GMT->session.f_NaN : (gmt_grdfloat)grd[ij];
 						break;
 				}
 			}
@@ -838,6 +867,8 @@ int GMT_psmask (void *V_API, int mode, void *args) {
 		}
 		if (!Ctrl->T.active) {	/* Must trace the outline of ON/OFF values in grd */
 			uint64_t max_alloc_points = GMT_CHUNK;
+			struct GMT_DATATABLE_HIDDEN *TH = NULL;
+			struct GMT_DATASEGMENT_HIDDEN *SH = NULL;
 			/* Arrays holding the contour xy values; preallocate space */
 			x = gmt_M_memory (GMT, NULL, max_alloc_points, double);
 			y = gmt_M_memory (GMT, NULL, max_alloc_points, double);
@@ -847,7 +878,7 @@ int GMT_psmask (void *V_API, int mode, void *args) {
 
 			if (make_plot) gmt_map_basemap (GMT);
 
-			GMT_Report (API, GMT_MSG_VERBOSE, "Tracing the clip path\n");
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Tracing the clip path\n");
 
 			section = 0;
 			first = 1;
@@ -867,10 +898,13 @@ int GMT_psmask (void *V_API, int mode, void *args) {
 					D->table[0]->n_segments++;	D->n_segments++;
 					D->table[0]->n_records += n;	D->n_records += n;
 					/* Generate a file name and increment cont_counts, if relevant */
-					if (io_mode == GMT_WRITE_SET && !D->table[0]->file[GMT_OUT])
-						D->table[0]->file[GMT_OUT] = gmt_make_filename (GMT, Ctrl->D.file, fmt, GMT->session.d_NaN, closed, cont_counts);
-					else if (io_mode == GMT_WRITE_SEGMENT)
-						S->file[GMT_OUT] = gmt_make_filename (GMT, Ctrl->D.file, fmt, GMT->session.d_NaN, closed, cont_counts);
+					TH = gmt_get_DT_hidden (D->table[0]);
+					if (io_mode == GMT_WRITE_SET && !TH->file[GMT_OUT])
+						TH->file[GMT_OUT] = gmt_make_filename (GMT, Ctrl->D.file, fmt, GMT->session.d_NaN, closed, cont_counts);
+					else if (io_mode == GMT_WRITE_SEGMENT) {
+						SH = gmt_get_DS_hidden (S);
+						SH->file[GMT_OUT] = gmt_make_filename (GMT, Ctrl->D.file, fmt, GMT->session.d_NaN, closed, cont_counts);
+					}
 				}
 				if (make_plot) draw_clip_contours (GMT, PSL, x, y, n, Ctrl->G.fill.rgb, section, first);
 				first = 0;
@@ -893,7 +927,7 @@ int GMT_psmask (void *V_API, int mode, void *args) {
 		else {	/* Just paint tiles */
 			uint64_t start, n_use, np, plot_n;
 			double y_bot, y_top, *xx = NULL, *yy = NULL, *xp = NULL, *yp = NULL;
-			GMT_Report (API, GMT_MSG_VERBOSE, "Tiling...\n");
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Tiling...\n");
 
 			for (row = 0; row < Grid->header->n_rows; row++) {
 				y_bot = grd_y0[row] - inc2[GMT_Y];
@@ -941,7 +975,7 @@ int GMT_psmask (void *V_API, int mode, void *args) {
 		if (Ctrl->S.active) gmt_M_free (GMT, d_col);
 		gmt_M_free (GMT, grd_x0);
 		gmt_M_free (GMT, grd_y0);
-		if (make_plot && !Ctrl->T.active) GMT_Report (API, GMT_MSG_VERBOSE, "Clipping on!\n");
+		if (make_plot && !Ctrl->T.active) GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Clipping on!\n");
 	}
 
 	gmt_set_pad (GMT, API->pad);		/* Reset default pad */
