@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *	Copyright (c) 1991-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  * Brief synopsis: psrose reads a file [or standard input] with azimuth and length information
@@ -25,36 +24,53 @@
  * and azimuth to be "y".  Hence, west = 0.0 and east = max_radius
  * south/north is -90,90 for halfcircle and 0,360 for full circle
  *
+ * Update Nov 2017 PW: psrose map machinery is ancient and did not support -J.  For modern
+ * mode we do need -J support.  To remain backwards compatible we now take these steps:
+ *  a) Before parsing of arguments we check if -J was given.  If not we find the radius
+ *     from -S (if given, else default to 3i) and create a -JX option that we add. If
+ *     normalization was set via -S then we retain a plain -S option that now only means
+ *     do normalization.
+ *  b) psrose then processes -JX and optionally -S. Because -JX is then parsed we must be
+ *     on the look-out for any -: as these are inert.  We reactivate that swap if needed.
+ *  c) We then extract the radius from the -JX string, decode it, and reestablish the old
+ *     -Jx1 scaling using the +/- radius as w/e/s/n.
+ *  d) The old syntax is undocumented but support by backwards compatibility through GMT 6.
+ *
  * Author:	Paul Wessel
  * Date:	1-JAN-2010
- * Version:	5 API
+ * Version:	6 API
  */
 
 #include "gmt_dev.h"
 
-#define THIS_MODULE_NAME	"psrose"
+#define THIS_MODULE_CLASSIC_NAME	"psrose"
+#define THIS_MODULE_MODERN_NAME	"rose"
 #define THIS_MODULE_LIB		"core"
 #define THIS_MODULE_PURPOSE	"Plot a polar histogram (rose, sector, windrose diagrams)"
 #define THIS_MODULE_KEYS	"<D{,CC(,ED(,>X},>D),>DI@<D{,ID),CC("
-#define THIS_MODULE_NEEDS	"RJ"
-#define THIS_MODULE_OPTIONS "-:>BKOPRUVXYbdehipstxy" GMT_OPT("Ec")
+#define THIS_MODULE_NEEDS	"JR"
+#define THIS_MODULE_OPTIONS "-:>BJKOPRUVXYbdehipstxy" GMT_OPT("c")
 
 struct PSROSE_CTRL {	/* All control options for this program (except common args) */
 	/* active is true if the option has been activated */
-	struct A {	/* -A<sector_angle>[r] */
+	struct A {	/* -A<sector_angle>[+r] */
 		bool active;
 		bool rose;
 		double inc;
 	} A;
-	struct C {	/* -Cm|[+w]<modefile> */
+	struct C {	/* -C<cpt> */
 		bool active;
-		bool mode;
-		bool mean;
 		char *file;
 	} C;
 	struct D {	/* -D */
 		bool active;
 	} D;
+	struct E {	/* -Em|[+w]<modefile> */
+		bool active;
+		bool mode;
+		bool mean;
+		char *file;
+	} E;
 	struct F {	/* -F */
 		bool active;
 	} F;
@@ -80,10 +96,10 @@ struct PSROSE_CTRL {	/* All control options for this program (except common args
 		bool active;
 		double value;
 	} Q;
-	struct S {	/* -Sscale[n] */
+	struct S {	/* -S */
 		bool active;
 		bool normalize;
-		double scale;
+		double scale;	/* Get this via -JX */
 	} S;
 	struct T {	/* -T */
 		bool active;
@@ -109,17 +125,13 @@ GMT_LOCAL void *New_Ctrl (struct GMT_CTRL *GMT) {	/* Allocate and initialize a n
 	C->M.S.symbol = PSL_VECTOR;
 	C->W.pen[0] = C->W.pen[1] = GMT->current.setting.map_default_pen;
 	C->Q.value = 0.05;
-	C->S.scale = 3.0;
 	C->Z.scale = 1.0;
-	if (GMT->current.setting.proj_length_unit == GMT_CM) {
-		C->S.scale = 7.5 / 2.54;
-	}
 	return (C);
 }
 
 GMT_LOCAL void Free_Ctrl (struct GMT_CTRL *GMT, struct PSROSE_CTRL *C) {	/* Deallocate control structure */
 	if (!C) return;
-	gmt_M_str_free (C->C.file);
+	gmt_M_str_free (C->E.file);
 	gmt_M_str_free (C->L.w);
 	gmt_M_str_free (C->L.e);
 	gmt_M_str_free (C->L.s);
@@ -143,36 +155,39 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 
 	/* This displays the psrose synopsis and optionally full usage information */
 
-	gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
+	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: psrose [<table>] [-A[r]<sector_angle>] [%s] [-C[m|[+w]<modefile>]] [-D] [-G<fill>] [-I] [-K]\n", GMT_B_OPT);
-	GMT_Message (API, GMT_TIME_NONE, "\t[-L[<wlab>,<elab>,<slab>,<nlab>]] [-M[<size>][<modifiers>]] [-N] [-O] [-P] [-Q<alpha>]\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t[-R<r0>/<r1>/<theta0>/<theta1>] [-S[n]<scale>] [-T] [%s]\n", GMT_U_OPT);
-	GMT_Message (API, GMT_TIME_NONE, "\t[%s] [-W[v]<pen>] [%s] [%s]\n\t[-Zu|<scale>] [%s] [%s]\n\t[%s] [%s] [%s]\n\t[%s] [%s]\n\t[%s] [%s]\n\n",
-		GMT_V_OPT, GMT_X_OPT, GMT_Y_OPT, GMT_bi_OPT, GMT_di_OPT, GMT_e_OPT, GMT_h_OPT, GMT_i_OPT, GMT_p_OPT, GMT_s_OPT, GMT_t_OPT, GMT_colon_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "usage: %s [<table>] [-A<sector_angle>[+r]] [%s] [-C<cpt>] [-D] [-E[m|[+w]<modefile>]] [-G<fill>] [-I]\n", name, GMT_B_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "\t[-JX<width[u]>] %s[-L[<wlab>,<elab>,<slab>,<nlab>]] [-M[<size>][<modifiers>]] [-N] %s%s[-Q<alpha>]\n", API->K_OPT, API->O_OPT, API->P_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "\t[-R<r0>/<r1>/<theta0>/<theta1>] [-S] [-T] [%s]\n", GMT_U_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "\t[%s] [-W[v]<pen>] [%s] [%s]\n\t[-Zu|<scale>] [%s] %s[%s] [%s]\n\t[%s] [%s]\n\t[%s] [%s]\n\t[%s] [%s] [%s]\n\n",
+		GMT_V_OPT, GMT_X_OPT, GMT_Y_OPT, GMT_bi_OPT, API->c_OPT, GMT_di_OPT, GMT_e_OPT, GMT_h_OPT, GMT_i_OPT, GMT_p_OPT, GMT_s_OPT, GMT_t_OPT, GMT_colon_OPT, GMT_PAR_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
 	GMT_Message (API, GMT_TIME_NONE, "\tOPTIONS:\n");
 	GMT_Option (API, "<");
 	GMT_Message (API, GMT_TIME_NONE, "\t-A Sector width in degrees for sector diagram [Default is windrose];\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Use -Ar to get rose diagram.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   Append +r to get rose diagram.\n");
 	GMT_Option (API, "B-");
 	if (gmt_M_showusage (API)) {
 		GMT_Message (API, GMT_TIME_NONE, "\t   The scale bar length is set to the radial gridline spacing.\n");
 		GMT_Message (API, GMT_TIME_NONE, "\t   (Remember: radial is x-direction, azimuthal is y-direction).\n");
 	}
-	GMT_Message (API, GMT_TIME_NONE, "\t-C Plot vectors listed in the <modefile> file. For calculated mean direction, choose -Cm.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   To write the calculated mean direction etc. to file, append +w<modfile>.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t-C Use CPT to assign fill to sectors based on the r-value. Requires -A (sector diagram)\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-D Will center the sectors.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t-E Plot vectors listed in the <modefile> file. For calculated mean direction, choose -Em.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   To write the calculated mean direction etc. to file, append +w<modfile>.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-F Do not draw the scale length bar [Default plots scale in lower right corner].\n");
-	gmt_fill_syntax (API->GMT, 'G', "Specify color for diagram [Default is no fill].");
+	gmt_fill_syntax (API->GMT, 'G', NULL, "Specify color for diagram [Default is no fill].");
 	GMT_Message (API, GMT_TIME_NONE, "\t-I Inquire mode; only compute and report statistics - no plot is created.\n");
+	r = (API->GMT->current.setting.proj_length_unit == GMT_CM) ? 7.5 : 3.0;
+	GMT_Message (API, GMT_TIME_NONE, "\t-J Use -JX<width>[unit] to set the plot diameter [%g %s].\n", r, API->GMT->session.unit_name[API->GMT->current.setting.proj_length_unit]);
 	GMT_Option (API, "K");
 	GMT_Message (API, GMT_TIME_NONE, "\t-L Override default labels [West,East,South,North (depending on GMT_LANGUAGE)\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   for full circle and 90W,90E,-,0 for half-circle].  If no argument \n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   is given then labels will be disabled.  Give - to disable an individual label.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-M Specify arrow attributes.  If -C is used then the attributes apply to the -C vector(s).\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t-M Specify arrow attributes.  If -E is used then the attributes apply to the -E vector(s).\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   Otherwise, if windrose mode is selected we apply vector attributes to individual directions.\n");
 	gmt_vector_syntax (API->GMT, 15);
 	GMT_Message (API, GMT_TIME_NONE, "\t   Default is %gp+gblack+p1p.\n", VECTOR_HEAD_LENGTH);
@@ -183,19 +198,17 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t-R Specifies the region (<r0> = 0, <r1> = max_radius).  For azimuth:\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   Specify <theta0>/<theta1> = -90/90 or 0/180 (half-circles) or 0/360 only).\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   If <r0> = <r1> = 0, psrose will compute a reasonable <r1> value.\n");
-	r = (API->GMT->current.setting.proj_length_unit == GMT_CM) ? 7.5 : 3.0;
-	GMT_Message (API, GMT_TIME_NONE, "\t-S Specify the plot radius of the unit circle in %s [%g].\n", API->GMT->session.unit_name[API->GMT->current.setting.proj_length_unit], r);
-	GMT_Message (API, GMT_TIME_NONE, "\t   Use -Sn to normalize data, i.e., divide all radii (or bin counts) by the maximum radius (or count).\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t-S Normalize data, i.e., divide all radii (or bin counts) by the maximum radius (or count).\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-T Indicate that the vectors are oriented (two-headed), not directed [Default].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   This implies both <azimuth> and <azimuth> + 180 will be counted as inputs.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   Ignored if -R sets a half-circle domain.\n");
 	GMT_Option (API, "U,V");
-	gmt_pen_syntax (API->GMT, 'W', "Set pen attributes for outline of rose [Default is no outline].", 0);
-	GMT_Message (API, GMT_TIME_NONE, "\t   Use -Wv<pen> to set a different pen for the vector (requires -C) [Same as rose outline].\n");
+	gmt_pen_syntax (API->GMT, 'W', NULL, "Set pen attributes for outline of rose [Default is no outline].", 0);
+	GMT_Message (API, GMT_TIME_NONE, "\t   Use -Wv<pen> to set a different pen for the vector (requires -E) [Same as rose outline].\n");
 	GMT_Option (API, "X");
 	GMT_Message (API, GMT_TIME_NONE, "\t-Z Multiply the radii by <scale> before plotting; use -Zu to set input radii to 1.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-: Expect (azimuth,radius) input rather than (radius,azimuth) [%s].\n", choice[API->GMT->current.setting.io_lonlat_toggle[GMT_IN]]);
-	GMT_Option (API, "bi2,di,e,h,i,p,s,t,.");
+	GMT_Option (API, "bi2,c,di,e,h,i,p,s,t,.");
 
 	return (GMT_MODULE_USAGE);
 }
@@ -225,37 +238,69 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct PSROSE_CTRL *Ctrl, struct GMT_
 
 			/* Processes program-specific parameters */
 
-			case 'A':	/* Get Sector angle in degrees */
+			case 'A':	/* Get Sector angle in degrees -A<inc>[+r]*/
 				Ctrl->A.active = true;
-				if (strchr (opt->arg, 'r')) Ctrl->A.rose = true;
-				k = (opt->arg[0] == 'r') ? 1 : 0;
+				k = 0;
+				if ((c = strstr (opt->arg, "+r"))) {
+					Ctrl->A.rose = true;
+					c[0] = '\0';	/* Chop off modifier */
+				}
+				else if (strchr (opt->arg, 'r')) {	/* Old syntax -A[r]<inc> */
+					Ctrl->A.rose = true;
+					if (opt->arg[0] == 'r') k = 1;
+				}
 				Ctrl->A.inc = atof (&opt->arg[k]);
+				if (c) c[0] = '+';	/* Restore modifier */
 				break;
-			case 'C':	/* Read mode file and plot mean directions */
+			case 'C':
+				if (gmt_M_compat_check (GMT, 5)) {	/* Need to check for deprecated -Cm|[+w]<modefile> option */
+					if (((c = strstr (opt->arg, "+w")) || (opt->arg[0] == 'm' && opt->arg[1] == '\0') || opt->arg[0] == '\0') && strstr (opt->arg, ".cpt") == NULL) {
+						GMT_Report (API, GMT_MSG_COMPAT, "Option -C for mode-vector(s) is deprecated; use -E instead.\n");
+						Ctrl->E.active = true;
+						if ((c = strstr (opt->arg, "+w"))) {	/* Wants to write out mean direction */
+							gmt_M_str_free (Ctrl->E.file);
+							if (c[2]) Ctrl->E.file = strdup (&c[2]);
+							Ctrl->E.mode = GMT_OUT;
+							c[0] = '\0';	/* Chop off temporarily */
+						}
+						if ((opt->arg[0] == 'm' && opt->arg[1] == '\0') || (API->external == 0 && opt->arg[0] == '\0'))
+							Ctrl->E.mean = true;
+						else if (Ctrl->E.mode == GMT_IN) {
+							gmt_M_str_free (Ctrl->E.file);
+							if (opt->arg[0]) Ctrl->E.file = strdup (opt->arg);
+						}
+						break;
+					}
+				}
 				Ctrl->C.active = true;
-				if ((c = strstr (opt->arg, "+w"))) {	/* Wants to write out mean direction */
-					gmt_M_str_free (Ctrl->C.file);
-					if (c[2]) Ctrl->C.file = strdup (&c[2]);
-					Ctrl->C.mode = GMT_OUT;
-					c[0] = '\0';	/* Chop off temporarily */
-				}
-				if ((opt->arg[0] == 'm' && opt->arg[1] == '\0') || (API->external == 0 && opt->arg[0] == '\0'))
-					Ctrl->C.mean = true;
-				else if (Ctrl->C.mode == GMT_IN) {
-					gmt_M_str_free (Ctrl->C.file);
-					if (opt->arg[0]) Ctrl->C.file = strdup (opt->arg);
-				}
+				gmt_M_str_free (Ctrl->C.file);
+				if (opt->arg[0]) Ctrl->C.file = strdup (opt->arg);
 				break;
 			case 'D':	/* Center the bins */
 				Ctrl->D.active = true;
 				break;
+			case 'E':	/* Read mode file and plot mean directions */
+				Ctrl->E.active = true;
+				if ((c = strstr (opt->arg, "+w"))) {	/* Wants to write out mean direction */
+					gmt_M_str_free (Ctrl->E.file);
+					if (c[2]) Ctrl->E.file = strdup (&c[2]);
+					Ctrl->E.mode = GMT_OUT;
+					c[0] = '\0';	/* Chop off temporarily */
+				}
+				if ((opt->arg[0] == 'm' && opt->arg[1] == '\0') || (API->external == 0 && opt->arg[0] == '\0'))
+					Ctrl->E.mean = true;
+				else if (Ctrl->E.mode == GMT_IN) {
+					gmt_M_str_free (Ctrl->E.file);
+					if (opt->arg[0]) Ctrl->E.file = strdup (opt->arg);
+				}
+				break;
 			case 'F':	/* Disable scalebar plotting */
 				Ctrl->F.active = true;
 				break;
 			case 'G':	/* Set Gray shade */
 				Ctrl->G.active = true;
 				if (gmt_getfill (GMT, opt->arg, &Ctrl->G.fill)) {
-					gmt_fill_syntax (GMT, 'G', " ");
+					gmt_fill_syntax (GMT, 'G', NULL, " ");
 					n_errors++;
 				}
 				break;
@@ -320,18 +365,13 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct PSROSE_CTRL *Ctrl, struct GMT_
 			case 'N':	/* Make sectors area be proportional to frequency instead of radius */
 				Ctrl->N.active = true;
 				break;
-			case 'Q':	/* Scale radii before using data */
+			case 'Q':	/* Set critical value [0.05] */
 				Ctrl->Q.active = true;
-				Ctrl->Q.value = atof (opt->arg);
+				if (opt->arg[0]) Ctrl->Q.value = atof (opt->arg);
 				break;
-			case 'S':	/* Get radius of unit circle in inches */
+			case 'S':	/* Normalization */
 				Ctrl->S.active = true;
-				if (strchr (opt->arg, 'n')) Ctrl->S.normalize = true;
-				n = (int)strlen (opt->arg) - 1;
-				if (opt->arg[n] == 'n') opt->arg[n] = 0;	/* Temporarily remove the n */
-				k = (opt->arg[0] == 'n') ? 1 : 0;		/* If we got -Sn<radius> */
-				Ctrl->S.scale = gmt_M_to_inch (GMT, &opt->arg[k]);
-				if (Ctrl->S.normalize && k == 0) opt->arg[n] = 'n';	/* Put back the n */
+				Ctrl->S.normalize = true;
 				break;
 			case 'T':	/* Oriented instead of directed data */
 				Ctrl->T.active = true;
@@ -340,7 +380,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct PSROSE_CTRL *Ctrl, struct GMT_
 				n = (opt->arg[0] == 'v') ? 1 : 0;
 				Ctrl->W.active[n] = true;
 				if (gmt_getpen (GMT, &opt->arg[n], &Ctrl->W.pen[n])) {
-					gmt_pen_syntax (GMT, 'W', " ", 0);
+					gmt_pen_syntax (GMT, 'W', NULL, " ", 0);
 					n_errors++;
 				}
 				break;
@@ -358,21 +398,27 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct PSROSE_CTRL *Ctrl, struct GMT_
 		}
 	}
 
+	gmt_consider_current_cpt (API, &Ctrl->C.active, &(Ctrl->C.file));
+
 	/* Check that the options selected are mutually consistent */
 
 	GMT->common.R.wesn[XLO] = 0.0;
 	range = GMT->common.R.wesn[YHI] - GMT->common.R.wesn[YLO];
 	if (doubleAlmostEqual (range, 180.0) && Ctrl->T.active) {
-		GMT_Report (API, GMT_MSG_NORMAL, "Warning: -T only needed for 0-360 range data (ignored)");
+		GMT_Report (API, GMT_MSG_VERBOSE, "-T only needed for 0-360 range data (ignored)");
 		Ctrl->T.active = false;
 	}
-	n_errors += gmt_M_check_condition (GMT, Ctrl->C.active && Ctrl->C.file && Ctrl->C.mode == GMT_IN && gmt_access (GMT, Ctrl->C.file, R_OK),
-	                                 "Syntax error -C: Cannot read file %s!\n", Ctrl->C.file);
-	n_errors += gmt_M_check_condition (GMT, Ctrl->S.scale <= 0.0, "Syntax error -S option: radius must be nonzero\n");
+	n_errors += gmt_M_check_condition (GMT, Ctrl->E.active && Ctrl->E.file && Ctrl->E.mode == GMT_IN && gmt_access (GMT, Ctrl->E.file, R_OK),
+	                                 "Syntax error -E: Cannot read file %s!\n", Ctrl->E.file);
 	n_errors += gmt_M_check_condition (GMT, gmt_M_is_zero (Ctrl->Z.scale), "Syntax error -Z option: factor must be nonzero\n");
 	n_errors += gmt_M_check_condition (GMT, Ctrl->A.inc < 0.0, "Syntax error -A option: sector width must be positive\n");
 	n_errors += gmt_M_check_condition (GMT, Ctrl->Q.value <= 0.0 || Ctrl->Q.value >= 1.0, "Syntax error -Q option: confidence level must be in 0-1 range\n");
+	n_errors += gmt_M_check_condition (GMT, Ctrl->C.active && Ctrl->G.active, "Syntax error -C option: Cannot give both -C and -G\n");
+	n_errors += gmt_M_check_condition (GMT, Ctrl->C.active && Ctrl->A.rose, "Syntax error -C option: Cannot be used with -A+r\n");
+	n_errors += gmt_M_check_condition (GMT, Ctrl->C.active && !Ctrl->A.active, "Syntax error -C option: Requires -A\n");
 	if (!Ctrl->I.active) {
+		n_errors += gmt_M_check_condition (GMT, !GMT->common.J.active, "Syntax error: Must specify -JX option\n");
+		n_errors += gmt_M_check_condition (GMT, GMT->current.proj.projection != GMT_LINEAR, "Syntax error: Must specify -JX option\n");
 		n_errors += gmt_M_check_condition (GMT, !GMT->common.R.active[RSET], "Syntax error: Must specify -R option\n");
 		n_errors += gmt_M_check_condition (GMT, !((GMT->common.R.wesn[YLO] == -90.0 && GMT->common.R.wesn[YHI] == 90.0) \
 			|| (GMT->common.R.wesn[YLO] == 0.0 && GMT->common.R.wesn[YHI] == 180.0)
@@ -387,10 +433,20 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct PSROSE_CTRL *Ctrl, struct GMT_
 #define bailout(code) {gmt_M_free_options (mode); return (code);}
 #define Return(code) {Free_Ctrl (GMT, Ctrl); gmt_end_module (GMT, GMT_cpy); bailout (code);}
 
+int GMT_rose (void *V_API, int mode, void *args) {
+	/* This is the GMT6 modern mode name */
+	struct GMTAPI_CTRL *API = gmt_get_api_ptr (V_API);	/* Cast from void to GMTAPI_CTRL pointer */
+	if (API->GMT->current.setting.run_mode == GMT_CLASSIC && !API->usage) {
+		GMT_Report (API, GMT_MSG_NORMAL, "Shared GMT module not found: rose\n");
+		return (GMT_NOT_A_VALID_MODULE);
+	}
+	return GMT_psrose (V_API, mode, args);
+}
+
 int GMT_psrose (void *V_API, int mode, void *args) {
 	bool do_fill = false, automatic = false, sector_plot = false, windrose = true;
 	unsigned int n_bins, n_modes = 0, form, n_in, half_only = 0, bin;
-	int error = 0, k, n_annot, n_alpha, sbin, significant;
+	int error = 0, k, n_annot, n_alpha, sbin, significant, index;
 
 	uint64_t n = 0, i;
 
@@ -403,11 +459,14 @@ int GMT_psrose (void *V_API, int mode, void *args) {
 	double x1, x2, y1, y2, total = 0.0, total_arc, off, max_radius, az_offset, start_angle;
 	double asize, lsize, this_az, half_bin_width, diameter, wesn[4], mean_vector, mean_resultant;
 	double *xx = NULL, *yy = NULL, *in = NULL, *sum = NULL, *azimuth = NULL, critical_resultant;
-	double *length = NULL, *mode_direction = NULL, *mode_length = NULL, dim[PSL_MAX_DIMS];
+	double *length = NULL, *mode_direction = NULL, *mode_length = NULL, dim[PSL_MAX_DIMS], rgb[4];
 
 	struct PSROSE_CTRL *Ctrl = NULL;
+	struct GMT_FILL *F = NULL;
 	struct GMT_DATASET *Cin = NULL;
-	struct GMT_DATATABLE *P = NULL;
+	struct GMT_DATATABLE *T = NULL;
+	struct GMT_PALETTE *P = NULL;
+	struct GMT_RECORD *In = NULL;
 	struct GMT_CTRL *GMT = NULL, *GMT_cpy = NULL;		/* General GMT internal parameters */
 	struct GMT_OPTION *options = NULL;
 	struct PSL_CTRL *PSL = NULL;		/* General PSL internal parameters */
@@ -419,19 +478,18 @@ int GMT_psrose (void *V_API, int mode, void *args) {
 	if (mode == GMT_MODULE_PURPOSE) return (usage (API, GMT_MODULE_PURPOSE));	/* Return the purpose of program */
 	options = GMT_Create_Options (API, mode, args);	if (API->error) return (API->error);	/* Set or get option list */
 
-	if (!options || options->option == GMT_OPT_USAGE) bailout (usage (API, GMT_USAGE));	/* Return the usage message */
-	if (options->option == GMT_OPT_SYNOPSIS) bailout (usage (API, GMT_SYNOPSIS));	/* Return the synopsis */
+	if ((error = gmt_report_usage (API, options, 0, usage)) != GMT_NOERROR) bailout (error);	/* Give usage if requested */
 
 	/* Parse the command-line arguments; return if errors are encountered */
 
-	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
 	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
 	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
 	if ((error = parse (GMT, Ctrl, options)) != 0) Return (error);
 
 	/*---------------------------- This is the psrose main code ----------------------------*/
 
-	GMT_Report (API, GMT_MSG_VERBOSE, "Processing input table data\n");
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Processing input table data\n");
 	asize = GMT->current.setting.font_annot[GMT_PRIMARY].size * GMT->session.u2u[GMT_PT][GMT_INCH];
 	lsize = GMT->current.setting.font_annot[GMT_PRIMARY].size * GMT->session.u2u[GMT_PT][GMT_INCH];
 	gmt_M_memset (dim, PSL_MAX_DIMS, double);
@@ -467,10 +525,14 @@ int GMT_psrose (void *V_API, int mode, void *args) {
 
 	/* Read data and do some stats */
 
-	n = 0;
-	n_in = (GMT->common.i.active && GMT->common.i.n_cols == 1) ? 1 : 2;
+	if (Ctrl->C.active && (P = GMT_Read_Data (API, GMT_IS_PALETTE, GMT_IS_FILE, GMT_IS_NONE, GMT_READ_NORMAL, NULL, Ctrl->C.file, NULL)) == NULL) {
+		Return (GMT_DATA_READ_ERROR);
+	}
 
-	if ((error = gmt_set_cols (GMT, GMT_IN, n_in)) != GMT_NOERROR) {
+	n = 0;
+	n_in = (GMT->common.i.select && GMT->common.i.n_cols == 1) ? 1 : 2;
+
+	if ((error = GMT_Set_Columns (API, GMT_IN, n_in, GMT_COL_FIX_NO_TEXT)) != GMT_NOERROR) {
 		Return (error);
 	}
 	if (GMT_Init_IO (API, GMT_IS_DATASET, GMT_IS_POINT, GMT_IN, GMT_ADD_DEFAULT, 0, options) != GMT_NOERROR) {	/* Register data input */
@@ -487,9 +549,14 @@ int GMT_psrose (void *V_API, int mode, void *args) {
 	yy = gmt_M_memory (GMT, NULL, n_bins+2, double);
 	azimuth = gmt_M_memory (GMT, NULL, n_alloc, double);
 	length = gmt_M_memory (GMT, NULL, n_alloc, double);
-
+	
+	/* Because of -JX being parsed already, any -: will have no effect.  For backwards compatibility we
+	 * check if -: was given and turn that on again here before reading */
+	if (GMT->common.colon.active)
+		GMT->current.setting.io_lonlat_toggle[GMT_IN] = true;
+	
 	do {	/* Keep returning records until we reach EOF */
-		if ((in = GMT_Get_Record (API, GMT_READ_DATA, NULL)) == NULL) {	/* Read next record, get NULL if special case */
+		if ((In = GMT_Get_Record (API, GMT_READ_DATA, NULL)) == NULL) {	/* Read next record, get NULL if special case */
 			if (gmt_M_rec_is_error (GMT)) { 	/* Bail if there are any read errors */
 				gmt_M_free (GMT, length);		gmt_M_free (GMT, xx);	gmt_M_free (GMT, sum);
 				gmt_M_free (GMT, azimuth);		gmt_M_free (GMT, yy);
@@ -501,6 +568,7 @@ int GMT_psrose (void *V_API, int mode, void *args) {
 		}
 
 		/* Data record to process */
+		in = In->data;	/* Only need to process numerical part here */
 
 		if (n_in == 2) {	/* Read azimuth and length */
 			length[n]  = in[GMT_X];
@@ -592,18 +660,20 @@ int GMT_psrose (void *V_API, int mode, void *args) {
 			for (i = 0; i < n; i++) length[i] /= max;
 	}
 
-	if (Ctrl->I.active || gmt_M_is_verbose (GMT, GMT_MSG_VERBOSE)) {
+	if (Ctrl->I.active || gmt_M_is_verbose (GMT, GMT_MSG_LONG_VERBOSE)) {
 		char *kind[2] = {"r", "bin sum"};
 		sprintf (format, "Info for data: n = %% " PRIu64 " mean az = %s mean r = %s mean resultant length = %s max %s = %s scaled mean r = %s linear length sum = %s sign@%%.2f = %%d\n",
 			GMT->current.setting.format_float_out, GMT->current.setting.format_float_out, GMT->current.setting.format_float_out, kind[Ctrl->A.active],
 			GMT->current.setting.format_float_out, GMT->current.setting.format_float_out, GMT->current.setting.format_float_out);
-		GMT_Report (API, GMT_MSG_NORMAL, format, n, mean_theta, mean_vector, mean_resultant, max, mean_radius, total, Ctrl->Q.value, significant);
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, format, n, mean_theta, mean_vector, mean_resultant, max, mean_radius, total, Ctrl->Q.value, significant);
 		if (Ctrl->I.active) {	/* That was all we needed to do, wrap up */
 			double out[7];
 			unsigned int col_type[2];
+			struct GMT_RECORD *Rec = gmt_new_record (GMT, out, NULL);
 			gmt_M_memcpy (col_type, GMT->current.io.col_type[GMT_OUT], 2U, unsigned int);	/* Save first 2 current output col types */
-			GMT->current.io.col_type[GMT_OUT][0] = GMT->current.io.col_type[GMT_OUT][1] = GMT_IS_FLOAT;
-			if ((error = gmt_set_cols (GMT, GMT_OUT, 7U)) != GMT_NOERROR) {
+			gmt_set_column (GMT, GMT_OUT, GMT_X, GMT_IS_FLOAT);
+			gmt_set_column (GMT, GMT_OUT, GMT_Y, GMT_IS_FLOAT);
+			if ((error = GMT_Set_Columns (API, GMT_OUT, 7U, GMT_COL_FIX_NO_TEXT)) != GMT_NOERROR) {
 				gmt_M_free (GMT, sum);
 				gmt_M_free (GMT, xx);
 				gmt_M_free (GMT, yy);
@@ -639,21 +709,17 @@ int GMT_psrose (void *V_API, int mode, void *args) {
 			out[0] = (double)n; out[1] = mean_theta;	out[2] = mean_vector;	out[3] = mean_resultant;
 			out[4] = max;	out[5] = mean_radius;	out[6] = total;
 			GMT_Put_Record (API, GMT_WRITE_TABLE_HEADER, format);	/* Write this to output if -ho */
-			GMT_Put_Record (API, GMT_WRITE_DATA, out);
-			if (GMT_End_IO (API, GMT_OUT, 0) != GMT_NOERROR) {	/* Disables further data output */
-				gmt_M_free (GMT, sum);
-				gmt_M_free (GMT, xx);
-				gmt_M_free (GMT, yy);
-				gmt_M_free (GMT, azimuth);
-				gmt_M_free (GMT, length);
-				Return (API->error);
-			}
-			gmt_M_memcpy (GMT->current.io.col_type[GMT_OUT], col_type, 2U, unsigned int);	/* Restore 2 current output col types */
+			GMT_Put_Record (API, GMT_WRITE_DATA, Rec);
+			gmt_M_free (GMT, Rec);
 			gmt_M_free (GMT, sum);
 			gmt_M_free (GMT, xx);
 			gmt_M_free (GMT, yy);
 			gmt_M_free (GMT, azimuth);
 			gmt_M_free (GMT, length);
+			if (GMT_End_IO (API, GMT_OUT, 0) != GMT_NOERROR) {	/* Disables further data output */
+				Return (API->error);
+			}
+			gmt_M_memcpy (GMT->current.io.col_type[GMT_OUT], col_type, 2U, unsigned int);	/* Restore 2 current output col types */
 			Return (GMT_NOERROR);
 		}
 	}
@@ -676,6 +742,8 @@ int GMT_psrose (void *V_API, int mode, void *args) {
 
 	/* Ready to plot.  So set up GMT projections (not used by psrose), we set region to actual plot width and scale to 1 */
 
+	Ctrl->S.scale = 0.5 * gmt_M_to_inch (GMT, &GMT->common.J.string[1]);	/* Get radius from full width */
+	GMT->common.J.active = false;	/* Reset projection machinery */
 	gmt_parse_common_options (GMT, "J", 'J', "x1i");
 	GMT->common.R.active[RSET] = GMT->common.J.active = true;
 	wesn[XLO] = wesn[YLO] = -Ctrl->S.scale;	wesn[XHI] = wesn[YHI] = Ctrl->S.scale;
@@ -714,19 +782,29 @@ int GMT_psrose (void *V_API, int mode, void *args) {
 	}
 	if (GMT->common.B.active[0]) {	/* Draw frame */
 		int symbol = (half_only) ? PSL_WEDGE : PSL_CIRCLE;
-		double dim[3];
+		double dim[PSL_MAX_DIMS];
 		struct GMT_FILL no_fill;
 
 		gmt_init_fill (GMT, &no_fill, -1.0, -1.0, -1.0);
+		gmt_M_memset (dim, PSL_MAX_DIMS, double);
 		dim[0] = (half_only) ? 0.5 * diameter : diameter;
 		dim[1] = 0.0;
 		dim[2] = (half_only) ? 180.0 : 360.0;
+		dim[7] = 2;	/* Do draw line */
 		gmt_setpen (GMT, &GMT->current.setting.map_frame_pen);
 		gmt_setfill (GMT, &no_fill, true);
 		PSL_plotsymbol (PSL, 0.0, 0.0, dim, symbol);
 	}
 
 	gmt_setpen (GMT, &Ctrl->W.pen[0]);
+	if (Ctrl->M.S.v.status & PSL_VEC_OUTLINE2) {	/* Gave specific head outline pen */
+		PSL_defpen (GMT->PSL, "PSL_vecheadpen", Ctrl->M.S.v.pen.width, Ctrl->M.S.v.pen.style, Ctrl->M.S.v.pen.offset, Ctrl->M.S.v.pen.rgb);
+		dim[11] = Ctrl->M.S.v.pen.width;
+	}
+	else if (Ctrl->M.active) {
+		PSL_defpen (GMT->PSL, "PSL_vecheadpen", 0.5 * Ctrl->W.pen[1].width, Ctrl->W.pen[1].style, Ctrl->W.pen[1].offset, Ctrl->W.pen[1].rgb);
+		dim[11] = 0.5 * Ctrl->W.pen[1].width;
+	}
 	if (windrose) {	/* Here we draw individual vectors */
 		if (Ctrl->M.active) { /* Initialize vector head settings */
 			gmt_init_vector_param (GMT, &Ctrl->M.S, false, false, NULL, false, NULL);
@@ -787,10 +865,22 @@ int GMT_psrose (void *V_API, int mode, void *args) {
 		}
 	}
 
-	if (sector_plot && !Ctrl->A.rose && Ctrl->G.active) {	/* Draw pie slices for sector plot if fill is requested */
+	if (sector_plot && !Ctrl->A.rose && (Ctrl->C.active || Ctrl->G.active)) {	/* Draw pie slices for sector plot if fill is requested */
 
-		gmt_setfill (GMT, &(Ctrl->G.fill), false);
+		if (Ctrl->G.active)
+			gmt_setfill (GMT, &(Ctrl->G.fill), false);
+		dim[7] = 0;
+		if (Ctrl->G.active) dim[7] = 1;
+		if (Ctrl->W.active[0]) dim[7] += 2;
 		for (bin = 0; bin < n_bins; bin++) {
+			if (Ctrl->C.active) {
+				index = gmt_get_rgb_from_z (GMT, P, sum[bin] * Ctrl->S.scale, rgb);
+				F = gmt_M_get_cptslice_pattern (P, index);
+				if (F)	/* Pattern */
+					gmt_setfill (GMT, F, false);
+				else
+					PSL_setfill (PSL, rgb, false);
+			}
 			az = bin * Ctrl->A.inc - az_offset + half_bin_width;
 			dim[1] = (start_angle - az - Ctrl->A.inc);
 			dim[2] = dim[1] + Ctrl->A.inc;
@@ -836,11 +926,11 @@ int GMT_psrose (void *V_API, int mode, void *args) {
 		}
 	}
 
-	if (Ctrl->C.active) {
+	if (Ctrl->E.active) {
 		unsigned int this_mode;
 		int v4_outline = Ctrl->W.active[1];
 		double *this_rgb = NULL;
-		if (Ctrl->C.mode == GMT_OUT) {	/* Write mean vector and statistics to file */
+		if (Ctrl->E.mode == GMT_OUT) {	/* Write mean vector and statistics to file */
 			uint64_t dim[GMT_DIM_SIZE] = {1, 1, 1, 8};	/* One record with 8 columns */
 			struct GMT_DATASET *V = NULL;
 			struct GMT_DATASEGMENT *S = NULL;
@@ -849,41 +939,42 @@ int GMT_psrose (void *V_API, int mode, void *args) {
 			}
 			S = V->table[0]->segment[0];	/* The only segment */
 			sprintf (format, "mean_az\tmean_r\tmean_resultant\tmax\tscaled_mean_r\tlength_sum\tn\tsign@%.2f", Ctrl->Q.value);
-			S->coord[GMT_X][0] = mean_theta;
-			S->coord[GMT_Y][0] = mean_radius;
-			S->coord[GMT_Z][0] = mean_resultant;
-			S->coord[3][0] = max;
-			S->coord[4][0] = mean_radius;
-			S->coord[5][0] = total;
-			S->coord[6][0] = (double)n;
-			S->coord[7][0] = significant;
+			S->data[GMT_X][0] = mean_theta;
+			S->data[GMT_Y][0] = mean_radius;
+			S->data[GMT_Z][0] = mean_resultant;
+			S->data[3][0] = max;
+			S->data[4][0] = mean_radius;
+			S->data[5][0] = total;
+			S->data[6][0] = (double)n;
+			S->data[7][0] = significant;
+			S->n_rows = 1;
 			if (GMT_Set_Comment (API, GMT_IS_DATASET, GMT_COMMENT_IS_COLNAMES, format, V)) {
 				Return (API->error);
 			}
-			if (GMT_Write_Data (API, GMT_IS_DATASET, GMT_IS_FILE, GMT_IS_POINT, GMT_WRITE_SET, NULL, Ctrl->C.file, V) != GMT_NOERROR) {
+			if (GMT_Write_Data (API, GMT_IS_DATASET, GMT_IS_FILE, GMT_IS_POINT, GMT_WRITE_SET, NULL, Ctrl->E.file, V) != GMT_NOERROR) {
 				Return (API->error);
 			}
 		}
 		if (!Ctrl->W.active[1]) Ctrl->W.pen[1] = Ctrl->W.pen[0];	/* No separate pen specified; use same as for rose outline */
-		if (Ctrl->C.mean) {	/* Not given, calculate and use mean direction only */
+		if (Ctrl->E.mean) {	/* Not given, calculate and use mean direction only */
 			n_modes = 1;
 			mode_direction = gmt_M_memory (GMT, NULL, 1, double);
 			mode_length = gmt_M_memory (GMT, NULL, 1, double);
 			mode_direction[0] = mean_theta;
 			mode_length[0] = mean_radius;
 		}
-		else if (Ctrl->C.mode == GMT_IN) {	/* Get mode parameters from separate file */
-			if ((Cin = GMT_Read_Data (API, GMT_IS_DATASET, GMT_IS_FILE, GMT_IS_NONE, GMT_READ_NORMAL, NULL, Ctrl->C.file, NULL)) == NULL) {
+		else if (Ctrl->E.mode == GMT_IN) {	/* Get mode parameters from separate file */
+			if ((Cin = GMT_Read_Data (API, GMT_IS_DATASET, GMT_IS_FILE, GMT_IS_NONE, GMT_READ_NORMAL, NULL, Ctrl->E.file, NULL)) == NULL) {
 				Return (API->error);
 			}
 			if (Cin->n_columns < 2) {
-				GMT_Report (API, GMT_MSG_NORMAL, "Input file %s has %d column(s) but at least 2 are needed\n", Ctrl->C.file, (int)Cin->n_columns);
+				GMT_Report (API, GMT_MSG_NORMAL, "Input file %s has %d column(s) but at least 2 are needed\n", Ctrl->E.file, (int)Cin->n_columns);
 				Return (GMT_DIM_TOO_SMALL);
 			}
-			P = Cin->table[0];	/* Can only be one table since we read a single file; We also only use the first segment */
-			n_modes = (unsigned int)P->n_records;
-			mode_direction = P->segment[0]->data[GMT_X];
-			mode_length = P->segment[0]->data[GMT_Y];
+			T = Cin->table[0];	/* Can only be one table since we read a single file; We also only use the first segment */
+			n_modes = (unsigned int)T->n_records;
+			mode_direction = T->segment[0]->data[GMT_X];
+			mode_length = T->segment[0]->data[GMT_Y];
 		}
 		if (!Ctrl->M.active) {	/* Must supply defaults for the vector attributes */
 			Ctrl->M.S.size_x = VECTOR_HEAD_LENGTH * GMT->session.u2u[GMT_PT][GMT_INCH];	/* 9p */
@@ -909,6 +1000,16 @@ int GMT_psrose (void *V_API, int mode, void *args) {
 				this_rgb = GMT->session.no_rgb;
 			if (v4_outline) gmt_setpen (GMT, &Ctrl->W.pen[1]);
 		}
+		else {
+			if (Ctrl->M.S.v.status & PSL_VEC_OUTLINE2) {	/* Gave specific head outline pen */
+				PSL_defpen (GMT->PSL, "PSL_vecheadpen", Ctrl->M.S.v.pen.width, Ctrl->M.S.v.pen.style, Ctrl->M.S.v.pen.offset, Ctrl->M.S.v.pen.rgb);
+				dim[11] = Ctrl->M.S.v.pen.width;
+			}
+			else if (Ctrl->M.active) {
+				PSL_defpen (GMT->PSL, "PSL_vecheadpen", 0.5 * Ctrl->W.pen[1].width, Ctrl->W.pen[1].style, Ctrl->W.pen[1].offset, Ctrl->W.pen[1].rgb);
+				dim[11] = 0.5 * Ctrl->W.pen[1].width;
+			}
+		}
 		for (this_mode = 0; this_mode < n_modes; this_mode++) {
 			if (Ctrl->N.active) mode_length[this_mode] = sqrt (mode_length[this_mode]);
 			if (half_only && mode_direction[this_mode] > 90.0 && mode_direction[this_mode] <= 270.0) mode_direction[this_mode] -= 180.0;
@@ -955,7 +1056,7 @@ int GMT_psrose (void *V_API, int mode, void *args) {
 		if (GMT->current.map.frame.axis[GMT_X].item[GMT_GRID_UPPER].interval > 0.0) {
 			n_bins = urint (max_radius / GMT->current.map.frame.axis[GMT_X].item[GMT_GRID_UPPER].interval);
 			for (bin = 1; bin <= n_bins; bin++)
-				PSL_plotarc (PSL, 0.0, 0.0, bin * GMT->current.map.frame.axis[GMT_X].item[GMT_GRID_UPPER].interval * Ctrl->S.scale, 0.0, total_arc, PSL_MOVE + PSL_STROKE);
+				PSL_plotarc (PSL, 0.0, 0.0, bin * GMT->current.map.frame.axis[GMT_X].item[GMT_GRID_UPPER].interval * Ctrl->S.scale, 0.0, total_arc, PSL_MOVE|PSL_STROKE);
 		}
 		PSL_setcolor (PSL, GMT->current.setting.map_frame_pen.rgb, PSL_IS_STROKE);
 		y = lsize + 6.0 * GMT->current.setting.map_annot_offset[GMT_PRIMARY];
@@ -1047,8 +1148,8 @@ int GMT_psrose (void *V_API, int mode, void *args) {
 	gmt_M_free (GMT, yy);
 	gmt_M_free (GMT, azimuth);
 	gmt_M_free (GMT, length);
-	if (Ctrl->C.active) {
-		if (Ctrl->C.mean) {
+	if (Ctrl->E.active) {
+		if (Ctrl->E.mean) {
 			gmt_M_free (GMT, mode_length);
 			gmt_M_free (GMT, mode_direction);
 		}
