@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *	Copyright (c) 1991-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  *
@@ -23,7 +22,7 @@
  *
  * Author:	W. H. F. Smith
  * Date:	1-JAN-2010
- * Version:	5 API
+ * Version:	6 API
  * References:	Julius S. Bendat & Allan G. Piersol,
  *    	"Random Data", 2nd revised edition, 566pp.,
  *    	1986, John Wiley & Sons, New York. [ B&P below]
@@ -38,7 +37,8 @@
 
 #include "gmt_dev.h"
 
-#define THIS_MODULE_NAME	"spectrum1d"
+#define THIS_MODULE_CLASSIC_NAME	"spectrum1d"
+#define THIS_MODULE_MODERN_NAME	"spectrum1d"
 #define THIS_MODULE_LIB		"core"
 #define THIS_MODULE_PURPOSE	"Compute auto- [and cross-] spectra from one [or two] time series"
 #define THIS_MODULE_KEYS	"<D{,>D},T-)"
@@ -85,7 +85,7 @@ struct SPECTRUM1D_CTRL {
 struct SPECTRUM1D_INFO {	/* Control structure for spectrum1d */
 	unsigned int y_given;
 	int n_spec, window, window_2;
-	float *datac;
+	gmt_grdfloat *datac;
 	double dt, x_variance, y_variance, d_n_windows, y_pow;
 	struct SPEC {
 		double xpow;	/* PSD in X(t)  */
@@ -102,7 +102,7 @@ GMT_LOCAL void alloc_arrays (struct GMT_CTRL *GMT, struct SPECTRUM1D_INFO *C) {
 	C->window_2 = 2 * C->window;		/* This is for complex array stuff  */
 
 	C->spec = gmt_M_memory (GMT, NULL, C->n_spec, struct SPEC);
-	C->datac = gmt_M_memory (GMT, NULL, C->window_2, float);
+	C->datac = gmt_M_memory (GMT, NULL, C->window_2, gmt_grdfloat);
 }
 
 GMT_LOCAL void detrend_and_hanning (struct SPECTRUM1D_INFO *C, bool leave_trend, unsigned int mode) {
@@ -155,11 +155,11 @@ GMT_LOCAL void detrend_and_hanning (struct SPECTRUM1D_INFO *C, bool leave_trend,
 			hc = cos(t * h_period);
 			hw = h_scale * (1.0 - hc * hc);
 			tt = t * t_factor - 1.0;
-			C->datac[i] -= (float)(x_mean + tt * x_slope);
-			C->datac[i] *= (float)hw;
+			C->datac[i] -= (gmt_grdfloat)(x_mean + tt * x_slope);
+			C->datac[i] *= (gmt_grdfloat)hw;
 			C->x_variance += (C->datac[i] * C->datac[i]);
-			C->datac[i+1] -= (float)(y_mean + tt * y_slope);
-			C->datac[i+1] *= (float)hw;
+			C->datac[i+1] -= (gmt_grdfloat)(y_mean + tt * y_slope);
+			C->datac[i+1] *= (gmt_grdfloat)hw;
 			C->y_variance += (C->datac[i+1] * C->datac[i+1]);
 		}
 		C->x_variance /= C->window;
@@ -170,8 +170,8 @@ GMT_LOCAL void detrend_and_hanning (struct SPECTRUM1D_INFO *C, bool leave_trend,
 			hc = cos(t * h_period);
 			hw = h_scale * (1.0 - hc * hc);
 			tt = t * t_factor - 1.0;
-			C->datac[i] -= (float)(x_mean + tt * x_slope);
-			C->datac[i] *= (float)hw;
+			C->datac[i] -= (gmt_grdfloat)(x_mean + tt * x_slope);
+			C->datac[i] *= (gmt_grdfloat)hw;
 			C->x_variance += (C->datac[i] * C->datac[i]);
 		}
 		C->x_variance /= C->window;
@@ -203,14 +203,14 @@ GMT_LOCAL int compute_spectra (struct GMT_CTRL *GMT, struct SPECTRUM1D_INFO *C,
 		t_stop = t_start + C->window;
 		if (C->y_given) {
 			for (t = t_start, i = 0; t < t_stop; t++, i+=2) {
-				C->datac[i] = (float)x[t];
-				C->datac[i+1] = (float)y[t];
+				C->datac[i] = (gmt_grdfloat)x[t];
+				C->datac[i+1] = (gmt_grdfloat)y[t];
 			}
 		}
 		else {
 			for (t = t_start, i = 0; t < t_stop; t++, i+=2) {
-				C->datac[i] = (float)x[t];
-				C->datac[i+1] = 0.0f;
+				C->datac[i] = (gmt_grdfloat)x[t];
+				C->datac[i+1] = 0.0;
 			}
 		}
 
@@ -310,12 +310,12 @@ GMT_LOCAL int write_output_separate (struct GMT_CTRL *GMT, struct SPECTRUM1D_INF
 				gmt_M_free (GMT, f_or_w);
 				return (GMT_ERROR_ON_FOPEN);
 			}
-			GMT_Report (GMT->parent, GMT_MSG_VERBOSE, " Writing %s\n", fname);
+			GMT_Report (GMT->parent, GMT_MSG_LONG_VERBOSE, " Writing %s\n", fname);
 			for (i = 0; i < C->n_spec; i++) {
 				out[GMT_X] = f_or_w[i];
 				out[GMT_Y] = C->spec[i].xpow;
 				out[GMT_Z] = eps_pow * C->spec[i].xpow;
-				GMT->current.io.output (GMT, fpout, 3, out);
+				GMT->current.io.output (GMT, fpout, 3, out, NULL);
 			}
 			gmt_fclose (GMT, fpout);
 			break;
@@ -327,12 +327,12 @@ GMT_LOCAL int write_output_separate (struct GMT_CTRL *GMT, struct SPECTRUM1D_INF
 				gmt_M_free (GMT, f_or_w);
 				return (GMT_ERROR_ON_FOPEN);
 			}
-			GMT_Report (GMT->parent, GMT_MSG_VERBOSE, " Writing %s\n", fname);
+			GMT_Report (GMT->parent, GMT_MSG_LONG_VERBOSE, " Writing %s\n", fname);
 			for (i = 0; i < C->n_spec; i++) {
 				out[GMT_X] = f_or_w[i];
 				out[GMT_Y] = C->spec[i].ypow;
 				out[GMT_Z] = eps_pow * C->spec[i].ypow;
-				GMT->current.io.output (GMT, fpout, 3, out);
+				GMT->current.io.output (GMT, fpout, 3, out, NULL);
 			}
 			gmt_fclose (GMT, fpout);
 			break;
@@ -343,12 +343,12 @@ GMT_LOCAL int write_output_separate (struct GMT_CTRL *GMT, struct SPECTRUM1D_INF
 				gmt_M_free (GMT, f_or_w);
 				return (GMT_ERROR_ON_FOPEN);
 			}
-			GMT_Report (GMT->parent, GMT_MSG_VERBOSE, " Writing %s\n", fname);
+			GMT_Report (GMT->parent, GMT_MSG_LONG_VERBOSE, " Writing %s\n", fname);
 			for (i = 0; i < C->n_spec; i++) {
 				out[GMT_X] = f_or_w[i];
 				out[GMT_Y] = C->spec[i].ypow * C->spec[i].coh;
 				out[GMT_Z] = out[GMT_Y] * eps_pow * sqrt ( (2.0 - C->spec[i].coh) / C->spec[i].coh);
-				GMT->current.io.output (GMT, fpout, 3, out);
+				GMT->current.io.output (GMT, fpout, 3, out, NULL);
 			}
 			gmt_fclose (GMT, fpout);
 			break;
@@ -359,12 +359,12 @@ GMT_LOCAL int write_output_separate (struct GMT_CTRL *GMT, struct SPECTRUM1D_INF
 				gmt_M_free (GMT, f_or_w);
 				return (GMT_ERROR_ON_FOPEN);
 			}
-			GMT_Report (GMT->parent, GMT_MSG_VERBOSE, " Writing %s\n", fname);
+			GMT_Report (GMT->parent, GMT_MSG_LONG_VERBOSE, " Writing %s\n", fname);
 			for (i = 0; i < C->n_spec; i++) {
 				out[GMT_X] = f_or_w[i];
 				out[GMT_Y] = C->spec[i].ypow * (1.0 - C->spec[i].coh);
 				out[GMT_Z] = out[GMT_Y] * eps_pow;
-				GMT->current.io.output (GMT, fpout, 3, out);
+				GMT->current.io.output (GMT, fpout, 3, out, NULL);
 			}
 			gmt_fclose (GMT, fpout);
 			break;
@@ -375,12 +375,12 @@ GMT_LOCAL int write_output_separate (struct GMT_CTRL *GMT, struct SPECTRUM1D_INF
 				gmt_M_free (GMT, f_or_w);
 				return (GMT_ERROR_ON_FOPEN);
 			}
-			GMT_Report (GMT->parent, GMT_MSG_VERBOSE, " Writing %s\n", fname);
+			GMT_Report (GMT->parent, GMT_MSG_LONG_VERBOSE, " Writing %s\n", fname);
 			for (i = 0; i < C->n_spec; i++) {
 				out[GMT_X] = f_or_w[i];
 				out[GMT_Y] = C->spec[i].gain;
 				out[GMT_Z] = out[GMT_Y] * eps_pow * sqrt( (1.0 - C->spec[i].coh) / (2.0 * C->spec[i].coh) );
-				GMT->current.io.output (GMT, fpout, 3, out);
+				GMT->current.io.output (GMT, fpout, 3, out, NULL);
 			}
 			gmt_fclose (GMT, fpout);
 			break;
@@ -398,12 +398,12 @@ GMT_LOCAL int write_output_separate (struct GMT_CTRL *GMT, struct SPECTRUM1D_INF
 				GMT_Report (GMT->parent, GMT_MSG_NORMAL, " Cannot open w %s\n", fname);
 				return (GMT_ERROR_ON_FOPEN);
 			}
-			GMT_Report (GMT->parent, GMT_MSG_VERBOSE, " Writing %s\n", fname);
+			GMT_Report (GMT->parent, GMT_MSG_LONG_VERBOSE, " Writing %s\n", fname);
 			for (i = 0; i < C->n_spec; i++) {
 				out[GMT_X] = f_or_w[i];
 				out[GMT_Y] = C->spec[i].radmit;
 				out[GMT_Z] = fabs (eps_pow * sqrt( (1.0 - C->spec[i].coh) / (2.0 * C->spec[i].coh) ) * out[GMT_Y]);
-				GMT->current.io.output (GMT, fpout, 3, out);
+				GMT->current.io.output (GMT, fpout, 3, out, NULL);
 			}
 			gmt_fclose (GMT, fpout);
 			break;
@@ -414,12 +414,12 @@ GMT_LOCAL int write_output_separate (struct GMT_CTRL *GMT, struct SPECTRUM1D_INF
 				gmt_M_free (GMT, f_or_w);
 				return (GMT_ERROR_ON_FOPEN);
 			}
-			GMT_Report (GMT->parent, GMT_MSG_VERBOSE, " Writing %s\n", fname);
+			GMT_Report (GMT->parent, GMT_MSG_LONG_VERBOSE, " Writing %s\n", fname);
 			for (i = 0; i < C->n_spec; i++) {
 				out[GMT_X] = f_or_w[i];
 				out[GMT_Y] = C->spec[i].phase;
 				out[GMT_Z] = eps_pow * sqrt( (1.0 - C->spec[i].coh) / (2.0 * C->spec[i].coh) );
-				GMT->current.io.output (GMT, fpout, 3, out);
+				GMT->current.io.output (GMT, fpout, 3, out, NULL);
 			}
 			gmt_fclose (GMT, fpout);
 			break;
@@ -430,12 +430,12 @@ GMT_LOCAL int write_output_separate (struct GMT_CTRL *GMT, struct SPECTRUM1D_INF
 				gmt_M_free (GMT, f_or_w);
 				return (GMT_ERROR_ON_FOPEN);
 			}
-			GMT_Report (GMT->parent, GMT_MSG_VERBOSE, " Writing %s\n", fname);
+			GMT_Report (GMT->parent, GMT_MSG_LONG_VERBOSE, " Writing %s\n", fname);
 			for (i = 0; i < C->n_spec; i++) {
 				out[GMT_X] = f_or_w[i];
 				out[GMT_Y] = C->spec[i].coh;
 				out[GMT_Z] = out[GMT_Y] * eps_pow * (1.0 - C->spec[i].coh) * sqrt(2.0 / C->spec[i].coh);
-				GMT->current.io.output (GMT, fpout, 3, out);
+				GMT->current.io.output (GMT, fpout, 3, out, NULL);
 			}
 			gmt_fclose (GMT, fpout);
 			break;
@@ -540,10 +540,11 @@ GMT_LOCAL void Free_Ctrl (struct GMT_CTRL *GMT, struct SPECTRUM1D_CTRL *C) {	/*
 }
 
 GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
-	gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
+	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: spectrum1d [<table>] -S<segment_size> [-C[<xycnpago>]] [-D<dt>] [-L[m|h]] [-N[<name_stem>]] [-T]\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t[%s] [-W] [%s] [%s] [%s] [%s]\n\t[%s]\n\t[%s] [%s]\n\t[%s]\n\n", GMT_V_OPT, GMT_b_OPT, GMT_d_OPT, GMT_e_OPT, GMT_f_OPT, GMT_g_OPT, GMT_h_OPT, GMT_i_OPT, GMT_s_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "usage: %s [<table>] -S<segment_size> [-C[<xycnpago>]] [-D<dt>] [-L[m|h]] [-N[<name_stem>]] [-T]\n", name);
+	GMT_Message (API, GMT_TIME_NONE, "\t[%s] [-W] [%s] [%s] [%s] [%s]\n\t[%s]\n\t[%s] [%s]\n\t[%s] [%s]\n\n",
+		GMT_V_OPT, GMT_b_OPT, GMT_d_OPT, GMT_e_OPT, GMT_f_OPT, GMT_g_OPT, GMT_h_OPT, GMT_i_OPT, GMT_s_OPT, GMT_PAR_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
@@ -630,7 +631,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct SPECTRUM1D_CTRL *Ctrl, struct
 				if (opt->arg[0]) {
 					if (opt->arg[0] == '+') {	/* Obsolete syntax */
 						if (gmt_M_compat_check (GMT, 5)) {
-							GMT_Report (API, GMT_MSG_COMPAT, "Warning: Modifier -N+<file> is deprecated; use -N in the future to save to stdout.\n");
+							GMT_Report (API, GMT_MSG_COMPAT, "Modifier -N+<file> is deprecated; use -N in the future to save to stdout.\n");
 							if (n_files++ == 0) Ctrl->Out.file = strdup (&opt->arg[1]);
 						}
 						else {
@@ -702,19 +703,18 @@ int GMT_spectrum1d (void *V_API, int mode, void *args) {
 	if (mode == GMT_MODULE_PURPOSE) return (usage (API, GMT_MODULE_PURPOSE));	/* Return the purpose of program */
 	options = GMT_Create_Options (API, mode, args);	if (API->error) return (API->error);	/* Set or get option list */
 
-	if (!options || options->option == GMT_OPT_USAGE) bailout (usage (API, GMT_USAGE));/* Return the usage message */
-	if (options->option == GMT_OPT_SYNOPSIS) bailout (usage (API, GMT_SYNOPSIS));	/* Return the synopsis */
+	if ((error = gmt_report_usage (API, options, 0, usage)) != GMT_NOERROR) bailout (error);	/* Give usage if requested */
 
 	/* Parse the command-line arguments */
 
-	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
 	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
 	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
 	if ((error = parse (GMT, Ctrl, options)) != 0) Return (error);
 
 	/*---------------------------- This is the spectrum1d main code ----------------------------*/
 
-	GMT_Report (API, GMT_MSG_VERBOSE, "Processing input table data\n");
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Processing input table data\n");
 	gmt_M_memset (&C, 1, struct SPECTRUM1D_INFO);
 	
 	C.dt = Ctrl->D.inc;
@@ -728,7 +728,7 @@ int GMT_spectrum1d (void *V_API, int mode, void *args) {
 		Ctrl->C.active = true;
 	}
 
-	if ((error = gmt_set_cols (GMT, GMT_IN, 1 + C.y_given)) != GMT_NOERROR) {
+	if ((error = GMT_Set_Columns (API, GMT_IN, 1 + C.y_given, GMT_COL_FIX_NO_TEXT)) != GMT_NOERROR) {
 		Return (error);
 	}
 	if (GMT_Init_IO (API, GMT_IS_DATASET, GMT_IS_NONE, GMT_IN, GMT_ADD_DEFAULT, 0, options) != GMT_NOERROR) {	/* Establishes data input */
@@ -757,11 +757,11 @@ int GMT_spectrum1d (void *V_API, int mode, void *args) {
 	for (tbl = 0; tbl < Din->n_tables; tbl++) {
 		if (!Ctrl->T.active) {
 			gmt_free_table (GMT, Dout->table[tbl]);	/* Free it, then allocate separately */
-			Dout->table[tbl] = Tout = gmt_create_table (GMT, Din->table[tbl]->n_segments, C.n_spec, n_cols_tot, false);
+			Dout->table[tbl] = Tout = gmt_create_table (GMT, Din->table[tbl]->n_segments, C.n_spec, n_cols_tot, 0U, false);
 		}
 		for (seg = 0; seg < Din->table[tbl]->n_segments; seg++) {
 			S = Din->table[tbl]->segment[seg];	/* Current segment */
-			GMT_Report (API, GMT_MSG_VERBOSE, "Read %" PRIu64 " data points.\n", S->n_rows);
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Read %" PRIu64 " data points.\n", S->n_rows);
 
 			y = (C.y_given) ? S->data[GMT_Y] : NULL;
 			if (compute_spectra (GMT, &C, S->data[GMT_X], y, S->n_rows, Ctrl->L.active, Ctrl->L.mode)) {
@@ -769,8 +769,10 @@ int GMT_spectrum1d (void *V_API, int mode, void *args) {
 			}
 
 			if (!Ctrl->T.active) {
-				Sout = GMT_Alloc_Segment (GMT->parent, GMT_IS_DATASET, C.n_spec, Tout->n_columns, NULL, Tout->segment[seg]);
+				unsigned smode = (Tout->segment[seg]->text) ? GMT_WITH_STRINGS : GMT_NO_STRINGS;
+				Sout = Tout->segment[seg] = GMT_Alloc_Segment (GMT->parent, smode, C.n_spec, Tout->n_columns, NULL, Tout->segment[seg]);
 				assign_output_spectrum1d (GMT, &C, Ctrl->C.col, n_outputs, Ctrl->W.active, Sout->data);
+				Sout->n_rows = C.n_spec;
 			}
 			if (Ctrl->N.mode == 0) {	/* Write separate tables */
 				if (write_output_separate (GMT, &C, Ctrl->C.col, n_outputs, Ctrl->W.active, Ctrl->N.name))
@@ -781,7 +783,7 @@ int GMT_spectrum1d (void *V_API, int mode, void *args) {
 	
 	free_space_spectrum1d (GMT, &C);
 	
-	if (!Ctrl->T.active && GMT_Write_Data (API, GMT_IS_DATASET, GMT_IS_FILE, GMT_IS_NONE, Dout->io_mode, NULL, Ctrl->Out.file, Dout) != GMT_NOERROR) {
+	if (!Ctrl->T.active && GMT_Write_Data (API, GMT_IS_DATASET, GMT_IS_FILE, GMT_IS_NONE, 0, NULL, Ctrl->Out.file, Dout) != GMT_NOERROR) {
 		Return (API->error);
 	}
 
