@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *	Copyright (c) 2008-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 2008-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  * Spherical nearest distances - via Voronoi polygons.  We read input
@@ -31,19 +30,20 @@
  *
  * Author:	Paul Wessel
  * Date:	1-AUG-2011
- * Version:	5 API
+ * Version:	6 API
  *
  */
 
 #include "gmt_dev.h"
 #include "gmt_sph.h"
 
-#define THIS_MODULE_NAME	"sphdistance"
+#define THIS_MODULE_CLASSIC_NAME	"sphdistance"
+#define THIS_MODULE_MODERN_NAME	"sphdistance"
 #define THIS_MODULE_LIB		"core"
 #define THIS_MODULE_PURPOSE	"Create Voronoi distance, node, or natural nearest-neighbor grid on a sphere"
 #define THIS_MODULE_KEYS	"<D{,ND(,QD(,GG},Q-("
 #define THIS_MODULE_NEEDS	"R"
-#define THIS_MODULE_OPTIONS "-:RVbdehirs" GMT_OPT("F")
+#define THIS_MODULE_OPTIONS "-:RVbdehijrs" GMT_OPT("F")
 
 enum sphdist_modes {
 	SPHD_DIST = 0,
@@ -87,6 +87,7 @@ GMT_LOCAL void prepare_polygon (struct GMT_CTRL *GMT, struct GMT_DATASEGMENT *P)
 	 * is a polar cap; if so set the required metadata flags */
 	uint64_t row;
 	double lon_sum = 0.0, lat_sum = 0.0, dlon;
+	struct GMT_DATASEGMENT_HIDDEN *PH = gmt_get_DS_hidden (P);
 
 	gmt_set_seg_minmax (GMT, GMT_IS_POLY, 0, P);	/* Set the domain of the segment */
 
@@ -97,17 +98,17 @@ GMT_LOCAL void prepare_polygon (struct GMT_CTRL *GMT, struct GMT_DATASEGMENT *P)
 		lon_sum += dlon;
 		lat_sum += P->data[GMT_Y][row];
 	}
-	P->pole = 0;
+	PH->pole = 0;
 	if (gmt_M_360_range (lon_sum, 0.0)) {	/* Contains a pole */
 		if (lat_sum < 0.0) { /* S */
-			P->pole = -1;
-			P->lat_limit = P->min[GMT_Y];
+			PH->pole = -1;
+			PH->lat_limit = P->min[GMT_Y];
 			P->min[GMT_Y] = -90.0;
 			
 		}
 		else {	/* N */
-			P->pole = +1;
-			P->lat_limit = P->max[GMT_Y];
+			PH->pole = +1;
+			PH->lat_limit = P->max[GMT_Y];
 			P->max[GMT_Y] = 90.0;
 		}
 		P->min[GMT_X] = 0.0;	P->max[GMT_X] = 360.0;
@@ -132,12 +133,12 @@ GMT_LOCAL void Free_Ctrl (struct GMT_CTRL *GMT, struct SPHDISTANCE_CTRL *C) {	/*
 }
 
 GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
-	gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
+	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
 	GMT_Message (API, GMT_TIME_NONE, "==> The hard work is done by algorithms 772 (STRIPACK) & 773 (SSRFPACK) by R. J. Renka [1997] <==\n\n");
-	GMT_Message (API, GMT_TIME_NONE, "usage: sphdistance [<table>] -G<outgrid> %s [-C] [-En|z|d[<dr>]]\n", GMT_I_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "usage: %s [<table>] -G<outgrid> %s [-C] [-En|z|d[<dr>]]\n", name, GMT_I_OPT);
 	GMT_Message (API, GMT_TIME_NONE, "\t[-L<unit>] [-N<nodetable>] [-Q<voronoitable>] [%s] [%s] [%s] [%s]\n", GMT_Rgeo_OPT, GMT_V_OPT, GMT_bi_OPT, GMT_di_OPT);
-	GMT_Message (API, GMT_TIME_NONE, "\t[%s] [%s] [%s]\n\t[%s] [%s] [%s]\n\n", GMT_e_OPT, GMT_h_OPT, GMT_i_OPT, GMT_r_OPT, GMT_s_OPT, GMT_colon_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "\t[%s] [%s] [%s]\n\t[%s] [%s] [%s] [%s] [%s]\n\n", GMT_e_OPT, GMT_h_OPT, GMT_i_OPT, GMT_j_OPT, GMT_r_OPT, GMT_s_OPT, GMT_colon_OPT, GMT_PAR_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
@@ -157,13 +158,12 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t   -Ed The distance to the nearest data point [Default].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   Optionally append resampling interval in spherical degrees for polygon arcs [1].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-L Set distance unit arc (d)egree, m(e)ter, (f)eet, (k)m, arc (m)inute, (M)ile, (n)autical mile,\n\t   or arc (s)econd [e].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   PROJ_ELLIPSOID determines if geodesic or great-circle distances are used.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-N Specify node filename for the Voronoi polygons (sphtriangulate -N output).\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-Q Specify table with Voronoi polygons in sphtriangulate -Qv format\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   [Default performs Voronoi construction on input data first].\n");
 	GMT_Option (API, "Rg");
-	GMT_Message (API, GMT_TIME_NONE, "\t   If no region is specified we default to the entire world [-Rg].\n");
-	GMT_Option (API, "V,bi2,di,e,h,i,r,s,:,.");
+	if (gmt_M_showusage (API)) GMT_Message (API, GMT_TIME_NONE, "\t   If no region is specified we default to the entire world [-Rg].\n");
+	GMT_Option (API, "V,bi2,di,e,h,i,j,r,s,:,.");
 
 	return (GMT_MODULE_USAGE);
 }
@@ -193,7 +193,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct SPHDISTANCE_CTRL *Ctrl, struct
 				break;
 			case 'D':
 				if (gmt_M_compat_check (GMT, 4))
-					GMT_Report (API, GMT_MSG_COMPAT, "Warning: -D option is deprecated; duplicates are automatically removed.\n");
+					GMT_Report (API, GMT_MSG_COMPAT, "-D option is deprecated; duplicates are automatically removed.\n");
 				else
 					n_errors += gmt_default_error (GMT, opt->option);
 				break;
@@ -265,9 +265,10 @@ int GMT_sphdistance (void *V_API, int mode, void *args) {
 	double *grid_lon = NULL, *grid_lat = NULL, *in = NULL;
 	double *xx = NULL, *yy = NULL, *zz = NULL, *lon = NULL, *lat = NULL;
 	
-	float f_val = 0.0, *z_val = NULL;
+	gmt_grdfloat f_val = 0.0, *z_val = NULL;
 
 	struct GMT_GRID *Grid = NULL;
+	struct GMT_RECORD *In = NULL;
 	struct SPHDISTANCE_CTRL *Ctrl = NULL;
 	struct STRIPACK T;
 	struct GMT_DATASEGMENT *P = NULL;
@@ -284,12 +285,11 @@ int GMT_sphdistance (void *V_API, int mode, void *args) {
 	if (mode == GMT_MODULE_PURPOSE) return (usage (API, GMT_MODULE_PURPOSE));	/* Return the purpose of program */
 	options = GMT_Create_Options (API, mode, args);	if (API->error) return (API->error);	/* Set or get option list */
 
-	if (!options || options->option == GMT_OPT_USAGE) bailout (usage (API, GMT_USAGE));/* Return the usage message */
-	if (options->option == GMT_OPT_SYNOPSIS) bailout (usage (API, GMT_SYNOPSIS));	/* Return the synopsis */
+	if ((error = gmt_report_usage (API, options, 0, usage)) != GMT_NOERROR) bailout (error);	/* Give usage if requested */
 
 	/* Parse the command-line arguments */
 
-	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
 	gmt_parse_common_options (GMT, "f", 'f', "g"); /* Implicitly set -fg since this is spherical triangulation */
 	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
 	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
@@ -308,7 +308,7 @@ int GMT_sphdistance (void *V_API, int mode, void *args) {
 	/* Now we are ready to take on some input values */
 
 	if (Ctrl->Q.active) {	/* Expect a single file with Voronoi polygons */
-		GMT_Report (API, GMT_MSG_VERBOSE, "Read Volonoi polygons from %s ...", Ctrl->Q.file);
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Read Volonoi polygons from %s ...", Ctrl->Q.file);
 		gmt_disable_bhi_opts (GMT);	/* Do not want any -b -h -i to affect the reading from -Q files */
 		if ((Qin = GMT_Read_Data (API, GMT_IS_DATASET, GMT_IS_FILE, GMT_IS_POLY, GMT_READ_NORMAL, NULL, Ctrl->Q.file, NULL)) == NULL) {
 			Return (API->error);
@@ -319,16 +319,16 @@ int GMT_sphdistance (void *V_API, int mode, void *args) {
 		}
 		gmt_reenable_bhi_opts (GMT);	/* Recover settings provided by user (if -b -h -i were used at all) */
 		Table = Qin->table[0];	/* Only one table in a file */
-		GMT_Report (API, GMT_MSG_VERBOSE, "Found %" PRIu64 " segments\n", Table->n_segments);
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Found %" PRIu64 " segments\n", Table->n_segments);
 	 	lon = gmt_M_memory (GMT, NULL, Table->n_segments, double);
 	 	lat = gmt_M_memory (GMT, NULL, Table->n_segments, double);
 		if (Ctrl->N.active) {	/* Must get nodes from separate file */
 			struct GMT_DATASET *Nin = NULL;
 			struct GMT_DATATABLE *NTable = NULL;
-			if ((error = gmt_set_cols (GMT, GMT_IN, 3)) != GMT_NOERROR) {
+			if ((error = GMT_Set_Columns (API, GMT_IN, 3, GMT_COL_FIX_NO_TEXT)) != GMT_NOERROR) {
 				Return (error);
 			}
-			GMT_Report (API, GMT_MSG_VERBOSE, "Read Nodes from %s ...", Ctrl->N.file);
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Read Nodes from %s ...", Ctrl->N.file);
 			gmt_disable_bhi_opts (GMT);	/* Do not want any -b -h -i to affect the reading from -N files */
 			if ((Nin = GMT_Read_Data (API, GMT_IS_DATASET, GMT_IS_FILE, GMT_IS_POINT, GMT_READ_NORMAL, NULL, Ctrl->N.file, NULL)) == NULL) {
 				Return (API->error);
@@ -352,7 +352,7 @@ int GMT_sphdistance (void *V_API, int mode, void *args) {
 			if (GMT_Destroy_Data (API, &Nin) != GMT_NOERROR) {
 				Return (API->error);
 			}
-			GMT_Report (API, GMT_MSG_VERBOSE, "Found %" PRIu64 " records\n", NTable->n_records);
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Found %" PRIu64 " records\n", NTable->n_records);
 		}
 		else {	/* Get extract them from the segment header */
 			for (node = 0; node < Table->n_segments; node++) {
@@ -369,7 +369,7 @@ int GMT_sphdistance (void *V_API, int mode, void *args) {
 	}
 	else {	/* Must process input point/line data */
 		n_in = (Ctrl->E.mode == SPHD_VALUES) ? 3 : 2;
-		if ((error = gmt_set_cols (GMT, GMT_IN, n_in)) != GMT_NOERROR) {
+		if ((error = GMT_Set_Columns (API, GMT_IN, n_in, GMT_COL_FIX_NO_TEXT)) != GMT_NOERROR) {
 			Return (error);
 		}
 		if (GMT_Init_IO (API, GMT_IS_DATASET, GMT_IS_POINT, GMT_IN, GMT_ADD_DEFAULT, 0, options) != GMT_NOERROR) {	/* Registers default input sources, unless already set */
@@ -385,12 +385,15 @@ int GMT_sphdistance (void *V_API, int mode, void *args) {
 		if (!Ctrl->C.active) gmt_M_malloc2 (GMT, lon, lat, 0, &n_alloc, double);
 		n_alloc = 0;
 		gmt_M_malloc3 (GMT, xx, yy, zz, 0, &n_alloc, double);
-		if (Ctrl->E.mode == SPHD_VALUES) z_val = gmt_M_memory (GMT, NULL, n_alloc, float);
+		if (Ctrl->E.mode == SPHD_VALUES) z_val = gmt_M_memory (GMT, NULL, n_alloc, gmt_grdfloat);
 
 		n = 0;
 		do {	/* Keep returning records until we reach EOF */
-			if ((in = GMT_Get_Record (API, GMT_READ_DATA, NULL)) == NULL) {	/* Read next record, get NULL if special case */
+			if ((In = GMT_Get_Record (API, GMT_READ_DATA, NULL)) == NULL) {	/* Read next record, get NULL if special case */
 				if (gmt_M_rec_is_error (GMT)) {		/* Bail if there are any read errors */
+					if (Ctrl->E.mode == SPHD_VALUES) gmt_M_free (GMT, z_val);
+					if (!Ctrl->C.active) {gmt_M_free (GMT, lon);	gmt_M_free (GMT, lat);}
+					if (!Ctrl->Q.active) {gmt_M_free (GMT, xx);	gmt_M_free (GMT, yy);	gmt_M_free (GMT, zz);}
 					Return (GMT_RUNTIME_ERROR);
 				}
 				else if (gmt_M_rec_is_eof (GMT)) 		/* Reached end of file */
@@ -401,6 +404,7 @@ int GMT_sphdistance (void *V_API, int mode, void *args) {
 			}
 
 			/* Data record to process - avoid duplicate points as gmt_stripack_lists cannot handle that */
+			in = In->data;	/* Only need to process numerical part here */
 
 			if (first) {	/* Beginning of new segment; keep track of the very first coordinate in case of duplicates */
 				first_x = prev_x = in[GMT_X];	first_y = prev_y = in[GMT_Y];
@@ -424,12 +428,12 @@ int GMT_sphdistance (void *V_API, int mode, void *args) {
 			if (!Ctrl->C.active) {
 				lon[n] = in[GMT_X];	lat[n] = in[GMT_Y];
 			}
-			if (Ctrl->E.mode == SPHD_VALUES) z_val[n] = (float)in[GMT_Z];
+			if (Ctrl->E.mode == SPHD_VALUES) z_val[n] = (gmt_grdfloat)in[GMT_Z];
 
 			if (++n == n_alloc) {	/* Get more memory */
 				if (!Ctrl->C.active) { size_t n_tmp = n_alloc; gmt_M_malloc2 (GMT, lon, lat, n, &n_tmp, double); }
 				gmt_M_malloc3 (GMT, xx, yy, zz, n, &n_alloc, double);
-				if (Ctrl->E.mode == SPHD_VALUES) z_val = gmt_M_memory (GMT, z_val, n_alloc, float);
+				if (Ctrl->E.mode == SPHD_VALUES) z_val = gmt_M_memory (GMT, z_val, n_alloc, gmt_grdfloat);
 			}
 			first = false;
 		} while (true);
@@ -439,7 +443,7 @@ int GMT_sphdistance (void *V_API, int mode, void *args) {
 		gmt_M_malloc3 (GMT, xx, yy, zz, 0, &n_alloc, double);
 
 		if (n_dup) GMT_Report (API, GMT_MSG_VERBOSE, "Skipped %" PRIu64 " duplicate points in segments\n", n_dup);
-		GMT_Report (API, GMT_MSG_VERBOSE, "Do Voronoi construction using %" PRIu64 " points\n", n);
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Do Voronoi construction using %" PRIu64 " points\n", n);
 
 		T.mode = VORONOI;
 		gmt_stripack_lists (GMT, n, xx, yy, zz, &T);	/* Do the basic triangulation */
@@ -460,14 +464,15 @@ int GMT_sphdistance (void *V_API, int mode, void *args) {
 
 	if ((Grid = GMT_Create_Data (API, GMT_IS_GRID, GMT_IS_SURFACE, GMT_CONTAINER_AND_DATA, NULL, NULL, NULL, \
 		GMT_GRID_DEFAULT_REG, GMT_NOTSET, NULL)) == NULL) Return (API->error);
-	GMT_Report (API, GMT_MSG_VERBOSE, "Start processing distance grid\n");
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Start processing distance grid\n");
 
-	grid_lon = gmt_grd_coord (GMT, Grid->header, GMT_X);
-	grid_lat = gmt_grd_coord (GMT, Grid->header, GMT_Y);
+	grid_lon = Grid->x;
+	grid_lat = Grid->y;
 
 	nx1 = (Grid->header->registration == GMT_GRID_PIXEL_REG) ? Grid->header->n_columns : Grid->header->n_columns - 1;
 	periodic = gmt_M_360_range (GMT->common.R.wesn[XLO], GMT->common.R.wesn[XHI]);
 	duplicate_col = (periodic && Grid->header->registration == GMT_GRID_NODE_REG);	/* E.g., lon = 0 column should match lon = 360 column */
+	gmt_set_inside_mode (GMT, NULL, GMT_IOO_SPHERICAL);
 
 	if (Ctrl->Q.active)	/* Pre-chewed, just get number of nodes */
 		n = Table->n_segments;
@@ -475,13 +480,13 @@ int GMT_sphdistance (void *V_API, int mode, void *args) {
 		V = &T.V;
 
 	for (node = 0; node < n; node++) {
-		GMT_Report (API, GMT_MSG_VERBOSE, "Processing polygon %7ld\r", node);
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Processing polygon %7ld\r", node);
 		if (Ctrl->Q.active) {	/* Just point to next polygon */
 			P = Table->segment[node];
 		}
 		else {	/* Obtain current polygon from Voronoi listings */
 			if (P == NULL) {	/* Need a single polygon structure that we reuse for each polygon */
-				P = gmt_M_memory (GMT, NULL, 1, struct GMT_DATASEGMENT);	/* Needed as pointer below */
+				P = gmt_get_segment (GMT);	/* Needed as pointer below */
 				P->data = gmt_M_memory (GMT, NULL, 2, double *);	/* Needed as pointers below */
 				P->min = gmt_M_memory (GMT, NULL, 2, double);	/* Needed to hold min lon/lat */
 				P->max = gmt_M_memory (GMT, NULL, 2, double);	/* Needed to hold max lon/lat */
@@ -515,7 +520,7 @@ int GMT_sphdistance (void *V_API, int mode, void *args) {
 			if ((++vertex) == p_alloc) gmt_M_malloc2 (GMT, P->data[GMT_X], P->data[GMT_Y], vertex, &p_alloc, double);
 			P->n_rows = vertex;
 			switch (Ctrl->E.mode) {
-				case SPHD_NODES:	f_val = (float)node;	break;
+				case SPHD_NODES:	f_val = (gmt_grdfloat)node;	break;
 				case SPHD_VALUES:	f_val = z_val[node];	break;
 				default:	break;	/* Must compute distances below */
 			}
@@ -526,7 +531,6 @@ int GMT_sphdistance (void *V_API, int mode, void *args) {
 
 		if ((n_new = gmt_fix_up_path (GMT, &P->data[GMT_X], &P->data[GMT_Y], P->n_rows, Ctrl->E.dist, GMT_STAIRS_OFF)) == 0) {
 			gmt_M_free (GMT, P);
-			gmt_M_free (GMT, grid_lon);	gmt_M_free (GMT, grid_lat);
 			Return (GMT_RUNTIME_ERROR);
 		}
 		P->n_rows = n_new;
@@ -558,7 +562,7 @@ int GMT_sphdistance (void *V_API, int mode, void *args) {
 				if (side == 0) continue;	/* Outside spherical polygon */
 				ij = gmt_M_ijp (Grid->header, row, col);
 				if (Ctrl->E.mode == SPHD_DIST)
-					f_val = (float)gmt_distance (GMT, grid_lon[col], grid_lat[row], lon[node], lat[node]);
+					f_val = (gmt_grdfloat)gmt_distance (GMT, grid_lon[col], grid_lat[row], lon[node], lat[node]);
 				Grid->data[ij] = f_val;
 				n_set++;
 				if (duplicate_col) {	/* Duplicate the repeating column on the other side of this one */
@@ -568,15 +572,10 @@ int GMT_sphdistance (void *V_API, int mode, void *args) {
 			}
 		}
 	}
-	GMT_Report (API, GMT_MSG_VERBOSE, "Processing polygon %7ld\n", node);
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Processing polygon %7ld\n", node);
 
 	if (!Ctrl->Q.active) {
-		gmt_M_free (GMT, P->data[GMT_X]);
-		gmt_M_free (GMT, P->data[GMT_Y]);
-		gmt_M_free (GMT, P->min);
-		gmt_M_free (GMT, P->max);
-		gmt_M_free (GMT, P->data);
-		gmt_M_free (GMT, P);
+		gmt_free_segment (GMT, &P);
 		gmt_M_free (GMT, T.V.lon);
 		gmt_M_free (GMT, T.V.lat);
 		gmt_M_free (GMT, T.V.lend);
@@ -585,7 +584,6 @@ int GMT_sphdistance (void *V_API, int mode, void *args) {
 		gmt_M_free (GMT, xx);
 		gmt_M_free (GMT, yy);
 	}
-	gmt_M_free (GMT, grid_lon);	gmt_M_free (GMT, grid_lat);
 	if (!Ctrl->C.active) {
 		gmt_M_free (GMT, lon);
 		gmt_M_free (GMT, lat);
@@ -598,7 +596,7 @@ int GMT_sphdistance (void *V_API, int mode, void *args) {
 	}
 
 	if (n_set > Grid->header->nm) n_set = Grid->header->nm;	/* Not confuse the public */
-	GMT_Report (API, GMT_MSG_VERBOSE, "Spherical distance calculation completed, %" PRIu64 " nodes visited (at least once)\n", n_set);
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Spherical distance calculation completed, %" PRIu64 " nodes visited (at least once)\n", n_set);
 
 	Return (GMT_NOERROR);
 }
