@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *   Copyright (c) 1999-2018 by P. Wessel
+ *   Copyright (c) 1999-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Lesser General Public License as published by
@@ -12,7 +11,7 @@
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *   GNU Lesser General Public License for more details.
  *
- *   Contact info: www.soest.hawaii.edu/pwessel
+ *   Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  * grdrotater will read a grid file, apply a finite rotation to the grid
@@ -26,7 +25,8 @@
 #include "gmt_dev.h"
 #include "spotter.h"
 
-#define THIS_MODULE_NAME	"grdrotater"
+#define THIS_MODULE_CLASSIC_NAME	"grdrotater"
+#define THIS_MODULE_MODERN_NAME	"grdrotater"
 #define THIS_MODULE_LIB		"spotter"
 #define THIS_MODULE_PURPOSE	"Finite rotation reconstruction of geographic grid"
 #define THIS_MODULE_KEYS	"<G{,FD(,GG},TD("
@@ -41,6 +41,10 @@ struct GRDROTATER_CTRL {	/* All control options for this program (except common
 		bool active;
 		char *file;
 	} In;
+	struct A {	/* -Aw/e/s/n */
+		bool active;
+		double wesn[4];
+	} A;
 	struct D {	/* -Drotpolfile or -Dtemplate */
 		bool active;
 		char *file;
@@ -90,11 +94,11 @@ GMT_LOCAL void Free_Ctrl (struct GMT_CTRL *GMT, struct GRDROTATER_CTRL *C) {	/*
 }
 
 GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
-	gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
+	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: grdrotater <grid> %s -G<outgrid> [-F<polygontable>]\n", SPOTTER_E_OPT);
-	GMT_Message (API, GMT_TIME_NONE, "\t[-D<rotoutline>] [-N] [%s] [-S] [-T<time(s)>] [%s]\n", GMT_Rgeo_OPT, GMT_V_OPT);
-	GMT_Message (API, GMT_TIME_NONE, "\t[%s] [%s] [%s]\n\t[%s]\n\t[%s] [%s]\n\n", GMT_b_OPT, GMT_d_OPT, GMT_g_OPT, GMT_h_OPT, GMT_n_OPT, GMT_o_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "usage: %s <grid> %s -G<outgrid> [-F<polygontable>]\n", name, SPOTTER_E_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "\t[-A<region>] [-D<rotoutline>] [-N] [%s] [-S] [-T<time(s)>] [%s]\n", GMT_Rgeo_OPT, GMT_V_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "\t[%s] [%s] [%s]\n\t[%s]\n\t[%s] [%s] [%s]\n\n", GMT_b_OPT, GMT_d_OPT, GMT_g_OPT, GMT_h_OPT, GMT_n_OPT, GMT_o_OPT, GMT_PAR_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
@@ -107,6 +111,8 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	spotter_rot_usage (API, 'E');
 	GMT_Message (API, GMT_TIME_NONE, "\t   Alternatively, specify a single finite rotation (in degrees) to be applied.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\n\tOPTIONS:\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t-A Set the west/east/south/north bounds for the rotated grid [Default will\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   determine the natural extent of the rotated grid instead].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-D Write the rotated polygon or grid outline to <rotoutline> [stdout].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   Required if more than one reconstruction time is chosen and -N is not set\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   and must then contain a C-format specifier for formatting a double (for the variable time).\n");
@@ -116,7 +122,7 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Option (API, "Rg");
 	GMT_Message (API, GMT_TIME_NONE, "\t-S Do NOT rotate the grid - just produce the rotated outlines (requires -D).\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-T Set the time(s) of reconstruction.  Append a single time (-T<time>),\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   an equidistant range of times (-T<min>/<max>/<inc> or -T<min>/<max>/<npoints>+),\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   an equidistant range of times (-T<min>/<max>/<inc> or -T<min>/<max>/<npoints>+n),\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   or the name of a file with a list of times (-T<tfile>).  If no -T is set\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   then the reconstruction times equal the rotation times given in -E.\n");
 	GMT_Option (API, "V,bi2,bo,d,g,h,n,:,.");
@@ -133,9 +139,11 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRDROTATER_CTRL *Ctrl, struct
 	 */
 
 	unsigned int n_errors = 0, k, n_files = 0;
+	int n_items;
 	uint64_t t = 0;
 	bool gave_e = false;
 	char txt_a[GMT_LEN256] = {""}, txt_b[GMT_LEN256] = {""}, txt_c[GMT_LEN256] = {""};
+	char txt[4][GMT_LEN64];
 	struct GMT_OPTION *opt = NULL;
 	struct GMTAPI_CTRL *API = GMT->parent;
 
@@ -154,9 +162,42 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRDROTATER_CTRL *Ctrl, struct
 
 			/* Supplemental parameters */
 
+			case 'A':
+				Ctrl->A.active = true;
+				if (opt->arg[0] == 'g' && opt->arg[1] == '\0') {	/* Got -Ag */
+					Ctrl->A.wesn[0] = 0.0;	Ctrl->A.wesn[1] = 360.0;	Ctrl->A.wesn[2] = -90.0;	Ctrl->A.wesn[3] = 90.0;
+					break;
+				}
+				if (opt->arg[0] == 'd' && opt->arg[1] == '\0') {	/* Got -Rd */
+					Ctrl->A.wesn[0] = -180.0;	Ctrl->A.wesn[1] = 180.0;	Ctrl->A.wesn[2] = -90.0;	Ctrl->A.wesn[3] = 90.0;
+					break;
+				}
+				if (!gmt_access (GMT, opt->arg, R_OK)) {	/* Gave a readable file, presumably a grid */
+					struct GMT_GRID *G = NULL;
+					if ((G = GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_CONTAINER_ONLY, NULL, opt->arg, NULL)) == NULL) {	/* Get header only */
+						return (API->error);
+					}
+					Ctrl->A.wesn[0] = G->header->wesn[XLO]; Ctrl->A.wesn[1] = G->header->wesn[XHI];
+					Ctrl->A.wesn[2] = G->header->wesn[YLO]; Ctrl->A.wesn[3] = G->header->wesn[YHI];
+					if (GMT_Destroy_Data (API, &G) != GMT_NOERROR) {
+						return (API->error);
+					}
+					break;
+				}
+				/* Only get here if the above cases did not trip */
+				n_items = sscanf (opt->arg, "%[^/]/%[^/]/%[^/]/%s", txt[0], txt[1], txt[2], txt[3]);
+				if (n_items != 4) {
+					GMT_Report (API, GMT_MSG_NORMAL, "Syntax error -A option: Give g, d, <grdfile>, or <west/east/south/north>\n");
+					n_errors++;
+				}
+				n_errors += gmt_verify_expectations (GMT, gmt_M_type (GMT, GMT_IN, GMT_X), gmt_scanf_arg (GMT, txt[0], gmt_M_type (GMT, GMT_IN, GMT_X), true, &Ctrl->A.wesn[0]), txt[0]);
+				n_errors += gmt_verify_expectations (GMT, gmt_M_type (GMT, GMT_IN, GMT_X), gmt_scanf_arg (GMT, txt[1], gmt_M_type (GMT, GMT_IN, GMT_X), true, &Ctrl->A.wesn[1]), txt[1]);
+				n_errors += gmt_verify_expectations (GMT, gmt_M_type (GMT, GMT_IN, GMT_Y), gmt_scanf_arg (GMT, txt[2], gmt_M_type (GMT, GMT_IN, GMT_Y), true, &Ctrl->A.wesn[2]), txt[2]);
+				n_errors += gmt_verify_expectations (GMT, gmt_M_type (GMT, GMT_IN, GMT_Y), gmt_scanf_arg (GMT, txt[3], gmt_M_type (GMT, GMT_IN, GMT_Y), true, &Ctrl->A.wesn[3]), txt[3]);
+				break;
 			case 'C':	/* Now done automatically in spotter_init */
 				if (gmt_M_compat_check (GMT, 4))
-					GMT_Report (API, GMT_MSG_COMPAT, "Warning: -C is no longer needed as total reconstruction vs stage rotation is detected automatically.\n");
+					GMT_Report (API, GMT_MSG_COMPAT, "-C is no longer needed as total reconstruction vs stage rotation is detected automatically.\n");
 				else
 					n_errors += gmt_default_error (GMT, opt->option);
 				break;
@@ -165,7 +206,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRDROTATER_CTRL *Ctrl, struct
 				Ctrl->D.file = strdup (opt->arg);
 				break;
 			case 'e':
-				GMT_Report (API, GMT_MSG_COMPAT, "Warning: -e is deprecated and will be removed in 5.2.x. Use -E instead.\n");
+				GMT_Report (API, GMT_MSG_COMPAT, "-e is deprecated and was removed in 5.3. Use -E instead.\n");
 				/* Fall-through on purpose */
 			case 'E':	/* File with stage poles or a single rotation pole */
 				Ctrl->E.active = true;
@@ -214,20 +255,20 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRDROTATER_CTRL *Ctrl, struct
 				}
 				/* Not a file */
 				k = sscanf (opt->arg, "%[^/]/%[^/]/%s", txt_a, txt_b, txt_c);
-				if (k == 3) {	/* Gave -Ttstart/tstop/tinc or possibly ancient -Tlon/lat/angle */
+				if (k == 3) {	/* Gave -Ttstart/tstop/tinc[+n] or possibly ancient -Tlon/lat/angle */
 					if (gmt_M_compat_check (GMT, 4) && !gave_e) {	/* No -E|e so likely ancient syntax */
-						GMT_Report (API, GMT_MSG_COMPAT, "Warning: -T<lon>/<lat>/<angle> is deprecated; use -E<lon>/<lat>/<angle> instead.\n");
+						GMT_Report (API, GMT_MSG_COMPAT, "-T<lon>/<lat>/<angle> is deprecated; use -E<lon>/<lat>/<angle> instead.\n");
 						Ctrl->E.rot.single = Ctrl->E.active = true;
 						Ctrl->T.active = false;
 						Ctrl->E.rot.w = atof (txt_c);
 						Ctrl->E.rot.age = GMT->session.d_NaN;
-						n_errors += gmt_verify_expectations (GMT, GMT->current.io.col_type[GMT_IN][GMT_X], gmt_scanf_arg (GMT, txt_a, GMT->current.io.col_type[GMT_IN][GMT_X], &Ctrl->E.rot.lon), txt_a);
-						n_errors += gmt_verify_expectations (GMT, GMT->current.io.col_type[GMT_IN][GMT_Y], gmt_scanf_arg (GMT, txt_b, GMT->current.io.col_type[GMT_IN][GMT_Y], &Ctrl->E.rot.lat), txt_b);
+						n_errors += gmt_verify_expectations (GMT, gmt_M_type (GMT, GMT_IN, GMT_X), gmt_scanf_arg (GMT, txt_a, gmt_M_type (GMT, GMT_IN, GMT_X), false, &Ctrl->E.rot.lon), txt_a);
+						n_errors += gmt_verify_expectations (GMT, gmt_M_type (GMT, GMT_IN, GMT_Y), gmt_scanf_arg (GMT, txt_b, gmt_M_type (GMT, GMT_IN, GMT_Y), false, &Ctrl->E.rot.lat), txt_b);
 					}
 					else {	/* Must be Ttstart/tstop/tinc */
 						double min, max, inc;
 						min = atof (txt_a);	max = atof (txt_b);	inc = atof (txt_c);
-						if (opt->arg[strlen(opt->arg)-1] == '+')	/* Gave number of points instead; calculate inc */
+						if (strstr(opt->arg, "+n") || opt->arg[strlen(opt->arg)-1] == '+')	/* Gave number of points instead; calculate inc */
 							inc = (max - min) / (inc - 1.0);
 						if (inc <= 0.0) {
 							GMT_Report (API, GMT_MSG_NORMAL, "Syntax error -T option: Age increment must be positive\n");
@@ -277,6 +318,7 @@ GMT_LOCAL struct GMT_DATASET *get_grid_path (struct GMT_CTRL *GMT, struct GMT_GR
 	uint64_t dim[GMT_DIM_SIZE] = {1, 1, 0, 2};
 	struct GMT_DATASET *D = NULL;
 	struct GMT_DATASEGMENT *S = NULL;
+	struct GMT_DATASEGMENT_HIDDEN *SH = NULL;
 
 	if ((D = GMT_Create_Data (GMT->parent, GMT_IS_DATASET, GMT_IS_POLY, 0, dim, NULL, NULL, 0, 0, NULL)) == NULL)
 		return (NULL);	/* An empty table with one segment, two cols */
@@ -344,8 +386,9 @@ GMT_LOCAL struct GMT_DATASET *get_grid_path (struct GMT_CTRL *GMT, struct GMT_GR
 	S->n_columns = 2;
 	S->min[GMT_X] = h->wesn[XLO];	S->max[GMT_X] = h->wesn[XHI];
 	S->min[GMT_Y] = h->wesn[YLO];	S->max[GMT_Y] = h->wesn[YHI];
-	S->pole = 0;
-	S->alloc_mode = GMT_ALLOC_INTERNALLY;
+	SH = gmt_get_DS_hidden (S);
+	SH->pole = 0;
+	SH->alloc_mode = GMT_ALLOC_INTERNALLY;
 
 	return (D);
 }
@@ -355,7 +398,7 @@ GMT_LOCAL bool skip_if_outside (struct GMT_CTRL *GMT, struct GMT_DATATABLE *P, d
 	uint64_t seg;
 	unsigned int inside = 0;
 	for (seg = 0; seg < P->n_segments && !inside; seg++) {	/* Use degrees since function expects it */
-		if (gmt_M_polygon_is_hole (P->segment[seg])) continue;	/* Holes are handled within gmt_inonout */
+		if (gmt_polygon_is_hole (GMT, P->segment[seg])) continue;	/* Holes are handled within gmt_inonout */
 		inside = (gmt_inonout (GMT, lon, lat, P->segment[seg]) > 0);
 	}
 	return ((inside) ? false : true);	/* true if outside */
@@ -369,7 +412,7 @@ int GMT_grdrotater (void *V_API, int mode, void *args) {
 	int n_stages;
 	bool not_global, global = false;
 	unsigned int col, row, col_o, row_o, start_row, stop_row, start_col, stop_col;
-	char gfile[GMT_BUFSIZ] = {""};
+	char gfile[PATH_MAX] = {""};
 
 	uint64_t ij, ij_rot, seg, rec, t;
 
@@ -382,6 +425,7 @@ int GMT_grdrotater (void *V_API, int mode, void *args) {
 	struct GMT_DATASEGMENT *S = NULL, *Sr = NULL;
 	struct GMT_OPTION *ptr = NULL;
 	struct GMT_GRID *G = NULL, *G_rot = NULL;
+	struct GMT_GRID_HEADER_HIDDEN *HH = NULL;
 	struct GRDROTATER_CTRL *Ctrl = NULL;
 	struct GMT_CTRL *GMT = NULL, *GMT_cpy = NULL;
 	struct GMT_OPTION *options = NULL;
@@ -393,12 +437,11 @@ int GMT_grdrotater (void *V_API, int mode, void *args) {
 	if (mode == GMT_MODULE_PURPOSE) return (usage (API, GMT_MODULE_PURPOSE));	/* Return the purpose of program */
 	options = GMT_Create_Options (API, mode, args);	if (API->error) return (API->error);	/* Set or get option list */
 
-	if (!options || options->option == GMT_OPT_USAGE) bailout (usage (API, GMT_USAGE));	/* Return the usage message */
-	if (options->option == GMT_OPT_SYNOPSIS) bailout (usage (API, GMT_SYNOPSIS));	/* Return the synopsis */
+	if ((error = gmt_report_usage (API, options, 0, usage)) != GMT_NOERROR) bailout (error);	/* Give usage if requested */
 
 	/* Parse the command-line arguments */
 
-	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
 	if ((ptr = GMT_Find_Option (API, 'f', options)) == NULL) gmt_parse_common_options (GMT, "f", 'f', "g"); /* Did not set -f, implicitly set -fg */
 	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
 	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
@@ -414,7 +457,7 @@ int GMT_grdrotater (void *V_API, int mode, void *args) {
 		if ((G = GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_CONTAINER_ONLY, NULL, Ctrl->In.file, NULL)) == NULL) {	/* Get header only */
 			Return (API->error);
 		}
-
+		HH = gmt_get_H_hidden (G->header);
 		if (!GMT->common.R.active[RSET]) gmt_M_memcpy (GMT->common.R.wesn, G->header->wesn, 4, double);	/* -R was not set so we use the grid domain */
 
 		/* Determine the wesn to be used to read the Ctrl->In.file; or exit if file is outside -R */
@@ -425,12 +468,12 @@ int GMT_grdrotater (void *V_API, int mode, void *args) {
 		}
 		global = (doubleAlmostEqual (GMT->common.R.wesn[XHI] - GMT->common.R.wesn[XLO], 360.0)
 							&& doubleAlmostEqual (GMT->common.R.wesn[YHI] - GMT->common.R.wesn[YLO], 180.0));
-		if (!GMT->common.R.active[RSET]) global = gmt_M_grd_is_global (GMT, G->header);
+		if (!GMT->common.R.active[RSET]) global = gmt_grd_is_global (GMT, G->header);
 	}
 	not_global = !global;
 
 	if (!Ctrl->S.active) {	/* Read the input grid */
-		GMT_Report (API, GMT_MSG_VERBOSE, "Allocates memory and read grid file\n");
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Allocates memory and read grid file\n");
 		if (GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_DATA_ONLY, GMT->common.R.wesn, Ctrl->In.file, G) == NULL) {
 			Return (API->error);
 		}
@@ -454,6 +497,7 @@ int GMT_grdrotater (void *V_API, int mode, void *args) {
 		Dr = GMT_Duplicate_Data (API, GMT_IS_DATASET, GMT_DUPLICATE_ALLOC, D);	/* Same table length as D */
 		polr = Dr->table[0];	/* Since we know it is also a single file */
 	}
+	gmt_set_inside_mode (GMT, NULL, GMT_IOO_SPHERICAL);
 
 	if (Ctrl->E.rot.single) {	/* Got a single rotation, create a rotation table with one entry */
 		Ctrl->T.n_times = n_stages = 1;
@@ -467,7 +511,7 @@ int GMT_grdrotater (void *V_API, int mode, void *args) {
 	}
 	else {	/* Got a rotation file with multiple rotations in total reconstruction format */
 		double t_max = 0.0;
-		n_stages = spotter_init (GMT, Ctrl->E.rot.file, &p, false, true, Ctrl->E.rot.invert, &t_max);
+		n_stages = spotter_init (GMT, Ctrl->E.rot.file, &p, 0, true, Ctrl->E.rot.invert, &t_max);
 		gmt_set_segmentheader (GMT, GMT_OUT, true);
 	}
 
@@ -494,7 +538,7 @@ int GMT_grdrotater (void *V_API, int mode, void *args) {
 	for (t = 0; t < Ctrl->T.n_times; t++) {	/* For each reconstruction time */
 		if (Ctrl->E.rot.single) {
 			plon = Ctrl->E.rot.lon;	plat = Ctrl->E.rot.lat;	pw = Ctrl->E.rot.w;
-			GMT_Report (API, GMT_MSG_VERBOSE, "Using rotation (%g, %g, %g)\n", plon, plat, pw);
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Using rotation (%g, %g, %g)\n", plon, plat, pw);
 		}
 		else {	/* Extract rotation for given time */
 			if (Ctrl->T.value[t] < p[0].t_stop || Ctrl->T.value[t] > p[n_stages-1].t_start) {
@@ -502,14 +546,14 @@ int GMT_grdrotater (void *V_API, int mode, void *args) {
 				continue;
 			}
 			spotter_get_rotation (GMT, p, n_stages, Ctrl->T.value[t], &plon, &plat, &pw);
-			GMT_Report (API, GMT_MSG_VERBOSE, "Time %g Ma: Using rotation (%g, %g, %g)\n", Ctrl->T.value[t], plon, plat, pw);
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Time %g Ma: Using rotation (%g, %g, %g)\n", Ctrl->T.value[t], plon, plat, pw);
 		}
 		gmt_make_rot_matrix (GMT, plon, plat, pw, R);	/* Make rotation matrix from rotation parameters */
 
 		if (Ctrl->E.rot.single)
-			GMT_Report (API, GMT_MSG_VERBOSE, "Reconstruct polygon outline\n");
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Reconstruct polygon outline\n");
 		else
-			GMT_Report (API, GMT_MSG_VERBOSE, "Reconstruct polygon outline for time %g\n", Ctrl->T.value[t]);
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Reconstruct polygon outline for time %g\n", Ctrl->T.value[t]);
 
 		/* First reconstruct the polygon outline */
 
@@ -528,7 +572,7 @@ int GMT_grdrotater (void *V_API, int mode, void *args) {
 		}
 		gmt_set_tbl_minmax (GMT, GMT_IS_POLY, polr);	/* Update table domain */
 		if (!Ctrl->N.active && not_global) {
-			char dfile[GMT_BUFSIZ] = {""}, *file = NULL;
+			char dfile[PATH_MAX] = {""}, *file = NULL;
 			if (Ctrl->T.n_times > 1) {
 				sprintf (dfile, Ctrl->D.file, Ctrl->T.value[t]);
 				file = dfile;
@@ -536,7 +580,7 @@ int GMT_grdrotater (void *V_API, int mode, void *args) {
 			else
 				file = Ctrl->D.file;
 			if (!Ctrl->E.rot.single) {	/* Add a segment header with the age via -Z */
-				char txt[BUFSIZ] = {""};
+				char txt[PATH_MAX] = {""};
 				sprintf (txt, "-Z%g", Ctrl->T.value[t]);
 				for (seg = 0; seg < polr->n_segments; seg++) {
 					Sr = polr->segment[seg];	/* Shorthand for current rotated segment */
@@ -555,11 +599,17 @@ int GMT_grdrotater (void *V_API, int mode, void *args) {
 
 		if (global)
 			gmt_M_memcpy (GMT->common.R.wesn, G->header->wesn, 4, double);
+		else if (Ctrl->A.active) {
+			gmt_M_memcpy (GMT->common.R.wesn, Ctrl->A.wesn, 4, double);
+			/* Adjust longitude range, as indicated by FORMAT_GEO_OUT */
+			gmt_lon_range_adjust (GMT->current.io.geo.range, &GMT->common.R.wesn[XLO]);
+			gmt_lon_range_adjust (GMT->current.io.geo.range, &GMT->common.R.wesn[XHI]);
+		}
 		else {
-			GMT->common.R.wesn[XLO] = floor (polr->min[GMT_X] * G->header->r_inc[GMT_X]) * G->header->inc[GMT_X];
-			GMT->common.R.wesn[XHI] = ceil  (polr->max[GMT_X] * G->header->r_inc[GMT_X]) * G->header->inc[GMT_X];
-			GMT->common.R.wesn[YLO] = floor (polr->min[GMT_Y] * G->header->r_inc[GMT_Y]) * G->header->inc[GMT_Y];
-			GMT->common.R.wesn[YHI] = ceil  (polr->max[GMT_Y] * G->header->r_inc[GMT_Y]) * G->header->inc[GMT_Y];
+			GMT->common.R.wesn[XLO] = floor (polr->min[GMT_X] * HH->r_inc[GMT_X]) * G->header->inc[GMT_X];
+			GMT->common.R.wesn[XHI] = ceil  (polr->max[GMT_X] * HH->r_inc[GMT_X]) * G->header->inc[GMT_X];
+			GMT->common.R.wesn[YLO] = floor (polr->min[GMT_Y] * HH->r_inc[GMT_Y]) * G->header->inc[GMT_Y];
+			GMT->common.R.wesn[YHI] = ceil  (polr->max[GMT_Y] * HH->r_inc[GMT_Y]) * G->header->inc[GMT_Y];
 			/* Adjust longitude range, as indicated by FORMAT_GEO_OUT */
 			gmt_lon_range_adjust (GMT->current.io.geo.range, &GMT->common.R.wesn[XLO]);
 			gmt_lon_range_adjust (GMT->current.io.geo.range, &GMT->common.R.wesn[XHI]);
@@ -571,17 +621,17 @@ int GMT_grdrotater (void *V_API, int mode, void *args) {
 			GMT_GRID_DEFAULT_REG, GMT_NOTSET, NULL)) == NULL) Return (API->error);
 
 		/* Precalculate node coordinates in both degrees and radians */
-		grd_x = gmt_grd_coord (GMT, G_rot->header, GMT_X);
-		grd_y = gmt_grd_coord (GMT, G_rot->header, GMT_Y);
+		grd_x = G_rot->x;
+		grd_y = G_rot->y;
 		grd_yc = gmt_M_memory (GMT, NULL, G_rot->header->n_rows, double);
 		for (row = 0; row < G_rot->header->n_rows; row++) grd_yc[row] = gmt_lat_swap (GMT, grd_y[row], GMT_LATSWAP_G2O);
 
 		/* Loop over all nodes in the new rotated grid and find those inside the reconstructed polygon */
 
 		if (Ctrl->E.rot.single)
-			GMT_Report (API, GMT_MSG_VERBOSE, "Interpolate reconstructed grid\n");
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Interpolate reconstructed grid\n");
 		else
-			GMT_Report (API, GMT_MSG_VERBOSE, "Interpolate reconstructed grid for time %g\n", Ctrl->T.value[t]);
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Interpolate reconstructed grid for time %g\n", Ctrl->T.value[t]);
 
 		gmt_make_rot_matrix (GMT, plon, plat, -pw, R);	/* Make inverse rotation using negative angle */
 
@@ -597,7 +647,7 @@ int GMT_grdrotater (void *V_API, int mode, void *args) {
 			yy = gmt_lat_swap (GMT, yy, GMT_LATSWAP_O2G);			/* Convert back to geodetic */
 			xx -= 360.0;
 			while (xx < G->header->wesn[XLO]) xx += 360.0;	/* Make sure we deal with 360 issues */
-			G_rot->data[ij_rot] = (float)gmt_bcr_get_z (GMT, G, xx, yy);
+			G_rot->data[ij_rot] = (gmt_grdfloat)gmt_bcr_get_z (GMT, G, xx, yy);
 		}
 
 		/* Also loop over original node locations to make sure the nearest nodes are set */
@@ -640,9 +690,9 @@ int GMT_grdrotater (void *V_API, int mode, void *args) {
 		/* Now write rotated grid */
 
 		if (Ctrl->E.rot.single)
-			GMT_Report (API, GMT_MSG_VERBOSE, "Write reconstructed grid\n");
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Write reconstructed grid\n");
 		else
-			GMT_Report (API, GMT_MSG_VERBOSE, "Write reconstructed grid for time %g\n", Ctrl->T.value[t]);
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Write reconstructed grid for time %g\n", Ctrl->T.value[t]);
 
 		gmt_set_pad (GMT, API->pad);	/* Reset to session default pad before output */
 
@@ -654,15 +704,13 @@ int GMT_grdrotater (void *V_API, int mode, void *args) {
 		if (Ctrl->T.n_times > 1)	/* Use template to create name */
 			sprintf (gfile, Ctrl->G.file, Ctrl->T.value[t]);
 		else
-			strcpy (gfile, Ctrl->G.file);
+			strncpy (gfile, Ctrl->G.file, PATH_MAX-1);
 		if (GMT_Write_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_CONTAINER_AND_DATA, NULL, gfile, G_rot) != GMT_NOERROR) {
 			Return (API->error);
 		}
 		if (G_rot && GMT_Destroy_Data (API, &G_rot) != GMT_NOERROR)
 			Return (API->error);
 
-		gmt_M_free (GMT, grd_x);
-		gmt_M_free (GMT, grd_y);
 		gmt_M_free (GMT, grd_yc);
 	} /* End of loop over reconstruction times */
 
@@ -681,7 +729,5 @@ int GMT_grdrotater (void *V_API, int mode, void *args) {
 	if (Dr && GMT_Destroy_Data (API, &Dr) != GMT_NOERROR)
 		Return (API->error);
 
-	GMT_Report (API, GMT_MSG_VERBOSE, "Done!\n");
-
 	Return (GMT_NOERROR);
 }
