@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *   Copyright (c) 1999-2018 by P. Wessel
+ *   Copyright (c) 1999-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Lesser General Public License as published by
@@ -12,7 +11,7 @@
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *   GNU Lesser General Public License for more details.
  *
- *   Contact info: www.soest.hawaii.edu/pwessel
+ *   Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  * GRDSPOTTER will (1) read a grid with topo/grav/whatever of seamounts,
@@ -105,9 +104,10 @@
 #include "gmt_dev.h"
 #include "spotter.h"
 
-#define THIS_MODULE_NAME	"grdspotter"
+#define THIS_MODULE_CLASSIC_NAME	"grdspotter"
+#define THIS_MODULE_MODERN_NAME	"grdspotter"
 #define THIS_MODULE_LIB		"spotter"
-#define THIS_MODULE_PURPOSE	"Create CVA image from a gravity or topography grid"
+#define THIS_MODULE_PURPOSE	"Create CVA grid from a gravity or topography grid"
 #define THIS_MODULE_KEYS	"<G{,AG(,DG),LG),GG}"
 #define THIS_MODULE_NEEDS	"g"
 #define THIS_MODULE_OPTIONS "-:>RVhr" GMT_OPT("F")
@@ -208,12 +208,12 @@ GMT_LOCAL void Free_Ctrl (struct GMT_CTRL *GMT, struct GRDSPOTTER_CTRL *C) {	/*
 }
 
 GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
-	gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
+	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: grdspotter <ingrid> -E[+]<rottable> -G<CVAgrid> %s\n", GMT_I_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "usage: %s <ingrid> -E[+]<rottable> -G<CVAgrid> %s\n", name, GMT_I_OPT);
 	GMT_Message (API, GMT_TIME_NONE, "\t%s [-A<agegrid>] [-D[i|p]<grdfile>] [-L<IDgrid>] [-M]\n", GMT_Rgeo_OPT);
 	GMT_Message (API, GMT_TIME_NONE, "\t[-N<upper_age>] [-Q<IDinfo>] [-S] [-Tt|-u<age>] [%s] [-W<n_try]\n", GMT_V_OPT);
-	GMT_Message (API, GMT_TIME_NONE, "\t[-Z<z_min>[/<z_max>[/<z_inc>]]] [%s] [%s]\n\n", GMT_ho_OPT, GMT_r_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "\t[-Z<z_min>[/<z_max>[/<z_inc>]]] [%s] [%s] [%s]\n\n", GMT_ho_OPT, GMT_r_OPT, GMT_PAR_OPT);
 	
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 	
@@ -278,7 +278,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRDSPOTTER_CTRL *Ctrl, struct
 				break;
 			case 'C':	/* Now done automatically in spotter_init */
 				if (gmt_M_compat_check (GMT, 4))
-					GMT_Report (API, GMT_MSG_COMPAT, "Warning: -C is no longer needed as total reconstruction vs stage rotation is detected automatically.\n");
+					GMT_Report (API, GMT_MSG_COMPAT, "-C is no longer needed as total reconstruction vs stage rotation is detected automatically.\n");
 				else
 					n_errors += gmt_default_error (GMT, opt->option);
 				break;
@@ -465,7 +465,7 @@ GMT_LOCAL bool set_age (struct GMT_CTRL *GMT, double *t_smt, struct GMT_GRID *A,
 	return (true);	/* We are returning a useful age */
 }
 
-GMT_LOCAL void normalize_grid (struct GMT_CTRL *GMT, struct GMT_GRID *G, float *data) {
+GMT_LOCAL void normalize_grid (struct GMT_CTRL *GMT, struct GMT_GRID *G, gmt_grdfloat *data) {
 	/* Determines the grid's min/max z-values and normalizes the grid
 	 * so that zmax is 100% */
 	unsigned int row, col;
@@ -479,12 +479,12 @@ GMT_LOCAL void normalize_grid (struct GMT_CTRL *GMT, struct GMT_GRID *G, float *
 		if (data[node] < G->header->z_min) G->header->z_min = data[node];
 		if (data[node] > G->header->z_max) G->header->z_max = data[node];
 	}
-	GMT_Report (GMT->parent, GMT_MSG_VERBOSE, "CVA min/max: %g %g -> ", G->header->z_min, G->header->z_max);
+	GMT_Report (GMT->parent, GMT_MSG_LONG_VERBOSE, "CVA min/max: %g %g -> ", G->header->z_min, G->header->z_max);
 	CVA_scale = 100.0 / G->header->z_max;
-	for (node = 0; node < G->header->size; node++) data[node] *= (float)CVA_scale;
+	for (node = 0; node < G->header->size; node++) data[node] *= (gmt_grdfloat)CVA_scale;
 	G->header->z_min *= CVA_scale;
 	G->header->z_max *= CVA_scale;
-	GMT_Report (GMT->parent, GMT_MSG_VERBOSE, "%g %g\n", G->header->z_min, G->header->z_max);
+	GMT_Report (GMT->parent, GMT_MSG_LONG_VERBOSE, "%g %g\n", G->header->z_min, G->header->z_max);
 }
 
 #define bailout(code) {gmt_M_free_options (mode); return (code);}
@@ -496,7 +496,7 @@ int GMT_grdspotter (void *V_API, int mode, void *args) {
 	unsigned int row, row2, col, col2, k_step;
 	unsigned int forth_flag;	/* Holds the do_time + 10 flag passed to forthtrack */
 	bool keep_flowlines = false;	/* true if Ctrl->D.active, Ctrl->PA.active, or bootstrap is true */
-	int error = 0;			/* nonzero when arguments are wrong */
+	int error = GMT_NOERROR;			/* nonzero when arguments are wrong */
 	int i, j;			/* Signed row,col variables */
 	int *ID = NULL;		/* Optional array with IDs for each node */
 	
@@ -509,7 +509,7 @@ int GMT_grdspotter (void *V_API, int mode, void *args) {
 	
 	unsigned short pa = 0;		/* Placeholder for PA along track */
 
-	float zz;
+	gmt_grdfloat zz;
 
 	double sampling_int_in_km;	/* Sampling interval along flowline (in km) */
 	double *x_smt = NULL;		/* node longitude (input degrees, stored as radians) */
@@ -521,8 +521,10 @@ int GMT_grdspotter (void *V_API, int mode, void *args) {
 	double *lat_area = NULL;	/* Area of each dx by dy note in km as function of latitude */
 	double this_wesn[4];
 	double this_pa, pa_val = 0.0, n_more_than_once = 0.0;
-	double *x_cva = NULL, *y_cva = NULL;		/* Coordinates on the CVA grid */
-	
+#ifdef DEBUG2
+	double *x_cva = NULL;		/* x-coordinates on the CVA grid */
+	double *y_cva = NULL;		/* y-coordinates on the CVA grid */
+#endif	
 	struct GMT_GRID *G = NULL;	/* Grid structure for output CVA grid */
 	struct GMT_GRID *G_rad = NULL;	/* Same but has radians in header (no grid) */
 	struct GMT_GRID *Z = NULL;	/* Grid structure for input topo/grav grid */
@@ -552,12 +554,11 @@ int GMT_grdspotter (void *V_API, int mode, void *args) {
 	if (mode == GMT_MODULE_PURPOSE) return (usage (API, GMT_MODULE_PURPOSE));	/* Return the purpose of program */
 	options = GMT_Create_Options (API, mode, args);	if (API->error) return (API->error);	/* Set or get option list */
 
-	if (!options || options->option == GMT_OPT_USAGE) bailout (usage (API, GMT_USAGE));	/* Return the usage message */
-	if (options->option == GMT_OPT_SYNOPSIS) bailout (usage (API, GMT_SYNOPSIS));	/* Return the synopsis */
+	if ((error = gmt_report_usage (API, options, 0, usage)) != GMT_NOERROR) bailout (error);	/* Give usage if requested */
 
 	/* Parse the command-line arguments */
 
-	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
 	if ((ptr = GMT_Find_Option (API, 'f', options)) == NULL) gmt_parse_common_options (GMT, "f", 'f', "g"); /* Did not set -f, implicitly set -fg */
 	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
 	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
@@ -574,7 +575,7 @@ int GMT_grdspotter (void *V_API, int mode, void *args) {
 
 	/* Load in the Euler stage poles */
 
-	n_stages = spotter_init (GMT, Ctrl->E.file, &p, true, false, Ctrl->E.mode, &Ctrl->N.t_upper);
+	n_stages = spotter_init (GMT, Ctrl->E.file, &p, 1, false, Ctrl->E.mode, &Ctrl->N.t_upper);
 
 	/* Assign grid-region variables in radians to avoid conversions inside convolution loop */
 
@@ -601,7 +602,7 @@ int GMT_grdspotter (void *V_API, int mode, void *args) {
 	sampling_int_in_km = G_rad->header->inc[GMT_X] * EQ_RAD * ((fabs (G_rad->header->wesn[YHI]) > fabs (G_rad->header->wesn[YLO])) ?
 	                     cos (G_rad->header->wesn[YHI]) : cos (G_rad->header->wesn[YLO]));
 	if (Ctrl->S2.dist != 0.0) sampling_int_in_km = Ctrl->S2.dist;
-	GMT_Report (API, GMT_MSG_VERBOSE, "Flowline sampling interval = %.3f km\n", sampling_int_in_km);
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Flowline sampling interval = %.3f km\n", sampling_int_in_km);
 
 	if (Ctrl->T.active[TRUNC]) GMT_Report (API, GMT_MSG_VERBOSE, "Ages truncated to %g\n", Ctrl->N.t_upper);
 
@@ -625,8 +626,10 @@ int GMT_grdspotter (void *V_API, int mode, void *args) {
 
 	for (row = 0; row < Z->header->n_rows; row++) lat_area[row] = area * cos (y_smt[row]);
 	
-	x_cva = gmt_grd_coord (GMT, G->header, GMT_X);
-	y_cva = gmt_grd_coord (GMT, G->header, GMT_Y);
+#ifdef DEBUG2
+	x_cva = G->x;
+	y_cva = G->y;
+#endif
 
 	if (Ctrl->A.file) {
 		if ((A = GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_CONTAINER_ONLY, NULL, Ctrl->A.file, NULL)) == NULL) {	/* Get header only */
@@ -641,6 +644,7 @@ int GMT_grdspotter (void *V_API, int mode, void *args) {
 		}
 	}
 	if (Ctrl->L.file) {
+		struct GMT_GRID_HIDDEN *GH = gmt_get_G_hidden (G);
 		if ((L = GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_CONTAINER_ONLY, NULL, Ctrl->L.file, NULL)) == NULL) {	/* Get header only */
 			Return (API->error);
 		}
@@ -655,7 +659,7 @@ int GMT_grdspotter (void *V_API, int mode, void *args) {
 		/* Store IDs in a int array instead */
 		ID = gmt_M_memory (GMT, NULL, L->header->size, int);
 		for (ij = 0; ij < L->header->size; ij++) ID[ij] = irint ((double)L->data[ij]);
-		if (G->alloc_mode == GMT_ALLOC_INTERNALLY) gmt_M_free_aligned (GMT, L->data);	/* Just free the array since we use ID; Grid struct is destroyed at end */
+		if (GH->alloc_mode == GMT_ALLOC_INTERNALLY) gmt_M_free_aligned (GMT, L->data);	/* Just free the array since we use ID; Grid struct is destroyed at end */
 		ID_info = gmt_M_memory (GMT, NULL, lrint (L->header->z_max) + 1, struct ID);
 		if (Ctrl->Q.mode == 1) {	/* Only doing one CVA with no extra restrictions */
 			ID_info[Ctrl->Q.id].ok = true;	/* Every other info in struct array is NULL or 0 */
@@ -666,8 +670,8 @@ int GMT_grdspotter (void *V_API, int mode, void *args) {
 			double wq, eq, sq, nq;
 			char line[GMT_BUFSIZ] = {""};
 			
-			if ((fp = fopen (Ctrl->Q.file, "r")) == NULL) {	/* Oh, oh... */
-				GMT_Report (API, GMT_MSG_NORMAL, "Error: -Q info file unreadable/nonexistent\n");
+			if ((fp = gmt_fopen (GMT, Ctrl->Q.file, "r")) == NULL) {	/* Oh, oh... */
+				GMT_Report (API, GMT_MSG_NORMAL, "-Q info file unreadable/nonexistent\n");
 				GMT_exit (GMT, GMT_ERROR_ON_FOPEN); return GMT_ERROR_ON_FOPEN;
 			}
 			while (fgets (line, GMT_BUFSIZ, fp)) {
@@ -762,7 +766,7 @@ int GMT_grdspotter (void *V_API, int mode, void *args) {
 			/* If we did not keep flowlines then there is no PA to skip over (hence no k++) */
 			if (node != UINTMAX_MAX) {
 				if (!processed_node[node]) {	/* Have not added to the CVA at this node yet */
-					G->data[node] += (float)cva_contribution;
+					G->data[node] += (gmt_grdfloat)cva_contribution;
 					processed_node[node] = true;		/* Now we have visited this node */
 					n_unique_nodes++;
 #ifdef DEBUG2
@@ -790,29 +794,27 @@ int GMT_grdspotter (void *V_API, int mode, void *args) {
 			gmt_M_memset (&(flowline[old_n_alloc]), n_alloc - old_n_alloc, struct FLOWLINE);	/* Set to NULL/0 */
 		}
 		
-		if (!(n_nodes%100)) GMT_Report (API, GMT_MSG_VERBOSE, "Row %5ld Processed %5" PRIu64" nodes [%5ld/%.1f]\r", row, n_nodes, n_flow, mem * B_TO_MB);
+		if (!(n_nodes%100)) GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Row %5ld Processed %5" PRIu64" nodes [%5ld/%.1f]\r", row, n_nodes, n_flow, mem * B_TO_MB);
 	}
-	GMT_Report (API, GMT_MSG_VERBOSE, "Row %5ld Processed %5" PRIu64 " nodes [%5ld/%.1f]\n", row, n_nodes, n_flow, mem * B_TO_MB);
-	GMT_Report (API, GMT_MSG_VERBOSE, "On average, each node was visited %g times\n", n_more_than_once / n_unique_nodes);
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Row %5ld Processed %5" PRIu64 " nodes [%5ld/%.1f]\n", row, n_nodes, n_flow, mem * B_TO_MB);
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "On average, each node was visited %g times\n", n_more_than_once / n_unique_nodes);
 
 	if (keep_flowlines && n_nodes != n_alloc) flowline = gmt_M_memory (GMT, flowline, n_nodes, struct FLOWLINE);
 	
 	/* OK, Done processing, time to write out */
 
 	if (Ctrl->S.active) {	/* Convert CVA values to percent of CVA maximum */		
-		GMT_Report (API, GMT_MSG_VERBOSE, "Normalize CVS grid to percentages of max CVA\n");
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Normalize CVS grid to percentages of max CVA\n");
 		normalize_grid (GMT, G, G->data);
 	}
 		
-	GMT_Report (API, GMT_MSG_VERBOSE, "Write CVA grid %s\n", Ctrl->G.file);
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Write CVA grid %s\n", Ctrl->G.file);
 
 	if (GMT_Set_Comment (API, GMT_IS_GRID, GMT_COMMENT_IS_OPTION | GMT_COMMENT_IS_COMMAND, options, G)) {
-		gmt_M_free (GMT, x_cva);	gmt_M_free (GMT, y_cva);
 		gmt_M_free (GMT, y_smt);	gmt_M_free (GMT, lat_area);
 		Return (API->error);
 	}
 	if (GMT_Write_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_CONTAINER_AND_DATA, NULL, Ctrl->G.file, G) != GMT_NOERROR) {
-		gmt_M_free (GMT, x_cva);	gmt_M_free (GMT, y_cva);
 		gmt_M_free (GMT, y_smt);	gmt_M_free (GMT, lat_area);
 		Return (API->error);
 	}
@@ -820,24 +822,24 @@ int GMT_grdspotter (void *V_API, int mode, void *args) {
 	if (Ctrl->Z.mode) {	/* Do CVA calculations for each z-slice using stored flowlines */
 		unsigned int layer, nz;
 		size_t len;
-		char file[GMT_BUFSIZ] = {""}, format[GMT_BUFSIZ] = {""};
+		char file[PATH_MAX] = {""}, format[PATH_MAX] = {""};
 		double z0, z1;
-		float *CVA_inc = NULL, *old = G->data;
+		gmt_grdfloat *CVA_inc = NULL, *old = G->data;
 		
-		GMT_Report (API, GMT_MSG_VERBOSE, "Start z-slice CVA calculations\n");
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Start z-slice CVA calculations\n");
 		for (len = strlen (Ctrl->G.file); len > 0 && Ctrl->G.file[len] != '.'; len--);
 		if (Ctrl->G.file[len] == '.') {	/* Make a filename template from the CVA filename using the period as delimiter */
 			strncpy (format, Ctrl->G.file, len);	/* Should keep the prefix from a file called prefix.ext */
 			strcat (format, "_%%d");		/* Make filenames like prefix_#.ext */
-			strcat (format, &Ctrl->G.file[len]);	/* Should add the extension from said file */
+			strncat (format, &Ctrl->G.file[len], PATH_MAX-1);	/* Should add the extension from said file */
 		}
-		CVA_inc = gmt_M_memory (GMT, NULL, G->header->size, float);
+		CVA_inc = gmt_M_memory (GMT, NULL, G->header->size, gmt_grdfloat);
 		nz = urint ((Ctrl->Z.max - Ctrl->Z.min) / Ctrl->Z.inc);
 		for (layer = 0; layer < nz; layer++) {
 			z0 = Ctrl->Z.min + layer * Ctrl->Z.inc;
 			z1 = z0 + Ctrl->Z.inc;
-			GMT_Report (API, GMT_MSG_VERBOSE, "Start z-slice %g - %g\n", z0, z1);
-			gmt_M_memset (CVA_inc, G->header->size, float);	/* Fresh start for this z-slice */
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Start z-slice %g - %g\n", z0, z1);
+			gmt_M_memset (CVA_inc, G->header->size, gmt_grdfloat);	/* Fresh start for this z-slice */
 			for (m = 0; m < n_nodes; m++) {				/* Loop over all active flowlines */
 				ij = flowline[m].ij;
 				if (Z->data[ij] <= z0 || Z->data[ij] > z1) continue;	/* z outside current slice */
@@ -847,7 +849,7 @@ int GMT_grdspotter (void *V_API, int mode, void *args) {
 				for (k = 0; k < flowline[m].n; k++) {			/* For each point along this flowline */
 					node = flowline[m].node[k];
 					if (node != UINTMAX_MAX && !processed_node[node]) {	/* Have not added to the CVA at this node yet */
-						CVA_inc[node] += (float)cva_contribution;
+						CVA_inc[node] += (gmt_grdfloat)cva_contribution;
 						processed_node[node] = true;		/* Now we have visited this node */
 					}
 				}
@@ -857,24 +859,22 @@ int GMT_grdspotter (void *V_API, int mode, void *args) {
 			
 			/* Time to write out this z-slice grid */
 			if (Ctrl->S.active) {	/* Convert CVA values to percent of CVA maximum */		
-				GMT_Report (API, GMT_MSG_VERBOSE, "Normalize CVS grid to percentages of max CVA\n");
+				GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Normalize CVS grid to percentages of max CVA\n");
 				normalize_grid (GMT, G, CVA_inc);
 			}
 			snprintf (G->header->remark, GMT_GRID_REMARK_LEN160, "CVA for z-range %g - %g only", z0, z1);
 			sprintf (file, format, layer);
 			G->data = CVA_inc;	/* Temporarily change the array pointer */
-			GMT_Report (API, GMT_MSG_VERBOSE, "Save z-slice CVA to file %s\n", file);
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Save z-slice CVA to file %s\n", file);
 			if (GMT_Set_Comment (API, GMT_IS_GRID, GMT_COMMENT_IS_OPTION | GMT_COMMENT_IS_COMMAND, options, G)) {
-				gmt_M_free (GMT, x_cva);	gmt_M_free (GMT, y_cva);
-				gmt_M_free (GMT, y_smt);	gmt_M_free (GMT, lat_area);
+				error = API->error;
 				gmt_M_free (GMT, CVA_inc);
-				Return (API->error);
+				goto END;
 			}
 			if (GMT_Write_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_CONTAINER_AND_DATA, NULL, file, G) != GMT_NOERROR) {
-				gmt_M_free (GMT, x_cva);	gmt_M_free (GMT, y_cva);
-				gmt_M_free (GMT, y_smt);	gmt_M_free (GMT, lat_area);
+				error = API->error;
 				gmt_M_free (GMT, CVA_inc);
-				Return (API->error);
+				goto END;
 			}
 		}
 		G->data = old;	/* Reset the array pointer */
@@ -883,12 +883,18 @@ int GMT_grdspotter (void *V_API, int mode, void *args) {
 			
 	if (Ctrl->D.active || Ctrl->PA.active) {	/* Must determine max CVA along each flowline */
 		if (Ctrl->D.active) {
-			if ((DI = GMT_Duplicate_Data (API, GMT_IS_GRID, GMT_DUPLICATE_ALLOC, Z)) == NULL) Return (API->error);
+			if ((DI = GMT_Duplicate_Data (API, GMT_IS_GRID, GMT_DUPLICATE_ALLOC, Z)) == NULL) {
+				error = API->error;
+				goto END;
+			}
 		}
 		if (Ctrl->PA.active) {
-			if ((PA = GMT_Duplicate_Data (API, GMT_IS_GRID, GMT_DUPLICATE_ALLOC, Z)) == NULL) Return (API->error);
+			if ((PA = GMT_Duplicate_Data (API, GMT_IS_GRID, GMT_DUPLICATE_ALLOC, Z)) == NULL) {
+				error = API->error;
+				goto END;
+			}
 		}
-		GMT_Report (API, GMT_MSG_VERBOSE, "Compute DI and/or PA grids\n");
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Compute DI and/or PA grids\n");
 
 		if (keep_flowlines) {
 			for (m = 0; m < n_nodes; m++) {	/* Loop over all active flowlines */
@@ -900,8 +906,8 @@ int GMT_grdspotter (void *V_API, int mode, void *args) {
 						if (Ctrl->PA.active) pa = flowline[m].PA[k];	/* Keep the estimate of age at highest CVA */
 					}
 				}
-				if (Ctrl->D.active) DI->data[flowline[m].ij] = (float)CVA_max;	/* Store the maximum CVA associated with this node's flowline */
-				if (Ctrl->PA.active) PA->data[flowline[m].ij] = (float) (pa * PA_2_T);
+				if (Ctrl->D.active) DI->data[flowline[m].ij] = (gmt_grdfloat)CVA_max;	/* Store the maximum CVA associated with this node's flowline */
+				if (Ctrl->PA.active) PA->data[flowline[m].ij] = (gmt_grdfloat) (pa * PA_2_T);
 				if (!(m%10000)) GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Processed %5ld flowlines\r", m);
 			}
 			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Processed %5" PRIu64 " flowlines\n", n_nodes);
@@ -938,8 +944,8 @@ int GMT_grdspotter (void *V_API, int mode, void *args) {
 					CVA_max = G->data[node];
 					if (Ctrl->PA.active) this_pa = pa_val;
 				}
-				if (Ctrl->D.active) DI->data[ij] = (float)CVA_max;	/* Store the maximum CVA associated with this node's flowline */
-				if (Ctrl->PA.active) PA->data[ij] = (float) this_pa;
+				if (Ctrl->D.active) DI->data[ij] = (gmt_grdfloat)CVA_max;	/* Store the maximum CVA associated with this node's flowline */
+				if (Ctrl->PA.active) PA->data[ij] = (gmt_grdfloat) this_pa;
 				gmt_M_free (GMT, c);
 				GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Row %5ld: Processed %5" PRIu64 " flowlines\r", row, n_nodes);
 			}
@@ -948,28 +954,27 @@ int GMT_grdspotter (void *V_API, int mode, void *args) {
 		
 
 		if (Ctrl->D.active) {
-			GMT_Report (API, GMT_MSG_VERBOSE, "Write DI grid %s\n", Ctrl->D.file);
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Write DI grid %s\n", Ctrl->D.file);
 			snprintf (DI->header->remark, GMT_GRID_REMARK_LEN160, "CVA maxima along flowlines from each node");
 			if (GMT_Set_Comment (API, GMT_IS_GRID, GMT_COMMENT_IS_OPTION | GMT_COMMENT_IS_COMMAND, options, DI)) Return (API->error);
 			if (GMT_Write_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_CONTAINER_AND_DATA, NULL, Ctrl->D.file, DI) != GMT_NOERROR) {
-				gmt_M_free (GMT, x_cva);	gmt_M_free (GMT, y_cva);
-				gmt_M_free (GMT, y_smt);	gmt_M_free (GMT, lat_area);
-				Return (API->error);
+				error = API->error;
+				goto END;
 			}
 		}
 		if (Ctrl->PA.active) {
-			GMT_Report (API, GMT_MSG_VERBOSE, "Write PA grid %s\n", Ctrl->PA.file);
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Write PA grid %s\n", Ctrl->PA.file);
 			snprintf (PA->header->remark, GMT_GRID_REMARK_LEN160, "Predicted age for each node");
 			if (GMT_Set_Comment (API, GMT_IS_GRID, GMT_COMMENT_IS_OPTION | GMT_COMMENT_IS_COMMAND, options, PA)) Return (API->error);
 			if (GMT_Write_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_CONTAINER_AND_DATA, NULL, Ctrl->PA.file, PA) != GMT_NOERROR) {
-				gmt_M_free (GMT, x_cva);	gmt_M_free (GMT, y_cva);
-				gmt_M_free (GMT, y_smt);	gmt_M_free (GMT, lat_area);
-				Return (API->error);
+				error = API->error;
+				goto END;
 			}
 		}
 	}
 	
 	if (Ctrl->W.active) {	/* Use bootstrapping to estimate confidence region for CVA maxima */
+		struct GMT_RECORD *Out = gmt_new_record (GMT, out, NULL);
 
 		if (gmt_M_is_verbose (GMT, GMT_MSG_VERBOSE)) {
 			GMT_Message (API, GMT_TIME_NONE, "Preprocessed %5" PRIu64 " flowlines\n", n_nodes);
@@ -978,23 +983,20 @@ int GMT_grdspotter (void *V_API, int mode, void *args) {
 			GMT_Message (API, GMT_TIME_NONE, "Estimate %d CVA max locations using bootstrapping\n", Ctrl->W.n_try);
 		}
 
-		if ((error = gmt_set_cols (GMT, GMT_OUT, 3)) != GMT_NOERROR) {
-			Return (error);
+		if ((error = GMT_Set_Columns (API, GMT_OUT, 3, GMT_COL_FIX_NO_TEXT)) != GMT_NOERROR) {
+			goto END;
 		}
 		if (GMT_Init_IO (API, GMT_IS_DATASET, GMT_IS_POINT, GMT_OUT, GMT_ADD_DEFAULT, 0, options) != GMT_NOERROR) {	/* Registers default output destination, unless already set */
-			gmt_M_free (GMT, x_cva);	gmt_M_free (GMT, y_cva);
-			gmt_M_free (GMT, y_smt);	gmt_M_free (GMT, lat_area);
-			Return (API->error);
+			error = API->error;
+			goto END;
 		}
 		if (GMT_Begin_IO (API, GMT_IS_DATASET, GMT_OUT, GMT_HEADER_ON) != GMT_NOERROR) {	/* Enables data output and sets access mode */
-			gmt_M_free (GMT, x_cva);	gmt_M_free (GMT, y_cva);
-			gmt_M_free (GMT, y_smt);	gmt_M_free (GMT, lat_area);
-			Return (API->error);
+			error = API->error;
+			goto END;
 		}
 		if (GMT_Set_Geometry (API, GMT_OUT, GMT_IS_POINT) != GMT_NOERROR) {	/* Sets output geometry */
-			gmt_M_free (GMT, x_cva);	gmt_M_free (GMT, y_cva);
-			gmt_M_free (GMT, y_smt);	gmt_M_free (GMT, lat_area);
-			Return (API->error);
+			error = API->error;
+			goto END;
 		}
 
 		/* Now do bootstrap sampling of flowlines */
@@ -1003,9 +1005,9 @@ int GMT_grdspotter (void *V_API, int mode, void *args) {
 		x_scale = (double)G->header->n_columns / (double)RAND_MAX;
 		y_scale = (double)G->header->n_rows / (double)RAND_MAX;
 		for (try = 1; try <= Ctrl->W.n_try; try++) {
-			GMT_Report (API, GMT_MSG_VERBOSE, "Bootstrap try %d\r", try);
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Bootstrap try %d\r", try);
 		
-			gmt_M_memset (G->data, G->header->size, float);	/* Start with fresh grid */
+			gmt_M_memset (G->data, G->header->size, gmt_grdfloat);	/* Start with fresh grid */
 			for (m = 0; m < n_nodes; m++) {	/* Loop over all indices */
 				row = urint (floor (gmt_rand(GMT) * y_scale));		/* Get a random integer in 0 to n_rows-1 range */
 				col = urint (floor (gmt_rand(GMT) * x_scale));		/* Get a random integer in 0 to n_columns-1 range */
@@ -1038,16 +1040,19 @@ int GMT_grdspotter (void *V_API, int mode, void *args) {
 			out[1] = gmt_M_grd_row_to_y (GMT, row, G->header);
 			out[2] = CVA_max;
 			
-			GMT_Put_Record (API, GMT_WRITE_DATA, out);	/* Write this to output */
+			GMT_Put_Record (API, GMT_WRITE_DATA, Out);	/* Write this to output */
 		}
-		GMT_Report (API, GMT_MSG_VERBOSE, "Bootstrap try %d\n", Ctrl->W.n_try);
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Bootstrap try %d\n", Ctrl->W.n_try);
+		gmt_M_free (GMT, Out);
 		if (GMT_End_IO (API, GMT_OUT, 0) != GMT_NOERROR) {	/* Disables further data output */
-			Return (API->error);
+			error = API->error;
+			goto END;
 		}
 	}
 	
 	/* Clean up memory */
 
+END:
 	gmt_M_free (GMT, processed_node);
 	for (m = 0; keep_flowlines && m < n_nodes; m++) {
 		gmt_M_free (GMT, flowline[m].node);
@@ -1055,7 +1060,6 @@ int GMT_grdspotter (void *V_API, int mode, void *args) {
 	}
 	gmt_M_free (GMT, p);
 	gmt_M_free (GMT, x_smt);	gmt_M_free (GMT, y_smt);
-	gmt_M_free (GMT, x_cva);	gmt_M_free (GMT, y_cva);
 	gmt_M_free (GMT, lat_area);
 	if (keep_flowlines) gmt_M_free (GMT, flowline);
 	if (Ctrl->Q.mode) gmt_M_free (GMT, ID_info);
@@ -1063,7 +1067,5 @@ int GMT_grdspotter (void *V_API, int mode, void *args) {
 		GMT_Report (API, GMT_MSG_NORMAL, "Failed to free G_rad\n");
 	}
 
-	GMT_Report (API, GMT_MSG_VERBOSE, "Done\n");
-
 	Return (GMT_NOERROR);
 }
