@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *   Copyright (c) 1999-2018 by P. Wessel
+ *   Copyright (c) 1999-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Lesser General Public License as published by
@@ -45,7 +44,7 @@
  *	2. Draw FROM a seamount TO a hotspot (and beyond): Use spotter_forthtrack
  *
  * All coordinates herein are assumed to be GEOCENTRIC.  The main programs are
- * respondible for coverting to/from geodetic data coordinates.  Rotation pole
+ * respondible for converting to/from geodetic data coordinates.  Rotation pole
  * latitudes are usually implied to be geocentric.
  */
 
@@ -62,7 +61,7 @@ EXTERN_MSC void gmtlib_get_point_from_r_az (struct GMT_CTRL *GMT, double lon0, d
 
 void spotter_rot_usage (struct GMTAPI_CTRL *API, char option) {
 	GMT_Message (API, GMT_TIME_NONE, "\t-%c Specify file with the rotations to be used (see documentation for format).\n", option);
-	GMT_Message (API, GMT_TIME_NONE, "\t   Prepend + if you want to invert the finite rotations prior to use.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   Append +i if you want to invert the finite rotations prior to use.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   Alternative 1: Give a single rotation as plon/plat/prot.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   Alternative 2: Give two plate IDs separated by a hyphen (e.g., PAC-MBL)\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   to extract that rotation from the GPlates rotation database.\n");
@@ -347,15 +346,21 @@ bool spotter_GPlates_pair (char *file) {
 
 unsigned int spotter_parse (struct GMT_CTRL *GMT, char option, char *arg, struct SPOTTER_ROT *R) {
 	unsigned int n_errors = 0, k = (arg[0] == '+') ? 1 : 0;
-	char txt_a[GMT_LEN256] = {""}, txt_b[GMT_LEN256] = {""}, txt_c[GMT_LEN256] = {""};
+	char txt_a[GMT_LEN256] = {""}, txt_b[GMT_LEN256] = {""}, txt_c[GMT_LEN256] = {""}, *c = NULL;
 	gmt_M_unused(GMT);
+	if ((c = strstr (arg, "+i"))) c[0] = '\0';	/* Chop off modifier */
 	if (k == 0 && spotter_GPlates_pair (arg)) {	/* A GPlates plate pair to look up in the rotation table */
 		R->file = strdup (arg);
 		GMT_Report (GMT->parent, GMT_MSG_DEBUG, "Received GPlates pair: %s\n", arg);
 	}
 	else if (!gmt_access (GMT, &arg[k], F_OK) && gmt_check_filearg (GMT, option, &arg[k], GMT_IN, GMT_IS_DATASET)) {	/* Was given a file (with possible leading + flag) */
 		R->file = strdup (&arg[k]);
-		if (k == 1) R->invert = true;
+		if (k == 1 || c) R->invert = true;
+		GMT_Report (GMT->parent, GMT_MSG_DEBUG, "Received rotation file: %s\n", R->file);
+	}
+	else if (gmt_M_file_is_cache (arg)) {	/* Was given a remote file */
+		R->file = strdup (&arg[k]);
+		if (k == 1 || c) R->invert = true;
 		GMT_Report (GMT->parent, GMT_MSG_DEBUG, "Received rotation file: %s\n", R->file);
 	}
 	else {	/* Apply a fixed total reconstruction rotation to all input points  */
@@ -365,8 +370,8 @@ unsigned int spotter_parse (struct GMT_CTRL *GMT, char option, char *arg, struct
 		if (ns == 2 || ns == 3) {	/* Looks like we got lon/lat/omega[/age] */
 			R->single  = true;
 			sscanf (arg, "%[^/]/%[^/]/%[^/]/%lg", txt_a, txt_b, txt_c, &(R->age));
-			n_errors += gmt_verify_expectations (GMT, GMT->current.io.col_type[GMT_IN][GMT_X], gmt_scanf_arg (GMT, txt_a, GMT->current.io.col_type[GMT_IN][GMT_X], &(R->lon)), txt_a);
-			n_errors += gmt_verify_expectations (GMT, GMT->current.io.col_type[GMT_IN][GMT_Y], gmt_scanf_arg (GMT, txt_b, GMT->current.io.col_type[GMT_IN][GMT_Y], &(R->lat)), txt_b);
+			n_errors += gmt_verify_expectations (GMT, gmt_M_type (GMT, GMT_IN, GMT_X), gmt_scanf_arg (GMT, txt_a, gmt_M_type (GMT, GMT_IN, GMT_X), false, &(R->lon)), txt_a);
+			n_errors += gmt_verify_expectations (GMT, gmt_M_type (GMT, GMT_IN, GMT_Y), gmt_scanf_arg (GMT, txt_b, gmt_M_type (GMT, GMT_IN, GMT_Y), false, &(R->lat)), txt_b);
 			R->w = atof (txt_c);
 			if (ns == 2)	/* No age given */
 				R->age = GMT->session.d_NaN;
@@ -377,8 +382,8 @@ unsigned int spotter_parse (struct GMT_CTRL *GMT, char option, char *arg, struct
 			n_errors++;
 		}
 	}
-	if (n_errors) GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Error: Rotation argument is neither GPlates pair, rotation file, or rotation parameters: %s\n", arg);
-	
+	if (n_errors) GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Rotation argument is neither GPlates pair, rotation file, or rotation parameters: %s\n", arg);
+	if (c) c[0] = '+';	/* Restore modifier */
 	return (n_errors);
 }
 
@@ -393,10 +398,10 @@ void spotter_setrot (struct GMT_CTRL *GMT, struct EULER *e) {
 	e->lat_r = e->lat * D2R;
 }
 
-unsigned int spotter_init (struct GMT_CTRL *GMT, char *file, struct EULER **p, bool flowline, bool total_out, bool invert, double *t_max) {
+unsigned int spotter_init (struct GMT_CTRL *GMT, char *file, struct EULER **p, unsigned int flowline, bool total_out, bool invert, double *t_max) {
 	/* file;	Name of file with backward stage poles, always GEOCENTRIC */
 	/* p;		Pointer to stage pole array */
-	/* flowline;	true if flowlines rather than hotspot-tracks are needed */
+	/* flowline;	1 if flowlines rather than hotspot-tracks are needed */
 	/* total_out;	true if we want to return finite (total construction poles) [alternative is stage poles] */
 	/* invert;	true if we want to invert all the rotations */
 	/* t_max;	Extend earliest stage pole back to this age */
@@ -411,11 +416,17 @@ unsigned int spotter_init (struct GMT_CTRL *GMT, char *file, struct EULER **p, b
 	char Plates[GMT_BUFSIZ] = {""}, Rotations[GMT_BUFSIZ] = {""}, *this_c = NULL;
 	double K[9];
 
+	if (gmt_M_file_is_cache (file)) {	/* Must be a cache file */
+		gmt_download_file_if_not_found (GMT, file, 0);
+	}
 	if (spotter_GPlates_pair (file)) {	/* Got PLATE_A-PLATE_B specification for GPlates lookup, e.g., IND-CIB */
 		sscanf (file, "%[^-]-%s", A, B);
-		if ((this_c = getenv ("GPLATES_PLATES")))
+		if ((this_c = getenv ("GPLATES_PLATES"))) {
 			strncpy (Plates, this_c, GMT_BUFSIZ-1);
+			GMT_Report (GMT->parent, GMT_MSG_LONG_VERBOSE, "Use environmental value of GPLATES_PLATES = %s\n", Plates);
+		}
 		else {
+			GMT_Report (GMT->parent, GMT_MSG_LONG_VERBOSE, "Use default plate ID file = %s\n", GPLATES_PLATES);
 			if (!gmt_getsharepath (GMT, "spotter", GPLATES_PLATES, ".txt", Plates, R_OK)) {	/* Decode GPlates ID file */
 				GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Unable to find GPLATES_PLATES file : %s\n", Plates);
 				GMT_exit (GMT, GMT_FILE_NOT_FOUND); return GMT_FILE_NOT_FOUND;
@@ -425,7 +436,7 @@ unsigned int spotter_init (struct GMT_CTRL *GMT, char *file, struct EULER **p, b
 		gmt_dos_path_fix (Plates);
 #endif
 		if ((fp = gmt_fopen (GMT, Plates, "r")) == NULL) {
-			GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Error: Cannot open GPlates plate id file %s\n", Plates);
+			GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Cannot open GPlates plate id file %s\n", Plates);
 			GMT_exit (GMT, GMT_ERROR_ON_FOPEN); return GMT_ERROR_ON_FOPEN;
 		}
 		GMT_Report (GMT->parent, GMT_MSG_LONG_VERBOSE, "Using GPlates plate id file %s\n", Plates);
@@ -438,17 +449,20 @@ unsigned int spotter_init (struct GMT_CTRL *GMT, char *file, struct EULER **p, b
 		}
 		gmt_fclose (GMT, fp);
 		if (A_id == 0) {
-			GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Error: Could not find an entry for plate %s in the GPlates plate id file\n", A);
+			GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Could not find an entry for plate %s in the GPlates plate id file\n", A);
 			GMT_exit (GMT, GMT_NOT_A_VALID_ARG); return GMT_NOT_A_VALID_ARG;
 		}
 		if (B_id == 0) {
-			GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Error: Could not find an entry for plate %s in the GPlates plate id file\n", B);
+			GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Could not find an entry for plate %s in the GPlates plate id file\n", B);
 			GMT_exit (GMT, GMT_NOT_A_VALID_ARG); return GMT_NOT_A_VALID_ARG;
 		}
 		/* OK, here we have the two IDs */
-		if ((this_c = getenv ("GPLATES_ROTATIONS")))
+		if ((this_c = getenv ("GPLATES_ROTATIONS"))) {
 			strncpy (Rotations, this_c, GMT_BUFSIZ-1);
+			GMT_Report (GMT->parent, GMT_MSG_LONG_VERBOSE, "Use environmental value of GPLATES_ROTATIONS = %s\n", Rotations);
+		}
 		else {
+			GMT_Report (GMT->parent, GMT_MSG_LONG_VERBOSE, "Use default rotation file = %s\n", GPLATES_ROTATIONS);
 			if (!gmt_getsharepath (GMT, "spotter", GPLATES_ROTATIONS, ".rot", Rotations, R_OK)) {	/* Decode GPlates rotations file */
 				GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Unable to find GPLATES_ROTATIONS file : %s\n", Rotations);
 				GMT_exit (GMT, GMT_FILE_NOT_FOUND); return GMT_FILE_NOT_FOUND;
@@ -458,14 +472,14 @@ unsigned int spotter_init (struct GMT_CTRL *GMT, char *file, struct EULER **p, b
 		gmt_dos_path_fix (Rotations);
 #endif
 		if ((fp = gmt_fopen (GMT, Rotations, "r")) == NULL) {
-			GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Error: Cannot open GPlates rotation file %s\n", Rotations);
+			GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Cannot open GPlates rotation file %s\n", Rotations);
 			GMT_exit (GMT, GMT_ERROR_ON_FOPEN); return GMT_ERROR_ON_FOPEN;
 		}
 		GMT_Report (GMT->parent, GMT_MSG_LONG_VERBOSE, "Using GPlates rotation file %s\n", Rotations);
 		GPlates = total_in = true;
 	}
 	else if ((fp = gmt_fopen (GMT, file, "r")) == NULL) {
-		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Error: Cannot open stage pole file: %s\n", file);
+		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Cannot open stage pole file: %s\n", file);
 		GMT_exit (GMT, GMT_ERROR_ON_FOPEN); return GMT_ERROR_ON_FOPEN;
 	}
 
@@ -494,14 +508,14 @@ unsigned int spotter_init (struct GMT_CTRL *GMT, char *file, struct EULER **p, b
 			nf = sscanf (buffer, "%lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf",
 				&e[i].lon, &e[i].lat, &e[i].t_start, &e[i].t_stop, &e[i].omega, &K[0], &K[1], &K[2], &K[3], &K[4], &K[5], &K[6], &K[7], &K[8]);
 			if (! (nf == 4 || nf == 5 || nf == 13 || nf == 14)) {
-				GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Error: Rotation file format must be lon lat t0 [t1] omega [k_hat a b c d e f g df]\n");
+				GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Rotation file format must be lon lat t0 [t1] omega [k_hat a b c d e f g df]\n");
 				gmt_fclose (GMT, fp);
 				gmt_M_free(GMT, e);
 				GMT_exit (GMT, GMT_PARSE_ERROR); return GMT_PARSE_ERROR;
 			}
 			if (nf == 4 || nf == 13) {	/* total reconstruction format: Got lon lat t0 omega [covars], must shift the K's by one */
 				if (i && !total_in) {
-					GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Error: Rotation file mixes total reconstruction and stage rotation format\n");
+					GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Rotation file mixes total reconstruction and stage rotation format\n");
 					GMT_exit (GMT, GMT_PARSE_ERROR); return GMT_PARSE_ERROR;
 				}
 				total_in = true;
@@ -518,7 +532,7 @@ unsigned int spotter_init (struct GMT_CTRL *GMT, char *file, struct EULER **p, b
 		}
 
 		if (total_in && e[i].t_start < last_t) {
-			GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Error: Rotation %d has time reversal\n", i);
+			GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Rotation %d has time reversal\n", i);
 			gmt_fclose (GMT, fp);
 			gmt_M_free(GMT, e);
 			GMT_exit (GMT, GMT_RUNTIME_ERROR); return GMT_RUNTIME_ERROR;
@@ -526,7 +540,7 @@ unsigned int spotter_init (struct GMT_CTRL *GMT, char *file, struct EULER **p, b
 		last_t = e[i].t_start;
 		//if (e[i].omega == 0.0) continue;	/* skip null rotations [2016/10/18 PW: No, since GPlates sometimes contain null rotations between stable plates] */
 		if (e[i].t_stop >= e[i].t_start) {
-			GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Error: Stage rotation %d has start time younger than stop time\n", i);
+			GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Stage rotation %d has start time younger than stop time\n", i);
 			gmt_fclose (GMT, fp);
 			gmt_M_free(GMT, e);
 			GMT_exit (GMT, GMT_RUNTIME_ERROR); return GMT_RUNTIME_ERROR;
@@ -545,7 +559,7 @@ unsigned int spotter_init (struct GMT_CTRL *GMT, char *file, struct EULER **p, b
 	gmt_fclose (GMT, fp);
 
 	if (GPlates && i == 0) {
-		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Error: Could not find rotations for the plate pair %s - %s\n", A, B);
+		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Could not find rotations for the plate pair %s - %s\n", A, B);
 		gmt_M_free(GMT, e);
 		GMT_exit (GMT, GMT_RUNTIME_ERROR); return GMT_RUNTIME_ERROR;
 	}
@@ -595,7 +609,7 @@ unsigned int spotter_init (struct GMT_CTRL *GMT, char *file, struct EULER **p, b
 }
 
 /* hotspot_init: Reads a file with hotspot information and returns pointer to
- * array of structures.  Hotspot locations are stored as geodetic coordintaes
+ * array of structures.  Hotspot locations are stored as geodetic coordinates
  * but are converted to GEOCENTRIC by this function if geocentric == true */
 
 int spotter_hotspot_init (struct GMT_CTRL *GMT, char *file, bool geocentric, struct HOTSPOT **p) {
@@ -694,7 +708,7 @@ unsigned int spotter_backtrack (struct GMT_CTRL *GMT, double xp[], double yp[],
 
 	if (path) {
 		track = gmt_M_memory (GMT, NULL, n_alloc, double);
-		i_km = EQ_RAD / d_km;
+		if (d_km != 0.0) i_km = EQ_RAD / d_km;
 	}
 
 	if (p[ns-1].t_stop > t_zero) t_zero = p[ns-1].t_stop;	/* In case we don't go all the way to zero */
@@ -875,7 +889,7 @@ unsigned int spotter_forthtrack (struct GMT_CTRL *GMT, double xp[], double yp[],
 
 	if (path) {
 		track = gmt_M_memory (GMT, NULL, n_alloc, double);
-		i_km = EQ_RAD / d_km;
+		if (d_km != 0.0) i_km = EQ_RAD / d_km;
 	}
 
 	if (p[ns-1].t_stop > t_zero) t_zero = p[ns-1].t_stop;	/* In case we don't go all the way to zero */
@@ -1354,13 +1368,18 @@ void spotter_get_rotation (struct GMT_CTRL *GMT, struct EULER *p, unsigned int n
 	/* Here we must add a partial rotation to the last finite rotation */
 	
 	i--;
-	gmt_M_memcpy (e, &p[i], 2, struct EULER);	/* Duplicate the two finite rotations bracketing the desired time */
+	if ((i+2) > np) {	/* p[i] and p[i+1] must should exist but Coverity is not sure */
+		GMT_Report (GMT->parent, GMT_MSG_NORMAL, "spotter_get_rotation: Internal error - cannot copy two rotations!");
+		return;
+	}
+	gmt_M_memcpy (&e[0], &p[i], 1, struct EULER);	/* Duplicate the two finite rotations bracketing the desired time */
+	gmt_M_memcpy (&e[1], &p[i+1], 1, struct EULER);
 	spotter_total_to_stages (GMT, e, 2, true, true);	/* Convert total reconstruction poles to forward stage poles */
 	gmt_make_rot_matrix (GMT, e[1].lon, e[1].lat, e[1].omega * e[1].duration, R);	/* Get matrix R for main rotation */
 	omega = e[1].omega * (t - e[0].t_stop);						/* Compute rotation angle for the partial rotation */
 	gmt_make_rot_matrix (GMT, e[0].lon, e[0].lat, omega, dR);			/* Get matrix Dr for the partial rotation */
 	spotter_matrix_mult (GMT, R, dR, X);						/* Calculate the combined rotation ,X */
-	spotter_matrix_to_pole (GMT, X, lon, lat, w);						/* Convert to rotation parameters lon, lat, w */
+	spotter_matrix_to_pole (GMT, X, lon, lat, w);					/* Convert to rotation parameters lon, lat, w */
 }
 
 bool spotter_conf_ellipse (struct GMT_CTRL *GMT, double lon, double lat, double t, struct EULER *p, unsigned int np, char flag, bool forward, double out[]) {
@@ -1537,14 +1556,14 @@ unsigned int spotter_confregion_radial (struct GMT_CTRL *GMT, double alpha, stru
 	axis[GMT_X] = (axis[GMT_Z] == 0) ? 1 : 0;	/* u will be either first or second original axis */
 	axis[GMT_Y] = axis[GMT_X] + 1;				/* Let v be the next (either second or third) */
 	if (axis[GMT_Y] == axis[GMT_Z]) axis[GMT_Y]++;	/* Occupied by w, go to next */
-	GMT_Report (GMT->parent, GMT_MSG_VERBOSE, "Spinning in the %c-%c coordinate system\n", name[axis[GMT_X]], name[axis[GMT_Y]]);
+	GMT_Report (GMT->parent, GMT_MSG_LONG_VERBOSE, "Spinning in the %c-%c coordinate system\n", name[axis[GMT_X]], name[axis[GMT_Y]]);
 	
 	/* We will loop over 360 degrees and determine where the radial vector intersects the projected ellipse P'(u,v)
 	 * (1) If the origin (0,0) is inside P' then we will always get two real roots that differ in sign. in that case
 	 * we simply always go with the positive root for all angles.  (2) If origin is outside P'(u,v) then there will
 	 * be angles for which the radius vector does not intersect the curve (and we get two complex roots to ignore),
 	 * otherwise the real roots come in pairs of the same sign.  Since the negative pairs repeat the information
-	 * of hte positive pairs (except they are 180 degrees shifted) we only keep the positive pairs.  These two
+	 * of the positive pairs (except they are 180 degrees shifted) we only keep the positive pairs.  These two
 	 * roots represents two different angles 180 degrees apart so therefore we store both and do the stitching
 	 * further down.
 	*/
