@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *   Copyright (c) 1999-2018 by P. Wessel
+ *   Copyright (c) 1999-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Lesser General Public License as published by
@@ -12,7 +11,7 @@
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *   GNU Lesser General Public License for more details.
  *
- *   Contact info: www.soest.hawaii.edu/pwessel
+ *   Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  * Program for converting between total reconstruction and stage poles or to add rotations.
@@ -65,7 +64,8 @@
 #include "gmt_dev.h"
 #include "spotter.h"
 
-#define THIS_MODULE_NAME	"rotconverter"
+#define THIS_MODULE_CLASSIC_NAME	"rotconverter"
+#define THIS_MODULE_MODERN_NAME	"rotconverter"
 #define THIS_MODULE_LIB		"spotter"
 #define THIS_MODULE_PURPOSE	"Manipulate total reconstruction and stage rotations"
 #define THIS_MODULE_KEYS	">D}"
@@ -80,10 +80,6 @@ struct ROTCONVERTER_CTRL {	/* All control options for this program (except commo
 	struct D {	/* -D */
 		bool active;
 	} D;
-	struct E {	/* -E[<value>] */
-		bool active;
-		double value;
-	} E;
 	struct F {	/* -F */
 		bool active;
 		bool mode;	/* out mode (true if total reconstruction rotations) */
@@ -91,6 +87,10 @@ struct ROTCONVERTER_CTRL {	/* All control options for this program (except commo
 	struct G {	/* -G */
 		bool active;
 	} G;
+	struct M {	/* -M[<value>] */
+		bool active;
+		double value;
+	} M;
 	struct N {	/* -N */
 		bool active;
 	} N;
@@ -112,7 +112,7 @@ GMT_LOCAL void *New_Ctrl (struct GMT_CTRL *GMT) {	/* Allocate and initialize a n
 	
 	/* Initialize values whose defaults are not 0/false/NULL */
 	
-	C->E.value = 0.5;	/* To get half-angles */
+	C->M.value = 0.5;	/* To get half-angles */
 	C->F.mode = true;	/* Default format is total reconstruction rotation poles for both input and output */
 	return (C);
 }
@@ -123,10 +123,10 @@ GMT_LOCAL void Free_Ctrl (struct GMT_CTRL *GMT, struct ROTCONVERTER_CTRL *C) {	/
 }
 
 GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
-	gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
+	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: rotconverter [+][-] <rotA> [[+][-] <rotB>] [[+][-] <rotC>] ... [-A] [-D] [-E[<factor>]]\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t[-F<out>] [-G] [-N] [-S] [-T] [%s] [-W]\n\t[%s]\n\n", GMT_V_OPT, GMT_h_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "usage: %s [+][-] <rotA> [[+][-] <rotB>] [[+][-] <rotC>] ... [-A] [-D]\n", name);
+	GMT_Message (API, GMT_TIME_NONE, "\t[-F<out>] [-G] [-M[<factor>]] [-N] [-S] [-T] [%s] [-W]\n\t[%s] [%s]\n\n", GMT_V_OPT, GMT_h_OPT, GMT_PAR_OPT);
 	
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
@@ -139,10 +139,10 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\n\tOPTIONS:\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-A Report angles as time [Default uses time].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-D Report all longitudes in -180/+180 range [Default is 0-360].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-E Reduce opening angles for stage rotations by <factor> [0.5].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Typically used to get half-rates needed for flowlines.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-F Set output file type: t for total reconstruction and s for stage rotations [Default is -Ft].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-G Write rotations using GPlates format [Default is spotter format].\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t-M Reduce opening angles for stage rotations by <factor> [0.5].\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   Typically used to get half-rates needed for flowlines.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-N Ensure all poles are in northern hemisphere [Default ensures positive opening angles/rates].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-S Ensure all poles are in southern hemisphere [Default ensures positive opening angles/rates].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-T Transpose the result (i.e., change sign of final rotation angle).\n");
@@ -180,24 +180,32 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct ROTCONVERTER_CTRL *Ctrl, struc
 				Ctrl->D.active = true;
 				break;
 
-			case 'E':	/* Convert to total reconstruction rotation poles instead */
-				Ctrl->E.active = true;
-				if (opt->arg[0]) Ctrl->E.value = atof (opt->arg);
+			case 'E':	/* Convert to half-spreading stage rotations [NOW -M] */
+				if (gmt_M_compat_check (GMT, 5)) { /* Warn and fall through */
+					GMT_Report (API, GMT_MSG_COMPAT, "-E is deprecated; use -M instead.\n");
+					Ctrl->M.active = true;
+					if (opt->arg[0]) Ctrl->M.value = atof (opt->arg);
+				}
+				else {
+					GMT_Report (API, GMT_MSG_NORMAL, "No such option -E\n");
+					n_errors++;
+				}
 				break;
 
 			case 'F':
 				Ctrl->F.active = true;
 				if (strlen (opt->arg) != 1) {
-					GMT_Report (API, GMT_MSG_NORMAL, "Error: Must specify -F<out>\n");
+					GMT_Report (API, GMT_MSG_NORMAL, "Must specify -F<out>\n");
 					n_errors++;
 					continue;
 				}
 				switch (opt->arg[0]) {	/* Output format */
 					case 'f':
 						if (gmt_M_compat_check (GMT, 4)) /* Warn and fall through */
-							GMT_Report (API, GMT_MSG_COMPAT, "Warning: -Ff is deprecated; use -Ft instead.\n");
+							GMT_Report (API, GMT_MSG_COMPAT, "-Ff is deprecated; use -Ft instead.\n");
+							/* Fall through on purpose to 't' */
 						else {
-							GMT_Report (API, GMT_MSG_NORMAL, "Error: Must specify t|s\n");
+							GMT_Report (API, GMT_MSG_NORMAL, "Must specify t|s\n");
 							n_errors++;
 							break;
 						}
@@ -208,7 +216,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct ROTCONVERTER_CTRL *Ctrl, struc
 						Ctrl->F.mode = false;
 						break;
 					default:
-						GMT_Report (API, GMT_MSG_NORMAL, "Error: Must specify t|s\n");
+						GMT_Report (API, GMT_MSG_NORMAL, "Must specify t|s\n");
 						n_errors++;
 						break;
 				}
@@ -218,6 +226,11 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct ROTCONVERTER_CTRL *Ctrl, struc
 				Ctrl->G.active = true;
 				break;
 
+			case 'M':	/* Convert to total reconstruction rotation poles instead */
+				Ctrl->M.active = true;
+				if (opt->arg[0]) Ctrl->M.value = atof (opt->arg);
+				break;
+
 			case 'N':	/* Ensure all poles reported are in northern hemisphere */
 				Ctrl->N.active = true;
 				break;
@@ -245,7 +258,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct ROTCONVERTER_CTRL *Ctrl, struc
 	}
 
 	n_errors += gmt_M_check_condition (GMT, (Ctrl->S.active + Ctrl->N.active + Ctrl->W.active) > 1, "Syntax error: Specify only one of -N, -S, and -W!\n");
-	n_errors += gmt_M_check_condition (GMT, Ctrl->E.active && Ctrl->F.mode, "Syntax error: -E requires stage rotations on output\n");
+	n_errors += gmt_M_check_condition (GMT, Ctrl->M.active && Ctrl->F.mode, "Syntax error: -M requires stage rotations on output.  Please add -Fs\n");
 	n_errors += gmt_M_check_condition (GMT, Ctrl->G.active && !Ctrl->F.mode, "Syntax error: -G requires total reconstruction rotations on output\n");
 
 	return (n_errors ? GMT_PARSE_ERROR : GMT_NOERROR);
@@ -277,8 +290,8 @@ int GMT_rotconverter (void *V_API, int mode, void *args) {
 	char *end_text[2] = {"tend(My)", "aend(deg)"};
 	char *time_text[2] = {"ttime(My)", "tangle(deg)"};
 	char record[GMT_BUFSIZ] = {""};
-	
 
+	struct GMT_RECORD *Out = NULL;
 	struct GMT_OPTION *ptr = NULL, *opt = NULL;
 	struct ROTCONVERTER_CTRL *Ctrl = NULL;
 	struct GMT_CTRL *GMT = NULL, *GMT_cpy = NULL;
@@ -321,7 +334,7 @@ int GMT_rotconverter (void *V_API, int mode, void *args) {
 
 	/* Parse the command-line arguments */
 
-	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
 	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
 	if ((ptr = GMT_Find_Option (API, 'f', options)) == NULL) gmt_parse_common_options (GMT, "f", 'f', "g"); /* Did not set -f, implicitly set -fg */
 	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
@@ -331,9 +344,10 @@ int GMT_rotconverter (void *V_API, int mode, void *args) {
 
 	gmt_M_memset (out, 20, double);
 	if (Ctrl->G.active) {
-		GMT->current.io.col_type[GMT_OUT][0] = GMT->current.io.col_type[GMT_OUT][1] = GMT_IS_FLOAT;
-		GMT->current.io.col_type[GMT_OUT][2] = GMT_IS_LAT;
-		GMT->current.io.col_type[GMT_OUT][3] = GMT_IS_LON;
+		gmt_set_column (GMT, GMT_OUT, 0, GMT_IS_FLOAT);
+		gmt_set_column (GMT, GMT_OUT, 1, GMT_IS_FLOAT);
+		gmt_set_column (GMT, GMT_OUT, 2, GMT_IS_LAT);
+		gmt_set_column (GMT, GMT_OUT, 3, GMT_IS_LON);
 		strcpy (GMT->current.setting.format_float_out, "%g");
 	}
 	
@@ -358,7 +372,7 @@ int GMT_rotconverter (void *V_API, int mode, void *args) {
 		else if (gmt_access (GMT, opt->arg, R_OK)) {	/* Not a readable file, is it a lon/lat/t0[/t1]/omega specification? */
 			for (k = n_slash = 0; opt->arg[k]; k++) if (opt->arg[k] == '/') n_slash++;
 			if (n_slash < 2 || n_slash > 4) {	/* No way it can be a online rotation, cry foul */
-				GMT_Report (API, GMT_MSG_NORMAL, "Error: Cannot read file %s\n", opt->arg);
+				GMT_Report (API, GMT_MSG_NORMAL, "Cannot read file %s\n", opt->arg);
 				gmt_M_free (GMT, a);
 				Return (GMT_RUNTIME_ERROR);
 			}
@@ -368,11 +382,11 @@ int GMT_rotconverter (void *V_API, int mode, void *args) {
 				if (k == 4) angle = t1, t1 = 0.0;			/* Only 4 input values */
 				if (n_slash == 2) angle = t0, t0 = 1.0, t1 = 0.0, no_time = true;	/* Quick lon/lat/angle total reconstruction rotation, no time */
 				if (t0 < t1) {
-					GMT_Report (API, GMT_MSG_NORMAL, "Error: Online rotation has t_start (%g) younger than t_stop (%g)\n", t0, t1);
+					GMT_Report (API, GMT_MSG_NORMAL, "Online rotation has t_start (%g) younger than t_stop (%g)\n", t0, t1);
 					Return (GMT_RUNTIME_ERROR);
 				}
 				if (angle == 0.0) {
-					GMT_Report (API, GMT_MSG_NORMAL, "Error: Online rotation has zero opening angle\n");
+					GMT_Report (API, GMT_MSG_NORMAL, "Online rotation has zero opening angle\n");
 					Return (GMT_RUNTIME_ERROR);
 				}
 				online_rot = true;
@@ -393,7 +407,7 @@ int GMT_rotconverter (void *V_API, int mode, void *args) {
 				if (online_stage) spotter_stages_to_total (GMT, a, n_a, true, true);
 			}
 			else
-				n_a = spotter_init (GMT, opt->arg, &a, false, true, false, &zero);	/* Return total reconstruction rotations */
+				n_a = spotter_init (GMT, opt->arg, &a, 0, true, false, &zero);	/* Return total reconstruction rotations */
 			zero = 0.0;
 			if (last_sign == -1) {	/* Leading - sign, simply reverse the rotation angles */
 				for (stage = 0; stage < n_a; stage++) {
@@ -415,7 +429,7 @@ int GMT_rotconverter (void *V_API, int mode, void *args) {
 				if (online_stage) spotter_stages_to_total (GMT, b, n_b, true, true);
 			}
 			else
-				n_b = spotter_init (GMT, opt->arg, &b, false, true, false, &zero);	/* Return total reconstruction rotations */
+				n_b = spotter_init (GMT, opt->arg, &b, 0, true, false, &zero);	/* Return total reconstruction rotations */
 			zero = 0.0;
 			spotter_add_rotations (GMT, a, n_a, b, last_sign * n_b, &p, &n_p);		/* Add the two total reconstruction rotations sets, returns total reconstruction rotations in p */
 			gmt_M_free (GMT, a);
@@ -426,7 +440,7 @@ int GMT_rotconverter (void *V_API, int mode, void *args) {
 	}
 
 	if (a == NULL) {
-		GMT_Report (API, GMT_MSG_NORMAL, "Error: No rotation resulting from operation\n");
+		GMT_Report (API, GMT_MSG_NORMAL, "No rotation resulting from operation\n");
 		Return (GMT_RUNTIME_ERROR);
 	}
 	n_out = 3 + ((Ctrl->F.mode) ? 1 - no_time : 2);
@@ -434,7 +448,7 @@ int GMT_rotconverter (void *V_API, int mode, void *args) {
 	if (a[0].has_cov) n_out += 9;
 	if (Ctrl->G.active) n_out = 6;
 	
-	if ((error = gmt_set_cols (GMT, GMT_OUT, n_out)) != GMT_NOERROR) {
+	if ((error = GMT_Set_Columns (API, GMT_OUT, n_out, GMT_COL_FIX_NO_TEXT)) != GMT_NOERROR) {
 		gmt_M_free (GMT, a);
 		Return (error);
 	}
@@ -451,16 +465,17 @@ int GMT_rotconverter (void *V_API, int mode, void *args) {
 		Return (API->error);
 	}
 
+	Out = gmt_new_record (GMT, out, NULL);	/* Since we only need to worry about numerics in this module */
 	if (Ctrl->G.active)		/* GPlates header */
-		sprintf (record, "#plateid%stime%slatitude%slongitude%sangle%sfixedplateid\n", GMT->current.setting.io_col_separator, GMT->current.setting.io_col_separator, GMT->current.setting.io_col_separator, \
+		sprintf (record, "plateid%stime%slatitude%slongitude%sangle%sfixedplateid\n", GMT->current.setting.io_col_separator, GMT->current.setting.io_col_separator, GMT->current.setting.io_col_separator, \
 			GMT->current.setting.io_col_separator, GMT->current.setting.io_col_separator);
 	else if (Ctrl->F.mode && no_time)
-		sprintf (record, "#longitude%slatitude%sangle(deg)\n", GMT->current.setting.io_col_separator, GMT->current.setting.io_col_separator);
+		sprintf (record, "longitude%slatitude%sangle(deg)\n", GMT->current.setting.io_col_separator, GMT->current.setting.io_col_separator);
 	else if (Ctrl->F.mode)	/* Easy, simply output what we've got following a header*/
-		sprintf (record, "#longitude%slatitude%s%s%sangle(deg)\n", GMT->current.setting.io_col_separator, GMT->current.setting.io_col_separator, time_text[Ctrl->A.active], GMT->current.setting.io_col_separator);
+		sprintf (record, "longitude%slatitude%s%s%sangle(deg)\n", GMT->current.setting.io_col_separator, GMT->current.setting.io_col_separator, time_text[Ctrl->A.active], GMT->current.setting.io_col_separator);
 	else {	/* Convert total reconstruction to stages before output */
 		spotter_total_to_stages (GMT, a, n_a, true, true);				/* To ensure we have the right kind of poles for output */
-		printf (record, "#longitude%slatitude%s%s%s%s%sangle(deg)\n", GMT->current.setting.io_col_separator, GMT->current.setting.io_col_separator, start_text[Ctrl->A.active], GMT->current.setting.io_col_separator, end_text[Ctrl->A.active], GMT->current.setting.io_col_separator);
+		printf (record, "longitude%slatitude%s%s%s%s%sangle(deg)\n", GMT->current.setting.io_col_separator, GMT->current.setting.io_col_separator, start_text[Ctrl->A.active], GMT->current.setting.io_col_separator, end_text[Ctrl->A.active], GMT->current.setting.io_col_separator);
 	}
 	if (GMT->current.setting.io_header[GMT_OUT]) GMT_Put_Record (API, GMT_WRITE_TABLE_HEADER, record);
 	
@@ -472,7 +487,7 @@ int GMT_rotconverter (void *V_API, int mode, void *args) {
 		while (a[stage].lon <= -180.0) a[stage].lon += 360.0;				/* Force geodetic longitude range */
 		while (!Ctrl->D.active && a[stage].lon < 0.0) a[stage].lon += 360.0;		/* Force geodetic longitude range */
 		while (Ctrl->D.active && a[stage].lon > 180.0) a[stage].lon -= 360.0;		/* Force a geographic longitude range */
-		if (Ctrl->E.active) a[stage].omega *= Ctrl->E.value;
+		if (Ctrl->M.active) a[stage].omega *= Ctrl->M.value;
 		out[GMT_X] = a[stage].lon;	out[GMT_Y] = a[stage].lat;
 		col = 2;
 		if (Ctrl->G.active) {
@@ -499,10 +514,11 @@ int GMT_rotconverter (void *V_API, int mode, void *args) {
 			spotter_covar_to_record (GMT, &a[stage], K);
 			for (k = 0; k < 9; k++) out[col++] = K[k];
 		}
-		GMT_Put_Record (API, GMT_WRITE_DATA, out);
+		GMT_Put_Record (API, GMT_WRITE_DATA, Out);
 	}
 	
 	gmt_M_free (GMT, a);
+	gmt_M_free (GMT, Out);
 
 	if (GMT_End_IO (API, GMT_OUT, 0) != GMT_NOERROR) {		/* Disables further data output */
 		Return (API->error);
