@@ -32,4 +32,8 @@
 { "Palatino-Bold",		  0.665,	0,	0 },
 { "Palatino-BoldItalic",	  0.677,	0,	0 },
 { "ZapfChancery-MediumItalic",	  0.610,	0,	0 },
-{ "ZapfDingbats",		  0.700,	1,	1 }
+{ "ZapfDingbats",		  0.700,	1,	1 },
+{ "Ryumin-Light-EUC-H",		  0.700,	1,	1 },
+{ "Ryumin-Light-EUC-V",		  0.700,	1,	1 },
+{ "GothicBBB-Medium-EUC-H",	  0.700,	1,	1 },
+{ "GothicBBB-Medium-EUC-V",	  0.700,	1,	1 },
