@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *	Copyright (c) 1991-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  * Demonstrate how to use the table i/o machinery on a record-by-record
@@ -30,7 +29,7 @@
 
 int main (int argc, char *argv[]) {
 	int n_fields = 0, mode = 0,  error = 0;
-	double *in = NULL;
+	struct GMT_RECORD *In = NULL;
 	struct GMT_OPTION *options = NULL;	/* Linked list of options */
 	struct GMTAPI_CTRL *API = NULL;		/* GMT API control structure */
 	struct GMT_CTRL *GMT = NULL;
@@ -47,7 +46,7 @@ int main (int argc, char *argv[]) {
 
 	/* 4. Initializing data input via stdin */
 	
-	if ((error = gmt_set_cols (GMT, GMT_IN, 0)) != 0) exit (EXIT_FAILURE);
+	if ((error = GMT_Set_Columns (API, GMT_IN, 0, GMT_COL_FIX)) != GMT_NOERROR) exit (EXIT_FAILURE);
 	if (GMT_Init_IO (API, GMT_IS_DATASET, GMT_IS_POINT, GMT_IN,  GMT_ADD_DEFAULT, 0, options) != GMT_NOERROR) exit (EXIT_FAILURE);	/* Establishes data input */
 	if (GMT_Init_IO (API, GMT_IS_DATASET, GMT_IS_POINT, GMT_OUT, GMT_ADD_DEFAULT, 0, options) != GMT_NOERROR) exit (EXIT_FAILURE);	/* Establishes data output */
 	
@@ -62,7 +61,7 @@ int main (int argc, char *argv[]) {
 	
 	do {	/* Keep returning records until we reach EOF */
 		mode = GMT_WRITE_DATA;	/* Normally we treat data as double precision values */
-		if ((in = GMT_Get_Record (API, GMT_READ_DATA | GMT_READ_FILEBREAK, &n_fields)) == NULL) {	/* Read next record, get NULL if special case */
+		if ((In = GMT_Get_Record (API, GMT_READ_DATA | GMT_READ_FILEBREAK, &n_fields)) == NULL) {	/* Read next record, get NULL if special case */
 			if (gmt_M_rec_is_error (GMT)) {	/* This check kicks in if the data has bad formatting, text etc */
 				GMT_Report (API, GMT_MSG_VERBOSE, "Error found in record %" PRIu64 "\n", GMT->current.io.rec_no);
 				API->print_func (stdout, "E: ");
@@ -95,12 +94,11 @@ int main (int argc, char *argv[]) {
 			assert (false);						/* Should never get here */
 		}
 		if (gmt_M_rec_is_data (GMT)) {	/* Found a data record */
-			if ((error = gmt_set_cols (GMT, GMT_IN, n_fields)) != 0) exit (EXIT_FAILURE);
-			if ((error = gmt_set_cols (GMT, GMT_OUT, n_fields)) != 0) exit (EXIT_FAILURE);
+			if ((error = GMT_Set_Columns (API, GMT_OUT, n_fields, gmt_M_colmode (In->text))) != GMT_NOERROR) exit (EXIT_FAILURE);
 			GMT_Report (API, GMT_MSG_VERBOSE, "Data found in record %" PRIu64 "\n", GMT->current.io.rec_no);
 			API->print_func (stdout, "D: ");
 		}
-		GMT_Put_Record (API, mode, in);
+		GMT_Put_Record (API, mode, In);
 	} while (true);
 	
 	API->print_func (stdout, "B: --- End of All Files ---\n");
