@@ -1,7 +1,6 @@
 /*-----------------------------------------------------------------
- *	$Id$
  *
- *      Copyright (c) 1999-2018 by P. Wessel
+ *      Copyright (c) 1999-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *      See LICENSE.TXT file for copying and redistribution conditions.
  *
  *      This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU Lesser General Public License for more details.
  *
- *      Contact info: www.soest.hawaii.edu/pwessel
+ *      Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /* x2sys.h contains the declaration for the X2SYS and XOVER structures
  * used in the XSYSTEM programs
@@ -81,6 +80,8 @@ struct GMTMGG_REC {	/* Format of old *.gmt file records */
 #define S_RDONLY 0000444
 
 #define X2SYS_VERSION "1.2"
+#define X2SYS_FMT_EXT "fmt"		/* File extension for crossover track definition format files */
+#define X2SYS_FMT_EXT_OLD "def"		/* Old file extension for crossover track definition format files */
 
 /* Make sure structure sizes are multiples of 8 */
 
@@ -122,12 +123,13 @@ struct X2SYS_INFO {
 	bool multi_segment;		/* true if there are multiple segments in this file */
 	bool geographic;		/* true if x/y data are lon/lat */
 	bool ms_next;			/* true if we just read 1st record in a new segments in this file */
+	bool rel_time;			/* true if we have relative rather than absolute time */
 	char unit[2][2];		/* Units for distance (c = Cartesian, e = meter, k = km, m = miles, n = nautical miles)
 	 				   and speed (c = Cartesian, e = m/s, k = km/hr, m = miles/hr, n = knots) */
 	char ms_flag;			/* Multi-segment header flag */
 	char suffix[16];		/* Suffix for these data files */
 	char fflags[GMT_BUFSIZ];	/* Text copy of selected columns */
-	char path[GMT_BUFSIZ];		/* Full path to current data file */
+	char path[PATH_MAX];		/* Full path to current data file */
 	char separators[8];		/* List of characters used for column separators */
 	struct X2SYS_DATA_INFO *info;	/* Array of info for each data field */
 };
@@ -167,8 +169,10 @@ struct X2SYS_BIX {
 	int nx_bin;	/* Number of x bins */
 	int ny_bin;	/* Number of y bins */
 	uint64_t nm_bin;	/* Total number of bins */
+	uint64_t n_tracks;	/* Total number of tracks */
 	bool periodic;	/* 1 if x is periodic */
 	unsigned int *binflag;	/* The bin array */
+	unsigned int mode;	/* 0: head is linked list, 1: head is fixed array */
 	struct X2SYS_BIX_DATABASE *base;
 	struct X2SYS_BIX_TRACK_INFO *head;
 };
@@ -258,6 +262,7 @@ EXTERN_MSC struct X2SYS_BIX_TRACK *x2sys_bix_make_track (struct GMT_CTRL *GMT, u
 EXTERN_MSC int x2sys_bix_read_tracks (struct GMT_CTRL *GMT, struct X2SYS_INFO *s, struct X2SYS_BIX *B, int mode, uint32_t *ID);
 EXTERN_MSC int x2sys_bix_read_index (struct GMT_CTRL *GMT, struct X2SYS_INFO *s, struct X2SYS_BIX *B, bool swap);
 EXTERN_MSC int x2sys_bix_get_index (struct GMT_CTRL *GMT, double x, double y, int *i, int *j, struct X2SYS_BIX *B, uint64_t *ID);
+EXTERN_MSC int x2sys_bix_free (struct GMT_CTRL *GMT, struct X2SYS_BIX *B);
 
 EXTERN_MSC void x2sys_path_init (struct GMT_CTRL *GMT, struct X2SYS_INFO *s);
 EXTERN_MSC int x2sys_get_data_path (struct GMT_CTRL *GMT, char *track_path, char *track, char *suffix);
