@@ -1,7 +1,6 @@
 /*-----------------------------------------------------------------
- *	$Id$
  *
- *      Copyright (c) 1999-2018 by P. Wessel
+ *      Copyright (c) 1999-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *      See LICENSE.TXT file for copying and redistribution conditions.
  *
  *      This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU Lesser General Public License for more details.
  *
- *      Contact info: www.soest.hawaii.edu/pwessel
+ *      Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /* x2sys_binlist will read one or several data files and dump their
  * contents to stdout in ASCII or binary (double precision) mode.
@@ -30,10 +29,11 @@
 #include "mgd77/mgd77.h"
 #include "x2sys.h"
 
-#define THIS_MODULE_NAME	"x2sys_binlist"
+#define THIS_MODULE_CLASSIC_NAME	"x2sys_binlist"
+#define THIS_MODULE_MODERN_NAME	"x2sys_binlist"
 #define THIS_MODULE_LIB		"x2sys"
 #define THIS_MODULE_PURPOSE	"Create bin index listing from track data files"
-#define THIS_MODULE_KEYS	">T}"
+#define THIS_MODULE_KEYS	">D}"
 #define THIS_MODULE_NEEDS	""
 #define THIS_MODULE_OPTIONS "->V"
 
@@ -75,16 +75,16 @@ GMT_LOCAL void Free_Ctrl (struct GMT_CTRL *GMT, struct X2SYS_BINLIST_CTRL *C) {
 }
 
 GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
-	gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
+	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: x2sys_binlist <files> -T<TAG> [-D] [-E] [%s]\n\n", GMT_V_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "usage: %s <files> -T<TAG> [-D] [-E] [%s] [%s]\n\n", name, GMT_V_OPT, GMT_PAR_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
 	GMT_Message (API, GMT_TIME_NONE, "\t<files> is one or more datafiles, or give =<files.lis> for a file with a list of datafiles.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-T <TAG> is the system tag for this compilation.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\n\tOPTIONS:\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-D Calculate track-lengths per bin (see x2sys_init -C for method and -N for units).\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t-D Calculate track-lengths per bin (see x2sys_init -j for method and -N for units).\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-E Bin tracks using equal-area bins (with -D only).\n");
 	GMT_Option (API, "V,.");
 	
@@ -174,7 +174,7 @@ GMT_LOCAL int comp_bincross (const void *p1, const void *p2) {
 #define Return(code) {Free_Ctrl (GMT, Ctrl); gmt_end_module (GMT, GMT_cpy); bailout (code);}
 
 int GMT_x2sys_binlist (void *V_API, int mode, void *args) {
-	char **trk_name = NULL, record[GMT_BUFSIZ] = {""}, text[GMT_LEN64] = {""};
+	char **trk_name = NULL, record[GMT_BUFSIZ] = {""};
 
 	uint64_t this_bin_index, index, last_bin_index, row, col, trk, n_tracks;
 	unsigned int curr_x_pt, prev_x_pt;
@@ -190,7 +190,9 @@ int GMT_x2sys_binlist (void *V_API, int mode, void *args) {
 	unsigned int nav_flag;
 
 	double **data = NULL, *dist_km = NULL, *dist_bin = NULL, dist_scale, x, y, dx, del_x, del_y, y_max = 90.0;
+	double out[5];
 
+	struct GMT_RECORD *Out = NULL;
 	struct X2SYS_INFO *s = NULL;
 	struct X2SYS_FILE_INFO p;		/* File information */
 	struct X2SYS_BIX B;
@@ -206,12 +208,11 @@ int GMT_x2sys_binlist (void *V_API, int mode, void *args) {
 	if (mode == GMT_MODULE_PURPOSE) return (usage (API, GMT_MODULE_PURPOSE));	/* Return the purpose of program */
 	options = GMT_Create_Options (API, mode, args);	if (API->error) return (API->error);	/* Set or get option list */
 
-	if (!options || options->option == GMT_OPT_USAGE) bailout (usage (API, GMT_USAGE));	/* Return the usage message */
-	if (options->option == GMT_OPT_SYNOPSIS) bailout (usage (API, GMT_SYNOPSIS));	/* Return the synopsis */
+	if ((error = gmt_report_usage (API, options, 0, usage)) != GMT_NOERROR) bailout (error);	/* Give usage if requested */
 
 	/* Parse the command-line arguments */
 
-	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
 	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
 	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
 	if ((error = parse (GMT, Ctrl, options)) != 0) Return (error);
@@ -239,7 +240,11 @@ int GMT_x2sys_binlist (void *V_API, int mode, void *args) {
 	}
 	else
 		gmt_set_cartesian (GMT, GMT_OUT);
-	GMT->current.io.col_type[GMT_OUT][GMT_Z] = GMT_IS_FLOAT;
+	gmt_set_column (GMT, GMT_OUT, GMT_Z, GMT_IS_FLOAT);
+	gmt_set_column (GMT, GMT_OUT, 3, GMT_IS_FLOAT);
+	if (Ctrl->D.active) gmt_set_column (GMT, GMT_OUT, 4, GMT_IS_FLOAT);
+	/* Ensure we write integers for the next two columns */
+	GMT->current.io.o_format[2] = GMT->current.io.o_format[3] = strdup ("%g"); 
 	
 	MGD77_Set_Unit (GMT, s->unit[X2SYS_DIST_SELECTION], &dist_scale, -1);	/* Gets scale which multiplies meters to chosen distance unit */
 
@@ -256,12 +261,12 @@ int GMT_x2sys_binlist (void *V_API, int mode, void *args) {
 		}
 		GMT->current.setting.proj_ellipsoid = gmt_get_ellipsoid (GMT, "Sphere");	/* Make sure we use a spherical projection */
 		mid = 0.5 * (B.wesn[XHI] + B.wesn[XLO]);	/* Central longitude to use */
-		GMT_Report (API, GMT_MSG_VERBOSE,
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE,
 		            "To undo equal-area projection, use -R%g/%g/%g/%g -JY%g/%s/360i\n",
 		            B.wesn[XLO], B.wesn[XHI], B.wesn[YLO], B.wesn[YHI], mid, EA_LAT);
 		sprintf (proj, "Y%g/%s/360", mid, EA_LAT);
 		gmt_parse_common_options (GMT, "J", 'J', proj);
-		if (gmt_M_err_pass (GMT, gmt_map_setup (GMT, B.wesn), "")) {
+		if (gmt_M_err_pass (GMT, gmt_proj_setup (GMT, B.wesn), "")) {
 			x2sys_free_list (GMT, trk_name, n_tracks);
 			x2sys_end (GMT, s);
 			Return (GMT_PROJECTION_ERROR);
@@ -283,35 +288,46 @@ int GMT_x2sys_binlist (void *V_API, int mode, void *args) {
 		dist_bin = gmt_M_memory (GMT, NULL, B.nm_bin, double);
 	}
 
-	if (GMT_Init_IO (API, GMT_IS_TEXTSET, GMT_IS_POINT, GMT_OUT, GMT_ADD_DEFAULT, 0, options) != GMT_NOERROR) {	/* Establishes data output */
+	if ((error = GMT_Set_Columns (API, GMT_OUT, (Ctrl->D.active) ? 5 : 4, GMT_COL_FIX_NO_TEXT)) != GMT_NOERROR) {
 		gmt_M_free (GMT, X);
 		if (Ctrl->D.active) gmt_M_free (GMT, dist_bin);
 		x2sys_free_list (GMT, trk_name, n_tracks);
+		x2sys_end (GMT, s);
+		Return (error);
+	}
+	if (GMT_Init_IO (API, GMT_IS_DATASET, GMT_IS_POINT, GMT_OUT, GMT_ADD_DEFAULT, 0, options) != GMT_NOERROR) {	/* Establishes data output */
+		gmt_M_free (GMT, X);
+		if (Ctrl->D.active) gmt_M_free (GMT, dist_bin);
+		x2sys_free_list (GMT, trk_name, n_tracks);
+		x2sys_end (GMT, s);
 		Return (API->error);
 	}
-	if (GMT_Begin_IO (API, GMT_IS_TEXTSET, GMT_OUT, GMT_HEADER_ON) != GMT_NOERROR) {	/* Enables data output and sets access mode */
+	if (GMT_Begin_IO (API, GMT_IS_DATASET, GMT_OUT, GMT_HEADER_ON) != GMT_NOERROR) {	/* Enables data output and sets access mode */
 		gmt_M_free (GMT, X);
 		if (Ctrl->D.active) gmt_M_free (GMT, dist_bin);
 		x2sys_free_list (GMT, trk_name, n_tracks);
+		x2sys_end (GMT, s);
 		Return (API->error);
 	}
 	if (GMT_Set_Geometry (API, GMT_OUT, GMT_IS_POINT) != GMT_NOERROR) {	/* Sets output geometry */
 		gmt_M_free (GMT, X);
 		if (Ctrl->D.active) gmt_M_free (GMT, dist_bin);
 		x2sys_free_list (GMT, trk_name, n_tracks);
+		x2sys_end (GMT, s);
 		Return (API->error);
 	}
 	gmt_set_tableheader (GMT, GMT_OUT, true);	/* Turn on -ho explicitly */
 	gmt_set_segmentheader (GMT, GMT_OUT, true);	/* Turn on segment headers on output */
 	sprintf (record, " %s", Ctrl->T.TAG);	/* Preserve the leading space for backwards compatibility */
 	GMT_Put_Record (API, GMT_WRITE_TABLE_HEADER, record);
+	Out = gmt_new_record (GMT, out, NULL);	/* Since we only need to worry about numerics in this module */
 
 	for (trk = 0; trk < n_tracks; trk++) {
 
-		GMT_Report (API, GMT_MSG_VERBOSE, "Reading file %s ", trk_name[trk]);
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Reading file %s ", trk_name[trk]);
 
 		x2sys_err_fail (GMT, (s->read_file) (GMT, trk_name[trk], &data, s, &p, &GMT->current.io, &row), trk_name[trk]);
-		GMT_Report (API, GMT_MSG_VERBOSE, "[%s]\n", s->path);
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "[%s]\n", s->path);
 		
 		if (p.n_rows == 0) {
 			GMT_Report (API, GMT_MSG_VERBOSE, "No data records found - skipping %s\n", trk_name[trk]);
@@ -320,7 +336,8 @@ int GMT_x2sys_binlist (void *V_API, int mode, void *args) {
 		}
 		
 		if (Ctrl->E.active) {	/* Project coordinates */
-			for (row = 0; row < p.n_rows; row++) gmt_geo_to_xy (GMT, data[s->x_col][row], data[s->y_col][row], &data[s->x_col][row], &data[s->y_col][row]);
+			for (row = 0; row < p.n_rows; row++)
+				gmt_geo_to_xy (GMT, data[s->x_col][row], data[s->y_col][row], &data[s->x_col][row], &data[s->y_col][row]);
 		}
 		
 		/* Reset bin flags */
@@ -329,7 +346,8 @@ int GMT_x2sys_binlist (void *V_API, int mode, void *args) {
 		if (Ctrl->D.active) {
 			int signed_flag = s->dist_flag;
 			gmt_M_memset (dist_bin, B.nm_bin, double);
-			if ((dist_km = gmt_dist_array_2 (GMT, data[s->x_col], data[s->y_col], p.n_rows, dist_scale, -signed_flag)) == NULL) gmt_M_err_fail (GMT, GMT_MAP_BAD_DIST_FLAG, "");	/* -ve gives increments */
+			if ((dist_km = gmt_dist_array_2 (GMT, data[s->x_col], data[s->y_col], p.n_rows, dist_scale, -signed_flag)) == NULL)
+				gmt_M_err_fail (GMT, GMT_MAP_BAD_DIST_FLAG, "");	/* -ve gives increments */
 		}
 
 		last_bin_index = UINT_MAX;
@@ -337,7 +355,8 @@ int GMT_x2sys_binlist (void *V_API, int mode, void *args) {
 		last_bin_col = last_bin_row = -1;
 		for (row = 0; row < p.n_rows; row++) {
 			if (outside (data[s->x_col][row], data[s->y_col][row], &B, s->geographic)) continue;
-			 x2sys_err_fail (GMT, x2sys_bix_get_index (GMT, data[s->x_col][row], data[s->y_col][row], &this_bin_col, &this_bin_row, &B, &this_bin_index), "");
+			x2sys_err_fail (GMT, x2sys_bix_get_index (GMT, data[s->x_col][row], data[s->y_col][row], &this_bin_col,
+			                                          &this_bin_row, &B, &this_bin_index), "");
 
 			/* While this may be the same bin as the last bin, the data available may have changed so we keep
 			 * turning the data flags on again and again. */
@@ -460,34 +479,31 @@ int GMT_x2sys_binlist (void *V_API, int mode, void *args) {
 			row = index / B.nx_bin;	/* To hold the row number */
 			col = index % B.nx_bin;	/* To hold the col number */
 			if (B.binflag[index] == 0) continue;
-			x = B.wesn[XLO] + (col + 0.5) * B.inc[GMT_X];
-			y = B.wesn[YLO] + (row + 0.5) * B.inc[GMT_Y];
-			gmt_ascii_format_col (GMT, record, x, GMT_OUT, GMT_X);
-			strcat (record, GMT->current.setting.io_col_separator);
-			gmt_ascii_format_col (GMT, text, y, GMT_OUT, GMT_Y);
-			strcat (record, text);
-			sprintf (text, "%s%" PRIu64 "%s%u", GMT->current.setting.io_col_separator, index, GMT->current.setting.io_col_separator, B.binflag[index]);
-			strcat (record, text);
-			if (Ctrl->D.active) {
-				strcat (record, GMT->current.setting.io_col_separator);
-				gmt_ascii_format_col (GMT, text, dist_bin[index], GMT_OUT, GMT_Z);
-				strcat (record, text);
-			}
-			GMT_Put_Record (API, GMT_WRITE_TEXT, record);
+			out[GMT_X] = B.wesn[XLO] + (col + 0.5) * B.inc[GMT_X];
+			out[GMT_Y] = B.wesn[YLO] + (row + 0.5) * B.inc[GMT_Y];
+			out[GMT_Z] = (double)index;
+			out[3] = B.binflag[index];
+			if (Ctrl->D.active)
+				out[4] = dist_bin[index];
+			GMT_Put_Record (API, GMT_WRITE_DATA, Out);
 		}
 
 		if (Ctrl->D.active) gmt_M_free (GMT, dist_km);
 	}
 	
-	if (GMT_End_IO (API, GMT_OUT, 0) != GMT_NOERROR) {	/* Disables further data output */
-		Return (API->error);
-	}
+	error = GMT_NOERROR;
+	if (GMT_End_IO (API, GMT_OUT, 0) != GMT_NOERROR) 	/* Disables further data output */
+		error = API->error;
 
+	gmt_M_str_free (GMT->current.io.o_format[2]);
+	GMT->current.io.o_format[3] = NULL; /* Since we only allocate one for sharing */
+
+	gmt_M_free (GMT, Out);
 	gmt_M_free (GMT, X);
 	x2sys_end (GMT, s);
 	gmt_M_free (GMT, B.binflag);
 	if (Ctrl->D.active) gmt_M_free (GMT, dist_bin);
 	x2sys_free_list (GMT, trk_name, n_tracks);
 
-	Return (GMT_NOERROR);
+	Return (error);
 }
