@@ -1,7 +1,6 @@
 /*-----------------------------------------------------------------
- *	$Id$
  *
- *      Copyright (c) 1999-2018 by P. Wessel
+ *      Copyright (c) 1999-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *      See LICENSE.TXT file for copying and redistribution conditions.
  *
  *      This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU Lesser General Public License for more details.
  *
- *      Contact info: gmt.soest.hawaii.edu
+ *      Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /* x2sys_init will accept command line options to specify track data
  * file formats, region of interest, bin spacing, etc, and a unique
@@ -31,12 +30,13 @@
 #include "mgd77/mgd77.h"
 #include "x2sys.h"
 
-#define THIS_MODULE_NAME	"x2sys_init"
+#define THIS_MODULE_CLASSIC_NAME	"x2sys_init"
+#define THIS_MODULE_MODERN_NAME	"x2sys_init"
 #define THIS_MODULE_LIB		"x2sys"
 #define THIS_MODULE_PURPOSE	"Initialize a new x2sys track database"
 #define THIS_MODULE_KEYS	""
 #define THIS_MODULE_NEEDS	""
-#define THIS_MODULE_OPTIONS "->RV"
+#define THIS_MODULE_OPTIONS "->RVj"
 
 EXTERN_MSC void x2sys_set_home (struct GMT_CTRL *GMT);
 
@@ -45,7 +45,7 @@ struct X2SYS_INIT_CTRL {
 		bool active;
 		char *TAG;
 	} In;
-	struct C {	/* -C */
+	struct C {	/* -C [Deprecated, now use -j] */
 		bool active;
 		char *string;
 	} C;
@@ -113,27 +113,23 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 #else
 	static char *par = "$X2SYS_HOME";
 #endif
-	gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
+	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: x2sys_init <TAG> [-Cc|f|g|e] [-D<deffile>] [-E<suffix>] [-F] [-G[d/g]] [-I[<binsize>]]\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t[-N[d|s][c|e|f|k|M|n]]] [%s] [%s] [-Wt|d|n<gap>]\n\t[-m]\n\n", GMT_Rgeo_OPT, GMT_V_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "usage: %s <TAG> [-D<deffile>] [-E<suffix>] [-F] [-G[d|g]] [-I[<binsize>]]\n", name);
+	GMT_Message (API, GMT_TIME_NONE, "\t[-N[d|s][c|e|f|k|M|n]]] [%s] [%s] [-Wt|d|n<gap>]\n\t[-m] [%s]] [%s]\n\n", GMT_Rgeo_OPT, GMT_V_OPT, GMT_j_OPT, GMT_PAR_OPT);
 	GMT_Message (API, GMT_TIME_NONE, "\t<TAG> is the unique system identifier.  Files created will be placed in the directory %s/<TAG>.\n", par);
 	GMT_Message (API, GMT_TIME_NONE, "\t   Note: The environmental parameter %s must be defined.\n\n", par);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
 	GMT_Message (API, GMT_TIME_NONE, "\n\tOPTIONS:\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-C Select procedure for along-track distance and azimuth calculations:\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   c Plain Cartesian.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   f Flat Earth.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   g Great circle [Default].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   e Ellipsoidal (geodesic) using current ellipsoid.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-D Definition file for the track data set [<TAG>.def].\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t-D Definition file for the track data set [<TAG>.%s].\n", X2SYS_FMT_EXT);
+	GMT_Message (API, GMT_TIME_NONE, "\t   (Note: deprecated extension .%s will work but consider renaming the file)\n", X2SYS_FMT_EXT_OLD);
 	GMT_Message (API, GMT_TIME_NONE, "\t-E Extension (suffix) for these data files\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   [Default equals the prefix for the definition file].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-F Force creating new files if old ones are present [Default will abort if old files are found].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-G Geographical coordinates; append g for discontinuity at Greenwich (output 0/360 [Default])\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   and append d for discontinuity at Dateline (output -180/+180).\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   or append d for discontinuity at Dateline (output -180/+180).\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-I Set bin size for track bin index output [1/1].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-N Append (d)istances or (s)peed, and your choice for unit. Choose among:\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   c Cartesian distance (user-dist-units, user-dist-units/user-time-units).\n");
@@ -143,14 +139,15 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t   M British/US units II (miles, miles/hr).\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   n Nautical units (nautical miles, knots).\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   u Old US units (survey feet, survey feet/s).\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   See -j for distance calculation modes.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   [Default is -Ndk -Nse].\n");
 	GMT_Option (API, "R");
-	GMT_Message (API, GMT_TIME_NONE, "\t   [Default region is 0/360/-90/90].\n");
+	if (gmt_M_showusage (API)) GMT_Message (API, GMT_TIME_NONE, "\t   [Default region is 0/360/-90/90].\n");
 	GMT_Option (API, "V");
 	GMT_Message (API, GMT_TIME_NONE, "\t-W Set maximum gaps allowed at crossover.  Option may be repeated.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   -Wt sets maximum time gap (in user units) [Default is infinite].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   -Wd sets maximum distance gap (in user units) [Default is infinite].\n");
-	GMT_Option (API, "m,.");
+	GMT_Option (API, "j,m,.");
 	
 	return (GMT_MODULE_USAGE);
 }
@@ -182,18 +179,25 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct X2SYS_INIT_CTRL *Ctrl, struct
 			/* Processes program-specific parameters */
 			
 			case 'C':	/* Distance calculation flag */
-				Ctrl->C.active = true;
-				if (!strchr ("cefg", (int)opt->arg[0])) {
-					GMT_Report (API, GMT_MSG_NORMAL, "Syntax error -C: Flag must be c, f, g, or e\n");
+				if (gmt_M_compat_check (API->GMT, 6)) {
+					GMT_Report (API, GMT_MSG_COMPAT, "The -C option is deprecated; use the GMT common option -j<mode> instead\n");
+					Ctrl->C.active = true;
+					if (!strchr ("cefg", (int)opt->arg[0])) {
+						GMT_Report (API, GMT_MSG_NORMAL, "Syntax error -C: Flag must be c, f, g, or e\n");
+						n_errors++;
+					}
+					if (!n_errors) Ctrl->C.string = strdup (opt->arg);
+				}
+				else {
+					GMT_Report (API, GMT_MSG_NORMAL, "Unrecognized option -C\n");
 					n_errors++;
 				}
-				if (!n_errors) Ctrl->C.string = strdup (opt->arg);
 				break;
 			case 'D':
 				Ctrl->D.active = true;
-				if ((c = strstr (opt->arg, ".def")) == NULL)
-					Ctrl->D.file = strdup (opt->arg);
-				else {
+				if ((c = strstr (opt->arg, "." X2SYS_FMT_EXT)) == NULL && (c = strstr (opt->arg, "." X2SYS_FMT_EXT_OLD)) == NULL)
+					Ctrl->D.file = strdup (opt->arg);	/* Gave no extension so store everything */
+				else {	/* Must avoid the extension */
 					c[0] = '\0';	/* Chop off extension */
 					Ctrl->D.file = strdup (opt->arg);
 					c[0] = '.';	/* Restore extension */
@@ -278,8 +282,9 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct X2SYS_INIT_CTRL *Ctrl, struct
 int GMT_x2sys_init (void *V_API, int mode, void *args) {
 
 	time_t right_now;
-	char tag_file[GMT_BUFSIZ] = {""}, track_file[GMT_BUFSIZ] = {""}, bin_file[GMT_BUFSIZ] = {""}, def_file[GMT_BUFSIZ] = {""};
-	char path_file[GMT_BUFSIZ] = {""}, path[GMT_BUFSIZ] = {""}, line[GMT_BUFSIZ] = {""};
+	char tag_file[PATH_MAX] = {""}, track_file[PATH_MAX] = {""}, bin_file[PATH_MAX] = {""}, def_file[PATH_MAX] = {""};
+	char path_file[PATH_MAX] = {""}, path[PATH_MAX] = {""}, line[GMT_BUFSIZ] = {""};
+	char *name = NULL;
 
 	int error = 0;
 
@@ -298,12 +303,11 @@ int GMT_x2sys_init (void *V_API, int mode, void *args) {
 	if (mode == GMT_MODULE_PURPOSE) return (usage (API, GMT_MODULE_PURPOSE));	/* Return the purpose of program */
 	options = GMT_Create_Options (API, mode, args);	if (API->error) return (API->error);	/* Set or get option list */
 
-	if (!options || options->option == GMT_OPT_USAGE) bailout (usage (API, GMT_USAGE));	/* Return the usage message */
-	if (options->option == GMT_OPT_SYNOPSIS) bailout (usage (API, GMT_SYNOPSIS));	/* Return the synopsis */
+	if ((error = gmt_report_usage (API, options, 0, usage)) != GMT_NOERROR) bailout (error);	/* Give usage if requested */
 
 	/* Parse the command-line arguments */
 
-	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
 	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
 	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
 	if ((error = parse (GMT, Ctrl, options)) != 0) Return (error);
@@ -311,11 +315,18 @@ int GMT_x2sys_init (void *V_API, int mode, void *args) {
 	/*---------------------------- This is the x2sys_init main code ----------------------------*/
 
 	if (!Ctrl->D.active) Ctrl->D.file = strdup (Ctrl->In.TAG);	/* Default */
-	sprintf (def_file, "%s.def", Ctrl->D.file);
-	if (access (def_file, R_OK)) {	/* No such local *.def file */
-		if (!gmt_getsharepath (GMT, "x2sys", Ctrl->D.file, ".def", def_file, R_OK)) {	/* Decode Custom font file */
-			GMT_Report (API, GMT_MSG_NORMAL, "Unable to find local definition file : %s\n", def_file);
-			Return (GMT_ERROR_ON_FOPEN);
+	sprintf (def_file, "%s.%s", Ctrl->D.file, X2SYS_FMT_EXT);
+	if (access (def_file, R_OK)) {	/* No such local *.fmt file */
+		if (!gmt_getsharepath (GMT, "x2sys", Ctrl->D.file, "." X2SYS_FMT_EXT, def_file, R_OK)) {	/* Not found in GMT x2sys share path */
+			/* Try old deprecated extension */
+			sprintf (def_file, "%s.%s", Ctrl->D.file, X2SYS_FMT_EXT_OLD);
+			if (access (def_file, R_OK)) {	/* No such local *.def file */
+				if (!gmt_getsharepath (GMT, "x2sys", Ctrl->D.file, "." X2SYS_FMT_EXT_OLD, def_file, R_OK)) {	/* Not found in GMT x2sys share path */
+					GMT_Report (API, GMT_MSG_NORMAL, "Unable to find definition file : %s\n", def_file);
+					Return (GMT_ERROR_ON_FOPEN);
+				}
+			}
+			GMT_Report (API, GMT_MSG_VERBOSE, "Found deprecated file extension .%s, please rename to .%s\n", X2SYS_FMT_EXT_OLD, X2SYS_FMT_EXT);
 		}
 	}
 	if ((fp_def = fopen (def_file, "r")) == NULL) {
@@ -330,11 +341,7 @@ int GMT_x2sys_init (void *V_API, int mode, void *args) {
 	x2sys_set_home (GMT);
 	x2sys_path (GMT, Ctrl->In.TAG, path);
 	if (x2sys_access (GMT, Ctrl->In.TAG, R_OK)) {	/* No such dir */
-#ifndef _WIN32
-		if (mkdir (path, (mode_t)0777)) {
-#else
-		if (mkdir (path)) {
-#endif
+		if (gmt_mkdir (path)) {
 			GMT_Report (API, GMT_MSG_NORMAL, "Unable to create TAG directory : %s\n", path);
 			fclose (fp_def);	/* Close local def file */
 			Return (GMT_RUNTIME_ERROR);
@@ -349,7 +356,7 @@ int GMT_x2sys_init (void *V_API, int mode, void *args) {
 	/* Initialize the system TAG files in X2SYS_HOME/TAG */
 
 	sprintf (tag_file,   "%s/%s.tag",       Ctrl->In.TAG, Ctrl->In.TAG);
-	sprintf (def_file,   "%s/%s.def",       Ctrl->In.TAG, &Ctrl->D.file[d_start]);
+	sprintf (def_file,   "%s/%s.%s",       Ctrl->In.TAG, &Ctrl->D.file[d_start], X2SYS_FMT_EXT);
 	sprintf (path_file,  "%s/%s_paths.txt", Ctrl->In.TAG, Ctrl->In.TAG);
 	sprintf (track_file, "%s/%s_tracks.d",  Ctrl->In.TAG, Ctrl->In.TAG);
 	sprintf (bin_file,   "%s/%s_index.b",   Ctrl->In.TAG, Ctrl->In.TAG);
@@ -421,19 +428,22 @@ int GMT_x2sys_init (void *V_API, int mode, void *args) {
 	}
 
 
-	GMT_Report (API, GMT_MSG_VERBOSE, "Initialize %s\n", tag_file);
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Initialize %s\n", tag_file);
 	if ((fp = x2sys_fopen (GMT, tag_file, "w")) == NULL) {
 		GMT_Report (API, GMT_MSG_NORMAL, "Could not create file %s\n", tag_file);
 		fclose (fp_def);	/* Close local def file */
 		Return (GMT_ERROR_ON_FOPEN);
 	}
 
-        right_now = time ((time_t *)0);
+	name = gmt_putusername(NULL);
+	right_now = time ((time_t *)0);
 	fprintf (fp, "# TAG file for system: %s\n", Ctrl->In.TAG);
 	fprintf (fp, "#\n# Initialized on: %s", ctime (&right_now));
-	fprintf (fp, "# Initialized by: %s\n#\n", gmt_putusername(GMT));
+	fprintf (fp, "# Initialized by: %s\n#\n", name);
+	gmt_M_str_free (name);
 	fprintf (fp, "-D%s", &Ctrl->D.file[d_start]);	/* Now a local *.def file in the TAG directory */
-	if (Ctrl->C.active) fprintf (fp, " -C%s", Ctrl->C.string);
+	if (GMT->common.j.active) fprintf (fp, " -j%s", GMT->common.j.string);
+	else if (Ctrl->C.active) fprintf (fp, " -j%s", Ctrl->C.string);
 	if (Ctrl->E.active) fprintf (fp, " -E%s", Ctrl->E.string);
 	if (Ctrl->G.active) fprintf (fp, " -G%s", Ctrl->G.string);
 	if (Ctrl->m.active) fprintf (fp, " -m%s", Ctrl->m.string);
@@ -448,7 +458,7 @@ int GMT_x2sys_init (void *V_API, int mode, void *args) {
 
 	/* Initialize the system's definition file  */
 
-	GMT_Report (API, GMT_MSG_VERBOSE, "Initialize %s\n", def_file);
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Initialize %s\n", def_file);
 	if ((fp = x2sys_fopen (GMT, def_file, "w")) == NULL) {
 		GMT_Report (API, GMT_MSG_NORMAL, "Could not create %s\n", def_file);
 		fclose (fp_def);	/* Close local def file */
@@ -460,7 +470,7 @@ int GMT_x2sys_init (void *V_API, int mode, void *args) {
 
 	/* Initialize the system's tracks data base  */
 
-	GMT_Report (API, GMT_MSG_VERBOSE, "Initialize %s\n", track_file);
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Initialize %s\n", track_file);
 	if ((fp = x2sys_fopen (GMT, track_file, "w")) == NULL) {
 		GMT_Report (API, GMT_MSG_NORMAL, "Could not create %s\n", track_file);
 		Return (GMT_ERROR_ON_FOPEN);
@@ -471,7 +481,7 @@ int GMT_x2sys_init (void *V_API, int mode, void *args) {
 
 	/* Initialize the system's index data base  */
 
-	GMT_Report (API, GMT_MSG_VERBOSE, "Initialize %s\n", bin_file);
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Initialize %s\n", bin_file);
 	if ((fp = x2sys_fopen (GMT, bin_file, "wb")) == NULL) {
 		GMT_Report (API, GMT_MSG_NORMAL, "Could not create %s\n", bin_file);
 		Return (GMT_ERROR_ON_FOPEN);
@@ -480,7 +490,7 @@ int GMT_x2sys_init (void *V_API, int mode, void *args) {
 
 	/* Initialize the system's track path file  */
 
-	GMT_Report (API, GMT_MSG_VERBOSE, "Initialize %s\n", path_file);
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Initialize %s\n", path_file);
 	if ((fp = x2sys_fopen (GMT, path_file, "wb")) == NULL) {
 		GMT_Report (API, GMT_MSG_NORMAL, "Could not create %s\n", path_file);
 		Return (GMT_ERROR_ON_FOPEN);
@@ -490,7 +500,7 @@ int GMT_x2sys_init (void *V_API, int mode, void *args) {
 	fprintf (fp, "# Add full paths to search additional directories\n");
 	x2sys_err_fail (GMT, x2sys_fclose (GMT, path_file, fp), path_file);
 	
-	GMT_Report (API, GMT_MSG_VERBOSE, "completed successfully\n");
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "completed successfully\n");
 
 	gmt_M_free (GMT, X2SYS_HOME);
 
