@@ -1,7 +1,6 @@
 /*-----------------------------------------------------------------
- *	$Id$
  *
- *      Copyright (c) 1999-2018 by P. Wessel
+ *      Copyright (c) 1999-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *      See LICENSE.TXT file for copying and redistribution conditions.
  *
  *      This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU Lesser General Public License for more details.
  *
- *      Contact info: gmt.soest.hawaii.edu
+ *      Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /* x2sys_list will read the crossover data base and output a subset of
  * the crossovers in a format determined by the options.
@@ -29,7 +28,8 @@
 #include "mgd77/mgd77.h"
 #include "x2sys.h"
 
-#define THIS_MODULE_NAME	"x2sys_list"
+#define THIS_MODULE_CLASSIC_NAME	"x2sys_list"
+#define THIS_MODULE_MODERN_NAME	"x2sys_list"
 #define THIS_MODULE_LIB		"x2sys"
 #define THIS_MODULE_PURPOSE	"Extract subset from crossover data base"
 #define THIS_MODULE_KEYS	">D}"
@@ -37,7 +37,7 @@
 #define THIS_MODULE_OPTIONS "->RV"
 
 #define GMT_T	3	/* Just used to indicate abs time formatting */
-#define LETTERS "acdhiInNtTvwxyz"
+#define LETTERS "acdhiINtTvwxyzn"
 
 struct X2SYS_LIST_CTRL {
 	struct X2SYS_LIST_In {
@@ -57,6 +57,7 @@ struct X2SYS_LIST_CTRL {
 	} E;
 	struct X2SYS_LIST_F {	/* -F */
 		bool active;
+		bool mixed;
 		char *flags;
 	} F;
 	struct X2SYS_LIST_I {	/* -I */
@@ -67,9 +68,10 @@ struct X2SYS_LIST_CTRL {
 		bool active;
 		char *file;
 	} L;
-	struct X2SYS_LIST_N {	/* -N */
+	struct X2SYS_LIST_N {	/* -N<cutoff>[+p] */
 		bool active;
 		unsigned int min;
+		unsigned int mode;
 	} N;
 	struct X2SYS_LIST_Q {	/* -Q */
 		bool active;
@@ -115,10 +117,11 @@ GMT_LOCAL void Free_Ctrl (struct GMT_CTRL *GMT, struct X2SYS_LIST_CTRL *C) {	/*
 }
 
 GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
-	gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
+	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: x2sys_list -C<column> -T<TAG> [<COEdbase>] [-A<asymm_max] [-E] [-F<flags>] [-I<ignorelist>]\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t[-L[<corrtable.txt>]] [-N<nx_min>] [-Qe|i] [-S[+]<track>]\n\t[%s] [%s] [-W<weight>] [%s] [%s]\n\n", GMT_Rgeo_OPT, GMT_V_OPT, GMT_bo_OPT, GMT_do_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "usage: %s -C<column> -T<TAG> [<COEdbase>] [-A<asymm_max] [-E] [-F<flags>] [-I<ignorelist>]\n", name);
+	GMT_Message (API, GMT_TIME_NONE, "\t[-L[<corrtable.txt>]] [-N<nx_min>[+p]] [-Qe|i] [-S<track>[+b]]\n\t[%s] [%s] [-W<weight>] [%s] [%s] [%s]\n\n",
+		GMT_Rgeo_OPT, GMT_V_OPT, GMT_bo_OPT, GMT_do_OPT, GMT_PAR_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
@@ -131,6 +134,7 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t   asymmetry = (n_right - n_left)/(n_right + n_left) [1, i.e., use all tracks].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-E Enhanced ASCII output: Add segment header with track names and number of crossovers [no segment headers].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-F Specify any combination of %s in the order of desired output:\n", LETTERS);
+	GMT_Message (API, GMT_TIME_NONE, "\t   Exception: n, if chosen, will always be placed at the end of the output record.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   a Angle (<= 90) between the two tracks at the crossover.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   c Crossover error in chosen observable (see -C).\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   d Distance along tracks at the crossover.\n");
@@ -150,12 +154,13 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t-I List of tracks to ignore [Use all tracks].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-L Subtract systematic corrections from the data. If no correction file is given,\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   the default file <TAG>_corrections.txt in $X2SYS_HOME/<TAG> is assumed.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-N Output results for tracks with more than <nx_min> crossovers only [Use all tracks].\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t-N Suppress results involving tracks with less than a total of <nx_min> crossovers [Use all tracks].\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   Alternatively, append +p to suppress pairs with less than <nx_min> crossovers [Use all pairs].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-Q Append e or i for external or internal crossovers [Default is both].\n");
 	GMT_Option (API, "R");
-	GMT_Message (API, GMT_TIME_NONE, "\t   [Default region is the entire data domain].\n");
+	if (gmt_M_showusage (API)) GMT_Message (API, GMT_TIME_NONE, "\t   [Default region is the entire data domain].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-S Return only crossovers involving this track [Use all tracks].\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Prepend a '+' to make it print info relative to both tracks [Default is selected track].\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   Append +b to make it print info relative to both tracks [Default is selected track].\n");
 	GMT_Option (API, "V");
 	GMT_Message (API, GMT_TIME_NONE, "\t-W If argument can be opened as a file then we expect a List of tracks and their\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   relative weights; otherwise the argument is the constant weight for all tracks [1].\n");
@@ -173,7 +178,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct X2SYS_LIST_CTRL *Ctrl, struct
 	 */
 
 	unsigned int n_errors = 0, i, n_files[2] = {0, 0};
-	bool mixed = false;
+	char *c = NULL;
 	struct GMT_OPTION *opt = NULL;
 	struct GMTAPI_CTRL *API = GMT->parent;
 
@@ -208,7 +213,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct X2SYS_LIST_CTRL *Ctrl, struct
 				Ctrl->F.flags = strdup (opt->arg);
 				break;
 			case 'I':
-				if ((Ctrl->I.active = gmt_check_filearg (GMT, 'I', opt->arg, GMT_IN, GMT_IS_TEXTSET)) != 0)
+				if ((Ctrl->I.active = gmt_check_filearg (GMT, 'I', opt->arg, GMT_IN, GMT_IS_DATASET)) != 0)
 					Ctrl->I.file = strdup (opt->arg);
 				else
 					n_errors++;
@@ -216,7 +221,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct X2SYS_LIST_CTRL *Ctrl, struct
 			case 'L':	/* Crossover correction table */
 				Ctrl->L.active = true;
 				if (opt->arg[0]) {
-					if (gmt_check_filearg (GMT, 'L', opt->arg, GMT_IN, GMT_IS_TEXTSET))
+					if (gmt_check_filearg (GMT, 'L', opt->arg, GMT_IN, GMT_IS_DATASET))
 						Ctrl->L.file = strdup (opt->arg);
 					else
 						n_errors++;
@@ -224,7 +229,12 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct X2SYS_LIST_CTRL *Ctrl, struct
 				break;
 			case 'N':
 				Ctrl->N.active = true;
+				if ((c = strstr (opt->arg, "+p"))) {
+					c[0] = '\0';	/* Chop off modifier */
+					Ctrl->N.mode = 1;
+				}
 				Ctrl->N.min = atoi (opt->arg);
+				if (c) c[0] = '+';	/* Restore modifier */
 				break;
 			case 'Q':	/* Specify internal or external only */
 				Ctrl->Q.active = true;
@@ -238,12 +248,18 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct X2SYS_LIST_CTRL *Ctrl, struct
 					Ctrl->S.both = true;
 					Ctrl->S.file = strdup (&opt->arg[1]);
 				}
+				else if ((c = strstr (opt->arg, "+b"))) {
+					Ctrl->S.both = true;
+					c[0] = '\0';	/* Chop off modifier */
+					Ctrl->S.file = strdup (opt->arg);
+				}
 				else if (opt->arg[0])
 					Ctrl->S.file = strdup (opt->arg);
 				else {
 					GMT_Report (API, GMT_MSG_NORMAL, "Syntax error -S: Must supply a track name.\n");
 					n_errors++;
 				}
+				if (c) c[0] = '+';	/* Restore modifier */
 				break;
 			case 'T':
 				Ctrl->T.active = true;
@@ -271,29 +287,12 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct X2SYS_LIST_CTRL *Ctrl, struct
 			GMT_Report (API, GMT_MSG_NORMAL, "Syntax error -F: Unknown item %c.\n", Ctrl->F.flags[i]);
 			n_errors++;			
 		}
-		if (Ctrl->F.flags[i] == 'n') mixed = true;		/* Both numbers and text - cannot use binary output */
+		if (Ctrl->F.flags[i] == 'n') Ctrl->F.mixed = true;	/* Output will have trailing text */
 	}
-	/* GMT->parent->external means we are calling from mex or Python and don't want to get textsets back */
-	n_errors += gmt_M_check_condition (GMT, (mixed || GMT->parent->external) && GMT->common.b.active[GMT_OUT], "Syntax error: Cannot use -Fn with binary output\n");
 
 	return (n_errors ? GMT_PARSE_ERROR : GMT_NOERROR);
 }
 
-GMT_LOCAL void dump_ascii_cols (struct GMT_CTRL *GMT, double *val, int col, int n, bool first, char *record) {
-	/* Short-hand to dump n = 1 or 2 numerical values in chosen format.
-	 * col is used to set the format, and first is true for first item per record.
-	 */
-	int i;
-	char text[GMT_LEN64] = {""};
-	if (first) record[0] = 0;
-	for (i = 0; i < n; i++) {
-		if (!first) strcat (record, GMT->current.setting.io_col_separator);
-		gmt_ascii_format_col (GMT, text, val[i], GMT_OUT, col);
-		strcat (record, text);
-		first = false;
-	}
-}
-
 #define bailout(code) {gmt_M_free_options (mode); return (code);}
 #define Return(code) {Free_Ctrl (GMT, Ctrl); gmt_end_module (GMT, GMT_cpy); bailout (code);}
 
@@ -303,15 +302,16 @@ int GMT_x2sys_list (void *V_API, int mode, void *args) {
 	struct X2SYS_INFO *s = NULL;
 	struct X2SYS_BIX B;
 	struct X2SYS_COE_PAIR *P = NULL;
-	bool mixed = false, check_for_NaN = false, both, first;
+	bool check_for_NaN = false, both;
 	bool internal = true;	/* false if only external xovers are needed */
 	bool external = true;	/* false if only internal xovers are needed */
 	uint64_t i, j, k, one, two, n_items, n_tracks;
 	uint64_t p, np_use = 0, nx_use = 0, np, m, nx, *trk_nx = NULL;
-	unsigned int n_weights = 0, coe_kind, n_out, n_output, o_mode;
+	unsigned int n_weights = 0, coe_kind, n_output, cmode;
 	int error = 0, id;
 	double *wesn = NULL, val[2], out[128], corr[2] = {0.0, 0.0}, sec_2_unit = 1.0, w_k, w;
 	double fixed_weight = 1.0, *weights = NULL, *trk_symm = NULL;
+	struct GMT_RECORD *Out = NULL;
 	struct MGD77_CORRTABLE **CORR = NULL;
 	struct X2SYS_LIST_CTRL *Ctrl = NULL;
 	struct GMT_CTRL *GMT = NULL, *GMT_cpy = NULL;
@@ -324,22 +324,23 @@ int GMT_x2sys_list (void *V_API, int mode, void *args) {
 	if (mode == GMT_MODULE_PURPOSE) return (usage (API, GMT_MODULE_PURPOSE));	/* Return the purpose of program */
 	options = GMT_Create_Options (API, mode, args);	if (API->error) return (API->error);	/* Set or get option list */
 
-	if (!options || options->option == GMT_OPT_USAGE) bailout (usage (API, GMT_USAGE));	/* Return the usage message */
-	if (options->option == GMT_OPT_SYNOPSIS) bailout (usage (API, GMT_SYNOPSIS));	/* Return the synopsis */
+	if ((error = gmt_report_usage (API, options, 0, usage)) != GMT_NOERROR) bailout (error);	/* Give usage if requested */
 
 	/* Parse the command-line arguments */
 
-	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
 	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
 	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
 	if ((error = parse (GMT, Ctrl, options)) != 0) Return (error);
 	
  	/*---------------------------- This is the x2sys_list main code ----------------------------*/
 
-	for (i = 0; i < strlen (Ctrl->F.flags); i++) {
+	for (i = j = 0; i < strlen (Ctrl->F.flags); i++) {
 		if (Ctrl->F.flags[i] == 'c' || Ctrl->F.flags[i] == 'z') check_for_NaN = true; /* Do not output records where the crossover or values are NaN */
-		if (Ctrl->F.flags[i] == 'n') mixed = true;		/* Both numbers and text */
+		if (j < i) Ctrl->F.flags[j] = Ctrl->F.flags[i];
+		if (Ctrl->F.flags[i] != 'n') j++;
 	}
+	Ctrl->F.flags[j] = '\0';	/* Eliminating 'n' from the flags list */
 	if (Ctrl->Q.active) {
 		if (Ctrl->Q.mode == 1) internal = false;
 		if (Ctrl->Q.mode == 2) external = false;
@@ -355,7 +356,7 @@ int GMT_x2sys_list (void *V_API, int mode, void *args) {
 	
 	if (Ctrl->C.col) x2sys_err_fail (GMT, x2sys_pick_fields (GMT, Ctrl->C.col, s), "-C");
 	if (s->n_out_columns != 1) {
-		GMT_Report (API, GMT_MSG_NORMAL, "Error: -C must specify a single column name\n");
+		GMT_Report (API, GMT_MSG_NORMAL, "-C must specify a single column name\n");
 		x2sys_end (GMT, s);
 		Return (GMT_RUNTIME_ERROR);
 	}
@@ -371,9 +372,9 @@ int GMT_x2sys_list (void *V_API, int mode, void *args) {
 	
 	from = (Ctrl->In.file) ? Ctrl->In.file : tofrom[GMT_IN];
 	if (GMT->common.R.active[RSET]) wesn = GMT->common.R.wesn;	/* Passed a sub region request */
-	GMT_Report (API, GMT_MSG_VERBOSE, "Read crossover database from %s...\n", from);
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Read crossover database from %s...\n", from);
 	np = x2sys_read_coe_dbase (GMT, s, Ctrl->In.file, Ctrl->I.file, wesn, Ctrl->C.col, coe_kind, Ctrl->S.file, &P, &nx, &n_tracks);
-	GMT_Report (API, GMT_MSG_VERBOSE, "Found %" PRIu64 " pairs and a total of %" PRIu64 " crossover records.\n", np, nx);
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Found %" PRIu64 " pairs and a total of %" PRIu64 " crossover records.\n", np, nx);
 
 	if (np == 0 && nx == 0) {	/* End here since nothing was allocated */
 		x2sys_end (GMT, s);
@@ -398,70 +399,47 @@ int GMT_x2sys_list (void *V_API, int mode, void *args) {
 	/* Initialize column output types */
 	
 	one = 0;	two = 1;	/* Normal track order */
-	both = Ctrl->S.both;		/* Usually false unless -S+<track> is set */
+	both = Ctrl->S.both;		/* Usually false unless -S<track>+b is set */
 	if (!both) both = !Ctrl->S.active;	/* Two columns for many output choices */
-	n_out = 1 + both;		/* Number of column output for some cols if single is not specified */
 
-	GMT->current.io.col_type[GMT_OUT][GMT_X] = (s->geographic) ? GMT_IS_LON : GMT_IS_FLOAT;
-	GMT->current.io.col_type[GMT_OUT][GMT_Y] = (s->geographic) ? GMT_IS_LAT : GMT_IS_FLOAT;
-	GMT->current.io.col_type[GMT_OUT][GMT_T] = GMT_IS_ABSTIME;
+	gmt_set_column (GMT, GMT_OUT, GMT_X, (s->geographic) ? GMT_IS_LON : GMT_IS_FLOAT);
+	gmt_set_column (GMT, GMT_OUT, GMT_Y, (s->geographic) ? GMT_IS_LAT : GMT_IS_FLOAT);
+	gmt_set_column (GMT, GMT_OUT, GMT_T, GMT_IS_ABSTIME);
 
 	n_items = strlen (Ctrl->F.flags); 
-	for (i = j = 0; !mixed && i < n_items; i++, j++) {	/* Overwrite the above settings */
-		switch (Ctrl->F.flags[i]) {	/* acdhintTvxyz */
+	for (i = j = 0; i < n_items; i++, j++) {	/* Overwrite the above settings */
+		switch (Ctrl->F.flags[i]) {	/* acdhitTvxyz */
 			case 'a':	/* Angle between tracks */
-				GMT->current.io.col_type[GMT_OUT][j] = GMT_IS_FLOAT;
-				break;
 			case 'c':	/* Crossover value */
-				GMT->current.io.col_type[GMT_OUT][j] = GMT_IS_FLOAT;
-				break;
-			case 'd':	/* Distance along track */
-				GMT->current.io.col_type[GMT_OUT][j] = GMT_IS_FLOAT;
-				if (both) GMT->current.io.col_type[GMT_OUT][++j] = GMT_IS_FLOAT;
-				break;
-			case 'h':	/* Heading along track */
-				GMT->current.io.col_type[GMT_OUT][j] = GMT_IS_FLOAT;
-				if (both) GMT->current.io.col_type[GMT_OUT][++j] = GMT_IS_FLOAT;
-				break;
 			case 'I':	/* Time interval (unsigned) */
 			case 'i':	/* Time interval (signed) */
-				GMT->current.io.col_type[GMT_OUT][j] = GMT_IS_FLOAT;
+			case 'v':	/* Speed along track */
+			case 'w':	/* Crossover composite weight */
+				gmt_set_column (GMT, GMT_OUT, (unsigned int)j, GMT_IS_FLOAT);
 				break;
-			case 'n':	/* Names of the track(s) [need this case to fall through] */
+			case 'd':	/* Distance along track */
+			case 'h':	/* Heading along track */
 			case 'N':	/* ID numbers of tracks */
-				GMT->current.io.col_type[GMT_OUT][j] = GMT_IS_FLOAT;
-				if (both) GMT->current.io.col_type[GMT_OUT][++j] = GMT_IS_FLOAT;
+			case 'T':	/* Time along track since beginning of the first year of the track */
+			case 'z':	/* Observed value along track */
+				gmt_set_column (GMT, GMT_OUT, (unsigned int)j, GMT_IS_FLOAT);
+				if (both) gmt_set_column (GMT, GMT_OUT, (unsigned int)(++j), GMT_IS_FLOAT);
 				break;
 			case 't':	/* Time along track */
-				GMT->current.io.col_type[GMT_OUT][j] = GMT_IS_ABSTIME;
-				if (both) GMT->current.io.col_type[GMT_OUT][++j] = GMT_IS_ABSTIME;
-				break;
-			case 'T':	/* Time along track since beginning of the first year of the track */
-				GMT->current.io.col_type[GMT_OUT][j] = GMT_IS_FLOAT;
-				if (both) GMT->current.io.col_type[GMT_OUT][++j] = GMT_IS_FLOAT;
-				break;
-			case 'v':	/* Speed along track */
-				GMT->current.io.col_type[GMT_OUT][j] = GMT_IS_FLOAT;
-				if (both) GMT->current.io.col_type[GMT_OUT][++j] = GMT_IS_FLOAT;
-				break;
-			case 'w':	/* Crossover composite weight */
-				GMT->current.io.col_type[GMT_OUT][j] = GMT_IS_FLOAT;
+				gmt_set_column (GMT, GMT_OUT, (unsigned int)j, GMT_IS_ABSTIME);
+				if (both) gmt_set_column (GMT, GMT_OUT, (unsigned int)(++j), GMT_IS_ABSTIME);
 				break;
 			case 'x':	/* x coordinate of crossover */
-				GMT->current.io.col_type[GMT_OUT][j] = (s->geographic) ? GMT_IS_LON : GMT_IS_FLOAT;
+				gmt_set_column (GMT, GMT_OUT, (unsigned int)j, (s->geographic) ? GMT_IS_LON : GMT_IS_FLOAT);
 				break;
 			case 'y':	/* y coordinate of crossover */
-				GMT->current.io.col_type[GMT_OUT][j] = (s->geographic) ? GMT_IS_LAT : GMT_IS_FLOAT;
-				break;
-			case 'z':	/* Observed value along track */
-				GMT->current.io.col_type[GMT_OUT][j] = GMT_IS_FLOAT;
-				if (both) GMT->current.io.col_type[GMT_OUT][++j] = GMT_IS_FLOAT;
+				gmt_set_column (GMT, GMT_OUT, (unsigned int)j, (s->geographic) ? GMT_IS_LAT : GMT_IS_FLOAT);
 				break;
 		}
 	}
 
 	if (Ctrl->L.active && !check_for_NaN) {	/* Correction table would not be needed for output */
-		GMT_Report (API, GMT_MSG_VERBOSE, "Warning: Correction table not needed for chosen output (corrections ignored).\n");
+		GMT_Report (API, GMT_MSG_VERBOSE, "Correction table not needed for chosen output (corrections ignored).\n");
 		Ctrl->L.active = false;
 	}
 
@@ -497,12 +475,12 @@ int GMT_x2sys_list (void *V_API, int mode, void *args) {
 	}
 	/* Time to issue output */
 
+	cmode = (Ctrl->F.mixed) ? GMT_COL_FIX : GMT_COL_FIX_NO_TEXT;
 	n_output = (unsigned int)n_items;	/* But some items represent a pair */
 	for (i = 0; i < n_items; i++) {
 		switch (Ctrl->F.flags[i]) {	/* acdhintTvwxyz */
 			case 'd':	/* Distance along track */
 			case 'h':	/* Heading along track */
-			case 'n':	/* Names of the track(s) */
 			case 'N':	/* ID numbers of tracks */
 			case 't':	/* Time along track */
 			case 'T':	/* Time along track since beginning of the track */
@@ -513,15 +491,14 @@ int GMT_x2sys_list (void *V_API, int mode, void *args) {
 		}
 	}
 
-	o_mode = (mixed) ? GMT_IS_TEXTSET : GMT_IS_DATASET;
-	if (GMT_Init_IO (API, o_mode, GMT_IS_POINT, GMT_OUT, GMT_ADD_DEFAULT, 0, options) != GMT_NOERROR) {	/* Establishes data output */
+	GMT_Set_Columns (API, GMT_OUT, n_output, cmode);
+	if (GMT_Init_IO (API, GMT_IS_DATASET, GMT_IS_POINT, GMT_OUT, GMT_ADD_DEFAULT, 0, options) != GMT_NOERROR) {	/* Establishes data output */
 		gmt_M_free (GMT, trk_name);
 		gmt_M_free (GMT, trk_nx);
 		gmt_M_free (GMT, weights);
 		Return (API->error);
 	}
-	gmt_set_cols (GMT, GMT_OUT, n_output);
-	if (GMT_Begin_IO (API, o_mode, GMT_OUT, GMT_HEADER_ON) != GMT_NOERROR) {	/* Enables data output and sets access mode */
+	if (GMT_Begin_IO (API, GMT_IS_DATASET, GMT_OUT, GMT_HEADER_ON) != GMT_NOERROR) {	/* Enables data output and sets access mode */
 		gmt_M_free (GMT, trk_name);
 		gmt_M_free (GMT, trk_nx);
 		gmt_M_free (GMT, weights);
@@ -537,17 +514,17 @@ int GMT_x2sys_list (void *V_API, int mode, void *args) {
 	gmt_set_segmentheader (GMT, GMT_OUT, true);	/* Turn on segment headers on output */
 	if (!GMT->common.b.active[GMT_OUT]) {	/* Write 3 header records */
 		char *cmd = NULL;
-		sprintf (record, " Tag: %s %s", Ctrl->T.TAG, Ctrl->C.col);
+		sprintf (record, "Tag: %s %s", Ctrl->T.TAG, Ctrl->C.col);
 		GMT_Put_Record (API, GMT_WRITE_TABLE_HEADER, record);
 		cmd = GMT_Create_Cmd (API, options);
-		sprintf (record, " Command: %s %s", THIS_MODULE_NAME, cmd);	/* Build command line argument string */
+		sprintf (record, "Command: %s %s", THIS_MODULE_CLASSIC_NAME, cmd);	/* Build command line argument string */
 		GMT_Put_Record (API, GMT_WRITE_TABLE_HEADER, record);
 		gmt_M_free (GMT, cmd);
 		GMT_Put_Record (API, GMT_WRITE_TABLE_HEADER, "");
 		record[0] = 0;	/* Clean slate */
 		for (i = j = 0; i < n_items; i++, j++) {	/* Overwrite the above settings */
 			if (i > 0) strcat (record, GMT->current.setting.io_col_separator);
-			switch (Ctrl->F.flags[i]) {	/* acdhintTvxyz */
+			switch (Ctrl->F.flags[i]) {	/* acdhitTvxyz */
 				case 'a':	/* Angle between tracks */
 					strcat (record, "angle");
 					break;
@@ -574,13 +551,6 @@ int GMT_x2sys_list (void *V_API, int mode, void *args) {
 				case 'i':	/* Time interval (signed) */
 					strcat (record, "s_tint");
 					break;
-				case 'n':	/* Names of the track(s) [need this case to fall through] */
-					if (both) {
-						strcat (record, "track_1");	strcat (record, GMT->current.setting.io_col_separator);	strcat (record, "track_2");
-					}
-					else
-						strcat (record, "track");
-					break;
 				case 'N':	/* ID numbers of tracks */
 					if (both) {
 						strcat (record, "ID_1");	strcat (record, GMT->current.setting.io_col_separator);	strcat (record, "ID_2");
@@ -628,11 +598,28 @@ int GMT_x2sys_list (void *V_API, int mode, void *args) {
 					break;
 			}
 		}
+		if (Ctrl->F.mixed) {	/* Names of the track(s)*/
+			if (i > 0) strcat (record, GMT->current.setting.io_col_separator);
+			if (both) {
+				strcat (record, "track_1");	strcat (record, GMT->current.setting.io_col_separator);	strcat (record, "track_2");
+			}
+			else
+				strcat (record, "track");
+		}
 		GMT_Put_Record (API, GMT_WRITE_TABLE_HEADER, record);
 	}
+	Out = gmt_new_record (GMT, out, (Ctrl->F.mixed) ? record : NULL);
+	record[0] = '\0';
 	
 	for (p = 0; p < np; p++) {	/* For each pair of tracks that generated crossovers */
-		if (Ctrl->N.active && (trk_nx[P[p].id[0]] < Ctrl->N.min || trk_nx[P[p].id[1]] < Ctrl->N.min)) continue;			/* Not enough COEs */
+		if (Ctrl->N.active) {	/* Not enough COEs check */
+			if (Ctrl->N.mode) {	/* Checking this pair only*/
+				if (P[p].nx < Ctrl->N.min)
+					continue;	/* Not enough COEs in this pair */
+			}
+			else if ((trk_nx[P[p].id[0]] < Ctrl->N.min || trk_nx[P[p].id[1]] < Ctrl->N.min))
+				continue;	/* Not enough total COEs by either track */
+		}
 		if (Ctrl->A.active && (fabs (trk_symm[P[p].id[0]]) > Ctrl->A.value || fabs (trk_symm[P[p].id[1]]) > Ctrl->A.value)) continue;	/* COEs not distributed symmatrically */
 		np_use++;
 		nx_use += P[p].nx;
@@ -640,7 +627,7 @@ int GMT_x2sys_list (void *V_API, int mode, void *args) {
 			sprintf (record, "%s - %s nx = %d", P[p].trk[0], P[p].trk[1], P[p].nx);
 			GMT_Put_Record (API, GMT_WRITE_SEGMENT_HEADER, record);
 		}
-		GMT_Report (API, GMT_MSG_VERBOSE, "Crossovers from %s minus %s [%d].\n", P[p].trk[0], P[p].trk[1], P[p].nx);
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Crossovers from %s minus %s [%d].\n", P[p].trk[0], P[p].trk[1], P[p].nx);
 		if (Ctrl->S.active) {	/* May have to flip which is track one and two */
 			two = !strcmp (Ctrl->S.file, P[p].trk[0]);
 			one = 1 - two;
@@ -650,13 +637,12 @@ int GMT_x2sys_list (void *V_API, int mode, void *args) {
 			/* First check if this record has a non-NaN COE */
 			if (check_for_NaN && (gmt_M_is_dnan (P[p].COE[k].data[one][COE_Z]) || gmt_M_is_dnan (P[p].COE[k].data[two][COE_Z]))) continue;
 			record[0] = 0;	/* Clean slate */
-			for (i = j = 0, first = true; i < n_items; i++, j++) {
-				switch (Ctrl->F.flags[i]) {	/* acdhintTvwxyz */
+			for (i = j = 0; i < n_items; i++, j++) {
+				switch (Ctrl->F.flags[i]) {	/* acdhitTvwxyz */
 					case 'a':	/* Angle between tracks */
 						val[0] = fabs (P[p].COE[k].data[0][COE_H] - P[p].COE[k].data[1][COE_H]);
 						while (val[0] >= 180.0) val[0] -= 180.0;
 						out[j] = (val[0] > 90.0) ? 180.0 - val[0] : val[0];
-						if (mixed) dump_ascii_cols (GMT, val, GMT_Z, 1, first, record);
 						break;
 					case 'c':	/* Crossover value */
 					 	if (Ctrl->L.active) {
@@ -664,57 +650,36 @@ int GMT_x2sys_list (void *V_API, int mode, void *args) {
 							corr[two] = MGD77_Correction_Rec (GMT, CORR[P[p].id[two]][COE_Z].term, P[p].COE[k].data[two], NULL);
 						}
 						out[j] = val[0] = (P[p].COE[k].data[one][COE_Z] - corr[one]) - (P[p].COE[k].data[two][COE_Z] - corr[two]);
-						if (mixed) dump_ascii_cols (GMT, val, GMT_Z, 1, first, record);
 						break;
 					case 'd':	/* Distance along track */
 						out[j] = val[0] = P[p].COE[k].data[one][COE_D];
 						if (both) out[++j] = val[1] = P[p].COE[k].data[two][COE_D];
-						if (mixed) dump_ascii_cols (GMT, val, GMT_Z, n_out, first, record);
 						break;
 					case 'h':	/* Heading along track */
 						out[j] = val[0] = P[p].COE[k].data[one][COE_H];
 						if (both) out[++j] = val[1] = P[p].COE[k].data[two][COE_H];
-						if (mixed) dump_ascii_cols (GMT, val, GMT_Z, n_out, first, record);
 						break;
 					case 'i':	/* Time interval in current TIME_UNIT */
 						out[j] = val[0] = sec_2_unit * (P[p].COE[k].data[one][COE_T] - P[p].COE[k].data[two][COE_T]);
-						if (mixed) dump_ascii_cols (GMT, val, GMT_Z, 1, first, record);
 						break;
 					case 'I':	/* Time interval in current TIME_UNIT */
 						out[j] = val[0] = sec_2_unit * fabs (P[p].COE[k].data[one][COE_T] - P[p].COE[k].data[two][COE_T]);
-						if (mixed) dump_ascii_cols (GMT, val, GMT_Z, 1, first, record);
-						break;
-					case 'n':	/* Names of the track(s) */
-						if (both) {
-							if (!first) strcat (record, GMT->current.setting.io_col_separator);
-							strcat (record, P[p].trk[0]);
-							strcat (record, GMT->current.setting.io_col_separator);
-							strcat (record, P[p].trk[1]);
-						}
-						else {
-							if (!first) strcat (record, GMT->current.setting.io_col_separator);
-							strcat (record, P[p].trk[two]);
-						}
 						break;
 					case 'N':	/* ID numbers of tracks */
 						out[j] = val[0] = (double)P[p].id[one];
 						if (both) out[++j] = val[1] = (double)P[p].id[two];
-						if (mixed) dump_ascii_cols (GMT, val, GMT_Z, n_out, first, record);
 						break;
 					case 't':	/* Time along track */
 						out[j] = val[0] = P[p].COE[k].data[one][COE_T];
 						if (both) out[++j] = val[1] = P[p].COE[k].data[two][COE_T];
-						if (mixed) dump_ascii_cols (GMT, val, GMT_T, n_out, first, record);
 						break;
 					case 'T':	/* Time along track since beginning of the track */
 						out[j] = val[0] = sec_2_unit * (P[p].COE[k].data[one][COE_T] - P[p].start[one]);
 						if (both) out[++j] = val[1] = sec_2_unit * (P[p].COE[k].data[two][COE_T] - P[p].start[two]);
-						if (mixed) dump_ascii_cols (GMT, val, GMT_Z, n_out, first, record);
 						break;
 					case 'v':	/* Speed along track */
 						out[j] = val[0] = P[p].COE[k].data[one][COE_V];
 						if (both) out[++j] = val[1] = P[p].COE[k].data[two][COE_V];
-						if (mixed) dump_ascii_cols (GMT, val, GMT_Z, n_out, first, record);
 						break;
 					case 'w':	/* Weight for this crossover */
 						if (weights) {	/* Weightfile was given; compute composite weight for this COE */
@@ -731,15 +696,12 @@ int GMT_x2sys_list (void *V_API, int mode, void *args) {
 						}
 						else
 							out[j] = fixed_weight;
-						if (mixed) {val[0] = out[j];	dump_ascii_cols (GMT, val, GMT_Z, 1, first, record);}
 						break;
 					case 'x':	/* x coordinate of crossover */
 						out[j] = val[0] = P[p].COE[k].data[0][COE_X];
-						if (mixed) dump_ascii_cols (GMT, val, GMT_X, 1, first, record);
 						break;
 					case 'y':	/* y coordinate of crossover */
 						out[j] = val[0] = P[p].COE[k].data[0][COE_Y];
-						if (mixed) dump_ascii_cols (GMT, val, GMT_Y, 1, first, record);
 						break;
 					case 'z':	/* Observed value along track */
 						if (Ctrl->L.active) corr[one] = MGD77_Correction_Rec (GMT, CORR[P[p].id[one]][COE_Z].term, P[p].COE[k].data[one], NULL);
@@ -748,26 +710,31 @@ int GMT_x2sys_list (void *V_API, int mode, void *args) {
 							if (Ctrl->L.active) corr[two] = MGD77_Correction_Rec (GMT, CORR[P[p].id[two]][COE_Z].term, P[p].COE[k].data[two], NULL);
 							out[++j] = val[1] = P[p].COE[k].data[two][COE_Z] - corr[two];
 						}
-						if (mixed) dump_ascii_cols (GMT, val, GMT_Z, n_out, first, record);
 						break;
 				}
-				first = false;
 			}
-			if (mixed)
-				GMT_Put_Record (API, GMT_WRITE_TEXT, record);
-			else
-				GMT_Put_Record (API, GMT_WRITE_DATA, out);
+			if (Ctrl->F.mixed) {	/* Write names of the track(s) as trialing text */
+				if (both) {
+					strcpy (record, P[p].trk[0]);
+					strcat (record, GMT->current.setting.io_col_separator);
+					strcat (record, P[p].trk[1]);
+				}
+				else
+					strcpy (record, P[p].trk[two]);
+			}
+			GMT_Put_Record (API, GMT_WRITE_DATA, Out);
 		}
 	}
 	if (GMT_End_IO (API, GMT_OUT, 0) != GMT_NOERROR) {	/* Disables further data output */
 		Return (API->error);
 	}
-	GMT_Report (API, GMT_MSG_VERBOSE, "Output %" PRIu64 " pairs and a total of %" PRIu64 " crossover records.\n", np_use, nx_use);
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Output %" PRIu64 " pairs and a total of %" PRIu64 " crossover records.\n", np_use, nx_use);
 	
 	/* Done, free up data base array */
 	
 	x2sys_free_coe_dbase (GMT, P, np);
 	gmt_M_free (GMT, trk_nx);
+	gmt_M_free (GMT, Out);
 	if (Ctrl->A.active) gmt_M_free (GMT,  trk_symm);
 
 	if (Ctrl->L.active) MGD77_Free_Correction (GMT, CORR, (unsigned int)n_tracks);
