@@ -1,7 +1,6 @@
 /*-----------------------------------------------------------------
- *	$Id$
  *
- *      Copyright (c) 1999-2018 by P. Wessel
+ *      Copyright (c) 1999-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *      See LICENSE.TXT file for copying and redistribution conditions.
  *
  *      This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU Lesser General Public License for more details.
  *
- *      Contact info: gmt.soest.hawaii.edu
+ *      Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /* x2sys_put will read one track bin file (.tbf) (from x2sys_binlist)
  * and update the track index database. The tbf file has the index
@@ -33,7 +32,8 @@
 #include "mgd77/mgd77.h"
 #include "x2sys.h"
 
-#define THIS_MODULE_NAME	"x2sys_put"
+#define THIS_MODULE_CLASSIC_NAME	"x2sys_put"
+#define THIS_MODULE_MODERN_NAME	"x2sys_put"
 #define THIS_MODULE_LIB		"x2sys"
 #define THIS_MODULE_PURPOSE	"Update track index database from track bin file"
 #define THIS_MODULE_KEYS	""
@@ -78,9 +78,9 @@ GMT_LOCAL void Free_Ctrl (struct GMT_CTRL *GMT, struct X2SYS_PUT_CTRL *C) {	/* D
 }
 
 GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
-	gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
+	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: x2sys_put [<info.tbf>] -T<TAG> [-D] [-F] [%s]\n\n", GMT_V_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "usage: %s [<info.tbf>] -T<TAG> [-D] [-F] [%s] [%s]\n\n", name, GMT_V_OPT, GMT_PAR_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 	GMT_Message (API, GMT_TIME_NONE, "\t<info.tbf> is one track bin file from x2sys_binlist [Default reads stdin].\n");
@@ -193,9 +193,9 @@ int GMT_x2sys_put (void *V_API, int mode, void *args) {
 	struct X2SYS_BIX_TRACK_INFO *this_info = NULL, *new_info = NULL;
 	struct X2SYS_BIX_TRACK *this_track = NULL;
 
-	char track[GMT_LEN64] = {""}, line[GMT_BUFSIZ] = {""};
-	char track_file[GMT_BUFSIZ] = {""}, index_file[GMT_BUFSIZ] = {""}, old_track_file[GMT_BUFSIZ] = {""}, old_index_file[GMT_BUFSIZ] = {""};
-	char track_path[GMT_BUFSIZ] = {""}, index_path[GMT_BUFSIZ] = {""}, old_track_path[GMT_BUFSIZ] = {""}, old_index_path[GMT_BUFSIZ] = {""};
+	char track[GMT_LEN64] = {""}, line[PATH_MAX] = {""};
+	char track_file[PATH_MAX] = {""}, index_file[PATH_MAX] = {""}, old_track_file[PATH_MAX] = {""}, old_index_file[PATH_MAX] = {""};
+	char track_path[PATH_MAX] = {""}, index_path[PATH_MAX] = {""}, old_track_path[PATH_MAX] = {""}, old_index_path[PATH_MAX] = {""};
 
 	int error = 0, k;
 	bool found_it, skip;
@@ -215,14 +215,11 @@ int GMT_x2sys_put (void *V_API, int mode, void *args) {
 	if (mode == GMT_MODULE_PURPOSE) return (usage (API, GMT_MODULE_PURPOSE));	/* Return the purpose of program */
 	options = GMT_Create_Options (API, mode, args);	if (API->error) bailout (API->error);	/* Set or get option list */
 
-	if (!options || options->option == GMT_OPT_USAGE) 
-		bailout (usage (API, GMT_USAGE));	/* Return the usage message */
-	if (options->option == GMT_OPT_SYNOPSIS) 
-		bailout (usage (API, GMT_SYNOPSIS));	/* Return the synopsis */
+	if ((error = gmt_report_usage (API, options, 0, usage)) != GMT_NOERROR) bailout (error);	/* Give usage if requested */
 
 	/* Parse the command-line arguments */
 
-	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
 	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
 	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
 	if ((error = parse (GMT, Ctrl, options)) != 0) Return (error);
@@ -230,7 +227,7 @@ int GMT_x2sys_put (void *V_API, int mode, void *args) {
 	/*---------------------------- This is the x2sys_put main code ----------------------------*/
 
 	if (Ctrl->In.active && (fp = gmt_fopen (GMT, Ctrl->In.file, "r")) == NULL) {
-		GMT_Report (API, GMT_MSG_NORMAL, "Error: Could not open file %s\n", Ctrl->In.file);
+		GMT_Report (API, GMT_MSG_NORMAL, "Could not open file %s\n", Ctrl->In.file);
 		Return (GMT_ERROR_ON_FOPEN);
 	}
 	if (fp == NULL) fp = GMT->session.std[GMT_IN];	/* No file given; read stdin instead */
@@ -283,9 +280,9 @@ int GMT_x2sys_put (void *V_API, int mode, void *args) {
 		free_id = 0;	/* Default is to add a new track entry */
 		if (found_it) {	/* This track already exists in the database */
 			if (Ctrl->D.active) {	/* Here we wish to delete it (and possibly replace the contents) */
-				GMT_Report (API, GMT_MSG_VERBOSE, "Removing existing information for track: %s\n", track);
+				GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Removing existing information for track: %s\n", track);
 				free_id = x2sys_bix_remove_track (GMT, this_info->next_info->track_id, &B);
-				GMT_Report (API, GMT_MSG_VERBOSE, "track %s removed\n", track);
+				GMT_Report (API, GMT_MSG_LONG_VERBOSE, "track %s removed\n", track);
 				this_info->next_info = this_info->next_info->next_info;
 				skip = !Ctrl->F.active;	/* If we are not replacing the info then we skip the new info */
 			}
@@ -310,7 +307,7 @@ int GMT_x2sys_put (void *V_API, int mode, void *args) {
 		}
 		else {	/* Read the tbf information for this track */
 
-			GMT_Report (API, GMT_MSG_VERBOSE, "Adding track: %s\n", track);
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Adding track: %s\n", track);
 
 			/* If a track is replaced, then use the same id_no recover above, else increment to get a new one */
 
@@ -356,8 +353,8 @@ int GMT_x2sys_put (void *V_API, int mode, void *args) {
 	x2sys_path (GMT, track_file, track_path);
 	x2sys_path (GMT, index_file, index_path);
 
-	snprintf (old_track_file, (size_t)GMT_BUFSIZ, "%s_old", track_file);
-	snprintf (old_index_file, (size_t)GMT_BUFSIZ, "%s_old", index_file);
+	snprintf (old_track_file, (size_t)PATH_MAX, "%s_old", track_file);
+	snprintf (old_index_file, (size_t)PATH_MAX, "%s_old", index_file);
 	x2sys_path (GMT, old_track_file, old_track_path);
 	x2sys_path (GMT, old_index_file, old_index_path);
 
@@ -373,7 +370,7 @@ int GMT_x2sys_put (void *V_API, int mode, void *args) {
 	}
 	if (!access (track_path, F_OK)) {	/* Next rename current file (if it exists) to the old file */
 		GMT_Report (API, GMT_MSG_DEBUG, "Found current track file %s.  Try to rename it.\n", track_path);
-		if (gmt_rename_file (GMT, track_path, old_track_path)) {
+		if (gmt_rename_file (GMT, track_path, old_track_path, GMT_RENAME_FILE)) {
 			GMT_Report (API, GMT_MSG_NORMAL, "Rename failed for %s -> %s. Aborting!\n", track_path, old_track_path);
 			x2sys_end (GMT, s);
 			Return (GMT_RUNTIME_ERROR);
@@ -388,7 +385,7 @@ int GMT_x2sys_put (void *V_API, int mode, void *args) {
 	}
 	if (!access (index_path, F_OK)) {	/* Next rename current file (if it exists) to the old file */
 		GMT_Report (API, GMT_MSG_DEBUG, "Found current index file %s.  Try to rename it\n", index_path);
-		if (gmt_rename_file (GMT, index_path, old_index_path)) {
+		if (gmt_rename_file (GMT, index_path, old_index_path, GMT_RENAME_FILE)) {
 			GMT_Report (API, GMT_MSG_NORMAL, "Rename failed for %s. Aborts!\n", index_path);
 			x2sys_end (GMT, s);
 			Return (GMT_RUNTIME_ERROR);
@@ -396,7 +393,7 @@ int GMT_x2sys_put (void *V_API, int mode, void *args) {
 	}
 
 	/* Now we can create new files since any existing files have been removed or renamed */
-
+	
 	if ((ftrack = fopen (track_path, "w")) == NULL) {
 		GMT_Report (API, GMT_MSG_NORMAL, "Failed to create track data base %s. Aborts!\n", track_path);
 		x2sys_end (GMT, s);
@@ -408,13 +405,14 @@ int GMT_x2sys_put (void *V_API, int mode, void *args) {
 		x2sys_end (GMT, s);
 		Return (GMT_ERROR_ON_FOPEN);
 	}
+	
 	fprintf (ftrack,"# %s\n", Ctrl->T.TAG);
 	for (this_info = B.head->next_info; this_info; this_info = this_info->next_info)
 		fprintf (ftrack,"%s %d %d\n",this_info->trackname, this_info->track_id, this_info->flag);
 
 	fclose (ftrack);
 	if (chmod (track_file, (mode_t)S_RDONLY))
-		GMT_Report (API, GMT_MSG_NORMAL, "Warning: Failed to change track file %s to read-only!\n", track_file);
+		GMT_Report (API, GMT_MSG_VERBOSE, "Failed to change track file %s to read-only!\n", track_file);
 
 	for (index = 0; index < B.nm_bin; index++) {
 		if (B.base[index].n_tracks == 0) continue;
@@ -444,9 +442,11 @@ int GMT_x2sys_put (void *V_API, int mode, void *args) {
 	}
 	fclose (fbin);
 	if (chmod (index_file, (mode_t)S_RDONLY))
-		GMT_Report (API, GMT_MSG_NORMAL, "Warning: Failed to change index file %s to read-only!\n", index_file);
+		GMT_Report (API, GMT_MSG_VERBOSE, "Failed to change index file %s to read-only!\n", index_file);
 
-	GMT_Report (API, GMT_MSG_VERBOSE, "completed successfully\n");
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "completed successfully\n");
+
+	x2sys_bix_free (GMT, &B);
 
 	x2sys_end (GMT, s);
 
