@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id$
  *
- *	Copyright (c) 1991-2019 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2019 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  * Brief synopsis: xyz2grd reads a xyz file from standard input and creates the
@@ -22,20 +21,19 @@
  *
  * Author:	Paul Wessel
  * Date:	1-JAN-2010
- * Version:	5 API
+ * Version:	6 API
  */
 
 #include "gmt_dev.h"
 
-#define THIS_MODULE_NAME	"xyz2grd"
+#define THIS_MODULE_CLASSIC_NAME	"xyz2grd"
+#define THIS_MODULE_MODERN_NAME	"xyz2grd"
 #define THIS_MODULE_LIB		"core"
-#define THIS_MODULE_PURPOSE	"Convert data table to a grid file"
+#define THIS_MODULE_PURPOSE	"Convert data table to a grid"
 #define THIS_MODULE_KEYS	"<D{,SD),GG}"
 #define THIS_MODULE_NEEDS	"R"
 #define THIS_MODULE_OPTIONS "-:JRVbdefhirs" GMT_OPT("FH")
 
-EXTERN_MSC void gmtlib_str_tolower (char *string);
-
 struct XYZ2GRD_CTRL {
 	struct In {
 		bool active;
@@ -86,12 +84,12 @@ GMT_LOCAL void Free_Ctrl (struct GMT_CTRL *GMT, struct XYZ2GRD_CTRL *C) {	/* Dea
 }
 
 GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
-	gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
+	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: xyz2grd [<table>] -G<outgrid> %s\n", GMT_I_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "usage: %s [<table>] -G<outgrid> %s\n", name, GMT_I_OPT);
 	GMT_Message (API, GMT_TIME_NONE, "\t%s [-A[d|f|l|m|n|r|S|s|u|z]]\n\t[%s]\n", GMT_Rgeo_OPT, GMT_GRDEDIT);
-	GMT_Message (API, GMT_TIME_NONE, "\t[-S[<zfile]] [%s] [-Z[<flags>]] [%s] [%s]\n\t[%s] [%s] [%s]\n\t[%s] [%s] [%s] [%s]\n\n",
-		GMT_V_OPT, GMT_bi_OPT, GMT_di_OPT, GMT_e_OPT, GMT_f_OPT, GMT_h_OPT, GMT_i_OPT, GMT_r_OPT, GMT_s_OPT, GMT_colon_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "\t[-S[<zfile]] [%s] [-Z[<flags>]] [%s] [%s]\n\t[%s] [%s] [%s]\n\t[%s] [%s] [%s] [%s] [%s]\n\n",
+		GMT_V_OPT, GMT_bi_OPT, GMT_di_OPT, GMT_e_OPT, GMT_f_OPT, GMT_h_OPT, GMT_i_OPT, GMT_r_OPT, GMT_s_OPT, GMT_colon_OPT, GMT_PAR_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
@@ -143,7 +141,7 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t   [Default format is scanline orientation in ASCII representation: -ZTLa].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   This option assumes all nodes have data values.\n");
 	GMT_Option (API, "bi3,di");
-	GMT_Message (API, GMT_TIME_NONE, "\t   Also sets value for nodes without input xyz triplet [Default is NaN].\n");
+	if (gmt_M_showusage (API)) GMT_Message (API, GMT_TIME_NONE, "\t   Also sets value for nodes without input xyz triplet [Default is NaN].\n");
 	GMT_Option (API, "e,f,h,i,r,s,:,.");
 
 	return (GMT_MODULE_USAGE);
@@ -182,7 +180,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct XYZ2GRD_CTRL *Ctrl, struct GMT
 
 			case 'A':
 				if (!opt->arg[0] && gmt_M_compat_check (GMT, 4)) {	/* In GMT4, just -A implied -Az */
-					GMT_Report (API, GMT_MSG_COMPAT, "Warning: Option -A is deprecated; use -Az instead.\n");
+					GMT_Report (API, GMT_MSG_COMPAT, "Option -A is deprecated; use -Az instead.\n");
 					Ctrl->A.active = true;
 					Ctrl->A.mode = 'z';
 				}
@@ -201,7 +199,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct XYZ2GRD_CTRL *Ctrl, struct GMT
 				break;
 			case 'E':
 				if (gmt_M_compat_check (GMT, 4)) {
-					GMT_Report (API, GMT_MSG_COMPAT, "Warning: Option -E is deprecated; use grdconvert instead.\n");
+					GMT_Report (API, GMT_MSG_COMPAT, "Option -E is deprecated; use grdconvert instead.\n");
 					Ctrl->E.active = true;
 					if (opt->arg[0]) {
 						Ctrl->E.nodata = atof (opt->arg);
@@ -222,7 +220,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct XYZ2GRD_CTRL *Ctrl, struct GMT
 				break;
 			case 'N':
 				if (gmt_M_compat_check (GMT, 4)) {	/* Honor old -N<value> option */
-					GMT_Report (API, GMT_MSG_COMPAT, "Warning: Option -N is deprecated; use GMT common option -di<nodata> instead.\n");
+					GMT_Report (API, GMT_MSG_COMPAT, "Option -N is deprecated; use GMT common option -di<nodata> instead.\n");
 					if (opt->arg[0]) {
 						char arg[GMT_LEN64] = {""};
 						sprintf (arg, "i%s", opt->arg);
@@ -275,7 +273,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct XYZ2GRD_CTRL *Ctrl, struct GMT
 		GMT->common.b.type[GMT_IN] = Ctrl->Z.type;
 		if (b_only) {
 			GMT->common.b.active[GMT_IN] = false;
-			GMT_Report (API, GMT_MSG_NORMAL, "Warning: -Z overrides -bi\n");
+			GMT_Report (API, GMT_MSG_NORMAL, "-Z overrides -bi\n");
 		}
 	}
 
@@ -301,7 +299,7 @@ GMT_LOCAL void protect_J(struct GMTAPI_CTRL *API, struct GMT_OPTION *options) {
 		(void)GMT_Append_Option(API, opt, options);
 #else
 		GMT_Report(API, GMT_MSG_NORMAL,
-		           "Warning: -J option to set grid's referencing system is only available when GMT was build with GDAL\n");
+		           "-J option to set grid's referencing system is only available when GMT was build with GDAL\n");
 #endif
 	}
 }
@@ -317,13 +315,14 @@ int GMT_xyz2grd (void *V_API, int mode, void *args) {
 
 	double *in = NULL, wesn[4];
 
-	float no_data_f, *data = NULL;
+	gmt_grdfloat no_data_f, *data = NULL;
 
 	void * (*save_i) (struct GMT_CTRL *, FILE *, uint64_t *, int *) = NULL;
-	int (*save_o) (struct GMT_CTRL *, FILE *, uint64_t, double *);
+	int (*save_o) (struct GMT_CTRL *, FILE *, uint64_t, double *, char *);
 
 	struct GMT_GRID *Grid = NULL;
 	struct GMT_Z_IO io;
+	struct GMT_RECORD *In = NULL;
 	struct XYZ2GRD_CTRL *Ctrl = NULL;
 	struct GMT_CTRL *GMT = NULL, *GMT_cpy = NULL;
 	struct GMT_OPTION *options = NULL;
@@ -335,13 +334,12 @@ int GMT_xyz2grd (void *V_API, int mode, void *args) {
 	if (mode == GMT_MODULE_PURPOSE) return (usage (API, GMT_MODULE_PURPOSE));	/* Return the purpose of program */
 	options = GMT_Create_Options (API, mode, args);	if (API->error) return (API->error);	/* Set or get option list */
 
-	if (!options || options->option == GMT_OPT_USAGE) bailout (usage (API, GMT_USAGE));/* Return the usage message */
-	if (options->option == GMT_OPT_SYNOPSIS) bailout (usage (API, GMT_SYNOPSIS));	/* Return the synopsis */
+	if ((error = gmt_report_usage (API, options, 0, usage)) != GMT_NOERROR) bailout (error);	/* Give usage if requested */
 
 	/* Parse the command-line arguments */
 
 	protect_J(API, options);	/* If -J is used, add a -f0f,1f option to avoid later parsing errors due to -R & -J conflicts */
-	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
 	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
 	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
 	if ((error = parse (GMT, Ctrl, &io, options)) != 0) Return (error);
@@ -362,9 +360,9 @@ int GMT_xyz2grd (void *V_API, int mode, void *args) {
 		GMT->current.io.output = gmt_z_output;		/* Override output writer with chosen binary writer for selected type */
 		GMT->common.b.active[GMT_IN] = io.binary;	/* May have to set input binary as well */
 		GMT->common.b.active[GMT_OUT] = io.binary;	/* May have to set output binary as well */
-		if ((error = gmt_set_cols (GMT, GMT_IN, 1)) != 0) Return (error);
+		if ((error = GMT_Set_Columns (API, GMT_IN, 1, GMT_COL_FIX_NO_TEXT)) != GMT_NOERROR) Return (error);
 		/* Initialize the i/o since we are doing record-by-record reading/writing */
-		GMT_Report (API, GMT_MSG_VERBOSE, "Swapping data bytes only\n");
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Swapping data bytes only\n");
 		if (Ctrl->S.active) io.swab = true;	/* Need to pass swabbing down to the gut level */
 
 		/* Register the data source */
@@ -380,10 +378,10 @@ int GMT_xyz2grd (void *V_API, int mode, void *args) {
 		if (GMT_Init_IO (API, GMT_IS_DATASET, GMT_IS_NONE, GMT_OUT, w_mode, 0, options) != GMT_NOERROR) {	/* Establishes data output to stdout */
 			Return (API->error);
 		}
-		if ((error = gmt_set_cols (GMT, GMT_IN, 1)) != GMT_NOERROR) {	/* We don't really care or know about columns so must use 1 */
+		if ((error = GMT_Set_Columns (API, GMT_IN, 1, GMT_COL_FIX_NO_TEXT)) != GMT_NOERROR) {	/* We don't really care or know about columns so must use 1 */
 			Return (API->error);
 		}
-		if ((error = gmt_set_cols (GMT, GMT_OUT, 1)) != GMT_NOERROR) {	/* We don't really care or know about columns so must use 1 */
+		if ((error = GMT_Set_Columns (API, GMT_OUT, 1, GMT_COL_FIX_NO_TEXT)) != GMT_NOERROR) {	/* We don't really care or know about columns so must use 1 */
 			Return (API->error);
 		}
 		/* Initialize the i/o for doing record-by-record reading/writing */
@@ -397,7 +395,7 @@ int GMT_xyz2grd (void *V_API, int mode, void *args) {
 			Return (API->error);
 		}
 		do {	/* Keep returning records until we reach EOF */
-			if ((in = GMT_Get_Record (API, GMT_READ_DATA, NULL)) == NULL) {	/* Read next record, get NULL if special case */
+			if ((In = GMT_Get_Record (API, GMT_READ_DATA, NULL)) == NULL) {	/* Read next record, get NULL if special case */
 				if (gmt_M_rec_is_error (GMT)) 		/* Bail if there are any read errors */
 					Return (GMT_RUNTIME_ERROR);
 				if (gmt_M_rec_is_any_header (GMT)) 	/* Skip all headers */
@@ -409,7 +407,7 @@ int GMT_xyz2grd (void *V_API, int mode, void *args) {
 
 			/* Data record to process */
 
-			GMT_Put_Record (API, GMT_WRITE_DATA, in);
+			GMT_Put_Record (API, GMT_WRITE_DATA, In);
 		} while (true);
 
 		GMT->current.io.input = save_i;			/* Reset input pointer */
@@ -426,7 +424,7 @@ int GMT_xyz2grd (void *V_API, int mode, void *args) {
 		Return (GMT_NOERROR);	/* We are done here */
 	}
 
-	GMT_Report (API, GMT_MSG_VERBOSE, "Processing input table data\n");
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Processing input table data\n");
 
 	/* PW: This is now done in grdconvert since ESRI Arc Interchange is a recognized format */
 	if (Ctrl->E.active && gmt_M_compat_check (GMT, 4)) {	/* Read an ESRI Arc Interchange grid format in ASCII.  This must be a single physical file. */
@@ -477,17 +475,17 @@ int GMT_xyz2grd (void *V_API, int mode, void *args) {
 		gmt_set_grddim (GMT, Grid->header);
 		gmt_M_err_fail (GMT, gmt_grd_RI_verify (GMT, Grid->header, 1), Ctrl->G.file);
 
-		GMT_Report (API, GMT_MSG_VERBOSE, "n_columns = %d  n_rows = %d\n", Grid->header->n_columns, Grid->header->n_rows);
+		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "n_columns = %d  n_rows = %d\n", Grid->header->n_columns, Grid->header->n_rows);
 		n_left = Grid->header->nm;
 
-		Grid->data = gmt_M_memory_aligned (GMT, NULL, Grid->header->nm, float);
+		Grid->data = gmt_M_memory_aligned (GMT, NULL, Grid->header->nm, gmt_grdfloat);
 		/* ESRI grids are scanline oriented (top to bottom), as are the GMT grids */
 		row = col = 0;
 		if (fscanf (fp, "%s", line) != 1) {
 			GMT_Report (API, GMT_MSG_NORMAL, "Unable to read nodata-flag or first data record from ESRI file\n");
 			Return (GMT_DATA_READ_ERROR);
 		}
-		gmtlib_str_tolower (line);
+		gmt_str_tolower (line);
 		if (!strcmp (line, "nodata_value")) {	/* Found the optional nodata word */
 			if (fscanf (fp, "%lf", &value) != 1) {
 				GMT_Report (API, GMT_MSG_NORMAL, "Unable to parse nodata-flag from ESRI file\n");
@@ -502,13 +500,13 @@ int GMT_xyz2grd (void *V_API, int mode, void *args) {
 		else {	/* Instead got the very first data value */
 			ij = gmt_M_ijp (Grid->header, row, col);
 			value = atof (line);
-			Grid->data[ij] = (value == Ctrl->E.nodata) ? GMT->session.f_NaN : (float) value;
+			Grid->data[ij] = (value == Ctrl->E.nodata) ? GMT->session.f_NaN : (gmt_grdfloat) value;
 			if (++col == Grid->header->n_columns) col = 0, row++;
 			n_left--;
 		}
 		while (fscanf (fp, "%lf", &value) == 1 && n_left) {
 			ij = gmt_M_ijp (Grid->header, row, col);
-			Grid->data[ij] = (value == Ctrl->E.nodata) ? GMT->session.f_NaN : (float) value;
+			Grid->data[ij] = (value == Ctrl->E.nodata) ? GMT->session.f_NaN : (gmt_grdfloat) value;
 			if (++col == Grid->header->n_columns) col = 0, row++;
 			n_left--;
 		}
@@ -525,9 +523,9 @@ int GMT_xyz2grd (void *V_API, int mode, void *args) {
 
 	/* Here we will read either x,y,z or z data, using -R -I [-r] for sizeing */
 
-	no_data_f = (GMT->common.d.active[GMT_IN]) ? (float)GMT->common.d.nan_proxy[GMT_IN] : GMT->session.f_NaN;
+	no_data_f = (GMT->common.d.active[GMT_IN]) ? (gmt_grdfloat)GMT->common.d.nan_proxy[GMT_IN] : GMT->session.f_NaN;
 
-	/* Set up and allocate output grid [note: zero padding specificied since no BCs required] */
+	/* Set up and allocate output grid [note: zero padding specified since no BCs required] */
 	if ((Grid = GMT_Create_Data (API, GMT_IS_GRID, GMT_IS_SURFACE, GMT_CONTAINER_AND_DATA, NULL, NULL, NULL, \
 		GMT_GRID_DEFAULT_REG, 0, NULL)) == NULL) Return (API->error);
 
@@ -539,13 +537,13 @@ int GMT_xyz2grd (void *V_API, int mode, void *args) {
 
 	/* For Amode = 'd' or 'S' we need a second grid, and also require a minimum of 2 points per grid */
 	if (Amode == 'd' || Amode == 'S') {
-		data = gmt_M_memory_aligned (GMT, NULL, Grid->header->nm, float);
+		data = gmt_M_memory_aligned (GMT, NULL, Grid->header->nm, gmt_grdfloat);
 		n_min = 2;
 	}
 
-	if (GMT->common.b.active[GMT_IN] && GMT->current.io.col_type[GMT_IN][GMT_Z] & GMT_IS_RATIME && GMT->current.io.fmt[GMT_IN][GMT_Z].type == GMT_FLOAT) {
-		GMT_Report (API, GMT_MSG_NORMAL, "Warning: Your single precision binary input data are unlikely to hold absolute time coordinates without serious truncation.\n");
-		GMT_Report (API, GMT_MSG_NORMAL, "Warning: You must use double precision when storing absolute time coordinates in binary data tables.\n");
+	if (GMT->common.b.active[GMT_IN] && gmt_M_type (GMT, GMT_IN, GMT_Z) & GMT_IS_RATIME && GMT->current.io.fmt[GMT_IN][GMT_Z].type == GMT_FLOAT) {
+		GMT_Report (API, GMT_MSG_VERBOSE, "Your single precision binary input data are unlikely to hold absolute time coordinates without serious truncation.\n");
+		GMT_Report (API, GMT_MSG_VERBOSE, "You must use double precision when storing absolute time coordinates in binary data tables.\n");
 	}
 
 	if (Ctrl->D.active && gmt_decode_grd_h_info (GMT, Ctrl->D.information, Grid->header)) {
@@ -553,7 +551,7 @@ int GMT_xyz2grd (void *V_API, int mode, void *args) {
 		Return (GMT_PARSE_ERROR);
 	}
 
-	GMT_Report (API, GMT_MSG_VERBOSE, "n_columns = %d  n_rows = %d  nm = %" PRIu64 "  size = %" PRIuS "\n", Grid->header->n_columns, Grid->header->n_rows, Grid->header->nm, Grid->header->size);
+	GMT_Report (API, GMT_MSG_LONG_VERBOSE, "n_columns = %d  n_rows = %d  nm = %" PRIu64 "  size = %" PRIuS "\n", Grid->header->n_columns, Grid->header->n_rows, Grid->header->nm, Grid->header->size);
 
 	gmt_M_err_fail (GMT, gmt_set_z_io (GMT, &io, Grid), Ctrl->G.file);
 
@@ -575,7 +573,7 @@ int GMT_xyz2grd (void *V_API, int mode, void *args) {
 		GMT->current.setting.io_nan_records = false;	/* Cannot have x,y as NaNs here */
 	}
 
-	if ((error = gmt_set_cols (GMT, GMT_IN, n_req)) != GMT_NOERROR) {
+	if ((error = GMT_Set_Columns (API, GMT_IN, (unsigned int)n_req, GMT_COL_FIX_NO_TEXT)) != GMT_NOERROR) {
 		gmt_M_free (GMT, data);
 		gmt_M_free (GMT, flag);
 		Return (error);
@@ -587,8 +585,7 @@ int GMT_xyz2grd (void *V_API, int mode, void *args) {
 		Return (API->error);	/* Establishes data input */
 	}
 	if (GMT_Begin_IO (API, GMT_IS_DATASET, GMT_IN, GMT_HEADER_ON) != GMT_NOERROR) {
-		gmt_M_free (GMT, data);
-		gmt_M_free (GMT, flag);
+		gmt_M_free (GMT, data);		gmt_M_free (GMT, flag);
 		Return (API->error);	/* Enables data input and sets access mode */
 	}
 
@@ -600,8 +597,9 @@ int GMT_xyz2grd (void *V_API, int mode, void *args) {
 	}
 
 	do {	/* Keep returning records until we reach EOF */
-		if ((in = GMT_Get_Record (API, GMT_READ_DATA, NULL)) == NULL) {	/* Read next record, get NULL if special case */
+		if ((In = GMT_Get_Record (API, GMT_READ_DATA, NULL)) == NULL) {	/* Read next record, get NULL if special case */
 			if (gmt_M_rec_is_error (GMT)) {		/* Bail if there are any read errors */
+				gmt_M_free (GMT, data);		gmt_M_free (GMT, flag);
 				Return (GMT_RUNTIME_ERROR);
 			}
 			else if (gmt_M_rec_is_eof (GMT)) 		/* Reached end of file */
@@ -611,15 +609,17 @@ int GMT_xyz2grd (void *V_API, int mode, void *args) {
 
 		/* Data record to process */
 		n_read++;
+		in = In->data;	/* Only need to process numerical part here */
 
 		if (Ctrl->Z.active) {	/* Read separately because of all the possible formats */
 			if (ij == io.n_expected) {
 				GMT_Report (API, GMT_MSG_NORMAL, "More than %" PRIu64 " records, only %" PRIu64 " was expected (aborting)!\n", ij, io.n_expected);
 				GMT_Report (API, GMT_MSG_NORMAL, "(You are probably misterpreting xyz2grd with an interpolator; see 'surface' man page)\n");
+				gmt_M_free (GMT, data);		gmt_M_free (GMT, flag);
 				Return (GMT_RUNTIME_ERROR);
 			}
 			ij_gmt = io.get_gmt_ij (&io, Grid, ij);	/* Convert input order to output node (with padding) as per -Z */
-			Grid->data[ij_gmt] = (gmt_input_is_nan_proxy (GMT, in[zcol])) ? GMT->session.f_NaN : (float)in[zcol];
+			Grid->data[ij_gmt] = (gmt_input_is_nan_proxy (GMT, in[zcol])) ? GMT->session.f_NaN : (gmt_grdfloat)in[zcol];
 			ij++;
 		}
 		else {	/* Get x, y, z */
@@ -640,50 +640,50 @@ int GMT_xyz2grd (void *V_API, int mode, void *args) {
 			switch (Amode) {
 				case 'f':	/* Want the first value to matter only */
 					if (flag[ij] == 0) {	/* Assign first value and that is the end of it */
-						Grid->data[ij] = (float)in[zcol];
+						Grid->data[ij] = (gmt_grdfloat)in[zcol];
 						flag[ij] = (unsigned int)n_read;
 					}
 					break;
 				case 's':	/* Want the last value to matter only */
-					Grid->data[ij] = (float)in[zcol];	/* Assign last value and that is it */
+					Grid->data[ij] = (gmt_grdfloat)in[zcol];	/* Assign last value and that is it */
 					flag[ij] = (unsigned int)n_read;
 					break;
 				case 'l':	/* Keep lowest value */
 					if (flag[ij]) {	/* Already assigned the first value */
-						if (in[zcol] < (double)Grid->data[ij]) Grid->data[ij] = (float)in[zcol];
+						if (in[zcol] < (double)Grid->data[ij]) Grid->data[ij] = (gmt_grdfloat)in[zcol];
 					}
 					else {	/* First time, just assign the current value */
-						Grid->data[ij] = (float)in[zcol];
+						Grid->data[ij] = (gmt_grdfloat)in[zcol];
 					}
 					flag[ij]++;
 					break;
 				case 'u':	/* Keep highest value */
 					if (flag[ij]) {	/* Already assigned the first value */
-						if (in[zcol] > (double)Grid->data[ij]) Grid->data[ij] = (float)in[zcol];
+						if (in[zcol] > (double)Grid->data[ij]) Grid->data[ij] = (gmt_grdfloat)in[zcol];
 					}
 					else {	/* First time, just assign the current value */
-						Grid->data[ij] = (float)in[zcol];
+						Grid->data[ij] = (gmt_grdfloat)in[zcol];
 					}
 					flag[ij]++;
 					break;
 				case 'd':	/* Keep highest and lowest value */
 					if (flag[ij]) {	/* Already assigned the first value */
-						if (in[zcol] > (double)Grid->data[ij]) Grid->data[ij] = (float)in[zcol];
-						if (in[zcol] < (double)data[ij]) data[ij] = (float)in[zcol];
+						if (in[zcol] > (double)Grid->data[ij]) Grid->data[ij] = (gmt_grdfloat)in[zcol];
+						if (in[zcol] < (double)data[ij]) data[ij] = (gmt_grdfloat)in[zcol];
 					}
 					else {	/* First time, just assign the current value */
-						Grid->data[ij] = data[ij] = (float)in[zcol];
+						Grid->data[ij] = data[ij] = (gmt_grdfloat)in[zcol];
 					}
 					flag[ij]++;
 					break;
 				case 'S': 	/* Add up squares and means to compute standard deviation */
-					data[ij] += (float)in[zcol];	/* This adds up the means; we fall through to next case to also add up squares */
+					data[ij] += (gmt_grdfloat)in[zcol];	/* This adds up the means; we fall through to next case on purpose to also add up squares */
 				case 'r': 	/* Add up squares in case we must rms */
-					Grid->data[ij] += (float)in[zcol] * (float)in[zcol];
+					Grid->data[ij] += (gmt_grdfloat)in[zcol] * (gmt_grdfloat)in[zcol];
 					flag[ij]++;
 					break;
 				default:	/* Add up in case we must sum or mean */
-					Grid->data[ij] += (float)in[zcol];
+					Grid->data[ij] += (gmt_grdfloat)in[zcol];
 					flag[ij]++;
 					break;
 			}
@@ -692,6 +692,7 @@ int GMT_xyz2grd (void *V_API, int mode, void *args) {
 	} while (true);
 
 	if (GMT_End_IO (API, GMT_IN, 0) != GMT_NOERROR) {	/* Disables further data input */
+		gmt_M_free (GMT, flag);		gmt_M_free (GMT, data);
 		Return (API->error);
 	}
 
@@ -701,6 +702,7 @@ int GMT_xyz2grd (void *V_API, int mode, void *args) {
 		if (ij != io.n_expected) {	/* Input amount does not match expectations */
 			GMT_Report (API, GMT_MSG_NORMAL, "Found %" PRIu64 " records, but %" PRIu64 " was expected (aborting)!\n", ij, io.n_expected);
 				GMT_Report (API, GMT_MSG_NORMAL, "(You are probably misterpreting xyz2grd with an interpolator; see 'surface' man page)\n");
+			gmt_M_free (GMT, flag);		gmt_M_free (GMT, data);
 			Return (GMT_RUNTIME_ERROR);
 		}
 		gmt_check_z_io (GMT, &io, Grid);	/* This fills in missing periodic row or column */
@@ -737,12 +739,14 @@ int GMT_xyz2grd (void *V_API, int mode, void *args) {
 							break;
 						case 'd':	/* Keep the lowest in 'data' */
 							if (data[ij_east] < data[ij_west]) data[ij_west] = data[ij_east];
+							/* Fall through on purpose since range also needs the highsets */
 						case 'u':	/* Keep the highest */
 							if (Grid->data[ij_east] > Grid->data[ij_west]) Grid->data[ij_west] = Grid->data[ij_east];
 							flag[ij_west] += flag[ij_east];
 							break;
 						case 'S':	/* Sum up the sums in 'data' */
 							data[ij_west] += data[ij_east];
+							/* Fall through on purpose */
 						default:	/* Add up in case we must sum, rms, mean, or standard deviation */
 							Grid->data[ij_west] += Grid->data[ij_east];
 							flag[ij_west] += flag[ij_east];
@@ -762,15 +766,15 @@ int GMT_xyz2grd (void *V_API, int mode, void *args) {
 			}
 			else {	/* Enough values went to this node */
 				if (Amode == 'n')
-					Grid->data[ij] = (float)flag[ij];
+					Grid->data[ij] = (gmt_grdfloat)flag[ij];
 				else if (Amode == 'm')
-					Grid->data[ij] /= (float)flag[ij];
+					Grid->data[ij] /= (gmt_grdfloat)flag[ij];
 				else if (Amode == 'r')
-					Grid->data[ij] = (float)sqrt (Grid->data[ij] / (float)flag[ij]);
+					Grid->data[ij] = (gmt_grdfloat)sqrt (Grid->data[ij] / (gmt_grdfloat)flag[ij]);
 				else if (Amode == 'd')
 					Grid->data[ij] -= data[ij];
 				else if (Amode == 'S') {
-					Grid->data[ij] = (float)sqrt ((Grid->data[ij] - data[ij] * data[ij] / (float)flag[ij]) / (float)(flag[ij] - 1));
+					Grid->data[ij] = (gmt_grdfloat)sqrt ((Grid->data[ij] - data[ij] * data[ij] / (gmt_grdfloat)flag[ij]) / (gmt_grdfloat)(flag[ij] - 1));
 
 				}
 				/* Implicit else means return the currently stored value */
@@ -784,17 +788,22 @@ int GMT_xyz2grd (void *V_API, int mode, void *args) {
 			char line[GMT_BUFSIZ], e_value[GMT_LEN32];
 			sprintf (line, "%s\n", GMT->current.setting.format_float_out);
 			(GMT->common.d.active[GMT_IN]) ? sprintf (e_value, GMT->current.setting.format_float_out, GMT->common.d.nan_proxy[GMT_IN]) : sprintf (e_value, "NaN");
-			GMT_Report (API, GMT_MSG_VERBOSE, "Data records read: %" PRIu64 "  used: %" PRIu64 "  nodes filled: %" PRIu64 " nodes empty: %" PRIu64 " [set to %s]\n",
+			GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Data records read: %" PRIu64 "  used: %" PRIu64 "  nodes filled: %" PRIu64 " nodes empty: %" PRIu64 " [set to %s]\n",
 				n_read, n_used, n_filled, n_empty, e_value);
 			if (n_confused) GMT_Report (API, GMT_MSG_VERBOSE, "Warning - %" PRIu64 " values gave bad indices: Pixel vs Gridline registration confusion?\n", n_confused);
 		}
 	}
 
 	gmt_grd_pad_on (GMT, Grid, GMT->current.io.pad);	/* Restore padding */
-	if (GMT_Set_Comment (API, GMT_IS_GRID, GMT_COMMENT_IS_OPTION | GMT_COMMENT_IS_COMMAND, options, Grid)) Return (API->error);
+	if (GMT_Set_Comment (API, GMT_IS_GRID, GMT_COMMENT_IS_OPTION | GMT_COMMENT_IS_COMMAND, options, Grid)) {
+		gmt_M_free (GMT, flag);		gmt_M_free (GMT, data);
+		Return (API->error);
+	}
 	if (GMT_Write_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_CONTAINER_AND_DATA, NULL, Ctrl->G.file, Grid) != GMT_NOERROR) {
+		gmt_M_free (GMT, flag);		gmt_M_free (GMT, data);
 		Return (API->error);
 	}
 
+	gmt_M_free (GMT, flag);		gmt_M_free (GMT, data);
 	Return (GMT_NOERROR);
 }
